
Option Strict Off
Option Explicit On
Module Analysis_Utilities
	
    '-------------------------------------------------------------------------------------
    ' FUNCTION   FFT_DataArray()
    '
    ' PURPOSE    Perform a hamming window, fft, and magnitude calculation on
    '            the data() array. data() must be an existing arrya that is
    '            at least (2^power+2) long. We return the # of calculated
    '            magnidues inside 'numCalculatedMagnitudes'.
    '
    '            The actual calulations occur within the instruNet iNet32.dll.
    '
    ' HISTORY    6/5/02 - updated, gsw
    '            6/6/02   updated to be compatible with Visual Studio (Visual Basic) .Net v7
    '-------------------------------------------------------------------------------------
    Public Sub FFT_DataArray( _
        ByRef dataArrayY As DataArrayType, _
        ByVal power As Short, _
        ByRef numCalculatedMagnitudes As Integer, _
        ByVal digitized_sampleRate As Double)

        Dim numRemaingTimePtsAfterFFTdata As Integer
        Dim ok As Double
        Dim lastPt, firstPt As Single
        Dim numpts As Integer

        ' calc the # of time pts to do the fft on
        numpts = 2 ^ power

        ' # of complex {real, imaginary} values
        numCalculatedMagnitudes = numpts / 2

        ' this will show an error if the data does not exist
        firstPt = dataArrayY.data(0)
        lastPt = dataArrayY.data(numpts + 1)

        ' multiply the time data by a hamming window via an iNet32.dll routine
        ok = MultiplyArrayByHammWindow_VB(dataArrayY.data, power)
        If (ok = False) Then
            numCalculatedMagnitudes = 0
            Exit Sub
        End If

        ' do an fft on the data (we us a version that is compiled C inside
        ' the iNet32.dll).
        Call fft_VB(dataArrayY.data, power)

        ' calc magnitudes via an iNet32.dll routine
        Call ComplexToMagnitudes_VB(dataArrayY.data, numCalculatedMagnitudes)

        ' calc # of pts after the magnitudes
        numRemaingTimePtsAfterFFTdata = numpts - numCalculatedMagnitudes

        ' update the # of resulting data points
        dataArrayY.numValidPts = numCalculatedMagnitudes

        ' update the pts-per Hz unit (we are plotting amplitudes vs. Hz).
        dataArrayY.ptsPerXunit = numCalculatedMagnitudes / (digitized_sampleRate / 2)

        ' update 1st pt xUnit
        dataArrayY.firstPt_Xunits = 0

    End Sub

End Module