Option Strict Off
Option Explicit On
Imports VB = Microsoft.VisualBasic
Imports System.Runtime.InteropServices
Imports System.Threading

Module Digitize_Utilities

    ' ===========================================================================
    ' ===========================================================================
    ' FILE: INET_Digitize.vb
    '
    ' ------------------ D O C U  M E N T A T I O N ---------------------------
    ' This file contains code that digitizes from instruNet I/O channels.
    '
    ' -------------------------  H I S T O R Y --------------------------------
    ' HISTORY  Copyright c 1994-98 GWI.  All rights reserved.
    ' 7/29/98  created - gsw
    ' 6/5/02   updated for VB 7 .Net - gsw
    '
    ' ===========================================================================
    ' ===========================================================================


    ' ===========================================================================
    ' GLOBAL DECLARATIONS
    ' ===========================================================================

    Public gNumScans As Integer                     ' # of scans (i.e. buffers) to digitize
    Public gNumDigiCh As Short                      ' # of channels to digitize

    Public gDigiChan() As chAddr                    ' stores a list of digitize channels

    Public gPtsPerScan_nextDigi As Integer          ' points per scan per channel (next data acq)
    Public gPtsPerScan_currentData As Integer       ' points per scan per channel (current data)

    Public gSampleRate_nextDigi As Double           ' sample rate per channel (pts/sec)(next data acq)
    Public gSampleRate_currentData As Double        ' sample rate per channel (pts/sec)(current data)

    Public gMaxNumOfDigiChannels As Short           ' maximum number of digitize channels

    Public gContinuousScans As Short                ' if true, we run with scans that are continous
    '                                                 in time; otherwise, we retrigger for each

    Public gProcessSegmentsInsteadOfScans As Boolean ' if true, we process a scan when it is finished
    '                                                 digitizing; otherwise, we process little segments
    '                                                 as they are recorded.

    Public gRun_EndUser_PostAcqAnalysis As Boolean  ' if true, then run EndUser_PostAcqAnalysis()
    '                                                 at the end of the digitization.  This is an
    '                                                 end user routine that processes the resulting data

    Public gRun_EndUser_SegmentProcessor As Boolean ' if true, then run EndUser_SegmentProcessor()
    '                                                 while data is being digitized.  This is an end
    '                                                 user routine that processes little segments of data.

    Public gWaitTillDigitizationCompletes As Boolean  ' If true, then DigitizeProcess_Start() waits for
    '                                                 the entire digitization to complete; otherwise,
    '                                                 the Timer control triggers the servicing of the
    '                                                 digitize buffers, every 200ms or so, while digitizing.

    Public gSimulatingHardware As Boolean            ' true if simulation is on

    Public gCurrentlyDigitizing As Boolean           ' true if we are currently digitizing

    Public gCopyChannelsFromNetworkPage As Boolean   ' true if we are to copy the Enabled channels
    '                                                  in the Network page to rebuild the Digitize
    '                                                  list, after the user closed the Network Window.

    '                                                  true if we are inside the Interrupt routine
    Public gInsideTimerServiceRoutine As Boolean = False

    Public gOverCallInterruptRoutine As Long = 0     ' we record how many times we over call the
    '                                                  interrupt routine that processes the data

    '                                                  Records how many interrupts routine
    '                                                  calls are queued up after digitizing ends
    Public gInterruptRoutineQueueStillHotAfterDigitizing As Long = 0

    ' "Start"/"Stop" button, set to "Nothing" if doesn't exit
    Public gStartStopButton As System.Windows.Forms.Button

    ' Digitize Timer is used to run interrupts
    Public gDigitizeProcessTimer As System.Windows.Forms.Timer
    Public gDigitizeProcessCtr As Double

    ' flags that keep track of buffers as we digitize
    Public gWeAreDoingOurFirstAccessOfData As Short
    Public gTheNextDataAccessIsTheLast_ofEntireDigitization As Short
    Public gTheNextDataAccessWillPullToTheEndOfAScan As Short
    Public gControllerIsFinishedWithDigitization As Short
    Public gLast_scanNum_base1_pulled_inFull As Integer
    Public gLast_scanNum_base1_pushed_inFull As Integer

    ' helps manage digitize buffers
    Public gUseDriverRamBuffer As Short
    Public gBufferStartAddrP As Integer
    Public gNumPointsToPull As Integer
    Public gPointToPullindex_base1 As Integer
    Public gScanNumberToPull_base1 As Integer

    ' describes latest segment of digitized data
    Public gProcessDataAsItIsAcquired As Short
    Public gSegmentStart_ptNum As Integer
    Public gSegmentEnd_ptNum As Integer

    ' -----------------------------------------------------
    ' Ways of massaging data, immediately after it is digitized.

    Public Const jNoMassage As Short = 1    ' don't touch the data
    Public Const jFFTMassage As Short = 2   ' do an fft on the data, after digitizing

    '==========================================================================
    '==========================================================================
    ' SUBROUTINES
    '==========================================================================
    '==========================================================================

    ' -------------------------------------------------------------------------
    ' FUNCTION    VbDataBuffers_LoadWithSineWaves()
    '
    ' PURPOSE     We load the digitize data() arrays with dummy data,
    '             for purposes of simulations.
    '
    ' HISTORY     2/25/97  created - unoWarrior
    ' -------------------------------------------------------------------------
    Sub VbDataBuffers_LoadWithSineWaves()

        Dim Pi As Double
        Dim ptNum As Integer
        Dim cNum As Integer

        ' load Digitize Data with sine waves
        For cNum = 0 To (gNumDigiCh - 1)

            ' scan through each point in one wave
            For ptNum = 0 To (gPtsPerScan_nextDigi - 1)

                gDigiChan(cNum).dataArrayY.data(ptNum) = System.Math.Sin(ptNum * Pi / (60 / cNum))

            Next ptNum

        Next cNum

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    DigitizeProcess_ServiceIncomingData()
    '
    ' PURPOSE     Services incoming data from digitization process.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Function DigitizeProcess_ServiceIncomingData() As Short

        Dim scanNumberToPull_base1 As Integer
        Dim power As Integer
        Dim cNum As Integer

        Dim dNum, bNum, e As Integer
        Dim chanLinkedToDisplay As Short

        ' -------------------------------------------
        ' Make sure the TIMER does not call this subroutine while
        ' it is already running.

        If gInsideTimerServiceRoutine Then

            gOverCallInterruptRoutine += 1

            ' exit this subroutine and allow the interrupt routine
            ' that we interrupted to run
            Exit Function

        End If

        ' record that we are inside the Interrupt routine
        gInsideTimerServiceRoutine = True

        ' -----------------------------------
        ' If we are not currently digitizing, then bail out of here
        ' (we could get here if the Interrupt Timer still has some
        ' interrupt routines in it's queue).

        If Not (gCurrentlyDigitizing) Then

            gInterruptRoutineQueueStillHotAfterDigitizing += 1

            ' make sure the Interrupt Timer is no longer running (we Stop and Dispose)
            If Not IsNothing(gDigitizeProcessTimer) Then
                Stop_And_Dispose_Of_Timer()
            End If

            ' Exit out of here
            e = 1 : GoTo ExitFunction

        End If

        ' -----------------------------------
        ' Keep track of how many times we came in here.

        ' inc the timer counter
        gDigitizeProcessCtr = gDigitizeProcessCtr + 1

        ' -----------------------------------
        ' SERVICE iNet's DIGITIZE BUFFERS
        '
        ' It is ABSOLUTELY NECESSARY to let your computer's processor
        ' service iNet digitizing buffers periodicaly (10 to .3 times
        ' per second) while digitizing.

        e = Service_All_iNet_Digitize_Buffers(netNum_1stController, _
            gTheNextDataAccessIsTheLast_ofEntireDigitization, _
            gTheNextDataAccessWillPullToTheEndOfAScan, _
            gControllerIsFinishedWithDigitization, _
            gLast_scanNum_base1_pulled_inFull, gLast_scanNum_base1_pushed_inFull)

        If ErrorHere(e, "ServiceDigitizeBuffers()") Then GoTo ExitAndStopDigitizing

        gNumPointsToPull = 0 ' Initialize the number of points to pull out of buffers

        '-----------------------------------
        ' Pull out of the buffer.
        '
        ' If we are to process the data little segments at a time, or
        ' we are processing 1 scan at at time and we have that 1
        ' scan available in the buffers, then go process the data.

        Dim numCalculatedMagnitudes As Integer
        If ((gProcessSegmentsInsteadOfScans <> iNetFalse) Or (gTheNextDataAccessWillPullToTheEndOfAScan <> iNetFalse)) Then

            ' Service the channels that have been turned on for digitizing
            For cNum = 0 To (gNumDigiCh - 1)

                '-----------------------------------
                ' READ DIGITIZED DATA
                '
                ' Here we get an address in memory where
                ' the most recent digitized data resides.
                '
                ' 'bufferStartAddrP[]'
                '
                ' iNetFLT32 data buffer that is
                ' 1 Scan long
                '
                ' 'pointToPullindex_base1'
                '
                ' new data is available starting at
                ' bufferStartAddrP[pointToPullindex_base1 - 1]
                '
                ' 'numPointsToPull'
                '
                ' number of new data points that are
                ' available
                '
                ' 'scanNumberToPull_base1'
                '
                ' scan number {1...numScans} of the
                ' scan that we are currently accessing.accessing

                ' We use USER_RAM_BUFFER [i.e. Visual Basic array
                ' digitizeData(ptNum, cNum)] instead of DRIVER_RAM_BUFFER.

                gUseDriverRamBuffer = iNetFalse

                e = Access_Digitized_Data_In_Ram_Buffer(gDigiChan(cNum).chAddrDLL, _
                    gUseDriverRamBuffer, gBufferStartAddrP, gPointToPullindex_base1, _
                    gNumPointsToPull, gScanNumberToPull_base1)

                If ErrorHere(e, "AccessDigitizeData()") Then GoTo ExitAndStopDigitizing

                ' ---------------------------------------------------------------
                ' Test to see if ptr stored in DLL is correct, gsw 6/1/02

                If gUseDriverRamBuffer = iNetFalse Then
                    With gDigiChan(cNum).dataArrayY
                        ' if the pinned address is not the same, we have a problem
                        If (.pinnedAddress <> gBufferStartAddrP) Then
                            MsgBox("Hey, we have an error at Access_Digitized_Data_In_Ram_Buffer(). Did the VB array move after we gave it's address to the instruNet driver?")
                            GoTo ExitAndStopDigitizing
                        End If
                    End With
                End If

                ' ---------------------------------------------------------------
                ' calc indices into array of data where we just pulled in new data (i.e. we
                ' process little segments as they are digitized.

                gSegmentStart_ptNum = gPointToPullindex_base1 - 1
                gSegmentEnd_ptNum = gSegmentStart_ptNum + gNumPointsToPull - 1

                ' make sure segment # does not go past the end of the SCAN buffer
                Dim gSegmentEnd_ptNum_MAX As Integer = gPtsPerScan_currentData - 1
                If gSegmentEnd_ptNum > gSegmentEnd_ptNum_MAX Then gSegmentEnd_ptNum = gSegmentEnd_ptNum_MAX

                ' update the # of valid data pts that are currently in the data buffer.
                gDigiChan(cNum).dataArrayY.numValidPts = gSegmentEnd_ptNum + 1

                ' determine if this channel is linked to a display
                bNum = gDigiChan(cNum).bNum
                dNum = gDigiChan(cNum).dNum
                chanLinkedToDisplay = bNum >= 0 And dNum >= 0

                ' ---------------------------------------------------------------
                ' DO FFT's AS NEEDED.

                ' if we finished the scan and are massaging an fft...
                If (gTheNextDataAccessWillPullToTheEndOfAScan <> False) And _
                   (gDigiChan(cNum).massageType = jFFTMassage) Then

                    ' calc the # of pts (numPts = 2 ^ power)
                    power = Int(System.Math.Log(gSegmentEnd_ptNum + 1) / System.Math.Log(2))

                    ' do an fft on the digitize data, in place; and update
                    ' the data array's numValidPts/ptsPerXunit fields.
                    Call FFT_DataArray(gDigiChan(cNum).dataArrayY, power, numCalculatedMagnitudes, gSampleRate_currentData)

                End If

                ' ---------------------------------------------------------------
                ' RUN END USER DATA PROCESSING ROUTINE

                If gDigiChan(cNum).runEndUser_SegmentProcessor And _
                   gRun_EndUser_SegmentProcessor Then

                    ' Run the user's waveform processing routine (inside
                    ' the user's source code file).
                    Call gEndUser_CLASS.EndUser_SegmentProcessor(cNum, _
                        gSegmentStart_ptNum, gSegmentEnd_ptNum, scanNumberToPull_base1)

                End If

                ' ---------------------------------------------------------------
                ' CLEAR DISPLAYS AS NEEDED.

                ' if this data is tied to a display...
                If chanLinkedToDisplay = True Then

                    ' if we are to clear the display right before plotting...
                    If gBox(bNum).disp(dNum).clrBeforePlot = True Then

                        ' clear the plot region, and give it a new grid
                        Call Plot_Clear_OneDisp(bNum, dNum, gEndUser_GRAPHICS)

                        Call Grid_Draw_OneDisp(bNum, dNum, gBox(bNum), _
                            gBox(bNum).disp(dNum), gEndUser_GRAPHICS)

                    End If

                    ' if we finished the scan, then clear & apply a grid right
                    ' before we next plot...
                    If (gTheNextDataAccessWillPullToTheEndOfAScan <> False) And _
                        gContinuousScans = False And _
                        gTheNextDataAccessIsTheLast_ofEntireDigitization = False Then

                        gBox(bNum).disp(dNum).clrBeforePlot = True
                    Else
                        gBox(bNum).disp(dNum).clrBeforePlot = False
                    End If

                End If

                ' ---------------------------------------------------------------
                ' PLOT DATA AS NEEDED

                ' if this data is tied to a display...
                If chanLinkedToDisplay = True Then

                    ' If we are to process little segments, then plot this little segment.
                    If (gProcessSegmentsInsteadOfScans And _
                       (gDigiChan(cNum).massageType <> jFFTMassage)) Then

                        ' Plot entire display, from left to right edges
                        Call Plot_Segment_OneDisp(bNum, dNum, gBox(bNum), _
                            gBox(bNum).disp(dNum), gSegmentStart_ptNum, _
                            gSegmentEnd_ptNum, gEndUser_GRAPHICS)

                    Else

                        ' if we finished the scan...
                        If (gTheNextDataAccessWillPullToTheEndOfAScan <> False) Then

                            ' Plot entire display, from left to right edges
                            Call Plot_Segment_OneDisp(bNum, dNum, gBox(bNum), _
                                gBox(bNum).disp(dNum), 0, _
                                (gDigiChan(cNum).dataArrayY.numValidPts - 1), _
                                gEndUser_GRAPHICS)

                        End If

                    End If

                End If

                ' We process data one channel at a time.
            Next cNum

            'This "End If" Matches: If ((processOneScanAtATime = iNetFalse) Or (theNextDataAccessWillPullToTheEndOfAScan = iNetTrue)) Then
        End If

        ' ----------------------------------------------------------------
        ' IF AT END OF ENTIRE DIGITIZATION...

        ' If we are at the end of the entire digitization, then power down
        ' the digitizing.

        If gTheNextDataAccessIsTheLast_ofEntireDigitization <> iNetFalse Then

ExitAndStopDigitizing:

            ' shut down the digitizing, stop the timer from firing
            Call DigitizeProcess_Stop()

        End If

        ' ----------------------------------------------------------------
        ' Exit

ExitFunction:

        DigitizeProcess_ServiceIncomingData = e

        ' record that we are no longer inside this Interrupt routine
        gInsideTimerServiceRoutine = False

    End Function




    ' -------------------------------------------------------------------------
    ' FUNCTION    Stop_And_Dispose_Of_Timer()
    '
    ' PURPOSE     Stops and Disposes of a Timer.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Sub Stop_And_Dispose_Of_Timer()

        If IsNothing(gDigitizeProcessTimer) Then Exit Sub

        gDigitizeProcessTimer.Stop()

        gDigitizeProcessTimer.Dispose()

        gDigitizeProcessTimer = Nothing

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    METHOD_Channels_Button_Click()
    '
    ' PURPOSE     User just pressed the Channel Setup button, and we need
    '             to open the instruNet World network window.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub METHOD_Channels_Button_Click()

        Dim massageType, e, bNum As Integer

        ' if we are currently digitizing, then don't allow one to open this window.
        If gCurrentlyDigitizing Then
            MsgBox("We cannot open this window while digitizing.")
            Exit Sub
        End If

        ' Disable all channels selected in the instruNet World Network page
        ' (this kills the DRIVER RAM buffers).
        e = DisableAllChannelsForDigitizing()
        If ErrorHere(e, "mcb1") Then Exit Sub

        ' Select in the instruNet World Network window the channels in
        ' the DigiChan() list.
        ' (this also sets the PtsPerScan parameter inside iNet32.dll)
        Call VbDataBuffers_CopyToDLLDvrBuffers()

        ' Open the instruNet World network page.
        Open_instruNet_Window_VB(iNetCM_OpenWindow_Network)

        ' If we are to copy the Enabled channels in the Network page
        ' to rebuild the Digitize list, after the user closed the Network Window.
         If gCopyChannelsFromNetworkPage Then

            ' Disable all channels in the DigiChan() list, and then load it with
            ' the channels Selected in the instruNet World network page.
            massageType = jNoMassage
            Call DigitizeList_CopyFromDriverDLL(massageType, gRun_EndUser_SegmentProcessor)

            ' Automatically add channels to DisplayBoxes that are set up to do this.
            For bNum = 0 To gNumBoxes - 1

                ' if we are automatically add channels to this box...
                If (gBox(bNum).autoAddPolicy <> hAutoAddNone) Then

                    ' Automatically add Digitize Channels to this display box.
                    '
                    ' This first Deletes all Displays in this DisplayBox before
                    ' it adds the channels to the box, one channel per Display
                    ' in this one DisplayBox.
                    '
                    ' This will set the display data() buffers to PtsPerScan_nextDigi size.
                    AutoAddChannelsToThisDisplayBox(bNum, gEndUser_GRAPHICS)

                    ' Clear 1 DispBox and then Redraw it
                    Call Redraw_OneBox(bNum, gEndUser_GRAPHICS)

                End If

            Next bNum

        End If

        ' Disable all DigitizeChannels inside the iNet driver (kills the DRIVER RAM buffers)
        e = DisableAllChannelsForDigitizing()
        If ErrorHere(e, "mcb2") Then Exit Sub

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    DigitizeProcess_SetTimingParams()
    '
    ' PURPOSE     Sets the digitize timing parameters.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------   
    Public Sub DigitizeProcess_SetTimingParams( _
        ByVal ptsPerScan As Integer, _
        ByVal numScans As Integer, _
        ByVal sampleRate As Double, _
        ByVal continousScans As Boolean, _
        ByVal processSegmentsInsteadOfScans As Boolean, _
        ByVal run_EndUser_PostAcqAnalysis As Boolean, _
        ByVal run_EndUser_SegmentProcessor As Boolean, _
        ByVal copyChannelsFromNetworkPage As Boolean, _
        ByVal waitTillDigitizationCompletes As Boolean)

        gNumScans = numScans ' # of scans to digitize

        ' if true, we run with scans that are continous in time; otherwise, we retrigger for
        ' each If stripChartRecorder <> 0 Then stripChartRecorder = True
        gContinuousScans = continousScans

        ' if true, we process a scan when it is finished digitizing; otherwise, we process
        ' little(segments) as they are recorded.
        If processSegmentsInsteadOfScans <> 0 Then processSegmentsInsteadOfScans = True
        gProcessSegmentsInsteadOfScans = processSegmentsInsteadOfScans

        ' If true, then run EndUser_PostAcqAnalysis() at the end of the digitization.  This
        ' is an end user routine that processes the resulting data
        If run_EndUser_PostAcqAnalysis <> 0 Then run_EndUser_PostAcqAnalysis = True
        gRun_EndUser_PostAcqAnalysis = run_EndUser_PostAcqAnalysis

        ' If true, then run EndUser_SegmentProcessor() while data is being digitized.  This
        ' is an end user routine that processes little segments of data.
        If run_EndUser_SegmentProcessor <> 0 Then run_EndUser_SegmentProcessor = True
        gRun_EndUser_SegmentProcessor = run_EndUser_SegmentProcessor

        ' If true, then DigitizeProcess_Start() waits for the entire digitization to complete;
        ' otherwise, the Timer control triggers the servicing of the digitize buffers, every
        ' 200ms or so, while digitizing.
        If waitTillDigitizationCompletes <> 0 Then waitTillDigitizationCompletes = True
        gWaitTillDigitizationCompletes = waitTillDigitizationCompletes

        gPtsPerScan_nextDigi = ptsPerScan           ' points per scan per channel
        gPtsPerScan_currentData = 0                 ' start w/ no data in ram

        gSampleRate_nextDigi = sampleRate           ' sample rate per channel (pts/sec)
        gSampleRate_currentData = gSampleRate_nextDigi

        ' True if we are to copy the Enabled channels in the Network page to rebuild the
        ' Digitize() list, after the user closed the Network Window.
        If copyChannelsFromNetworkPage <> 0 Then copyChannelsFromNetworkPage = True
        gCopyChannelsFromNetworkPage = copyChannelsFromNetworkPage

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION  Digitize_Initialize()  
    '
    ' PURPOSE   Initialize instruNet digitize code
    '
    ' HISTORY   6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------   
    Public Sub Digitize_Initialize()

        ' ---------------------------------------------------------------
        ' Initialize digitization parameters to default states
        '
        ' If you want these to be different, then define them
        ' differently after calling In Initialize_instruNet_MAIN().

        ' # of channels in the Digitize Channel List (we start with 0). This increments
        ' when you call AddToDigitzeChannelList().
        gNumDigiCh = 0

        ' # of scans to digitize
        gNumScans = 1

        ' If true, we run with scans that are continous in time; otherwise, 
        ' we retrigger for each.
        gContinuousScans = False

        ' If false, we process a scan when it is finished digitizing; otherwise,
        ' we process little segments as they are recorded.
        gProcessSegmentsInsteadOfScans = True

        gPtsPerScan_nextDigi = 1000         ' points per scan per channel
        gPtsPerScan_currentData = 0         ' start w/ no data in ram

        gSampleRate_nextDigi = 10000        ' sample rate per channel (pts/sec)
        gSampleRate_currentData = gSampleRate_nextDigi

        gMaxNumOfDigiChannels = 128         ' max # of digitize channels

        '                                     used to store a list of digitize channels
        ReDim Preserve gDigiChan(gMaxNumOfDigiChannels)

        gCurrentlyDigitizing = False        ' not currently digitizing

        gDigitizeProcessCtr = 0             ' timer counter

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    DigitizeList_AddOneItem()
    '
    ' PURPOSE     Add a channel to the list of channels to digitize.
    '
    '             'massageType' is set to {jNoMassage, jFFTMassage} and indicates how
    '             the data is to be massaged immediately after it is digitized.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------   
    Public Function DigitizeList_AddOneItem( _
        ByVal netNum As Short, _
        ByVal devNum As Short, _
        ByVal modNum As Short, _
        ByVal chanNum As Short, _
        ByVal massageType_arg As Short, _
        ByVal runEndUser_SegmentProcessor As Short) As Short

        ' ------------------------------------------------
        ' Local Variables

        Dim e, chExists, cNum_new As Short

        e = iNetErr_None

        cNum_new = gNumDigiCh           ' channel number to work on
        gNumDigiCh = gNumDigiCh + 1     ' increment total # of channels

        ' ------------------------------------------------
        ' ReAllocate memory for the array that describes the channels.

        ReDim Preserve gDigiChan(gNumDigiCh)

        ' ------------------------------------------------
        ' Set channel parameters

        With gDigiChan(cNum_new)

            ' remember the address in these VB fields
            .netNum = netNum
            .deviceNum = devNum
            .moduleNum = modNum
            .chanNum = chanNum

            .bNum = -1      ' we are not tied to a display at this time
            .dNum = -1

            ' If true, then the digitize code calls EndUser_SegmentProcessor()
            ' each time a little segment of data is digitized.
            .runEndUser_SegmentProcessor = runEndUser_SegmentProcessor

            ' 'massageType' is set to {jNoMassage, jFFTMassage} and indicates how
            ' the data is to be massaged immediately after it is digitized.
            .massageType = massageType_arg

            ' Resets the data(), numPts, and sRate parameters for a digitize channel,
            ' resizes the data() array, and zeros it.
            DigitizeList_ResetDataArray(cNum_new)

            ' load channel into the DigiCh() list
            chExists = LoadChannelAddress_VB(gDigiChan(cNum_new), netNum, devNum, modNum, chanNum)

            If chExists = iNetFalse Then ' if channel does not exists, holler
                e = NotifyWeCannotLocateThisChannel(gDigiChan(gNumDigiCh).chAddrDLL)
                DigitizeList_AddOneItem = False
            Else
                DigitizeList_AddOneItem = True ' channel exists ok
            End If

        End With

    End Function


    ' -------------------------------------------------------------------------
    ' FUNCTION    DigitizeList_ResetDataArray()
    '
    ' PURPOSE     Resets the data(), numPts, and sRate parameters for a
    '             digitize channel, resizes the data() array, and zeros it. 
    '             This function does this is based on gPtsPerScan_nextDigi
    '             and gSampleRate_nextDigi values.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------   
    Public Sub DigitizeList_ResetDataArray(ByVal cNum As Integer)

        Dim ptNum As Integer
        Dim last_ptNum As Integer
        Dim numCalculatedMagnitudes As Integer
        Dim power As Integer

        ' ------------------------------------------------
        ' Set channel parameters.

        With gDigiChan(cNum).dataArrayY

            ' if we pull in a timewaveform...
            If gDigiChan(cNum).massageType = jNoMassage Then

                ' set the sample rate for this data
                .ptsPerXunit = gSampleRate_nextDigi

                ' first point time in seconds
                .firstPt_Xunits = 0

                ' no of valid pts in the buffer
                .numValidPts = gPtsPerScan_nextDigi

                ' buffer size
                .numBufferPts = gPtsPerScan_nextDigi + 2

                ' else if we do an fft immediately after digitizing...
            ElseIf gDigiChan(cNum).massageType = jFFTMassage Then

                ' calc the # of pts (numPts = 2 ^ power)
                power = Int(System.Math.Log(gPtsPerScan_nextDigi) / System.Math.Log(2))

                ' update the # of resulting freq amplitude points
                numCalculatedMagnitudes = (2 ^ power) / 2

                ' update the pts-per Hz unit (we are plotting amplitudes vs. Hz).
                .ptsPerXunit = numCalculatedMagnitudes / (gSampleRate_currentData / 2)

                ' first point time in seconds
                .firstPt_Xunits = 0

                ' no of valid pts in the buffer
                .numValidPts = numCalculatedMagnitudes

                ' buffer size
                .numBufferPts = gPtsPerScan_nextDigi + 2

            Else

                EndProgram_iNet("bad massageType")

            End If

            ' ------------------------------------------------
            ' Make sure the data array is not pinned. gsw 6/1/02

            If (.arrayIsPinned) Then
                .gcHandle.Free()       ' unlock the data() handle
            End If

            .gcHandle = Nothing        ' handle to data() array, after being pinned
            .pinnedAddress = 0         ' address to data() array, after being pinned
            .arrayIsPinned = False     ' true if data() has been pinned

            ' ------------------------------------------------
            ' ReAllocate memory for the digitized data.
            '
            ' Digitize data() contains the digitized data in the form:
            '   gDigiChan(0...gNumDigiCh).dataArrayY.data(0..numPts)

            ' resize the data() array
            ReDim Preserve .data(.numBufferPts)

            ' ------------------------------------------------
            ' Zero the digitize data array.

            last_ptNum = .numBufferPts - 1
            For ptNum = 0 To last_ptNum
                .data(ptNum) = 0
            Next ptNum

        End With

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    DigitizeList_ResetList()
    '
    ' PURPOSE     Resets the number of items in the list to 0
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------   
    Public Sub DigitizeList_ResetList()

        ' delete all the data and reset the number of items in the
        ' Digitize Channel list to 0
        ReDim gDigiChan(0)

        ' establish there are no items in the list
        gNumDigiCh = 0

    End Sub


    '-------------------------------------------------------------------------------------
    ' FUNCTION   METHOD_DigitizeProcess_Timer()
    '
    ' PURPOSE    Respond to the periodic interrupt coming from the front
    '            panel timer control, which  tells vb to services the digitize buffers.
    '
    ' HISTORY    6/5/02 - updated, gsw
    '-------------------------------------------------------------------------------------
    Public Sub METHOD_DigitizeProcess_Timer(ByVal myObject As Object, ByVal myEventArgs As EventArgs)

        ' takes care of incoming data
        DigitizeProcess_ServiceIncomingData()

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    DigitizeProcess_SetControlItems()
    '
    ' PURPOSE     Sets the StartStopButton control.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------   
    Public Sub DigitizeProcess_SetControlItems( _
        ByRef StartStopButton As System.Windows.Forms.Button)

        ' "Start"/"Stop" button, set to "Nothing" if doesn't exist
        gStartStopButton = StartStopButton

        ' End User no longer needs to place a timer control on the front panel

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    LoadChannelAddress_VB()
    '
    ' PURPOSE     Loads an instruNet channel address into a digiChAddr() structure.
    '             This structure holds the address in a vb format, and a format
    '             that is understood by the instruNet DLL.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------   
    Public Function LoadChannelAddress_VB( _
        ByRef digiChAddr As chAddr, _
        ByRef netNum As Short, _
        ByRef devNum As Short, _
        ByRef modNum As Short, _
        ByRef chanNum As Short) As Short

        ' ------------------------------------------------
        ' Local Variables

        Dim chExists As Short

        ' load channel address into a LONG, to be used by the instruNet DLL
        chExists = LoadChannelAddress(digiChAddr.chAddrDLL, netNum, devNum, modNum, chanNum)

        LoadChannelAddress_VB = chExists

        ' load channel address into several VB fields, to be used by VB
        digiChAddr.netNum = netNum
        digiChAddr.moduleNum = modNum
        digiChAddr.deviceNum = devNum
        digiChAddr.chanNum = chanNum

    End Function


    ' -------------------------------------------------------------------------
    ' FUNCTION    METHOD_StartStopButton_Click()
    '
    ' PURPOSE     This handles a mouse down on the Digitize "Start"/"Stop" button.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------   
    Public Sub METHOD_StartStopButton_Click()

        If gCurrentlyDigitizing Then        ' if we are currently digitizing...
            DigitizeProcess_Stop()          ' tell instruNet to stop digitizing
        Else                                ' Else we are not digitizing, and need to
            DigitizeProcess_Start()         ' tell instruNet to start digitizing
        End If

    End Sub


    ' ----------------------------------------------------------------
    ' FUNCTION    Digitize_iNet_Data()
    '
    ' PURPOSE     This routine digitizes one scan of data, and then returns
    '             control back to the caller.  The digitization is based
    '             on the following passed arguments.
    '
    ' PARAMETERS  processOneScanAtATime
    '                   If true (1), we access digitized data 1 complete
    '                   Scan at a time; otherwise if FALSE (0), we process
    '                   little segments of data, as they become available.
    '
    '             ptsPerScan
    '                   Number of points per scan.
    '
    '             ptsPerSecPerChan
    '                   Sample rate in Points-Per-Second-Per-Channel
    '
    '             numScans
    '                   number of scans to acquire
    '
    '             numDigiCh
    '                   number of channels to digitize
    '
    '             stripChartRecorder
    '                   if TRUE (1) we do Strip Chart Recorder work with
    '                   Continuous Scans, otherwise if FALSE (0) we do
    '                   Oscilloscope mode (non-Coninuous Scans)
    '
    '             digiChAddr(0...numDigiCh-1)
    '                   When this routine is called, this must contain
    '                   a list of the channels to digitize.
    '
    '                   Digitize_iNet_Data() loads the digitize data into:
    '                   digiChAddr(0...numDigiCh-1).dataArrayY.data(0...ptsPerScan)
    '
    ' HISTORY     2/2/96  created - gsw
    '             2/20/96  created Visual Basic version - sjm
    ' -------------------------------------------------------------------------
    Function Digitize_iNet_Data( _
        ByVal processOneScanAtATime As Short, _
        ByVal ptsPerScan As Integer, _
        ByVal ptsPerSecPerChan As Integer, _
        ByVal numScans As Integer, _
        ByRef numDigiCh As Short, _
        ByVal stripChartRecorder As Short, _
        ByRef digiChAddr() As chAddr) As Short

        ' ---------------------------------------------------------------
        ' Local Variables

        Dim currentValue As Double
        Dim processDataAsItIsAcquired, useDriverRamBuffer As Short
        Dim triggerThreshold As Double
        Dim setupTrigger, switchChannelsAccurately, showAlertOnOverflow As Short
        Dim message As String
        Dim userBufferSize, ptNum, lastPoint_Of_LittleSegment As Integer
        Dim theNextDataAccessIsTheLast_ofEntireDigitization As Short
        Dim bufferStartAddrP As Integer
        Dim weAreDoingOurFirstAccessOfData As Short
        Dim numPointsToPull, scanNumberToPull_base1 As Integer
        Dim theNextDataAccessWillPullToTheEndOfAScan, showAlertWas_ON_BeforeThisCall As Short
        Dim last_scanNum_base1_pulled_inFull, last_scanNum_base1_pushed_inFull As Integer
        Dim controllerIsFinishedWithDigitization As Short
        Dim triggerChannel As chAddr
        Dim sumOfPoints_fast As Double
        Dim firstPoint_Of_LittleSegment, pointToPullindex_base1 As Integer
        Dim netNum, deviceNum, moduleNum, chanNum, cNum, chExists, e As Short

        ' -----------------------------------------------------------------------
        ' Make sure the caller gave us a list of channels to digitize.

        e = iNetErr_None
        If numDigiCh = 0 Then ' if we have no channels to digitize...
            MsgBox("No channels have been selected to digitize.")
            GoTo ExitFunction
        End If

        ' Convert to instruNet DLL 1/0 true/false format.
        If stripChartRecorder <> 0 Then stripChartRecorder = True
        If processOneScanAtATime <> 0 Then processOneScanAtATime = True

        ' -----------------------------------------------------------------------
        ' Check the list of digitize channels, and make sure they are ok.  The
        ' caller passes us this list inside 'digiChAddr()'.

        For cNum = 0 To numDigiCh - 1

            ' get the channel address stored inside 'digiChAddr()'
            Call GetChannelAddress(digiChAddr(cNum).chAddrDLL, _
                netNum, deviceNum, moduleNum, chanNum)

            ' see if the channel exists
            chExists = THIS_CHANNEL_EXISTS(netNum, deviceNum, moduleNum, chanNum)

            If chExists = iNetFalse Then
                e = NotifyWeCannotLocateThisChannel(digiChAddr(cNum).chAddrDLL)
                Exit Function
            End If

        Next

        ' -----------------------------------------------------------------------
        ' Allocate a data() array for the digitized data.

        For cNum = 0 To numDigiCh - 1
            With gDigiChan(cNum)
                ReDim Preserve .dataArrayY.data(ptsPerScan + 2)
            End With
        Next cNum

        ' -----------------------------------------------------------------------
        ' Enabled digitizing for the selected channels and prepare data buffer.

        e = VbDataBuffers_AttachToDLLUserBuffers(numDigiCh, digiChAddr, ptsPerScan)
        If ErrorHere(e, "Digitize_iNet_Data()") Then GoTo ExitFunction

        ' -----------------------------------------------------------------------
        ' Set TIMING parameters.  Tell it to digitize 1000 points per Scan at 100
        ' points per second per channel (i.e. 10sec Scans) and do this
        ' 3 times (i.e. 3 Scans).  We also tell iNet to put the digitized data
        ' into the DRIVER ram buffer (we will get access to this buffer as the
        ' data is digitized).  And we tell iNet to show an alert if we
        ' overflow a data buffer.  stripChartRecorder mode is turned ON, which
        ' means that the Scans will be continuous (last point of one Scan is directly
        ' followed by first point of next Scan).

        showAlertOnOverflow = iNetTrue
        switchChannelsAccurately = iNetTrue

        ' We are setting the timing parameters in here.
        e = Set_iNet_TIMING_Parameters(ptsPerSecPerChan, ptsPerScan, numScans, _
            stripChartRecorder, ion_gSaveDataPopup_ToRamBuffer, showAlertOnOverflow, _
            switchChannelsAccurately)

        If ErrorHere(e, "Set_iNet_TIMING_Parameters()") Then GoTo ExitFunction

        ' -----------------------------------------------------------------------
        ' Set TRIGGER parameters.  We specify the trigger source to be the
        ' channel specified in the 'triggerChannel' address. We also say that the
        ' channel is set to trigger on the rising edge of the source when it
        ' crosses 0.0.  At the moment, the trigger function is deactivated
        ' (i.e. ion_TriggerMode_Off).

        setupTrigger = iNetFalse

        If setupTrigger = iNetTrue Then

            'Vh1 Vin+ is the trigger channel
            chExists = LoadChannelAddress_VB(triggerChannel, 1, 1, 1, 1)

            triggerThreshold = 0.0#

            ' We are setting the trigger to be OFF in here.
            e = Set_iNet_TRIGGER_Parameters(ion_TriggerMode_Off, triggerThreshold, _
                ion_Rising, 0, triggerChannel.chAddrDLL)

            If ErrorHere(e, "Set_iNet_TRIGGER_Parameters()") Then GoTo ExitFunction

        End If

        '-----------------------------------------------------------------------
        ' Tell iNet driver to show an alert if there is an error (in addition
        ' to returning an error code).

        showAlertWas_ON_BeforeThisCall = SHOW_MESSAGE_DIALOG_UPON_ERROR()

        ' -----------------------------------------------------------------------
        ' START DIGITIZING

        ' Press the start button to tell iNet to begin digitizing.
        e = PRESS_iNet_BUTTON(iNetCM_BtnPress_Record_Start)
        If ErrorHere(e, "StartDigitizeCommand") Then GoTo ExitFunction

        ' say that we are starting out
        weAreDoingOurFirstAccessOfData = iNetTrue
        theNextDataAccessIsTheLast_ofEntireDigitization = iNetFalse

        Do  ' loop until the digitization is finished..

            '-----------------------------------
            ' SERVICE iNet's DIGITIZE BUFFERS
            '
            ' It is ABSOLUTELY NECESSARY to let your computer's processor
            ' service iNet digitizing buffers periodicaly (10 to .3 times
            ' per second) while digitizing.

            e = Service_All_iNet_Digitize_Buffers(netNum_1stController, _
                theNextDataAccessIsTheLast_ofEntireDigitization, _
                theNextDataAccessWillPullToTheEndOfAScan, _
                controllerIsFinishedWithDigitization, _
                last_scanNum_base1_pulled_inFull, last_scanNum_base1_pushed_inFull)

            If ErrorHere(e, "ServiceDigitizeBuffers()") Then GoTo ExitFunction

            numPointsToPull = 0 ' Initialize the number of points to pull out of buffers

            '-----------------------------------
            ' Pull out of the buffer.
            '
            ' If we are to process the data little segments at a time, or
            ' we are processing 1 scan at at time and we have that 1
            ' scan available in the buffers, then go process the data.

            If ((processOneScanAtATime = iNetFalse) Or (theNextDataAccessWillPullToTheEndOfAScan = iNetTrue)) Then

                ' service the channels that have been turned on for digitizing
                For cNum = 0 To (numDigiCh - 1)

                    '-----------------------------------
                    ' READ DIGITIZED DATA
                    '
                    ' Here we get an address in memory where
                    ' the most recent digitized data resides.
                    '
                    ' 'bufferStartAddrP[]'
                    '
                    ' iNetFLT32 data buffer that is
                    ' 1 Scan long
                    '
                    ' 'pointToPullindex_base1'
                    '
                    ' new data is available starting at
                    ' bufferStartAddrP[pointToPullindex_base1 - 1]
                    '
                    ' 'numPointsToPull'
                    '
                    ' number of new data points that are
                    ' available
                    '
                    ' 'scanNumberToPull_base1'
                    '
                    ' scan number {1...numScans} of the
                    ' scan that we are currently accessing.accessing

                    ' We use USER_RAM_BUFFER [i.e. Visual Basic array
                    ' digitizeData(ptNum, cNum)] instead of DRIVER_RAM_BUFFER.
                    useDriverRamBuffer = iNetFalse

                    e = Access_Digitized_Data_In_Ram_Buffer(digiChAddr(cNum).chAddrDLL, _
                        useDriverRamBuffer, bufferStartAddrP, pointToPullindex_base1, _
                        numPointsToPull, scanNumberToPull_base1)

                    If ErrorHere(e, "AccessDigitizeData()") Then GoTo ExitFunction

                    ' test
                    ' OLD: If (iNet_Get_VarPtr(gDigiChan(cNum).dataArrayY.data(0)) <> bufferStartAddrP) Then
                    ' OLD: MsgBox("Hey, we have an error at Access_Digitized_Data_In_Ram_Buffer(). Did the VB array move after we gave it's address to the instruNet driver?")
                    ' OLD: GoTo ExitFunction
                    ' OLD: End If

                    '---------------------------------------------------------------
                    ' Process incoming data, as it is acquired, in little segments.

                    processDataAsItIsAcquired = False

                    If processDataAsItIsAcquired = True Then

                        ' calc indices into array of data where
                        ' we just pulled in new data (i.e. we
                        ' process little segments as they are digitized.
                        firstPoint_Of_LittleSegment = pointToPullindex_base1 - 1
                        lastPoint_Of_LittleSegment = firstPoint_Of_LittleSegment + numPointsToPull - 1

                        ' scan through each point in the little segment
                        For ptNum = firstPoint_Of_LittleSegment To lastPoint_Of_LittleSegment
                            currentValue = gDigiChan(cNum).dataArrayY.data(ptNum)
                        Next ptNum

                    End If

                    ' We process data one channel at a time.
                Next cNum

                'This "End If" Matches: If ((processOneScanAtATime = iNetFalse) Or (theNextDataAccessWillPullToTheEndOfAScan = iNetTrue)) Then
            End If

            ' We sit in loop while there is data to be pulled out of the buffers;
            ' if someone presses the STOP button, "theNextDataAccessIsTheLast"
            ' is set to iNetTrue, and this ends the digitizing.
        Loop Until theNextDataAccessIsTheLast_ofEntireDigitization <> iNetFalse

        ' ----------------------------------------------------------------
        ' Exit

ExitFunction:

        ' Now that we have finished digitizing, tell the instruNet driver
        ' to forget about these VB memory buffers that we let it use
        ' for digitization (this insures that the instruNet driver does not
        ' try to use them later on, when VB may have moved this).  This
        ' does not effect the DigitizeData() in VB memory.

        e = VbDataBuffers_DetachFromDLLUserBuffers(numDigiCh, digiChAddr)

        Digitize_iNet_Data = e

    End Function


    ' -------------------------------------------------------------------------
    ' FUNCTION    DigitizeProcess_Start()
    '
    ' PURPOSE     This routine starts the digitizing process, as it had
    '             been set up by:
    '
    '                   DigitizeList_AddOneItem()
    '                   DigitizeProcess_SetTimingParams()
    '
    ' -------------------------------------------------------------------------
    Function DigitizeProcess_Start() As Short

        Dim DEFAULT_PROCESS_TIME_MS As Integer
        Dim triggerThreshold As Double
        Dim cNum As Integer
        Dim chExists, netNum, deviceNum, moduleNum, chanNum, e As Short
        Dim showAlertOnOverflow, switchChannelsAccurately, setupTrigger As Short
        Dim triggerChannel As chAddr

        ' -------------------------------------------
        ' if we are currently digitizing, then bail out of here.

        If (gCurrentlyDigitizing) Then

            ' turn off the timer that caused this routine to be called
            ' (we might have multiple threads here, where we are processing
            ' one while another starts running).
            If Not IsNothing(gDigitizeProcessTimer) Then
                Stop_And_Dispose_Of_Timer()
            End If

            MsgBox("You cannot start digitizing while you are digitizing.")

            Exit Function

        End If

        gDigitizeProcessCtr = 0             ' Timer counter.
        gInsideTimerServiceRoutine = False  ' We are not in the Interrupt routine.
        gOverCallInterruptRoutine = 0       ' Overcall interrupt routine counter.
        '                                     Records how many interrupts routine
        '                                     calls are queued up after digitizing ends.
        gInterruptRoutineQueueStillHotAfterDigitizing = 0

        ' -------------------------------------------
        ' Make sure the caller gave us a list of channels to digitize.

        e = iNetErr_None
        DigitizeProcess_Start = e

        If gNumDigiCh = 0 Then ' if we have no channels to digitize...
            MsgBox("No channels have been selected to digitize.")
            e = 1
            GoTo ExitFunction
        End If

        ' -------------------------------------------
        ' Check the list of digitize channels, and make sure they are ok.  The
        ' caller passes us this list inside 'digiChAddr()'.

        For cNum = 0 To gNumDigiCh - 1

            ' get the channel address stored inside 'digiChAddr()'
            Call GetChannelAddress(gDigiChan(cNum).chAddrDLL, netNum, deviceNum, moduleNum, chanNum)

            ' see if the channel exists
            chExists = THIS_CHANNEL_EXISTS(netNum, deviceNum, moduleNum, chanNum)

            If chExists = iNetFalse Then
                e = NotifyWeCannotLocateThisChannel(gDigiChan(cNum).chAddrDLL)
                GoTo ExitFunction
            End If

        Next

        ' -------------------------------------------
        ' Measures the actual sample rate (also, this will set the
        ' PtsPerScan parameter within the iNet32.dll)

        gSampleRate_nextDigi = DigitizeProcess_MeasureActualSampleRate()
        gSampleRate_currentData = gSampleRate_nextDigi

        ' -------------------------------------------
        ' Update our understanding of the data.

        gPtsPerScan_currentData = gPtsPerScan_nextDigi

        ' -------------------------------------------
        ' Update the data 'numValidPts' and 'ptsPerXunit' field, and the
        ' data buffer sizes.  The reason we do this now, is so that the resetting
        ' of the controls will match the new data.

        For cNum = 0 To gNumDigiCh - 1

            ' Resets the data(), numPts, and sRate parameters for a digitize channel,
            ' resizes the data() array, and zeros it.
            DigitizeList_ResetDataArray(cNum)

        Next

        ' Update the horiz pts-per-xUnits (i.e. pts per pixel)
        Reset_Controls_AllBoxes(reset_ptsPerHscale, gEndUser_GRAPHICS)

        ' -------------------------------------------
        ' Enabled digitizing for the selected channels and prepare data buffer.
        ' This routine tells the instruNet DLL to digitize into the VB digitize data()
        ' buffers, and makes it very easy for VB code to access this data.

        e = VbDataBuffers_AttachToDLLUserBuffers(gNumDigiCh, gDigiChan, gPtsPerScan_nextDigi)
        If ErrorHere(e, "VbDataBuffers_AttachToDLLUserBuffers()") Then GoTo ExitFunction

        ' -------------------------------------------
        ' Set TIMING parameters.  Tell it to digitize 1000 points per Scan at 100
        ' points per second per channel (i.e. 10sec Scans) and do this
        ' 3 times (i.e. 3 Scans).  We also tell iNet to put the digitized data
        ' into the DRIVER ram buffer (we will get access to this buffer as the
        ' data is digitized).  And we tell iNet to show an alert if we
        ' overflow a data buffer.  stripChartRecorder mode is turned ON, which
        ' means that the Scans will be continuous (last point of one Scan is directly
        ' followed by first point of next Scan).

        showAlertOnOverflow = iNetTrue
        switchChannelsAccurately = iNetTrue

        ' We are setting the timing parameters in here.
        e = Set_iNet_TIMING_Parameters(gSampleRate_nextDigi, gPtsPerScan_nextDigi, _
            gNumScans, gContinuousScans, ion_gSaveDataPopup_ToRamBuffer, _
            showAlertOnOverflow, switchChannelsAccurately)

        If ErrorHere(e, "Set_iNet_TIMING_Parameters()") Then GoTo ExitFunction

        ' -------------------------------------------
        ' Set TRIGGER parameters.  We specify the trigger source to be the
        ' channel specified in the 'triggerChannel' address. We also say that the
        ' channel is set to trigger on the rising edge of the source when it
        ' crosses 0.0.  At the moment, the trigger function is deactivated
        ' (i.e. ion_TriggerMode_Off).

        setupTrigger = False

        If setupTrigger <> False Then

            'Vh1 Vin+ is the trigger channel
            chExists = LoadChannelAddress_VB(triggerChannel, 1, 1, 1, 1)

            triggerThreshold = 0.0#

            ' We are setting the trigger to be OFF in here.
            e = Set_iNet_TRIGGER_Parameters(ion_TriggerMode_Off, triggerThreshold, _
                ion_Rising, 0, triggerChannel.chAddrDLL)

            If ErrorHere(e, "Set_iNet_TRIGGER_Parameters()") Then GoTo ExitFunction

        End If

        ' -------------------------------------------
        ' Tell iNet driver to show an alert if there is an error (in addition
        ' to returning an error code).

        Dim showAlertWas_ON_BeforeThisCall As Short
        showAlertWas_ON_BeforeThisCall = SHOW_MESSAGE_DIALOG_UPON_ERROR()

        ' -------------------------------------------
        ' START DIGITIZING

        ' Press the start button to tell iNet to begin digitizing.
        e = PRESS_iNet_BUTTON(iNetCM_BtnPress_Record_Start)
        If ErrorHere(e, "BtnPress_Record_Start") Then GoTo ExitFunction

        ' say that we are starting out
        gWeAreDoingOurFirstAccessOfData = iNetTrue
        gTheNextDataAccessIsTheLast_ofEntireDigitization = iNetFalse

        ' report that we are currently digitizing
        gCurrentlyDigitizing = True

        ' Show "Stop" on Start/Stop button
        If ValidObject(gStartStopButton) Then gStartStopButton.Text = "Stop"

        ' -------------------------------------------
        ' CLEAR THE SCREEN & GET READY FOR DATA.
        '
        ' Specify that screens are to be cleared right before their next plot.

        Dim bNum As Short
        Dim dNum As Short
        For bNum = 0 To gNumBoxes - 1

            For dNum = 0 To (gBox(bNum).numDisp - 1)

                With gBox(bNum).disp(dNum)

                    ' we are attached to a digitize channel..
                    If .cNum <> NO_DIGI_CHAN Then

                        ' say that the display is NOT to be clear just before plotting
                        .clrBeforePlot = False

                        ' Clear the plot regions for this display
                        Call Plot_Clear_OneDisp(bNum, dNum, gEndUser_GRAPHICS)

                        ' Draw a grid in this display
                        Call Grid_Draw_OneDisp(bNum, dNum, gBox(bNum), _
                            gBox(bNum).disp(dNum), gEndUser_GRAPHICS)

                    End If

                End With

            Next dNum

        Next bNum

        ' -------------------------------------------
        ' If we are to wait till digitizing finishes...

        Dim scanTime_secs As Double
        Dim scanTime_ms As Double

        ' If wait till digitizing finishes ...
        If gWaitTillDigitizationCompletes Then

            ' When we first come here, gCurrentlyDigitizing
            ' will be TRUE, and we need to call 
            ' DigitizeProcess_ServiceIncomingData() multiple
            ' times until the digitization ends where
            ' DigitizeProcess_ServiceIncomingData() will set
            ' gCurrentlyDigitizing to FALSE.

            ' sit here while instruNet is digitizing
            Do

                ' Services incoming data from digitization process.
                Call DigitizeProcess_ServiceIncomingData()

            Loop While gCurrentlyDigitizing

        Else

            ' -------------------------------------------
            ' Otherwise, we do not wait for digitizing to finish and
            ' instead setup a timer that will fire every now and and will service the
            ' digitizing buffers at that time.  ONE MUST RETURN CPU CONTROL
            ' TO THE MAIN WINDOW SO THAT THE OS CAN SEND EVENTS TO THE
            ' WINDOW, IN ADDITION TO THE TIMER, OTHERWISE, THE TIMER
            ' WILL NOT RUN.

            ' make sure the Timer control has been placed onto front panel and
            ' it's name has been passe to DigitizeProcess_SetControlItems().

            ' calc the time to acquire 1 scan of data
            scanTime_secs = (1 / gSampleRate_nextDigi) * gPtsPerScan_nextDigi
            scanTime_ms = scanTime_secs * 1000

            ' Set the scan interval to once every 200ms, to make it so
            ' the buffer does not overflow, and if plotting segments,
            ' we will plot 5x per second.  If we are processing whole scans
            ' and each scan is 10secs, it will not hurt to service the
            ' buffers 5x per sec, and then do the processing automatically
            ' when we finish the scan.

            DEFAULT_PROCESS_TIME_MS = 200

            Dim timerInterval_ms As Integer = DEFAULT_PROCESS_TIME_MS ' 200ms

            If (timerInterval_ms > scanTime_ms) Then

                ' Set the timer interval in units of 1ms, to fire 2x
                ' per scan.  It should not fall behind given this rate.

                timerInterval_ms = scanTime_ms / 2

                ' Don't allow it to go less than 2ms

                If (timerInterval_ms < 2) Then
                    timerInterval_ms = 2
                End If

            End If

            ' We create a TIMER object that will execute a subroutine every timerInterval_ms
            ' milliseconds, after we wait timerInterval_ms.  We can then use this to service 
            ' the digitize buffers, if we are processing little segments at a time.
            ' The reason this goes through the end user CLASS is because the end user
            ' FORM controls the processor Timing and Threading.

            ' OLD: gDigitizeProcessTimer = New Timer( _
            ' OLD:      New TimerCallback(AddressOf gEndUser_CLASS.DigitizeProcess_Tick), _
            ' OLD:      Nothing, timerInterval_ms, timerInterval_ms)

            gDigitizeProcessTimer = New System.Windows.Forms.Timer()

            ' Adds the event and the event handler for the method that will
            ' process the timer event to the timer.
            AddHandler gDigitizeProcessTimer.Tick, AddressOf gEndUser_CLASS.DigitizeProcess_Tick

            ' Sets the timer interval to 5 seconds.
            gDigitizeProcessTimer.Interval = timerInterval_ms
            gDigitizeProcessTimer.Start()

            If IsNothing(gDigitizeProcessTimer) Then EndProgram_iNet("Timer did not set up properly.")

        End If

        ' -------------------------------------------
        ' EXIT

ExitFunction:

        DigitizeProcess_Start = e

    End Function


    ' -------------------------------------------------------------------------   
    ' FUNCTION    DigitizeProcess_WaitTillDone()
    '
    ' PURPOSE     Wait till the digitizing finishes.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------   
    Public Sub DigitizeProcess_WaitTillDone()

        Dim timeout As Double
        timeout = VB.Timer() + 3

        Do  ' wait till digitizing finishes

            ' NOTE:
            ' This does not always work, since you need to exit
            ' the program and give control to the WINDOW, so
            ' that the OS can then service the timer, and the
            ' WINDOW events interchangeably.  The Timer
            ' is a little funky in this way. gsw 8/11/98

            ' if 3secs have gone by
            If (timeout < VB.Timer()) Then

                ' if the Timer is not telling the digitize
                ' process to be serviced...
                If (gDigitizeProcessCtr = 0) Then

                    ' end the program
                    EndProgram_iNet("The gDigitizeProcessTimer does not appear to be moving.")

                End If

            End If

            ' sit here while instruNet is digitizing
        Loop While gCurrentlyDigitizing

    End Sub

    ' -------------------------------------------------------------------------
    ' FUNCTION    DigitizeProcess_Stop()
    '
    ' PURPOSE     This routine stops the digitizing process.
    '
    ' HISTORY     8/4/98  created - gsw
    ' -------------------------------------------------------------------------
    Public Function DigitizeProcess_Stop() As Short

        Dim bNum As Short

        Dim e As Short
        DigitizeProcess_Stop = iNetErr_None

        ' if we are not currently digitizing, then bail out of here
        If Not (gCurrentlyDigitizing) Then
            MsgBox("You cannot stop digitizing if you are not digitizing.")
            Exit Function
        End If

        ' tell instruNet to stop digitizing
        PRESS_iNet_BUTTON(iNetCM_BtnPress_Record_Stop)

        ' wait until instruNet has stopped
        Dim timeOutCtr As Integer ' 32bit
        timeOutCtr = 0
        Do

            timeOutCtr = timeOutCtr + 1
            If (timeOutCtr > 100000) Then EndProgram_iNet("stop timed out")

        Loop While (CONTROLLER_IS_DIGITIZING(1) <> False)

        ' report that we are no longer digitizing
        gCurrentlyDigitizing = False

        ' make sure the Interrupt Timer is no longer running (we Stop and Dispose)
        If Not IsNothing(gDigitizeProcessTimer) Then
            Stop_And_Dispose_Of_Timer()
        End If

        ' update the "Start"/"Stop" button text
        If ValidObject(gStartStopButton) Then gStartStopButton.Text = "Start"

        ' Now that we have finished digitizing, tell the instruNet driver
        ' to forget about these VB memory buffers that we let it use
        ' for digitization (this insures that the instruNet driver does not
        ' try to use them later on, when VB may have moved this).  This
        ' does not effect the DigitizeData() in VB memory.

        e = VbDataBuffers_DetachFromDLLUserBuffers(gNumDigiCh, gDigiChan)

        For bNum = 0 To gNumBoxes - 1 ' scan through all DisplayBoxes

            ' draw vertical marker line
            Call Marker_Draw_Line_OneBox(bNum, True, gBox(bNum), gEndUser_GRAPHICS)

            ' draw marker text at top of boxes
            Call Marker_Draw_Text_OneBox(bNum, gBox(bNum), gEndUser_GRAPHICS)

        Next bNum

        ' If true, then run EndUser_PostAcqAnalysis() at the end of the digitization.  This is an
        ' end user routine that processes the resulting data
        If gRun_EndUser_PostAcqAnalysis = True Then

            ' User routine that processed digitized data, when the digitization stops.
            Call gEndUser_CLASS.EndUser_PostAcqAnalysis()

        End If

        DigitizeProcess_Stop = e

    End Function


    ' -------------------------------------------------------------------------   
    ' FUNCTION    VbDataBuffers_AttachToDLLUserBuffers()
    '
    ' PURPOSE     Attach VB gDigiChan().dataArrayY.data() memory to
    '             iNet32.DLL USER RAM BUFFER, so that the iNet
    '             driver will digitize into VB ram.
    '
    ' HISTORY     2/25/97  created - unoWarrior
    ' -------------------------------------------------------------------------
    Function VbDataBuffers_AttachToDLLUserBuffers(ByRef numDigiCh As Short, _
        ByRef digiChAddr() As chAddr, ByVal ptsPerScan As Integer) As Short

        Dim userBufferSize As Long
        Dim e, cNum As Short

        'disable all channels for digitizing (initially)
        e = DisableAllChannelsForDigitizing()
        If ErrorHere(e, "DisableAllChannelsForDigitizing") Then GoTo ExitFunction

        'scan through all digitize channels
        For cNum = 0 To numDigiCh - 1

            ' get address to VB data() array's data and lock it's handle, gsw 6/1/02
            With digiChAddr(cNum).dataArrayY

                ' One needs to call VBDataBuffers_DetachFromDLLUserBuffers() before hitting this.
                If (.arrayIsPinned) Then Error ("Need to call VBDataBuffers_DetachFromDLLUserBuffers() before re-pinning.")

                ' resize the data() array
                ReDim Preserve .data(.numBufferPts)

                ' Get an address to the VB data() array's data, and lock it's handle
                .pinnedAddress = VbFlt32Array_to_Address(.data, .gcHandle)
                .arrayIsPinned = True     ' record that data() has been pinned

            End With

            ' tell the iNet driver to use the VB array memory as the
            ' destination for the data (we pass in the address if the
            ' 0th point for each channel).
            e = SetField_int32(digiChAddr(cNum).chAddrDLL, sgt_UserRamBuffer, _
                fldNum_UserBuffer_UserBufferAddr, digiChAddr(cNum).dataArrayY.pinnedAddress)
            If ErrorHere(e, "SetField_int32(userAddr)") Then GoTo ExitFunction

            ' tell the instruNet driver the length of the is VB buffer
            ' (4bytes for each "single" precision point)
            userBufferSize = 4 * ptsPerScan
            e = SetField_int32(digiChAddr(cNum).chAddrDLL, sgt_UserRamBuffer, _
                fldNum_UserBuffer_UserPtrSizeInBytes, userBufferSize)
            If ErrorHere(e, "SetField_int32(userBuffSize)") Then GoTo ExitFunction

            ' tell the instruNet driver to place digitize data into this buffer
            e = SetField_int32(digiChAddr(cNum).chAddrDLL, sgt_UserRamBuffer, _
                fldNum_UserBuffer_DigitizeOnOff, ion_gOnOffPopup_On)
            If ErrorHere(e, "SetField_int32(digitizeOnOff)") Then GoTo ExitFunction

            ' NOT GOOD, SINCE we end up with a USER and DRIVER buffer, and it randomly uses 1 of these)
            '   enable digitizing for each channel (set Digitize ON in Display settings area).
            '   e = EnableChannelForDigitizing(digiChAddr(cNum))
            '   If ErrorHere(e, "EnableChannelForDigitizing()") Then GoTo ExitFunction

        Next cNum

ExitFunction:

        VbDataBuffers_AttachToDLLUserBuffers = e

    End Function


    ' -------------------------------------------------------------------------   
    ' FUNCTION    VbDataBuffers_CopyToDLLDvrBuffers()
    '
    ' PURPOSE     Copy data from VB gDigiChan().dataArrayY.data() memory
    '             to the iNet32.DLL DRIVER RAM BUFFERS.
    '
    ' HISTORY     2/25/97  created - unoWarrior
    ' -------------------------------------------------------------------------
    Public Sub VbDataBuffers_CopyToDLLDvrBuffers()

        Dim littleTest As Double
        Dim numValidBytes As Long
        Dim cNum, e As Short
        Dim bufferAddr As Long ' 64bit integer
        Dim bufferPts, bufferBytes As Long  ' 64bit integer

        ' Set the timing parameters (e.g. PtsPerScan), so that the DRIVER RAM BUFFERS
        ' are set to the correct size.
        e = Set_iNet_TIMING_Parameters(gSampleRate_nextDigi, _
            gPtsPerScan_nextDigi + 6, gNumScans, gContinuousScans, _
            ion_gSaveDataPopup_ToRamBuffer, iNetTrue, iNetTrue)
        If ErrorHere(e, "ctd1") Then Exit Sub

        ' Disable all DigitizeChannels inside the iNet driver (we'll enable them below)
        e = DisableAllChannelsForDigitizing()
        If ErrorHere(e, "ctd2") Then Exit Sub

        ' scan through all Digitize Channels
        For cNum = 0 To (gNumDigiCh - 1)

            ' turn the DISPLAY:DIGITIZE field ON
            e = SetField_int32(gDigiChan(cNum).chAddrDLL, sgt_Display, _
                fldNum_Display_dispOnOff, ion_gOnOffPopup_On)
            If ErrorHere(e, "ctd3") Then Exit Sub

        Next cNum

        ' Sets up digitization, establishes sample rate, sets up BUFFERS, yet does not
        ' actually do the digitization. gsw 8/16/98
        Call SET_UP_DIGITIZATION()

        ' scan through all Digitize Channels
        For cNum = 0 To (gNumDigiCh - 1)

            ' get DRIVER RAM BUFFER size
            bufferBytes = GetField_int32(gDigiChan(cNum).chAddrDLL, sgt_DriverRamBuffer, _
                fldNum_DvrBuffer_BufferAddrSizeInBytes, e)
            If ErrorHere(e, "ctd6") Then Exit Sub

            ' calc # of valid pts in the buffer
            numValidBytes = gDigiChan(cNum).dataArrayY.numValidPts * 4

            ' make sure the buffer is large enough
            If (numValidBytes > bufferBytes) Then
                Call ErrorHere(1, "BufferTooSmall") : Exit Sub
            End If

            ' get the addr of the DRIVER RAM buffer (created inside iNet32.dll via NewPtr)
            bufferAddr = GetField_int32(gDigiChan(cNum).chAddrDLL, sgt_DriverRamBuffer, _
                fldNum_DvrBuffer_BufferAddr, e)
            If ErrorHere(e, "ctd7") Then Exit Sub

            ' make sure our array is long enough
            If numValidBytes > 0 Then
                littleTest = gDigiChan(cNum).dataArrayY.data((numValidBytes / 4) - 1)
            End If

            ' copy data into iNet DRIVER RAM buffer frin VB buffer
            Call iNet_memcopy_flt32_addr( _
                gDigiChan(cNum).dataArrayY.data, bufferAddr, numValidBytes / 4)

            ' set the DRIVER RAM buffer numValidPts
            e = SetField_int32(gDigiChan(cNum).chAddrDLL, sgt_DriverRamBuffer, _
                fldNum_DvrBuffer_PtNumIn, numValidBytes / 4)
            If ErrorHere(e, "ctd8") Then Exit Sub

        Next cNum

    End Sub


    ' -------------------------------------------------------------------------   
    ' FUNCTION    VbDataBuffers_CopyFromDLLDvrBuffers()
    '
    ' PURPOSE     Copy data from iNet32.DLL DRIVER RAM BUFFERS into
    '             VB gDigiChan().dataArrayY.data() memory.
    '
    ' HISTORY     2/25/97  created - unoWarrior
    ' -------------------------------------------------------------------------
    Public Sub VbDataBuffers_CopyFromDLLDvrBuffers()

        Dim cNum As Short
        Dim e As Short
        Dim bufferAddr As Long ' 64bit integer
        Dim bufferPts, bufferBytes As Long  ' 64bit integer

        For cNum = 0 To gNumDigiCh - 1 ' scan through all digitize channels

            With gDigiChan(cNum) ' w/ this channel...

                ' get the address of the DRIVER ram buffer (created inside iNet32.dll via NewPtr)
                bufferAddr = GetField_int32(gDigiChan(cNum).chAddrDLL, sgt_DriverRamBuffer, _
                    fldNum_DvrBuffer_BufferAddr, e)
                If ErrorHere(e, "cfd1") Then Exit Sub

                ' get the # of data bytes stored inside the DRIVER ram buffer
                bufferBytes = GetField_int32(gDigiChan(cNum).chAddrDLL, sgt_DriverRamBuffer, _
                    fldNum_DvrBuffer_BufferAddrSizeInBytes, e)
                If ErrorHere(e, "cfd2") Then Exit Sub

                ' make sure we do not load in too much data
                bufferPts = bufferBytes / 4
                If (bufferPts > .dataArrayY.numBufferPts) Then bufferPts = .dataArrayY.numBufferPts

                ' Set the size of the gDigiChan() dataArrayY, gsw 6/1/02
                ReDim Preserve .dataArrayY.data(.dataArrayY.numBufferPts)

                ' if we have some data...
                If (bufferPts > 0) Then ' copy data from iNet driver ram buffer into VB buffer
                    Call iNet_memcopy_addr_flt32(bufferAddr, .dataArrayY.data, bufferPts)
                End If

                ' scan through all Digitize Channels, and turn the DISPLAY:DIGITIZE field OFF
                ' to delete the temporary DRIVER RAM buffers.
                e = SetField_int32(gDigiChan(cNum).chAddrDLL, sgt_Display, _
                    fldNum_Display_dispOnOff, ion_gOnOffPopup_Off)
                If ErrorHere(e, "cfd3") Then Exit Sub

            End With

        Next cNum

    End Sub

    ' -------------------------------------------------------------------------   
    ' FUNCTION    VbDataBuffers_DetachFromDLLUserBuffers()
    '
    ' PURPOSE     After digitizing, this is used to tell the instruNet driver
    '             to forget about the VB memory buffers that we let it use
    '             for digitizing (this insures that the instruNet driver does not
    '             try to use them later on, when VB may have moved them or changed
    '             them).  This does not effect the DigitizeData() in VB memory.
    '
    ' PARAMETERS  numDigiCh
    '                   This is the number of channels that have
    '                   been selected for digitizing.
    '
    '             digiChAddr(0...numDigiCh-1)
    '                   This specifies the list of channels that have been
    '                   selected for digitizing.
    '
    '             Returns iNetError code.
    '
    ' HISTORY     2/25/97  created - gsw
    ' -------------------------------------------------------------------------
    Function VbDataBuffers_DetachFromDLLUserBuffers(ByRef numDigiCh As Short, _
        ByRef digiChAddr() As chAddr) As Short

        ' -------------------------------------------------
        ' Local Variables

        Dim addr As Long
        Dim Value As Double
        Dim testing, e, cNum As Short

        'scan through all digitize channels
        For cNum = 0 To numDigiCh - 1

            ' testing stuff
            testing = False
            If testing = True Then

                ' make sure driver ram buffer is off
                Value = GetField_int32(digiChAddr(cNum).chAddrDLL, sgt_DriverRamBuffer, _
                    fldNum_DvrBuffer_DigitizeOnOff, e)
                If ErrorHere(e, "SetField_int32(userAddr)") Then GoTo ExitFunction
                If Value = ion_gOnOffPopup_On Then
                    MsgBox("aaa")
                End If

                addr = GetField_int32(digiChAddr(cNum).chAddrDLL, sgt_DriverRamBuffer, _
                    fldNum_DvrBuffer_BufferAddr, e)
                If ErrorHere(e, "SetField_int32(userAddr)") Then GoTo ExitFunction
                If addr <> 0 Then
                    MsgBox("bbb")
                End If

            End If

            ' Turn ON the display digitize field (so that the Network page will show
            ' the channel as being turned on). This might also turn on the DriverRamBuffer,
            ' yet everything is turned off when we call DisableAllDigitizing().

            e = SetField_int32(digiChAddr(cNum).chAddrDLL, sgt_Display, _
                fldNum_Display_dispOnOff, ion_gOnOffPopup_On)
            If ErrorHere(e, "SetField_int32(dispOnOff)") Then GoTo ExitFunction

            ' turn off instruNet driver user buffer
            e = SetField_int32(digiChAddr(cNum).chAddrDLL, sgt_UserRamBuffer, _
                fldNum_UserBuffer_UserBufferAddr, 0)
            If ErrorHere(e, "SetField_int32(userAddr)") Then GoTo ExitFunction

            e = SetField_int32(digiChAddr(cNum).chAddrDLL, sgt_UserRamBuffer, _
                fldNum_UserBuffer_UserPtrSizeInBytes, 0)
            If ErrorHere(e, "SetField_int32(userBuffSize)") Then GoTo ExitFunction

            e = SetField_int32(digiChAddr(cNum).chAddrDLL, sgt_UserRamBuffer, _
                fldNum_UserBuffer_DigitizeOnOff, ion_gOnOffPopup_Off)
            If ErrorHere(e, "SetField_int32(userBuffSize)") Then GoTo ExitFunction

            ' with the DataY array ...
            With gDigiChan(cNum).dataArrayY

                ' if the dataY array had been pinned (VB term for locking a handle),
                ' then unlock it, gsw 6/1/02
                If .arrayIsPinned Then

                    ' unlock the handle, and free this address
                    .gcHandle.Free()
                    .gcHandle = Nothing
                    .pinnedAddress = 0
                    .arrayIsPinned = False

                End If

            End With


        Next cNum

ExitFunction:

        VbDataBuffers_DetachFromDLLUserBuffers = e

    End Function


    ' -------------------------------------------------------------------------
    ' FUNCTION   DigitizeList_CopyFromDriverDLL()
    '
    ' PURPOSE    This routine loads 'gDigiChan()' with the {netNum, driverNum, 
    '            moduleNum, chanNum} addresses of all channels that have been enabled
    '            for digitizing in the NETWORK window.  Typical sequence:
    '
    '               1. show user NETWORK page and ask user to select
    '                  channels for digitizing (click on red rectangle).
    '
    '               2. call this routine to capture that info.
    '
    '               3. digitize.
    '
    ' HISTORY    2/20/96  created Visual Basic version - sjm
    '            8/23/99  bug fix, numberOfChannels = GET_NUMBER_OF_CHANNELS(netNum, moduleNum, deviceNum)
    ' -------------------------------------------------------------------------
    Function DigitizeList_CopyFromDriverDLL( _
        ByVal massageType As Short, _
        ByVal runEndUser_SegmentProcessor As Short) As Short

        ' -------------------------------------------------
        ' Local Variables

        Dim deviceNum, chanNum, netNum, moduleNum As Byte
        Dim numberOfDevices, numberOfModules, numberOfChannels, numberOfNetworks, e As Short
        Dim message As String
        Dim onOffPopup, thisFieldExist As Short

        ' start out with an empty Digitize Channel list
        Call DigitizeList_ResetList()

        ' start out with no channels selected for digitizing
        e = iNetErr_None

        ' get # of iNet Networks in this Computer
        numberOfNetworks = GET_NUMBER_OF_NETWORKS()
        e = Get_iNet_Error()
        If ErrorHere(e, "GET_NUMBER_OF_NETWORKS()") Then GoTo ExitFunction

        'scan through all Networks
        For netNum = 1 To numberOfNetworks

            numberOfDevices = GET_NUMBER_OF_DEVICES(netNum)
            e = Get_iNet_Error()
            If ErrorHere(e, "GET_NUMBER_OF_DEVICES()") Then GoTo ExitFunction

            'scan through all devices
            For deviceNum = 1 To numberOfDevices

                numberOfModules = GET_NUMBER_OF_MODULES(netNum, deviceNum)
                e = Get_iNet_Error()
                If ErrorHere(e, "GET_NUMBER_OF_MODULES()") Then GoTo ExitFunction

                'scan through all modules
                For moduleNum = 1 To numberOfModules

                    ' bug fix 8/23/99, numberOfChannels = GET_NUMBER_OF_CHANNELS(netNum, moduleNum, deviceNum)
                    numberOfChannels = GET_NUMBER_OF_CHANNELS(netNum, deviceNum, moduleNum)
                    e = Get_iNet_Error()
                    If ErrorHere(e, "GET_NUMBER_OF_CHANNELS()") Then GoTo ExitFunction

                    'scan through Channels
                    For chanNum = 1 To numberOfChannels

                        ' see if the Display Digitize on/off fields exists...
                        thisFieldExist = THIS_FIELD_EXISTS(netNum, deviceNum, moduleNum, chanNum, _
                            sgt_Display, fldNum_Display_dispOnOff)
                        If thisFieldExist = iNetTrue Then

                            ' if this channels is enabled for digitizing...
                            onOffPopup = iNet_int32(netNum, deviceNum, moduleNum, chanNum, _
                                sgt_Display, fldNum_Display_dispOnOff, intention_getValue, 0)
                            If onOffPopup = ion_gOnOffPopup_On Then

                                ' add this channel to the digitize list
                                Call DigitizeList_AddOneItem(netNum, deviceNum, moduleNum, _
                                    chanNum, massageType, runEndUser_SegmentProcessor)

                            End If

                        End If

                    Next chanNum

                Next moduleNum

            Next deviceNum

        Next netNum

ExitFunction:

        DigitizeList_CopyFromDriverDLL = e

    End Function


    ' -------------------------------------------------------------------------
    ' FUNCTION    DigitizeProcess_MeasureActualSampleRate()
    '
    ' PURPOSE     Measures the actual sample rate.
    '
    ' HISTORY     2/25/97  created - unoWarrior
    ' -------------------------------------------------------------------------
    Function DigitizeProcess_MeasureActualSampleRate() As Single

        Dim cNum, e As Short

        ' Deselect channels inside the Netwokr page that had been enabled
        ' for digitizing (destroys their DIGITIZE BUFFERS).
        e = DisableAllChannelsForDigitizing()
        If ErrorHere(e, "dpmasr0") Then GoTo ExitWithError

        ' Select these channels for digitizing.
        For cNum = 0 To gNumDigiCh - 1

            e = SetField_int32(gDigiChan(cNum).chAddrDLL, sgt_Display, _
                fldNum_Display_dispOnOff, ion_gOnOffPopup_On)
            If ErrorHere(e, "dpmasr1") Then GoTo ExitWithError

        Next cNum

        ' We are setting the timing parameters in here.
        e = Set_iNet_TIMING_Parameters(gSampleRate_nextDigi, gPtsPerScan_nextDigi, _
            gNumScans, gContinuousScans, ion_gSaveDataPopup_ToRamBuffer, iNetTrue, iNetTrue)
        If ErrorHere(e, "dpmasr2") Then GoTo ExitWithError

        ' Sets up digitization, establishes sample rate, sets up BUFFERS, yet does not
        ' actually do the digitization. gsw 8/16/98
        Call SET_UP_DIGITIZATION()

        ' read in the resulting sample rate
        DigitizeProcess_MeasureActualSampleRate = GET_SAMPLE_RATE_IN_HZ(netNum_1stController)

        ' Deselect channels inside the Netwokr page that had been enabled
        ' for digitizing (destroys their DIGITIZE BUFFERS).
        e = DisableAllChannelsForDigitizing()
        If ErrorHere(e, "dpmasr3") Then GoTo ExitWithError

ExitWithoutError:

        Exit Function

ExitWithError:

        DigitizeProcess_MeasureActualSampleRate = 1

        MsgBox("Digitize_iNet_Data() Error. Simulation is being turned on")

    End Function

End Module