Option Strict Off
Option Explicit On 
Imports System.Runtime.InteropServices

Module Interface_Utilities

    ' ===========================================================================
    ' ===========================================================================
    ' FILE:  INET_Interface.vb
    '
    ' ------------------------- A B S T R A C T ----------------------------
    '
    ' This file contains interface code for the iNet DLL. For these calls
    ' to work you must have the files iNet32.dll and Inet95.vxd in your
    ' Windows95/System folder
    '
    ' ------------------------ H I S T O R Y ------------------------------
    ' HISTORY   Copyright c 1994-96 GWI.  All rights reserved.
    ' 5/18/94   created - gsw
    ' 2/11/97   created VB prototypes - sjm
    ' 6/6/02    updated to be compatible with Visual Studio (Visual Basic) .Net v7
    ' ===========================================================================
    ' ===========================================================================

    ' -------------------------------------------------------------------------
    ' Global Variables

    Public gNumNetworks As Short = 0    ' loaded w/ # of instruNet networks

    Public Const iSave As Short = 1     ' save data
    Public Const iLoad As Short = 2     ' load data

    Public giNetDLLIsLoaded As Short    ' true if instruNet Driver (iNet32.dll) is loaded ok

    '                                     true if iW+ license is installed ok
    Public g_iWp_licenseIsInstalledOk As Boolean = False

    ' -------------------------------------------------------------------------
    ' Simulation Policy

    ' If true, we jump into demo mode if no instruNet devices is attached to the
    ' computer.  Demo mode simulates an iNet-200 and iNet-100 device.
    Public Const wJumpIntoDemoModeIfNoDeviceExists As Short = 1

    ' Don't jump into demo mode if an iNet devices is not attached.
    Public Const wDoNotJumpIntoDemoModeIfNoDeviceExists As Short = 0

    ' jump into demo mode right now
    Public Const wJumpIntoRightNow As Short = 2

    '  ===================== CLASS: BASIC READ/WRITE CHANNEL ========================
    '  ===================== CLASS: BASIC READ/WRITE CHANNEL ========================
    '  ===================== CLASS: BASIC READ/WRITE CHANNEL ========================

    '-------------------------------------------------------------------------
    ' FUNCTIONS
    '
    '        GetChannelValue_iNet()          Read value of instruNet channel.
    '        SetChannelValue_iNet()          Set value of instruNet channel.
    '        GetFieldValue_iNet()            Read value of instruNet field.
    '        SetFieldValue_iNet()            Set value of instruNet field.
    '
    '        SetField_int32()                Set instruNet field with 32bit signed integer number
    '        SetField_uint32()               Set instruNet field with 32bit unsigned integer number
    '        SetField_flt32()                Set instruNet field with 32bit floating point number
    '        SetField_cStr()                 Set instruNet field with C string (terminated with 0x00)
    '
    '        GetField_int32()                Read instruNet field into a 32bit signed integer variable
    '        GetField_uint32()               Read instruNet field into a 32bit unsigned integer variable
    '        GetField_flt32()                Read instruNet field into a 32bit floating point variable
    '        GetField_cStr()                 Read instruNet field into a C string variable (terminated with 0x00)
    '
    '        iNet_int32()                    Reads or writes to a field via a 32bit signed integer variable
    '        iNet_uint32()                   Reads or writes to a field via a 32bit unsigned integer variable
    '        iNet_flt32()                    Reads or writes to a field via a 32bit floating point variable
    '        iNet_cString()                  Reads or writes to a field via a C string variable
    '        iNet_pString()                  Reads or writes to a field via a Pascal string variable
    '        iNet_DLL()                      Reads or writes to a field via any type of variable
    '
    ' PARAMETERS
    '
    '    netNum     NETWORK number = {0...numNetworks}, 0 is Driver,
    '    deviceNum  DEVICE number {0...numDevices}, 0 = "Controller",
    '    moduleNum  MODULE number within a hardware DEVICE {1...32}.
    '    chanNum    Hardware CHANNEL number {1...32}.
    '
    ' HISTORY       1/27/97 created - gsw
    '-------------------------------------------------------------------------

    Declare Function GetChannelValue_iNet Lib "iNet32.DLL" (ByVal netNum As Byte, _
        ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte, _
        ByRef errP As Short) As Single

    Declare Function SetChannelValue_iNet Lib "iNet32.DLL" (ByVal netNum As Byte, _
        ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte, _
        ByVal flt32_newValue As Single) As Short

    Declare Function GetFieldValue_iNet Lib "iNet32.DLL" (ByVal netNum As Byte, _
        ByVal deviceNum As Byte, ByVal moduleNum As Byte, _
        ByVal chanNum As Byte, ByVal settingGroupNumOrType As Short, _
        ByVal fieldNum As Short, ByRef errP As Short) As Single

    Declare Function SetFieldValue_iNet Lib "iNet32.DLL" (ByVal netNum As Byte, _
        ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte, _
        ByVal settingGroupNumOrType As Short, ByVal fieldNum As Short, _
        ByVal flt32_newValue As Single) As Short

    Declare Function SetField_int32 Lib "iNet32.DLL" ( _
        ByRef chAddrDLL As chAddrP, _
        ByVal settingGroupNumOrType As Short, ByVal fieldNum As Short, _
        ByVal int32Value As Integer) As Short

    Declare Function SetField_uint32 Lib "iNet32.DLL" (ByRef chAddrDLL As chAddrP, _
        ByVal settingGroupNumOrType As Short, ByVal fieldNum As Short, _
        ByVal uint32Value As Integer) As Short

    Declare Function SetField_flt32 Lib "iNet32.DLL" (ByRef chAddrDLL As chAddrP, _
        ByVal settingGroupNumOrType As Short, ByVal fieldNum As Short, _
        ByVal flt32Value As Single) As Short

    Declare Function GetField_int32 Lib "iNet32.DLL" (ByRef chAddrDLL As chAddrP, _
        ByVal settingGroupNumOrType As Short, ByVal fieldNum As Short, _
        ByRef iNetError As Short) As Integer

    Declare Function GetField_uint32 Lib "iNet32.DLL" (ByRef chAddrDLL As chAddrP, _
        ByVal settingGroupNumOrType As Short, ByVal fieldNum As Short, _
        ByRef iNetError As Short) As Integer

    Declare Function GetField_flt32 Lib "iNet32.DLL" (ByRef chAddrDLL As chAddrP, _
        ByVal settingGroupNumOrType As Short, ByVal fieldNum As Short, _
        ByRef iNetError As Short) As Single

    ' -------------------------------------------------------------------------------------
    ' FUNCTION   iNet_VB_GetFieldcStr()
    '            iNet_VB_SetFieldcStr()
    '
    ' PURPOSE    Read/Write instruNet field into a string variable.
    '
    ' HISTORY    2/24/97 created - gsw
    ' -----------------------------------------------------------------------------------

    ' DLL Version
    Declare Function GetField_cStr Lib "iNet32.DLL" ( _
        ByRef chAddrDLL As chAddrP, _
        ByVal settingGroupNumOrType As Short, _
        ByVal fieldNum As Short, _
        ByVal cString_Ptr As Integer) As Short

    ' VB Friendly Version
    Function iNet_VB_GetFieldcStr( _
            ByRef chAddr As chAddr, _
            ByVal settingGroupNumOrType As Short, _
            ByVal fieldNum As Short, _
            ByRef arg_VB_string As String) As Short

        ' use the Friendly get/set Field via cString routine to implement this, gsw 6/1/02
        iNet_VB_GetFieldcStr = iNet_VB_cString( _
                    chAddr.netNum, chAddr.deviceNum, chAddr.moduleNum, chAddr.chanNum, _
                    settingGroupNumOrType, fieldNum, intention_getValue, arg_VB_string)

    End Function

    ' DLL Version
    Declare Function SetField_cStr Lib "iNet32.DLL" ( _
        ByRef chAddrDLL As chAddrP, _
        ByVal settingGroupNumOrType As Short, _
        ByVal fieldNum As Short, _
        ByVal cString_Ptr As Integer) As Short

    ' VB Friendly Version
    Function iNet_VB_SetFieldcStr( _
           ByRef chAddr As chAddr, _
           ByVal settingGroupNumOrType As Short, _
           ByVal fieldNum As Short, _
           ByRef arg_VB_string As String) As Short

        ' use the Friendly get/set Field via cString routine to implement this, gsw 6/1/02
        iNet_VB_SetFieldcStr = iNet_VB_cString( _
                    chAddr.netNum, chAddr.deviceNum, chAddr.moduleNum, chAddr.chanNum, _
                    settingGroupNumOrType, fieldNum, intention_setValue, arg_VB_string)

    End Function


    '-------------------------------------------------------------------------
    ' FUNCTION      LoadChannelAddress()
    '
    '                   Loads a channel's address into a chAddr structure (i.e. contains
    '                   {netNum, deviceNum, moduleNum, chanNum}.  Returns TRUE (1)
    '                   if this channel exists; FALSE (0) otherwise.
    '
    '               GetChannelAddress()
    '
    '                   Returns a channel's address stored in a chAddr structure in
    '                   {netNum, deviceNum, moduleNum, chanNum} form.
    ' PARAMETERS
    '
    '    chAddrP    LoadChannelAddress() loads this channel address with
    '               {netNum, deviceNum, moduleNum, chanNum}
    '    netNum     NETWORK number = {0...numNetworks}, 0 is Driver,
    '    deviceNum  DEVICE number {0...numDevices}, 0 = "Controller",
    '    moduleNum  MODULE number within a hardware DEVICE {1...32}.
    '    chanNum    Hardware CHANNEL number {1...32}.
    '
    ' HISTORY       1/27/97 created - gsw
    '-------------------------------------------------------------------------

    ' Please also see LoadChannelAddress_VB(), which is more VB friendly.
    Declare Function LoadChannelAddress Lib "iNet32.DLL" ( _
        ByRef chAddressP As chAddrP, ByVal netNum As Byte, _
        ByVal deviceNum As Byte, ByVal moduleNum As Byte, _
        ByVal chanNum As Byte) As Short

    Declare Sub GetChannelAddress Lib "iNet32.DLL" (ByRef chAddrDLL As chAddrP, _
        ByRef netNumP As Short, ByRef deviceNumP As Short, _
        ByRef moduleNumP As Short, ByRef chanNumP As Short)


    '-------------------------------------------------------------------------
    ' FUNCTION      Set_instruNet_Network_State()
    '               Get_instruNet_Network_State()
    '
    ' PURPOSE       Sets/Gets the state of the instruNet network with the data in
    '               the passed array.
    ' PARAMETERS
    '
    '    dest_int16()       Data is loaded into here (this should be 10000 long or so)
    '
    '    numValidBytesP     This is the # of valid bytes in the array
    '
    ' HISTORY     3/15/97 - created
    '-------------------------------------------------------------------------

    ' DLL version
    Declare Function Set_instruNet_Network_State Lib "iNet32.DLL" ( _
        ByVal dataAddressP As Integer, _
        ByVal numValidBytes As Integer) As Short

    ' VB Friendly version, gsw 6/1/02
    Public Function Set_instruNet_Network_State_VB( _
        ByRef src_int16() As Short, _
        ByVal numValidBytes As Integer) As Short

        ' get ptr to dest_int16 array data, and lock the data handle
        Dim srcHandle As GCHandle
        Dim srcAddr As Integer = VbInt16Array_to_Address(src_int16, srcHandle)

        ' process the array(s) inside the iNet32.dll
        Set_instruNet_Network_State_VB = Set_instruNet_Network_State(srcAddr, numValidBytes)

        ' unlock the data handle
        srcHandle.Free()

    End Function

    ' DLL version
    Declare Function Get_instruNet_Network_State Lib "iNet32.DLL" ( _
        ByVal dataAddressP As Integer, _
        ByVal arraySizeBytes As Integer, _
        ByRef numValidBytesReturnedP As Integer) As Short

    ' VB friendly version, gsw 6/1/02
    Public Function Get_instruNet_Network_State_VB( _
          ByRef dest_int16() As Short, _
          ByVal arraySizeBytes As Integer, _
          ByRef numValidBytesReturnedP As Integer) As Short

        Get_instruNet_Network_State_VB = iNetErr_General

        ' make sure this array is as large as we say it is to the iNet32.dll
        ' (else, the iNet32.dll could overwrite memory).
        If (dest_int16.GetUpperBound(0) + 1) < (arraySizeBytes / 2) Then
            Error ("Get_instruNet_Network_State_VB needs to be passed a larger array.")
            Exit Function
        End If

        ' make sure this array is as large enough
        If (dest_int16.GetUpperBound(0) < 9000) Then
            Error ("I recommend you pass an array to Get_instruNet_Network_State_VB() that is at least 10K pts long.")
            Exit Function
        End If

        ' get ptr to dest_int16 array data and lock the data handle
        Dim destHandle As GCHandle
        Dim destAddr As Integer = VbInt16Array_to_Address(dest_int16, destHandle)

        ' process the array(s) inside the iNet32.dll
        Get_instruNet_Network_State_VB = _
            Get_instruNet_Network_State(destAddr, arraySizeBytes, numValidBytesReturnedP)

        ' unlock the data handle
        destHandle.Free()

    End Function

    '  =========================== CLASS: BASIC DIGITIZE ============================
    '  =========================== CLASS: BASIC DIGITIZE ============================
    '  =========================== CLASS: BASIC DIGITIZE ============================

    '-----------------------------------------------------------------------------------
    ' FUNCTION    EnableChannelForDigitizing()         SIMPLE FORMAT FUNCTIONS
    '
    ' PURPOSE     Specifies that this channel is enabled for digitizing.
    '
    ' PARAMETERS
    '
    '    chAddrP  This specifies a channel via it's channel Address
    '             = {netNum, deviceNum, moduleNum, chanNum}
    '
    ' HISTORY     1/27/97 created - gsw
    '---------------------------------------------------------------------------------

    Declare Function EnableChannelForDigitizing Lib "iNet32.DLL" ( _
        ByRef chAddrDLL As chAddrP) As Short

    '-----------------------------------------------------------------------------------
    ' FUNCTION    DisableAllChannelsForDigitizing()    SIMPLE FORMAT FUNCTIONS
    '
    ' PURPOSE     Disables ALL channels for digitizing.
    '
    ' PARAMETERS  None
    '
    ' HISTORY     1/27/97 created - gsw
    '---------------------------------------------------------------------------------

    Declare Function DisableAllChannelsForDigitizing Lib "iNet32.DLL" () As Short

    '-------------------------------------------------------------------------------------
    ' FUNCTION    Set_iNet_TIMING_Parameters()    SIMPLE FORMAT FUNCTION
    '
    ' PURPOSE     Sets the instruNet TIMING Parameters.  All networks are
    '             set the same.
    '
    ' PARAMETERS
    '
    '    samplesPerSecond,
    '             sample rate (points per second per channel)
    '    pointsPerScan,
    '             number of points in 1 scan, for 1 channel
    '    numberOfScans,
    '             number of scans that are digitized when the start button
    '             is pressed
    '    stripChartRecorder,
    '             if TRUE (1) we do Strip Chart Recorder work with
    '             Continuous Scans, otherwise if FALSE (0) we do
    '             Oscilloscope mode (non-Coninuous Scans)
    '    saveDataOption,
    '             Save Data option = { ion_gSaveDataPopup_Off,
    '             ion_gSaveDataPopup_ToRamBuffer, ion_gSaveDataPopup_ToDisk,
    '             ion_gSaveDataPopup_UserControl}
    '    showAlertOnOverflow,
    '             if TRUE (1), the driver will show an alert if
    '             a data buffer overflows; otherwise if FALSE (0)
    '             it just keeps going if there is an overflow.
    '    switchAccurately
    '             if TRUE (1), channel switching should be accurate;
    '             otherwise, it is less accurate, yet faster.'
    ' HISTORY     11/30/95  created - gsw
    '-----------------------------------------------------------------------------------'/

    Declare Function Set_iNet_TIMING_Parameters Lib "iNet32.DLL" ( _
        ByVal samplesPerSecond As Single, ByVal pointsPerScan As Integer, _
        ByVal numberOfScans As Integer, ByVal stripChartRecorder As Short, _
        ByVal saveDataOption As Short, ByVal showAlertOnOverflow As Short, _
        ByVal switchAccurately As Short) As Short

    '-------------------------------------------------------------------------------------
    ' FUNCTION    Set_iNet_TRIGGER_Parameters()        SIMPLE FORMAT FUNCTION
    '
    ' PURPOSE     Sets the instruNet TRIGGER Parameters.  This function
    '             sets the trigger for all networks to the same channel
    '             in each network.
    ' PARAMETERS
    '
    '    triggerMode - trigger mode = {ion_TriggerMode_Off, ion_TriggerMode_Auto, ion_TriggerMode_Norm}
    '    triggerThreshold - trigger threshold in engineering units
    '    triggerSlope - trigger slope = {ion_Rising, ion_Falling}
    '    preTriggerSeconds - not supported
    '    triggerSourceP - trig source Channel Address = {netNum, deviceNum, moduleNum,
    '        chanNum}, NETWORK number = {0...numNetworks}, 0 is Driver,
    '
    ' HISTORY     11/30/95  created - gsw
    '-----------------------------------------------------------------------------------'/

    Declare Function Set_iNet_TRIGGER_Parameters Lib "iNet32.DLL" ( _
        ByVal triggerMode As Short, _
        ByVal triggerThreshold As Single, _
        ByVal triggerSlope As Short, _
        ByVal preTriggerSeconds As Single, _
        ByRef trigSrc_chAddrDLL As chAddrP) As Short

    '-----------------------------------------------------------------------------------
    ' FUNCTION    Service_All_iNet_Digitize_Buffers()
    '
    ' PURPOSE     This routine must be called continuously while
    '             digitizing to allow the processor to service all digitize
    '             buffers.
    '
    ' PARAMETERS
    '
    '    netNum_forScanInfo
    '             We get scan  info from this controller,
    '             even though we service ALL controllers.
    '
    '    theNextDataAccessIsTheLast_ofEntireDigitization_allCtlr,
    '             Returns TRUE (1) if the next data pulled out
    '             out of the buffers via Access_Digitized_Data_In_Ram_Buffer()
    '             is that last pull for all networks; FALSE (0) otherwise
    '             (i.e. at least 1 network still has data in it)
    '
    '    theNextDataAccessWillPullToTheEndOfAScan_oneCtlr,
    '             Returns TRUE (1) if the next data access via
    '             Access_Digitized_Data_In_Ram_Buffer() accesses data
    '             to the end of 1 Scan in network 'netNum_forScanInfo' (i.e. 1
    '             Scan's worth of data); FALSE (0) otherwise.  If you do not
    '             access 1 scan at a time, you can access little
    '             segments at a time, as they become available.
    '
    '     controllerIsFinishedWithDigitization_allCtlr,
    '             set to TRUE if controller is finished with digitization
    '             (i.e. the last piece of data has been moved from controller
    '             ram buffer to host memory ram buffer); FALSE otherwise.
    '
    '     last_scanNum_base1_pulled_inFull_oneCtlr,
    '             last scan  (base 1) that we pulled (e.g. a/d) in 'netNum_forScanInfo'
    '             full from controller ram buffer TO user or
    '             driver ram buffer when Service_All_iNet_Digitize_Buffers()
    '             was called.
    '
    '     last_scanNum_base1_pushed_inFull_oneCtlr
    '             last scan  (base 1) that we pushed (e.g. d/a) in 'netNum_forScanInfo'
    '             full from user or driver ram buffer TO controller
    '             ram buffer when Service_All_iNet_Digitize_Buffers()
    '             was called.
    '
    ' HISTORY  11/30/95 created - gsw
    '-----------------------------------------------------------------------------------

    Declare Function Service_All_iNet_Digitize_Buffers Lib "iNet32.DLL" ( _
        ByVal netNum As Byte, _
        ByRef theNextDataAccIsTheLast As Short, _
        ByRef theNextDataAccessWillPullToTheEndOfASc As Short, _
        ByRef controllerIsFinishedWithDigitization As Short, _
        ByRef lastscanNumBase1PulledInFull As Integer, _
        ByRef lastscanNumBase1PushedInFull As Integer) As Short

    '-----------------------------------------------------------------------------------
    ' FUNCTION    Access_Digitized_Data_In_Ram_Buffer()  SIMPLE FORMAT FUNCTION
    '
    ' PURPOSE     While digitizing, this will pull a segment out of the iNet
    '             DRIVER RAM BUFFER or USER RAM BUFFER and pass you an address
    '             to the first FLT32 point in the buffer (i.e. beginning of the
    '             entire buffer), the index into the buffer to begin reading
    '             data, and the  of points to pull.
    '
    '             1st point to read
    '             = (*bufferStartAddrP) +'pointToPullindex_base1 - 1
    '
    '             last point to read
    '             = (*bufferStartAddrP) +'pointToPullindex_base1 +'numPointsToPull - 1
    '
    '             The buffer is 1 scan long, and the scan number {1...numScans}
    '             that we are reading is returned in '*scanNumberToPull_base1'.
    '
    '             The  of times the iNet circular buffers overflowed is returned
    '             in '*totalNumOfTimesWeOverflowed'.
    '
    ' PARAMETERS
    '
    '    channelAddr[],
    '             Channel Address = {netNum, deviceNum, moduleNum, chanNum}
    '             NETWORK number = {0...numNetworks}, 0 is Driver,
    '    driverRamBuffer,
    '             if TRUE, we go to DRIVER RAM BUFFER, otherwise,
    '             we go to USER RAM BUFFER.
    '    bufferStartAddrP,
    '             we will load this with the address to the
    '             first FLT32 point in the buffer
    '    pointToPullindex_base1,
    '             we will load this with the {1... numPtsPerScan)
    '             index of the first point of the segment that we need
    '             to process (base 1 index).
    '    numPointsToPull,
    '             of the buffer
    '    scanNumberToPull_base1
    '             we load this with the scan number {1...numScans}
    '             of the scan that we are currently pulling (base 1).
    '
    ' HISTORY     11/25/95 created - gsw
    '             11/30/95 modified - gsw
    '---------------------------------------------------------------------------------'/

    Declare Function Access_Digitized_Data_In_Ram_Buffer Lib "iNet32.DLL" ( _
        ByRef chAddrDLL As chAddrP, _
        ByVal driverRamBuffer As Byte, _
        ByRef bufferStartAddrP As Integer, _
        ByRef pointToPullindex_base1 As Integer, _
        ByRef numPointsToPull As Integer, _
        ByRef scanNumberToPull As Integer) As Short

    ' ==================================================================================
    '
    '                  ADVANCED FIELD READ/WRITE ROUTINES
    '
    ' ==================================================================================

    '-----------------------------------------------------------------------------------
    ' FUNCTION    iNet_int32()     Calls iNet() to read or write a field in the
    '                              iNet network with a  signed 32bit integer
    '                              value (+/- 2billion).
    '
    '             iNet_flt32()     Calls iNet() to read or write a field in the
    '                              iNet network with a 32bit floating point number.
    '
    '             iNet_cString()   Calls iNet() to read or write a field in the
    '                              iNet network with a C string variable (0-255
    '                              chars, 0 terminated). Please use iNet_VB_cString() instead
    '                              of iNet_cString() when working with Visual Basic.
    '
    '             iNet_DLL()       Calls iNet() to read or write a field via a
    '                              variable of any type
    '
    ' PURPOSE     If we are setting a field value (i.e. intention == intention_setValue)
    '             then we return an iNetError error code. If intention is not
    '             intention_setValue and we get an error condition, then we
    '             0; otherwise we return the resulting field value.
    '
    '             If you want to write to the field, set the 'intention' field to
    '             'intention_setValue' and pass the new value in 'int32Value'.
    '
    '             If you want to read the value of a field, set the 'intention' field
    '             to 'intention_getValue' and then refer to the returned value.
    '
    ' PARAMETERS
    '
    '    netNum     NETWORK number = {0...numNetworks}, 0 is Driver,
    '    deviceNum  DEVICE number {0...numDevices}, 0 = "Controller",
    '    moduleNum  MODULE number within a hardware DEVICE {1...32}.
    '    chanNum    Hardware CHANNEL number {1...32}.
    '
    '    settingGroupNumOrType
    '              This references a specific Setting Group.
    '
    '              If > 0, this is a settings group Number:
    '                  {1...numSettingGroups}, which corresponds to
    '                   the order in which the SettingGroup appears
    '                   in the Setting popup menu, with the first item
    '                   in the menu is 1.
    '
    '              If < 0, this is a settings group Type:
    '                  ion_settingGroupType = { sgt_VinHardware (-3)
    '                  sgt_Vout (-4)... }

    '    fieldNum  Specifies the field number within the settingGroup.
    '              The 1st field is 1, then next 2, etc.  See fgNums_...
    '              enum's in INET_Declarations.Bas file for a list of these.
    '
    '    intention Tells iNet() what to do with the field.  Please see
    '              ion_intention enum for details. ion_intention =
    '              {intention_getValue, intention_setValue,
    '              intention_getNameStr, intention_getMaxValue, etc}
    '              we set field with this value if that is
    '              the specified intention
    '
    ' HISTORY     2/4/96 created - gsw
    '-----------------------------------------------------------------------------------

    Declare Function iNet_int32 Lib "iNet32.DLL" (ByVal netNum As Byte, _
        ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte, _
        ByVal settingGroupNumOrType As Short, ByVal fieldNum As Short, _
        ByVal intention As Short, ByVal int32Value As Integer) As Short

    Declare Function iNet_uint32 Lib "iNet32.DLL" (ByVal netNum As Byte, _
        ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte, _
        ByVal settingGroupNumOrType As Short, ByVal fieldNum As Short, _
        ByVal intention As Short, ByVal int32Value As Integer) As Short

    Declare Function iNet_flt32 Lib "iNet32.DLL" (ByVal netNum As Byte, _
        ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte, _
        ByVal settingGroupNumOrType As Short, ByVal fieldNum As Short, _
        ByVal intention As Short, ByVal flt32Value As Single) As Single

    ' -------------------------------------------------------------------------------------
    ' FUNCTION   iNet_VB_cString()
    '
    ' PURPOSE    Read or write a field within the iNet system, via the passed String.
    '
    ' HISTORY    2/24/97 created - gsw
    ' -------------------------------------------------------------------------------------

    ' DLL Version.
    Declare Function iNet_cString Lib "iNet32.DLL" (ByVal netNum As Byte, _
        ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte, _
        ByVal settingGroupNumOrType As Short, ByVal fieldNum As Short, _
        ByVal intention As Short, ByVal cString_Ptr As Integer) As Short

    ' VB Friendly Version
    Function iNet_VB_cString( _
            ByVal netNum As Byte, _
            ByVal deviceNum As Byte, _
            ByVal moduleNum As Byte, _
            ByVal chanNum As Byte, _
            ByVal settingGroupNumOrType As Short, _
            ByVal fieldNum As Short, _
            ByVal intention As Short, _
            ByRef VB_string As String) As Short

        ' The iNet32.dll will work w/ this temp array of 1byte/char chars
        ' null terminated c string.
        Dim cString(260) As Char

        ' if we are setting the Field's value with VB_string ...
        If intention = intention_setValue Then

            ' copy the caller's VB string (2bytes/char) into the ANSI C string
            Copy_VbStr_TO_cStr(VB_string, cString)

        End If

        ' get handle to cString internal information
        Dim gcHandle As GCHandle = gcHandle.Alloc(cString, GCHandleType.Pinned)

        ' get addr to cString array, and lock it's handle
        Dim cString_ptr As Integer = gcHandle.AddrOfPinnedObject.ToInt32

        ' Read or Write the field with the passed string (via the iNet32.dll). If passing
        ' the string into the dll, VB will automatically append the \0 null char to the end
        iNet_VB_cString = iNet_cString(netNum, deviceNum, moduleNum, chanNum, _
            settingGroupNumOrType, fieldNum, intention, cString_ptr)

        ' unlock the handle and free the pointer to the cString
        gcHandle.Free()

        ' if we are reading the Field's value into VB_string...
        If intention <> intention_setValue Then

            ' copy the cString string (1byte/char) into the caller's VB_string (2bytes/char)
            Copy_cStr_TO_VbStr(cString, VB_string)

        End If

    End Function

    ' -------------------------------------------------------------------------------------
    ' FUNCTION   Copy_VbStr_TO_cStr()
    '            Copy_cStr_TO_VbStr()
    '
    ' PURPOSE    Copy a cString (1byte/char) stored in char array with 0x00 null termination
    '            into a regular VB string (2bytes/char); or visa-versa.
    '
    ' HISTORY    2/24/97 created - gsw
    ' -------------------------------------------------------------------------------------
    '
    Sub Copy_VbStr_TO_cStr(ByRef VB_string As String, ByRef cString() As Char)

        ' get handle to cString internal information
        Dim gcHandle As GCHandle = gcHandle.Alloc(cString, GCHandleType.Pinned)

        ' get addr to cString array, and lock it's handle
        Dim cString_ptr As Integer = gcHandle.AddrOfPinnedObject.ToInt32()
        Dim cString_len As Integer = cString.Length

        ' convert to a VB string to a c string and place it into memory
        ' at an address pointed to by cString_IntPtr
        Dim cStr_CopyOf_VbString_IntPtr As IntPtr = Marshal.StringToHGlobalAnsi(VB_string)

        ' Get size and length of the IntPtr in memory
        Dim cStr_CopyOf_VbString_ptr As Integer = cStr_CopyOf_VbString_IntPtr.ToInt32
        Dim cStr_CopyOf_VbString_len As Integer = VB_string.Length

        ' make sure we don't overwrite the caller's c string
        If cStr_CopyOf_VbString_len > (cString_len - 1) Then
            Error ("cStr in Copy_VbStr_TO_cStr() needs to be longer")
            cStr_CopyOf_VbString_len = cString_len - 1
        End If

        ' copy the temp Ansi C string into the caller's ansi string
        iNet_memcopy(cStr_CopyOf_VbString_ptr, cString_ptr, cStr_CopyOf_VbString_len + 1)

        ' free the gcHandle and unlock the cString() Data(handle)
        gcHandle.Free()

        ' free this temp block of memory used by the Marshal...() routine
        Marshal.FreeHGlobal(cStr_CopyOf_VbString_IntPtr)

    End Sub

    ' -----------------------------
    ' Copy c string into VB string
    '
    Sub Copy_cStr_TO_VbStr(ByRef cString() As Char, ByRef VB_string As String)

        ' get handle to cString internal information
        Dim gcHandle As GCHandle = gcHandle.Alloc(cString, GCHandleType.Pinned)

        ' get addr to cString array, and lock it's handle
        Dim cString_ptr As Integer = gcHandle.AddrOfPinnedObject.ToInt32()

        ' convert the cString to a VB string
        VB_string = Marshal.PtrToStringAnsi(IntPtr.op_Explicit(cString_ptr))

        ' free the gcHandle and unlock the data handle
        gcHandle.Free()

    End Sub


    ' -------------------------------------------------------------------------------------
    ' FUNCTION   iNet_DLL()
    '
    ' PURPOSE    Call the iNet32.dll directly. Here, one can read or write a variable
    '            of any type.
    '
    '            ptrToArg must be a ptr into memory that is pinned if it is an array.
    '
    ' HISTORY    2/24/97 created - gsw
    ' -------------------------------------------------------------------------------------
    Declare Function iNet_DLL Lib "iNet32.DLL" ( _
        ByVal netNum As Byte, _
        ByVal deviceNum As Byte, _
        ByVal moduleNum As Byte, _
        ByVal chanNum As Byte, _
        ByVal settingGroupNumOrType As Short, _
        ByVal fieldNum As Short, _
        ByVal intention As Short, _
        ByVal argType As Short, _
        ByRef ptrToArg As Integer) As Short


    ' ==================================================================================
    ' ==================================================================================
    '
    '                                    DRIVER ROUTINES
    '
    ' ==================================================================================
    ' ==================================================================================

    '-----------------------------------------------------------------------------------
    ' FUNCTION    Load_instruNet_Driver()
    '
    ' PURPOSE     Loads the instruNet Driver file if 'reset_instruNet' is
    '             true we reset the network
    '
    ' PARAMETERS
    '
    '    reset_instruNet
    '             if true we force the resetting of the instrunet network
    '    driverIsInstalledOK as integer,
    '             loaded with TRUE (1) if driver is installed ok;
    '             [e.g. Win95 'iNet32.DLL' or Mac 'instruNet Driver (ppc)']
    '             FALSE (0) otherwise.
    '    numNetworks
    '             loaded with the number of networks (i.e. controllers found)
    '
    ' HISTORY     1/27/97 created - gsw
    '---------------------------------------------------------------------------------

    Declare Function Load_instruNet_Driver Lib "iNet32.DLL" ( _
        ByVal reset_instruNet_ As Short, _
        ByRef driverIsInstalledOK_ As Short, _
        ByRef numNetworks_ As Short) As Short

    '-----------------------------------------------------------------------------------
    ' FUNCTION    CloseDriverAndReleaseDriverRam()
    '
    ' PURPOSE     This routine closes the instruNet driver and releases the
    '             memory used to hold it. This will delete ram use to hold
    '             iNet95.vxd, iNetNT.dvr, internal buffers, etc.
    '
    '             ONLY CALL THIS ONCE, AND DON'T CALL iNet() AFTER CALLING
    '             THIS ROUTINE (e.g. CALL THIS WHEN YOUR APPLICATION QUITS).
    '
    '             Exiting an application usually releases the ram used to hold
    '             the driver as well; therefore, this is sometimes unnecessary
    '             when someone is exiting the calling application.
    '
    '
    ' PARAMETERS  None
    '
    ' HISTORY     9/8/96  created - gsw
    '---------------------------------------------------------------------------------

    Declare Sub CloseDriverAndReleaseDriverRam Lib "iNet32.DLL" ()

    '-----------------------------------------------------------------------------------
    ' FUNCTION    Show_ALERT_if_hit_iNet_Error()
    '
    ' PURPOSE     This communicates an error value to the user, the easiest
    '             simplest way possible.
    '
    ' PARAMETERS  None
    '
    ' HISTORY     11/30/95 created - gsw
    '             1/22/96  kag - Added Windows code
    '---------------------------------------------------------------------------------

    Declare Function Show_ALERT_if_hit_iNet_Error Lib "iNet32.DLL" ( _
        ByVal iNetError_ As Short) As Short

    '-----------------------------------------------------------------------------------
    ' FUNCTION    Show_Simple_Alert()
    '
    ' PURPOSE     Shows a simple alert message.
    '
    ' PARAMETERS  cString  message to be displayed.
    '
    ' HISTORY     12/20/96  created - gsw
    '---------------------------------------------------------------------------------

    ' Use VB MsgBox() instead
    ' OLD: Declare Sub Show_Simple_Alert Lib "iNet32.DLL" (ByVal cString As String)

    '-----------------------------------------------------------------------------------
    ' FUNCTION    Get_iNet_Error()
    '
    ' PURPOSE     Returns the error code returned by the last call
    '             to the instruNet Driver
    '
    ' PARAMETERS  None
    '
    ' HISTORY     2/4/96  created - gsw
    '---------------------------------------------------------------------------------

    Declare Function Get_iNet_Error Lib "iNet32.DLL" () As Short

    '-----------------------------------------------------------------------------------
    ' FUNCTION    Get_Last_iNet_Call()
    '
    ' PURPOSE     Returns a pointer to a structure that contains the
    '             internal parameters passed to the last instruNet() call.
    '
    ' PARAMETERS  pointerToData   This Long is filled with a pointer
    '                             to where the internal data is kept.
    '
    ' HISTORY     2/4/96  created - gsw
    '---------------------------------------------------------------------------------

    ' No longer supported, gsw 6/1/02
    ' OLD Declare Sub Get_Last_iNet_Call Lib "iNet32.DLL" (ByRef pointerToData As Integer)

    ' ==================================================================================
    ' ==================================================================================
    '
    '                            UTILITY ROUTINES
    '
    ' ==================================================================================
    ' ==================================================================================

    '-----------------------------------------------------------------------------------
    ' FUNCTIONS   Set_iNet_Demo_Mode()
    '
    ' PURPOSE     Sets Demo mode, via "demoMode", to one of:
    '
    '               0:  iNet_NoDemo - real mode (no demo)
    '               1:  iNet_FullSimulation - we fully simulate iNet hardware w/ triangle
    '                   waves feeding a fake iNet-100 & iNet-200.  This will reset instruNet.
    '               2:  iNet_CorruptData - we read real instruNet hardware, yet we alter the
    '                   data randomly, to make it unuseable.
    '
    ' HISTORY     2/6/96 created - gsw
    '---------------------------------------------------------------------------------
    Declare Function Set_iNet_Demo_Mode Lib "iNet32.DLL" (ByVal demoMode As Short) As Short

    ' -------------------------------------------------------------------------------------
    ' FUNCTION    Get_iWp_LicenseState_Safe()
    '
    ' PURPOSE     Returns TRUE if iW+ license is installed, FALSE otherwise. If we
    '             are in Demo mode, then we say the license is installed, even if
    '             it is not.  If working with an old pre iW+ < v2.0 iNet dll, then we will
    '             say the license is Not installed
    '
    ' HISTORY     7/5/03  created - gsw
    ' -------------------------------------------------------------------------------------
    Declare Function Get_iWp_LicenseState Lib "iNet32.DLL" () As Short

    Function Get_iWp_LicenseState_Safe() As Boolean

        Dim iWp_license_installed_ok As Boolean = False

        On Error GoTo iNetErrErr

        ' Pre v2.0 iNet32.dll will throw an error due to no recognizing
        ' Get_iWp_LicenseState().  We get the state of the iW+ license here.

        If (Get_iWp_LicenseState() <> 0) Then iWp_license_installed_ok = True

iNetErrErr:

        Get_iWp_LicenseState_Safe = iWp_license_installed_ok

    End Function


    '-----------------------------------------------------------------------------------
    ' FUNCTIONS   iNet_Peek_int16()  Returns the signed 16bit integer value at 'address'.
    '             iNet_Peek_int32()  Returns the signed 32bit integer value at 'address'.
    '             iNet_Peek_flt32()  Returns the signed 32bit floating point value at 'address'.
    '
    ' PARAMETERS  address  Address value.
    '
    ' HISTORY     2/6/96 created - gsw
    '---------------------------------------------------------------------------------

    Declare Function iNet_Peek_int16 Lib "iNet32.DLL" (ByRef address As Integer) As Short
    Declare Function iNet_Peek_int32 Lib "iNet32.DLL" (ByRef address As Integer) As Integer
    Declare Function iNet_Peek_flt32 Lib "iNet32.DLL" (ByRef address As Integer) As Single

    '-----------------------------------------------------------------------------------
    ' FUNCTIONS   iNet_Poke_int16()  Sets the 16bit integer value at 'address'.
    '             iNet_Poke_int32()  Sets the 32bit integer value at 'address'.
    '             iNet_Poke_flt32()  Sets the 32bit floating point value at 'address'.
    '
    ' PARAMETERS  address     address to read
    '             int16value  value to write
    '
    ' HISTORY     2/6/96 created - gsw
    '-----------------------------------------------------------------------------------

    Declare Sub iNet_Poke_int16 Lib "iNet32.DLL" (ByRef address As Integer, _
        ByVal int16value As Short)
    Declare Sub iNet_Poke_int32 Lib "iNet32.DLL" (ByRef address As Integer, _
        ByVal int32Value As Integer)
    Declare Sub iNet_Poke_flt32 Lib "iNet32.DLL" (ByRef address As Integer, _
        ByVal flt32Value As Single)

    '-----------------------------------------------------------------------------------
    ' FUNCTION    iNet_Get_VarPtr()
    '
    ' PURPOSE     Returns the address of where in memory the passed variable
    '             is stored.
    '
    ' PARAMETERS  Variable      Any variable or array (e.g. "x", "y(0)").
    '
    ' HISTORY     2/6/96 created - gsw
    '-----------------------------------------------------------------------------------

    ' Use VbFlt32Array_to_Address() instead, gsw 6/1/02
    '
    ' OLD: Declare Function iNet_Get_VarPtr Lib "iNet32.DLL" ( _
    ' OLD: ByRef Variable As Object) As Integer

    '-----------------------------------------------------------------------------------
    ' FUNCTION    iNet_memcopy_addr_flt32   copy one address to a flt32 VB array
    '             iNet_memcopy_flt32_addr   copy one flt32 array to an address
    '             iNet_memcopy_flt32_flt32  copy one VB flt32 array to another
    '             iNet_memcopy              copy one address to another
    '
    ' PURPOSE     Moves 'numFlt32Pts' of data starting at 'srcAddr' address (or flt32 array)
    '             to 'destAddr' address(or flt32 array) .
    '
    ' PARAMETERS  srcAddr     source address
    '             destAddr    destination address
    '             numFlt32Pts number of flt32 points to move
    '
    ' HISTORY     2/6/96 created - gsw
    '-----------------------------------------------------------------------------------

    ' -----------------------------------------------------------
    ' copy one flt32 VB array to another flt32 VB array
    '
    Public Sub iNet_memcopy_flt32_flt32( _
        ByRef src_flt32() As Single, ByRef dest_flt32() As Single, ByVal numFlt32Pts As Integer)

        ' get ptr to src_flt32 array data
        Dim srcHandle As GCHandle
        Dim srcAddr As Integer = VbFlt32Array_to_Address(src_flt32, srcHandle)

        ' get ptr to dest_flt32 array data
        Dim destHandle As GCHandle
        Dim destAddr As Integer = VbFlt32Array_to_Address(dest_flt32, destHandle)

        ' copy the data
        iNet_memcopy(srcAddr, destAddr, numFlt32Pts * 4)

        ' unlock the data handle
        srcHandle.Free()
        destHandle.Free()

    End Sub

    ' -----------------------------------------------------------
    ' copy one flt32 VB array to an address
    '
    Public Sub iNet_memcopy_flt32_addr( _
        ByRef src_flt32() As Single, ByVal destAddr As Integer, ByVal numFlt32Pts As Integer)

        ' get ptr to src_flt32 array data, and lock the data handle
        Dim srcHandle As GCHandle
        Dim srcAddr As Integer = VbFlt32Array_to_Address(src_flt32, srcHandle)

        ' copy the data
        iNet_memcopy(srcAddr, destAddr, numFlt32Pts * 4)

        ' unlock the data handle
        srcHandle.Free()

    End Sub

    ' -----------------------------------------------------------
    ' copy one address to a flt32 VB array
    '
    Public Sub iNet_memcopy_addr_flt32( _
        ByVal srcAddr As Integer, ByRef dest_flt32() As Single, ByVal numFlt32Pts As Integer)

        ' get ptr to dest_flt32 array data and lock the data handle
        Dim destHandle As GCHandle
        Dim destAddr As Integer = VbFlt32Array_to_Address(dest_flt32, destHandle)

        ' copy the data
        iNet_memcopy(srcAddr, destAddr, numFlt32Pts * 4)

        ' unlock the data handle
        destHandle.Free()

    End Sub

    ' -----------------------------------------------------------
    ' copy data from one addr to another via the iNet32.dll
    ' It is easier to use the below VB versions instead of this one.
    '
    Declare Sub iNet_memcopy Lib "iNet32.DLL" ( _
        ByVal srcAddr As Integer, ByVal destAddr As Integer, ByVal numBytes As Integer)

    ' OLD Declare Sub iNet_memcopy Lib "iNet32.DLL" (ByVal srcAddr As Integer, _
    ' OLD       ByVal destAddr As Integer, ByVal numBytes As Integer)

    ' OLD iNet_memcopy_int16(), no longer exported from DLL
    ' OLD Declare Sub iNet_memcopy_int16 Lib "iNet32.DLL" (ByRef srcAddr() As Short, _
    ' OLD     ByRef destAddr() As Short, ByVal numBytes As Integer)

    ' OLD iNet_memcopy_flt32(), no longer exported from DLL
    ' OLD Declare Sub iNet_memcopy_flt32 Lib "iNet32.DLL" (ByRef srcAddr() As Single, _
    ' OLD     ByRef destAddr() As Single, ByVal numBytes As Integer)

    ' ----------------------------------------------------------------------------
    '  FUNCTION    VbFlt32_to_Address()
    '
    '  PURPOSE     Returns addr to FLT32 VB array data and locks handle to data.
    '              After using the address, you must call gcHandle.free() to unlock the handle
    '
    '  HISTORY     6/1/02   created, gsw
    ' -----------------------------------------------------------------------------
    Function VbFlt32Array_to_Address(ByRef flt32array() As Single, ByRef gcHandle As GCHandle) As Integer

        ' get handle to FLT32 internal information
        gcHandle = gcHandle.Alloc(flt32array, GCHandleType.Pinned)

        ' get addr to FLT32 array, and lock it's handle
        Dim arrayDataAddr As Integer = gcHandle.AddrOfPinnedObject.ToInt32

        ' return a ptr to the FLT32 array data
        VbFlt32Array_to_Address = arrayDataAddr

    End Function

    ' ----------------------------------------------------------------------------
    '  FUNCTION    VbInt16_to_Address()
    '
    '  PURPOSE     Returns addr to INT16 VB array data and locks handle to data.
    '              After using the address, you must call free.gcHandle() to unlock the handle
    '
    '  HISTORY     6/1/02   created, gsw
    ' -----------------------------------------------------------------------------
    Function VbInt16Array_to_Address(ByRef int16array() As Short, _
        ByRef gcHandle As GCHandle) As Integer

        ' get handle to FLT32 internal information
        gcHandle = gcHandle.Alloc(int16array, GCHandleType.Pinned)

        ' get addr to FLT32 array, and lock it's handle
        Dim arrayDataAddr As Integer = gcHandle.AddrOfPinnedObject.ToInt32

        ' return a ptr to the FLT32 array data
        VbInt16Array_to_Address = arrayDataAddr

    End Function


    ' ----------------------------------------------------------------------------
    '  FUNCTION    fft_DLL()
    '
    '  PURPOSE     This function computes the fft on an array of numbers.
    '              It is given 2^power real values and returns 2^(power-1)
    '              complex pairs (real, imaginary, real, imaginary).  Please
    '              give it an array that is 2 longer than 2^power -- the algorithm
    '              needs this.  Several notes:
    '
    '              *   This routine is given 2^power real values.
    '              *   The data[] array must contain 2 extra elements at
    '                  the end -- CAREFUL!
    '              *   We return 2^(power-1) complex pairs values.
    '              *   If you take a sine wave and multiply it by a hamming
    '                  window, it is like convolving the frequency response
    '                  of the hamming window with a delta function, and
    '                  that is what you see at the fft output.  The
    '                  frequency response of the hamming window is
    '                  one main lobe with side lobes 42dB down.
    '
    '  HISTORY     8/10/98     tsfr'ed to iNet - gsw
    ' -----------------------------------------------------------------------------

    ' DLL version
    Declare Sub fft_DLL Lib "iNet32.DLL" (ByVal dataAddressP As Integer, ByVal power As Short)

    ' VB Friendly version
    Public Sub fft_VB(ByRef dest_flt32() As Single, ByVal power As Short)

        ' get ptr to dest_flt32 array data and lock the data handle
        Dim destHandle As GCHandle
        Dim destAddr As Integer = VbFlt32Array_to_Address(dest_flt32, destHandle)

        ' process the array(s) inside the iNet32.dll
        fft_DLL(destAddr, power)

        ' unlock the data handle
        destHandle.Free()

    End Sub

    ' -------------------------------------------------------------------------------------
    '  FUNCTION    MultiplyArrayByHammWindow()
    '
    '  PURPOSE     This multiplies the flt32 array by a Hamming window,
    '              # of pts = 2 ^ power, power is {1...18}.  Returns TRUE if successful.
    '
    '  HISTORY     8/10/98 created - gsw
    ' ----------------------------------------------------------------------------------- */

    ' DLL version
    Declare Function MultiplyArrayByHammWindow_DLL Lib "iNet32.DLL" ( _
        ByVal dataAddressP As Integer, ByVal power As Short) As Short

    ' VB Friendly version
    Public Function MultiplyArrayByHammWindow_VB( _
        ByRef dest_flt32() As Single, ByVal power As Short) As Short

        ' get ptr to dest_flt32 array data and lock the data handle
        Dim destHandle As GCHandle
        Dim destAddr As Integer = VbFlt32Array_to_Address(dest_flt32, destHandle)

        ' process the array(s) inside the iNet32.dll
        MultiplyArrayByHammWindow_VB = MultiplyArrayByHammWindow_DLL(destAddr, power)

        ' unlock the data handle
        destHandle.Free()

    End Function

    ' -----------------------------------------------------------------------------
    '  FUNCTION    CalculateWindow_DLL()
    '
    '  PURPOSE     This uses the cos() function to create a float (0.0 to 1.0 values)
    '              window (Hamm=0, Hann=1, Blackman=2). Returns TRUE if successful; FALSE otherwise.
    '
    '              Ham:        h(n) = .54 - .46*cos(2n/(N-1)) 0 <= n <= N-1
    '              Hann:       h(n) = .50 - .50*cos(2n/(N-1)) 0 <= n <= N-1
    '              Blackman    h(n) = .42 - .50*cos(2n/(N-1)) + .08*cos(4n/(N-1))    0 <= n <= N-1
    '
    '  HISTORY     8/10/98 created - gsw
    ' -----------------------------------------------------------------------------

    ' DLL Version
    Declare Function CalculateWindow_DLL Lib "iNet32.DLL" (ByVal numpts As Integer, _
        ByVal wType As Short, ByVal windowAddressP As Integer) As Short

    ' VB Friendly version
    Public Function CalculateWindow_VB(ByVal numpts As Integer, ByVal wType As Short, _
        ByRef dest_flt32() As Single) As Short

        ' get ptr to dest_flt32 array data and lock the data handle
        Dim destHandle As GCHandle
        Dim destAddr As Integer = VbFlt32Array_to_Address(dest_flt32, destHandle)

        ' process the array(s) inside the iNet32.dll
        CalculateWindow_VB = CalculateWindow_DLL(numpts, wType, destAddr)

        ' unlock the data handle
        destHandle.Free()

    End Function

    ' -----------------------------------------------------------------------------
    '  FUNCTION    MultiplyTwoFlt32Arrays_DLL()
    '
    '  PURPOSE     This multiplies two flt32 arrays.
    '
    '  HISTORY     8/10/98 created - gsw
    ' -----------------------------------------------------------------------------

    ' DLL version
    Declare Sub MultiplyTwoFlt32Arrays_DLL Lib "iNet32.DLL" (ByVal srcAddressP As Integer, _
        ByVal destAddressP As Integer, ByVal numpts As Integer)

    ' VB Friendly version
    Public Sub MultiplyTwoFlt32Arrays_DLL( _
        ByRef src_flt32() As Single, ByRef dest_flt32() As Single, ByVal numFlt32Pts As Integer)

        ' get ptr to src_flt32 array data and lock the data handle
        Dim srcHandle As GCHandle
        Dim srcAddr As Integer = VbFlt32Array_to_Address(src_flt32, srcHandle)

        ' get ptr to dest_flt32 array data
        Dim destHandle As GCHandle
        Dim destAddr As Integer = VbFlt32Array_to_Address(dest_flt32, destHandle)

        ' process the array(s) inside the iNet32.dll
        MultiplyTwoFlt32Arrays_DLL(srcAddr, destAddr, numFlt32Pts)

        ' unlock the data handle
        srcHandle.Free()
        destHandle.Free()

    End Sub

    ' -----------------------------------------------------------------------------
    '   FUNCTION    ComplexToMagnitudes_DLL()
    '
    '   PURPOSE     This converts {real, imaginary} complex pairs in a flt32 array to
    '               magnitudes = sqrt(real^2 + imaginary^2).
    '
    '   HISTORY     8/10/98 created - gsw
    ' -----------------------------------------------------------------------------

    ' DLL Version
    Declare Sub ComplexToMagnitudes_DLL Lib "iNet32.DLL" (ByVal dataAddressP As Integer, _
        ByVal numComplexPairs As Integer)

    ' VB Friendly version
    Public Sub ComplexToMagnitudes_VB(ByRef dest_flt32() As Single, ByVal numComplexPairs As Short)

        ' get ptr to dest_flt32 array data and lock the data handle
        Dim destHandle As GCHandle
        Dim destAddr As Integer = VbFlt32Array_to_Address(dest_flt32, destHandle)

        ' process the array(s) inside the iNet32.dll
        ComplexToMagnitudes_DLL(destAddr, numComplexPairs)

        ' unlock the data handle
        destHandle.Free()

    End Sub

    ' -------------------------------------------------------------------------------------
    '   FUNCTION    MultConstantByArray_DLL()
    '
    '   PURPOSE     Multiply a constant by a flt32 array.
    '
    '   HISTORY     8/10/98 created - gsw
    ' ----------------------------------------------------------------------------------- */

    ' DLL Version
    Declare Sub MultConstantByArray_DLL Lib "iNet32.DLL" (ByVal srcAddressP As Integer, _
        ByVal destAddressP As Integer, ByVal constant As Single, ByVal numpts As Integer)

    ' VB Friendly version
    Public Sub MultConstantByArray_VB( _
        ByRef src_flt32() As Single, ByRef dest_flt32() As Single, _
        ByVal constant As Single, ByVal numFlt32Pts As Integer)

        ' get ptr to src_flt32 array data
        Dim srcHandle As GCHandle
        Dim srcAddr As Integer = VbFlt32Array_to_Address(src_flt32, srcHandle)

        ' get ptr to dest_flt32 array data
        Dim destHandle As GCHandle
        Dim destAddr As Integer = VbFlt32Array_to_Address(dest_flt32, destHandle)

        ' process the array(s) inside the iNet32.dll
        MultConstantByArray_DLL(srcAddr, destAddr, constant, numFlt32Pts)

        ' unlock the data handle
        srcHandle.Free()
        destHandle.Free()

    End Sub

    ' -------------------------------------------------------------------------------------
    '   FUNCTION    AlertOnError()
    '
    '   PURPOSE     Show an alert if e contains an error (i.e. non 0 value).
    '
    '   HISTORY     8/10/98 created - gsw
    ' ----------------------------------------------------------------------------------- */
    Public Function AlertOnError(ByVal e As Short) As Short

        AlertOnError = Show_ALERT_if_hit_iNet_Error(e)

    End Function


    ' -------------------------------------------------------------------------------------
    '   FUNCTION    SaveOrLoad_DataAndState()
    '
    '   PURPOSE     Save or Load instruNet waveform data and program parameters
    '               via the File Save/Open dialog.
    '
    '   PARAMS      saveLoadEndUserData - If true, EndUserData_SaveOrLoad(), an end user routine,
    '                   is called to save/load end user parameters.
    '
    '               saveOrLoad - set to iSave if you want to Save, and set to iLoad if you want to load
    '
    '               doTimingInfo - set true if you want to save/load digitize timing parameters
    '
    '               doDisplayBoxInfo - set true if you want to save/load DisplayBox parameters
    '
    '               doDisplayInfo - set true if you want to save/load Display parameters
    '
    '               doDigitizeData - set true if you want to save/load digitize data
    '
    '               reportOnUnrecognizedLines - set to true if want to show an alert if you load in a file
    '                  and hit some data that you do not recognize.
    '
    '   HISTORY     8/10/98 created - gsw
    ' -----------------------------------------------------------------------------------
    Public Sub SaveOrLoad_DataAndState( _
        ByVal saveLoadEndUserData As Short, _
        ByVal saveOrLoad As Short, _
        ByVal doTimingInfo As Short, _
        ByVal doDisplayBoxInfo As Short, _
        ByVal doDisplayInfo As Short, _
        ByVal doDigitizeData As Short, _
        ByVal reportOnUnrecognizedLines As Short, _
        ByRef g_form As System.Drawing.Graphics)

        ' ---------------------------------------------------------------
        ' Local Variables

        Dim cNum_newDisplay, numDisp_inFile, gNumBoxes_inFile, fileNum As Integer
        Dim numUnrecognizedLines, numValidBytes As Integer
        Dim hitError, save, e, bNum, dNum, cNum As Short
        Dim dValue As Double
        Dim info, sPathName As String

        ' if we are currently digitizing, we cannot do save/load
        If gCurrentlyDigitizing Then Exit Sub

        ' --------------------------------------------------------------
        ' SAVE/LOAD DATA IN BINARY MERGE FILE ("BMerge1.ioW") AND
        ' SAVE INSTRUNET NETWORK SETTINGS IN A ".prf" FILE.  This is placed
        ' into a new folder that is named by the user.

        save = (saveOrLoad = iSave) ' true if we are saving
        hitError = False
        On Error GoTo ErrorHandler

        e = iNetErr_None ' initialize
        numUnrecognizedLines = 0

        If save Then ' if saving

            ' Copy the VB Digitize data buffer data into the iNet32.DLL DRIVER RAM BUFFERS
            ' (this also sets the PtsPerScan parameter inside iNet32.dll)
            Call VbDataBuffers_CopyToDLLDvrBuffers()

            ' set the fileType to BinaryMerge
            Call SetFieldValue_iNet(0, 0, 1, driver_ChanNum_RecordOptions, _
                sgt_MasterDirectory, fldNum_Directory_FileType, ft_BinaryMerge)

            ' You cannot show the iNet World window in here, since
            ' closing that window causes the Driver Ram buffers to be disposed.

            ' Press the Save button in instruNet world, to save the data
            e = PRESS_iNet_BUTTON(iNetCM_BtnPress_Record_Save)
            If ErrorHere(e, "ss21") Then Exit Sub

            ' Disable all DigitizeChannels inside the iNet driver (kills the DRIVER RAM buffers)
            e = DisableAllChannelsForDigitizing()
            If ErrorHere(e, "ss22") Then Exit Sub

        Else ' else we are loading

            ' Press the Open button in instruNet world, to load the data.
            e = PRESS_iNet_BUTTON(iNetCM_BtnPress_Record_Open)
            If ErrorHere(e, "ss23") Then Exit Sub

        End If

        ' get a copy of pathname of the folder that contains this data.
        Call iNet_VB_cString(0, 0, 1, driver_ChanNum_RecordOptions, _
            sgt_MasterDirectory, fldNum_Directory_PathName, intention_getValue, sPathName)

        ' --------------------------------------------------------------
        ' CREATE or OPEN "\Internal.iNt" FILE

        ' create/open a file and place it in the data folder that we just created.
        fileNum = FreeFile()
        If save Then
            FileOpen(fileNum, (sPathName & "\Internal.iNt"), OpenMode.Output)
        Else
            FileOpen(fileNum, (sPathName & "\Internal.iNt"), OpenMode.Input)
        End If
        If hitError Then Exit Sub

        ' --------------------------------------------------------------
        ' SAVE/LOAD INFORMATION ABOUT TIMING PARAMETERS

        If save And doTimingInfo Then ' if saving...

            WriteLine(fileNum, "TimingParams_Start", 0)

            WriteLine(fileNum, "gNumScans", gNumScans)

            WriteLine(fileNum, "gPtsPerScan_nextDigi", gPtsPerScan_nextDigi)

            WriteLine(fileNum, "gSampleRate_nextDigi", gSampleRate_nextDigi)

            WriteLine(fileNum, "gContinuousScans", gContinuousScans)

            WriteLine(fileNum, "gProcessSegmentsInsteadOfScans", gProcessSegmentsInsteadOfScans)

            WriteLine(fileNum, "gRun_EndUser_PostAcqAnalysis", gRun_EndUser_PostAcqAnalysis)

            WriteLine(fileNum, "gRun_EndUser_SegmentProcessor", gRun_EndUser_SegmentProcessor)

            WriteLine(fileNum, "gCopyChannelsFromNetworkPage", gCopyChannelsFromNetworkPage)

            WriteLine(fileNum, "TimingParams_End", 0)

        ElseIf doTimingInfo Then  ' else, we are loading

            Do  ' load in the DisplayInfo data

                Input(fileNum, info)
                Input(fileNum, dValue) ' load in 1 line of info

                If StrComp(info, "TimingParams_Start") = 0 Then
                    dValue = dValue * 1
                ElseIf StrComp(info, "TimingParams_End") = 0 Then : Exit Do

                ElseIf StrComp(info, "gNumScans") = 0 Then : gNumScans = dValue
                ElseIf StrComp(info, "gPtsPerScan_nextDigi") = 0 Then : gPtsPerScan_nextDigi = dValue
                ElseIf StrComp(info, "gSampleRate_nextDigi") = 0 Then : gSampleRate_nextDigi = dValue
                ElseIf StrComp(info, "gContinuousScans") = 0 Then : gContinuousScans = dValue
                ElseIf StrComp(info, "gProcessSegmentsInsteadOfScans") = 0 Then : gProcessSegmentsInsteadOfScans = dValue
                ElseIf StrComp(info, "gRun_EndUser_PostAcqAnalysis") = 0 Then : gRun_EndUser_PostAcqAnalysis = dValue
                ElseIf StrComp(info, "gRun_EndUser_SegmentProcessor") = 0 Then : gRun_EndUser_SegmentProcessor = dValue
                ElseIf StrComp(info, "gCopyChannelsFromNetworkPage") = 0 Then : gCopyChannelsFromNetworkPage = dValue

                Else
                    numUnrecognizedLines = numUnrecognizedLines + 1
                End If

            Loop While Not (EOF(fileNum))

            ' Set the digitize parameters
            Call DigitizeProcess_SetTimingParams(gPtsPerScan_nextDigi, gNumScans, _
                gSampleRate_nextDigi, gContinuousScans, gProcessSegmentsInsteadOfScans, _
                gRun_EndUser_PostAcqAnalysis, gRun_EndUser_SegmentProcessor, _
                gCopyChannelsFromNetworkPage, gWaitTillDigitizationCompletes)

        End If

        ' --------------------------------------------------------------
        ' SAVE/LOAD INFORMATION ABOUT Digitize Data

        If save And doDigitizeData Then ' if saving...

            WriteLine(fileNum, 0, "DigitizeData_Start", gNumDigiCh)

            ' write out info associated with the displays

            For cNum = 0 To (gNumDigiCh - 1) ' scan through all Digitize Channels

                With gDigiChan(cNum)

                    WriteLine(fileNum, cNum, "netNum", .netNum)

                    WriteLine(fileNum, cNum, "deviceNum", .deviceNum)

                    WriteLine(fileNum, cNum, "moduleNum", .moduleNum)

                    WriteLine(fileNum, cNum, "chanNum", .chanNum)

                    WriteLine(fileNum, cNum, "bNum", .bNum)

                    WriteLine(fileNum, cNum, "dNum", .dNum)

                    WriteLine(fileNum, cNum, "numValidPts", .dataArrayY.numValidPts)

                    WriteLine(fileNum, cNum, "numBufferPts", .dataArrayY.numBufferPts)

                    WriteLine(fileNum, cNum, "ptsPerXunit", .dataArrayY.ptsPerXunit)

                    WriteLine(fileNum, cNum, "firstPt_Xunits", .dataArrayY.firstPt_Xunits)

                    WriteLine(fileNum, cNum, "massageType", .massageType)

                    WriteLine(fileNum, cNum, "runD", .runEndUser_SegmentProcessor)

                End With

            Next cNum

            If hitError Then Exit Sub

            WriteLine(fileNum, 0, "DigitizeData_End", 0)

        ElseIf doDigitizeData Then  ' else, we are loading

            Do  ' load in the DisplayInfo data

                ' load in 1 line of info
                Input(fileNum, cNum)
                Input(fileNum, info)
                Input(fileNum, dValue)

                ' if we are at the beginnning, get the # of
                ' DigitizeChannels, and set the gDigiChan() array accordingly.
                If StrComp(info, "DigitizeData_Start") = 0 Then

                    gNumDigiCh = dValue ' # of digitize channels

                    ' Set the size of the gDigiChan() array to accomodate the info.
                    ReDim Preserve gDigiChan(gNumDigiCh)

                End If

                ' if this is not in our ChannelList...
                If (cNum < 0 Or cNum >= gNumDigiCh) Then

                    ' bail out of this section if we are at the end
                    If StrComp(info, "DigitizeData_End") = 0 Then Exit Do

                    ' record that we hit an unrecognized line
                    numUnrecognizedLines = numUnrecognizedLines + 1

                Else ' else, read this data & place into internal parameters


                    ' write this data into internal parameters
                    With gDigiChan(cNum)

                        If StrComp(info, "DigitizeData_End") = 0 Then
                            Exit Do
                        ElseIf StrComp(info, "DigitizeData_Start") = 0 Then : dValue = dValue * 1 ' this was handled above
                        ElseIf StrComp(info, "netNum") = 0 Then : .netNum = dValue
                        ElseIf StrComp(info, "deviceNum") = 0 Then : .deviceNum = dValue
                        ElseIf StrComp(info, "moduleNum") = 0 Then : .moduleNum = dValue
                        ElseIf StrComp(info, "chanNum") = 0 Then : .chanNum = dValue
                        ElseIf StrComp(info, "bNum") = 0 Then : .bNum = dValue
                        ElseIf StrComp(info, "dNum") = 0 Then : .dNum = dValue
                        ElseIf StrComp(info, "numValidPts") = 0 Then : .dataArrayY.numValidPts = dValue
                        ElseIf StrComp(info, "numBufferPts") = 0 Then : .dataArrayY.numBufferPts = dValue
                        ElseIf StrComp(info, "ptsPerXunit") = 0 Then : .dataArrayY.ptsPerXunit = dValue
                        ElseIf StrComp(info, "firstPt_Xunits") = 0 Then : .dataArrayY.firstPt_Xunits = dValue
                        ElseIf StrComp(info, "massageType") = 0 Then : .massageType = dValue
                        ElseIf StrComp(info, "runD") = 0 Then : .runEndUser_SegmentProcessor = dValue
                        Else
                            numUnrecognizedLines = numUnrecognizedLines + 1
                        End If

                    End With

                End If

            Loop While Not (EOF(fileNum))

            ' load channel address into the gDigiChan() array.
            For cNum = 0 To gNumDigiCh - 1 ' scan through all digitize channels
                With gDigiChan(cNum) ' w/ this channel...
                    Call LoadChannelAddress(gDigiChan(cNum).chAddrDLL, .netNum, .deviceNum, .moduleNum, .chanNum)
                End With
            Next cNum

            ' Load iNet32.DLL DRIVER RAM buffer data into VB dataArrayY.data() arrays
            Call VbDataBuffers_CopyFromDLLDvrBuffers()

            ' Disable all DigitizeChannels inside the iNet driver (to close
            ' the Driver Ram Buffers).
            e = DisableAllChannelsForDigitizing()
            If ErrorHere(e, "ctd2") Then Exit Sub

        End If

        ' --------------------------------------------------------------
        ' SAVE/LOAD INFORMATION ABOUT DISPLAY BOXES

        If save And doDisplayBoxInfo Then ' if saving...

            WriteLine(fileNum, 0, "DisplayBox_Start", gNumBoxes)

            ' write out info associated with the displays
            For bNum = 0 To gNumBoxes - 1 ' scan through all DisplayBoxes

                With gBox(bNum)

                    WriteLine(fileNum, bNum, "numDisp", .numDisp)

                    WriteLine(fileNum, bNum, "hPos_scroll_lastValue", .hPos_scroll_lastValue)

                    WriteLine(fileNum, bNum, "vPos_scroll_lastValue", .vPos_scroll_lastValue)

                    WriteLine(fileNum, bNum, "autoAddPolicy", .autoAddPolicy)

                End With

                If hitError Then Exit Sub

            Next bNum

            WriteLine(fileNum, 0, "DisplayBox_End", 0)

        ElseIf doDisplayBoxInfo Then  ' else, we are loading

            Do  ' load in the DisplayInfo data

                ' load in 1 line of info
                Input(fileNum, bNum)
                Input(fileNum, info)
                Input(fileNum, dValue)

                ' if we do not have a box or display to
                ' put the data in, then go to the next
                ' section in the file.
                If (bNum < 0 Or bNum >= gNumBoxes) Then

                    ' bail out of this section if we are at the end
                    If StrComp(info, "DisplayBox_End") = 0 Then Exit Do

                    ' record that we hit an unrecognized line
                    If StrComp(info, "DisplayBox_Start") <> 0 Then
                        numUnrecognizedLines = numUnrecognizedLines + 1
                    End If

                Else ' else, read this data & place into internal parameters

                    With gBox(bNum)

                        If StrComp(info, "DisplayBox_Start") = 0 Then
                            gNumBoxes_inFile = dValue
                        ElseIf StrComp(info, "DisplayBox_End") = 0 Then : Exit Do

                        ElseIf StrComp(info, "numDisp") = 0 Then
                            numDisp_inFile = dValue
                        ElseIf StrComp(info, "hPos_scroll_lastValue") = 0 Then
                            .hPos_scroll_lastValue = dValue
                        ElseIf StrComp(info, "vPos_scroll_lastValue") = 0 Then
                            .vPos_scroll_lastValue = dValue
                        ElseIf StrComp(info, "autoAddPolicy") = 0 Then
                            .autoAddPolicy = dValue

                        Else
                            numUnrecognizedLines = numUnrecognizedLines + 1
                        End If

                    End With

                End If

            Loop While Not (EOF(fileNum))

        End If

        ' --------------------------------------------------------------
        ' SAVE/LOAD INFORMATION ABOUT DISPLAYS

        If save And doDisplayInfo Then ' if saving...

            WriteLine(fileNum, 0, 0, "DisplayInfo_Start", 0)

            ' write out info associated with the displays
            For bNum = 0 To gNumBoxes - 1 ' scan through all DisplayBoxes

                For dNum = 0 To (gBox(bNum).numDisp - 1) ' scan through all Displays

                    With gBox(bNum).disp(dNum)

                        WriteLine(fileNum, bNum, dNum, "left", .dispLeft_xEU)

                        WriteLine(fileNum, bNum, dNum, "right", .dispRight_xEU)

                        WriteLine(fileNum, bNum, dNum, "top", .dispTop_yEu)

                        WriteLine(fileNum, bNum, dNum, "bot", .dispBot_yEu)

                        WriteLine(fileNum, bNum, dNum, "showGrid", .showGrid)

                        WriteLine(fileNum, bNum, dNum, "cNum", .cNum)

                        WriteLine(fileNum, bNum, dNum, "displayType", .displayType)

                    End With

                Next dNum

                If hitError Then Exit Sub

            Next bNum

            WriteLine(fileNum, 0, 0, "DisplayInfo_End", 0)

        ElseIf doDisplayInfo Then  ' else, we are loading

            Do  ' load in the DisplayInfo data

                ' load in 1 line of info
                Input(fileNum, bNum)
                Input(fileNum, dNum)
                Input(fileNum, info)
                Input(fileNum, dValue)

                ' if we do not have a box or display to put the data in, then go to the next
                ' section in the file.
                If ((bNum < 0 Or bNum >= gNumBoxes) Or (dNum < 0)) Then

                    ' bail out of this section if we are at the end
                    If StrComp(info, "DisplayInfo_End") = 0 Then Exit Do

                    ' record that we hit an unrecognized line
                    If StrComp(info, "DisplayInfo_Start") <> 0 Then
                        numUnrecognizedLines = numUnrecognizedLines + 1
                    End If

                Else ' else, read this data & place into internal parameters

                    ' if we do not have a Display for this data,
                    ' we'll add it to the DisplayBox.
                    If (dNum >= gBox(bNum).numDisp) Then

                        ' we'll assume that this Box shows cNum #0 to cNum# (numDisplays-1)
                        cNum_newDisplay = dNum

                        ' Add 1 Display the Box and attach it to the digitize channel.
                        Call Add_DigitizeChannels_To_OneBox(bNum, cNum_newDisplay, cNum_newDisplay, kYindexDisplay, g_form)

                        ' make sure we made it
                        If (dNum >= gBox(bNum).numDisp) Then EndProgram_iNet(("AddDisp Err"))

                    End If

                    ' write this data into internal parameters
                    With gBox(bNum).disp(dNum)

                        If StrComp(info, "DisplayInfo_Start") = 0 Then
                            dValue = dValue * 1

                        ElseIf StrComp(info, "DisplayInfo_End") = 0 Then : Exit Do
                        ElseIf StrComp(info, "left") = 0 Then
                            .dispLeft_xEU = dValue
                        ElseIf StrComp(info, "right") = 0 Then
                            .dispRight_xEU = dValue
                        ElseIf StrComp(info, "top") = 0 Then
                            .dispTop_yEu = dValue
                        ElseIf StrComp(info, "bot") = 0 Then
                            .dispBot_yEu = dValue
                        ElseIf StrComp(info, "showGrid") = 0 Then
                            .showGrid = dValue
                        ElseIf StrComp(info, "cNum") = 0 Then
                            .cNum = dValue
                        ElseIf StrComp(info, "displayType") = 0 Then
                            .displayType = dValue

                        Else
                            numUnrecognizedLines = numUnrecognizedLines + 1
                        End If

                    End With

                End If

            Loop While Not (EOF(fileNum))

        End If

        ' --------------------------------------------------------------
        ' CLOSE FILE

        ' close the open file
        FileClose(fileNum)
        If hitError Then Exit Sub

        ' --------------------------------------------------------------
        ' Save/Load EndUser Data, if necessary

        If saveLoadEndUserData Then

            ' call the end user's save routine
            Call gEndUser_CLASS.EndUser_SaveLoadResults(sPathName, saveOrLoad)

        End If

        ' --------------------------------------------------------------
        ' Reset controls and redraw if loading Display info.

        If Not (save) And doDisplayInfo Then ' if we are loading

            ' Set the horiz and vertical scales and positions for
            ' all the displays to the saved .dispLeft_xEU, .dispRight_xEU,
            ' .dispTop_yEU, .dispBot_yEU values.  We also update
            ' scrollbars to their past values.

            Reset_Controls_AllBoxes( _
                (reset_hScaleToInternalValues + reset_ptsPerHscale + reset_vScaleToInternalValues), g_form)

            ' Clear all DispBoxes and then redraw them with the new hScales.

            Redraw_AllBoxes(g_form)

        End If

        ' --------------------------------------------------------------
        ' Exit

        If (numUnrecognizedLines > 0) And reportOnUnrecognizedLines Then
            MsgBox("We hit " & Str(numUnrecognizedLines) & " lines of unrecognized data.")
        End If

        Exit Sub ' end of this subroutine

        ' --------------------------------------------------------------
        ' Handle file I/O error case.

ErrorHandler:

        If (hitError = False) Then      ' if this is a vb error...
            MsgBox(Err.Description)     ' show msg about error
        Else
            MsgBox("We hit an error during file I/O.")
        End If

        ' stop saving/loading after error

    End Sub


    '-------------------------------------------------------------------------------------
    ' FUNCTION   Setup_ListBox()
    '
    ' PURPOSE    Establishes a list box on the Form that will accept
    '            Print_to_ListBox() commands.
    '
    ' HISTORY    6/22/2002  created - gsw
    '-------------------------------------------------------------------------------------

    Public gPrint_to_ListBox_ListBox As System.Windows.Forms.ListBox = Nothing

    Public Sub Setup_ListBox(ByRef Print_to_ListBox_ListBox As System.Windows.Forms.ListBox)

        gPrint_to_ListBox_ListBox = Print_to_ListBox_ListBox

    End Sub


    '-------------------------------------------------------------------------------------
    ' FUNCTION   Print_to_ListBox()
    '
    ' PURPOSE    Add the newRowStr to the list box, and automatically scroll as needed.
    '
    ' HISTORY    6/22/2002  created - gsw
    '-------------------------------------------------------------------------------------
    Public Sub Print_to_ListBox(ByVal newRowStr As String)

        If IsNothing(gPrint_to_ListBox_ListBox) Then
            MsgBox("You need to call Setup_ListBox() before calling Print_to_ListBox()")
            Exit Sub
        End If

        ' add text to bottom of listbox
        gPrint_to_ListBox_ListBox.Items.Add(newRowStr)

        ' calc # of items (each is printed on one row) in listBox
        Dim numOfItemsInListbox As Integer = gPrint_to_ListBox_ListBox.Items.Count()

        ' calc # of rows in listbox
        Dim listBoxHeight_numRows As Integer = -0.5 + (gPrint_to_ListBox_ListBox.Height _
            / gPrint_to_ListBox_ListBox.ItemHeight)

        ' if the Items are printed below the last row shown on the list box, then move it up
        If ((gPrint_to_ListBox_ListBox.TopIndex + listBoxHeight_numRows) < _
            numOfItemsInListbox) Then

            ' scroll up so that the last item is showing (.TopIndex is BASE 0)
            gPrint_to_ListBox_ListBox.TopIndex = numOfItemsInListbox - listBoxHeight_numRows

        End If

    End Sub

    '-------------------------------------------------------------------------------------
    ' FUNCTION   Setup_TextBox()
    '
    ' PURPOSE    Establishes a Text box on the Form that will accept
    '            Print_to_TextBox() commands.
    '
    ' HISTORY    6/22/2002  created - gsw
    '-------------------------------------------------------------------------------------

    Public gPrint_to_TextBox_TextBox As System.Windows.Forms.TextBox = Nothing

    Public Sub Setup_TextBox(ByRef Print_to_TextBox_TextBox As System.Windows.Forms.TextBox)

        gPrint_to_TextBox_TextBox = Print_to_TextBox_TextBox

    End Sub


    '-------------------------------------------------------------------------------------
    ' FUNCTION   Print_to_TextBox()
    '
    ' PURPOSE    Add the newRowStr to the list box, add a carriage return to the end
    '            end of the added string, and automatically scroll as needed.
    '
    ' HISTORY    6/22/2002  created - gsw
    '-------------------------------------------------------------------------------------
    Public Sub Print_to_TextBox(ByVal newRowStr As String)

        If IsNothing(gPrint_to_TextBox_TextBox) Then
            MsgBox("You need to call Setup_TextBox() before calling Print_to_TextBox()")
            Exit Sub
        End If

        ' append text to exisiting text in TextBox
        gPrint_to_TextBox_TextBox.AppendText(newRowStr & gReturnStr)

    End Sub

    ' -----------------------------------------------------------------------------------
    '  FUNCTION    Open_instruNet_Window_VB()       Opens an instruNet window
    '
    '              Open_instruNet_Window()          Opens an instruNet window and returns
    '                                               the resulting chan selected by the end
    '                                               user in the netNum..fieldNum parameters
    '
    '  iNetWindowType      1  = iNetCM_OpenWindow_Record
    '                      2  = iNetCM_OpenWindow_Network
    '                      3  = iNetCM_OpenWindow_Test
    '                      4  = iNetCM_OpenWindow_FullProbe
    '                      5  = iNetCM_OpenWindow_ChanSelection
    '                      6  = iNetCM_OpenWindow_FieldSelection
    '                      7  = iNetCM_OpenWindow_OneFieldSet
    '                      8  = iNetCM_OpenWindow_AllFieldSets
    '                      9  = iNetCM_OpenWindow_RecTiming
    '                      10 = iNetCM_OpenWindow_RecTrigger
    '                      11 = iNetCM_OpenWindow_RecOptions
    '                      12 = iNetCM_OpenWindow_RecFile
    '                      13 = iNetCM_OpenWindow_SimpleTiming
    '                      (pts-per-scan and sample-rate)
    '                      14 = iNetCM_OpenWindow_CalOptions
    '
    '  HISTORY     7/5/98  added - gsw
    ' -----------------------------------------------------------------------------------
    Public Sub Open_instruNet_Window_VB(ByVal iNetWindowType As Short)

        Dim netNum, deviceNum, moduleNum, chanNum, settingGp As Short
        Dim fieldNum, userPressedOk As Short

        Call Open_instruNet_Window(iNetWindowType, netNum, deviceNum, moduleNum, chanNum, _
            settingGp, fieldNum, userPressedOk)

    End Sub

    ' This is the direct to DLL version, Open_instruNet_Window_VB() is easier to use.
    Declare Function Open_instruNet_Window Lib "iNet32.DLL" ( _
        ByVal iNetWindowType As Short, _
        ByRef netNumP As Short, _
        ByRef deviceNumP As Short, _
        ByRef moduleNumP As Short, _
        ByRef chanNumP As Short, _
        ByRef settingGpP As Short, _
        ByRef fieldNumP As Short, _
        ByRef userPressedOkP As Short) As Short


    ' -----------------------------------------------------------------------------------  
    ' FUNCTION  ErrorHere()
    '
    ' PURPOSE   This function shows an alert if there is an error (i.e.
    '           "errorCode" is not 0).  Also, we show a message as
    '           to where the error occured in the code.
    '
    ' HISTORY   6/6/02  Updated to VS .Net v7, gsw
    ' -----------------------------------------------------------------------------------  
    Function ErrorHere(ByVal errorCode As Short, ByVal message As String) As Short

        If errorCode <> 0 Then

            ' show alert if errorCode != 0
            AlertOnError(errorCode)

            If Len(message) <> 0 Then
                message = "We hit an error at " & message
                MsgBox(message)
            End If

        End If

    End Function


    ' -----------------------------------------------------------------------------------  
    ' FUNCTION  Initialize_instruNet_interface()
    '
    ' PURPOSE   Initialize instruNet interface code.
    '
    ' HISTORY   6/6/02  Updated to VS .Net v7, gsw
    ' -----------------------------------------------------------------------------------  
    Public Sub Initialize_instruNet_interface()

        ' Make sure we only do this once per application launch
        Static weHaveAlreadyInitialized As Short = 0

        ' if we are here for the 1st time...
        If weHaveAlreadyInitialized = 0 Then

            weHaveAlreadyInitialized = 1

            ' Load instruNet driver DLL.  Program will End if this fails.
            Call Load_instruNet_Driver_VB()

        End If

    End Sub


    ' -----------------------------------------------------------------------------------  
    ' FUNCTION  Initialize_instruNet_MAIN()
    '
    ' PURPOSE   Load instruNet Driver and Initialize VB interface utilities
    '
    '           'demoPolicy' is set to:
    '
    '                wJumpIntoDemoModeIfNoDeviceExists - only go to demo if not hard exist
    '                wDoNotJumpIntoDemoModeIfNoDeviceExists - never enter demo mode
    '                wJumpIntoRightNow - jump into demo mode right now.
    '
    ' HISTORY   6/6/02  Updated to VS .Net v7, gsw
    ' ----------------------------------------------------------------------------------- 
    Public Sub Initialize_instruNet_MAIN( _
        ByRef endUser_Class As EndUser_Window, _
        ByRef endUser_form As System.Windows.Forms.Form, _
        ByVal demoPolicy As Short)

        Dim weHaveAtLeastOneDevice As Short

        ' -------------------------------------------------------------
        ' Initialize instruNet Interface utilities.

        Call Initialize_instruNet_interface()

        ' -------------------------------------------------------------
        ' If a network device does not exist, then jump into demo mode.

        gSimulatingHardware = False

        ' if we are to enter demo mode right now...
        If demoPolicy = wJumpIntoRightNow Then

            GoTo JumpIntoDemoMode

            ' else if we are to enter demo mode only if there is no hardware...
        ElseIf demoPolicy = wJumpIntoDemoModeIfNoDeviceExists Then

            ' calc if we have a network device attached
            weHaveAtLeastOneDevice = False
            If (gNumNetworks >= 1) Then
                If (GET_NUMBER_OF_DEVICES(1) >= 1) Then
                    weHaveAtLeastOneDevice = True
                End If
            End If

            'if we do not have a device, then jump into demo mode.
            If Not (weHaveAtLeastOneDevice) Then

JumpIntoDemoMode:

                ' Fully simulate iNet hardware w/ triangle waves feeding a
                ' fake iNet-100 & iNet-200.  This will reset instruNet.
                '
                ' We call Set_iNet_Demo_Mode() instead of SET_DEMO_MODE() to
                ' fix a bug that caused us to not update the # of networks
                ' in the iNet32.dll after switching from 0 real i2x0's, to
                ' one simulated i2x0. gsw 7/1/02
                Call Set_iNet_Demo_Mode(1)

                ' record that simulation is now on
                gSimulatingHardware = True

                ' assume we have 1 controller
                gNumNetworks = 1

            End If

        End If

        ' -------------------------------------------------------------
        ' Determine if an iW+ license has been installed, gsw 7/6/03

        g_iWp_licenseIsInstalledOk = Get_iWp_LicenseState_Safe()

        ' -------------------------------------------------------------
        ' Initialize instruNet Digitize utilities.

        Call Digitize_Initialize()

        ' -------------------------------------------------------------
        ' Initialize instruNet Display utilities.

        Call Initialize_instruNet_Displays(endUser_Class, endUser_form)

    End Sub


    ' -----------------------------------------------------------------------------------  
    ' FUNCTION    NotifyWeCannotLocateThisChannel()
    '
    ' PURPOSE     We show an alert stating that we cannot locate
    '             this channel on the network.
    '
    ' PARAMETERS  digiChAddr - specifies an instruNet channel
    '             via it's {netNum, deviceNum, moduleNum, chanNum}
    '             address.
    '
    ' HISTORY     2/25/97  created - gsw
    ' -----------------------------------------------------------------------------------  
    Function NotifyWeCannotLocateThisChannel(ByRef chAddrDLL As chAddrP) As Short

        Dim message As String
        Dim e, netNum, deviceNum, moduleNum, chanNum As Short

        e = iNetErr_None

        ' get the channel address
        Call GetChannelAddress(chAddrDLL, netNum, deviceNum, moduleNum, chanNum)

        ' create a string that says we cannot find this channel.
        message = "We cannot locate channel {"
        message = message & Str(netNum) & ", "
        message = message & Str(deviceNum) & ", "
        message = message & Str(moduleNum) & ", "
        message = message & Str(chanNum) & "}.  "
        message = message & "Do you think the device is connected?"

        ' show the newly created string in an alert box.
        MsgBox(message)

        NotifyWeCannotLocateThisChannel = e

    End Function


    '-------------------------------------------------------------------------------------
    ' FUNCTION   EndProgram_iNet()
    '
    ' PURPOSE    This ends the instruNet program and shows
    '            an alert saying why we are doing it if
    '            message length is > 0.
    '
    '            Your Window UNLOAD (i.e. Close) method may want
    '            to call this.
    '
    ' HISTORY    2/24/97 created - gsw
    '-----------------------------------------------------------------------------------
    Sub EndProgram_iNet(ByVal message As String)

        If Len(message) <> 0 Then
            message = "We are ending the program because " & message
            MsgBox(message)
        End If

        ' if there is a Timer, make sure it is off
        If Not IsNothing(gDigitizeProcessTimer) Then
            gDigitizeProcessTimer.Dispose()
            gDigitizeProcessTimer = Nothing
        End If

        ' close instruNet driver
        CloseDriverAndReleaseDriverRam()

        ' close instruNet VB Displays
        Close_instruNet_Displays()

        ' (end of program here)
        ' (end of program here)
        ' (end of program here)
        End

    End Sub


    '-------------------------------------------------------------------------------------
    ' FUNCTION   Load_instruNet_Driver_VB()
    '
    ' PURPOSE    Loads the instruNet driver.  This must be done before anything else.
    '
    ' HISTORY    2/24/97 created - gsw
    '-------------------------------------------------------------------------------------
    Public Sub Load_instruNet_Driver_VB()

        Dim driverIsInstalledOK, forceInstruNetReset, e As Short

        ' If true, this forces instruNet to reset (this is not always necessary
        ' since instruNet automatically resets when the driver is first loaded).
        forceInstruNetReset = iNetTrue
        driverIsInstalledOK = iNetTrue

        ' Load instruNet driver and reset the instruNet network.
        e = Load_instruNet_Driver(forceInstruNetReset, driverIsInstalledOK, gNumNetworks)

        ' if no driver installed then stop
        If e <> 0 Or driverIsInstalledOK = iNetFalse Then
            EndProgram_iNet(("Cannot Load instruNet Driver (is file 'iNet32.dll' not installed?)."))
        End If

        giNetDLLIsLoaded = True

    End Sub


    ' -------------------------------------------------------------------------------------
    ' FUNCTION    iNetError_TO_OSErr()
    '
    ' PURPOSE     Converts error of type iNetError to error of type OSError.
    '
    ' HISTORY     2/25/97  created - gsw
    ' -------------------------------------------------------------------------------------
    Function iNetError_TO_OSErr(ByVal iNetErr As Integer) As Integer

        On Error GoTo iNetErrErr

        iNetError_TO_OSErr = (iNetErr - iNetErr_OsBase)

        Exit Function

iNetErrErr:

        MsgBox("Problems in iNetErr function")
        Exit Function

    End Function


    ' -------------------------------------------------------------------------------------
    ' FUNCTION    OSErr_TO_iNetError()
    '
    ' PURPOSE     Converts error of type OSError to to one of two iNet 
    '             Errors {iNetErr_None, iNetErr_General}.
    '
    ' HISTORY     2/25/97  created - gsw
    ' -------------------------------------------------------------------------------------
    Function OSErr_TO_iNetError(ByVal osErr As Integer) As Integer

        If CBool(ErrorToString()) Then GoTo OSErr_TO_iNetErrorErr

        ' Convert OS error to one of two iNet Errors {iNetErr_None, iNetErr_General}
        OSErr_TO_iNetError = IIf(osErr = 0, iNetErr_None, iNetErr_General)

        Exit Function

OSErr_TO_iNetErrorErr:

        MsgBox("Error in OSErr_TO_iNetError function")
        OSErr_TO_iNetError = iNetErr_General

    End Function


    ' -------------------------------------------------------------------------------------
    ' FUNCTION    ion_Wiring_DEF()
    '
    ' PURPOSE     Returns default wiring type.
    '
    ' HISTORY     2/25/97  created - gsw
    ' -------------------------------------------------------------------------------------
    Function ion_Wiring_DEF(ByVal sensorType As Short) As Short

        On Error GoTo sensorTypeErr

        'Need to turn Get_Default_Wiring_For_This_Sensor into a function of ddl to work
        'ion_Wiring_DEF = Get_Default_Wiring_For_This_Sensor (sensorType)

        MsgBox("ion_Wiring_DEF() needs to be coded")

        Exit Function

sensorTypeErr:

        MsgBox("Error in function sensorType")
        Exit Function

    End Function

End Module