Option Strict Off
Option Explicit On 
Imports System.Runtime.InteropServices

Module Macro_Utilities

    ' ===========================================================================
    ' ===========================================================================
    ' FILE: INET_Macros.vb         
    '
    ' ----------------------- D O C U M E N T A T I O N ----------------------
    '
    ' This iNet Interface file contains macros used by the End User
    ' to control the iNet driver.
    '
    ' ------------------------  H I S T O R Y --------------------------------
    ' HISTORY  Copyright c 1994-96 GWI.  All rights reserved.
    ' 4/16/94  created - gsw
    ' 2/8/96   converted to Visual Basic - sjm
    ' 8/17/98  updated - gsw
    ' 6/6/02   updated to be compatible with Visual Studio (Visual Basic) .Net v7
    '
    ' ===========================================================================
    ' ===========================================================================

    ' ------------------------------------------------------------------------
    ' Returns TRUE (1) if the controller is digitizing; FALSE (0) otherwise.
    '
    Function CONTROLLER_IS_DIGITIZING(ByVal netNum As Byte) As Short

        Dim digitizeOnOff As Short

        digitizeOnOff = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, _
            controller_ChanNum_C12_Digitizer, sgt_Timing, fldNum_Timing_digitizeOnOff, _
            intention_getValue, 0)

        If digitizeOnOff = fldNum_Timing_digitizeOnOff Then
            CONTROLLER_IS_DIGITIZING = iNetTrue
        Else
            CONTROLLER_IS_DIGITIZING = iNetFalse
        End If

    End Function

    ' ------------------------------------------------------------------------
    ' Disables channel digitizing for all channels in this network.  This will make
    ' sure that the "Display" field in the DISPLAY SettingGroups is OFF, and that
    ' the "Digitize" field is OFF in the FILE, USERBUFFER, and DRIVERBUFFER
    ' SettingGroups. Returns iNet Error code. gsw 3/16/96
    '
    Function DISABLE_ALL_CHANNEL_DIGITIZING(ByVal netNum As Byte) As Short

        DISABLE_ALL_CHANNEL_DIGITIZING = iNet_int32(netNum, 1, 1, 1, 1, 1, _
            intention_DisableAllChannelDigitizing, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns TRUE (1) if the channel digitization is turned off;
    ' FALSE (0) otherwise.
    '
    Function DISABLE_CHANNEL_DIGITIZATION(ByVal netNum As Byte, ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte) As Short

        Dim iNetErrorCode As Short
        iNetErrorCode = iNet_int32(netNum, deviceNum, moduleNum, chanNum, sgt_Display, _
            fldNum_Display_dispOnOff, intention_setValue, ion_gOnOffPopup_Off)
        DISABLE_CHANNEL_DIGITIZATION = iNetErrorCode

    End Function

    ' ------------------------------------------------------------------------
    ' Dispose of all iNet network state.  This will cause the the next
    ' call to the iNet driver to implement the POWER ON boot process.
    ' Returns iNet error code.
    '
    Function DISPOSE_OF_ALL_NETWORK_STATE() As Short

        DISPOSE_OF_ALL_NETWORK_STATE = iNet_int32(0, 0, 1, 1, 1, 1, _
            intention_disposeOfNetworkState, 0)

    End Function


    ' ------------------------------------------------------------------------
    ' Tells Driver to NOT show an Error message dialog box whenever an error occurs.
    ' Returns TRUE (1) if Driver WAS (before this call) set up to show an alert
    ' upon error; FALSE (0) otherwise
    ' Returns TRUE (1) if Driver is set up to show an alert upon error;
    ' FALSE (0) otherwise
    '
    Function DONT_SHOW_MESSAGE_DIALOG_UPON_ERROR() As Short

        DONT_SHOW_MESSAGE_DIALOG_UPON_ERROR = iNet_int32(0, 0, 0, 0, 0, 0, _
            intention_NoShowAlertOnError, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Sets up digitization, establishes sample rate, sets up BUFFERS, yet does not
    ' actually do the digitization. gsw 8/16/98
    '
    Sub SET_UP_DIGITIZATION()

        Call iNet_int32(0, 0, 0, 0, 0, 0, intention_SetUpDigitization, 0)

    End Sub

    ' ------------------------------------------------------------------------
    ' Sets Demo mode to one of:
    '
    '   0:  iNet_NoDemo - real mode (no demo)
    '   1:  iNet_FullSimulation - we fully simulate iNet hardware w/ triangle
    '       waves feeding a fake iNet-100 & iNet-200.  This will reset instruNet.
    '   2:  iNet_CorruptData - we read real instruNet hardware, yet we alter the
    '       data randomly, to make it unuseable.

    ' End Users should call Set_iNet_Demo_Mode() instead of SET_DEMO_MODE()
    ' End Users should call Set_iNet_Demo_Mode() instead of SET_DEMO_MODE()
    Sub SET_DEMO_MODE(ByVal demoMode As Short)

        Call iNet_int32(0, 0, 0, 0, 0, 0, intention_SetDemoMode, demoMode)

    End Sub


    ' ------------------------------------------------------------------------
    ' Returns TRUE (1) if the iNet Driver is installed; FALSE (0) otherwise.
    '
    Function DRIVER_IS_INSTALLED() As Short

        Dim num As Short
        Dim test As Short

        num = GET_NUMBER_OF_NETWORKS()

        test = Get_iNet_Error()

        If test = iNetErr_None Then
            DRIVER_IS_INSTALLED = iNetTrue
        Else
            DRIVER_IS_INSTALLED = iNetFalse
        End If

    End Function

    ' ------------------------------------------------------------------------
    ' ENABLE_CHANNEL_DIGITIZATION(netNum,deviceNum,moduleNum,chanNum)
    ' Returns TRUE (1) if the channel digitization is turned on;
    ' FALSE (0) otherwise.
    '
    Function ENABLE_CHANNEL_DIGITIZATION(ByVal netNum As Byte, ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte) As Short

        Dim iNetErrorCode As Short
        ENABLE_CHANNEL_DIGITIZATION = iNet_int32(netNum, deviceNum, moduleNum, _
            chanNum, sgt_Display, fldNum_Display_dispOnOff, intention_setValue, _
            ion_gOnOffPopup_On)
        iNetErrorCode = ENABLE_CHANNEL_DIGITIZATION

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the direction of the channel: 0 is input (e.g. a/d), and 1 is output (e.g. d/a).
    ' gsw 5/1/96
    '
    Function GET_CHANNEL_DIRECTION(ByVal netNum As Byte, ByVal deviceNum As Byte, _
        ByVal moduleNum As Byte, ByVal chanNum As Byte) As Short

        GET_CHANNEL_DIRECTION = iNet_int32(netNum, deviceNum, moduleNum, chanNum, _
            1, 1, intention_GetChannelDirection, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the realtime value of the channel, into the VB_string string 
    ' Returns iNet error code.
    '
    Function GET_CHANNEL_VALUE_VbString(ByVal netNum As Byte, ByVal deviceNum As Byte, _
        ByVal moduleNum As Byte, ByVal chanNum As Byte, _
        ByRef VB_string As String) As Short

        GET_CHANNEL_VALUE_VbString = iNet_VB_cString(netNum, deviceNum, moduleNum, _
            chanNum, sgt_General, fldNum_General_valueEU, intention_getValue, _
            VB_string)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the realtime value of the channel, in floating point format (double).
    '
    Function GET_CHANNEL_VALUE_FLT32(ByVal netNum As Byte, ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte) As Double

        GET_CHANNEL_VALUE_FLT32 = iNet_flt32(netNum, deviceNum, moduleNum, chanNum, _
            sgt_General, fldNum_General_valueEU, intention_getValue, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the realtime value of the channel, in +/- 2billion 32bit integer format.
    '
    Function GET_CHANNEL_VALUE_INT32(ByVal netNum As Byte, ByVal deviceNum As Byte, _
        ByVal moduleNum As Byte, ByVal chanNum As Byte) As Integer

        GET_CHANNEL_VALUE_INT32 = iNet_int32(netNum, deviceNum, moduleNum, chanNum, _
            sgt_General, fldNum_General_valueEU, intention_getValue, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the Continous Mode (On (1), Off (2) or Off With Skip (3)).
    '
    Function GET_SCAN_MODE(ByVal netNum As Byte) As Short

        GET_SCAN_MODE = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, _
            controller_ChanNum_C12_Digitizer, sgt_Timing, fldNum_Timing_scanMode, _
            intention_getValue, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the native storage type for the field (i.e. iNetDataType enum in
    ' header file; e.g. iNetDT_INT16 (0), iNetDT_INT32 (3),
    ' iNetDT_UINT32 (4), iNetDT_FLT32 (5)).
    '
    Function GET_NATIVE_STORAGE_TYPE(ByVal netNum As Byte, ByVal deviceNum As Byte, _
        ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal settingGroupNum As Short, _
        ByVal fieldNum As Short) As Short

        GET_NATIVE_STORAGE_TYPE = iNet_int32(netNum, deviceNum, moduleNum, chanNum, _
            settingGroupNum, fieldNum, intention_nativeStorageType, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the network data transfer rate, in bits per second.
    '
    Function GET_NETWORK_BPS(ByVal netNum As Byte) As Integer

        GET_NETWORK_BPS = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, _
            controller_ChanNum_C12_Digitizer, sgt_Timing, fldNum_Timing_network_bps, _
            intention_getValue, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the # of Channels in the specified {Network, Device, Module}.
    '
    Function GET_NUMBER_OF_CHANNELS(ByVal netNum As Byte, ByVal deviceNum As Byte, _
        ByVal moduleNum As Byte) As Short

        GET_NUMBER_OF_CHANNELS = iNet_int32(netNum, deviceNum, moduleNum, 1, 1, 1, _
            intention_GetNumChannels, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the # of Devices in the specified Network.
    '
    Function GET_NUMBER_OF_DEVICES(ByVal netNum As Byte) As Short

        GET_NUMBER_OF_DEVICES = iNet_int32(netNum, 0, 1, 1, 1, 1, intention_GetNumDevices, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the # of Fields in the specified {Network, Device, Module, Channel, SettingGroup}.
    '
    Function GET_NUMBER_OF_FIELDS(ByVal netNum As Byte, ByVal deviceNum As Byte, _
        ByVal moduleNum As Byte, ByVal chanNum As Byte, ByRef settingGroupNum As Short) As Short

        GET_NUMBER_OF_FIELDS = iNet_int32(netNum, deviceNum, moduleNum, chanNum, _
            settingGroupNum, 1, intention_GetNumFields, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the # of Channels in the specified {Network, Device, Module}.
    '
    Function GET_NUMBER_OF_MODULES(ByVal netNum As Byte, ByVal deviceNum As Byte) As Short

        GET_NUMBER_OF_MODULES = iNet_int32(netNum, deviceNum, 1, 1, 1, 1, _
            intention_GetNumModules, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the # of iNet Networks.
    '
    Function GET_NUMBER_OF_NETWORKS() As Short

        GET_NUMBER_OF_NETWORKS = iNet_int32(0, 0, 1, 1, 1, 1, intention_GetNumNetworks, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the number of items in a popup, if the field user interface is a popup.
    ' (e.g. returns 2 if the popup shows {"ON", "Off"}).
    '
    Function GET_NUMBER_OF_POPUP_ITEMS(ByVal netNum As Byte, ByVal deviceNum As Byte, _
        ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal settingGroupNum As Short, _
        ByVal fieldNum As Short) As Short

        GET_NUMBER_OF_POPUP_ITEMS = iNet_int32(netNum, deviceNum, moduleNum, _
            chanNum, settingGroupNum, fieldNum, intention_Get_NumPopupItems, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the number SCANS.
    '
    Function GET_NUMBER_OF_SCANS(ByVal netNum As Byte) As Integer

        GET_NUMBER_OF_SCANS = iNet_int32(netNum, deviceNum_CONTROLLER, _
            moduleNum_1stModule, controller_ChanNum_C12_Digitizer, sgt_Timing, _
            fldNum_Timing_noOfScans, intention_getValue, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the  of Settings in the specified {Network, Device, Module, Channel}.
    '
    Function GET_NUMBER_OF_SETTINGS(ByVal netNum As Byte, ByVal deviceNum As Byte, _
        ByVal moduleNum As Byte, ByVal chanNum As Byte) As Short

        GET_NUMBER_OF_SETTINGS = iNet_int32(netNum, deviceNum, moduleNum, chanNum, _
            1, 1, intention_GetNumSettings, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the number of POINTS in each SCAN.
    '
    Function GET_POINTS_PER_SCAN(ByVal netNum As Byte) As Long

        GET_POINTS_PER_SCAN = iNet_int32(netNum, deviceNum_CONTROLLER, _
            moduleNum_1stModule, controller_ChanNum_C12_Digitizer, sgt_Timing, _
            fldNum_Timing_ptsPerScan, intention_getValue, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the pointer to an array of VB strings that contain the names
    ' of the Network, Device, Module, Channel, Settings, Field (i.e.
    ' ArrayOfStrings[0] = Network Name, ArrayOfStrings[1] = Device Name,   ArrayOfStrings[2] = Module Name,
    ' ArrayOfStrings[3] = Channel Name, ArrayOfStrings[4] = Settings Name, ArrayOfStrings[5] = Field Name).

    ' THIS IS NOT WORKING, PROBABLY BECAUSE VB (ver >= 7) IS HAVING TROUBLE 
    ' GETTING ACCESS TO STATIC VARIABLE SPACE INSIDE THE iNet32.dll, gsw 6/1/02.

    Function GET_ARRAY_OF_ADDRESS_NAMES_VB(ByVal netNum As Byte, ByVal deviceNum As Byte, _
      ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal settingGroupNum As Short, _
      ByVal fieldNum As Short, ByRef ArrayOfStrings() As String) As Long

        ' get ptr to this array of C Str15's (this is a static array in iNet32.dll memory)
        Dim ArrayOf_cStr_ptr As Integer = iNet_int32(netNum, deviceNum, moduleNum, _
            chanNum, settingGroupNum, fieldNum, intention_Get_AddrNamesStringArray, 0)

        ' set up space to receive 6 VB strings
        ReDim ArrayOfStrings(6)

        Dim strNum As Integer
        For strNum = 0 To 5

            ' convert the memory ptr to a VB IntPtr
            Dim inPtr As System.IntPtr = IntPtr.op_Explicit(ArrayOf_cStr_ptr + (16 * strNum))

            ' THIS IS NOT WORKING, PROBABLY BECAUSE VB (ver >= 7) IS HAVING TROUBLE 
            ' GETTING ACCESS TO STATIC VARIABLE SPACE INSIDE THE iNet32.dll, gsw 6/1/02.

            ' copy the cString string (1byte/char) into the caller's VB_string (2bytes/char)
            ArrayOfStrings(strNum) = Marshal.PtrToStringAnsi(inPtr)

        Next

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the pointer to an array of C Str15 strings that contain the items
    ' in the popup (i.e. iNetStr15 popupItems[]).
    '
    ' No longer supported, gsw 6/1/02

    'OLD: Function GET_PTR_TO_POPUP_ITEMS(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    'OLD:     ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal settingGroupNum As Short, _
    'OLD:     ByVal fieldNum As Short) As Long
    'OLD: 
    'OLD:     GET_PTR_TO_POPUP_ITEMS = iNet_int32(netNum, deviceNum, moduleNum, chanNum, _
    'OLD:         settingGroupNum, fieldNum, intention_Get_PopupStringArray, 0)
    'OLD: 
    'OLD: End Function

    ' ------------------------------------------------------------------------
    ' Returns the sample rate, in samples per second for each channel.
    '
    Function GET_SAMPLE_RATE_IN_HZ(ByVal netNum As Byte) As Double

        GET_SAMPLE_RATE_IN_HZ = iNet_flt32(netNum, deviceNum_CONTROLLER, _
            moduleNum_1stModule, controller_ChanNum_C12_Digitizer, sgt_Timing, _
            fldNum_Timing_sampleRate, intention_getValue, 0)

    End Function

    ' ------------------------------------------------------------------------
    '
    Function GET_SENSOR_TYPE(ByVal netNum As Byte, ByVal deviceNum As Byte, _
        ByVal modNum As Byte, ByVal chanNum As Byte) As Short

        GET_SENSOR_TYPE = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, _
            chanNum, sgt_VinHardware, fldNum_Vin_sensorType, intention_getValue, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the start of data acquisition time in seconds relative to
    ' a specified trigger condition
    '
    Function GET_START_TIME_IN_SEC(ByVal netNum As Byte) As Double

        GET_START_TIME_IN_SEC = iNet_flt32(netNum, deviceNum_CONTROLLER, _
            moduleNum_1stModule, controller_ChanNum_C12_Digitizer, sgt_Trigger, _
            fldNum_Trigger_preTrigSec, intention_getValue, 0)

    End Function


    ' ------------------------------------------------------------------------
    ' Returns TRUE (1) if Driver is set up to show an alert upon error; FALSE (0) otherwise
    '
    Function GET_SHOW_MESSAGE_UPON_ERROR_STATUS() As Short

        GET_SHOW_MESSAGE_UPON_ERROR_STATUS = iNet_int32(0, 0, 0, 0, 0, 0, _
            intention_GetShowAlertOnErrorStatus, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the channel switching mode as Accurate (1) or Fast (2).
    '
    Function GET_SWITCHING(ByVal netNum As Byte) As Short

        GET_SWITCHING = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, _
            controller_ChanNum_C12_Digitizer, sgt_Timing, fldNum_Timing_switching, _
            intention_getValue, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the user interface catagory (i.e. itemType_iNet enum in
    ' header file; e.g. ion_StaticText (3), ion_EditText (4),
    ' ion_UserItem (popup) (5)).
    '
    Function GET_USER_INTERFACE_CATAGORY(ByVal netNum As Byte, ByVal deviceNum As Byte, _
        ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal settingGroupNum As Short, _
        ByVal fieldNum As Short) As Short

        GET_USER_INTERFACE_CATAGORY = iNet_int32(netNum, deviceNum, moduleNum, _
            chanNum, settingGroupNum, fieldNum, intention_Get_UI_Catagory, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the user interface type (i.e. fieldUserInterfaceType enum in
    ' header file; e.g. fuit_editField_flt32, fuit_staticText_int16,
    ' fuit_popup_OnOff).
    '
    Function GET_USER_INTERFACE_TYPE(ByVal netNum As Byte, ByVal deviceNum As Byte, _
        ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal settingGroupNum As Short, _
        ByVal fieldNum As Short) As Short

        GET_USER_INTERFACE_TYPE = iNet_int32(netNum, deviceNum, moduleNum, chanNum, _
            settingGroupNum, fieldNum, intention_getUserInterfaceType, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Lock internal memory so that future calls to iNet() are less likely to move
    ' memory (useful when calling iNet from interrupts). gsw 1/27/97
    '
    Function LOCK_INTERNAL_INET_MEMORY() As Short

        LOCK_INTERNAL_INET_MEMORY = iNet_int32(0, 0, 0, 0, 0, 0, intention_lockInternalMemory, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Please use Open_instruNet_Window() instead of OPEN_iNet_WINDOW()
    '
    ' OLD: Function OPEN_iNet_WINDOW(ByVal specificPage As Short) As Short
    ' OLD: 
    ' OLD:        Dim iNetErrorCode As Short
    ' OLD: 
    ' OLD:        iNetErrorCode = iNet_int32(netNum_DRIVER, deviceNum_CONTROLLER, _
    ' OLD:            moduleNum_1stModule, driver_ChanNum_OpenWindow, sgt_General, _
    ' OLD:            fldNum_General_valueEU, intention_setValue, specificPage)
    ' OLD: 
    ' OLD:        OPEN_iNet_WINDOW = iNetErrorCode
    ' OLD: 
    ' OLD:    End Function

    ' ------------------------------------------------------------------------
    ' This function is used to press a Button within the iNet World window (i.e. the buttons
    ' at the top of the iNet World window).  The iNet World window does not need to
    ' be open in order to do this.  For example, if you press the 'Store' button in NETWORK page,
    ' the state of all fields will be stored to disk, even though the user
    ' does not "see anything".  Below are several 'specificButton' values:
    '
    '   iNetCM_BtnPress_Record_Start    Press 'Start' button in RECORD page
    '   iNetCM_BtnPress_Record_Stop     Press 'Stop' button in RECORD page
    '   iNetCM_BtnPress_Record_Open     Press 'Open' button in RECORD page
    '   iNetCM_BtnPress_Record_Save     Press 'Save' button in RECORD page
    '   iNetCM_BtnPress_Record_Options  Press 'Options' button in RECORD page
    '   iNetCM_BtnPress_Record_Timing   Press 'Timing' button in RECORD page
    '   iNetCM_BtnPress_Record_Trigger  Press 'Trigger' button in RECORD page
    '   iNetCM_BtnPress_Record_Probe    Press 'Probe' button in RECORD page
    '   iNetCM_BtnPress_Network_Restore Press 'Restore' button in NETWORK page
    '   iNetCM_BtnPress_Network_Store   Press 'Store' button in NETWORK page
    '   iNetCM_BtnPress_Network_Open    Press 'Open' button in NETWORK page
    '   iNetCM_BtnPress_Network_Save    Press 'Start' button in NETWORK page
    '   iNetCM_BtnPress_Network_Clear   Press 'Clear' button in NETWORK page
    '   iNetCM_BtnPress_Network_Reset   Press 'Reset' button in NETWORK page
    '   iNetCM_BtnPress_Test_Search     Press 'Search' button in TEST page
    '   iNetCM_BtnPress_Test_Test       Press 'Test' button in TEST page
    '   iNetCM_BtnPress_Test_BigTest    Press 'Big Test' button in TEST page
    '   iNetCM_BtnPress_Test_Open       Press 'Open' button in TEST page
    '   iNetCM_BtnPress_Test_SaveAs     Press 'Save As' button in TEST page
    '   iNetCM_BtnPress_Test_Clear      Press 'Clear' button in TEST page
    '
    Function PRESS_iNet_BUTTON(ByVal specificButton As Short) As Short

        Dim iNetErrorCode As Short

        iNetErrorCode = iNet_int32(netNum_DRIVER, deviceNum_CONTROLLER, _
            moduleNum_1stModule, driver_ChanNum_PushButton, sgt_General, _
            fldNum_General_valueEU, intention_setValue, specificButton)

        PRESS_iNet_BUTTON = iNetErrorCode

    End Function

    ' ------------------------------------------------------------------------
    ' Returns the  of seconds since reset.  This is derived from a 62bit
    ' 4MHZ counter on the iNet Controller board
    '
    Function SECONDS_SINCE_RESET(ByVal netNum As Byte) As Double

        Dim vbStr As String

        ' get secs since reset from instruNet
        Call iNet_VB_cString(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, _
            controller_ChanNum_C11_Time, sgt_General, fldNum_General_valueEU, _
            intention_getValue, vbStr)

        SECONDS_SINCE_RESET = Val(vbStr)

        ' SECONDS_SINCE_RESET = iNet_flt32(netNum, deviceNum_CONTROLLER, _
        ''    moduleNum_1stModule, controller_ChanNum_C11_Time, sgt_General, _
        ''    fldNum_General_valueEU, intention_getValue, 0&)

    End Function

    ' ------------------------------------------------------------------------
    ' Services buffers during the digitization process.  This must be done continuously
    ' while digitizing.  Returns TRUE (1) if the next data pulled out
    ' out of the buffers is that last pull; FALSE (0) otherwise. A_PARAM()
    ' is set to TRUE (1) if we pulled to the end of a Scan; FALSE (0)
    ' otherwise.
    '
    Function SERVICE_ALL_DIGITIZE_BUFFERS() As Short

        SERVICE_ALL_DIGITIZE_BUFFERS = iNet_int32(0, 0, 1, 1, 1, 1, intention_serviceAllDigitizeBuffers, 0)

    End Function

    ' ------------------------------------------------------------------------
    ' Sets the realtime value of the channel, with a pointer to the caller's 
    ' VB_string string. Returns iNet Error code.
    '
    Function SET_CHANNEL_VALUE_VbString(ByVal netNum As Byte, ByVal deviceNum As Byte, _
        ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal VB_string As String) As Short

        SET_CHANNEL_VALUE_VbString = iNet_VB_cString(netNum, deviceNum, moduleNum, _
            chanNum, sgt_General, fldNum_General_valueEU, intention_setValue, _
            VB_string)

    End Function

    ' ------------------------------------------------------------------------
    ' Sets the value of the channel, with a floating point number.
    ' Returns iNet Error code.
    '
    Function SET_CHANNEL_VALUE_FLT32(ByVal netNum As Byte, ByVal deviceNum As Byte, _
        ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal newValue_flt32 As Double) As Short

        Dim iNetErrorCode As Short

        iNetErrorCode = iNet_flt32(netNum, deviceNum, moduleNum, chanNum, sgt_General, _
            fldNum_General_valueEU, intention_setValue, newValue_flt32)

        SET_CHANNEL_VALUE_FLT32 = iNetErrorCode

    End Function

    ' ------------------------------------------------------------------------
    ' Sets the realtime value of the channel, with a +/- 2billion integer format.
    ' Returns iNet Error code.
    '
    Function SET_CHANNEL_VALUE_INT32(ByVal netNum As Byte, ByVal deviceNum As Byte, _
        ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal newValue_int32 As Integer) As Short

        Dim iNetErrorCode As Short

        iNetErrorCode = iNet_int32(netNum, deviceNum, moduleNum, chanNum, _
            sgt_General, fldNum_General_valueEU, intention_setValue, newValue_int32)

        SET_CHANNEL_VALUE_INT32 = iNetErrorCode

    End Function

    ' ------------------------------------------------------------------------
    ' Sets the CONTINOUS scans mode (ON means strip chart with continuous
    ' scans, OFF WITH SKIP is Oscilloscope mode with non-continuous traces), e.g.
    '
    '   SET_SCAN_MODE(netNum, ion_gScanModePopup_StripChart (1))
    '   SET_SCAN_MODE(netNum, ion_gScanModePopup_OscilloQueued (2))
    '   SET_SCAN_MODE(netNum, ion_gScanModePopup_Oscilloscope (3))
    '
    ' Returns iNet Error code
    '
    Function SET_SCAN_MODE(ByVal netNum As Byte, ByVal continuousMode As Integer) As Short

        Dim iNetErrorCode As Short

        iNetErrorCode = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, _
            controller_ChanNum_C12_Digitizer, sgt_Timing, fldNum_Timing_scanMode, _
            intention_setValue, continuousMode)

        SET_SCAN_MODE = iNetErrorCode

    End Function

    ' ------------------------------------------------------------------------
    ' Sets the network data transfer rate, in bits per second.
    '
    Function SET_NETWORK_BPS(ByVal netNum As Byte, ByVal new_Network_BPS As Integer) As Short

        Dim iNetErrorCode As Short

        iNetErrorCode = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, _
            controller_ChanNum_C12_Digitizer, sgt_Timing, fldNum_Timing_network_bps, _
            intention_setValue, new_Network_BPS)

        SET_NETWORK_BPS = iNetErrorCode

    End Function

    ' ------------------------------------------------------------------------
    ' Sets the network state with the passed Handle. The Caller must dispose after
    ' calling this routine.  The handle must have originated from a
    ' GET_HANDLE_TO_NETWORK_STATE() call that was done previously.  Returns iNet Error code.

    ' Use Set_instruNet_Network_State_VB()/Get_instruNet_Network_State_VB() instead of this.

    ' OLD: Function SET_NETWORK_STATE_WITH_HANDLE(ByVal callerHandleToNetworkState As Long) As Short
    ' OLD:  SET_NETWORK_STATE_WITH_HANDLE = iNet_uint32(0, 0, 1, 1, 1, 1, intention_setNetworkStateWithHandle, callerHandleToNetworkState)
    ' OLD:  End Function

    ' OLD: Returns a Handle to the Network State (in uint32).
    ' OLD: The Caller must dispose of it after using it
    ' OLD: (e.g. DisposHandle() on Macintosh, GlobalFree() on Windows).

    ' OLD: Function GET_HANDLE_TO_NETWORK_STATE() As Long
    ' OLD:  GET_HANDLE_TO_NETWORK_STATE = iNet_uint32(0, 0, 1, 1, 1, 1, intention_getHandleToNetworkState, 0)
    ' OLD: End Function

    ' ------------------------------------------------------------------------
    ' Sets the number SCANS.  Returns iNet Error code.
    '
    Function SET_NUMBER_OF_SCANS(ByVal netNum As Byte, ByVal numberOfScans As Long) As Short

        Dim iNetErrorCode As Short

        iNetErrorCode = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, _
            controller_ChanNum_C12_Digitizer, sgt_Timing, fldNum_Timing_noOfScans, _
            intention_setValue, numberOfScans)

        SET_NUMBER_OF_SCANS = iNetErrorCode

    End Function

    ' ------------------------------------------------------------------------
    ' Set the Overflow Alert Field, e.g.
    '
    '   SET_OVERFLOW_ALERT(ion_gOnOffPopup_On)   Show an alert when buffer overflow occurs.
    '   SET_OVERFLOW_ALERT(ion_gOnOffPopup_Off)   Don't show an alert when buffer overflow occurs.
    '
    Function SET_OVERFLOW_ALERT(ByVal overflowAlertOption As Integer) As Short

        Dim iNetErrorCode As Short

        iNetErrorCode = iNet_int32(netNum_DRIVER, deviceNum_CONTROLLER, _
            moduleNum_1stModule, driver_ChanNum_RecordOptions, sgt_RecordOptions, _
            fldNum_RecordOptions_overflowAlert, intention_setValue, overflowAlertOption)

        SET_OVERFLOW_ALERT = iNetErrorCode

    End Function

    ' ------------------------------------------------------------------------
    ' Sets the number of POINTS in each SCAN.
    '
    Function SET_POINTS_PER_SCAN(ByVal netNum As Byte, ByVal numberOfPointsPerScan As Integer) As Short

        Dim iNetErrorCode As Short

        iNetErrorCode = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, _
            controller_ChanNum_C12_Digitizer, sgt_Timing, fldNum_Timing_ptsPerScan, _
            intention_setValue, numberOfPointsPerScan)

        SET_POINTS_PER_SCAN = iNetErrorCode

    End Function

    ' ------------------------------------------------------------------------
    ' Sets the sample rate, in samples per second for each channel.
    '
    Function SET_SAMPLE_RATE_IN_HZ(ByVal netNum As Byte, ByVal newSampleRate_Hz As Single) As Short

        Dim iNetErrorCode As Short

        iNetErrorCode = iNet_flt32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, _
            controller_ChanNum_C12_Digitizer, sgt_Timing, fldNum_Timing_sampleRate, _
            intention_setValue, newSampleRate_Hz)

        SET_SAMPLE_RATE_IN_HZ = iNetErrorCode

    End Function

    ' ------------------------------------------------------------------------
    ' Save data to RAM, Disk or nowhere, e.g.
    '
    '   SET_SAVE_DATA(ion_gSaveDataPopup_Off)   Don't save data
    '   SET_SAVE_DATA(ion_gSaveDataPopup_ToRamBuffer) Save Data to RAM
    '   SET_SAVE_DATA(ion_gSaveDataPopup_ToDisk)  Save Data to DISK
    '   SET_SAVE_DATA(ion_gSaveDataPopup_UserControl) Let user determine where to save
    '
    Function SET_SAVE_DATA(ByVal saveDataOption As Integer) As Short

        Dim iNetErrorCode As Short

        iNetErrorCode = iNet_int32(netNum_DRIVER, deviceNum_CONTROLLER, _
            moduleNum_1stModule, driver_ChanNum_RecordOptions, sgt_RecordOptions, _
            fldNum_RecordOptions_saveData, intention_setValue, saveDataOption)

        SET_SAVE_DATA = iNetErrorCode

    End Function

    ' ------------------------------------------------------------------------
    ' Sets the channel switching mode to Accurate (1) or Fast (2).
    '
    Function SET_SWITCHING(ByVal netNum As Byte, ByVal switchingMode As Integer) As Short

        Dim iNetErrorCode As Short

        iNetErrorCode = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, _
            controller_ChanNum_C12_Digitizer, sgt_Timing, fldNum_Timing_switching, _
            intention_setValue, switchingMode)

        SET_SWITCHING = iNetErrorCode

    End Function

    ' ------------------------------------------------------------------------
    ' Sets the Trigger mode to OFF, NORMAL or AUTOMATIC, e.g.
    '
    '   SET_TRIGGER_MODE(netNum, ion_TriggerMode_Off)
    '   SET_TRIGGER_MODE(netNum, ion_TriggerMode_Auto)
    '   SET_TRIGGER_MODE(netNum, ion_TriggerMode_Norm)
    '
    Function SET_TRIGGER_MODE(ByVal netNum As Byte, ByVal triggerMode As Integer) As Short

        Dim iNetErrorCode As Short

        iNetErrorCode = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, _
            controller_ChanNum_C12_Digitizer, sgt_Trigger, fldNum_Trigger_triggerModePop, _
            intention_setValue, triggerMode)

        SET_TRIGGER_MODE = iNetErrorCode

    End Function

    ' ------------------------------------------------------------------------
    ' Set the Trigger slope to rising or falling, e.g.
    '
    '   SET_TRIGGER_SLOPE(netNum, ion_Rising)
    '   SET_TRIGGER_SLOPE(netNum, ion_Falling)
    '
    Function SET_TRIGGER_SLOPE(ByVal netNum As Byte, ByVal triggerSlope As Integer) As Short

        Dim iNetErrorCode As Short

        iNetErrorCode = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, _
            controller_ChanNum_C12_Digitizer, sgt_Trigger, fldNum_Trigger_slopeRisFalPop, _
            intention_setValue, triggerSlope)

        SET_TRIGGER_SLOPE = iNetErrorCode

    End Function

    ' ------------------------------------------------------------------------
    ' Sets the network address of the trigger source, e.g.
    '
    '   SET_TRIGGER_SOURCE(netNum, fldNum_Trigger_srcNet, netNum)
    '   SET_TRIGGER_SOURCE(netNum, fldNum_Trigger_srcDevice, deviceNum)
    '   SET_TRIGGER_SOURCE(netNum, fldNum_Trigger_srcModule, moduleNum)
    '   SET_TRIGGER_SOURCE(netNum, fldNum_Trigger_srcChannel, chanNum)
    '
    Function SET_TRIGGER_SOURCE(ByVal netNum As Byte, ByVal fldNum_Src As Short, _
        ByVal srcNum As Integer) As Short

        Dim iNetErrorCode As Short

        iNetErrorCode = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, _
            controller_ChanNum_C12_Digitizer, sgt_Trigger, fldNum_Src, intention_setValue, _
            srcNum)

        SET_TRIGGER_SOURCE = iNetErrorCode

    End Function

    ' ------------------------------------------------------------------------
    ' Sets the Trigger threshold (engineering units) value.
    '
    Function SET_TRIGGER_THRESHOLD(ByVal netNum As Byte, ByVal triggerThresholdEu As Single) As Short

        Dim iNetErrorCode As Short

        iNetErrorCode = iNet_flt32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, _
            controller_ChanNum_C12_Digitizer, sgt_Trigger, fldNum_Trigger_thresholdEu, _
            intention_setValue, triggerThresholdEu)

        SET_TRIGGER_THRESHOLD = iNetErrorCode

    End Function

    ' ------------------------------------------------------------------------
    ' Tells Driver to show a Error message dialog box whenever an error occurs.
    ' Returns TRUE (1) if Driver WAS (before this call) set up to show an alert
    ' upon error; FALSE (0) otherwise
    '
    Function SHOW_MESSAGE_DIALOG_UPON_ERROR() As Short

        Dim iNetErrorCode As Short
        iNetErrorCode = iNet_int32(0, 0, 0, 0, 0, 0, intention_ShowAlertOnError, 0)
        SHOW_MESSAGE_DIALOG_UPON_ERROR = iNetErrorCode

    End Function


    ' ------------------------------------------------------------------------
    ' Returns 1 if CHANNEL = {netNum, deviceNum, moduleNum, channelNum} exists;
    ' FALSE (0) otherwise.
    '
    Function THIS_CHANNEL_EXISTS(ByVal netNum As Byte, ByVal deviceNum As Byte, _
        ByVal moduleNum As Byte, ByVal chanNum As Byte) As Short

        Dim thisExists As Short

        thisExists = iNet_int32(netNum, deviceNum, moduleNum, chanNum, 1, 1, _
            intention_ThisExists_Channel, 0)

        If thisExists = 0 Then
            THIS_CHANNEL_EXISTS = iNetFalse
        Else
            THIS_CHANNEL_EXISTS = iNetTrue
        End If

    End Function


    ' ------------------------------------------------------------------------
    ' Returns 1 if DEVICE = {netNum, deviceNum, moduleNum, channelNum} exists;
    ' FALSE (0) otherwise.
    '
    Function THIS_DEVICE_EXISTS(ByVal netNum As Byte, ByVal deviceNum As Byte) As Short

        Dim thisExists As Short
        thisExists = iNet_int32(netNum, deviceNum, 1, 1, 1, 1, intention_ThisExists_Device, 0)

        If thisExists = 0 Then
            THIS_DEVICE_EXISTS = iNetFalse
        Else
            THIS_DEVICE_EXISTS = iNetTrue
        End If

    End Function


    ' ------------------------------------------------------------------------
    ' Returns 1 if FIELD = {netNum, deviceNum, moduleNum, channelNum, settingNum,
    ' fieldNum} exists; FALSE (0) otherwise.
    '
    Function THIS_FIELD_EXISTS(ByVal netNum As Byte, ByVal deviceNum As Byte, _
        ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal settingGroupNum As Short, _
        ByVal fieldNum As Short) As Short

        Dim thisExists As Short
        thisExists = iNet_int32(netNum, deviceNum, moduleNum, chanNum, _
            settingGroupNum, fieldNum, intention_ThisExists_Field, 0)

        If thisExists = 0 Then
            THIS_FIELD_EXISTS = iNetFalse
        Else
            THIS_FIELD_EXISTS = iNetTrue
        End If

    End Function

    ' ------------------------------------------------------------------------
    ' Returns 1 if MODULE = {netNum, deviceNum, moduleNum} exists;
    ' FALSE (0) otherwise.
    '
    Function THIS_MODULE_EXISTS(ByVal netNum As Byte, ByVal deviceNum As Byte, _
        ByVal moduleNum As Byte) As Short

        Dim thisExists As Short
        thisExists = iNet_int32(netNum, deviceNum, moduleNum, 1, 1, 1, _
            intention_ThisExists_Module, 0)

        If thisExists = 0 Then
            THIS_MODULE_EXISTS = iNetFalse
        Else
            THIS_MODULE_EXISTS = iNetTrue
        End If

    End Function

    ' ------------------------------------------------------------------------
    ' Returns 1 if NETWORK = {netNum} exists; FALSE (0) otherwise.
    '
    Function THIS_NETWORK_EXISTS(ByVal netNum As Byte) As Short

        Dim thisExists As Short
        thisExists = iNet_int32(netNum, 0, 1, 1, 1, 1, intention_ThisExists_Network, 0)

        If thisExists = 0 Then
            THIS_NETWORK_EXISTS = iNetFalse
        Else
            THIS_NETWORK_EXISTS = iNetTrue
        End If

    End Function

    ' ------------------------------------------------------------------------
    ' Returns 1 if SETTING = {netNum, deviceNum, moduleNum, channelNum, settingNum}
    ' exists; FALSE (0) otherwise.
    '
    Function THIS_SETTING_EXISTS(ByVal netNum As Byte, ByVal deviceNum As Byte, _
        ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal settingGroupNum As Short) As Short

        Dim thisExists As Short
        thisExists = iNet_int32(netNum, deviceNum, moduleNum, chanNum, _
            settingGroupNum, 1, intention_ThisExists_Setting, 0)

        If thisExists = 0 Then
            THIS_SETTING_EXISTS = iNetFalse
        Else
            THIS_SETTING_EXISTS = iNetTrue
        End If

    End Function

    ' ------------------------------------------------------------------------
    ' Unlocks internal memory when the processor is outside of instruNet driver.
    '
    Function UNLOCK_INTERNAL_INET_MEMORY() As Short

        UNLOCK_INTERNAL_INET_MEMORY = iNet_int32(0, 0, 0, 0, 0, 0, _
            intention_unlockInternalMemory, 0)

    End Function

End Module