Attribute VB_Name = "Interface_Utilities"
' **************************************************************************
' INET_Interface.Bas
'
'------------------------- A B S T R A C T ----------------------------
'
' This file contains interface code for the iNet DLL. For these calls
' to work you must have the files iNet32.dll and Inet95.vxd in your
' Windows95/System folder
'
'------------------------ H I S T O R Y ------------------------------
' HISTORY         Copyright c 1994-96 GWI.  All rights reserved.
' 5/18/94         created - gsw
' 2/11/97         created VB prototypes - sjm
' *******************************************************************************

' -------------------------------------------------------------------------
' Global Variables

Public gNumNetworks As Integer          ' loaded w/ # of instruNet networks

Global Const iSave = 1                  ' save data
Global Const iLoad = 2                  ' load data

Global giNetDLLIsLoaded As Integer      ' true if instruNet Driver (iNet32.dll) is loaded ok
                                    
' -------------------------------------------------------------------------
' Simulation Policy
                                        ' if true, we jump into demo mode if
                                        ' no instruNet devices is attached to the
                                        ' computer.  Demo mode simulates an iNet-200
                                        ' and iNet-100 device.
Global Const wJumpIntoDemoModeIfNoDeviceExists = 1

                                        ' don't jump into demo mode if an
                                        ' iNet devices is not attached.
Global Const wDoNotJumpIntoDemoModeIfNoDeviceExists = 0

Global Const wJumpIntoRightNow = 2      ' jump into demo mode right now

'  ===================== CLASS: BASIC READ/WRITE CHANNEL ========================
'  ===================== CLASS: BASIC READ/WRITE CHANNEL ========================
'  ===================== CLASS: BASIC READ/WRITE CHANNEL ========================

'-------------------------------------------------------------------------
' FUNCTIONS
'
'        GetChannelValue_iNet()          Read value of instruNet channel.
'        SetChannelValue_iNet()          Set value of instruNet channel.
'        GetFieldValue_iNet()            Read value of instruNet field.
'        SetFieldValue_iNet()            Set value of instruNet field.
'
'        SetField_int32()                Set instruNet field with 32bit signed integer number
'        SetField_uint32()               Set instruNet field with 32bit unsigned integer number
'        SetField_flt32()                Set instruNet field with 32bit floating point number
'        SetField_cStr()                 Set instruNet field with C string (terminated with 0x00)
'
'        GetField_int32()                Read instruNet field into a 32bit signed integer variable
'        GetField_uint32()               Read instruNet field into a 32bit unsigned integer variable
'        GetField_flt32()                Read instruNet field into a 32bit floating point variable
'        GetField_cStr()                 Read instruNet field into a C string variable (terminated with 0x00)
'
'        iNet_int32()                    Reads or writes to a field via a 32bit signed integer variable
'        iNet_uint32()                   Reads or writes to a field via a 32bit unsigned integer variable
'        iNet_flt32()                    Reads or writes to a field via a 32bit floating point variable
'        iNet_cString()                  Reads or writes to a field via a C string variable
'        iNet_pString()                  Reads or writes to a field via a Pascal string variable
'        iNet_DLL()                      Reads or writes to a field via any type of variable
'
' PARAMETERS
'
'    netNum     NETWORK number = {0...numNetworks}, 0 is Driver,
'    deviceNum  DEVICE number {0...numDevices}, 0 = "Controller",
'    moduleNum  MODULE number within a hardware DEVICE {1...32}.
'    chanNum    Hardware CHANNEL number {1...32}.
'
' HISTORY       1/27/97 created - gsw
'-------------------------------------------------------------------------

Declare Function GetChannelValue_iNet Lib "iNet32.DLL" (ByVal netNum As Byte, _
    ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte, _
    ByRef errP As Integer) As Single

Declare Function SetChannelValue_iNet Lib "iNet32.DLL" (ByVal netNum As Byte, _
    ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte, _
    ByVal flt32_newValue As Single) As Integer

Declare Function GetFieldValue_iNet Lib "iNet32.DLL" (ByVal netNum As Byte, _
    ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte, _
    ByVal settingGroupNumOrType As Integer, ByVal fieldNum As Integer, _
    ByRef errP As Integer) As Single

Declare Function SetFieldValue_iNet Lib "iNet32.DLL" (ByVal netNum As Byte, _
    ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte, _
    ByVal settingGroupNumOrType As Integer, ByVal fieldNum As Integer, _
    ByVal flt32_newValue As Single) As Integer

Declare Function SetField_int32 Lib "iNet32.DLL" (chAddrP As chAddr, _
    ByVal settingGroupNumOrType As Integer, ByVal fieldNum As Integer, _
    ByVal int32Value As Long) As Integer
    
Declare Function SetField_uint32 Lib "iNet32.DLL" (chAddrP As chAddr, _
    ByVal settingGroupNumOrType As Integer, ByVal fieldNum As Integer, _
    ByVal uint32Value As Long) As Integer
    
Declare Function SetField_flt32 Lib "iNet32.DLL" (chAddrP As chAddr, _
    ByVal settingGroupNumOrType As Integer, ByVal fieldNum As Integer, _
    ByVal flt32Value As Single) As Integer
    
Declare Function SetField_cStr Lib "iNet32.DLL" (chAddrP As chAddr, _
    ByVal settingGroupNumOrType As Integer, ByVal fieldNum As Integer, _
    ByVal cString As String) As Integer

Declare Function GetField_int32 Lib "iNet32.DLL" (chAddrP As chAddr, _
    ByVal settingGroupNumOrType As Integer, ByVal fieldNum As Integer, _
    iNetError As Integer) As Long
    
Declare Function GetField_uint32 Lib "iNet32.DLL" (chAddrP As chAddr, _
    ByVal settingGroupNumOrType As Integer, ByVal fieldNum As Integer, _
    iNetError As Integer) As Long
    
Declare Function GetField_flt32 Lib "iNet32.DLL" (chAddrP As chAddr, _
    ByVal settingGroupNumOrType As Integer, ByVal fieldNum As Integer, _
    iNetError As Integer) As Single

' use iNet_VB_cString() and iNet_VB_GetFieldcStr() instead of GetField_cStr()
Declare Function GetField_cStr Lib "iNet32.DLL" (chAddrP As chAddr, _
    ByVal settingGroupNumOrType As Integer, ByVal fieldNum As Integer, _
    ByVal cString As String) As Integer

'-------------------------------------------------------------------------
' FUNCTION      LoadChannelAddress()
'
'                   Loads a channel's address into a chAddr structure (i.e. contains
'                   {netNum, deviceNum, moduleNum, chanNum}.  Returns TRUE (1)
'                   if this channel exists; FALSE (0) otherwise.
'
'               GetChannelAddress()
'
'                   Returns a channel's address stored in a chAddr structure in
'                   {netNum, deviceNum, moduleNum, chanNum} form.
' PARAMETERS
'
'    chAddrP    LoadChannelAddress() loads this channel address with
'               {netNum, deviceNum, moduleNum, chanNum}
'    netNum     NETWORK number = {0...numNetworks}, 0 is Driver,
'    deviceNum  DEVICE number {0...numDevices}, 0 = "Controller",
'    moduleNum  MODULE number within a hardware DEVICE {1...32}.
'    chanNum    Hardware CHANNEL number {1...32}.
'
'
' HISTORY       1/27/97 created - gsw
'-------------------------------------------------------------------------

' Please also see LoadChannelAddress_VB(), which is more VB friendly.
Declare Function LoadChannelAddress Lib "iNet32.DLL" (chAddrP As chAddr, _
    ByVal netNum As Byte, ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte) As Integer

Declare Sub GetChannelAddress Lib "iNet32.DLL" (chAddrP As chAddr, _
    netNumP As Integer, deviceNumP As Integer, moduleNumP As Integer, chanNumP As Integer)

'-------------------------------------------------------------------------
' FUNCTION      Set_instruNet_Network_State()
'               Get_instruNet_Network_State()
'
' PURPOSE       Sets/Gets the state of the instruNet network with the data in
'               the passed array.
' PARAMETERS
'
'    array    data is loaded into here (this should be 10000 long or so)
'
'    numValidBytesP -- this is the # of valid bytes in the array
'
' HISTORY     3/15/97 - created
'-------------------------------------------------------------------------

Declare Function Set_instruNet_Network_State Lib "iNet32.DLL" ( _
    arrayFirstPoint As Integer, ByVal numValidBytesP As Long) As Integer
    
Declare Function Get_instruNet_Network_State Lib "iNet32.DLL" ( _
    arrayFirstPoint As Integer, ByVal arraySize As Long, numValidBytesP As Long) As Integer

'  =========================== CLASS: BASIC DIGITIZE ============================
'  =========================== CLASS: BASIC DIGITIZE ============================
'  =========================== CLASS: BASIC DIGITIZE ============================

'-----------------------------------------------------------------------------------
' FUNCTION    EnableChannelForDigitizing()         SIMPLE FORMAT FUNCTIONS
'
' PURPOSE     Specifies that this channel is enabled for digitizing.
'
' PARAMETERS
'
'    chAddrP  This specifies a channel via it's channel Address
'             = {netNum, deviceNum, moduleNum, chanNum}
'
' HISTORY     1/27/97 created - gsw
'---------------------------------------------------------------------------------

Declare Function EnableChannelForDigitizing Lib "iNet32.DLL" (chAddrP As chAddr) As Integer

'-----------------------------------------------------------------------------------
' FUNCTION    DisableAllChannelsForDigitizing()    SIMPLE FORMAT FUNCTIONS
'
' PURPOSE     Disables ALL channels for digitizing.
'
' PARAMETERS  None
'
' HISTORY     1/27/97 created - gsw
'---------------------------------------------------------------------------------

Declare Function DisableAllChannelsForDigitizing Lib "iNet32.DLL" () As Integer

'-------------------------------------------------------------------------------------
' FUNCTION    Set_iNet_TIMING_Parameters()    SIMPLE FORMAT FUNCTION
'
' PURPOSE     Sets the instruNet TIMING Parameters.  All networks are
'             set the same.
'
' PARAMETERS
'
'    samplesPerSecond,
'             sample rate (points per second per channel)
'    pointsPerScan,
'             number of points in 1 scan, for 1 channel
'    numberOfScans,
'             number of scans that are digitized when the start button
'             is pressed
'    stripChartRecorder,
'             if TRUE (1) we do Strip Chart Recorder work with
'             Continuous Scans, otherwise if FALSE (0) we do
'             Oscilloscope mode (non-Coninuous Scans)
'    saveDataOption,
'             Save Data option = { ion_gSaveDataPopup_Off,
'             ion_gSaveDataPopup_ToRamBuffer, ion_gSaveDataPopup_ToDisk,
'             ion_gSaveDataPopup_UserControl}
'    showAlertOnOverflow,
'             if TRUE (1), the driver will show an alert if
'             a data buffer overflows; otherwise if FALSE (0)
'             it just keeps going if there is an overflow.
'    switchAccurately
'             if TRUE (1), channel switching should be accurate;
'             otherwise, it is less accurate, yet faster.'
' HISTORY     11/30/95  created - gsw
'-----------------------------------------------------------------------------------'/

Declare Function Set_iNet_TIMING_Parameters Lib "iNet32.DLL" (ByVal samplesPerSecond As Single, _
    ByVal pointsPerScan As Long, ByVal numberOfScans As Long, _
    ByVal stripChartRecorder As Integer, ByVal saveDataOption As Integer, _
    ByVal showAlertOnOverflow As Integer, ByVal switchAccurately As Integer) As Integer

'-------------------------------------------------------------------------------------
' FUNCTION    Set_iNet_TRIGGER_Parameters()        SIMPLE FORMAT FUNCTION
'
' PURPOSE     Sets the instruNet TRIGGER Parameters.  This function
'             sets the trigger for all networks to the same channel
'             in each network.
' PARAMETERS
'
'    triggerMode,
'             trigger mode = {ion_TriggerMode_Off, ion_TriggerMode_Auto,
'             ion_TriggerMode_Norm}
'    triggerThreshold,
'             trigger threshold in engineering units
'    triggerSlope,
'             trigger slope = {ion_Rising, ion_Falling}
'    preTriggerSeconds,
'             pre trigger duration, in seconds
'    triggerSourceP[]
'             trig source Channel Address = {netNum, deviceNum,
'             moduleNum, chanNum}/  NETWORK number = {0...numNetworks},
'             0 is Driver,
'
' HISTORY     11/30/95  created - gsw
'-----------------------------------------------------------------------------------'/

Declare Function Set_iNet_TRIGGER_Parameters Lib "iNet32.DLL" (ByVal triggerMode As Integer, _
    ByVal triggerThreshold As Single, ByVal triggerSlope As Integer, _
    ByVal preTriggerSeconds As Single, trigSrc_chAddrP As chAddr) As Integer

'-----------------------------------------------------------------------------------
' FUNCTION    Service_All_iNet_Digitize_Buffers()
'
' PURPOSE     This routine must be called continuously while
'             digitizing to allow the processor to service all digitize
'             buffers.
'
' PARAMETERS
'
'    netNum_forScanInfo
'             We get scan  info from this controller,
'             even though we service ALL controllers.
'
'    theNextDataAccessIsTheLast_ofEntireDigitization_allCtlr,
'             Returns TRUE (1) if the next data pulled out
'             out of the buffers via Access_Digitized_Data_In_Ram_Buffer()
'             is that last pull for all networks; FALSE (0) otherwise
'             (i.e. at least 1 network still has data in it)
'
'    theNextDataAccessWillPullToTheEndOfAScan_oneCtlr,
'             Returns TRUE (1) if the next data access via
'             Access_Digitized_Data_In_Ram_Buffer() accesses data
'             to the end of 1 Scan in network 'netNum_forScanInfo' (i.e. 1
'             Scan's worth of data); FALSE (0) otherwise.  If you do not
'             access 1 scan at a time, you can access little
'             segments at a time, as they become available.
'
'     controllerIsFinishedWithDigitization_allCtlr,
'             set to TRUE if controller is finished with digitization
'             (i.e. the last piece of data has been moved from controller
'             ram buffer to host memory ram buffer); FALSE otherwise.
'
'     last_scanNum_base1_pulled_inFull_oneCtlr,
'             last scan  (base 1) that we pulled (e.g. a/d) in 'netNum_forScanInfo'
'             full from controller ram buffer TO user or
'             driver ram buffer when Service_All_iNet_Digitize_Buffers()
'             was called.
'
'     last_scanNum_base1_pushed_inFull_oneCtlr
'             last scan  (base 1) that we pushed (e.g. d/a) in 'netNum_forScanInfo'
'             full from user or driver ram buffer TO controller
'             ram buffer when Service_All_iNet_Digitize_Buffers()
'             was called.
'
' HISTORY  11/30/95 created - gsw
'-----------------------------------------------------------------------------------

Declare Function Service_All_iNet_Digitize_Buffers Lib "iNet32.DLL" ( _
    ByVal netNum As Byte, theNextDataAccIsTheLast As Integer, _
    theNextDataAccessWillPullToTheEndOfASc As Integer, _
    controllerIsFinishedWithDigitization As Integer, _
    lastscanNumBase1PulledInFull As Long, lastscanNumBase1PushedInFull As Long) As Integer

'-----------------------------------------------------------------------------------
' FUNCTION    Access_Digitized_Data_In_Ram_Buffer()  SIMPLE FORMAT FUNCTION
'
' PURPOSE     While digitizing, this will pull a segment out of the iNet
'             DRIVER RAM BUFFER or USER RAM BUFFER and pass you an address
'             to the first FLT32 point in the buffer (i.e. beginning of the
'             entire buffer), the index into the buffer to begin reading
'             data, and the  of points to pull.
'
'             1st point to read
'             = (*bufferStartAddrP) +'pointToPullindex_base1 - 1
'
'             last point to read
'             = (*bufferStartAddrP) +'pointToPullindex_base1 +'numPointsToPull - 1
'
'             The buffer is 1 scan long, and the scan number {1...numScans}
'             that we are reading is returned in '*scanNumberToPull_base1'.
'
'             The  of times the iNet circular buffers overflowed is returned
'             in '*totalNumOfTimesWeOverflowed'.
'
' PARAMETERS
'
'    channelAddr[],
'             Channel Address = {netNum, deviceNum, moduleNum, chanNum}
'             NETWORK number = {0...numNetworks}, 0 is Driver,
'    driverRamBuffer,
'             if TRUE, we go to DRIVER RAM BUFFER, otherwise,
'             we go to USER RAM BUFFER.
'    bufferStartAddrP,
'             we will load this with the address to the
'             first FLT32 point in the buffer
'    pointToPullindex_base1,
'             we will load this with the {1... numPtsPerScan)
'             index of the first point of the segment that we need
'             to process (base 1 index).
'    numPointsToPull,
'             of the buffer
'    scanNumberToPull_base1
'             we load this with the scan number {1...numScans}
'             of the scan that we are currently pulling (base 1).
'
' HISTORY     11/25/95 created - gsw
'             11/30/95 modified - gsw
'---------------------------------------------------------------------------------'/

Declare Function Access_Digitized_Data_In_Ram_Buffer Lib "iNet32.DLL" ( _
    chAddrP As chAddr, ByVal driverRamBuffer As Byte, bufferStartAddrP As Long, _
    pointToPullindex_base1 As Long, numPointsToPull As Long, scanNumberToPull As Long) As Integer

'                  ADVANCED FIELD READ/WRITE ROUTINES

'-----------------------------------------------------------------------------------
' FUNCTION    iNet_int32()     Calls iNet() to read or write a field in the
'                              iNet network with a  signed 32bit integer
'                              value (+/- 2billion).
'
'             iNet_flt32()     Calls iNet() to read or write a field in the
'                              iNet network with a 32bit floating point number.
'
'             iNet_cString()   Calls iNet() to read or write a field in the
'                              iNet network with a C string variable (0-255
'                              chars, 0 terminated).
'
'             iNet_DLL()       Calls iNet() to read or write a field via a
'                              variable of any type
'
' PURPOSE     If we are setting a field value (i.e. intention == intention_setValue)
'             then we return an iNetError error code. If intention is not
'             intention_setValue and we get an error condition, then we
'             0; otherwise we return the resulting field value.
'
'             If you want to write to the field, set the 'intention' field to
'             'intention_setValue' and pass the new value in 'int32Value'.
'
'             If you want to read the value of a field, set the 'intention' field
'             to 'intention_getValue' and then refer to the returned value.
'
' PARAMETERS
'
'    netNum     NETWORK number = {0...numNetworks}, 0 is Driver,
'    deviceNum  DEVICE number {0...numDevices}, 0 = "Controller",
'    moduleNum  MODULE number within a hardware DEVICE {1...32}.
'    chanNum    Hardware CHANNEL number {1...32}.
'
'    settingGroupNumOrType
'              This references a specific Setting Group.
'
'              If > 0, this is a settings group Number:
'                  {1...numSettingGroups}, which corresponds to
'                   the order in which the SettingGroup appears
'                   in the Setting popup menu, with the first item
'                   in the menu is 1.
'
'              If < 0, this is a settings group Type:
'                  ion_settingGroupType = { sgt_VinHardware (-3)
'                  sgt_Vout (-4)... }

'    fieldNum  Specifies the field number within the settingGroup.
'              The 1st field is 1, then next 2, etc.  See fgNums_...
'              enum's in INET_Declarations.Bas file for a list of these.
'
'    intention Tells iNet() what to do with the field.  Please see
'              ion_intention enum for details. ion_intention =
'              {intention_getValue, intention_setValue,
'              intention_getNameStr, intention_getMaxValue, etc}
'              we set field with this value if that is
'              the specified intention
'
' HISTORY     2/4/96 created - gsw
'-----------------------------------------------------------------------------------

Declare Function iNet_int32 Lib "iNet32.DLL" (ByVal netNum As Byte, _
    ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte, _
    ByVal settingGroupNumOrType As Integer, ByVal fieldNum As Integer, _
    ByVal intention As Integer, ByVal int32Value As Long) As Integer
    
Declare Function iNet_uint32 Lib "iNet32.DLL" (ByVal netNum As Byte, _
    ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte, _
    ByVal settingGroupNumOrType As Integer, ByVal fieldNum As Integer, _
    ByVal intention As Integer, ByVal int32Value As Long) As Integer
    
Declare Function iNet_flt32 Lib "iNet32.DLL" (ByVal netNum As Byte, _
    ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte, _
    ByVal settingGroupNumOrType As Integer, ByVal fieldNum As Integer, _
    ByVal intention As Integer, ByVal flt32Value As Single) As Single
    
Declare Function iNet_DLL Lib "iNet32.DLL" (ByVal netNum As Byte, _
    ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte, _
    ByVal settingGroupNumOrType As Integer, ByVal fieldNum As Integer, _
    ByVal intention As Integer, ByVal argType As Integer, ptrToArg As Any) As Integer

' Please use iNet_VB_cString() instead of iNet_cString().
Declare Function iNet_cString Lib "iNet32.DLL" (ByVal netNum As Byte, _
    ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte, _
    ByVal settingGroupNumOrType As Integer, ByVal fieldNum As Integer, _
    ByVal intention As Integer, ByVal cString As String) As Integer


'/////////////////////////////////////////////////////////////////////////
'                          DRIVER ROUTINES
'/////////////////////////////////////////////////////////////////////////

'-----------------------------------------------------------------------------------
' FUNCTION    Load_instruNet_Driver()
'
' PURPOSE     Loads the instruNet Driver file if 'reset_instruNet' is
'             true we reset the network
'
' PARAMETERS
'
'    reset_instruNet
'             if true we force the resetting of the instrunet network
'    driverIsInstalledOK as integer,
'             loaded with TRUE (1) if driver is installed ok;
'             [e.g. Win95 'iNet32.DLL' or Mac 'instruNet Driver (ppc)']
'             FALSE (0) otherwise.
'    numNetworks
'             loaded with the number of networks (i.e. controllers found)
'
' HISTORY     1/27/97 created - gsw
'---------------------------------------------------------------------------------

Declare Function Load_instruNet_Driver Lib "iNet32.DLL" (ByVal reset_instruNet_ As Integer, _
    ByRef driverIsInstalledOK_ As Integer, ByRef numNetworks_ As Integer) As Integer

'-----------------------------------------------------------------------------------
' FUNCTION    CloseDriverAndReleaseDriverRam()
'
' PURPOSE     This routine closes the instruNet driver and releases the
'             memory used to hold it. This will delete ram use to hold
'             iNet95.vxd, iNetNT.dvr, internal buffers, etc.
'
'             ONLY CALL THIS ONCE, AND DON'T CALL iNet() AFTER CALLING
'             THIS ROUTINE (e.g. CALL THIS WHEN YOUR APPLICATION QUITS).
'
'             Exiting an application usually releases the ram used to hold
'             the driver as well; therefore, this is sometimes unnecessary
'             when someone is exiting the calling application.
'
'
' PARAMETERS  None
'
' HISTORY     9/8/96  created - gsw
'---------------------------------------------------------------------------------

Declare Sub CloseDriverAndReleaseDriverRam Lib "iNet32.DLL" ()

'-----------------------------------------------------------------------------------
' FUNCTION    Show_ALERT_if_hit_iNet_Error()
'
' PURPOSE     This communicates an error value to the user, the easiest
'             simplest way possible.
'
' PARAMETERS  None
'
' HISTORY     11/30/95 created - gsw
'             1/22/96  kag - Added Windows code
'---------------------------------------------------------------------------------

Declare Function Show_ALERT_if_hit_iNet_Error Lib "iNet32.DLL" ( _
    ByVal iNetError_ As Integer) As Integer

'-----------------------------------------------------------------------------------
' FUNCTION    Show_Simple_Alert()
'
' PURPOSE     Shows a simple alert message.
'
' PARAMETERS  cString  message to be displayed.
'
' HISTORY     12/20/96  created - gsw
'---------------------------------------------------------------------------------

Declare Sub Show_Simple_Alert Lib "iNet32.DLL" (ByVal cString As String)

'-----------------------------------------------------------------------------------
' FUNCTION    Get_iNet_Error()
'
' PURPOSE     Returns the error code returned by the last call
'             to the instruNet Driver
'
' PARAMETERS  None
'
' HISTORY     2/4/96  created - gsw
'---------------------------------------------------------------------------------

Declare Function Get_iNet_Error Lib "iNet32.DLL" () As Integer

'-----------------------------------------------------------------------------------
' FUNCTION    Get_Last_iNet_Call()
'
' PURPOSE     Returns a pointer to a structure that contains the
'             internal parameters passed to the last instruNet() call.
'
' PARAMETERS  pointerToData   This Long is filled with a pointer
'                             to where the internal data is kept.
'
' HISTORY     2/4/96  created - gsw
'---------------------------------------------------------------------------------

Declare Sub Get_Last_iNet_Call Lib "iNet32.DLL" (pointerToData As Long)

 ' -----------------------------------------------------------------------------------
 '  FUNCTION    Open_instruNet_Window()     CLASS: BASIC UTILITY
 '
 '              Opens an instruNet window.
 '
 '  PARAMETERS          1  = iNetCM_OpenWindow_Record
 '                      2  = iNetCM_OpenWindow_Network
 '                      3  = iNetCM_OpenWindow_Test
 '                      4  = iNetCM_OpenWindow_FullProbe
 '                      5  = iNetCM_OpenWindow_ChanSelection
 '                      6  = iNetCM_OpenWindow_FieldSelection
 '                      7  = iNetCM_OpenWindow_OneFieldSet
 '                      8  = iNetCM_OpenWindow_AllFieldSets
 '                      9  = iNetCM_OpenWindow_RecTiming
 '                      10 = iNetCM_OpenWindow_RecTrigger
 '                      11 = iNetCM_OpenWindow_RecOptions
 '                      12 = iNetCM_OpenWindow_RecFile
 '                      13 = iNetCM_OpenWindow_SimpleTiming
 '                      (pts-per-scan and sample-rate)
 '                      14 = iNetCM_OpenWindow_CalOptions
 '
 '  netNumP, deviceNumP, moduleNumP, chanNumP, settingGpP, fieldNumP, userPressedOkP
 '
 '                      Loaded with result of user's dialog activity
 '
 '  HISTORY     9/5/97  added 9..12 - gsw
 ' ---------------------------------------------------------------------------------

' Please use Open_instruNet_Window_VB for a simplier version.
Declare Function Open_instruNet_Window Lib "iNet32.DLL" (ByVal iNetWindowType As Integer, ByRef netNumP As Integer, _
    ByRef deviceNumP As Integer, ByRef moduleNumP As Integer, _
    ByRef chanNumP As Integer, ByRef settingGpP As Integer, _
    ByRef fieldNumP As Integer, ByRef userPressedOkP As Integer) As Integer

'/////////////////////////////////////////////////////////////////////////
'                            UTILITY ROUTINES
'/////////////////////////////////////////////////////////////////////////

'-----------------------------------------------------------------------------------
' FUNCTIONS   iNet_Peek_int16()  Returns the signed 16bit integer value at 'address'.
'             iNet_Peek_int32()  Returns the signed 32bit integer value at 'address'.
'             iNet_Peek_flt32()  Returns the signed 32bit floating point value at 'address'.
'
' PARAMETERS  address  Address value.
'
' HISTORY     2/6/96 created - gsw
'---------------------------------------------------------------------------------

Declare Function iNet_Peek_int16 Lib "iNet32.DLL" (address As Long) As Integer
Declare Function iNet_Peek_int32 Lib "iNet32.DLL" (address As Long) As Long
Declare Function iNet_Peek_flt32 Lib "iNet32.DLL" (address As Long) As Single

'-----------------------------------------------------------------------------------
' FUNCTIONS   iNet_Poke_int16()  Sets the 16bit integer value at 'address'.
'             iNet_Poke_int32()  Sets the 32bit integer value at 'address'.
'             iNet_Poke_flt32()  Sets the 32bit floating point value at 'address'.
'
' PARAMETERS  address     address to read
'             int16value  value to write
'
' HISTORY     2/6/96 created - gsw
'-----------------------------------------------------------------------------------

Declare Sub iNet_Poke_int16 Lib "iNet32.DLL" (address As Long, ByVal int16value As Integer)
Declare Sub iNet_Poke_int32 Lib "iNet32.DLL" (address As Long, ByVal int32Value As Long)
Declare Sub iNet_Poke_flt32 Lib "iNet32.DLL" (address As Long, ByVal flt32Value As Single)

'-----------------------------------------------------------------------------------
' FUNCTION    iNet_Get_VarPtr()
'
' PURPOSE     Returns the address of where in memory the passed variable
'             is stored.
'
' PARAMETERS  Variable      Any variable or array (e.g. "x", "y(0)").
'
' HISTORY     2/6/96 created - gsw
'-----------------------------------------------------------------------------------

Declare Function iNet_Get_VarPtr Lib "iNet32.DLL" (Variable As Any) As Long

'-----------------------------------------------------------------------------------
' FUNCTION    iNet_memcopy()
'
' PURPOSE     Moves 'numBytes' of data starting at 'srcAddr' address to
'             'destAddr' address.
'
' PARAMETERS  srcAddr     source address
'             destAddr    destination address
'             numBytes    number of bytes to move
'
' HISTORY     2/6/96 created - gsw
'-----------------------------------------------------------------------------------

Declare Sub iNet_memcopy Lib "iNet32.DLL" (ByVal srcAddr As Long, _
    ByVal destAddr As Long, ByVal numBytes As Long)
    
Declare Sub iNet_memcopy_int16 Lib "iNet32.DLL" (srcAddr() As Integer, _
    destAddr() As Integer, ByVal numBytes As Long)
    
Declare Sub iNet_memcopy_flt32 Lib "iNet32.DLL" (srcAddr() As Single, _
    destAddr() As Single, ByVal numBytes As Long)

' ----------------------------------------------------------------------------
'  FUNCTION    fft_DLL()
'
'  PURPOSE     This function computes the fft on an array of numbers.
'              It is given 2^power real values and returns 2^(power-1)
'              complex pairs (real, imaginary, real, imaginary).  Please
'              give it an array that is 2 longer than 2^power -- the algorithm
'              needs this.  Several notes:
'
'              *   This routine is given 2^power real values.
'              *   The data[] array must contain 2 extra elements at
'                  the end -- CAREFUL!
'              *   We return 2^(power-1) complex pairs values.
'              *   If you take a sine wave and multiply it by a hamming
'                  window, it is like convolving the frequency response
'                  of the hamming window with a delta function, and
'                  that is what you see at the fft output.  The
'                  frequency response of the hamming window is
'                  one main lobe with side lobes 42dB down.
'
'  HISTORY     8/10/98     tsfr'ed to iNet - gsw
' -----------------------------------------------------------------------------
Declare Sub fft_DLL Lib "iNet32.DLL" ( _
    ByVal dataAddressP As Long, ByVal power As Integer)
    
' -------------------------------------------------------------------------------------
'  FUNCTION    MultiplyArrayByHammWindow()
'
'  PURPOSE     This multiplies the flt32 array by a Hamming window,
'              # of pts = 2 ^ power, power is {1...18}.  Returns TRUE if successful.
'
'  HISTORY     8/10/98 created - gsw
' ----------------------------------------------------------------------------------- */
Declare Function MultiplyArrayByHammWindow_DLL Lib "iNet32.DLL" ( _
    ByVal dataAddressP As Long, ByVal power As Integer) As Integer

' -----------------------------------------------------------------------------
'  FUNCTION    CalculateWindow_DLL()
'
'  PURPOSE     This uses the cos() function to create a float (0.0 to 1.0 values)
'              window (Hamm=0, Hann=1, Blackman=2). Returns TRUE if successful; FALSE otherwise.
'
'              Ham:        h(n) = .54 - .46*cos(2n/(N-1)) 0 <= n <= N-1
'              Hann:       h(n) = .50 - .50*cos(2n/(N-1)) 0 <= n <= N-1
'              Blackman    h(n) = .42 - .50*cos(2n/(N-1)) + .08*cos(4n/(N-1))    0 <= n <= N-1
'
'  HISTORY     8/10/98 created - gsw
' -----------------------------------------------------------------------------
Declare Function CalculateWindow_DLL Lib "iNet32.DLL" ( _
    ByVal numpts As Long, ByVal wType As Integer, ByVal windowAddressP As Long) As Integer

' -----------------------------------------------------------------------------
'  FUNCTION    MultiplyTwoFlt32Arrays_DLL()
'
'  PURPOSE     This multiplies two flt32 arrays.
'
'  HISTORY     8/10/98 created - gsw
' -----------------------------------------------------------------------------
Declare Sub MultiplyTwoFlt32Arrays_DLL Lib "iNet32.DLL" _
    (ByVal srcAddressP As Long, ByVal destAddressP As Long, ByVal numpts As Long)

' -----------------------------------------------------------------------------
'   FUNCTION    ComplexToMagnitudes_DLL()
'
'   PURPOSE     This converts {real, imaginary} complex pairs in a flt32 array to
'               magnitudes = sqrt(real^2 + imaginary^2).
'
'   HISTORY     8/10/98 created - gsw
' -----------------------------------------------------------------------------
Declare Sub ComplexToMagnitudes_DLL Lib "iNet32.DLL" ( _
    ByVal dataAddressP As Long, ByVal numComplexPairs As Long)

' -------------------------------------------------------------------------------------
'   FUNCTION    MultConstantByArray_DLL()
'
'   PURPOSE     Multiply a constant by a flt32 array.
'
'   HISTORY     8/10/98 created - gsw
' ----------------------------------------------------------------------------------- */

Declare Sub MultConstantByArray_DLL Lib "iNet32.DLL" ( _
    ByVal srcAddressP As Long, ByVal destAddressP As Long, _
    ByVal constant As Single, ByVal numpts As Long)


' show an alert if e contains an error (i.e. non 0 value)
Public Function AlertOnError(ByVal e As Integer) As Integer

    AlertOnError = Show_ALERT_if_hit_iNet_Error(e)
    
End Function


' Save or Load instruNet waveform data and program parameters via the File
' Save/Open dialog.
'
' PARAMETERS
'
'   saveLoadEndUserData - If true, EndUserData_SaveOrLoad(), an end user routine,
'      is called to save/load end user parameters.
'   saveOrLoad - set to iSave if you want to Save, and set to iLoad if you want to load
'   doTimingInfo - set true if you want to save/load digitize timing parameters
'   doDisplayBoxInfo - set true if you want to save/load DisplayBox parameters
'   doDisplayInfo - set true if you want to save/load Display parameters
'   doDigitizeData - set true if you want to save/load digitize data
'   reportOnUnrecognizedLines - set to true if want to show an alert if you load in a file
'      and hit some data that you do not recognize.
'
Public Sub SaveOrLoad_DataAndState( _
    ByVal saveLoadEndUserData As Integer, _
    ByVal saveOrLoad As Integer, _
    ByVal doTimingInfo As Integer, _
    ByVal doDisplayBoxInfo As Integer, _
    ByVal doDisplayInfo As Integer, _
    ByVal doDigitizeData As Integer, _
    ByVal reportOnUnrecognizedLines As Integer)
       
    Dim fileNum, hitError, save, e As Integer
    Dim sPathName As String
    Dim bNum, dNum, cNum As Integer
    Dim numUnrecognizedLines, numValidBytes As Long
     
    ' if we are currently digitizing, we cannot do save/load
    If gCurrentlyDigitizing Then Exit Sub
    
    ' --------------------------------------------------------------
    ' SAVE/LOAD DATA IN BINARY MERGE FILE ("BMerge1.ioW") AND
    ' SAVE INSTRUNET NETWORK SETTINGS IN A ".prf" FILE.  This is placed
    ' into a new folder that is named by the user.

    save = (saveOrLoad = iSave)     ' true if we are saving
    hitError = False
    On Error GoTo ErrorHandler
    
    e = iNetErr_None                ' initialize
    numUnrecognizedLines = 0
    
    If save Then                    ' if saving
    
        ' Copy the VB Digitize data buffer data into the iNet32.DLL DRIVER RAM BUFFERS
        Call VbDataBuffers_CopyToDLLDvrBuffers
           
        ' set the fileType to BinaryMerge
        Call SetFieldValue_iNet(0, 0, 1, driver_ChanNum_RecordOptions, _
            sgt_MasterDirectory, fldNum_Directory_FileType, ft_BinaryMerge)
                       
        ' You cannot show the iNet World window in here, since
        ' closing that window causes the Driver Ram buffers to be disposed.
        
        ' Press the Save button in instruNet world, to save the data.
        e = PRESS_iNet_BUTTON(iNetCM_BtnPress_Record_Save)
        If ErrorHere(e, "ss21") Then Exit Sub
                     
        ' Disable all DigitizeChannels inside the iNet driver (kills the DRIVER RAM buffers)
        e = DisableAllChannelsForDigitizing()
        If ErrorHere(e, "ss22") Then Exit Sub
                
    Else                            ' else we are loading
    
        ' Press the Open button in instruNet world, to load the data.
        e = PRESS_iNet_BUTTON(iNetCM_BtnPress_Record_Open)
        If ErrorHere(e, "ss23") Then Exit Sub
        
    End If
    
    ' get a copy of pathname of the folder that contains this data.
    Call iNet_VB_cString(0, 0, 1, driver_ChanNum_RecordOptions, _
        sgt_MasterDirectory, fldNum_Directory_PathName, _
        intention_getValue, sPathName)
        
    ' --------------------------------------------------------------
    ' CREATE or OPEN "\Internal.iNt" FILE
    
    ' create/open a file and place it in the data folder that we just created.
    fileNum = FreeFile(0)
    If save Then
        Open (sPathName + "\Internal.iNt") For Output As #fileNum
    Else
        Open (sPathName + "\Internal.iNt") For Input As #fileNum
    End If
    If hitError Then Exit Sub
    
    ' --------------------------------------------------------------
    ' SAVE/LOAD INFORMATION ABOUT TIMING PARAMETERS
      
    If save And doTimingInfo Then       ' if saving...
    
        Write #fileNum, "TimingParams_Start", 0
        
        Write #fileNum, "gNumScans", gNumScans
        Write #fileNum, "gPtsPerScan_nextDigi", gPtsPerScan_nextDigi
        Write #fileNum, "gSampleRate_nextDigi", gSampleRate_nextDigi
        Write #fileNum, "gContinuousScans", gContinuousScans
        Write #fileNum, "gProcessSegmentsInsteadOfScans", gProcessSegmentsInsteadOfScans
        Write #fileNum, "gRun_EndUser_PostAcqAnalysis", gRun_EndUser_PostAcqAnalysis
        Write #fileNum, "gRun_EndUser_SegmentProcessor", gRun_EndUser_SegmentProcessor
        Write #fileNum, "gCopyChannelsFromNetworkPage", gCopyChannelsFromNetworkPage
        
        Write #fileNum, "TimingParams_End", 0
       
    ElseIf doTimingInfo Then            ' else, we are loading
    
        Do                              ' load in the DisplayInfo data
                                                               
            Input #fileNum, info$, dValue#    ' load in 1 line of info
                        
            If StrComp(info$, "TimingParams_Start") = 0 Then
                dValue# = dValue# * 1
            ElseIf StrComp(info$, "TimingParams_End") = 0 Then: Exit Do
            
            ElseIf StrComp(info$, "gNumScans") = 0 Then: gNumScans = dValue#
            ElseIf StrComp(info$, "gPtsPerScan_nextDigi") = 0 Then: gPtsPerScan_nextDigi = dValue#
            ElseIf StrComp(info$, "gSampleRate_nextDigi") = 0 Then: gSampleRate_nextDigi = dValue#
            ElseIf StrComp(info$, "gContinuousScans") = 0 Then: gContinuousScans = dValue#
            ElseIf StrComp(info$, "gProcessSegmentsInsteadOfScans") = 0 Then: gProcessSegmentsInsteadOfScans = dValue#
            ElseIf StrComp(info$, "gRun_EndUser_PostAcqAnalysis") = 0 Then: gRun_EndUser_PostAcqAnalysis = dValue#
            ElseIf StrComp(info$, "gRun_EndUser_SegmentProcessor") = 0 Then: gRun_EndUser_SegmentProcessor = dValue#
            ElseIf StrComp(info$, "gCopyChannelsFromNetworkPage") = 0 Then: gCopyChannelsFromNetworkPage = dValue#
            
            Else:
                numUnrecognizedLines = numUnrecognizedLines + 1
            End If
                
        Loop While Not (EOF(fileNum))
    
        ' Set the digitize parameters
        Call DigitizeProcess_SetTimingParams(gPtsPerScan_nextDigi, gNumScans, _
            gSampleRate_nextDigi, gContinuousScans, gProcessSegmentsInsteadOfScans, _
            gRun_EndUser_PostAcqAnalysis, gRun_EndUser_SegmentProcessor, _
            gCopyChannelsFromNetworkPage, gWaitTillDigitizationCompletes)
    
    End If
    
    ' --------------------------------------------------------------
    ' SAVE/LOAD INFORMATION ABOUT Digitize Data
      
    If save And doDigitizeData Then            ' if saving...
    
        Write #fileNum, 0, "DigitizeData_Start", gNumDigiCh
        
        ' write out info associated with the displays
        
        For cNum = 0 To (gNumDigiCh - 1)           ' scan through all Digitize Channels
            With gDigiChan(cNum)
            
                Write #fileNum, cNum, "netNum"; .netNum
                Write #fileNum, cNum, "deviceNum"; .deviceNum
                Write #fileNum, cNum, "moduleNum"; .moduleNum
                Write #fileNum, cNum, "chanNum"; .chanNum
                Write #fileNum, cNum, "bNum"; .bNum
                Write #fileNum, cNum, "dNum"; .dNum
                Write #fileNum, cNum, "numValidPts"; .dataArrayY.numValidPts
                Write #fileNum, cNum, "numBufferPts"; .dataArrayY.numBufferPts
                Write #fileNum, cNum, "ptsPerXunit"; .dataArrayY.ptsPerXunit
                Write #fileNum, cNum, "firstPt_Xunits"; .dataArrayY.firstPt_Xunits
                Write #fileNum, cNum, "massageType"; .massageType
                Write #fileNum, cNum, "runD"; .runEndUser_SegmentProcessor
                
           End With
        Next cNum
        If hitError Then Exit Sub
        
        Write #fileNum, 0, "DigitizeData_End", 0
    
    ElseIf doDigitizeData Then                    ' else, we are loading
    
        Do                  ' load in the DisplayInfo data
                        
                            ' load in 1 line of info
            Input #fileNum, cNum, info$, dValue#
                    
                            ' if we are at the beginnning, get the # of
                            ' DigitizeChannels, and set the gDigiChan() array
                            ' accordingly.
            If StrComp(info$, "DigitizeData_Start") = 0 Then
            
                gNumDigiCh = dValue#        ' # of digitize channels
                
                            ' Set the size of the gDigiChan() array to accomodate the info.
                ReDim Preserve gDigiChan(gNumDigiCh)
                
            End If
                            ' if this is not in our ChannelList...
            If (cNum < 0 Or cNum >= gNumDigiCh) Then
                    
                            ' bail out of this section if we are at the end
                If StrComp(info$, "DigitizeData_End") = 0 Then Exit Do
                
                            ' record that we hit an unrecognized line
                numUnrecognizedLines = numUnrecognizedLines + 1
                
            Else            ' else, read this data & place into internal parameters
            
                
                                ' write this data into internal parameters
                With gDigiChan(cNum)
                
                    If StrComp(info$, "DigitizeData_End") = 0 Then
                        Exit Do
                    ElseIf StrComp(info$, "DigitizeData_Start") = 0 Then: dValue# = dValue# * 1 ' this was handled above
                    ElseIf StrComp(info$, "netNum") = 0 Then: .netNum = dValue#
                    ElseIf StrComp(info$, "deviceNum") = 0 Then: .deviceNum = dValue#
                    ElseIf StrComp(info$, "moduleNum") = 0 Then: .moduleNum = dValue#
                    ElseIf StrComp(info$, "chanNum") = 0 Then: .chanNum = dValue#
                    ElseIf StrComp(info$, "bNum") = 0 Then: .bNum = dValue#
                    ElseIf StrComp(info$, "dNum") = 0 Then: .dNum = dValue#
                    ElseIf StrComp(info$, "numValidPts") = 0 Then: .dataArrayY.numValidPts = dValue#
                    ElseIf StrComp(info$, "numBufferPts") = 0 Then: .dataArrayY.numBufferPts = dValue#
                    ElseIf StrComp(info$, "ptsPerXunit") = 0 Then: .dataArrayY.ptsPerXunit = dValue#
                    ElseIf StrComp(info$, "firstPt_Xunits") = 0 Then: .dataArrayY.firstPt_Xunits = dValue#
                    ElseIf StrComp(info$, "massageType") = 0 Then: .massageType = dValue#
                    ElseIf StrComp(info$, "runD") = 0 Then: .runEndUser_SegmentProcessor = dValue#
                    Else:
                        numUnrecognizedLines = numUnrecognizedLines + 1
                    End If
                    
                End With
            End If
        Loop While Not (EOF(fileNum))
                     
        ' load channel address into the gDigiChan() array.
        For cNum = 0 To gNumDigiCh - 1          ' scan through all digitize channels
            With gDigiChan(cNum)                ' w/ this channel...
                Call LoadChannelAddress(gDigiChan(cNum), .netNum, .deviceNum, .moduleNum, .chanNum)
             End With
        Next cNum
        
        ' Load iNet32.DLL DRIVER RAM buffer data into VB dataArrayY.data() arrays
        Call VbDataBuffers_CopyFromDLLDvrBuffers
        
        ' Disable all DigitizeChannels inside the iNet driver (to close
        ' the Driver Ram Buffers).
        e = DisableAllChannelsForDigitizing()
        If ErrorHere(e, "ctd2") Then Exit Sub
    
    End If
    
    ' --------------------------------------------------------------
    ' SAVE/LOAD INFORMATION ABOUT DISPLAY BOXES
      
    If save And doDisplayBoxInfo Then       ' if saving...
    
        Write #fileNum, 0, "DisplayBox_Start", gNumBoxes
   
        ' write out info associated with the displays
        For bNum = 0 To gNumBoxes - 1       ' scan through all DisplayBoxes
            With gBox(bNum)
            
                Write #fileNum, bNum, "numDisp"; .numDisp
                Write #fileNum, bNum, "hPos_scroll_lastValue"; .hPos_scroll_lastValue
                Write #fileNum, bNum, "vPos_scroll_lastValue"; .vPos_scroll_lastValue
                Write #fileNum, bNum, "autoAddPolicy"; .autoAddPolicy
            
            End With
            If hitError Then Exit Sub
        Next bNum

        Write #fileNum, 0, "DisplayBox_End", 0
    
    ElseIf doDisplayBoxInfo Then            ' else, we are loading
    
        Do                  ' load in the DisplayInfo data
                        
                            ' load in 1 line of info
            Input #fileNum, bNum, info$, dValue#
            
                            ' if we do not have a box or display to
                            ' put the data in, then go to the next
                            ' section in the file.
            If (bNum < 0 Or bNum >= gNumBoxes) Then
                        
                            ' bail out of this section if we are at the end
                If StrComp(info$, "DisplayBox_End") = 0 Then Exit Do
                            
                            ' record that we hit an unrecognized line
                If StrComp(info$, "DisplayBox_Start") <> 0 Then
                    numUnrecognizedLines = numUnrecognizedLines + 1
                End If
                
            Else            ' else, read this data & place into internal parameters
            
                With gBox(bNum)
                
                    If StrComp(info$, "DisplayBox_Start") = 0 Then
                        gNumBoxes_inFile = dValue#
                    ElseIf StrComp(info$, "DisplayBox_End") = 0 Then: Exit Do
                    
                    ElseIf StrComp(info$, "numDisp") = 0 Then: numDisp_inFile = dValue#
                    ElseIf StrComp(info$, "hPos_scroll_lastValue") = 0 Then: .hPos_scroll_lastValue = dValue#
                    ElseIf StrComp(info$, "vPos_scroll_lastValue") = 0 Then: .vPos_scroll_lastValue = dValue#
                    ElseIf StrComp(info$, "autoAddPolicy") = 0 Then: .autoAddPolicy = dValue#
                    
                    Else:
                        numUnrecognizedLines = numUnrecognizedLines + 1
                    End If
                    
                End With
            End If
            
        Loop While Not (EOF(fileNum))
    End If
   
    ' --------------------------------------------------------------
    ' SAVE/LOAD INFORMATION ABOUT DISPLAYS
      
    If save And doDisplayInfo Then            ' if saving...
    
        Write #fileNum, 0, 0, "DisplayInfo_Start", 0
   
        ' write out info associated with the displays
        For bNum = 0 To gNumBoxes - 1                   ' scan through all DisplayBoxes
            For dNum = 0 To (gBox(bNum).numDisp - 1)    ' scan through all Displays
                With gBox(bNum).disp(dNum)
                
                    Write #fileNum, bNum, dNum, "left"; .dispLeft_xEU
                    Write #fileNum, bNum, dNum, "right"; .dispRight_xEU
                    Write #fileNum, bNum, dNum, "top"; .dispTop_yEu
                    Write #fileNum, bNum, dNum, "bot"; .dispBot_yEu
                    Write #fileNum, bNum, dNum, "showGrid"; .showGrid
                    Write #fileNum, bNum, dNum, "cNum"; .cNum
                    Write #fileNum, bNum, dNum, "displayType"; .displayType
                    
                End With
            Next dNum
            If hitError Then Exit Sub
        Next bNum

        Write #fileNum, 0, 0, "DisplayInfo_End", 0
    
    ElseIf doDisplayInfo Then                   ' else, we are loading
    
        Do                  ' load in the DisplayInfo data
                        
                            ' load in 1 line of info
            Input #fileNum, bNum, dNum, info$, dValue#
            
                            ' if we do not have a box or display to
                            ' put the data in, then go to the next
                            ' section in the file.
            If ((bNum < 0 Or bNum >= gNumBoxes) Or (dNum < 0)) Then
            
                            ' bail out of this section if we are at the end
                If StrComp(info$, "DisplayInfo_End") = 0 Then Exit Do
                            
                            ' record that we hit an unrecognized line
                If StrComp(info$, "DisplayInfo_Start") <> 0 Then
                    numUnrecognizedLines = numUnrecognizedLines + 1
                End If
                
            Else            ' else, read this data & place into internal parameters
            
                            ' if we do not have a Display for this data,
                            ' we'll add it to the DisplayBox.
                If (dNum >= gBox(bNum).numDisp) Then
                                   
                    ' we'll assume that this Box shows cNum #0 to cNum# (numDisplays-1)
                    cNum_newDisplay = dNum
                    
                    ' Add 1 Display the Box and attach it to the digitize channel.
                    Call Add_DigitizeChannels_To_OneBox(bNum, _
                        cNum_newDisplay, cNum_newDisplay, kYindexDisplay)
                    
                    ' make sure we made it
                    If (dNum >= gBox(bNum).numDisp) Then EndProgram ("AddDisp Err")
                    
                End If
                                ' write this data into internal parameters
                With gBox(bNum).disp(dNum)
                
                    If StrComp(info$, "DisplayInfo_Start") = 0 Then
                        dValue# = dValue# * 1
    
                    ElseIf StrComp(info$, "DisplayInfo_End") = 0 Then: Exit Do
                    ElseIf StrComp(info$, "left") = 0 Then: .dispLeft_xEU = dValue#
                    ElseIf StrComp(info$, "right") = 0 Then: .dispRight_xEU = dValue#
                    ElseIf StrComp(info$, "top") = 0 Then: .dispTop_yEu = dValue#
                    ElseIf StrComp(info$, "bot") = 0 Then: .dispBot_yEu = dValue#
                    ElseIf StrComp(info$, "showGrid") = 0 Then: .showGrid = dValue#
                    ElseIf StrComp(info$, "cNum") = 0 Then: .cNum = dValue#
                    ElseIf StrComp(info$, "displayType") = 0 Then: .displayType = dValue#
                    
                    Else:
                        numUnrecognizedLines = numUnrecognizedLines + 1
                    End If
                    
                End With
            End If
        Loop While Not (EOF(fileNum))
    End If
    
    ' --------------------------------------------------------------
    ' CLOSE FILE
    
    ' close the open file
    Close #fileNum
    If hitError Then Exit Sub
    
    ' --------------------------------------------------------------
    ' Save/Load EndUser Data, if necessary
    
    If saveLoadEndUserData Then
    
        ' call the end user's save routine
        Call gEndUserWindow.EndUser_SaveLoadResults(sPathName, saveOrLoad)
        
    End If
    
    ' --------------------------------------------------------------
    ' Reset controls and redraw if loading Display info.
    
    If Not (save) And doDisplayInfo Then     ' if we are loading
   
        ' Set the horiz and vertical scales and positions for
        ' all the displays to the saved .dispLeft_xEU, .dispRight_xEU,
        ' .dispTop_yEU, .dispBot_yEU values.  We also update
        ' scrollbars to their past values.
        
        Reset_Controls_AllBoxes (reset_hScaleToInternalValues + _
                                 reset_ptsPerHscale + _
                                 reset_vScaleToInternalValues)
                                 
        ' Clear all DispBoxes and then redraw them with the new hScales.
        
        Redraw_AllBoxes
    
    End If
    
    ' --------------------------------------------------------------
    ' Exit
    
    If (numUnrecognizedLines > 0) And reportOnUnrecognizedLines Then
        MsgBox ("We hit " + Str(numUnrecognizedLines) + " lines of unrecognized data.")
    End If
    
    Exit Sub ' end of this subroutine

    ' --------------------------------------------------------------
    ' Handle file I/O error case.

ErrorHandler:

    If (hitError = False) Then      ' if this is a vb error...
        MsgBox (err.Description)    ' show msg about error
    Else
        MsgBox "We hit an error during file I/O."
    End If
    
    ' stop saving/loading after error
    
End Sub



 '  FUNCTION    Open_instruNet_Window_VB()
 '
 '              Opens an instruNet window.
 '
 '  iNetWindowType      1  = iNetCM_OpenWindow_Record
 '                      2  = iNetCM_OpenWindow_Network
 '                      3  = iNetCM_OpenWindow_Test
 '                      4  = iNetCM_OpenWindow_FullProbe
 '                      5  = iNetCM_OpenWindow_ChanSelection
 '                      6  = iNetCM_OpenWindow_FieldSelection
 '                      7  = iNetCM_OpenWindow_OneFieldSet
 '                      8  = iNetCM_OpenWindow_AllFieldSets
 '                      9  = iNetCM_OpenWindow_RecTiming
 '                      10 = iNetCM_OpenWindow_RecTrigger
 '                      11 = iNetCM_OpenWindow_RecOptions
 '                      12 = iNetCM_OpenWindow_RecFile
 '                      13 = iNetCM_OpenWindow_SimpleTiming
 '                      (pts-per-scan and sample-rate)
 '                      14 = iNetCM_OpenWindow_CalOptions
 '
 '  HISTORY     7/5/98  added - gsw
 '
Public Sub Open_instruNet_Window_VB(ByVal iNetWindowType As Integer)

    Dim netNum As Integer: Dim deviceNum As Integer: Dim moduleNum As Integer
    Dim chanNum As Integer: Dim settingGp As Integer: Dim fieldNum As Integer
    Dim userPressedOk As Integer

    Call Open_instruNet_Window(iNetWindowType, netNum, deviceNum, moduleNum, _
        chanNum, settingGp, fieldNum, userPressedOk)

End Sub


' ErrorHere()
'
' This function shows an alert if there is an error (i.e.
' "errorCode" is not 0).  Also, we show a message as
' to where the error occured in the code.

Function ErrorHere(ByVal errorCode As Integer, ByVal message As String) As Integer

 If errorCode <> 0 Then
 
     ' show alert if errorCode != 0
     AlertOnError (errorCode)
  
     If Len(message) <> 0 Then
        message = "We hit an error at " + message
        MsgBox message
     End If
     
 End If

End Function


' Initialize instruNet interface code
Public Sub Initialize_instruNet_interface()

    ' Make sure we only do this once per application launch
    Static weHaveAlreadyInitialized As Integer
    
    ' if we are here for the 1st time...
    If weHaveAlreadyInitialized = 0 Then
    
        weHaveAlreadyInitialized = 1
    
        ' Load instruNet driver DLL.  Program will End if this fails.
        Call Load_instruNet_Driver_VB
    
    End If
    
End Sub

 
' Load instruNet Driver and Initialize VB interface utilities
'
' 'demoPolicy' is set to:
'
'   wJumpIntoDemoModeIfNoDeviceExists - only go to demo if not hard exist
'   wDoNotJumpIntoDemoModeIfNoDeviceExists - never enter demo mode
'   wJumpIntoRightNow - jump into demo mode right now.
'
Public Sub Initialize_instruNet_MAIN( _
    ByRef endUserWindow As Variant, _
    ByVal demoPolicy)

    ' -------------------------------------------------------------
    ' Initialize instruNet Interface utilities.
    
    Call Initialize_instruNet_interface

    ' -------------------------------------------------------------
    ' If a network device does not exist, then jump into demo mode.
    
    gSimulatingHardware = False
    
    ' if we are to enter demo mode right now...
    If demoPolicy = wJumpIntoRightNow Then
    
        GoTo JumpIntoDemoMode
        
    ' else if we are to enter demo mode only if there is no hardware...
    ElseIf demoPolicy = wJumpIntoDemoModeIfNoDeviceExists Then
        
        ' calc if we have a network device attached
        weHaveAtLeastOneDevice = False
        If (gNumNetworks >= 1) Then
            If (GET_NUMBER_OF_DEVICES(1) >= 1) Then
                weHaveAtLeastOneDevice = True
            End If
        End If
        
        'if we do not have a device, then jump into demo mode.
        If Not (weHaveAtLeastOneDevice) Then
        
JumpIntoDemoMode:

            ' Fully simulate iNet hardware w/ triangle waves feeding a
            ' fake iNet-100 & iNet-200.  This will reset instruNet.
            Call SET_DEMO_MODE(1)
            
            ' record that simulation is now on
            gSimulatingHardware = True
        
            ' assume we have 1 controller
            gNumNetworks = 1
            
        End If
    
    End If
    
    ' -------------------------------------------------------------
    ' Initialize instruNet Digitize utilities.
    
    Call Digitize_Initialize
    
    ' -------------------------------------------------------------
    ' Initialize instruNet Display utilities.
    
    Call Initialize_instruNet_Displays(endUserWindow)
    
End Sub
' FUNCTION    NotifyWeCannotLocateThisChannel()
'
' PURPOSE     We show an alert stating that we cannot locate
'             this channel on the network.
'
' PARAMETERS  digiChAddr - specifies an instruNet channel
'             via it's {netNum, deviceNum, moduleNum, chanNum}
'             address.
'
' HISTORY     2/25/97  created - gsw
'
Function NotifyWeCannotLocateThisChannel(chAddrP As chAddr) As Integer

    Dim message As String
    Dim e As Integer
    Dim netNum As Integer
    Dim deviceNum As Integer
    Dim moduleNum As Integer
    Dim chanNum As Integer
    
    e = iNetErr_None
    
    ' get the channel address
    Call GetChannelAddress(chAddrP, netNum, deviceNum, moduleNum, chanNum)
    
    ' create a string that says we cannot find this channel.
    message = "We cannot locate channel {"
    message = message + Str$(netNum) + ", "
    message = message + Str$(deviceNum) + ", "
    message = message + Str$(moduleNum) + ", "
    message = message + Str$(chanNum) + "}.  "
    message = message + "Do you think the device is connected?"
    
    ' show the newly created string in an alert box.
    MsgBox message
    
    NotifyWeCannotLocateThisChannel = e

End Function
 
'-------------------------------------------------------------------------------------
' FUNCTION   EndProgram()
'
' PURPOSE    This ends the instruNet program and shows
'            an alert saying why we are doing it if
'            message length is > 0.
'
' HISTORY    2/24/97 created - gsw
'-----------------------------------------------------------------------------------
Sub EndProgram(ByVal message As String)
  
    If Len(message) <> 0 Then
        message = "We are ending the program because " + message
        MsgBox message
    End If
 
    ' if there is a Timer, make sure it is off
    If ValidObject(gDigitizeProcessTimer) Then
        gDigitizeProcessTimer.Enabled = False
    End If
    
    ' close instruNet driver
    CloseDriverAndReleaseDriverRam
    
    ' (end of program here)
    ' (end of program here)
    ' (end of program here)
    End

End Sub
' FUNCTION   Load_instruNet_Driver_VB()
'
' PURPOSE    Loads the instruNet driver.  This must be done before anything else.
'
' HISTORY    2/24/97 created - gsw
'
Public Sub Load_instruNet_Driver_VB()

    Dim driverIsInstalledOK As Integer
    Dim forceInstruNetReset As Integer
    Dim e As Integer
    
    ' if true, this forces instruNet to reset (this is not always necessary
    ' since instruNet automatically resets when the driver is first loaded).
    forceInstruNetReset = iNetTrue
    driverIsInstalledOK = iNetTrue
    
    ' Load instruNet driver and reset the instruNet network.
    e = Load_instruNet_Driver(forceInstruNetReset, driverIsInstalledOK, gNumNetworks)

    ' if no driver installed then stop
    If e <> 0 Or driverIsInstalledOK = iNetFalse Then
        EndProgram ("Cannot Load instruNet Driver (is file 'iNet32.dll' not installed?).")
    End If

    giNetDLLIsLoaded = True
    
End Sub

'-------------------------------------------------------------------------------------
' FUNCTION   iNet_VB_cString()
'
' PURPOSE    This is a wrapper function around iNet_cString().  For
'            a description of this, please see the iNet_VB_cString()
'            function header.
'
' HISTORY    2/24/97 created - gsw
'-----------------------------------------------------------------------------------
Function iNet_VB_cString(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal settingGroupNumOrType, _
    ByVal fieldNum, ByVal intention, arg_VB_string As String) As Integer
                                            
    If intention <> intention_setValue Then ' we assuming we are getting a string
        arg_VB_string = String$(255, 0)     ' create space in memory
    End If

    ' if passing the string into the dll, VB will automatically
    ' append the \0 null char to the end
    iNet_VB_cString = iNet_cString(netNum, deviceNum, moduleNum, chanNum, settingGroupNumOrType, fieldNum, intention, arg_VB_string)

    If intention <> intention_setValue Then

        ' We assume we are getting a string update the VB string prefix
        ' to be the size of the received string.
        
        endChar = InStr(1, arg_VB_string, Chr$(0))
        endChar = endChar - 1              ' take off \0 char
        If (endChar < 0) Then endChar = 0
        If (endChar > 255) Then endChar = 255
        arg_VB_string = Mid$(arg_VB_string, 1, endChar)
        
    End If

End Function
'-------------------------------------------------------------------------------------
' FUNCTION   iNet_VB_GetFieldcStr()
'
' PURPOSE    This is a wrapper function around GetFieldcStr().  For
'            a description of this, please see the GetFieldcStr()
'            function header.
'
' HISTORY    2/24/97 created - gsw
'-----------------------------------------------------------------------------------
Function iNet_VB_GetFieldcStr(ByVal cNum As Integer, ByRef digiChAddr() As chAddr, _
    ByVal settingGroupNumOrType, ByVal fieldNum, arg_VB_string As String) As Integer
                                            
    If intention <> intention_setValue Then ' we assuming we are getting a string
        arg_VB_string = String$(255, 0)     ' create space in memory
    End If

    ' if passing the string into the dll, VB will automatically
    ' append the \0 null char to the end
    e = GetField_cStr(digiChAddr(cNum), settingGroupNumOrType, fieldNum, arg_VB_string)

    If intention <> intention_setValue Then
    
        ' we assume we are getting a string update the VB string prefix
        ' to be the size of the received string
        endChar = InStr(1, arg_VB_string, Chr$(0))
        endChar = endChar - 1              ' take off \0 char
        If (endChar < 0) Then endChar = 0
        If (endChar > 255) Then endChar = 255
        arg_VB_string = Mid$(arg_VB_string, 1, endChar)
        
    End If

End Function

' ----------------------------------------------------------------
' FUNCTION    iNetError_TO_OSErr()
'
' PURPOSE     Converts error of type iNetError to error of type OSError.
'
' HISTORY     2/25/97  created - gsw
'-----------------------------------------------------------------------------------
Function iNetError_TO_OSErr(ByVal iNetErr)

    On Error GoTo iNetErrErr
    
        iNetError_TO_OSErr = (iNetErr - iNetErr_OsBase)
    
    Exit Function

iNetErrErr:

    MsgBox "Problems in iNetErr function"
    Exit Function

End Function


' FUNCTION    ion_Wiring_DEF()
'
' PURPOSE     Returns default wiring type.
'
' HISTORY     2/25/97  created - gsw
'
Function ion_Wiring_DEF(ByVal sensorType)

    On Error GoTo sensorTypeErr

      'Need to turn Get_Default_Wiring_For_This_Sensor into a function of ddl to work
      'ion_Wiring_DEF = Get_Default_Wiring_For_This_Sensor (sensorType)
    
    Exit Function

sensorTypeErr:

    MsgBox "Error in function sensorType"
    Exit Function

End Function


' FUNCTION    OSErr_TO_iNetError()
'
' PURPOSE     Converts error of type OSError to error of type iNetError.
'
' HISTORY     2/25/97  created - gsw
'
Function OSErr_TO_iNetError(ByVal osErr)

    'original 'C'define
    'define OSErr_TO_iNetError(osErr) ((osErr = 0) ? ( iNetErr_General ) : iNetErr_None)
    
    If Error Then GoTo OSErr_TO_iNetErrorErr
        If (osErr = 0) Then
            OSErr_TO_iNetError = iNetErr_None
        Else
            OSErr_TO_iNetError = iNetErr_General
        End If
    Exit Function

OSErr_TO_iNetErrorErr:

    MsgBox "Error in OSErr_TO_iNetError function"

    Exit Function

End Function

