Attribute VB_Name = "Digitize_Utilities"
'************************************************************************
' INET_Digitize.Bas
'
' ------------------ D O C U  M E N T A T I O N ---------------------------
' This file contains code that digitizes from instruNet I/O channels.
'
' -------------------------  H I S T O R Y --------------------------------
' HISTORY  Copyright c 1994-98 GWI.  All rights reserved.
' 7/29/98  created - gsw

' ------------------------------------------------------------------------
' Global declarations

Public gNumScans As Long                    ' # of scans (i.e. buffers) to digitize
Public gNumDigiCh As Integer                ' # of channels to digitize

Public gDigiChan() As chAddr                ' stores a list of digitize channels

Public gPtsPerScan_nextDigi As Long         ' points per scan per channel (next data acq)
Public gPtsPerScan_currentData As Long      ' points per scan per channel (current data)

Public gSampleRate_nextDigi As Double       ' sample rate per channel (pts/sec)(next data acq)
Public gSampleRate_currentData              ' sample rate per channel (pts/sec)(current data)

Public gMaxNumOfDigiChannels As Integer     ' maximum number of digitize channels

Public gContinuousScans As Integer          ' if true, we run with scans that are continous
                                            ' in time; otherwise, we retrigger for each
                        
                                            ' if true, we process a scan when it is finished
                                            ' digitizing; otherwise, we process little segments
                                            ' as they are recorded.
Public gProcessSegmentsInsteadOfScans As Integer

Public gRun_EndUser_PostAcqAnalysis As Integer ' if true, then run EndUser_PostAcqAnalysis()
                                            ' at the end of the digitization.  This is an
                                            ' end user routine that processes the resulting data
                                            
Public gRun_EndUser_SegmentProcessor As Integer ' if true, then run EndUser_SegmentProcessor()
                                            ' while data is being digitized.  This is an end
                                            ' user routine that processes little segments of data.

Public gWaitTillDigitizationCompletes As Integer ' If true, then DigitizeProcess_Start() waits for
                                            ' the entire digitization to complete; otherwise,
                                            ' the Timer control triggers the servicing of the
                                            ' digitize buffers, every 200ms or so, while digitizing.

Public gSimulatingHardware                  ' true if simulation is on

Public gCurrentlyDigitizing As Integer      ' true if we are currently digitizing

                                            ' true if we are to copy the Enabled channels
                                            ' in the Network page to rebuild the Digitize
                                            ' list, after the user closed the Network Window.
Public gCopyChannelsFromNetworkPage As Integer

' "Start"/"Stop" button, set to "Nothing" if doesn't exit
Public gStartStopButton As CommandButton

' Digitize Timer, set to "Nothing" if doesn't exit
Public gDigitizeProcessTimer As Timer
Public gDigitizeProcessCtr As Double

' flags that keep track of buffers as we digitize
Global gWeAreDoingOurFirstAccessOfData As Integer
Global gTheNextDataAccessIsTheLast_ofEntireDigitization As Integer
Global gTheNextDataAccessWillPullToTheEndOfAScan As Integer
Global gControllerIsFinishedWithDigitization As Integer
Global gLast_scanNum_base1_pulled_inFull As Long
Global gLast_scanNum_base1_pushed_inFull As Long

' helps manage digitize buffers
Global gUseDriverRamBuffer As Integer
Global gBufferStartAddrP As Long
Global gNumPointsToPull As Long
Global gPointToPullindex_base1 As Long
Global gScanNumberToPull_base1 As Long

' describes latest segment of digitized data
Global gProcessDataAsItIsAcquired As Integer
Global gSegmentStart_ptNum As Long
Global gSegmentEnd_ptNum As Long

' -----------------------------------------------------
' We now use gDigiChan(cNum).dataArrayY.data(0..numPts) instead of:
'    Global gDigitizeData() As Single

' We now use gDigiChan(cNum).dataArrayY.numValidPts instead of gNumValidPts().

' -----------------------------------------------------
' Ways of massaging data, immediately after it is digitized.

Global Const jNoMassage = 1          ' don't touch the data
Global Const jFFTMassage = 2         ' do an fft on the data, after digitizing

' FUNCTION    VbDataBuffers_LoadWithSineWaves()
'
' PURPOSE     We load the digitize data() arrays with dummy data,
'             for purposes of simulations.
'
' HISTORY     2/25/97  created - unoWarrior
'
Sub VbDataBuffers_LoadWithSineWaves()

    ' load Digitize Data with sine waves
    For cNum = 0 To (gNumDigiCh - 1)
    
        For ptNum = 0 To (gPtsPerScan_nextDigi - 1)
        
            gDigiChan(cNum).dataArrayY.data(ptNum) = Sin(ptNum * Pi / (60 / cNum))
            
        Next ptNum
        
    Next cNum

End Sub

' Services incoming data from digitization process.
Function DigitizeProcess_ServiceIncomingData() As Integer
            
    Dim bNum, dNum, e, chanLinkedToDisplay As Integer
    
    ' -----------------------------------
     ' if we are not currently digitizing, then bail out of here
     
    If Not (gCurrentlyDigitizing) Then
        MsgBox "We cannot service the digitize buffers if we are not digitizing."
        e = 1: GoTo ExitFunction
    End If
   
     ' inc the timer counter
    gDigitizeProcessCtr = gDigitizeProcessCtr + 1
    
    ' -----------------------------------
    ' SERVICE iNet's DIGITIZE BUFFERS
    '
    ' It is ABSOLUTELY NECESSARY to let your computer's processor
    ' service iNet digitizing buffers periodicaly (10 to .3 times
    ' per second) while digitizing.
    
    e = Service_All_iNet_Digitize_Buffers(netNum_1stController, _
            gTheNextDataAccessIsTheLast_ofEntireDigitization, _
            gTheNextDataAccessWillPullToTheEndOfAScan, _
            gControllerIsFinishedWithDigitization, _
            gLast_scanNum_base1_pulled_inFull, _
            gLast_scanNum_base1_pushed_inFull)
    If ErrorHere(e, "ServiceDigitizeBuffers()") Then GoTo ExitFunction
  
    gNumPointsToPull = 0  ' Initialize the number of points to pull out of buffers

    '-----------------------------------
    ' Pull out of the buffer.
    '
    ' If we are to process the data little segments at a time, or
    ' we are processing 1 scan at at time and we have that 1
    ' scan available in the buffers, then go process the data.

    If ((gProcessSegmentsInsteadOfScans <> iNetFalse) Or _
        (gTheNextDataAccessWillPullToTheEndOfAScan <> iNetFalse)) Then
   
        ' Service the channels that have been turned on for digitizing
        For cNum = 0 To (gNumDigiCh - 1)
    
            '-----------------------------------
            ' READ DIGITIZED DATA
            '
            ' Here we get an address in memory where
            ' the most recent digitized data resides.
            '
            ' 'bufferStartAddrP[]'
            '
            ' iNetFLT32 data buffer that is
            ' 1 Scan long
            '
            ' 'pointToPullindex_base1'
            '
            ' new data is available starting at
            ' bufferStartAddrP[pointToPullindex_base1 - 1]
            '
            ' 'numPointsToPull'
            '
            ' number of new data points that are
            ' available
            '
            ' 'scanNumberToPull_base1'
             '
            ' scan number {1...numScans} of the
            ' scan that we are currently accessing.accessing

            ' We use USER_RAM_BUFFER [i.e. Visual Basic array
            ' digitizeData(ptNum, cNum)] instead of DRIVER_RAM_BUFFER.
            
            gUseDriverRamBuffer = iNetFalse
 
            e = Access_Digitized_Data_In_Ram_Buffer(gDigiChan(cNum), _
                gUseDriverRamBuffer, gBufferStartAddrP, gPointToPullindex_base1, _
                gNumPointsToPull, gScanNumberToPull_base1)
            
            If ErrorHere(e, "AccessDigitizeData()") Then GoTo ExitFunction

            ' test
            If (iNet_Get_VarPtr(gDigiChan(cNum).dataArrayY.data(0)) <> gBufferStartAddrP) Then
                MsgBox "Hey, we have an error at Access_Digitized_Data_In_Ram_Buffer(). Did the VB array move after we gave it's address to the instruNet driver?"
                GoTo ExitFunction
            End If
  
            ' ---------------------------------------------------------------
            ' calc indices into array of data where we just pulled in new data (i.e. we
            ' process little segments as they are digitized.
            
            gSegmentStart_ptNum = gPointToPullindex_base1 - 1
            gSegmentEnd_ptNum = gSegmentStart_ptNum + gNumPointsToPull - 1
            
            ' update the # of valid data pts that are currently in the data buffer.
            gDigiChan(cNum).dataArrayY.numValidPts = gSegmentEnd_ptNum + 1
           
            ' determine if this channel is linked to a display
            bNum = gDigiChan(cNum).bNum
            dNum = gDigiChan(cNum).dNum
            chanLinkedToDisplay = bNum >= 0 And dNum >= 0
            
            ' ---------------------------------------------------------------
            ' DO FFT's AS NEEDED.
           
            ' if we finished the scan and are massaging an fft...
            If (gTheNextDataAccessWillPullToTheEndOfAScan <> False) And _
                (gDigiChan(cNum).massageType = jFFTMassage) Then

                ' calc the # of pts (numPts = 2 ^ power)
                power& = Int(Log(gSegmentEnd_ptNum + 1) / Log(2))
                
                ' do an fft on the digitize data, in place; and update
                ' the data array's numValidPts/ptsPerXunit fields.
                Dim numCalculatedMagnitudes As Long
                Call FFT_DataArray(gDigiChan(cNum).dataArrayY, power&, _
                    numCalculatedMagnitudes, gSampleRate_currentData)
               
           End If
            
            ' ---------------------------------------------------------------
            ' RUN END USER DATA PROCESSING ROUTINE
            
            If gDigiChan(cNum).runEndUser_SegmentProcessor And _
               gRun_EndUser_SegmentProcessor Then
            
                ' Run the user's waveform processing routine (inside
                ' the user's source code file).
                Call gEndUserWindow.EndUser_SegmentProcessor( _
                    cNum, gSegmentStart_ptNum, gSegmentEnd_ptNum, _
                    scanNumberToPull_base1)
            
            End If

            ' ---------------------------------------------------------------
            ' CLEAR DISPLAYS AS NEEDED.
            
            ' if this data is tied to a display...
            If chanLinkedToDisplay = True Then
                
                ' if we are to clear the display right before plotting...
                If gBox(bNum).disp(dNum).clrBeforePlot = True Then
                
                    ' clear the plot region, and give it a new grid
                    Call Plot_Clear_OneDisp(bNum, dNum, gBox(bNum).BoxObj)
                    
                    Call Grid_Draw_OneDisp(bNum, dNum, gBox(bNum), _
                        gBox(bNum).disp(dNum), gBox(bNum).BoxObj)
                
                End If
                
                ' if we finished the scan, then clear & apply a grid right
                ' before we next plot...
                If (gTheNextDataAccessWillPullToTheEndOfAScan <> False) And _
                    gContinuousScans = False And _
                    gTheNextDataAccessIsTheLast_ofEntireDigitization = False Then
                
                    gBox(bNum).disp(dNum).clrBeforePlot = True
                
                Else
                    gBox(bNum).disp(dNum).clrBeforePlot = False
                End If
                
            End If
            
            ' ---------------------------------------------------------------
            ' PLOT DATA AS NEEDED
            
            ' if this data is tied to a display...
            If chanLinkedToDisplay = True Then
            
                ' If we are to process little segments, then plot this little segment.
                If (gProcessSegmentsInsteadOfScans And _
                    (gDigiChan(cNum).massageType <> jFFTMassage)) Then
                
                    ' Plot entire display, from left to right edges
                    Call Plot_Segment_OneDisp(bNum, dNum, gBox(bNum), _
                        gBox(bNum).disp(dNum), gSegmentStart_ptNum, _
                        gSegmentEnd_ptNum, gBox(bNum).BoxObj)
                    
                Else
                
                    ' if we finished the scan...
                    If (gTheNextDataAccessWillPullToTheEndOfAScan <> False) Then
                    
                        ' Plot entire display, from left to right edges
                        Call Plot_Segment_OneDisp(bNum, dNum, gBox(bNum), _
                            gBox(bNum).disp(dNum), 0, _
                            (gDigiChan(cNum).dataArrayY.numValidPts - 1), _
                            gBox(bNum).BoxObj)
                            
                    End If
                    
                End If
            
            End If
                        
        ' We process data one channel at a time.
        Next cNum

    'This "End If" Matches: If ((processOneScanAtATime = iNetFalse) Or (theNextDataAccessWillPullToTheEndOfAScan = iNetTrue)) Then
    End If
    
    ' ----------------------------------------------------------------
    ' IF AT END OF ENTIRE DIGITIZATION...
    
    ' If we are at the end of the entire digitization, then power down
    ' the digitizing.
    
    If gTheNextDataAccessIsTheLast_ofEntireDigitization <> iNetFalse Then
    
        Call DigitizeProcess_Stop            ' shut down the digitizing
        
    End If
    
    ' ----------------------------------------------------------------
    ' Exit
    
ExitFunction:
    
    DigitizeProcess_ServiceIncomingData = e

End Function



' User just pressed the Channel Setup button, and we need
' to open the instruNet World network window.
Public Sub METHOD_Channels_Button_Click()

    ' if we are currently digitizing, then don't allow one to
    ' open this window.
    If gCurrentlyDigitizing Then
        MsgBox "We cannot open this window while digitizing."
        Exit Sub
    End If
    
    ' Disable all channels selected in the instruNet World Network page
    ' (this kills the DRIVER RAM buffers).
    e = DisableAllChannelsForDigitizing()
    If ErrorHere(e, "mcb1") Then Exit Sub
    
    ' Select in the instruNet World Network window the channels in
    ' the DigiChan() list.
    Call VbDataBuffers_CopyToDLLDvrBuffers
    
    ' Open the instruNet World network page.
    Open_instruNet_Window_VB (iNetCM_OpenWindow_Network)

    ' If we are to copy the Enabled channels in the Network page
    ' to rebuild the Digitize list, after the user closed the
    ' Network Window.
    If gCopyChannelsFromNetworkPage Then
           
        ' Disable all channels in the DigiChan() list, and then load it with
        ' the channels Selected in the instruNet World network page.
        massageType = jNoMassage
        Call DigitizeList_CopyFromDriverDLL(massageType, gRun_EndUser_SegmentProcessor)

        ' Automatically add channels to DisplayBoxes that are set up to do this.
        Dim bNum As Integer
        For bNum = 0 To gNumBoxes - 1   ' scan through all DisplayBoxes
            
            ' if we are automatically add channels to this box...
            If (gBox(bNum).autoAddPolicy <> hAutoAddNone) Then
            
                ' automatically add Digitize Channels to this display box
                AutoAddChannelsToThisDisplayBox (bNum)
                
                ' Clear 1 DispBox and then Redraw it
                Call Redraw_OneBox(bNum, gBox(bNum).BoxObj)

            End If
            
        Next bNum
        
    End If
    
    ' Disable all DigitizeChannels inside the iNet driver
    ' (kills the DRIVER RAM buffers)
    e = DisableAllChannelsForDigitizing()
    If ErrorHere(e, "mcb2") Then Exit Sub
    
End Sub


' Sets the digitize timing parameters.
Public Sub DigitizeProcess_SetTimingParams( _
    ByVal ptsPerScan As Long, _
    ByVal numScans As Long, _
    ByVal sampleRate As Double, _
    ByVal continousScans As Integer, _
    ByVal processSegmentsInsteadOfScans As Integer, _
    ByVal run_EndUser_PostAcqAnalysis As Integer, _
    ByVal run_EndUser_SegmentProcessor As Integer, _
    ByVal copyChannelsFromNetworkPage As Integer, _
    ByVal waitTillDigitizationCompletes As Integer)

    gNumScans = numScans                    ' # of scans to digitize
    
    gContinuousScans = continousScans       ' if true, we run with scans that are continous
                                            ' in time; otherwise, we retrigger for each    If stripChartRecorder <> 0 Then stripChartRecorder = True

                                            ' if true, we process a scan when it is finished
                                            ' digitizing; otherwise, we process little segments
                                            ' as they are recorded.
    If processSegmentsInsteadOfScans <> 0 Then processSegmentsInsteadOfScans = True
    gProcessSegmentsInsteadOfScans = processSegmentsInsteadOfScans
                                                
                                            ' if true, then run EndUser_PostAcqAnalysis()
                                            ' at the end of the digitization.  This is an
                                            ' end user routine that processes the resulting data
    If run_EndUser_PostAcqAnalysis <> 0 Then run_EndUser_PostAcqAnalysis = True
    gRun_EndUser_PostAcqAnalysis = run_EndUser_PostAcqAnalysis
    
                                            ' if true, then run EndUser_SegmentProcessor()
                                            ' while data is being digitized.  This is an end
                                            ' user routine that processes little segments of data.
    If run_EndUser_SegmentProcessor <> 0 Then run_EndUser_SegmentProcessor = True
    gRun_EndUser_SegmentProcessor = run_EndUser_SegmentProcessor
    
                                            ' If true, then DigitizeProcess_Start() waits for
                                            ' the entire digitization to complete; otherwise,
                                            ' the Timer control triggers the servicing of the
                                            ' digitize buffers, every 200ms or so, while digitizing.
    If waitTillDigitizationCompletes <> 0 Then waitTillDigitizationCompletes = True
    gWaitTillDigitizationCompletes = waitTillDigitizationCompletes
   
    gPtsPerScan_nextDigi = ptsPerScan       ' points per scan per channel
    gPtsPerScan_currentData = 0             ' start w/ no data in ram
     
    gSampleRate_nextDigi = sampleRate       ' sample rate per channel (pts/sec)
    gSampleRate_currentData = gSampleRate_nextDigi
    
                                            ' true if we are to copy the Enabled channels
                                            ' in the Network page to rebuild the Digitize
                                            ' list, after the user closed the Network Window.
    If copyChannelsFromNetworkPage <> 0 Then copyChannelsFromNetworkPage = True
    gCopyChannelsFromNetworkPage = copyChannelsFromNetworkPage
    
End Sub

' Initialize instruNet digitize code
Public Sub Digitize_Initialize()

   ' Initialize digitization parameters to default states
    '
    ' If you want these to be different, then define them
    ' differently after calling In Initialize_instruNet_MAIN().
    
    gNumDigiCh = 0                      ' # of channels in the Digitize
                                        ' Channel List. This increments
                                        ' when you call AddToDigitzeChannelList()
   
    gNumScans = 1                       ' # of scans to digitize
    
    gContinuousScans = False            ' if true, we run with scans that are continous
                                        ' in time; otherwise, we retrigger for each
     
    gProcessSegmentsInsteadOfScans = True ' if false, we process a scan when it is finished
                                        ' digitizing; otherwise, we process little segments
                                        ' as they are recorded.
    
    gPtsPerScan_nextDigi = 1000         ' points per scan per channel
    gPtsPerScan_currentData = 0         ' start w/ no data in ram
     
    gSampleRate_nextDigi = 10000        ' sample rate per channel (pts/sec)
    gSampleRate_currentData = gSampleRate_nextDigi
 
    gMaxNumOfDigiChannels = 128         ' max # of digitize channels
    
                                        ' stores a list of digitize channels
    ReDim Preserve gDigiChan(gMaxNumOfDigiChannels) As chAddr
    
    gCurrentlyDigitizing = False        ' not currently digitizing
    
    gDigitizeProcessCtr = 0                   ' timer counter
    
 End Sub
 
' Add a channel to the list of channels to digitize.
'
' 'massageType' is set to {jNoMassage, jFFTMassage} and indicates how
' the data is to be massaged immediately after it is digitized.
'
Public Function DigitizeList_AddOneItem( _
    ByVal netNum As Integer, _
    ByVal devNum As Integer, _
    ByVal modNum As Integer, _
    ByVal chanNum As Integer, _
    ByVal massageType As Integer, _
    ByVal runEndUser_SegmentProcessor As Integer) As Integer
    
    Dim cNum_new As Integer
    cNum_new = gNumDigiCh
    gNumDigiCh = gNumDigiCh + 1
    
    ' ------------------------------------------------
    ' ReAllocate memory for the array that describes the channels.
    
    ReDim Preserve gDigiChan(gNumDigiCh)
        
    ' ------------------------------------------------
    ' Set channel parameters
    
    With gDigiChan(cNum_new)
                                        ' remember the address in these VB fields
        .netNum = netNum
        .deviceNum = devNum
        .moduleNum = modNum
        .chanNum = chanNum
                                            
        .bNum = -1                ' we are not tied to a display at this time
        .dNum = -1
                                        ' if true, then the digitize code calls
                                        ' EndUser_SegmentProcessor() each time a little
                                        ' segment of data is digitized.
        .runEndUser_SegmentProcessor = runEndUser_SegmentProcessor
    
        .massageType = massageType      ' 'massageType' is set to {jNoMassage, jFFTMassage} and
                                        ' indicates how the data is to be massaged immediately
                                        ' after it is digitized.
        
                                        ' Resets the data(), numPts, and sRate parameters
                                        ' for a digitize channel, resizes the data() array,
                                        ' and zeros it.
        DigitizeList_ResetDataArray (cNum_new)
    
                                        ' load channel into the DigiCh() list
        chExists = LoadChannelAddress_VB(gDigiChan(cNum_new), netNum, devNum, modNum, chanNum)
    
        If chExists = iNetFalse Then    ' if channel does not exists, holler
            e = NotifyWeCannotLocateThisChannel(gDigiChan(gNumDigiCh))
            DigitizeList_AddOneItem = False
        Else
            DigitizeList_AddOneItem = True  ' channel exists ok
        End If
        
    End With
    
End Function

' Resets the data(), numPts, and sRate parameters for a digitize channel,
' resizes the data() array, and zeros it.  This function does this
' based on gPtsPerScan_nextDigi and gSampleRate_nextDigi values
Public Sub DigitizeList_ResetDataArray(ByVal cNum)
                
    ' ------------------------------------------------
    ' Set channel parameters.
    
    With gDigiChan(cNum).dataArrayY
         
        ' if we pull in a timewaveform...
        If gDigiChan(cNum).massageType = jNoMassage Then
        
            ' set the sample rate for this data
            .ptsPerXunit = gSampleRate_nextDigi
            
            ' first point time in seconds
            .firstPt_Xunits = 0
            
            ' no of valid pts in the buffer
            .numValidPts = gPtsPerScan_nextDigi
                                                                               
            ' buffer size
            .numBufferPts = gPtsPerScan_nextDigi + 2
        
        ' else if we do an fft immediately after digitizing...
        ElseIf gDigiChan(cNum).massageType = jFFTMassage Then
        
            ' calc the # of pts (numPts = 2 ^ power)
            power& = Int(Log(gPtsPerScan_nextDigi) / Log(2))
            
            ' update the # of resulting freq amplitude points
            numCalculatedMagnitudes& = (2 ^ power&) / 2
            
            ' update the pts-per Hz unit (we are plotting amplitudes vs. Hz).
            .ptsPerXunit = numCalculatedMagnitudes& / (gSampleRate_currentData / 2)
            
            ' first point time in seconds
            .firstPt_Xunits = 0
            
            ' no of valid pts in the buffer
            .numValidPts = numCalculatedMagnitudes&
                                                                               
            ' buffer size
            .numBufferPts = gPtsPerScan_nextDigi + 2
            
        Else
        
            EndProgram ("bad messageType")
            
        End If
         
        ' ------------------------------------------------
        ' ReAllocate memory for the digitized data.
        '
        ' Digitize data() contains the digitized data in the form:
        '   gDigiChan(0...gNumDigiCh).dataArrayY.data(0..numPts)
    
        ReDim Preserve .data(.numBufferPts) As Single
    
        ' ------------------------------------------------
        ' Zero the digitize data array.
        
        last_ptNum& = .numBufferPts - 1
        For ptNum& = 0 To last_ptNum&
            .data(ptNum&) = 0
        Next ptNum&
    
    End With
    
End Sub

' Resets the number of items in the list to 0
Public Sub DigitizeList_ResetList()
    
    ' delete all the data and reset the number of items in the
    ' Digitize Channel list to 0
    ReDim gDigiChan(0)
    
    ' establish there are no items in the list
    gNumDigiCh = 0
    
End Sub
' Responds to an interrupt from the timer
Sub METHOD_DigitizeProcess_Timer()

    ' takes care of incoming data
    DigitizeProcess_ServiceIncomingData
    
End Sub

' Sets the StartStopButton and DigitizeProcessTimer
Public Sub DigitizeProcess_SetControlItems( _
    ByRef StartStopButton As CommandButton, _
    ByRef digitizeProcessTimer As Timer)

    ' "Start"/"Stop" button, set to "Nothing" if doesn't exist
    Set gStartStopButton = StartStopButton
    
    ' Digitize Timer, set to "Nothing" if doesn't exit
    Set gDigitizeProcessTimer = digitizeProcessTimer

End Sub
' Loads an instruNet channel address into a digiChAddr() structure.
' This structure holds the address in a vb format, and a format
' that is understood by the instruNet DLL.
'
Public Function LoadChannelAddress_VB( _
    ByRef digiChAddr As chAddr, _
    ByRef netNum As Integer, _
    ByRef devNum As Integer, _
    ByRef modNum As Integer, _
    ByRef chanNum As Integer) As Integer
    
    Dim chExists As Integer

    ' load channel address into a LONG, to be used by the instruNet DLL
    chExists = LoadChannelAddress(digiChAddr, netNum, devNum, modNum, chanNum)
    
    LoadChannelAddress_VB = chExists
    
    ' load channel address into several VB fields, to be used by VB
    digiChAddr.netNum = netNum
    digiChAddr.moduleNum = modNum
    digiChAddr.deviceNum = devNum
    digiChAddr.chanNum = chanNum
    
End Function



'This handles a mouse down on the Digitize "Start"/"Stop" button.
Public Sub METHOD_StartStopButton_Click()
    
    If gCurrentlyDigitizing Then            ' if we are currently digitizing...
        DigitizeProcess_Stop                ' tell instruNet to stop digitizing
    Else                                    ' Else we are not digitizing, and need to
        DigitizeProcess_Start               ' tell instruNet to start digitizing
    End If

End Sub

' ----------------------------------------------------------------
' FUNCTION    Digitize_iNet_Data()
'
' PURPOSE     This routine digitizes one scan of data, and then returns
'             control back to the caller.  The digitization is based
'             on the following passed arguments.
'
' PARAMETERS  processOneScanAtATime
'                   If true (1), we access digitized data 1 complete
'                   Scan at a time; otherwise if FALSE (0), we process
'                   little segments of data, as they become available.
'
'             ptsPerScan
'                   Number of points per scan.
'
'             ptsPerSecPerChan
'                   Sample rate in Points-Per-Second-Per-Channel
'
'             numScans
'                   number of scans to acquire
'
'             numDigiCh
'                   number of channels to digitize
'
'             stripChartRecorder
'                   if TRUE (1) we do Strip Chart Recorder work with
'                   Continuous Scans, otherwise if FALSE (0) we do
'                   Oscilloscope mode (non-Coninuous Scans)
'
'             digiChAddr(0...numDigiCh-1)
'                   When this routine is called, this must contain
'                   a list of the channels to digitize.
'
'                   Digitize_iNet_Data() loads the digitize data into:
'                   digiChAddr(0...numDigiCh-1).dataArrayY.data(0...ptsPerScan)
'
' HISTORY     2/2/96  created - gsw
'             2/20/96  created Visual Basic version - sjm
' ----------------------------------------------------------------

Function Digitize_iNet_Data( _
    ByVal processOneScanAtATime As Integer, _
    ByVal ptsPerScan As Long, _
    ByVal ptsPerSecPerChan As Long, _
    ByVal numScans As Long, _
    numDigiCh As Integer, _
    ByVal stripChartRecorder As Integer, _
    digiChAddr() As chAddr) As Integer

' -------------------------------------------------------
' Local varibles

Dim message As String
Dim e As Integer
Dim userBufferSize As Long
Dim theNextDataAccessIsTheLast_ofEntireDigitization As Integer
Dim bufferStartAddrP As Long
Dim weAreDoingOurFirstAccessOfData As Integer
Dim numPointsToPull As Long
Dim scanNumberToPull_base1 As Long
Dim firstPoint, lastPoint As Single
Dim theNextDataAccessWillPullToTheEndOfAScan As Integer
Dim showAlertWas_ON_BeforeThisCall As Integer
Dim last_scanNum_base1_pulled_inFull As Long
Dim last_scanNum_base1_pushed_inFull As Long
Dim controllerIsFinishedWithDigitization As Integer
Dim cNum As Integer
Dim differenceBetweenAdjacentPoints, thisPoint, previousPoint As Single
Dim triggerChannel As chAddr
Dim chExists As Boolean
Dim addressOfBuffer As Long
Dim sumOfPoints_fast As Double
Dim firstPoint_Of_LittleSegment As Long
Dim pointToPullindex_base1 As Long
Dim netNum As Integer
Dim deviceNum As Integer
Dim moduleNum As Integer
Dim chanNum As Integer

' -----------------------------------------------------------------------
' Make sure the caller gave us a list of channels to digitize.

e = iNetErr_None
If numDigiCh = 0 Then   ' if we have no channels to digitize...
    MsgBox ("No channels have been selected to digitize.")
    GoTo ExitFunction
End If

' Convert to instruNet DLL 1/0 true/false format.
If stripChartRecorder <> 0 Then stripChartRecorder = True
If processOneScanAtATime <> 0 Then processOneScanAtATime = True

' -----------------------------------------------------------------------
' Check the list of digitize channels, and make sure they are ok.  The
' caller passes us this list inside 'digiChAddr()'.

For cNum = 0 To numDigiCh - 1

    ' get the channel address stored inside 'digiChAddr()'
    Call GetChannelAddress(digiChAddr(cNum), netNum, deviceNum, moduleNum, chanNum)

    ' see if the channel exists
    chExists = THIS_CHANNEL_EXISTS(netNum, deviceNum, moduleNum, chanNum)

    If chExists = iNetFalse Then
        e = NotifyWeCannotLocateThisChannel(digiChAddr(cNum))
        Exit Function
    End If

Next

' -----------------------------------------------------------------------
' Allocate a data() array for the digitized data.

For cNum = 0 To numDigiCh - 1
    ReDim Preserve gDigiChan(cNum).dataArrayY.data(ptsPerScan + 2)
Next cNum

' -----------------------------------------------------------------------
' Enabled digitizing for the selected channels and prepare data buffer.

e = VbDataBuffers_AttachToDLLUserBuffers(numDigiCh, digiChAddr(), ptsPerScan)
If ErrorHere(e, "Tell_Driver_To_Put_Digitize_Data_into_VB_Arrays()") Then GoTo ExitFunction

' -----------------------------------------------------------------------
' Set TIMING parameters.  Tell it to digitize 1000 points per Scan at 100
' points per second per channel (i.e. 10sec Scans) and do this
' 3 times (i.e. 3 Scans).  We also tell iNet to put the digitized data
' into the DRIVER ram buffer (we will get access to this buffer as the
' data is digitized).  And we tell iNet to show an alert if we
' overflow a data buffer.  stripChartRecorder mode is turned ON, which
' means that the Scans will be continuous (last point of one Scan is directly
' followed by first point of next Scan).

showAlertOnOverflow = iNetTrue
switchChannelsAccurately = iNetTrue

                        ' We are setting the timing parameters in here.
e = Set_iNet_TIMING_Parameters(ByVal ptsPerSecPerChan, ByVal ptsPerScan, numScans, _
    stripChartRecorder, ion_gSaveDataPopup_ToRamBuffer, showAlertOnOverflow, _
    switchChannelsAccurately)
 
If ErrorHere(e, "Set_iNet_TIMING_Parameters()") Then GoTo ExitFunction

' -----------------------------------------------------------------------
' Set TRIGGER parameters.  We specify the trigger source to be the
' channel specified in the 'triggerChannel' address. We also say that the
' channel is set to trigger on the rising edge of the source when it
' crosses 0.0.  At the moment, the trigger function is deactivated
' (i.e. ion_TriggerMode_Off).

setupTrigger = iNetFalse

If setupTrigger = iNetTrue Then
    
    'Vh1 Vin+ is the trigger channel
    chExists = LoadChannelAddress_VB(triggerChannel, 1, 1, 1, 1)

    triggerThreshold = 0#
    
    ' We are setting the trigger to be OFF in here.
    e = Set_iNet_TRIGGER_Parameters(ByVal ion_TriggerMode_Off, triggerThreshold, ByVal ion_Rising, 0, triggerChannel)
    If ErrorHere(e, "Set_iNet_TRIGGER_Parameters()") Then GoTo ExitFunction

End If

'-----------------------------------------------------------------------
' Tell iNet driver to show an alert if there is an error (in addition
' to returning an error code).
    
showAlertWas_ON_BeforeThisCall = SHOW_MESSAGE_DIALOG_UPON_ERROR()

' -----------------------------------------------------------------------
' START DIGITIZING

' Press the start button to tell iNet to begin digitizing.
e = PRESS_iNet_BUTTON(iNetCM_BtnPress_Record_Start)
If ErrorHere(e, "StartDigitizeCommand") Then GoTo ExitFunction
          
' say that we are starting out
weAreDoingOurFirstAccessOfData = iNetTrue
theNextDataAccessIsTheLast_ofEntireDigitization = iNetFalse

Do  ' loop until the digitization is finished..
 
    '-----------------------------------
    ' SERVICE iNet's DIGITIZE BUFFERS
    '
    ' It is ABSOLUTELY NECESSARY to let your computer's processor
    ' service iNet digitizing buffers periodicaly (10 to .3 times
    ' per second) while digitizing.

    e = Service_All_iNet_Digitize_Buffers(netNum_1stController, _
        theNextDataAccessIsTheLast_ofEntireDigitization, _
        theNextDataAccessWillPullToTheEndOfAScan, controllerIsFinishedWithDigitization, _
        last_scanNum_base1_pulled_inFull, last_scanNum_base1_pushed_inFull)
        
    If ErrorHere(e, "ServiceDigitizeBuffers()") Then GoTo ExitFunction
  
    numPointsToPull = 0  ' Initialize the number of points to pull out of buffers

    '-----------------------------------
    ' Pull out of the buffer.
    '
    ' If we are to process the data little segments at a time, or
    ' we are processing 1 scan at at time and we have that 1
    ' scan available in the buffers, then go process the data.

    If ((processOneScanAtATime = iNetFalse) Or (theNextDataAccessWillPullToTheEndOfAScan = iNetTrue)) Then
   
        ' service the channels that have been turned on for digitizing
        For cNum = 0 To (numDigiCh - 1)
    
          '-----------------------------------
          ' READ DIGITIZED DATA
          '
          ' Here we get an address in memory where
          ' the most recent digitized data resides.
          '
          ' 'bufferStartAddrP[]'
          '
          ' iNetFLT32 data buffer that is
          ' 1 Scan long
          '
          ' 'pointToPullindex_base1'
          '
          ' new data is available starting at
          ' bufferStartAddrP[pointToPullindex_base1 - 1]
          '
          ' 'numPointsToPull'
          '
          ' number of new data points that are
          ' available
          '
          ' 'scanNumberToPull_base1'
          '
          ' scan number {1...numScans} of the
          ' scan that we are currently accessing.accessing

          ' We use USER_RAM_BUFFER [i.e. Visual Basic array
          ' digitizeData(ptNum, cNum)] instead of DRIVER_RAM_BUFFER.
          useDriverRamBuffer = iNetFalse
 
          e = Access_Digitized_Data_In_Ram_Buffer(digiChAddr(cNum), useDriverRamBuffer, bufferStartAddrP, pointToPullindex_base1, numPointsToPull, scanNumberToPull_base1)
          If ErrorHere(e, "AccessDigitizeData()") Then GoTo ExitFunction

          ' test
          If (iNet_Get_VarPtr(gDigiChan(cNum).dataArrayY.data(0)) <> bufferStartAddrP) Then
            MsgBox "Hey, we have an error at Access_Digitized_Data_In_Ram_Buffer(). Did the VB array move after we gave it's address to the instruNet driver?"
            GoTo ExitFunction
          End If
  
          '---------------------------------------------------------------
          ' Process incoming data, as it is acquired, in little segments.
          
          processDataAsItIsAcquired = False
          
          If processDataAsItIsAcquired = True Then

                                ' calc indices into array of data where
                                ' we just pulled in new data (i.e. we
                                ' process little segments as they are digitized.
               firstPoint_Of_LittleSegment = pointToPullindex_base1 - 1
               lastPoint_Of_LittleSegment = firstPoint_Of_LittleSegment + numPointsToPull - 1

                                ' scan through each point in the little segment
               For ptNum = firstPoint_Of_LittleSegment To lastPoint_Of_LittleSegment
                  currentValue = gDigiChan(cNum).dataArrayY.data(ptNum)
               Next ptNum

          End If
         
        ' We process data one channel at a time.
        Next cNum

    'This "End If" Matches: If ((processOneScanAtATime = iNetFalse) Or (theNextDataAccessWillPullToTheEndOfAScan = iNetTrue)) Then
    End If

' We sit in loop while there is data to be pulled out of the buffers;
' if someone presses the STOP button, "theNextDataAccessIsTheLast"
' is set to iNetTrue, and this ends the digitizing.
Loop Until theNextDataAccessIsTheLast_ofEntireDigitization <> iNetFalse

' ----------------------------------------------------------------
' Exit

ExitFunction:

    ' Now that we have finished digitizing, tell the instruNet driver
    ' to forget about these VB memory buffers that we let it use
    ' for digitization (this insures that the instruNet driver does not
    ' try to use them later on, when VB may have moved this).  This
    ' does not effect the DigitizeData() in VB memory.
    
    e = VbDataBuffers_DetachFromDLLUserBuffers(numDigiCh, digiChAddr())
    
    Digitize_iNet_Data = e

End Function




' FUNCTION    DigitizeProcess_Start()
'
' PURPOSE     This routine starts the digitizing process, as it had
'             been set up by:
'
'                   DigitizeList_AddOneItem()
'                   DigitizeProcess_SetTimingParams()
'
' HISTORY     8/4/98  created - gsw
'
Function DigitizeProcess_Start() As Integer
    
    ' -------------------------------------------
    ' if we are currently digitizing, then bail out of here.
    
    If (gCurrentlyDigitizing) Then
        MsgBox "You cannot start digitizing while you are digitizing."
        Exit Function
    End If
    
    gDigitizeProcessCtr = 0                   ' timer counter
   
    ' -------------------------------------------
    ' Make sure the caller gave us a list of channels to digitize.
    
    Dim e As Integer
    e = iNetErr_None
    DigitizeProcess_Start = e
    
    If gNumDigiCh = 0 Then   ' if we have no channels to digitize...
        MsgBox ("No channels have been selected to digitize.")
        e = 1
        GoTo ExitFunction
    End If
    
    ' -------------------------------------------
    ' Check the list of digitize channels, and make sure they are ok.  The
    ' caller passes us this list inside 'digiChAddr()'.
    
    Dim cNum, chExists As Integer
    Dim netNum As Integer
    Dim deviceNum As Integer
    Dim moduleNum As Integer
    Dim chanNum As Integer
    
    For cNum = 0 To gNumDigiCh - 1
    
        ' get the channel address stored inside 'digiChAddr()'
        Call GetChannelAddress(gDigiChan(cNum), netNum, deviceNum, moduleNum, chanNum)
    
        ' see if the channel exists
        chExists = THIS_CHANNEL_EXISTS(netNum, deviceNum, moduleNum, chanNum)
    
        If chExists = iNetFalse Then
            e = NotifyWeCannotLocateThisChannel(gDigiChan(cNum))
            GoTo ExitFunction
        End If
    
    Next

    ' -------------------------------------------
    ' Measures the actual sample rate.

    gSampleRate_nextDigi = DigitizeProcess_MeasureActualSampleRate()
    gSampleRate_currentData = gSampleRate_nextDigi
    
    ' Update our understanding of the data
    gPtsPerScan_currentData = gPtsPerScan_nextDigi
              
    ' -------------------------------------------
    ' Update the data 'numValidPts' and 'ptsPerXunit' field, and the
    ' data buffer sizes.  The reason we do this now, is so that the resetting
    ' of the controls will match the new data.
    
    For cNum = 0 To gNumDigiCh - 1
    
        ' Resets the data(), numPts, and sRate parameters for a digitize channel,
        ' resizes the data() array, and zeros it.
        DigitizeList_ResetDataArray (cNum)
        
    Next
    
    ' Update the horiz pts-per-xUnits (i.e. pts per pixel)
    Reset_Controls_AllBoxes (reset_ptsPerHscale)
    
    ' -------------------------------------------
    ' Enabled digitizing for the selected channels and prepare data buffer.
    ' This routine tells the instruNet DLL to digitize into the VB digitize data()
    ' buffers, and makes it very easy for VB code to access this data.

    e = VbDataBuffers_AttachToDLLUserBuffers(gNumDigiCh, gDigiChan(), gPtsPerScan_nextDigi)
    If ErrorHere(e, "VbDataBuffers_AttachToDLLUserBuffers()") Then GoTo ExitFunction

    ' -------------------------------------------
    ' Set TIMING parameters.  Tell it to digitize 1000 points per Scan at 100
    ' points per second per channel (i.e. 10sec Scans) and do this
    ' 3 times (i.e. 3 Scans).  We also tell iNet to put the digitized data
    ' into the DRIVER ram buffer (we will get access to this buffer as the
    ' data is digitized).  And we tell iNet to show an alert if we
    ' overflow a data buffer.  stripChartRecorder mode is turned ON, which
    ' means that the Scans will be continuous (last point of one Scan is directly
    ' followed by first point of next Scan).
    
    Dim showAlertOnOverflow As Integer
    Dim switchChannelsAccurately As Integer
    
    showAlertOnOverflow = iNetTrue
    switchChannelsAccurately = iNetTrue
    
    ' We are setting the timing parameters in here.
    e = Set_iNet_TIMING_Parameters(ByVal gSampleRate_nextDigi, _
            ByVal gPtsPerScan_nextDigi, ByVal gNumScans, ByVal gContinuousScans, _
            ByVal ion_gSaveDataPopup_ToRamBuffer, _
            showAlertOnOverflow, switchChannelsAccurately)
     
    If ErrorHere(e, "Set_iNet_TIMING_Parameters()") Then GoTo ExitFunction

    ' -------------------------------------------
    ' Set TRIGGER parameters.  We specify the trigger source to be the
    ' channel specified in the 'triggerChannel' address. We also say that the
    ' channel is set to trigger on the rising edge of the source when it
    ' crosses 0.0.  At the moment, the trigger function is deactivated
    ' (i.e. ion_TriggerMode_Off).
    
    Dim triggerChannel As chAddr
    Dim setupTrigger As Integer
    setupTrigger = False
    
    If setupTrigger <> False Then
    
        'Vh1 Vin+ is the trigger channel
        chExists = LoadChannelAddress_VB(triggerChannel, 1, 1, 1, 1)
    
        triggerThreshold = 0#
        
        ' We are setting the trigger to be OFF in here.
        e = Set_iNet_TRIGGER_Parameters(ByVal ion_TriggerMode_Off, triggerThreshold, _
                ByVal ion_Rising, 0, triggerChannel)
            
        If ErrorHere(e, "Set_iNet_TRIGGER_Parameters()") Then GoTo ExitFunction
    
    End If

    ' -------------------------------------------
    ' Tell iNet driver to show an alert if there is an error (in addition
    ' to returning an error code).

    Dim showAlertWas_ON_BeforeThisCall As Integer
    showAlertWas_ON_BeforeThisCall = SHOW_MESSAGE_DIALOG_UPON_ERROR()

    ' -------------------------------------------
    ' START DIGITIZING

    ' Press the start button to tell iNet to begin digitizing.
    e = PRESS_iNet_BUTTON(iNetCM_BtnPress_Record_Start)
    If ErrorHere(e, "BtnPress_Record_Start") Then GoTo ExitFunction

    ' say that we are starting out
    gWeAreDoingOurFirstAccessOfData = iNetTrue
    gTheNextDataAccessIsTheLast_ofEntireDigitization = iNetFalse

    ' report that we are currently digitizing
    gCurrentlyDigitizing = True

    ' Show "Stop" on Start/Stop button
    If ValidObject(gStartStopButton) Then gStartStopButton.caption = "Stop"

    ' -------------------------------------------
    ' CLEAR THE SCREEN & GET READY FOR DATA.
    '
    ' Specify that screens are to be cleared right before their next plot.

    Dim bNum, dNum As Integer
    For bNum = 0 To gNumBoxes - 1

        For dNum = 0 To (gBox(bNum).numDisp - 1)

            With gBox(bNum).disp(dNum)

                ' we are attached to a digitize channel..
                If .cNum <> NO_DIGI_CHAN Then

                    ' say that the display is NOT to be clear just before plotting
                    .clrBeforePlot = False

                    ' Clear the plot regions for this display
                    Call Plot_Clear_OneDisp(bNum, dNum, gBox(bNum).BoxObj)

                    ' Draw a grid in this display
                    Call Grid_Draw_OneDisp(bNum, dNum, _
                        gBox(bNum), gBox(bNum).disp(dNum), gBox(bNum).BoxObj)
 
                End If

            End With

        Next dNum

    Next bNum

    ' -------------------------------------------
    ' If we are to wait till digitizing finishes...
  
    If gWaitTillDigitizationCompletes Then

        Do  ' wait till digitizing finishes

            ' Services incoming data from digitization process.
            Call DigitizeProcess_ServiceIncomingData

        ' sit here while instruNet is digitizing
        Loop While gCurrentlyDigitizing

    Else

        ' -------------------------------------------
        ' Otherwise, we do not wait for digitizing to finish and
        ' instead setup a timer that will fire every now and and will service the
        ' digitizing buffers at that time.  ONE MUST RETURN CPU CONTROL
        ' TO THE MAIN WINDOW SO THAT THE OS CAN SEND EVENTS TO THE
        ' WINDOW, IN ADDITION TO THE TIMER, OTHERWISE, THE TIMER
        ' WILL NOT RUN.
            
        ' make sure the Timer control has been placed onto front panel and
        ' it's name has been passe to DigitizeProcess_SetControlItems().
        
        If Not (ValidObject(gDigitizeProcessTimer)) Then EndProgram ("Need front panel Timer control")
        
        gDigitizeProcessTimer.Enabled = False
        
        ' calc the time to acquire 1 scan of data
        Dim scanTime_secs, scanTime_ms As Double
        scanTime_secs = (1 / gSampleRate_nextDigi) * gPtsPerScan_nextDigi
        scanTime_ms = scanTime_secs * 1000
        
        ' Set the scan interval to once every 200ms, to make it so
        ' the buffer does not overflow, and if plotting segments,
        ' we will plot 5x per second.  If we are processing whole scans
        ' and each scan is 10secs, it will not hurt to service the
        ' buffers 5x per sec, and then do the processing automatically
        ' when we finish the scan.
        
        DEFAULT_PROCESS_TIME_MS = 200
        
        gDigitizeProcessTimer.Interval = DEFAULT_PROCESS_TIME_MS    ' 200ms
        
        If (gDigitizeProcessTimer.Interval > scanTime_ms) Then
        
            ' Set the timer interval in units of 1ms, to fire 2x
            ' per scan.  It should not fall behind given this rate.
            
            gDigitizeProcessTimer.Interval = scanTime_ms / 2
            
            ' Don't allow it to go less than 2ms
            
            If (gDigitizeProcessTimer.Interval < 2) Then
                gDigitizeProcessTimer.Interval = 2
            End If
           
        End If
        
        ' Enable the timer to fire in a little while.
        
        gDigitizeProcessTimer.Enabled = True
                 
    End If
    
    ' -------------------------------------------
    ' EXIT

ExitFunction:

    DigitizeProcess_Start = e
   
End Function

' wait till the digitizing is done
Public Sub DigitizeProcess_WaitTillDone()

    timeout = Timer + 3
    
    Do  ' wait till digitizing finishes
    
        ' NOTE:
        ' This does not always work, since you need to exit
        ' the program and give control to the WINDOW, so
        ' that the OS can then service the timer, and the
        ' WINDOW events interchangeably.  The Timer
        ' is a little funky in this way. gsw 8/11/98
        
        ' if 3secs have gone by
        If (timeout < Timer) Then
        
            ' if the Timer is not telling the digitize
            ' process to be serviced...
            If (gDigitizeProcessCtr = 0) Then
                
                ' end the program
                EndProgram "The gDigitizeProcessTimer does not appear to be moving."
                
            End If
            
        End If
        
    ' sit here while instruNet is digitizing
    Loop While gCurrentlyDigitizing
    
End Sub

' ----------------------------------------------------------------
' FUNCTION    DigitizeProcess_Stop()
'
' PURPOSE     This routine stops the digitizing process.
'
' HISTORY     8/4/98  created - gsw
' ----------------------------------------------------------------

Public Function DigitizeProcess_Stop() As Integer

    Dim e As Integer
    DigitizeProcess_Stop = iNetErr_None
    
    ' if we are not currently digitizing, then bail out of here
    If Not (gCurrentlyDigitizing) Then
        MsgBox "You cannot stop digitizing if you are not digitizing."
        Exit Function
    End If
    
    ' tell instruNet to stop digitizing
    PRESS_iNet_BUTTON (iNetCM_BtnPress_Record_Stop)
    
    ' wait until instruNet has stopped
    Dim timeOutCtr As Long
    timeOutCtr = 0
    Do
        timeOutCtr = timeOutChr + 1
        If (timeOutCtr > 100000) Then EndProgram ("stop timed out")
    Loop While CONTROLLER_IS_DIGITIZING(1) <> False

    ' report that we are no longer digitizing
    gCurrentlyDigitizing = False
    
    ' turn off the timer
    If ValidObject(gDigitizeProcessTimer) Then gDigitizeProcessTimer.Enabled = False
    
    ' update the "Start"/"Stop" button text
    If ValidObject(gStartStopButton) Then gStartStopButton.caption = "Start"
    
    ' Now that we have finished digitizing, tell the instruNet driver
    ' to forget about these VB memory buffers that we let it use
    ' for digitization (this insures that the instruNet driver does not
    ' try to use them later on, when VB may have moved this).  This
    ' does not effect the DigitizeData() in VB memory.
    
    e = VbDataBuffers_DetachFromDLLUserBuffers(gNumDigiCh, gDigiChan())
    
    For bNum = 0 To gNumBoxes - 1   ' scan through all DisplayBoxes
   
        ' draw vertical marker line
        Call Marker_Draw_Line_OneBox(bNum, True, gBox(bNum), gBox(bNum).BoxObj)
     
        ' draw marker text at top of boxes
        Call Marker_Draw_Text_OneBox(bNum, gBox(bNum), gBox(bNum).BoxObj)
        
    Next bNum
         
    ' If true, then run EndUser_PostAcqAnalysis() at the end of the digitization.  This is an
    ' end user routine that processes the resulting data
    If gRun_EndUser_PostAcqAnalysis = True Then
                                                
        ' User routine that processed digitized data, when the digitization stops.
        Call gEndUserWindow.EndUser_PostAcqAnalysis
    
    End If
    
    DigitizeProcess_Stop = e
    
End Function

' FUNCTION    VbDataBuffers_AttachToDLLUserBuffers()
'
' PURPOSE     Attach VB gDigiChan().dataArrayY.data() memory to
'             iNet32.DLL USER RAM BUFFER, so that the iNet
'             driver will digitize into VB ram.
'
' HISTORY     2/25/97  created - unoWarrior
'
Function VbDataBuffers_AttachToDLLUserBuffers( _
    numDigiCh As Integer, _
    digiChAddr() As chAddr, _
    ByVal ptsPerScan As Long) As Integer

Dim e As Integer
Dim cNum As Integer

'disable all channels for digitizing (initially)
e = DisableAllChannelsForDigitizing()
If ErrorHere(e, "DisableAllChannelsForDigitizing") Then GoTo ExitFunction

'scan through all digitize channels
For cNum = 0 To numDigiCh - 1

    ' tell the iNet driver to use the VB array memory as the
    ' destination for the data (we pass in the address if the
    ' 0th point for each channel).
    addressOfBuffer = iNet_Get_VarPtr(digiChAddr(cNum).dataArrayY.data(0))
    e = SetField_int32(digiChAddr(cNum), sgt_UserRamBuffer, fldNum_UserBuffer_UserBufferAddr, addressOfBuffer)
    If ErrorHere(e, "SetField_int32(userAddr)") Then GoTo ExitFunction

    ' tell the instruNet driver the length of the is VB buffer
    ' (4bytes for each "single" precision point)
    userBufferSize = 4 * ptsPerScan
    e = SetField_int32(digiChAddr(cNum), sgt_UserRamBuffer, fldNum_UserBuffer_UserPtrSizeInBytes, userBufferSize)
    If ErrorHere(e, "SetField_int32(userBuffSize)") Then GoTo ExitFunction

    ' tell the instruNet driver to place digitize data into this buffer
    e = SetField_int32(digiChAddr(cNum), sgt_UserRamBuffer, fldNum_UserBuffer_DigitizeOnOff, ion_gOnOffPopup_On)
    If ErrorHere(e, "SetField_int32(digitizeOnOff)") Then GoTo ExitFunction

    ' NOT GOOD, SINCE we end up with a USER and DRIVER buffer, and it randomly uses 1 of these)
    '   enable digitizing for each channel (set Digitize ON in Display settings area).
    '   e = EnableChannelForDigitizing(digiChAddr(cNum))
    '   If ErrorHere(e, "EnableChannelForDigitizing()") Then GoTo ExitFunction

Next cNum

ExitFunction:

    Tell_Driver_To_Put_Digitize_Data_into_VB_Arrays = e

End Function

' FUNCTION    VbDataBuffers_CopyFromDLLDvrBuffers()
'
' PURPOSE     Copy data from VB gDigiChan().dataArrayY.data() memory
'             to the iNet32.DLL DRIVER RAM BUFFERS.
'
' HISTORY     2/25/97  created - unoWarrior
'
Public Sub VbDataBuffers_CopyToDLLDvrBuffers()

    Dim e As Integer
    Dim bufferAddr, bufferBytes, bufferPts As Long
    
    ' Setting the timing parameters, so that the DRIVER RAM BUFFERS
    ' are set to the correct size.
    e = Set_iNet_TIMING_Parameters(ByVal gSampleRate_nextDigi, _
            ByVal gPtsPerScan_nextDigi + 6, ByVal gNumScans, ByVal gContinuousScans, _
            ByVal ion_gSaveDataPopup_ToRamBuffer, iNetTrue, iNetTrue)
    If ErrorHere(e, "ctd1") Then Exit Sub
    
    ' Disable all DigitizeChannels inside the iNet driver (we'll enable them below)
    e = DisableAllChannelsForDigitizing()
    If ErrorHere(e, "ctd2") Then Exit Sub
           
    ' scan through all Digitize Channels
    For cNum = 0 To (gNumDigiCh - 1)
        
        ' turn the DISPLAY:DIGITIZE field ON
        e = SetField_int32(gDigiChan(cNum), sgt_Display, _
            fldNum_Display_dispOnOff, ion_gOnOffPopup_On)
        If ErrorHere(e, "ctd3") Then Exit Sub
        
    Next cNum
    
    ' Sets up digitization, establishes sample rate, sets up BUFFERS, yet does not
    ' actually do the digitization. gsw 8/16/98
    Call SET_UP_DIGITIZATION
    
    ' scan through all Digitize Channels
    For cNum = 0 To (gNumDigiCh - 1)
    
        ' get DRIVER RAM BUFFER size
        bufferBytes = GetField_int32(gDigiChan(cNum), sgt_DriverRamBuffer, fldNum_DvrBuffer_BufferAddrSizeInBytes, e)
        If ErrorHere(e, "ctd6") Then Exit Sub
        
        ' calc # of valid pts in the buffer
        numValidBytes = gDigiChan(cNum).dataArrayY.numValidPts * 4
        
        ' make sure the buffer is large enough
        If (numValidBytes > bufferBytes) Then
            Call ErrorHere(1, "BufferTooSmall"): Exit Sub
        End If
        
        ' get the addr of the DRIVER RAM buffer
        bufferAddr = GetField_int32(gDigiChan(cNum), sgt_DriverRamBuffer, fldNum_DvrBuffer_BufferAddr, e)
        If ErrorHere(e, "ctd7") Then Exit Sub
                
        ' make sure our array is long enough
        littleTest = gDigiChan(cNum).dataArrayY.data((numValidBytes / 4) - 1)
        
        ' copy data into iNet DRIVER RAM buffer frin VB buffer
        Call iNet_memcopy(iNet_Get_VarPtr(gDigiChan(cNum).dataArrayY.data(0)), bufferAddr, numValidBytes)
                    
        ' set the DRIVER RAM buffer numValidPts
        e = SetField_int32(gDigiChan(cNum), sgt_DriverRamBuffer, _
            fldNum_DvrBuffer_PtNumIn, (numValidBytes / 4))
        If ErrorHere(e, "ctd8") Then Exit Sub
    
    Next cNum
 
 End Sub

' FUNCTION    VbDataBuffers_CopyFromDLLDvrBuffers()
'
' PURPOSE     Copy data from iNet32.DLL DRIVER RAM BUFFERS into
'             VB gDigiChan().dataArrayY.data() memory.
'
' HISTORY     2/25/97  created - unoWarrior
'
Public Sub VbDataBuffers_CopyFromDLLDvrBuffers()

    Dim e As Integer
    Dim bufferAddr, bufferBytes, bufferPts As Long
    
    For cNum = 0 To gNumDigiCh - 1          ' scan through all digitize channels
         
      With gDigiChan(cNum)                  ' w/ this channel...
                                      
        ' get the # of data bytes stored inside the DRIVER ram buffer
        bufferAddr = GetField_int32(gDigiChan(cNum), sgt_DriverRamBuffer, fldNum_DvrBuffer_BufferAddr, e)
        If ErrorHere(e, "cfd1") Then Exit Sub
        
        ' get DRIVER RAM BUFFER size
        bufferBytes = GetField_int32(gDigiChan(cNum), sgt_DriverRamBuffer, fldNum_DvrBuffer_BufferAddrSizeInBytes, e)
        If ErrorHere(e, "cfd2") Then Exit Sub
        
        ' make sure we do not load in too much data
        bufferPts = bufferBytes / 4
        If (bufferPts > .dataArrayY.numBufferPts) Then bufferPts = .dataArrayY.numBufferPts
        
        ' Set the size of the gDigiChan() dataArrayY
        ReDim Preserve gDigiChan(cNum).dataArrayY.data(.dataArrayY.numBufferPts + 2)
        
        ' if we have some data...
        If (bufferPts > 0) Then     ' copy data from iNet driver ram buffer into VB buffer
            Call iNet_memcopy(bufferAddr, iNet_Get_VarPtr(.dataArrayY.data(0)), bufferPts * 4)
        End If
        
        ' scan through all Digitize Channels, and turn the DISPLAY:DIGITIZE field OFF
        ' to delete the temporary DRIVER RAM buffers.
        e = SetField_int32(gDigiChan(cNum), sgt_Display, fldNum_Display_dispOnOff, ion_gOnOffPopup_Off)
        If ErrorHere(e, "cfd3") Then Exit Sub

      End With
         
    Next cNum

End Sub

' FUNCTION    VbDataBuffers_DetachFromDLLUserBuffers()
'
' PURPOSE     After digitizing, this is used to tell the instruNet driver
'             to forget about the VB memory buffers that we let it use
'             for digitizing (this insures that the instruNet driver does not
'             try to use them later on, when VB may have moved them or changed
'             them).  This does not effect the DigitizeData() in VB memory.
'
' PARAMETERS  numDigiCh
'                   This is the number of channels that have
'                   been selected for digitizing.
'
'             digiChAddr(0...numDigiCh-1)
'                   This specifies the list of channels that have been
'                   selected for digitizing.
'
'             Returns iNetError code.
'
' HISTORY     2/25/97  created - gsw
'
Function VbDataBuffers_DetachFromDLLUserBuffers( _
    numDigiCh As Integer, _
    digiChAddr() As chAddr) As Integer

Dim e As Integer
Dim cNum As Integer

'scan through all digitize channels
For cNum = 0 To numDigiCh - 1
    
    ' testing stuff
    testing = False
    If testing = True Then
    
        ' make sure driver ram buffer is off
        Value = GetField_int32(digiChAddr(cNum), sgt_DriverRamBuffer, fldNum_DvrBuffer_DigitizeOnOff, e)
        If ErrorHere(e, "SetField_int32(userAddr)") Then GoTo ExitFunction
        If Value = ion_gOnOffPopup_On Then
            MsgBox "aaa"
        End If

        addr = GetField_int32(digiChAddr(cNum), sgt_DriverRamBuffer, fldNum_DvrBuffer_BufferAddr, e)
        If ErrorHere(e, "SetField_int32(userAddr)") Then GoTo ExitFunction
        If addr <> 0 Then
            MsgBox "bbb"
        End If
        
    End If

    ' Turn ON the display digitize field (so that the Network page will show
    ' the channel as being turned on). This might also turn on the DriverRamBuffer,
    ' yet everything is turned off when we call DisableAllDigitizing().
    
    e = SetField_int32(digiChAddr(cNum), sgt_Display, fldNum_Display_dispOnOff, ion_gOnOffPopup_On)
    If ErrorHere(e, "SetField_int32(dispOnOff)") Then GoTo ExitFunction

    ' turn off instruNet driver user buffer
    e = SetField_int32(digiChAddr(cNum), sgt_UserRamBuffer, fldNum_UserBuffer_UserBufferAddr, 0)
    If ErrorHere(e, "SetField_int32(userAddr)") Then GoTo ExitFunction

    e = SetField_int32(digiChAddr(cNum), sgt_UserRamBuffer, fldNum_UserBuffer_UserPtrSizeInBytes, 0)
    If ErrorHere(e, "SetField_int32(userBuffSize)") Then GoTo ExitFunction

    e = SetField_int32(digiChAddr(cNum), sgt_UserRamBuffer, fldNum_UserBuffer_DigitizeOnOff, ion_gOnOffPopup_Off)
    If ErrorHere(e, "SetField_int32(userBuffSize)") Then GoTo ExitFunction

Next cNum

ExitFunction:

    Tell_Driver_To_Put_Digitize_Data_into_VB_Arrays = e

End Function





' FUNCTION   DigitizeList_CopyFromDriverDLL()
'
' PURPOSE    This routine loads 'gDigiChan()' with the addresses of
'            all channels that have been enabled for digitizing in the
'            NETWORK window.  Typical sequence:
'
'               1. show user NETWORK page and ask user to select
'                  channels for digitizing (click on red rectangle).
'
'               2. call this routine to capture that info.
'
'               3. digitize.
'
' HISTORY    2/20/96  created Visual Basic version - sjm
'            8/23/99  bug fix, numberOfChannels = GET_NUMBER_OF_CHANNELS(netNum, moduleNum, deviceNum)
'
Function DigitizeList_CopyFromDriverDLL( _
    ByVal massageType As Integer, _
    ByVal runEndUser_SegmentProcessor As Integer) As Integer

    Dim deviceNum As Byte
    Dim chanNum As Byte
    Dim netNum As Byte
    Dim moduleNum As Byte
    Dim numberOfModules, numberOfDevices, numberOfChannels, numberOfNetworks As Integer
    Dim message As String
    Dim e As Integer
    Dim onOffPopup As Long
    Dim thisFieldExist As Long

    ' start out with an empty Digitize Channel list
    Call DigitizeList_ResetList
    
    ' start out with no channels selected for digitizing
    e = iNetErr_None
    
    ' get # of iNet Networks in this Computer
    numberOfNetworks = GET_NUMBER_OF_NETWORKS()
    e = Get_iNet_Error()
    If ErrorHere(e, "GET_NUMBER_OF_NETWORKS()") Then GoTo ExitFunction

    'scan through all Networks
    For netNum = 1 To numberOfNetworks

        numberOfDevices = GET_NUMBER_OF_DEVICES(netNum)
        e = Get_iNet_Error()
        If ErrorHere(e, "GET_NUMBER_OF_DEVICES()") Then GoTo ExitFunction

        'scan through all devices
        For deviceNum = 1 To numberOfDevices

            numberOfModules = GET_NUMBER_OF_MODULES(netNum, deviceNum)
            e = Get_iNet_Error()
            If ErrorHere(e, "GET_NUMBER_OF_MODULES()") Then GoTo ExitFunction

            'scan through all modules
            For moduleNum = 1 To numberOfModules

                ' bug fix 8/23/99, numberOfChannels = GET_NUMBER_OF_CHANNELS(netNum, moduleNum, deviceNum)
                numberOfChannels = GET_NUMBER_OF_CHANNELS(netNum, deviceNum, moduleNum)
                e = Get_iNet_Error()
                If ErrorHere(e, "GET_NUMBER_OF_CHANNELS()") Then GoTo ExitFunction

                'scan through Channels
                For chanNum = 1 To numberOfChannels

                    ' see if the Display Digitize on/off fields exists...
                    thisFieldExist = THIS_FIELD_EXISTS(ByVal netNum, ByVal deviceNum, ByVal moduleNum, ByVal chanNum, ByVal sgt_Display, ByVal fldNum_Display_dispOnOff)
                    If thisFieldExist = iNetTrue Then

                        ' if this channels is enabled for digitizing...
                        onOffPopup = iNet_int32(netNum, deviceNum, moduleNum, chanNum, sgt_Display, fldNum_Display_dispOnOff, intention_getValue, 0)
                        If onOffPopup = ion_gOnOffPopup_On Then
                    
                            ' add this channel to the digitize list
                            Call DigitizeList_AddOneItem(netNum, deviceNum, moduleNum, chanNum, _
                                massageType, runEndUser_SegmentProcessor)
                                
                        End If

                    End If

                Next chanNum

            Next moduleNum

        Next deviceNum

    Next netNum

ExitFunction:
    
    DigitizeList_CopyFromDriverDLL = e

End Function



' FUNCTION    DigitizeProcess_MeasureActualSampleRate()
'
' PURPOSE     Measures the actual sample rate.
'
' HISTORY     2/25/97  created - unoWarrior
'
Function DigitizeProcess_MeasureActualSampleRate() As Single
    
    ' Deselect channels inside the Netwokr page that had been enabled
    ' for digitizing (destroys their DIGITIZE BUFFERS).
    e = DisableAllChannelsForDigitizing()
    If ErrorHere(e, "dpmasr0") Then GoTo ExitWithError
    
    ' Select these channels for digitizing.
    For cNum = 0 To gNumDigiCh - 1
        e = SetField_int32(gDigiChan(cNum), sgt_Display, fldNum_Display_dispOnOff, ion_gOnOffPopup_On)
        If ErrorHere(e, "dpmasr1") Then GoTo ExitWithError
    Next cNum
                
    ' We are setting the timing parameters in here.
    e = Set_iNet_TIMING_Parameters(ByVal gSampleRate_nextDigi, _
            ByVal gPtsPerScan_nextDigi, ByVal gNumScans, ByVal gContinuousScans, _
            ByVal ion_gSaveDataPopup_ToRamBuffer, iNetTrue, iNetTrue)
    If ErrorHere(e, "dpmasr2") Then GoTo ExitWithError
    
    ' Sets up digitization, establishes sample rate, sets up BUFFERS, yet does not
    ' actually do the digitization. gsw 8/16/98
    Call SET_UP_DIGITIZATION
    
    ' read in the resulting sample rate
    DigitizeProcess_MeasureActualSampleRate = GET_SAMPLE_RATE_IN_HZ(netNum_1stController)
        
    ' Deselect channels inside the Netwokr page that had been enabled
    ' for digitizing (destroys their DIGITIZE BUFFERS).
    e = DisableAllChannelsForDigitizing()
    If ErrorHere(e, "dpmasr3") Then GoTo ExitWithError
        
ExitWithoutError:

    Exit Function

ExitWithError:

    DigitizeProcess_MeasureActualSampleRate = 1

    MsgBox ("Digitize_iNet_Data() Error. Simulation is being turned on")
        
End Function



