Attribute VB_Name = "Display_Utilities"
'************************************************************************
' INET_Display.Bas
'
'------------------ D O C U  M E N T A T I O N ---------------------------
'
' This iNet Interface file contains code to manage displays in visual basic.
' A "Display" is a PictureBox on the front panel that can show a variable
' number of waveforms, and is VERY programmable.
'
'-------------------------  H I S T O R Y --------------------------------
' HISTORY  Copyright c 1994-98 GWI.  All rights reserved.
' 7/29/98  created - gsw
'*************************************************************************

' -----------------------------------------------------
' Display Constants and Globals.
                                    ' max # of channels per displaybox
Global Const MAX_NUM_CH_PER_OneBox = 32

Global Const MAX_NUM_OneBox = 32    ' max # of Display Boxes

Global Const TOP_DISPLAY_NUM = 0    ' dNum for top display in DispBox

Global gNumBoxes As Integer         ' # of displays boxes

Global Const AUTO_GRID = -1         ' automatically select grid spacing

                                    ' # of pixels per grid when doing AUTO_GRID
Global Const pixels_per_grid_AUTO_GRID = 50
        
Global Const NO_PREVIOUS_MKR = -1   ' no previous marker position

Global Const vSPACE_yPix = 3        ' # of free pixels below & above label text

Global Const MAX_NUM_MKRS = 4       ' max # of markers

Global Const NO_SCROLL = -1         ' no scroll bar

Global Const NO_DIGI_CHAN = -1#     ' Display is not linked to a digitize channel

Global gReturnStr As String         ' return string (Chr$(13) + Chr(10))

' -----------------------------------------------------
' Line sytles types.

Global Const kBlack = 1             ' solid black line
Global Const kGray = 2              ' gray line if screen, dotted if Printer

' -----------------------------------------------------
' Display types.

Global Const kYindexDisplay = 1     ' Y vs index display
Global Const kXYDisplay = 2         ' X vs Y display

' -----------------------------------------------------
' Automatically add channels that have been enabled
' for digitizing in the instruNet World NETWORK window.
' This is done when the DisplayBox is added to the window, when
' you close the instruNet World Network window and when you Load in data.

Global Const hAutoAddNone = 1       ' don't automatically do anythin
Global Const hAutoAddTimeWaves = 2  ' automatically add time plots of digitized waves
Global Const hAutoAddFreqWaves = 3  ' automatically add fft's of digitized waves
Global Const hAutoAddXYplots = 4    ' automatically add xy plots of digitized waves

' -----------------------------------------------------
' Parameters for 1 channel (multiple channels within one display box).
            
                                    ' turns vPos/hPos methods on/off
Public gMethod_vPos_scrollObj_Enabled As Integer
Public gMethod_hPos_scrollObj_Enabled As Integer

' -----------------------------------------------------
' Parameters for 1 channel (multiple channels within one display box).

Public Type DispParm                ' contains parameters for 1 Display (1
                                    ' Channel per Display) that resides within
                                    ' a Multi-Display DisplayBox.
                                    
                                    ' --------------------------------------
                                    ' HORIZONTAL SCALE PARAMS
                                    
    dispLeft_xEU As Double          ' horiz Left/Right values in xEU coords
    dispRight_xEU As Double
    dispWidth_xEU As Double
   
    xCoord_Per_xEU As Double        ' xCoords per xEU (horiz plot scale)
        
                                    ' --------------------------------------
                                    ' Y vs INDEX PLOT PARAMS
    
    dispLeft_ptNum As Long          ' waveform ptNum corresponding to left edge
    dispRight_ptNum As Long         ' waveform ptNum corresponding to right
                                    ' edge (assumes the wave buffers are full
                                    ' and extend to this edge, which might not
                                    ' be the case).  See gDigiChan(chNum).numValidData
                                    ' for the # of valid pts in the waveform buffer.
    
    dispWidth_numPts As Long        ' width of display in points
    
    xCoord_Per_ptNum As Double      ' xCoords per points (horiz plot scale)
    
                                    ' --------------------------------------
                                    ' VERTICAL SCALE PARAMS
        
    dispTop_yEu As Double           ' top of display, yEU units
    dispBot_yEu As Double           ' bottom of display, yEU units
    
    dispMid_yEu As Double           ' middle of display, yEU units
    dispHeight_yEu As Double        ' height of display, yEU units
    
    yCoord_Per_yEU As Double        ' yCoord per yEU (yCoord to yEU translation)
        
    plotOffset_yEU As Double        ' plot offset, yEU units
    plotOffset_yCoord As Double     ' plot offset, yCoord units
            
                                    ' --------------------------------------
                                    ' X/Y DISPLAY COORDINATES
    
    dispLeft_xCoord As Integer      ' left of display, xCoord units  (i.e. "scale", not pixels)
    dispRight_xCoord As Integer     ' right of display, xCoord units
    dispTop_yCoord As Integer       ' top of display, yCoord units
    dispBot_yCoord As Integer       ' bottom of display, yCoord units
  
    dispHeight_yCoord As Integer    ' Height/Width - calc'ed from top/bot/lef/rht
    dispWidth_xCoord As Integer
        
                                    ' --------------------------------------
                                    ' GRID COORDINATES
    
    showGrid As Integer             ' if true, we show a grid
    
    numGridLines_horiz As Integer   ' # of horiz grid lines, e.g. 3 (0 to turn off grid, -1= autoselect)
    numGridLines_vert As Integer    ' # of vertical grid lines, e.g. 5 (0 to turn off grid, -1= autoselect)
    ticksPerGrid_vert As Integer    ' # of tics per vert grid, e.g. 3 (-1= autoselect)
    ticksPerGrid_horiz As Integer   ' # of tics per horiz grid, e.g. 3 (-1= autoselect)
    
                                    ' Calculated, based on the above values
    numGridLines_horiz_calc As Integer
    numGridLines_vert_calc As Integer
    ticksPerGrid_vert_calc As Integer
    ticksPerGrid_horiz_calc As Integer
    
    gridWidth_xCoord As Double      ' space between vertical grid lines
    gridHeight_yCoord As Double     ' space between horiz grid lines
        
                                    ' --------------------------------------
                                    ' DIGTIIZE DATA
    
    cNum As Integer                 ' index into gDigiChan(0..numChan) array to
                                    ' get to data associated w/ this display.
                                    ' if this = NO_DIGI_CHAN = -1, then use the LOCAL data
                                    ' below.
        
                                    ' --------------------------------------
                                    ' LOCAL DATA
                                    '
                                    ' Display data is kept in dataArrayY, if the source
                                    ' is not the gDigiChan(cNum).dataArrayY, which
                                    ' is the data directly from a digitization. If this
                                    ' is the case, then .cNum = NO_DIGI_CHAN
                                    '
    dataArrayY As DataArrayType     ' dataArrayY Contains (for Y data):
                                    '
                                    '   .numValidPts = # of valid points contained within the data() buffer
                                    '   .numBufferPts = data() buffer size
                                    '   .data() = data is stored in this array
                                    '   .ptsPerXunit = sample rate of the data
                                    '   .firstPt_Xunits = first point time in seconds
                                    
    dataArrayX As DataArrayType     ' dataArray for X data, when plotting X vs Y
                                    ' (i.e. displayType = kXYDisplay)
                                    
                                    ' --------------------------------------
                                    ' HOUSEKEEPING
    
    clrBeforePlot As Integer        ' if true, we need to clear the display right
                                    ' before plotting, the next time we plot
                                    
    name As String                  ' name of Display
    
    displayType As Integer          ' type of display = {kYindexDisplay, kXYDisplay}
    
                                    ' --------------------------------------
                                    ' MARKERS
    
                                    ' wave value at the marker position
    valueAtMarker_yEU(MAX_NUM_MKRS) As Double
                                    
End Type

' -----------------------------------------------------
' Parameters for 1 DisplayBox (contains multiple Displays, 1 channel per Display)

Public Type DispBoxParm             ' Contains parameters that define a VB
                                    ' iNet DisplayBox (which can show 1 to 32
                                    ' channels in one box).
        
                                    ' --------------------------------------
                                    ' ASSOCIATED WINDOW-BASED OBJECTS
                        
    BoxObj As PictureBox            ' box object on front panel that encloses all
                                    ' displays in this 1 DispBox.
    
    FormObj As Form                 ' window that contains DispBox
    
                                    ' --------------------------------------
                                    ' HORIZONTAL SCALE
                                                                    
                                    ' dispWidth_deltaChoice As Double ' change in display width when you click on it.
                                                                            
    hPos_scrollObj As HScrollBar    ' Horiz scrollbar, = "Nothing" if doesn't exist
    
    hPos_scroll_lastValue As Integer ' Horiz scrollbar last value
                                    
                                    ' --------------------------------------
                                    ' VERTICAL SCALE
             
    vPos_scrollObj As VScrollBar    ' Vert scrollbar, = "Nothing" if doesn't exist
    
    vPos_scroll_lastValue As Integer ' Vert scrollbar last value
    
                                    ' --------------------------------------
                                    ' MARKERS
 
                                    ' Current marker position, in waveform points
                                    ' this is the dNum of the display within the
                                    ' dispBox that holds the marker (the marker refers
                                    ' to the value of the wave in this 1 display).
    markerPos_xEU(MAX_NUM_MKRS) As Double
    markerPos_dNum(MAX_NUM_MKRS) As Integer
    
                                    ' Previous marker position (we need to replot wave here),
                                    ' display of previous marker, = NO_PREVIOUS_MKR if none
    prevMarkerPos_xEU(MAX_NUM_MKRS) As Double
    prevMarkerPos_dNum(MAX_NUM_MKRS) As Integer
    
    numMkrs As Integer              ' # of markers in the display
    supportMarkers As Integer       ' if True, this dispBox supports markers; otherwise not
    
                                    ' --------------------------------------
                                    ' X/Y COORDINATES
 
    dispBoxLeft_xCoord As Integer   ' left of dispBox, xCoord units (i.e. "scale", not pixels)
    dispBoxRight_xCoord As Integer  ' right of dispBox, xCoord units
    dispBoxTop_yCoord As Integer    ' top of dispBox, yCoord units
    dispBoxBot_yCoord As Integer    ' bottom of dispBox, yCoord units
        
    vLabelBoxLeft_xCoord As Integer ' this is the left of the Vertical Label region and
                                    ' the right the plot regions.
    
    hLowerLabel_Top_yCoord As Integer  ' top of the Lower Horizontal Label region at the
                                    ' bottom of the DisplayBox (and below the bottom-most Display)
    
    hUpperLabel_Bot_yCoord As Integer  ' bottom of the Upper Horizontal Label region at the
                                    ' top of the DisplayBox (and above the top-most Display)
    
                                    ' --------------------------------------
                                    ' GRID
    supportGrid_default As Integer          ' true if you draw grid
    numGridLines_horiz_default As Integer   ' # of horiz grid lines, e.g. 3 (0 to turn off grid)
    numGridLines_vert_default As Integer    ' # of vertical grid lines, e.g. 5 (0 to turn off grid)
    ticksPerGrid_vert_default As Integer    ' # of tics per vert grid, e.g. 3
    ticksPerGrid_horiz_default As Integer   ' # of tics per horiz grid, e.g. 3
    
                                    ' --------------------------------------
                                    ' ETC
 
    autoAddPolicy As Integer        ' automatically add DigiChan() channels
                                    ' to DisplayBox.  This is done when the
                                    ' DisplayBox is added to the window, when
                                    ' you close the instruNet World Network window
                                    ' and when you Load in data.
                                        
    hDispBoxTitle_text As String    ' text at top of DisplayBox, that labels the box
    
    numDisp As Integer              ' # of displays in DispBox (i.e. # of channels)
    
    disp() As DispParm              ' contains params for all Displays within the DispBox
                                    ' (we expand this each time we add a display)
                                    
                                    
End Type

' -----------------------------------------------------
' Display Constants and Globals.

Public gBox() As DispBoxParm        ' Contains all the DispBox params
                                    ' (the size of this array is ReDim'ed
                                    ' after initializing each DispBox).

Public gEndUserWindow As Variant    ' main window created by end user

' -----------------------------------------------------
' Flags passed to Reset_Controls() to tell it what to reset

Global Const reset_hPosition_toZero = 1
Global Const reset_hScale_toOneScanWideDisplays = 2
Global Const reset_hScaleToInternalValues = 4
Global Const reset_ptsPerHscale = 8
Global Const reset_vScalePosScrollBtns = 16
Global Const reset_vScaleToInternalValues = 32
Global Const reset_mkrs_leftEdge = 64


' Print the main window
Public Sub METHOD_Print_Screen_Button()
    
    ' This does not work correctly.  Due to conflict
    ' with printer driver?  Not sure (??). gsw 8/13/98
    
    ' set high resolution print quality
    Printer.PrintQuality = vbPRPQHigh
    
    ' print the main VB form
    gEndUserWindow.PrintForm
    
End Sub

' Convert a number to string
Public Function NumToStr(ByVal num As Double) As String
 
    Dim numAbs As Double
    numAbs = Abs(num)
    
    If (numAbs = 0) Then
        NumToStr = "0"
    ElseIf (numAbs < 0.001) Then
        NumToStr = Format(num, "scientific")    ' e.g. 3.23E-02
    ElseIf (numAbs < 0.01) Then
        NumToStr = Format(num, ".######")
    ElseIf (numAbs < 0.1) Then
        NumToStr = Format(num, ".#####")
    ElseIf (numAbs < 1#) Then
        NumToStr = Format(num, ".####")
    ElseIf (numAbs < 10#) Then
        NumToStr = Format(num, "0.###")
    ElseIf (numAbs < 100#) Then
        NumToStr = Format(num, "##.##")
    ElseIf (numAbs < 1000#) Then
        NumToStr = Format(num, "###.#")
    ElseIf (numAbs < 10000#) Then
        NumToStr = Format(num, "####.#")
    ElseIf (numAbs < 100000#) Then
        NumToStr = Format(num, "#####.#")
    Else
        NumToStr = Format(num, "scientific")    ' e.g. 3.23E+02
    End If
    
End Function

' This is a low level routine that is typically not used by the end user.
'
' Reset horiz/vert scroll/scale to default values, per 'thingsToReset':
'
'   Global Const reset_hPosition_toZero = 1
'   Global Const reset_hScale_toOneScanWideDisplays = 2
'   Global Const reset_hScaleToInternalValues = 4
'   Global Const reset_ptsPerHscale = 8
'   Global Const reset_vScalePosScrollBtns = 16
'   Global Const reset_vScaleToInternalValues = 32
'   Global Const reset_mkrs_leftEdge = 64
'
Sub Reset_Controls_OneBox(ByVal bNum As Integer, _
    ByRef thisBox As DispBoxParm, thingsToReset As Integer)

    Dim dNum, mkrNum, resetLeftEdgeToZero, cNum As Integer
    Dim reset_hScale_toOneScanWide As Integer
    Dim dispLeft_xEU_NEW, dispRight_xEU_NEWm, dispMid_xEU_NEW As Double
    Dim dispMid_xEU_current As Double
    Dim sRate As Double
    
    With thisBox
    
        ' Don't do anything in here if there are no displays in this DisplayBox.
        If (.numDisp <= 0) Then Exit Sub

        ' Get the # of data pts and the data's sample rate, for the TOP display.
        Dim ptsPerXunit_topDisp As Double
        Dim numValidPts_topDisp As Long
        Dim firstPt_Xunits_topDisp As Double
        Call GetDataParams(bNum, TOP_DISPLAY_NUM, _
            ptsPerXunit_topDisp, numValidPts_topDisp, firstPt_Xunits_topDisp)
     
        ' ---------------------------------------------------
        ' HORIZONTAL SCALE (i.e. xEU Left/Right stuff).
    
        ' if we are to reset the hPosition to 0...
        resetLeftEdgeToZero = False
        If (thingsToReset And reset_hPosition_toZero) <> 0 Then
    
            If ValidObject(.hPos_scrollObj) Then
                gMethod_hPos_scrollObj_Enabled = False      ' turn off hPos_Change() method for a moment
                ' if this is Y vs INDEX, then reset left edge to 0
                If (.disp(TOP_DISPLAY_NUM).displayType = kYindexDisplay) Then
                    .hPos_scrollObj.Value = thisBox.hPos_scrollObj.Min
                Else
                    .hPos_scrollObj.Value = .hPos_scrollObj.Min + _
                        ((.hPos_scrollObj.Max - .hPos_scrollObj.Min) * 0.5)
                End If
               .hPos_scroll_lastValue = .hPos_scrollObj.Value
                gMethod_hPos_scrollObj_Enabled = True
            End If
            resetLeftEdgeToZero = True
            
        End If

        ' if we are to reset the hScale to set displays 1scan wide...
        reset_hScale_toOneScanWide = False
        If (thingsToReset And reset_hScale_toOneScanWideDisplays) <> 0 Then
            reset_hScale_toOneScanWide = True
        End If
    
        ' reset the horiz scale control setting to 1 scan per display width
        If (reset_hScale_toOneScanWide = True) Or (resetLeftEdgeToZero = True) Then
                    
            ' Calc width of 1 scan for the top display.
            OneScanWide_TopDisplay_xEU! = numValidPts_topDisp / ptsPerXunit_topDisp
                
            ' Make sure this is ok.
            If OneScanWide_TopDisplay_xEU! < 1E-30 Then OneScanWide_TopDisplay_xEU! = 1
                             
            ' Set the scale for all displays.
            For dNum = 0 To (.numDisp - 1)
            
                With .disp(dNum)
                
                    ' if this is Y vs INDEX, then reset left edge to 0
                    If (.displayType = kYindexDisplay) Then
                        
                        dispLeft_xEU_NEW = IIf(resetLeftEdgeToZero = True, 0, .dispLeft_xEU)
                        
                        dispRight_xEU_NEW = IIf((reset_hScale_toOneScanWide = True), _
                            (dispLeft_xEU_NEW + (OneScanWide_TopDisplay_xEU!)), _
                            (dispLeft_xEU_NEW + (.dispRight_xEU - .dispLeft_xEU)))
                        
                    ' else, if this is X vs Y, then reset center to 0
                    Else
                    
                        dispMid_xEU_current = .dispLeft_xEU + (.dispRight_xEU - .dispLeft_xEU) / 2
                    
                        dispMid_xEU_NEW = IIf(resetLeftEdgeToZero = True, 0, dispMid_xEU_current)
                                                
                        If (reset_hScale_toOneScanWide = True) Then
                            dispLeft_xEU_NEW = dispMid_xEU_NEW - OneScanWide_TopDisplay_xEU! / 2
                            dispRight_xEU_NEW = dispMid_xEU_NEW + OneScanWide_TopDisplay_xEU! / 2
                        Else
                            dispLeft_xEU_NEW = dispMid_xEU_NEW - .dispWidth_xEU / 2
                            dispRight_xEU_NEW = dispMid_xEU_NEW + .dispWidth_xEU / 2
                        End If
 
                    End If
                    
                    ' force update by making it different
                    .dispLeft_xEU = dispRight_xEU_NEW + 1
                   
                    ' left and right display edges
                    Call hScale_SetWithNew_xEU_values_OneDisp(bNum, dNum, _
                            dispLeft_xEU_NEW, dispRight_xEU_NEW, gBox(bNum).BoxObj)
                End With
                
            Next dNum
        
        End If
    
        ' ---------------------------------------------------
        ' SET HORIZONTAL SCALE TO .dispLeft_xEU/.dispRight_xEU values
        
        ' if reset the vertical scale to ..dispLeft_xEU/.dispRight_xEU values...
        If (thingsToReset And reset_hScaleToInternalValues) <> 0 Then
                                                       
             ' Set hScroll bar to it's lastValue
            If ValidObject(.hPos_scrollObj) Then    ' if we have a scroll bar...
                gMethod_hPos_scrollObj_Enabled = False
                .hPos_scrollObj.Value = .hPos_scroll_lastValue
                gMethod_hPos_scrollObj_Enabled = True
            End If
            
            ' Scan through all Displays in the Box...
            For dNum = 0 To (.numDisp - 1)
            
                Dim left, right As Double
                left = .disp(dNum).dispLeft_xEU
                right = .disp(dNum).dispRight_xEU
                
                ' force update by making it different
                .disp(dNum).dispLeft_xEU = left + 0.001
                
                ' set vScale to .dispLeft_xEU/.dispRight_xEU values
                Call hScale_SetWithNew_xEU_values_OneDisp(bNum, dNum, left, right, _
                        gBox(bNum).BoxObj)
            
            Next dNum
            
        End If
        
        ' ---------------------------------------------------
        ' SET Pts-per-hSCALE based on the sample rate of
        ' the data in the data's 'ptsPerSec' field
        ' (i.e. set the ptNum-vs-xEU/xCoord stuff).
    
        ' if we are to reset the hPosition to 0...
        If (thingsToReset And reset_ptsPerHscale) <> 0 Then
    
            ' Scan through all Displays in the Box...
            For dNum = 0 To (.numDisp - 1)
        
                ' get digitize channel that is linked to this display
                cNum = .disp(dNum).cNum
                            
                ' get the # of data pts and the data's sample rate (this display)
                Dim ptsPerXunit As Double
                Dim firstPt_Xunits As Double
                Dim numValidPts As Long
                Call GetDataParams(bNum, dNum, ptsPerXunit, numValidPts, firstPt_Xunits)
                                
                Call hScale_SetPtsPerHscale_OneDisp(bNum, dNum, ptsPerXunit)
                    
            Next dNum
            
        End If

        ' ---------------------------------------------------
        ' SET VERTICAL SCALE TO +/-5V
        
        ' if reset the vertical scale to +/-5V...
        If (thingsToReset And reset_vScalePosScrollBtns) <> 0 Then
        
            ' Scan through all Displays in the Box...
            For dNum = 0 To (.numDisp - 1)
            
                ' force update by making it different
                .disp(dNum).dispBot_yEu = 0.001
                
                ' set vScale to -5V/+5V default
                Call vScale_SetWithNew_yEU_values_OneDisp(bNum, dNum, 5, -5)
            
            Next dNum
            
             ' Put vScroll bar in center
            If ValidObject(.vPos_scrollObj) Then    ' if we have a scroll bar...
                gMethod_vPos_scrollObj_Enabled = False
                .vPos_scrollObj.Value = _
                    .vPos_scrollObj.Min + ((.vPos_scrollObj.Max - .vPos_scrollObj.Min) * 0.5)
                .vPos_scroll_lastValue = .vPos_scrollObj.Value
                gMethod_vPos_scrollObj_Enabled = True
            End If
                                               
       End If
        
        ' ---------------------------------------------------
        ' SET VERTICAL SCALE TO .dispTop_yEU/.dispBox_yEU values
        
        ' if reset the vertical scale to .dispTop_yEU/.dispBox_yEU values...
        If (thingsToReset And reset_vScaleToInternalValues) <> 0 Then
                                                       
            ' Scan through all Displays in the Box...
            For dNum = 0 To (.numDisp - 1)
            
                ' Put vScroll bar to it's last value
                If ValidObject(.vPos_scrollObj) Then    ' if we have a scroll bar...
                    gMethod_vPos_scrollObj_Enabled = False
                    .vPos_scrollObj.Value = .vPos_scroll_lastValue
                    gMethod_vPos_scrollObj_Enabled = True
                End If
            
                Dim bot, top As Double
                bot = .disp(dNum).dispBot_yEu
                top = .disp(dNum).dispTop_yEu
                
                ' force update by making it different
                .disp(dNum).dispBot_yEu = bot + 0.001
                
                ' set vScale to .dispTop_yEU/.dispBox_yEU values
                Call vScale_SetWithNew_yEU_values_OneDisp(bNum, dNum, top, bot)
            
            Next dNum
            
        End If
        
        ' ---------------------------------------------------
        ' RESET MARKERS
            
        ' if reset the markers to the left edge...
        If (thingsToReset And reset_mkrs_leftEdge) <> 0 Then
        
            ' move markers to left edge
            For mkrNum = 0 To (.numMkrs - 1)
            
                .markerPos_xEU(mrkNum) = .disp(TOP_DISPLAY_NUM).dispLeft_xEU
                .prevMarkerPos_xEU(mrkNum) = .disp(TOP_DISPLAY_NUM).dispLeft_xEU
                
                .markerPos_dNum(mrkNum) = TOP_DISPLAY_NUM
                .prevMarkerPos_dNum(mrkNum) = NO_PREVIOUS_MKR        ' no previous marker at this time
                
            Next mkrNum
            
        End If
    
    End With
            
End Sub

' Handle a key down in the vPos scroll bar
Public Sub METHOD_vPos_scroll_KeyDown(ByVal bNum As Integer, _
    ByVal KeyCode As Integer, ByVal Shift As Integer)
          
    With gBox(bNum)
    
        ' don't do anything in here if there are not displays in this DisplayBox.
        If (.numDisp <= 0) Then Exit Sub
        
        ' if the scrollbar does not exist, bail out
        If Not (ValidObject(.vPos_scrollObj)) Then Exit Sub
        
        ' reset to left edge
        If KeyCode = &H30 Then
            
            ' reset vScroll to it's center
            .vPos_scrollObj.Value = .vPos_scrollObj.Min + _
                ((.vPos_scrollObj.Max - .vPos_scrollObj.Min) * 0.5)
            
            ' update the vScroll bar
            METHOD_vPos_scroll_Change (bNum)
            
        End If
    End With
End Sub


' Create and Initialize one dispBox. There must be a PictureBox
' object on the front panel that corresponds to the box that we
' are creating.
'
' DisplayBoxes are PictureBox rectangles on a form that are set up
' by the end user at design time using the VB toolbar.  They are
' indexed controls (i.e. set index property to 0, 1...) and
' must be named "Box" (i.e. set name property to "Box").  The
' code handles any number of these arrayed controls, and indentifies
' each with a 'bNum' which goes from 0 to the (numBoxes-1).
' This number corresponds to the index of the Picture box on the form.
' After the user has placed these "Boxes", the code must know they
' are there and this is done with Add_One_OneBox_To_Window().  This is called
' for each "Box" on the front panel.  One must pass the name of the
' box object, the form object, and optionally horiz/vertical scrollbars.
' If there are no scrollbars, please pass "Nothing".
'
Sub Add_One_OneBox_To_Window( _
    ByRef BoxObj_Panel As PictureBox, _
    ByRef hPos_scroll_Panel As HScrollBar, _
    ByRef vPos_scroll_Panel As VScrollBar, _
    ByRef FormObj_Panel As Form, _
    ByVal hDispBoxTitle_text As String, _
    ByRef autoAddPolicy As Integer)
    
    ' -------------------------------------------------------------
    ' Create RAM to hold the params for this new Display.
       
    Dim bNum, TextHeight As Integer
    
    gNumBoxes = gNumBoxes + 1           ' increment # of DisplayBoxes
    
    ReDim Preserve gBox(gNumBoxes)      ' make the gBox array one larger
    
    bNum = gNumBoxes - 1                ' work w/ last, new, box
                    
    ' -------------------------------------------------------------
    ' Initialize.
    
    With gBox(bNum)
        
        ' -------------------------------------------------------------
        ' Update the objects.
        
        Set .FormObj = FormObj_Panel    ' window containing DispBox
        Set .BoxObj = BoxObj_Panel      ' PictureBox object on form

                                        ' scrollbars, = "Nothing" if not there
        Set .hPos_scrollObj = hPos_scroll_Panel
        Set .vPos_scrollObj = vPos_scroll_Panel
        
                                        ' set up scrollbar to help with
                                        ' creating even numbers
        If ValidObject(.hPos_scrollObj) Then
        
            gMethod_hPos_scrollObj_Enabled = False
            .hPos_scrollObj.Value = 0
            .hPos_scrollObj.Min = 0
            .hPos_scrollObj.Max = 30000
            .hPos_scrollObj.SmallChange = 3000
            .hPos_scrollObj.LargeChange = 6000
            gMethod_hPos_scrollObj_Enabled = True
            
        End If
                                        ' set up scrollbar to help with
                                        ' creating even numbers
        If ValidObject(.vPos_scrollObj) Then
               
            gMethod_vPos_scrollObj_Enabled = False
            .vPos_scrollObj.Value = 15000
            .vPos_scrollObj.Min = 0
            .vPos_scrollObj.Max = 30000
            .vPos_scrollObj.SmallChange = 1500
            .vPos_scrollObj.LargeChange = 6000
            gMethod_vPos_scrollObj_Enabled = True
            
        End If
        
        ' -------------------------------------------------------------
        ' Initialize fundamental parameters. If you don't like any of these,
        ' you can change them after calling this Initialize routine.
       
                                        ' text at top of DisplayBox, that labels the box
        .hDispBoxTitle_text = hDispBoxTitle_text
        
        .numMkrs = 0                    ' no markers in the DispBox at this time
        
        .supportMarkers = False         ' don't support markers for now -- if
                                        ' one wants them, them must call
                                        ' Add_One_Marker_To_OneBox().
        
        .numDisp = 0                    ' this DispBox contains no Displays at this time
                                        ' (we add them later)
                
        .autoAddPolicy = autoAddPolicy  ' automatically add DigiChan() channels
                                        ' to DisplayBox
                                        
        ' -------------------------------------------------------------
        ' Initialize the default grid line space for each DisplayBox.
        ' The following params are for each Display inside this DisplayBox.
         
        ' # of horiz grid lines, e.g. 3 (0 to turn off grid, -1 = autoselect)
        .numGridLines_horiz_default = AUTO_GRID
        
        ' # of vertical grid lines, e.g. 5 (0 to turn off grid, -1 = autoselect)
        .numGridLines_vert_default = AUTO_GRID
        
        ' # of tics per vert grid, e.g. 5 (-1 = autoselect)
        .ticksPerGrid_vert_default = AUTO_GRID
        
        ' # of tics per horiz grid, e.g. 3 (-1 = autoselect)
        .ticksPerGrid_horiz_default = AUTO_GRID
       
        ' support drawing a grid
        .supportGrid_default = True
        
        ' -------------------------------------------------------------
        ' Initialize DispBox coords -- the plotting cordinate system
        ' uses these values.
        
        .BoxObj.ScaleMode = 3              ' Pixel coordinate system
        
        ' x/y plotting coordinate system for this DisplayBox.
        .dispBoxLeft_xCoord = .BoxObj.ScaleLeft
        .dispBoxRight_xCoord = .BoxObj.ScaleLeft + .BoxObj.ScaleWidth
        .dispBoxTop_yCoord = .BoxObj.ScaleTop
        .dispBoxBot_yCoord = .BoxObj.ScaleTop + .BoxObj.ScaleHeight
               
        ' put left edge of vertical label box 100 pixels from right edge of DisplayBox
        .vLabelBoxLeft_xCoord = .dispBoxRight_xCoord - _
            (.BoxObj.TextWidth("-8.88E-88") + 10)
        
        ' calc text hight in pixel units
        TextHeight = .BoxObj.TextHeight(.hDispBoxTitle_text + "A")
        
        ' put lower Horizontal Label region at the 20 lowest pixels in the DisplayBox
        .hLowerLabel_Top_yCoord = .dispBoxBot_yCoord - (TextHeight + 2 * (vSPACE_yPix))
        
        ' put bottom of the Upper Horizontal Label region at the 20pix below the
        ' top of the DisplayBox (and above the top-most Display)
        .hUpperLabel_Bot_yCoord = .dispBoxTop_yCoord + (TextHeight + 2 * (vSPACE_yPix))
        
        ' -------------------------------------------------------------
        ' If we are to automatically add channels to display, then do so now
        
        ' if we are automatically add channels to this box...
        If (.autoAddPolicy <> hAutoAddNone) Then
        
            ' automatically add Digitize Channels to this display box
            AutoAddChannelsToThisDisplayBox (bNum)
            
        End If
        
        ' -------------------------------------------------------------
        ' Reset the DispBox horiz/vertical position/scale controls
    
        Call Reset_Controls_OneBox(bNum, gBox(bNum), 255)
    
    End With
    
End Sub

' This is a low level routine that is typically not used by the end user.
'
' Adds a display to DisplayBox # 'bNum'.  'displayType' is the type
' of display = {kYindexDisplay, kXYDisplay}.
'
Sub LastDisplay_Initialize( _
    ByVal bNum As Integer, _
    ByVal displayType As Integer)

    Dim dNum_newDisplay As Integer
    Dim dNum As Integer
   
    With gBox(bNum)
    
        ' -------------------------------------------------------------
        ' Get the dNum of the last display (which is the one
        ' we are initializing).
        
         dNum_newDisplay = .numDisp - 1
                        
        ' -------------------------------------------------------------
        ' Set the display type (y, xy).
        
        .disp(dNum_newDisplay).displayType = displayType
        
        ' -------------------------------------------------------------
        ' Reposition all plot & labels Region within the DisplayBox.
    
        Dim top_pixel, bot_pixel As Integer
        Dim dispHeight_pixel As Double
            
        plotRegionHeight_yCoord = .hLowerLabel_Top_yCoord - .hUpperLabel_Bot_yCoord
        dispHeight_pixel = plotRegionHeight_yCoord / .numDisp
        top_pixel = .hUpperLabel_Bot_yCoord
        
    End With
    
    ' -------------------------------------------------------------
    ' set pixel coords for plotRegion for each Display in DispBox.
    
    For dNum = 0 To (gBox(bNum).numDisp - 1)
    
        With gBox(bNum).disp(dNum)
         
            ' Take same left and right pixel positions as DispBox itself,
            ' minus the Vertical label region at the right.
            .dispLeft_xCoord = gBox(bNum).dispBoxLeft_xCoord
            .dispRight_xCoord = gBox(bNum).vLabelBoxLeft_xCoord
            
             ' Take same left and right pixel positions as DispBox itself.
             bot_pixel = ((dNum + 1) * dispHeight_pixel) + gBox(bNum).hUpperLabel_Bot_yCoord
            
            ' Assign plotRegion top/bottom pixel coords.
            .dispBot_yCoord = bot_pixel
            .dispTop_yCoord = top_pixel
            
            ' Bottom becomes the new top.
            top_pixel = bot_pixel
        
            ' Calc display height and width.
            .dispHeight_yCoord = .dispBot_yCoord - .dispTop_yCoord
            .dispWidth_xCoord = .dispRight_xCoord - .dispLeft_xCoord
        
            ' Establishes the vertical/horiz grid for this display.
            Call Grid_SetSpacing(gBox(bNum), _
                gBox(bNum).disp(dNum), _
                gBox(bNum).supportGrid_default, _
                gBox(bNum).numGridLines_horiz_default, _
                gBox(bNum).numGridLines_vert_default, _
                gBox(bNum).ticksPerGrid_vert_default, _
                gBox(bNum).ticksPerGrid_horiz_default)
       
       End With
  
    Next dNum
        
    ' -------------------------------------------------------------
    ' MARKER
            
    With gBox(bNum).disp(dNum_newDisplay)
    
        ' clear the yEU wave values at each marker
        For mkrNum = 0 To (MAX_NUM_MKRS - 1)
            .valueAtMarker_yEU(mkrNum) = 0
        Next mkrNum
        
    End With
  
    ' -------------------------------------------------------------
    ' Reset the DispBox horiz/vertical position/scale controls to default value.
    '
    ' Sets the following HORIZONTAL SCALE params: xEU left/right,
    ' ptNum left/right .
    '
    ' Sets the following  VERTICAL SCALE params:
    '
    '   .yCoord_Per_yEU     yCoord per yEU
    '   .plotOffset_yEU     plot offset, yEU units
    '   .dispHeight_yEu     height of display, yEU units
    '
    '   .dispTop_yEu        top of display, yEU units
    '   .dispBot_yEu        bottom of display, yEU units
    '   .dispMid_yEu        middle of display, yEU units

    Call Reset_Controls_OneBox(bNum, gBox(bNum), _
                reset_hPosition_toZero + _
                reset_hScale_toOneScanWideDisplays + _
                reset_ptsPerHscale + _
                reset_vScalePosScrollBtns + _
                reset_mkrs_leftEdge)

End Sub

' Add one marker to the DisplayBox.
Public Sub Add_One_Marker_To_OneBox(ByVal bNum As Integer)
    
    With gBox(bNum)
    
        If .supportMarkers = True Then
                    
            .markerPos_xEU(.numMkrs) = 0#       ' marker position, in xEU coordinates
            .markerPos_dNum(.numMkrs) = TOP_DISPLAY_NUM
                                                ' this is the dNum of the display within the
                                                ' dispBox that holds the marker (the marker refers
                                                ' to the value of the wave in this 1 display).
            
            .prevMarkerPos_xEU(.numMkrs) = 0    ' previous marker position
            .prevMarkerPos_dNum(.numMkrs) = NO_PREVIOUS_MKR
            
            .numMkrs = .numMkrs + 1             ' # markers in the DispBox at this time
            
        End If
        
    End With
    
End Sub

' Set the grid spacing for 1 Display.  If these value are set to AUTO_GRID = -1,
' then the code selects something automatically that is pleasing to the eye.

Public Sub Grid_SetSpacing( _
    ByRef thisBox As DispBoxParm, _
    ByRef thisDisp As DispParm, _
    ByVal supportGrid As Integer, _
    ByVal numGridLines_horiz As Integer, _
    ByVal numGridLines_vert As Integer, _
    ByVal ticksPerGrid_vert As Integer, _
    ByVal ticksPerGrid_horiz As Integer)
                          
    With thisDisp
    
        ' # of horiz grid lines, e.g. 3 (0 to turn off grid, -1 = autoselect)
        .numGridLines_horiz = numGridLines_horiz
        
        ' # of vertical grid lines, e.g. 5  (0 to turn off grid, -1 = autoselect)
        .numGridLines_vert = numGridLines_vert
        
        ' # of tics per vert grid, e.g. 3 (-1 = autoselect)
        .ticksPerGrid_vert = ticksPerGrid_vert
        
        ' # of tics per horiz grid, e.g. 3 (-1 = autoselect)
        .ticksPerGrid_horiz = ticksPerGrid_horiz
                      
        ' if true, support drawing a grid
        .showGrid = supportGrid
                
        ' -----------------------------------------------------
        ' Calc grid params if doing AUTO_GRID (let computer decide grid spacing).
        
        .numGridLines_horiz_calc = .numGridLines_horiz
        If .numGridLines_horiz = AUTO_GRID Or .numGridLines_horiz = 0 Then
        
            ' Calc # of horiz grid lines in 1 Display
            .numGridLines_horiz_calc = .dispHeight_yCoord / pixels_per_grid_AUTO_GRID
            
            ' The display looks better if there is an odd # of lines,
            ' so that we have a zero line.
            If (.numGridLines_horiz_calc / 2) = Int((.numGridLines_horiz_calc + 1) / 2) Then
                .numGridLines_horiz_calc = .numGridLines_horiz_calc + 1
            End If
            
        End If
        
        .numGridLines_vert_calc = .numGridLines_vert
        If .numGridLines_vert = AUTO_GRID Or .numGridLines_vert = 0 Then
        
            ' Calc # of vert grid lines in 1 Display.
            .numGridLines_vert_calc = .dispWidth_xCoord / pixels_per_grid_AUTO_GRID
        
            ' The display looks better if there is an odd # of lines,
            ' so that we have a zero line.
            If (.numGridLines_vert_calc / 2) = Int((.numGridLines_vert_calc + 1) / 2) Then
                .numGridLines_vert_calc = .numGridLines_vert_calc + 1
            End If
            
        End If
        
        .ticksPerGrid_vert_calc = .ticksPerGrid_vert
        If .ticksPerGrid_vert = AUTO_GRID Or .ticksPerGrid_vert = 0 Then
            .ticksPerGrid_vert_calc = 3
        End If
        
        .ticksPerGrid_horiz_calc = .ticksPerGrid_horiz
        If .ticksPerGrid_horiz = AUTO_GRID Or .ticksPerGrid_horiz = 0 Then
            .ticksPerGrid_horiz_calc = 3
        End If
        
        ' -----------------------------------------------------
        ' Calc x/y space between each grid.
        
        .gridWidth_xCoord = (.dispRight_xCoord - .dispLeft_xCoord) / (.numGridLines_vert_calc + 1)
        .gridHeight_yCoord = (.dispBot_yCoord - .dispTop_yCoord) / (.numGridLines_horiz_calc + 1)
        
    End With
    
End Sub

' Automatically adds digitize channels to this display box.
'
Public Sub AutoAddChannelsToThisDisplayBox(ByVal bNum As Integer)

    With gBox(bNum)
        
        ' if this DisplayBox has AutoAdd turned off, then bail out
        If (.autoAddPolicy = hAutoAddNone) Then Exit Sub
                
        ' -------------------------------------------------------------
        ' Start off with no channels this box.
    
        .numDisp = 0
        ReDim Preserve .disp(0)
    
        ' -------------------------------------------------------------
        ' Add all channels in the gDigiChan() list to this DisplayBox.
        
        ' if there are not digitize channels, then bail out
        If (gNumDigiCh <= 0) Then Exit Sub
        
        ' if Automatically add time plots of digitized waves...
        If (.autoAddPolicy = hAutoAddTimeWaves) Then
        
            Call Add_DigitizeChannels_To_OneBox(bNum, 0, gNumDigiCh - 1, kYindexDisplay)
    
        ' else if Automatically add fft's of digitized waves...
        ElseIf (.autoAddPolicy = hAutoAddFreqWaves) Then

            Call Add_DigitizeChannels_To_OneBox(bNum, 0, gNumDigiCh - 1, kXYDisplay)
        
        ' else if automatically add xy plots of digitized waves...
        ElseIf (.autoAddPolicy = hAutoAddXYplots) Then
       
            MsgBox ("hAutoAddXYplots Not yet coded.")
            EndProgram ("")
            
        End If
    
    End With
    
End Sub

' Add_DigitizeChannels_To_OneBox()
'
' Add channels gDigiChan('cNum_top') to gDigiChan('cNum_bot')
' to DisplayBox # 'bNum'.  gDigiChan(..) channels are defined with
' DigitizeList_AddOneItem().
'
' Example: Supppose we created 6 channels with DigitizeList_AddOneItem()
' and we created 2 DisplayBoxes with Add_One_OneBox_To_Window(), and we want to
' put the 1st 3 channels into 1 box, and the later 3 channels into the
' 2nd box.  We would then call:
'
'   Add_DigitizeChannels_To_OneBox(0=dispBoxNum, 0=digiChNum_top, 2=digiChanNum_bot)
'   Add_DigitizeChannels_To_OneBox(1=dispBoxNum, 3=digiChNum_top, 5=digiChanNum_bot)
'
' 'displayType' is the type of display = {kYindexDisplay, kXYDisplay}.
'
Public Sub Add_DigitizeChannels_To_OneBox( _
    ByVal bNum As Integer, _
    ByVal cNum_top As Integer, _
    ByVal cNum_bot As Integer, _
    ByVal displayType As Integer)

    Dim cNum, dNum As Integer
        
    With gBox(bNum)
    
         For cNum = cNum_top To cNum_bot
                    
            ' -------------------------------------------------------------
            ' Create RAM to hold the params for this new Display.
            
            ' inc the # of Displays in this DispBox
            .numDisp = .numDisp + 1
            
            dNum_newDisplay = .numDisp - 1
            
            ' Make the disp() array larger. This array contains params for
            ' each display.
            ReDim Preserve .disp(.numDisp)
                                                
            ' -------------------------------------------------------------
            ' Attach the new display to the digitize data buffers.
            
            ' attach the display to the digitize channel.
            .disp(dNum_newDisplay).cNum = cNum
            
            ' since this display is linked to a digitize channel,
            ' then set its local data to nil
            .disp(dNum_newDisplay).dataArrayY.ptsPerXunit = 0
            .disp(dNum_newDisplay).dataArrayY.firstPt_Xunits = 0
            .disp(dNum_newDisplay).dataArrayY.numValidPts = 0
            .disp(dNum_newDisplay).dataArrayY.numBufferPts = 0
            ReDim .disp(dNum_newDisplay).dataArrayY.data(0)
                                  
            ' we set the X array to nil as well (only used w/ XY plots)
            .disp(dNum_newDisplay).dataArrayX.ptsPerXunit = 0
            .disp(dNum_newDisplay).dataArrayX.firstPt_Xunits = 0
            .disp(dNum_newDisplay).dataArrayX.numValidPts = 0
            .disp(dNum_newDisplay).dataArrayX.numBufferPts = 0
            ReDim .disp(dNum_newDisplay).dataArrayX.data(0)
            
            ' give the Display the name of the channel
            Call iNet_VB_GetFieldcStr(cNum, gDigiChan(), sgt_General, _
                 fldNum_General_userName, .disp(dNum_newDisplay).name)
                  
            ' attach the digiChan to the display
            gDigiChan(dNum_newDisplay).bNum = bNum
            gDigiChan(dNum_newDisplay).dNum = dNum_newDisplay
                                     
            ' -------------------------------------------------------------
            ' Initialize this new display
            
            Call LastDisplay_Initialize(bNum, displayType)
        
        Next cNum
    
    End With
    
End Sub


' Add_NonDigitizeDisplay_To_OneBox().
'
' Adds a non-digitize channel to a display. Non-digitize means
' that the code does not automatically place a digitized waveform
' into the display.  Data is stored in
'
'   gBox(bNum).disp(dNum).dataArrayY.data(ptNum)
'
' where bNum is this DisplayBox index, dNum is the Display index
' within the box, an ptNum is the point index within the data array.
' The following hold important information about the data:
'
'   gBox(bNum).disp(dNum).dataArrayY.ptsPerXunit - data sample rate
'   gBox(bNum).disp(dNum).dataArrayY.numBufferPts - data array size
'   gBox(bNum).disp(dNum).dataArrayY.numValidPts - # of valid data pts
'   gBox(bNum).disp(dNum).dataArrayY.data() - actual data array
'
' PARAMETERS:
'
' bNum - DisplayBox index in the main window = {0...gNumBoxes-1}
' displayType - type of display = {kYindexDisplay, kXYDisplay}.
' numValidPts - number of valid pts in the data array
' numBufferPts - data array size
' ptsPerXunit - data array sample rate, in pts per horizontal x units
' name - name of the Display
'
Public Sub Add_NonDigitizeDisplay_To_OneBox( _
    ByVal bNum As Integer, _
    ByVal displayType As Integer, _
    ByVal numValidPts As Long, _
    ByVal numBufferPts As Long, _
    ByVal ptsPerXunit As Single, _
    ByVal name As String)
           
    ' -------------------------------------------------------------
    ' Create RAM to hold the params for this new Display.
     
    ' inc the # of Displays in this DispBox
    gBox(bNum).numDisp = gBox(bNum).numDisp + 1
     
    dNum_newDisplay = gBox(bNum).numDisp - 1
     
    ' set disp() array size to the # of Displays
    ReDim Preserve gBox(bNum).disp(gBox(bNum).numDisp)
                                     
    With gBox(bNum).disp(dNum_newDisplay)

        ' -------------------------------------------------------------
        ' Set up local data buffer.

        ' we are NOT attached to a digitize channel
        .cNum = NO_DIGI_CHAN

        ' sample rate of the local_data(), in pts-per-plotted-xUnit
        .dataArrayY.ptsPerXunit = ptsPerXunit
        
        ' first point time in seconds
        .dataArrayY.firstPt_Xunits = 0
        
        ' # of valid points contained within the local_data() buffer.
        .dataArrayY.numValidPts = numValidPts

        ' data() buffer size
        .dataArrayY.numBufferPts = numBufferPts

        ' create ram to hold the data, add 2 for saftey
        ReDim .dataArrayY.data(numBufferPts + 2)

        ' zero the data
        last_ptNum& = numValidPts - 1
        For ptNum& = 0 To last_ptNum&
            .dataArrayY.data(ptNum&) = 0
        Next ptNum&

        ' if doing Y vs index Display...
        If (displayType = kYindexDisplay) Then
        
            ' set the X array to nil, since we only use the Y array
            .dataArrayX.ptsPerXunit = 1
            .dataArrayX.firstPt_Xunits = 0
            .dataArrayX.numValidPts = 0
            .dataArrayX.numBufferPts = 0
            ReDim .dataArrayX.data(.dataArrayX.numBufferPts)
        
        ' else we are doing xy displays, and need to set up the X data array
        Else
        
            ' setup the internal x axis data array the same as the y array
            .dataArrayX.ptsPerXunit = .dataArrayY.ptsPerXunit
            .dataArrayX.firstPt_Xunits = .dataArrayY.firstPt_Xunits
            .dataArrayX.numValidPts = .dataArrayY.numValidPts
            .dataArrayX.numBufferPts = .dataArrayY.numBufferPts
            ReDim .dataArrayX.data(.dataArrayX.numBufferPts + 2)
            
        End If
        
        ' display name
        .name = name

    End With
                                                                              
    ' -------------------------------------------------------------
    ' Initialize this new display.
    
    Call LastDisplay_Initialize(bNum, displayType)
                  
End Sub


' Returns true if this is a valid object that has been "set"
Public Function ValidObject(ByRef obj As Variant) As Integer

    ValidObject = False
    
    ' if we hit an error, jump to ErrorHandler
    On Error GoTo ErrorHandler
    
    ' test object
    If IsEmpty(obj) Then Exit Function
    If IsNull(obj) Then Exit Function
    If Not (IsObject(obj)) Then Exit Function
    
    ' if it Error's on the following line, then the obj might not be set
    hasValue = obj
    
    ValidObject = True

ErrorHandler:

    ' if we hit an error, there is a problem and the
    ' object is probably not there
    
End Function

' This is a low level routine that is typically not used by the end user.
'
' This changes the scale by a scaleMult, and then we update
' the vertical scale (dispTop_xEU, dispBot_xEU, etc).
'
'   INPUT    scaleMult, yCoord_Per_yEU, etc
'
'   UPDATES  dispHeight_yEu

Sub vScale_ChangeScale_OneBox( _
    ByVal bNum As Integer, _
    ByVal scaleMult As Double)
    
    Dim dispHeight_yEU_NEW, dispTop_yEu_NEW, dispBot_yEu_NEW As Double
    
    ' don't do anything in here if there are no displays in this DisplayBox.
    If (gBox(bNum).numDisp <= 0) Then Exit Sub
                    
    ' go thru each Display in this Box
    For dNum = 0 To gBox(bNum).numDisp - 1
        
        ' with this Display...
        With gBox(bNum).disp(dNum)
        
            ' calc new height w/ the scaleMult
            dispHeight_yEU_NEW = .dispHeight_yEu * scaleMult
            
            ' set the display top/bottom given the new height
            dispTop_yEu_NEW = .dispMid_yEu + (dispHeight_yEU_NEW / 2)
            dispBot_yEu_NEW = .dispMid_yEu - (dispHeight_yEU_NEW / 2)
            
            ' set the new vScale
            Call vScale_SetWithNew_yEU_values_OneDisp( _
                bNum, dNum, dispTop_yEu_NEW, dispBot_yEu_NEW)
                                
        End With
    
    Next dNum
            
End Sub


' This is a low level routine that is typically not used by the end user.
'
' This changes the scale by a scaleMult, and then we update
' the horizontal scale (dispLeft_xEU, dispRight_xEU, etc).
'
'   INPUT    scaleMult, xCoord_Per_xEU, etc
'
'   UPDATES  dispWidth_xEu

Sub hScale_StretchScale_OneBox( _
    ByVal bNum As Integer, _
    ByVal scaleMult As Double)
    
    Dim dispWidth_xEU_NEW, dispLeft_xEU_NEW, dispRight_xEU_NEW As Double
    
    ' don't do anything in here if there are no displays in this DisplayBox.
    If (gBox(bNum).numDisp <= 0) Then Exit Sub
                    
    ' go thru each Display in this Box
    For dNum = 0 To gBox(bNum).numDisp - 1
        
        ' with this Display...
        With gBox(bNum).disp(dNum)
                    
            ' calc new width w/ the scaleMult
            dispWidth_xEU_NEW = .dispWidth_xEU * scaleMult
            
            ' if xy display...
            If (.displayType = kXYDisplay) Then
            
                ' expand out from the center
                Dim dispMid_xEU As Double
                dispMid_xEU = ((.dispRight_xEU - .dispLeft_xEU) / 2) + .dispLeft_xEU
                dispLeft_xEU_NEW = dispMid_xEU - (dispWidth_xEU_NEW / 2)
                dispRight_xEU_NEW = dispMid_xEU + (dispWidth_xEU_NEW / 2)
            
            ' else if y vs index display...
            Else
            
                ' expand from left edge
                dispLeft_xEU_NEW = .dispLeft_xEU
                dispRight_xEU_NEW = dispLeft_xEU_NEW + dispWidth_xEU_NEW
                
            End If
            
            ' set the new vScale
            Call hScale_SetWithNew_xEU_values_OneDisp(bNum, dNum, _
                dispLeft_xEU_NEW, dispRight_xEU_NEW, gBox(bNum).BoxObj)
                                
        End With
    
    Next dNum
            
End Sub

' This is a low level routine that is typically not used by the end user.
'
' Given a new dispTop_yEU/dispBot_yEU, we update the vertical scale
' (dispTop_xEU, dispBot_xEU, etc) for 1 Display.
'
'   INPUT    dispTop_yEU/dispBot_yEU
'
'   UPDATES  dispMid_yEu, dispHeight_yEu, yCoord_Per_yEU, plotOffset_yEU.
'
Sub vScale_SetWithNew_yEU_values_OneDisp( _
    ByVal bNum As Integer, _
    ByVal dNum As Integer, _
    ByVal dispTop_yEu As Double, _
    ByVal dispBot_yEu As Double)
    
    ' don't do anything in here if there are no displays in this DisplayBox.
    If (gBox(bNum).numDisp <= 0) Then Exit Sub
   
    With gBox(bNum).disp(dNum)

        .dispTop_yEu = dispTop_yEu                          ' top of display, yEU units
        .dispBot_yEu = dispBot_yEu                          ' bottom of display, yEU units

        .dispHeight_yEu = dispTop_yEu - dispBot_yEu         ' height of display, yEU units
        .dispMid_yEu = .dispBot_yEu + (.dispHeight_yEu / 2) ' middle of display, yEU units

        ' yCoord per yEU (used w/ yCoord to yEU translation)
        .yCoord_Per_yEU = (.dispBot_yCoord - .dispTop_yCoord) / (.dispBot_yEu - .dispTop_yEu)

        ' Debugging:
        '   a = .plotOffset_yEU: b = .dispBot_yCoord: c = .yCoord_Per_yEU: d = .dispBot_yEu

        ' yCoord = (yEU + .plotOffset_yEU ) * .yCoord_Per_yEU
        .plotOffset_yEU = (.dispBot_yCoord / .yCoord_Per_yEU) - .dispBot_yEu

        ' yEU = (yCoord + .plotOffset_yCoord ) / .yCoord_Per_yEU
        .plotOffset_yCoord = (.dispBot_yEu * .yCoord_Per_yEU) - .dispBot_yCoord

        ' For debugging:
        '   temp = .dispBot_yCoord: tempB = .dispTop_yCoord: tempC = .dispHeight_yEu
        '   tempD = .yCoord_Per_yEU: tempE = .plotOffset_yCoord: tempF = .plotOffset_yEU

        ' update the vertical scale labels
        Call vScale_DrawLabel_OneDisp(bNum, dNum, gBox(bNum).disp(dNum), gBox(bNum).BoxObj)
        
    End With

End Sub


' This is a low level routine that is typically not used by the end user.
'
' Draws vScale labels to right of display.
Sub vScale_DrawLabel_OneDisp( _
    ByVal bNum As Integer, _
    ByVal dNum As Integer, _
    ByRef thisDisp As DispParm, _
    ByRef PlotObj As Variant)

    Dim yEU As Double
    Dim yCoord_double As Double
    Dim yEU_str As String
    Dim textWidth_xCoord, textHeight_yCoord, yCoord_int, needToPrintName As Integer
    needToPrintName = True
    
    With gBox(bNum)
    
        ' don't do anything in here if there are not displays in this DisplayBox.
        If (.numDisp <= 0) Then Exit Sub
              
        Call Set_AutoRedraw_Property(True, PlotObj)
        
        ' -----------------------------------------------------
        ' fill the entire vLabel region w/ white
        
        PlotObj.Line _
            ((.vLabelBoxLeft_xCoord + 1), (thisDisp.dispTop_yCoord + 1)) _
           -((.dispBoxRight_xCoord - 1), (thisDisp.dispBot_yCoord - 1)) _
            , &HFFFFFF, BF
        
        ' -----------------------------------------------------
        ' Draw vScale (e.g. "0   .1   .2   .3   .4 secs" vertically,
        ' one label w/ each grid line
        
        Const vLABEL_SPACE = 4
            
        ' select black line style
        Call Set_DrawStyle_OneBox(bNum, kBlack, PlotObj)
        
        textHeight_yCoord = .BoxObj.TextHeight("-23E-03")
        
        For i = 0 To (thisDisp.numGridLines_horiz_calc + 1)
            
            ' calc yCoord of vertical line
            yCoord_double = thisDisp.dispBot_yCoord - (thisDisp.gridHeight_yCoord * i)
            yCoord_int = yCoord_double
            
            ' get yEU value for this grid line
        
            ' yEU = (yCoord + .plotOffset_yCoord ) / .yCoord_Per_yEU
            yEU = (yCoord_double + thisDisp.plotOffset_yCoord) / thisDisp.yCoord_Per_yEU
            
            '  if the display height is > .1, then round off to nearest .001
            If thisDisp.dispHeight_yEu > 0.1 Then
               yEU = Int((yEU * 1000#) + 0.5) / 1000#
            End If
            
            yEU_str = NumToStr(yEU)
            
            ' if this is the center line, then print the channel name
            If (i = Int((.disp(dNum).numGridLines_horiz_calc / 2) + 0.51)) And needToPrintName Then
                yEU_str = .disp(dNum).name
                needToPrintName = False
            End If
                        
            If i = 0 Then           ' if we are at the bottom, then push up
                PlotObj.CurrentY = yCoord_int - (textHeight_yCoord + (vLABEL_SPACE / 2))
                                    ' if we are at the top, push down
            ElseIf (i = (.disp(dNum).numGridLines_horiz_calc + 1)) Then
                PlotObj.CurrentY = yCoord_int + (vLABEL_SPACE / 2)
            Else
                PlotObj.CurrentY = yCoord_int - (textHeight_yCoord / 2)
            End If
                                    
            ' set the upper left coord of the printed label
            PlotObj.CurrentX = .vLabelBoxLeft_xCoord + vLABEL_SPACE
                        
            ' print the xEU scale value
            PlotObj.Print yEU_str
NextGrid:
        Next i
    
        Call Set_AutoRedraw_Property(False, PlotObj)
    
    End With
        
End Sub

' Increment vertical position of display.
' Invoked by a Click on vPos_increment button.
Public Sub METHOD_vPos_increment_Click(ByVal bNum As Integer)
     
     ' don't do anything in here if there are not displays in this DisplayBox.
    If (gBox(bNum).numDisp <= 0) Then Exit Sub
        
    ' dec vertical scale of DispBox
    Call vScale_ChangeScale_OneBox(bNum, 2)

    ' redraw 1 DispBox
    Call Redraw_OneBox(bNum, gBox(bNum).BoxObj)

End Sub

' Decrement vertical position of display.
' Invoked by a Click on vPos_decrement button.
Public Sub METHOD_vPos_decrement_Click(ByVal bNum As Integer)
     
     ' don't do anything in here if there are not displays in this DisplayBox.
    If (gBox(bNum).numDisp <= 0) Then Exit Sub
        
    ' dec vertical scale of DispBox
    Call vScale_ChangeScale_OneBox(bNum, 0.5)

    ' redraw 1 DispBox
    Call Redraw_OneBox(bNum, gBox(bNum).BoxObj)

End Sub

' Update vertical position of a display.
' Invoked by the change of the Vertical Scroll bar.
Public Sub METHOD_vPos_scroll_Change(ByVal bNum As Integer)
    
    Dim deltaScrollValue_Normalized, delta_yEU As Double
      
    With gBox(bNum)
    
        ' don't do anything in here if there are not displays in this DisplayBox.
        If (.numDisp <= 0) Then Exit Sub
   
        ' if the scrollbar does not exist, bail out
        If Not (ValidObject(.vPos_scrollObj)) Then Exit Sub
                 
        ' if this method has been turned off, then bail out
        If gMethod_vPos_scrollObj_Enabled = False Then Exit Sub
        
        ' +1 if moved up 1 full bar, -1 if moved down 1 full bar
        deltaScrollValue_Normalized = _
            (.vPos_scrollObj.Value - .vPos_scroll_lastValue) / _
            (.vPos_scrollObj.Max - .vPos_scrollObj.Min)
        
        ' if there was no change, then bail out
        If deltaScrollValue_Normalized = 0 Then Exit Sub
        
        ' remember the new value of the scrollbar
        .vPos_scroll_lastValue = .vPos_scrollObj.Value
                   
        ' scroll bar adds +4 to -4 times DisplayHeight
        Const Scroll_Range = 4
        
        ' convert to a -4 to +4 value
        deltaScrollValue_Normalized = deltaScrollValue_Normalized * Scroll_Range
                
        ' update all displays in this box
        For dNum = 0 To (.numDisp - 1)
            
            With gBox(bNum).disp(dNum)
    
                ' calc the amount of change
                delta_yEU = .dispHeight_yEu * deltaScrollValue_Normalized
                
                ' Update vert scale given new yEU Top/Bot values.
                Call vScale_SetWithNew_yEU_values_OneDisp(bNum, dNum, _
                    (.dispTop_yEu + delta_yEU), (.dispBot_yEu + delta_yEU))
        
            End With
            
        Next dNum
        
        ' Clear 1 DispBox and then Redraw it
        Call Redraw_OneBox(bNum, .BoxObj)
            
    End With

End Sub


' This is a low level routine that is typically not used by the end user.
'
' Clear DispBox
Public Sub Box_Clear_OneBox(ByRef PlotObj As Variant)
          
    ' turn on AutoRedraw
    Call Set_AutoRedraw_Property(True, PlotObj)
    
    ' if this is a screen PictureBox, then clear it
    If TypeName(PlotObj) <> TypeName(Printer) Then PlotObj.Cls
    
    ' turn off AutoRedraw
    Call Set_AutoRedraw_Property(False, PlotObj)
            
End Sub


' Sets the AutoDraw property (which enables automatically drawing when
' window is reactivated
Public Sub Set_AutoRedraw_Property(ByVal newAutoDrawMode As Integer, ByRef PlotObj As Variant)
    
    ' if writing to the screen instead of the Printer...
    If TypeName(PlotObj) <> TypeName(Printer) Then
    
        ' set the auto draw mode
        PlotObj.AutoRedraw = newAutoDrawMode
        
    End If
    
End Sub
' This is a low level routine that is typically not used by the end user.
'
' Clear plot region for this display box
Public Sub Plot_Clear_OneBox(ByVal bNum As Integer)
               
    Dim dNum As Integer
                              
    For dNum = 0 To (gBox(bNum).numDisp - 1)
        
        Call Plot_Clear_OneDisp(bNum, dNum, gBox(bNum).BoxObj)
    
    Next dNum
    
End Sub

' This is a low level routine that is typically not used by the end user.
'
' Clear plot region for this display
Public Sub Plot_Clear_OneDisp( _
    ByVal bNum As Integer, _
    ByVal dNum As Integer, _
    ByRef PlotObj As Variant)
                                        
    Call Set_AutoRedraw_Property(True, PlotObj)
            
    With gBox(bNum).disp(dNum)
    
         ' Fill the plot region w/ white.
         PlotObj.Line _
              ((.dispLeft_xCoord + 1), (.dispTop_yCoord + 1))- _
              ((gBox(bNum).vLabelBoxLeft_xCoord - 1), (.dispBot_yCoord - 1)), _
              &HFFFFFF, BF
     
     End With
        
    Call Set_AutoRedraw_Property(False, PlotObj)

End Sub

' This is a low level routine that is typically not used by the end user.
'
' Draw the marker in Display.  If 'drawNewMarker' is True, we draw
' the marker at the new position; otherwise, we clear the old marker.
Public Sub Marker_Draw_Line_OneBox( _
    ByVal bNum As Integer, _
    ByVal drawNewMarker As Integer, _
    ByRef thisBox As DispBoxParm, _
    ByRef PlotObj As Variant)
    
    ' don't do anything in here if there are no displays in this DisplayBox.
    If (thisBox.numDisp <= 0) Then Exit Sub
        
    ' don't do anything in here if this DispBox does not support markers
    If (thisBox.supportMarkers = False) Then Exit Sub
        
    With thisBox
    
        Dim dispTop_yCoord, dispBot_yCoord, dNum As Integer
                                                          
        Call Set_AutoRedraw_Property(True, PlotObj)
                                                  
        ' ---------------------------------------------------
        ' Clear previous markers.
        
        ' if there is a previous marker to clear...
        If (drawNewMarker = False) Then
        
            ' scan thru the markers in the display
            For mkrNum = 0 To (.numMkrs - 1)
            
                ' with display that held old marker...
                dNum = .prevMarkerPos_dNum(mkrNum)
                
                If (dNum <> NO_PREVIOUS_MKR) Then
                
                    ' with Display that held previous marker...
                    With thisBox.disp(dNum)
                    
                        ' erase old marker line
                        dispTop_yCoord = .dispTop_yCoord + 1
                        dispBot_yCoord = .dispBot_yCoord
                
                        ' calc position of previous marker
                        point_xCoord = Translate_xEU_to_xCoord(bNum, dNum, _
                            thisBox.prevMarkerPos_xEU(mkrNum))
                                     
                        ' make sure we do not overflow
                        If (point_xCoord >= thisBox.vLabelBoxLeft_xCoord) Then
                            point_xCoord = (thisBox.vLabelBoxLeft_xCoord - 1)
                        End If
                                                
                        ' draw red vertical line
                        PlotObj.DrawStyle = 0     ' solid line
                        PlotObj.DrawMode = 16     ' white draw mode
                        PlotObj.Line (point_xCoord, dispTop_yCoord) _
                                    -(point_xCoord, dispBot_yCoord)
                    
                    End With
                End If
            Next mkrNum
        End If
                                              
        ' ---------------------------------------------------
        ' Draw new marker.
                
        ' if we are being asked to draw the new marker...
        If drawNewMarker = True Then
        
          ' scan thru the markers in the display
            For mkrNum = 0 To (.numMkrs - 1)
                
                ' with Display that holds new marker...
                dNum = .markerPos_dNum(mkrNum)
                                 
                With thisBox.disp(dNum)
                   
                   ' erase old marker line
                    dispTop_yCoord = .dispTop_yCoord + 1
                    dispBot_yCoord = .dispBot_yCoord
            
                    ' calc position of previous marker
                    point_xCoord = Translate_xEU_to_xCoord(bNum, dNum, thisBox.markerPos_xEU(mkrNum))
                                                             
                    ' make sure we do not overflow
                    If (point_xCoord >= thisBox.vLabelBoxLeft_xCoord) Then
                        point_xCoord = (thisBox.vLabelBoxLeft_xCoord - 1)
                    End If
                    
                    PlotObj.DrawStyle = 0       ' solid line
                    PlotObj.DrawMode = 13       ' copy
                    PlotObj.ForeColor = &HFF&   ' red
                    PlotObj.Line (point_xCoord, dispTop_yCoord)-(point_xCoord, dispBot_yCoord)
                
                End With
                                                             
                ' there is no longer a previous a marker that needs being cleared
                .prevMarkerPos_dNum(mkrNum) = NO_PREVIOUS_MKR
            
            Next mkrNum
        End If
        
        PlotObj.DrawStyle = 0                   ' solid line
        PlotObj.DrawMode = 13                   ' copy
        PlotObj.ForeColor = &H0                 ' black
        
        Call Set_AutoRedraw_Property(False, PlotObj)
        
    End With

End Sub

' Invoked by clicking on in a DisplayBox.
Public Sub METHOD_Box_MouseDown(Index As Integer, Button As Integer, _
    Shift As Integer, x As Single, Y As Single)

   ' we use a control array to handle all displays with one function
    Dim bNum As Integer
    bNum = Index
       
    With gBox(bNum)
    
        ' don't do anything in here if there are not displays in this DisplayBox.
        If (.numDisp <= 0) Then Exit Sub
        
        ' don't do anything in here if this DispBox does not support markers
        If (.supportMarkers = False) Then Exit Sub
        
        ' remember old marker position, so we can clear it later
        For mkrNum = 0 To .numMkrs
        
            .prevMarkerPos_dNum(mkrNum) = .markerPos_dNum(mkrNum)
            .prevMarkerPos_xEU(mkrNum) = .markerPos_xEU(mkrNum)
        
            ' determine which disp in the DispBox received the mouse down.
            .markerPos_dNum(mkrNum) = Int(((Y - .hUpperLabel_Bot_yCoord) _
                / (.hLowerLabel_Top_yCoord - .hUpperLabel_Bot_yCoord)) * .numDisp)
                
            If (.markerPos_dNum(mkrNum) < 0) Then .markerPos_dNum(mkrNum) = 0
            If (.markerPos_dNum(mkrNum) >= .numDisp) Then .markerPos_dNum(mkrNum) = (.numDisp - 1)
            
            ' calc the point # of the mouse down in the Display
            .markerPos_xEU(mkrNum) = Translate_xCoord_to_xEU(bNum, .markerPos_dNum(mkrNum), x)
        
        Next mkrNum
        
    End With
        
    ' erase the old marker
    Call Marker_Draw_Line_OneBox(bNum, False, gBox(bNum), gBox(bNum).BoxObj)
    
    ' go thru all markers
    For mkrNum = 0 To gBox(bNum).numMkrs
        
        Dim dNum As Integer
        dNum = gBox(bNum).prevMarkerPos_dNum(mkrNum)
        
        ' redraw the grid (in case we nibbled on it when we erased the maker)
        Call Grid_Draw_OneDisp(bNum, dNum, _
            gBox(bNum), gBox(bNum).disp(dNum), gBox(bNum).BoxObj)
    
        ' plot the full waveform (in case we nibbled on it when we drew the grid)
        Call Plot_FullScreen_OneDisp(bNum, dNum, _
            gBox(bNum), gBox(bNum).disp(dNum), gBox(bNum).BoxObj)
    
    Next mkrNum
    
    ' draw the new marker in it's new position
    Call Marker_Draw_Line_OneBox(bNum, True, gBox(bNum), gBox(bNum).BoxObj)
   
    ' plot the wave ONLY where the pervious marker was
    For dNum = 0 To (gBox(bNum).numDisp - 1)
        Call Plot_LastMarkerPos_OneDisp(bNum, dNum, _
            gBox(bNum), gBox(bNum).disp(dNum))
    Next dNum
    
    ' update text at top of marker
    Call Marker_Draw_Text_OneBox(bNum, gBox(bNum), gBox(bNum).BoxObj)

End Sub

' Translates an xCoord value to a pointNum value for the
' {bNum, dispNum} Display.
Public Function Translate_xCoord_to_pointNum( _
    ByVal bNum As Integer, _
    ByVal dNum As Integer, _
    ByVal xCoord As Integer) As Long
        
    With gBox(bNum).disp(dNum)
    
        Translate_xCoord_to_pointNum _
            = Int((xCoord - .dispLeft_xCoord) / .xCoord_Per_ptNum) + .dispLeft_ptNum
            
    End With

End Function


' Translates an xEU value to an xCoord value for the
' {bNum, dispNum} Display.
Public Function Translate_xEU_to_xCoord( _
    ByVal bNum As Integer, _
    ByVal dNum As Integer, _
    ByVal xEU As Double) As Long
    
    With gBox(bNum).disp(dNum)
    
        Translate_xEU_to_xCoord _
            = Int((xEU - .dispLeft_xEU) * .xCoord_Per_xEU) + .dispLeft_xCoord
            
    End With

End Function

' Translates an xEU value to an pointNum value for the
' {bNum, dispNum} Display.
Public Function Translate_xEU_to_ptNum( _
    ByVal bNum As Integer, _
    ByVal dNum As Integer, _
    ByVal xEU As Double) As Long
    
    With gBox(bNum).disp(dNum)
    
        Translate_xEU_to_ptNum _
            = Int((xEU - .dispLeft_xEU) * .xCoord_Per_xEU / .xCoord_Per_ptNum) + .dispLeft_ptNum
                      
    End With

End Function

' Translates an xCoord value to a pointNum value for the
' {bNum, dispNum} Display.
Public Function Translate_xCoord_to_xEU( _
    ByVal bNum As Integer, _
    ByVal dNum As Integer, _
    ByVal xCoord As Integer) As Double
    
    With gBox(bNum).disp(dNum)
    
        x = .xCoord_Per_xEU
        
        Translate_xCoord_to_xEU _
            = ((xCoord - .dispLeft_xCoord) / .xCoord_Per_xEU) + .dispLeft_xEU
            
    End With

End Function

' This is a low level routine that is typically not used by the end user.
'
' Sets the drawing style to a gray mesh, for grid lines
Public Sub Set_DrawStyle_OneBox( _
    ByVal bNum As Integer, _
    ByVal color As Integer, _
    ByRef PlotObj As Variant)
    
    With PlotObj    ' Printer or Screen-PictureBox-BoxObj
    
        ' if doing black...
        If (color = kBlack) Then
        
            .DrawStyle = 0                  ' solid line (not dotted)
            .ForeColor = &H0                ' black
            .DrawMode = 13                  ' copy color onto screen
            
        ' else doing gray...
        Else
            
            ' if writing to the screen instead of the Printer...
            If TypeName(PlotObj) <> TypeName(Printer) Then
            
                .DrawStyle = 0              ' solid line (not dotted)
                .ForeColor = &HC0C0C0       ' light gray mesh
            
            ' else writing to the Printer
            Else
                                   
                ' Print black thin lines since many printers
                ' are black and white, and gray mesh breaks up
                
                .DrawStyle = 0              ' solid line (not dotted)
                .ForeColor = &H0            ' black
                .DrawMode = 13              ' copy color onto screen
                
            End If
        
        End If
        
    End With
    
End Sub


' This is a low level routine that is typically not used by the end user.
'
' Clear all DispBoxes
Public Sub Box_Clear_AllBoxes()

    Dim bNum As Integer
    
    For bNum = 0 To gNumBoxes - 1
    
        Call Box_Clear_OneBox(gBox(bNum).BoxObj)
        
    Next bNum

End Sub


' This is a low level routine that is typically not used by the end user.
'
' Clear all the plot regions for all the DispBoxes
Public Sub Plot_Clear_AllBoxes()

    Dim bNum As Integer
    
    For bNum = 0 To gNumBoxes - 1
    
        Plot_Clear_OneBox (bNum)
        
    Next bNum

End Sub




' Decrement horizontal scale (e.g. ms/div).
' Invoked by a CLICK on the hPos_decrement (horiz scale) button.
Public Sub METHOD_hPos_decrement_Click(ByVal bNum As Integer)
     
    ' don't do anything in here if there are not displays in this DisplayBox.
    If (gBox(bNum).numDisp <= 0) Then Exit Sub
    
    ' decrement horiz scale 50%
    Call hScale_StretchScale_OneBox(bNum, 0.5)

    Call Redraw_OneBox(bNum, gBox(bNum).BoxObj)   ' redraw 1 DispBox

End Sub

' Increment horizontal scale (e.g. ms/div).
' Invoked by a CLICK on the hPos_increment (horiz scale) button.
Public Sub METHOD_hPos_increment_Click(ByVal bNum As Integer)
     
     ' don't do anything in here if there are not displays in this DisplayBox.
    If (gBox(bNum).numDisp <= 0) Then Exit Sub
    
    ' increment horiz scale 200%
    Call hScale_StretchScale_OneBox(bNum, 2#)

    Call Redraw_OneBox(bNum, gBox(bNum).BoxObj)       ' redraw 1 DispBox

End Sub

' Invoked by clicking on the Channel Options button.
Public Sub METHOD_ChanOpt_Button_Click()

    ' Open iNet World's Network Page.
    Open_instruNet_Window_VB (iNetCM_OpenWindow_Network)
    
End Sub

' This is a low level routine that is typically not used by the end user.
'
' Update horizontal position for this DispBox.
' Invoked by key down in Horiz scrollbar.
Public Sub METHOD_hPos_scroll_KeyDown(Index As Integer, KeyCode As Integer, _
    Shift As Integer)
          
    Dim bNum As Integer
    bNum = Index
      
    ' don't do anything in here if there are not displays in this DisplayBox.
    If (gBox(bNum).numDisp <= 0) Then Exit Sub
    
    ' if the scrollbar does not exist, bail out
    If Not (ValidObject(gBox(bNum).hPos_scrollObj)) Then Exit Sub
    
    ' reset to left edge
    If KeyCode = &H30 Then
        
        ' reset the left edge to 0
        gBox(bNum).hPos_scrollObj.Value = gBox(bNum).hPos_scrollObj.Min
        
        ' update the hScroll bar
        METHOD_hPos_scroll_Change (bNum)
        
    End If

 End Sub

' This is a low level routine that is typically not used by the end user.
'
' Update horizontal position for this DispBox.
' Invoked by key down in Horiz scrollbar.
Public Sub METHOD_hPos_scroll_Change(Index As Integer)
    
    Dim dispLeft_ptNum_NEW, dispLeft_ptNum_original As Long
    Dim scrollValue_Normalized, deltaScrollValue_Normalized As Double
    Dim bNum As Integer
    bNum = Index
      
    ' don't do anything in here if there are not displays in this DisplayBox.
    If (gBox(bNum).numDisp <= 0) Then Exit Sub

    ' get the # of data pts and the data's sample rate (for TOP display)
    Dim ptsPerXunit As Double
    Dim firstPt_Xunits As Double
    Dim numValidPts As Long
    Call GetDataParams(bNum, TOP_DISPLAY_NUM, ptsPerXunit, numValidPts, firstPt_Xunits)
    
    ' if we do not have any points in the display, then don't allow scrolling
    If (numValidPts = 0) Then Exit Sub
    
    ' if this method has been turned off, then bail out
    If gMethod_hPos_scrollObj_Enabled = False Then Exit Sub
     
    ' if the scrollbar does not exist, bail out
    If Not (ValidObject(gBox(bNum).hPos_scrollObj)) Then Exit Sub
    
    ' w/ the TOP-MOST display...
    With gBox(bNum).disp(TOP_DISPLAY_NUM)
        
        ' -----------------------------------
        ' XY Display
        
        If (.displayType = kXYDisplay) Then
        
            ' +1 if moved up 1 full bar, -1 if moved down 1 full bar
            deltaScrollValue_Normalized = _
                (gBox(bNum).hPos_scrollObj.Value - gBox(bNum).hPos_scroll_lastValue) / _
                (gBox(bNum).hPos_scrollObj.Max - gBox(bNum).hPos_scrollObj.Min)
            
            ' if there was no change, then bail out
            If deltaScrollValue_Normalized = 0 Then Exit Sub
            
            ' remember the new value of the scrollbar
            gBox(bNum).hPos_scroll_lastValue = gBox(bNum).hPos_scrollObj.Value
                       
            ' scroll bar adds +4 to -4 times DisplayWidth
            Const Scroll_Range = 4
            
            ' convert to a -4 to +4 value
            deltaScrollValue_Normalized = deltaScrollValue_Normalized * Scroll_Range
                    
            ' update all displays in this box
            For dNum = 0 To (gBox(bNum).numDisp - 1)
                With gBox(bNum).disp(dNum)
                
                    ' calc the amount of change
                    delta_xEU# = .dispWidth_xEU * deltaScrollValue_Normalized
                    
                    ' Update vert scale given new yEU Top/Bot values.
                    Call hScale_SetWithNew_xEU_values_OneDisp(bNum, dNum, _
                        (.dispLeft_xEU + delta_xEU#), (.dispRight_xEU + delta_xEU#), _
                        gBox(bNum).BoxObj)
                        
                End With
            Next dNum
        
            ' Clear 1 DispBox and then Redraw it
            Call Redraw_OneBox(bNum, gBox(bNum).BoxObj)
            
        ' -----------------------------------
        ' Y vs INDEX Display
        
        Else
        
            ' remember the old left edge value
            dispLeft_ptNum_original = .dispLeft_ptNum
            
            ' 0 is all the way to the left, 1 all the way to right
            scrollValue_Normalized = _
                (gBox(bNum).hPos_scrollObj.Value - gBox(bNum).hPos_scrollObj.Min) / _
                (gBox(bNum).hPos_scrollObj.Max - gBox(bNum).hPos_scrollObj.Min)
                   
            ' remember the new value of the scrollbar
            gBox(bNum).hPos_scroll_lastValue = gBox(bNum).hPos_scrollObj.Value
            
            ' calc new left edge ptNum, based on current scroll bar value
            dispLeft_ptNum_NEW = scrollValue_Normalized * numValidPts
        
            ' don't allows the scrolling to go past the waveform end points
            If dispLeft_ptNum_NEW < 0 Then dispLeft_ptNum_NEW = 0
            If dispLeft_ptNum_NEW > numValidPts Then dispLeft_ptNum_NEW = numValidPts - 1
                
            ' if the left edge changed...
            If (dispLeft_ptNum_original <> dispLeft_ptNum_NEW) Then
            
                ' calc the new left edge xEu value
                left_xEU_NEW = dispLeft_ptNum_NEW / ptsPerXunit
                
                ' update all displays in this box
                For dNum = 0 To (gBox(bNum).numDisp - 1)
                
                    ' Update horiz scale (pos/scale) given new xEU Left/Right values.
                    Call hScale_SetWithNew_xEU_values_OneDisp(bNum, dNum, _
                        left_xEU_NEW, (left_xEU_NEW + (.dispRight_xEU - .dispLeft_xEU)), _
                        gBox(bNum).BoxObj)
                
                Next dNum
            
                ' Clear 1 DispBox and then Redraw it
                Call Redraw_OneBox(bNum, gBox(bNum).BoxObj)
                
            End If
            
        End If
        
    End With
    
End Sub

' This is a low level routine that is typically not used by the end user.
'
' Clear all DispBoxes and then redraw them
Public Sub Redraw_AllBoxes()

   Dim bNum As Integer
    
    For bNum = 0 To gNumBoxes - 1
    
        ' redraw this box
        Call Redraw_OneBox(bNum, gBox(bNum).BoxObj)
   
   Next bNum
   
End Sub


' This is a low level routine that is typically not used by the end user.
'
' Clear 1 DispBox and then Redraw it
Public Sub Redraw_OneBox(ByVal bNum As Integer, ByRef PlotObj As Variant)

    Call Box_Clear_OneBox(PlotObj)              ' clear entire DisplayBox
    
    Call Grid_Draw_OneBox(bNum, PlotObj)        ' draw the grid (we draw stuff over it)
            
                                                ' draw lower hScale label
    Call hScale_DrawLowerLabel_OneBox(bNum, gBox(bNum), PlotObj)
                                            
    Call vScale_DrawLabel_OneBox(bNum, PlotObj) ' draw rightside vScale label
            
                                                ' draw vertical marker line
    Call Marker_Draw_Line_OneBox(bNum, True, gBox(bNum), PlotObj)
     
                                                ' draw marker text at top of boxes
    Call Marker_Draw_Text_OneBox(bNum, gBox(bNum), PlotObj)
                                                
    Call Plot_FullScreen_OneBox(bNum, PlotObj)  ' plot the data
    
    Call Borders_Draw_OneBox(bNum, PlotObj)     ' draw the borders around the display
   
End Sub


' This is a low level routine that is typically not used by the end user.
'
' Given a new dispLeft_xEU/dispRight_xEU, we update the horiz scale params.
'
'   INPUT    dispLeft_xEU/dispRight_xEU, disp(cNum).dataArrayY.ptsPerXunit
'
'   UPDATES  .xCoord_Per_xEU, xCoord_Per_ptNum
'            .dispLeft_ptNum, .dispRight_ptNum, .dispWidth_numPts
'
' This also updates the lower hScale label text if this is the TOP-MOST display.
'
Sub hScale_SetWithNew_xEU_values_OneDisp( _
    ByVal bNum As Integer, _
    ByVal dNum As Integer, _
    ByVal dispLeft_xEU As Double, _
    ByVal dispRight_xEU As Double, _
    ByRef PlotObj As Variant)
        
    ' don't do anything in here if there are no displays in this DisplayBox.
    If (gBox(bNum).numDisp <= dNum) Then Exit Sub
        
    ' get the # of data pts and the data's sample rate
    Dim ptsPerXunit As Double
    Dim firstPt_Xunits As Double
    Dim numValidPts As Long
    Call GetDataParams(bNum, dNum, ptsPerXunit, numValidPts, firstPt_Xunits)

    With gBox(bNum).disp(dNum)
                                        ' bail out if these values are not new
        If .dispLeft_xEU = dispLeft_xEU And .dispRight_xEU = dispRight_xEU Then Exit Sub
        
        .dispLeft_xEU = dispLeft_xEU    ' horiz Left/Right values in xEU coords
        .dispRight_xEU = dispRight_xEU
        .dispWidth_xEU = dispRight_xEU - dispLeft_xEU
        
                                        ' xCoords per xEU (horiz plot scale)
        .xCoord_Per_xEU = (.dispRight_xCoord - .dispLeft_xCoord) / (.dispRight_xEU - .dispLeft_xEU)
        
                                        ' help code recover from no points scenario
        If (.xCoord_Per_xEU = 0) Then .xCoord_Per_xEU = 1
                                        
                                        ' make sure sample rate is not 0, to avoid errors
        If (ptsPerXunit = 0) Then EndProgram ("why sRate 0?")
                                
        ' set the plotted pts-per-xUnit (e.g. pts-per-sec)
        Call hScale_SetPtsPerHscale_OneDisp(bNum, dNum, ptsPerXunit)
        
        ' Debugging
        ' temp = .dispWidth_numPts: tempB = .dispLeft_ptNum: tempC = .dispRight_ptNum
        ' tempD = .xCoord_Per_ptNum: tempE = .dispRight_xCoord: tempF = .dispLeft_xCoord
        
        ' If this is the top most display, then update the Lower hScale label
        If (dNum = TOP_DISPLAY_NUM) Then
        
            ' Draw Lower hScale label at base of DisplayBox.
            Call hScale_DrawLowerLabel_OneBox(bNum, gBox(bNum), PlotObj)
            
        End If
    
    End With

End Sub

' Set the pts-per-hScale (based on the sample rate of the data).
Sub hScale_SetPtsPerHscale_OneDisp( _
    ByVal bNum As Integer, _
    ByVal dNum As Integer, _
    ByVal ptsPerXunit_thisDisp As Double)
    
    If ptsPerXunit_thisDisp <= 1E-30 Then EndProgram ("bad srate")
    
    With gBox(bNum).disp(dNum)
    
        ' waveform ptNum corresponding to left/right/width
        ' (assumes waveform data exists, which it might not)
        .dispLeft_ptNum = .dispLeft_xEU * ptsPerXunit_thisDisp
        .dispRight_ptNum = .dispRight_xEU * ptsPerXunit_thisDisp
        .dispWidth_numPts = 1 + .dispRight_ptNum - .dispLeft_ptNum
        
        ' help code recover from no points scenario
        If (.dispWidth_numPts <= 0) Then .dispWidth_numPts = 1
                                                
        ' xCoords per pointNum (horiz plot scale)
        .xCoord_Per_ptNum = (.dispRight_xCoord - .dispLeft_xCoord) / .dispWidth_numPts
        
        ' help code recover from no points scenario
        If (.xCoord_Per_ptNum = 0) Then .xCoord_Per_ptNum = 1

    End With
    
End Sub
' This is a low level routine that is typically not used by the end user.
'
' Updates marker information that is shown at the top of the display
Public Sub Marker_Draw_Text_OneBox( _
    ByVal bNum As Integer, _
    ByRef thisBox As DispBoxParm, _
    ByRef PlotObj As Variant)
    
    Dim dNum As Integer
    Dim hUpperLabel_text As String
    Dim lastValid_ptNum As Long
    
    ' don't do anything in here if there are no displays in this DisplayBox.
    If (thisBox.numDisp <= 0) Then Exit Sub
        
    ' get the display Title text
    hUpperLabel_text = thisBox.hDispBoxTitle_text

    ' scan thru the markers in the display
    For mkrNum = 0 To (thisBox.numMkrs - 1)
    
        ' get Display that contains the marker
        dNum = thisBox.markerPos_dNum(mkrNum)
        
        With thisBox.disp(dNum)
        
            ' get ptNum at marker position
            Dim point_ptNum As Long
            point_ptNum = Translate_xEU_to_ptNum(bNum, dNum, _
                thisBox.markerPos_xEU(mkrNum))
            
            ' get the # of data pts and the data's sample rate
            Dim ptsPerXunit As Double
            Dim firstPt_Xunits As Double
            Dim numValidPts As Long
            Call GetDataParams(bNum, dNum, ptsPerXunit, numValidPts, firstPt_Xunits)
            
            ' in data buffer, last valid ptNum
            lastValid_ptNum = numValidPts - 1
            
            ' make sure ptNum does not overflow the wave
            If (point_ptNum < 0) Then point_ptNum = 0
            If (point_ptNum > lastValid_ptNum) Then point_ptNum = lastValid_ptNum
            
            ' get local_data(), or gDigiAddr()
            If point_ptNum <= lastValid_ptNum And point_ptNum >= 0 Then
                If (.cNum = NO_DIGI_CHAN) Then
                    data! = thisBox.disp(dNum).dataArrayY.data(point_ptNum)
                Else
                    data! = gDigiChan(.cNum).dataArrayY.data(point_ptNum)
                End If
            Else
                data! = 0
            End If
                
            ' get yEU value and round off till the last 6 digits
            .valueAtMarker_yEU(mkrNum) = data!
            
            ' if this box supports markers...
            If (thisBox.supportMarkers = True) Then
            
                ' add marker name
                hUpperLabel_text = hUpperLabel_text + "    Marker: " + .name + " = {" + _
                    NumToStr(thisBox.markerPos_xEU(mkrNum)) + ", " + _
                    NumToStr(.valueAtMarker_yEU(mkrNum)) + "}"
                    
            End If
        
        End With
        
    Next mkrNum
    
    ' ------------------------------------------------
    ' Draw upper Label text.
    
    With PlotObj    ' Printer or Screen-PictureBox-BoxObj
             
        Call Set_AutoRedraw_Property(True, PlotObj)
        
        ' fill the entire UpperLabel region w/ white
        PlotObj.Line _
            ((thisBox.dispBoxLeft_xCoord + 1), (thisBox.dispBoxTop_yCoord + 1)) _
           -((thisBox.dispBoxRight_xCoord - 1), (thisBox.hUpperLabel_Bot_yCoord - 1)) _
            , &HFFFFFF, BF
                    
        ' Sets the drawing style to a black
        Call Set_DrawStyle_OneBox(bNum, kBlack, PlotObj)
        
        ' move pen to upper label region
        .CurrentX = thisBox.dispBoxLeft_xCoord + 5
        .CurrentY = thisBox.dispBoxTop_yCoord + vSPACE_yPix
        
        ' print the marker text
        PlotObj.Print hUpperLabel_text
        
        Call Set_AutoRedraw_Property(False, PlotObj)
    
    End With

End Sub

' Returns inside 'sRate' and 'numValidPts' the sample rate and the
' the # of pts for the data tied to the {bNum, dNum display}.
' This data is either stored linked to the Digitize Data, or the
' Display Data.
Sub GetDataParams( _
    ByVal bNum As Integer, _
    ByVal dNum As Integer, _
    ByRef ptsPerXunit As Double, _
    ByRef numValidPts As Long, _
    ByRef firstPt_Xunits As Double)
    
    With gBox(bNum).disp(dNum)
    
        ' debugging
        ' a = gBox(bNum).disp(dNum).cNum
        ' b = gBox(bNum).disp(dNum).dataArrayY.numValidPts
        ' c = gBox(bNum).disp(dNum).dataArrayY.ptsPerXunit
            
        ' if the Display uses local data...
        If (.cNum = NO_DIGI_CHAN) Then
            
            ' then get the data from the display
            numValidPts = .dataArrayY.numValidPts
            ptsPerXunit = .dataArrayY.ptsPerXunit
            firstPt_Xunits = .dataArrayY.firstPt_Xunits
            
        ' else, get params associated w/ digitize data
        Else
        
            numValidPts = gDigiChan(.cNum).dataArrayY.numValidPts
            ptsPerXunit = gDigiChan(.cNum).dataArrayY.ptsPerXunit
            firstPt_Xunits = gDigiChan(.cNum).dataArrayY.firstPt_Xunits
            
        End If
    
    End With
    
End Sub
' This is a low level routine that is typically not used by the end user.
'
' Initialize horiz/vert scroll/scale to default values.
Sub Reset_Controls_AllBoxes(thingsToReset As Integer)

    Dim bNum As Integer
    
    For bNum = 0 To gNumBoxes - 1
    
        Call Reset_Controls_OneBox(bNum, gBox(bNum), thingsToReset)
        
    Next bNum

End Sub

' This is a low level routine that is typically not used by the end user.
'
' Plot data in all DispBoxes
Public Sub Plot_FullScreen_AllBoxes()

    Dim bNum As Integer
    
    For bNum = 0 To gNumBoxes - 1
    
        Call Plot_FullScreen_OneBox(bNum, gBox(dispBox).BoxObj)
        
    Next bNum

End Sub

' This is a low level routine that is typically not used by the end user.
'
' Draw the grid in all DispBoxes
Public Sub Grid_Draw_AllBoxes()

    Dim bNum As Integer
    
    For bNum = 0 To gNumBoxes - 1   ' scan through all DisplayBoxes
    
        Call Grid_Draw_OneBox(bNum, gBox(bNum).BoxObj)
        
    Next bNum

End Sub

' This is a low level routine that is typically not used by the end user.
'
' Draw the right-side vScale labels in all DispBoxes
Public Sub vScale_DrawLabel_OneBox(ByVal bNum As Integer, ByRef PlotObj As Variant)

    Dim dNum As Integer
    
    For dNum = 0 To (gBox(bNum).numDisp - 1)
        
        Call vScale_DrawLabel_OneDisp(bNum, dNum, gBox(bNum).disp(dNum), PlotObj)
                
    Next dNum

End Sub

' This is a low level routine that is typically not used by the end user.
'
' Draw the borders within in all DispBoxes
Public Sub Borders_Draw_AllBoxes()

    Dim bNum As Integer
    
    For bNum = 0 To gNumBoxes - 1   ' scan through all DisplayBoxes
    
        Call Borders_Draw_OneBox(bNum, gBox(bNum).BoxObj)
        
    Next bNum

End Sub

' This is a low level routine that is typically not used by the end user.
'
' Initialize instruNet display code.
Public Sub Initialize_instruNet_Displays(ByRef endUserWindow As Variant)

    ' main window created by end user
    Set gEndUserWindow = endUserWindow
    
    ' Init scroll bar methods.
    gMethod_vPos_scrollObj_Enabled = True
    gMethod_hPos_scrollObj_Enabled = True
    
    ' return string
    gReturnStr$ = Chr$(13) + Chr$(10)
     
End Sub

 


' This is a low level routine that is typically not used by the end user.
'
' Draw the marker in all DispBoxes
Public Sub Marker_Draw_Line_AllBoxes()

    Dim bNum As Integer
    
    For bNum = 0 To gNumBoxes - 1
    
        ' draw new marker
        Call Marker_Draw_Line_OneBox(bNum, True, gBox(bNum), gBox(bNum).BoxObj)
        
    Next bNum

End Sub


' This is a low level routine that is typically not used by the end user.
'
' Updates marker information that is shown on the front panel as text, all DispBoxes
Public Sub Marker_Draw_Text_AllBoxes()

    Dim bNum As Integer
    
    For bNum = 0 To gNumBoxes - 1
    
        Call Marker_Draw_Text_OneBox(bNum, gBox(bNum), gBox(bNum).BoxObj)
        
    Next bNum

End Sub

' This is a low level routine that is typically not used by the end user.
'
' Plot wave data on printer for all DispBoxes
Public Sub Printer_Plot_FullScreen_AllBoxes()

    ' set medium resolution print quality
    Printer.PrintQuality = vbPRPQMedium
    
    ' Go thru all Boxes in window (each Box contains a set of Displays) and
    ' print each Box one 1 page of paper.
    For bNum = 0 To gNumBoxes - 1
        
        With gBox(bNum).BoxObj
        
            ' Pixel coordinate system
            Printer.ScaleMode = .ScaleMode
            
            ' border percentage (The paper edge is 5% larger than
            ' the screen BoxObj).
            border! = 0.05
            
            ' x/y plotting coordinate system for this DisplayBox.
            Printer.ScaleLeft = .ScaleLeft - (1 * border! * .ScaleWidth)
            Printer.ScaleWidth = .ScaleWidth + (2 * border! * .ScaleWidth)
            Printer.ScaleTop = .ScaleTop - (1 * border! * .ScaleHeight)
            Printer.ScaleHeight = .ScaleHeight + (2 * border! * .ScaleHeight)
                                
            ' plot this display on 1 page to the printer
            Call Redraw_OneBox(bNum, Printer)
                        
             ' Print the footer
            vSPACEp = 2
            footer$ = Format(Date, "Long Date") + ", " + Format(Time, "hh:mm:ss AMPM") + ", Page #" + Str(bNum + 1)
            Printer.CurrentX = .ScaleLeft + 1
            Printer.CurrentY = .ScaleTop + .ScaleHeight + vSPACEp
            Call Set_DrawStyle_OneBox(bNum, kBlack, Printer)    ' go back to drawing black
            Printer.Print footer$
          
            ' print this page (1 page per Box)
            Printer.EndDoc

        End With
        
    Next bNum

End Sub



' Print to hard-copy printer all DisplayBoxes, one page per box.
Public Sub METHOD_Print_button_Click()

    ' print all DisplayBoxes, one page per box.
    Printer_Plot_FullScreen_AllBoxes

End Sub


' This is a low level routine that is typically not used by the end user.
'
' Plot the data in the display, from first_ptNum to last_ptNum.
Public Sub Plot_Segment_OneDisp( _
    ByVal bNum As Integer, _
    ByVal dNum As Integer, _
    ByRef thisBox As DispBoxParm, _
    ByRef thisDisp As DispParm, _
    ByVal first_ptNum As Long, _
    ByVal last_ptNum As Long, _
    ByRef PlotObj As Variant)
    
    ' -------------------------------------------------------------------
    ' Initialize.
    
    Dim plotTheNextPoint As Integer
    Dim lastValid_ptNum As Long
    Dim xy As Integer
            
    ' true if doing XY display; false if doing Y vs index.
    xy = (thisDisp.displayType = kXYDisplay)
    
    ' Don't do anything in here if there are not displays in this DisplayBox.
    If (thisBox.numDisp <= 0) Then Exit Sub
    
    ' get the # of data pts and the data's sample rate (for the Y data)
    Dim ptsPerXunit As Double
    Dim firstPt_Xunits As Double
    Dim numValidPts As Long
    Call GetDataParams(bNum, dNum, ptsPerXunit, numValidPts, firstPt_Xunits)

    ' in data buffer, last valid ptNum
    lastValid_ptNum = numValidPts - 1
    
    ' If there are no points to plot, then bail out
    If (first_ptNum > lastValid_ptNum) Then Exit Sub
    
    ' If there is no data to plot, then bail out.
    If (first_ptNum > last_ptNum) Then Exit Sub
        
    ' -------------------------------------------------------------------
    ' Do the plot.

    With PlotObj    ' Printer or Screen-PictureBox-BoxObj

        Call Set_AutoRedraw_Property(True, PlotObj)
        
        ' set pen color
        .DrawMode = 13          ' copy pen
        .DrawStyle = 0          ' solid line
        .ForeColor = &HFF0000   ' blue
        
    End With
    
    ' do the following w/ this Display
    With thisDisp
        
        ' if working with an Y vs index display...
        If (.displayType = kYindexDisplay) Then
        
            ' If the 1st point is to the left of the left edge, set it to the left edge...
            If (first_ptNum < .dispLeft_ptNum) Then first_ptNum = .dispLeft_ptNum
               
            ' If the last point is to the right of the right edge, set it to the right edge...
            If (last_ptNum > .dispRight_ptNum) Then last_ptNum = .dispRight_ptNum
            
        End If
        
        ' If the last plot point is past last valid data pt, then
        ' set it to the last valid data pt.
        If (last_ptNum > lastValid_ptNum) Then last_ptNum = lastValid_ptNum
        
        ' If we don't have any data to plot, then bail out...
        If (first_ptNum > last_ptNum) Then GoTo EndOfPlotting
        
        ' back up one, so that we can draw a line segment starting
        ' at the point before the plotted segment
        If (first_ptNum > 0) Then first_ptNum = first_ptNum - 1
        
        ' We don't plot the 1st point, since we need to prime the pump
        ' and locate the point before the 1st point, first.
        plotTheNextPoint = False
        
        ' Y vs Index: If dispWidth_numPts(1) < 2048 Then plotStep_ptNum = 1
        '   Else plotStep_ptNum is such that we plot between 2K and 4K points.
        ' X vx Y: plotStep is 1, and we plot every point
        plotStep_ptNum = IIf(xy, 1, (Int(.dispWidth_numPts / 2048) + 1))
            
        ' ------------------------------------------------------
        ' Scan through points to plot.
        
        Dim ptNum As Long
        For ptNum = first_ptNum To last_ptNum Step plotStep_ptNum
                
            ' ---------------------------
            ' CALCULATE Y DATA
            
            ' Get Y data (attached to Display or Digitize Channel)
            If .cNum = NO_DIGI_CHAN Then
                dataY! = .dataArrayY.data(ptNum)
            Else
                dataY! = gDigiChan(.cNum).dataArrayY.data(ptNum)
            End If
                    
            ' yCoord = (yEU + .plotOffset_yEU ) * .yCoord_Per_yEU
            nextPoint_yCoord! = (dataY! + .plotOffset_yEU) * .yCoord_Per_yEU
             
            ' make sure it does not overflow (else, we plot off the graph)
            If nextPoint_yCoord! < .dispTop_yCoord Then nextPoint_yCoord! = .dispTop_yCoord + 1
            If nextPoint_yCoord! > .dispBot_yCoord Then nextPoint_yCoord! = .dispBot_yCoord - 1
                            
            ' ---------------------------
            ' CALCULATE X DATA
            
            ' if doing a XY display...
            If (xy) Then
            
                ' Get X data if doing xy (attached to Display or Digitize Channel)
                If .cNum = NO_DIGI_CHAN Then
                    dataX! = .dataArrayX.data(ptNum)
                Else
                    dataX! = gDigiChan(.cNum + 1).dataArrayY.data(ptNum)
                End If
                
                ' debug
                ' a = .xCoord_Per_xEU: b = .dispLeft_xEU: c = .dispLeft_xCoord
                
                ' calc xCoord of point
                nextPoint_xCoord! = ((dataX! - .dispLeft_xEU) * .xCoord_Per_xEU) + .dispLeft_xCoord
                 
                ' make sure it does not overflow (else, we plot off the graph)
                If nextPoint_xCoord! < .dispLeft_xCoord Then nextPoint_xCoord! = .dispLeft_xCoord + 1
                If nextPoint_xCoord! > .dispRight_xCoord Then nextPoint_xCoord! = .dispRight_xCoord - 1
            
            ' else doing Y vs index
            Else
                nextPoint_xCoord! = (ptNum - .dispLeft_ptNum) * .xCoord_Per_ptNum
            End If
                                    
            ' if we are to plot this point...
            If plotTheNextPoint Then
            
                ' draw a little line segment between two plotted points
                PlotObj.Line (point_xCoord!, point_yCoord!)-(nextPoint_xCoord!, nextPoint_yCoord!)
                
                ' plot little circles if there are less than 64 plotted points
                If .dispWidth_numPts <= 64 And Not (xy) Then
                    PlotObj.Circle (nextPoint_xCoord!, nextPoint_yCoord!), 3
                End If
                
            ' otherwise, we are priming the pump, and will plot the next point
            Else
                plotTheNextPoint = True
            End If
            
            'update the next point begin coordinate
            point_xCoord! = nextPoint_xCoord!
            point_yCoord! = nextPoint_yCoord!
    
        Next ptNum
            
    End With
    
EndOfPlotting:

    With PlotObj    ' Printer or Screen-PictureBox-BoxObj
        .ForeColor = &H0                  ' black
        Call Set_AutoRedraw_Property(False, PlotObj)
    End With

End Sub

' plots where the marker was
Public Sub Plot_LastMarkerPos_OneDisp( _
    ByVal bNum As Integer, _
    ByVal dNum As Integer, _
    ByRef thisBox As DispBoxParm, _
    ByRef thisDisp As DispParm)

    ' -------------------------------------------------------------------
    ' Init.

    Dim prevMarkerPos_ptNum As Long
    Dim mkrNum As Integer
       
    ' don't do anything in here if there are not displays in this DisplayBox.
    If (thisBox.numDisp <= 0) Then Exit Sub
            
    ' -------------------------------------------------------------------
    ' If we are only replotting where marker was, make sure this display
    ' contained a previous marker.
    
    For mkrNum = 0 To thisBox.numMkrs      ' scan thru markers
    
        If (thisBox.prevMarkerPos_dNum(mkrNum) = dNum) Then
            
            ' calc point # of marker positions
            prevMarkerPos_ptNum = Translate_xEU_to_ptNum(bNum, dNum, _
                thisBox.prevMarkerPos_xEU(mkrNum))
            
            ' plot where the marker was
            Call Plot_Segment_OneDisp(bNum, dNum, thisBox, thisDisp, _
                (prevMarkerPos_ptNum - 3), (prevMarkerPos_ptNum + 3), _
                gBox(bNum).BoxObj)
                
        End If
        
    Next mkrNum

End Sub

' plot entire waveform within 1 Display.
Public Sub Plot_FullScreen_OneDisp( _
    ByVal bNum As Integer, _
    ByVal dNum As Integer, _
    ByRef thisBox As DispBoxParm, _
    ByRef thisDisp As DispParm, _
    ByRef PlotObj As Variant)

    Dim first_ptNum, last_ptNum As Long
    
    ' if XY Display..
    If (thisDisp.displayType = kXYDisplay) Then
    
        ' get the # of data pts and the data's sample rate (for the Y data)
        Dim ptsPerXunit As Double
        Dim firstPt_Xunits As Double
        Dim numValidPts As Long
        Call GetDataParams(bNum, dNum, ptsPerXunit, numValidPts, firstPt_Xunits)
        
        ' plot all points inside the data buffer
        first_ptNum = 0
        last_ptNum = numValidPts - 1
        
    ' else Y vs INDEX Display...
    Else
    
        ' plot from left to right edge of display
        first_ptNum = thisDisp.dispLeft_ptNum
        last_ptNum = thisDisp.dispRight_ptNum
    
    End If
    
    ' Plot entire display, from left to right edges
    Call Plot_Segment_OneDisp(bNum, dNum, thisBox, thisDisp, _
        first_ptNum, last_ptNum, PlotObj)
    
End Sub

' This is a low level routine that is typically not used by the end user.
'
' Draw the grid in all displays of one display box
Public Sub Grid_Draw_OneBox(ByVal bNum As Integer, ByRef PlotObj As Variant)
    
    Dim dNum As Integer
    
    For dNum = 0 To (gBox(bNum).numDisp - 1)
    
        ' draw the grid for 1 display
        Call Grid_Draw_OneDisp(bNum, dNum, gBox(bNum), _
            gBox(bNum).disp(dNum), PlotObj)
        
    Next dNum

End Sub



' This is a low level routine that is typically not used by the end user.
'
' Plot data in all displays of one display box
Public Sub Plot_FullScreen_OneBox(ByVal bNum As Integer, ByRef PlotObj As Variant)
    
    Dim dNum As Integer
    
    For dNum = 0 To (gBox(bNum).numDisp - 1)
    
        Call Plot_FullScreen_OneDisp(bNum, dNum, gBox(bNum), _
            gBox(bNum).disp(dNum), PlotObj)
        
    Next dNum

End Sub


' This is a low level routine that is typically not used by the end user.
'
' Draw grid in one display.
Public Sub Grid_Draw_OneDisp( _
    ByVal bNum As Integer, _
    ByVal dNum As Integer, _
    ByRef thisBox As DispBoxParm, _
    ByRef thisDisp As DispParm, _
    ByRef PlotObj As Variant)
                      
    ' don't do anything in here if there are not displays in this DisplayBox.
    If (thisBox.numDisp <= 0) Then Exit Sub
      
    ' if we don't show a grid in this display, bail out
    If (thisDisp.showGrid = False) Then Exit Sub
    
    ' turn on AutoRedraw
    Call Set_AutoRedraw_Property(True, PlotObj)
    
    ' select grid line style
    Call Set_DrawStyle_OneBox(bNum, kGray, PlotObj)
    
    Dim ticksPerGrid_vert As Integer    ' # of tics per vert grid, e.g. 3 (-1=autoselect)
    Dim ticksPerGrid_horiz As Integer   ' # of tics per horiz grid, e.g. 3 (-1=autoselect)
    
    With thisDisp
                
        ' -----------------------------------------------------
        ' Draw horiz grid lines.
        
        For i = 1 To .numGridLines_horiz_calc
        
            ' calc yCoord of horiz line
            yCoord = .gridHeight_yCoord * i + .dispTop_yCoord
            Dim tickSpacing_xCoord As Double
            
            ' if this is the center line, then draw tick marks
            If (i = (Int(.numGridLines_horiz_calc / 2) + 1)) Then
                                        
                ' now add little tick marks along this horiz line
                numTickSpaces_horiz = ((.numGridLines_vert_calc + 1) * (.ticksPerGrid_horiz_calc + 1))
                halfTickWidth_yCoord = 2    ' 2 pixel tick marks
                tick_top_yCoord = yCoord - halfTickWidth_yCoord
                tick_bottom_yCoord = yCoord + halfTickWidth_yCoord + 2
                tickSpacing_xCoord = (.dispRight_xCoord - .dispLeft_xCoord) / numTickSpaces_horiz
                For Ticks = 1 To (numTickSpaces_horiz - 1)
                    tick_xCoord = tickSpacing_xCoord * Ticks + .dispLeft_xCoord
                    PlotObj.Line (tick_xCoord, tick_top_yCoord)-(tick_xCoord, tick_bottom_yCoord)
                Next Ticks
                    
            End If
            
            ' draw grid line
            PlotObj.Line (.dispLeft_xCoord, yCoord)-(.dispRight_xCoord, yCoord)
            
        Next i
            
        ' -----------------------------------------------------
        ' Draw vertical grid lines
        
        For i = 1 To .numGridLines_vert_calc
        
            ' calc xCoord of vertical line
            xCoord = .gridWidth_xCoord * i
            Dim tickSpacing_yCoord As Double
            
            ' if this is the center line, then add tick marks
            If (i = (Int(.numGridLines_vert_calc / 2) + 1)) Then
                                         
                ' now add little tick marks along this vertical line
                numTickSpaces_vert = ((.numGridLines_horiz_calc + 1) * (.ticksPerGrid_vert_calc + 1))
                halfTickWidth_xCoord = 2    ' 2 pixel tick mark
                tick_left_xCoord = xCoord - halfTickWidth_xCoord
                tick_right_xCoord = xCoord + halfTickWidth_xCoord + 2
                tickSpacing_yCoord = (.dispBot_yCoord - .dispTop_yCoord) / numTickSpaces_vert
                For Ticks = 1 To (numTickSpaces_vert - 1)
                    tick_yCoord = tickSpacing_yCoord * Ticks + .dispTop_yCoord
                    PlotObj.Line (tick_left_xCoord, tick_yCoord)-(tick_right_xCoord, tick_yCoord)
                Next Ticks
                              
            End If
            
            ' draw grid line
            PlotObj.Line (xCoord, .dispTop_yCoord)-(xCoord, .dispBot_yCoord)
                        
        Next i
    
    End With
    
    Call Set_DrawStyle_OneBox(bNum, kBlack, PlotObj)    ' go back to drawing black
    Call Set_AutoRedraw_Property(False, PlotObj)        ' turn AutoRedraw off
    
End Sub
' This is a low level routine that is typically not used by the end user.
'
' Draw Lower hScale label at base of DisplayBox.
Public Sub hScale_DrawLowerLabel_OneBox( _
    ByVal bNum As Integer, _
    ByRef thisBox As DispBoxParm, _
    ByRef PlotObj As Variant)
                   
    Dim xEU As Double
    Dim xCoord_double As Double
    Dim xEU_str As String
    Dim textWidth_xCoord, prevLabelRightEdge_xCoord, xCoord_int As Integer
   
    With thisBox
    
        ' don't do anything in here if there are not displays in this DisplayBox.
        If (.numDisp <= 0) Then Exit Sub
              
        Call Set_AutoRedraw_Property(True, PlotObj)
        
        ' -----------------------------------------------------
        ' Fill the entire LowerLabel region w/ white.
        
        PlotObj.Line _
            ((.dispBoxLeft_xCoord + 1), (.dispBoxBot_yCoord - 1)) _
           -((.dispBoxRight_xCoord - 1), (.hLowerLabel_Top_yCoord + 1)) _
            , &HFFFFFF, BF
        
        ' -----------------------------------------------------
        ' Draw hScale (e.g. "0   .1   .2   .3   .4 secs"
        
        Const hLABEL_SPACE = 4
            
        prevLabelRightEdge_xCoord = -(hLABEL_SPACE * 2)
       
        ' select black line style
        Call Set_DrawStyle_OneBox(bNum, kBlack, PlotObj)
        
        For i = 0 To .disp(TOP_DISPLAY_NUM).numGridLines_vert_calc + 1
            
            ' calc xCoord of vertical line
            xCoord_double = .disp(TOP_DISPLAY_NUM).gridWidth_xCoord * i
            xCoord_int = xCoord_double
            
            ' get xEU value for this grid line
            With thisBox.disp(TOP_DISPLAY_NUM)
                
                ' calc xEU value
                xEU = ((xCoord_double - .dispLeft_xCoord) / .xCoord_Per_xEU) + .dispLeft_xEU
                
                '  if the display width is > .1, then round off to nearest .001
                If .dispWidth_xEU > 0.1 Then
                   xEU = Int((xEU * 1000#) + 0.5) / 1000#
                End If
            
            End With
            
            xEU_str = NumToStr(xEU)
            
            textWidth_xCoord = PlotObj.TextWidth(xEU_str)
            
            ' move pen
            If i = 0 Then           ' if we are at the left edge, then push right
                PlotObj.CurrentX = xCoord_int + hLABEL_SPACE
            Else
                PlotObj.CurrentX = xCoord_int - (textWidth_xCoord / 2)
            End If
            
            ' if we are hitting the previous label, then move right some
            If (PlotObj.CurrentX < (prevLabelRightEdge_xCoord + (hLABEL_SPACE * 3))) Then
                PlotObj.CurrentX = (prevLabelRightEdge_xCoord + (hLABEL_SPACE * 3))
            End If
            
            ' if we over shot the grid line, then skip this label
            If (PlotObj.CurrentX > xCoord_int + hLABEL_SPACE + 1) Then GoTo NextGrid
            
            ' set the upper left coord of the printed label
            PlotObj.CurrentY = .hLowerLabel_Top_yCoord + vSPACE_yPix
            
            ' remember the right edge of the recently drawn label
            prevLabelRightEdge_xCoord = PlotObj.CurrentX + textWidth_xCoord
            
            ' print the xEU scale value
            PlotObj.Print xEU_str
            
NextGrid:

        Next i
    
        Call Set_AutoRedraw_Property(False, PlotObj)
    
    End With
    
End Sub
' This is a low level routine that is typically not used by the end user.
'
' Draw borders in one DisplayBox (i.e. enclosing rect, lines between Displays,
' hUpperLabelBox border, vLabelBox border, hLowerLabelBox border).
Public Sub Borders_Draw_OneBox(ByVal bNum As Integer, ByRef PlotObj As Variant)
                
    Dim dNum As Integer
    
    Call Set_DrawStyle_OneBox(bNum, kBlack, PlotObj)   ' draw black
        
    With gBox(bNum)
    
        Call Set_AutoRedraw_Property(True, PlotObj)
       
        ' draw line below the Upper Horizontal Label region
        PlotObj.Line (.dispBoxLeft_xCoord, .hUpperLabel_Bot_yCoord) _
                    -(.dispBoxRight_xCoord, .hUpperLabel_Bot_yCoord)
        
        ' draw line above the Lower Horizontal Label region
        PlotObj.Line (.dispBoxLeft_xCoord, .hLowerLabel_Top_yCoord) _
                    -(.dispBoxRight_xCoord, .hLowerLabel_Top_yCoord)
        
        ' draw line to left of the Vertical Label region
        PlotObj.Line (.vLabelBoxLeft_xCoord, .hUpperLabel_Bot_yCoord) _
                    -(.vLabelBoxLeft_xCoord, .hLowerLabel_Top_yCoord)
        
        ' draw line under each Display plot region
        For dNum = 0 To (.numDisp - 2)
                  
            PlotObj.DrawWidth = 2           ' double thick line
            
            PlotObj.Line (.disp(dNum).dispLeft_xCoord, .disp(dNum).dispBot_yCoord) _
                        -(.dispBoxRight_xCoord, .disp(dNum).dispBot_yCoord)
                       
            PlotObj.DrawWidth = 1           ' thin line
        
        Next dNum
        
        ' if writing to the screen instead of the Printer...
        If TypeName(PlotObj) <> TypeName(Printer) Then
             PlotObj.DrawWidth = 1          ' thin line
        Else
            PlotObj.DrawWidth = 2           ' double thick line
        End If
        
        ' draw a box around the set of plot regions (i.e. the "DisplayBox")
        PlotObj.Line (.dispBoxLeft_xCoord, .dispBoxTop_yCoord) _
                    -(.dispBoxRight_xCoord, .dispBoxBot_yCoord), , B
        
        PlotObj.DrawWidth = 1               ' thin line
        
        Call Set_AutoRedraw_Property(False, PlotObj)
    
    End With
          
End Sub


