Attribute VB_Name = "Macro_Utilities"
' INET_Macros.Bas           Companion: (none)
'
' ----------------------- D O C U M E N T A T I O N ----------------------
'
' This iNet Interface file contains macros used by the End User
' to control the iNet driver.
'
' ------------------------  H I S T O R Y --------------------------------
' HISTORY  Copyright c 1994-96 GWI.  All rights reserved.
' 4/16/94  created - gsw
' 2/8/96   converted to Visual Basic - sjm
' 8/17/98  updated - gsw
                        

' Returns TRUE (1) if the controller is digitizing; FALSE (0) otherwise.

Function CONTROLLER_IS_DIGITIZING(ByVal netNum As Byte)

    Dim digitizeOnOff

    digitizeOnOff = iNet_int32(netNum, deviceNum_CONTROLLER, _
        moduleNum_1stModule, controller_ChanNum_C12_Digitizer, _
        sgt_Timing, fldNum_Timing_digitizeOnOff, intention_getValue, 0&)
  
    If digitizeOnOff = fldNum_Timing_digitizeOnOff Then
        CONTROLLER_IS_DIGITIZING = iNetTrue
    Else
        CONTROLLER_IS_DIGITIZING = iNetFalse
    End If

End Function

' Disables channel digitizing for all channels in this network.  This will make
' sure that the "Display" field in the DISPLAY SettingGroups is OFF, and that
' the "Digitize" field is OFF in the FILE, USERBUFFER, and DRIVERBUFFER
' SettingGroups. gsw 3/16/96

Function DISABLE_ALL_CHANNEL_DIGITIZING(ByVal netNum As Byte)

    DISABLE_ALL_CHANNEL_DIGITIZING = _
        iNet_int32(netNum, 1, 1, 1, 1, 1, intention_DisableAllChannelDigitizing, 0)

End Function

' Returns TRUE (1) if the channel digitization is turned off;
' FALSE (0) otherwise.

Function DISABLE_CHANNEL_DIGITIZATION(ByVal netNum As Byte, _
    ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte)
    
    Dim err As Integer
    err = iNet_int32(netNum, deviceNum, moduleNum, chanNum, sgt_Display, _
        fldNum_Display_dispOnOff, intention_setValue, ion_gOnOffPopup_Off)
        
    DISABLE_CHANNEL_DIGITIZATION = err

End Function

' Dispose of all iNet network state.  This will cause the the next
' call to the iNet driver to implement the POWER ON boot process.

Function DISPOSE_OF_ALL_NETWORK_STATE()

    DISPOSE_OF_ALL_NETWORK_STATE = iNet_int32(0, 0, 1, 1, 1, 1, _
        intention_disposeOfNetworkState, 0&)

End Function


' Tells Driver to NOT show an Error message dialog box whenever an error occurs.
' Returns TRUE (1) if Driver WAS (before this call) set up to show an alert
' upon error; FALSE (0) otherwise
' Returns TRUE (1) if Driver is set up to show an alert upon error;
' FALSE (0) otherwise

Function DONT_SHOW_MESSAGE_DIALOG_UPON_ERROR()

    DONT_SHOW_MESSAGE_DIALOG_UPON_ERROR = iNet_int32(0, 0, 0, 0, 0, 0, _
        intention_NoShowAlertOnError, 0)

End Function

' Sets up digitization, establishes sample rate, sets up BUFFERS, yet does not
' actually do the digitization. gsw 8/16/98
Sub SET_UP_DIGITIZATION()

    Call iNet_int32(0, 0, 0, 0, 0, 0, intention_SetUpDigitization, 0)

End Sub

' Sets Demo mode to one of:
'
'   0:  iNet_NoDemo - real mode (no demo)
'   1:  iNet_FullSimulation - we fully simulate iNet hardware w/ triangle
'       waves feeding a fake iNet-100 & iNet-200.  This will reset instruNet.
'   2:  iNet_CorruptData - we read real instruNet hardware, yet we alter the
'       data randomly, to make it unuseable.
'
Sub SET_DEMO_MODE(ByVal demoMode As Integer)

    Call iNet_int32(0, 0, 0, 0, 0, 0, intention_SetDemoMode, demoMode)

End Sub


' Returns TRUE (1) if the iNet Driver is installed; FALSE (0) otherwise.

Function DRIVER_IS_INSTALLED()

    Dim num, test As Integer

    num = GET_NUMBER_OF_NETWORKS()
    
    test = Get_iNet_Error()
    
    If test = iNetErr_None Then
        DRIVER_IS_INSTALLED = iNetTrue
    Else
        DRIVER_IS_INSTALLED = iNetFalse
    End If

End Function

' ENABLE_CHANNEL_DIGITIZATION(netNum,deviceNum,moduleNum,chanNum)
' Returns TRUE (1) if the channel digitization is turned on;
' FALSE (0) otherwise.

Function ENABLE_CHANNEL_DIGITIZATION(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte, ByVal chanNum As Byte)
    
    Dim err As Integer
    
    ENABLE_CHANNEL_DIGITIZATION = iNet_int32(netNum, deviceNum, _
        moduleNum, chanNum, sgt_Display, fldNum_Display_dispOnOff, _
        intention_setValue, ion_gOnOffPopup_On)
        
    err = ENABLE_CHANNEL_DIGITIZATION

End Function

' Returns the direction of the channel: 0 is input (e.g. a/d), and 1 is output (e.g. d/a).
' gsw 5/1/96

Function GET_CHANNEL_DIRECTION(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte, ByVal chanNum As Byte)

    GET_CHANNEL_DIRECTION = iNet_int32(netNum, deviceNum, moduleNum, chanNum, 1, 1, intention_GetChannelDirection, 0&)

End Function

' Returns the realtime value of the channel,
' into the string ptrTo_Callers_C_Str255$

Function GET_CHANNEL_VALUE_C_STR255(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte, ByVal chanNum As Byte, _
        ByVal ptrTo_Callers_C_Str255 As String) As Integer
    
    GET_CHANNEL_VALUE_C_STR255 = iNet_VB_cString(netNum, deviceNum, moduleNum, _
        chanNum, sgt_General, fldNum_General_valueEU, intention_getValue, _
        ptrTo_Callers_C_Str255)

End Function

' Returns the realtime value of the channel, in floating point format.

Function GET_CHANNEL_VALUE_FLT32(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte, ByVal chanNum As Byte)

    GET_CHANNEL_VALUE_FLT32 = iNet_flt32(netNum, deviceNum, moduleNum, _
        chanNum, sgt_General, fldNum_General_valueEU, intention_getValue, 0&)

End Function

' Returns the realtime value of the channel,
' in +/- 2billion integer format.

Function GET_CHANNEL_VALUE_INT32(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte, ByVal chanNum As Byte)
 
    GET_CHANNEL_VALUE_INT32 = iNet_int32(netNum, deviceNum, moduleNum, _
        chanNum, sgt_General, fldNum_General_valueEU, intention_getValue, 0&)

End Function

' Returns the Continous Mode (On (1), Off (2) or Off With Skip (3)).

Function GET_SCAN_MODE(ByVal netNum As Byte)

    GET_SCAN_MODE = iNet_int32(netNum, deviceNum_CONTROLLER, _
        moduleNum_1stModule, controller_ChanNum_C12_Digitizer, _
        sgt_Timing, fldNum_Timing_scanMode, intention_getValue, 0&)

End Function

' Returns a Handle to the Network State (in uint32).
' The Caller must dispose of it after using it
' (e.g. DisposHandle() on Macintosh, GlobalFree() on Windows).

Function GET_HANDLE_TO_NETWORK_STATE() As Long
 
    GET_HANDLE_TO_NETWORK_STATE = iNet_uint32(0, 0, 1, 1, 1, 1, _
        intention_getHandleToNetworkState, 0&)

End Function

' Returns the native storage type for the field (i.e. iNetDataType enum in
' header file; e.g. iNetDT_INT16 (0), iNetDT_INT32 (3),
' iNetDT_UINT32 (4), iNetDT_FLT32 (5)).

Function GET_NATIVE_STORAGE_TYPE(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte, ByVal chanNum As Byte, settingGroupNum_, fieldNum_)

    GET_NATIVE_STORAGE_TYPE = iNet_int32(netNum, deviceNum, moduleNum, _
        chanNum, settingGroupNum, fieldNum, intention_nativeStorageType, 0&)

End Function

' Returns the network data transfer rate, in bits per second.

Function GET_NETWORK_BPS(ByVal netNum As Byte)

    GET_NETWORK_BPS = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, controller_ChanNum_C12_Digitizer, sgt_Timing, fldNum_Timing_network_bps, intention_getValue, 0&)

End Function

' Returns the # of Channels in the specified {Network, Device, Module}.

Function GET_NUMBER_OF_CHANNELS(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte)
    
    GET_NUMBER_OF_CHANNELS = iNet_int32(netNum, deviceNum, moduleNum, 1, 1, 1, intention_GetNumChannels, 0)

End Function

' Returns the # of Devices in the specified Network.

Function GET_NUMBER_OF_DEVICES(ByVal netNum As Byte)

    GET_NUMBER_OF_DEVICES = iNet_int32(netNum, 0, 1, 1, 1, 1, intention_GetNumDevices, 0)

End Function

' Returns the # of Fields in the specified {Network, Device, Module, Channel, SettingGroup}.

Function GET_NUMBER_OF_FIELDS(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte, ByVal chanNum As Byte, settingGroupNum)

    GET_NUMBER_OF_FIELDS = iNet_int32(netNum, deviceNum, moduleNum, chanNum, settingGroupNum, 1, intention_GetNumFields, 0)

End Function

' Returns the # of Channels in the specified {Network, Device, Module}.

Function GET_NUMBER_OF_MODULES(ByVal netNum As Byte, ByVal deviceNum As Byte)

    GET_NUMBER_OF_MODULES = iNet_int32(netNum, deviceNum, 1, 1, 1, 1, _
        intention_GetNumModules, 0)

End Function

' Returns the # of iNet Networks.

Function GET_NUMBER_OF_NETWORKS()

    GET_NUMBER_OF_NETWORKS = iNet_int32(0, 0, 1, 1, 1, 1, intention_GetNumNetworks, 0&)

End Function

' Returns the number of items in a popup, if the field user interface is a popup.
' (e.g. returns 2 if the popup shows {"ON", "Off"}).

Function GET_NUMBER_OF_POPUP_ITEMS(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal settingGroupNum, ByVal fieldNum)
 
    GET_NUMBER_OF_POPUP_ITEMS = iNet_int32(netNum, deviceNum, moduleNum, chanNum, settingGroupNum, fieldNum, intention_Get_NumPopupItems, 0&)

End Function

' Returns the number SCANS.

Function GET_NUMBER_OF_SCANS(ByVal netNum As Byte)

    GET_NUMBER_OF_SCANS = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, controller_ChanNum_C12_Digitizer, sgt_Timing, fldNum_Timing_noOfScans, intention_getValue, 0&)

End Function

' Returns the  of Settings in the specified {Network, Device, Module, Channel}.

Function GET_NUMBER_OF_SETTINGS(ByVal netNum As Byte, ByVal deviceNum As Byte, ByVal moduleNum As Byte, ByVal chanNum As Byte)

    GET_NUMBER_OF_SETTINGS = iNet_int32(netNum, deviceNum, moduleNum, _
        chanNum, 1, 1, intention_GetNumSettings, 0)

End Function

' Returns the number of POINTS in each SCAN.

Function GET_POINTS_PER_SCAN(ByVal netNum As Byte)

    GET_POINTS_PER_SCAN = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, _
        controller_ChanNum_C12_Digitizer, sgt_Timing, fldNum_Timing_ptsPerScan, _
        intention_getValue, 0)

End Function

' Returns the pointer to an array of C Str15 strings that contain the names
' of the Network, Device, Module, Channel, Settings, Field (i.e.
' iNetStr15 array[0] = Network Name, array[1] = Device Name, array[2] = Module Name,
' array[3] = Channel Name, array[4] = Settings Name, array[5] = Field Name).

Function GET_PTR_TO_ADDRESS_NAMES(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal settingGroupNum, _
    ByVal fieldNum)

    GET_PTR_TO_ADDRESS_NAMES = iNet_int32(netNum, deviceNum, moduleNum, chanNum, settingGroupNum, fieldNum, intention_Get_AddrNamesStringArray, 0&)

End Function

' Returns the pointer to an array of C Str15 strings that contain the items
' in the popup (i.e. iNetStr15 popupItems[]).

Function GET_PTR_TO_POPUP_ITEMS(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal settingGroupNum, _
    ByVal fieldNum)

    GET_PTR_TO_POPUP_ITEMS = iNet_int32(netNum, deviceNum, moduleNum, _
        chanNum, settingGroupNum, fieldNum, intention_Get_PopupStringArray, 0&)

End Function

' Returns the sample rate, in samples per second for each channel.

Function GET_SAMPLE_RATE_IN_HZ(ByVal netNum As Byte)

    GET_SAMPLE_RATE_IN_HZ = iNet_flt32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, controller_ChanNum_C12_Digitizer, sgt_Timing, fldNum_Timing_sampleRate, intention_getValue, 0)

End Function

Function GET_SENSOR_TYPE(ByVal netNum, ByVal deviceNum, ByVal modNum, ByVal chanNum)

    GET_SENSOR_TYPE = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, chanNum, sgt_VinHardware, fldNum_Vin_sensorType, intention_getValue, 0&)

End Function

' Returns the start of data acquisition time in seconds relative to
' a specified trigger condition

Function GET_START_TIME_IN_SEC(ByVal netNum As Byte)

    GET_START_TIME_IN_SEC = iNet_flt32(netNum, deviceNum_CONTROLLER, _
        moduleNum_1stModule, controller_ChanNum_C12_Digitizer, _
        sgt_Trigger, fldNum_Trigger_preTrigSec, intention_getValue, 0)

End Function


' Returns TRUE (1) if Driver is set up to show an alert upon error; FALSE (0) otherwise

Function GET_SHOW_MESSAGE_UPON_ERROR_STATUS()

    GET_SHOW_MESSAGE_UPON_ERROR_STATUS = iNet_int32(0, 0, 0, 0, 0, 0, intention_GetShowAlertOnErrorStatus, 0)

End Function


' Returns the channel switching mode as Accurate (1) or Fast (2).

Function GET_SWITCHING(ByVal netNum As Byte)
    
    GET_SWITCHING = iNet_int32(netNum, deviceNum_CONTROLLER, _
        moduleNum_1stModule, controller_ChanNum_C12_Digitizer, _
        sgt_Timing, fldNum_Timing_switching, intention_getValue, 0)

End Function

' Returns the user interface catagory (i.e. itemType_iNet enum in
' header file; e.g. ion_StaticText (3), ion_EditText (4),
' ion_UserItem (popup) (5)).

Function GET_USER_INTERFACE_CATAGORY(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal settingGroupNum, _
    ByVal fieldNum)
 
    GET_USER_INTERFACE_CATAGORY = iNet_int32(netNum, deviceNum, moduleNum, chanNum, settingGroupNum, fieldNum, intention_Get_UI_Catagory, 0&)

End Function

' Returns the user interface type (i.e. fieldUserInterfaceType enum in
' header file; e.g. fuit_editField_flt32, fuit_staticText_int16,
' fuit_popup_OnOff).

Function GET_USER_INTERFACE_TYPE(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal settingGroupNum, _
    ByVal fieldNum)
 
    GET_USER_INTERFACE_TYPE = iNet_int32(netNum, deviceNum, moduleNum, chanNum, settingGroupNum, fieldNum, intention_getUserInterfaceType, 0&)

End Function


' Lock internal memory so that future calls to iNet() are less likely to move
' memory (useful when calling iNet from interrupts). gsw 1/27/97

Function LOCK_INTERNAL_INET_MEMORY()

    LOCK_INTERNAL_INET_MEMORY = iNet_int32(0, 0, 0, 0, 0, 0, intention_lockInternalMemory, 0)

End Function

' Please use Open_instruNet_Window() instead.

Function OPEN_iNet_WINDOW(ByVal specificPage)

    Dim err As Integer
    err = iNet_int32(netNum_DRIVER, deviceNum_CONTROLLER, moduleNum_1stModule, driver_ChanNum_OpenWindow, sgt_General, fldNum_General_valueEU, intention_setValue, specificPage)
    OPEN_iNet_WINDOW = err

End Function

' This function is used to press a Button within the iNet World window (i.e. the buttons
' at the top of the iNet World window).  The iNet World window does not need to
' be open in order to do this.  For example, if you press the 'Store' button in NETWORK page,
' the state of all fields will be stored to disk, even though the user
' does not "see anything".  Below are several 'specificButton' values:
'
'   iNetCM_BtnPress_Record_Start    Press 'Start' button in RECORD page
'   iNetCM_BtnPress_Record_Stop     Press 'Stop' button in RECORD page
'   iNetCM_BtnPress_Record_Open     Press 'Open' button in RECORD page
'   iNetCM_BtnPress_Record_Save     Press 'Save' button in RECORD page
'   iNetCM_BtnPress_Record_Options  Press 'Options' button in RECORD page
'   iNetCM_BtnPress_Record_Timing   Press 'Timing' button in RECORD page
'   iNetCM_BtnPress_Record_Trigger  Press 'Trigger' button in RECORD page
'   iNetCM_BtnPress_Record_Probe    Press 'Probe' button in RECORD page
'   iNetCM_BtnPress_Network_Restore Press 'Restore' button in NETWORK page
'   iNetCM_BtnPress_Network_Store   Press 'Store' button in NETWORK page
'   iNetCM_BtnPress_Network_Open    Press 'Open' button in NETWORK page
'   iNetCM_BtnPress_Network_Save    Press 'Start' button in NETWORK page
'   iNetCM_BtnPress_Network_Clear   Press 'Clear' button in NETWORK page
'   iNetCM_BtnPress_Network_Reset  Press 'Reset' button in NETWORK page
'   iNetCM_BtnPress_Test_Search    Press 'Search' button in TEST page
'   iNetCM_BtnPress_Test_Test       Press 'Test' button in TEST page
'   iNetCM_BtnPress_Test_BigTest    Press 'Big Test' button in TEST page
'   iNetCM_BtnPress_Test_Open       Press 'Open' button in TEST page
'   iNetCM_BtnPress_Test_SaveAs     Press 'Save As' button in TEST page
'   iNetCM_BtnPress_Test_Clear      Press 'Clear' button in TEST page

Function PRESS_iNet_BUTTON(ByVal specificButton)
    
    Dim err As Integer
    err = iNet_int32(netNum_DRIVER, deviceNum_CONTROLLER, moduleNum_1stModule, driver_ChanNum_PushButton, sgt_General, fldNum_General_valueEU, intention_setValue, specificButton)
    PRESS_iNet_BUTTON = err
 
End Function

' Returns the  of seconds since reset.  This is derived from a 62bit
' 4MHZ counter on the iNet Controller board

Function SECONDS_SINCE_RESET(ByVal netNum As Byte) As Double

    Dim vbStr As String

    ' get secs since reset from instruNet
    Call iNet_VB_cString(netNum, deviceNum_CONTROLLER, _
            moduleNum_1stModule, controller_ChanNum_C11_Time, sgt_General, _
            fldNum_General_valueEU, intention_getValue, vbStr)
            
    SECONDS_SINCE_RESET = Val(vbStr)
    
    ' SECONDS_SINCE_RESET = iNet_flt32(netNum, deviceNum_CONTROLLER, _
    '    moduleNum_1stModule, controller_ChanNum_C11_Time, sgt_General, _
    '    fldNum_General_valueEU, intention_getValue, 0&)

End Function

' Services buffers during the digitization process.  This must be done continuously
' while digitizing.  Returns TRUE (1) if the next data pulled out
' out of the buffers is that last pull; FALSE (0) otherwise. A_PARAM()
' is set to TRUE (1) if we pulled to the end of a Scan; FALSE (0)
' otherwise.

Function SERVICE_ALL_DIGITIZE_BUFFERS()

    SERVICE_ALL_DIGITIZE_BUFFERS = iNet_int32(0, 0, 1, 1, 1, 1, intention_serviceAllDigitizeBuffers, 0&)

End Function

' Sets the realtime value of the channel, with a pointer to the caller's C string.

Function SET_CHANNEL_VALUE_C_STR255(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte, ByVal chanNum As Byte, _
    ByVal ptrTo_Callers_C_Str255 As String)
    
    SET_CHANNEL_VALUE_C_STR255 = iNet_VB_cString(netNum, deviceNum, moduleNum, chanNum, sgt_General, fldNum_General_valueEU, intention_setValue, ptrTo_Callers_C_Str255)

End Function

' Sets the value of the channel, with a floating point number.

Function SET_CHANNEL_VALUE_FLT32(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal newValue_flt32)

    Dim err As Integer
    err = iNet_flt32(netNum, deviceNum, moduleNum, chanNum, sgt_General, fldNum_General_valueEU, intention_setValue, newValue_flt32)
    SET_CHANNEL_VALUE_FLT32 = err

End Function

' Sets the realtime value of the channel, with a +/- 2billion integer format.

Function SET_CHANNEL_VALUE_INT32(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal newValue_int32 As Long)

    Dim err As Integer
    err = iNet_int32(netNum, deviceNum, moduleNum, chanNum, sgt_General, fldNum_General_valueEU, intention_setValue, newValue_int32)
    SET_CHANNEL_VALUE_INT32 = err

End Function

' Sets the CONTINOUS scans mode (ON means strip chart with continuous
' scans, OFF WITH SKIP is Oscilloscope mode with non-continuous traces), e.g.
'
'   SET_SCAN_MODE(netNum, ion_gScanModePopup_StripChart (1))
'   SET_SCAN_MODE(netNum, ion_gScanModePopup_OscilloQueued (2))
'   SET_SCAN_MODE(netNum, ion_gScanModePopup_Oscilloscope (3))

Function SET_SCAN_MODE(ByVal netNum As Byte, ByVal continuousMode As Long)

    Dim err As Integer
    err = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, controller_ChanNum_C12_Digitizer, sgt_Timing, fldNum_Timing_scanMode, intention_setValue, continuousMode)
    SET_SCAN_MODE = err

End Function

' Sets the network data transfer rate, in bits per second.

Function SET_NETWORK_BPS(ByVal netNum As Byte, ByVal new_Network_BPS)

    Dim err As Integer
    err = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, controller_ChanNum_C12_Digitizer, sgt_Timing, fldNum_Timing_network_bps, intention_setValue, new_Network_BPS)
    SET_NETWORK_BPS = err

End Function

' Sets the network state with the passed Handle. The Caller must dispose after
' calling this routine.  The handle must have originated from a
' GET_HANDLE_TO_NETWORK_STATE() call that was done previously.

Function SET_NETWORK_STATE_WITH_HANDLE(ByVal callerHandleToNetworkState)
    
    SET_NETWORK_STATE_WITH_HANDLE = iNet_uint32(0, 0, 1, 1, 1, 1, intention_setNetworkStateWithHandle, callerHandleToNetworkState)

End Function

' Sets the number SCANS.

Function SET_NUMBER_OF_SCANS(ByVal netNum As Byte, ByVal numberOfScans As Long)

    Dim err As Integer
    err = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, controller_ChanNum_C12_Digitizer, sgt_Timing, fldNum_Timing_noOfScans, intention_setValue, numberOfScans)
    SET_NUMBER_OF_SCANS = err

End Function

' Set the Overflow Alert Field, e.g.
'
'   SET_OVERFLOW_ALERT(ion_gOnOffPopup_On)   Show an alert when buffer overflow occurs.
'   SET_OVERFLOW_ALERT(ion_gOnOffPopup_Off)   Don't show an alert when buffer overflow occurs.

Function SET_OVERFLOW_ALERT(ByVal overflowAlertOption As Long)

    Dim err As Integer
    err = iNet_int32(netNum_DRIVER, deviceNum_CONTROLLER, moduleNum_1stModule, driver_ChanNum_RecordOptions, sgt_RecordOptions, fldNum_RecordOptions_overflowAlert, intention_setValue, overflowAlertOption)
    SET_OVERFLOW_ALERT = err
 
End Function

' Sets the number of POINTS in each SCAN.

Function SET_POINTS_PER_SCAN(ByVal netNum As Byte, ByVal numberOfPointsPerScan As Long)

    Dim err As Integer
    err = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, controller_ChanNum_C12_Digitizer, sgt_Timing, fldNum_Timing_ptsPerScan, intention_setValue, numberOfPointsPerScan)
    SET_POINTS_PER_SCAN = err

End Function

' Sets the sample rate, in samples per second for each channel.

Function SET_SAMPLE_RATE_IN_HZ(ByVal netNum As Byte, ByVal newSampleRate_Hz As Single)

    Dim err As Integer
    err = iNet_flt32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, controller_ChanNum_C12_Digitizer, sgt_Timing, fldNum_Timing_sampleRate, intention_setValue, newSampleRate_Hz)
    SET_SAMPLE_RATE_IN_HZ = err

End Function

' Save data to RAM, Disk or nowhere, e.g.
'
'   SET_SAVE_DATA(ion_gSaveDataPopup_Off)   Don't save data
'   SET_SAVE_DATA(ion_gSaveDataPopup_ToRamBuffer) Save Data to RAM
'   SET_SAVE_DATA(ion_gSaveDataPopup_ToDisk)  Save Data to DISK
'   SET_SAVE_DATA(ion_gSaveDataPopup_UserControl) Let user determine where to save

Function SET_SAVE_DATA(ByVal saveDataOption As Long)

    Dim err As Integer
    err = iNet_int32(netNum_DRIVER, deviceNum_CONTROLLER, moduleNum_1stModule, driver_ChanNum_RecordOptions, sgt_RecordOptions, fldNum_RecordOptions_saveData, intention_setValue, saveDataOption)
    SET_SAVE_DATA = err

End Function

' Sets the channel switching mode to Accurate (1) or Fast (2).

Function SET_SWITCHING(ByVal netNum As Byte, ByVal switchingMode As Long)

    Dim err As Integer
    err = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, controller_ChanNum_C12_Digitizer, sgt_Timing, fldNum_Timing_switching, intention_setValue, switchingMode)
    SET_SWITCHING = err

End Function

' Sets the Trigger mode to OFF, NORMAL or AUTOMATIC, e.g.
'
'   SET_TRIGGER_MODE(netNum, ion_TriggerMode_Off)
'   SET_TRIGGER_MODE(netNum, ion_TriggerMode_Auto)
'   SET_TRIGGER_MODE(netNum, ion_TriggerMode_Norm)

Function SET_TRIGGER_MODE(ByVal netNum As Byte, ByVal triggerMode As Long)

    Dim err As Integer
    err = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, controller_ChanNum_C12_Digitizer, sgt_Trigger, fldNum_Trigger_triggerModePop, intention_setValue, triggerMode)
    SET_TRIGGER_MODE = err

End Function

' Set the Trigger slope to rising or falling, e.g.
'
'   SET_TRIGGER_SLOPE(netNum, ion_Rising)
'   SET_TRIGGER_SLOPE(netNum, ion_Falling)

Function SET_TRIGGER_SLOPE(ByVal netNum As Byte, ByVal triggerSlope As Long)
 
    Dim err As Integer
    err = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, controller_ChanNum_C12_Digitizer, sgt_Trigger, fldNum_Trigger_slopeRisFalPop, intention_setValue, triggerSlope)
    SET_TRIGGER_SLOPE = err

End Function

' Sets the network address of the trigger source, e.g.
'
'   SET_TRIGGER_SOURCE(netNum, fldNum_Trigger_srcNet, netNum)
'   SET_TRIGGER_SOURCE(netNum, fldNum_Trigger_srcDevice, deviceNum)
'   SET_TRIGGER_SOURCE(netNum, fldNum_Trigger_srcModule, moduleNum)
'   SET_TRIGGER_SOURCE(netNum, fldNum_Trigger_srcChannel, chanNum)

Function SET_TRIGGER_SOURCE(ByVal netNum As Byte, ByVal fldNum_Src As Integer, ByVal srcNum As Long)
   
    Dim err As Integer
    err = iNet_int32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, controller_ChanNum_C12_Digitizer, sgt_Trigger, fldNum_Src, intention_setValue, srcNum)
    SET_TRIGGER_SOURCE = err

End Function

' Sets the Trigger threshold (engineering units) value.

Function SET_TRIGGER_THRESHOLD(ByVal netNum As Byte, ByVal triggerThresholdEu As Single)

    Dim err As Integer
    err = iNet_flt32(netNum, deviceNum_CONTROLLER, moduleNum_1stModule, controller_ChanNum_C12_Digitizer, sgt_Trigger, fldNum_Trigger_thresholdEu, intention_setValue, triggerThresholdEu)
    SET_TRIGGER_THRESHOLD = err

End Function



' Tells Driver to show a Error message dialog box whenever an error occurs.
' Returns TRUE (1) if Driver WAS (before this call) set up to show an alert
' upon error; FALSE (0) otherwise

Function SHOW_MESSAGE_DIALOG_UPON_ERROR()

    Dim err As Integer
    err = iNet_int32(0, 0, 0, 0, 0, 0, intention_ShowAlertOnError, 0)
    SHOW_MESSAGE_DIALOG_UPON_ERROR = err

End Function



' Returns 1 if CHANNEL = {netNum, deviceNum, moduleNum, channelNum} exists;
' FALSE (0) otherwise.
'
Function THIS_CHANNEL_EXISTS(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte, ByVal chanNum As Byte)

    thisExists = iNet_int32(netNum, deviceNum, moduleNum, chanNum, 1, 1, intention_ThisExists_Channel, 0)
    If thisExists = 0 Then
        THIS_CHANNEL_EXISTS = iNetFalse
    Else
        THIS_CHANNEL_EXISTS = iNetTrue
    End If
    
End Function

' Returns 1 if DEVICE = {netNum, deviceNum, moduleNum, channelNum} exists;
' FALSE (0) otherwise.

Function THIS_DEVICE_EXISTS(ByVal netNum As Byte, ByVal deviceNum As Byte)

    thisExists = iNet_int32(netNum, deviceNum, 1, 1, 1, 1, intention_ThisExists_Device, 0)
    If thisExists = 0 Then
        THIS_DEVICE_EXISTS = iNetFalse
    Else
        THIS_DEVICE_EXISTS = iNetTrue
    End If

End Function

' Returns 1 if FIELD = {netNum, deviceNum, moduleNum, channelNum, settingNum,
' fieldNum} exists; FALSE (0) otherwise.

Function THIS_FIELD_EXISTS(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal settingGroupNum, _
    ByVal fieldNum)

    thisExists = iNet_int32(netNum, deviceNum, moduleNum, chanNum, settingGroupNum, fieldNum, intention_ThisExists_Field, 0&)
    If thisExists = 0 Then
        THIS_FIELD_EXISTS = iNetFalse
    Else
        THIS_FIELD_EXISTS = iNetTrue
    End If

End Function

' Returns 1 if MODULE = {netNum, deviceNum, moduleNum} exists;
' FALSE (0) otherwise.

Function THIS_MODULE_EXISTS(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte)

    thisExists = iNet_int32(netNum, deviceNum, moduleNum, 1, 1, 1, intention_ThisExists_Module, 0)
    If thisExists = 0 Then
        THIS_MODULE_EXISTS = iNetFalse
    Else
        THIS_MODULE_EXISTS = iNetTrue
    End If

End Function

' Returns 1 if NETWORK = {netNum} exists; FALSE (0) otherwise.

Function THIS_NETWORK_EXISTS(ByVal netNum As Byte)

    thisExists = iNet_int32(netNum, 0, 1, 1, 1, 1, intention_ThisExists_Network, 0)
    If thisExists = 0 Then
        THIS_NETWORK_EXISTS = iNetFalse
    Else
        THIS_NETWORK_EXISTS = iNetTrue
    End If

End Function

' Returns 1 if SETTING = {netNum, deviceNum, moduleNum, channelNum, settingNum}
' exists; FALSE (0) otherwise.

Function THIS_SETTING_EXISTS(ByVal netNum As Byte, ByVal deviceNum As Byte, _
    ByVal moduleNum As Byte, ByVal chanNum As Byte, ByVal settingGroupNum)

    thisExists = iNet_int32(netNum, deviceNum, moduleNum, chanNum, settingGroupNum, 1, intention_ThisExists_Setting, 0)
    If thisExists = 0 Then
        THIS_SETTING_EXISTS = iNetFalse
    Else
        THIS_SETTING_EXISTS = iNetTrue
    End If

End Function

' Unlocks internal memory when the processor is outside of instruNet driver.

Function UNLOCK_INTERNAL_INET_MEMORY()

    UNLOCK_INTERNAL_INET_MEMORY = iNet_int32(0, 0, 0, 0, 0, 0, intention_unlockInternalMemory, 0)

End Function


