/* ************************************************************************************
INET_EX1.C							Companions: "INET_INT.C", "INET_INT.H", "INET_MCS.H"

	------------------------------- A B S T R A C T ----------------------------------
		
	RECOMMENDATION
	
		Try to do your applications first with the BASIC class routines, documented
		in INET_INT.C, and demonstrated in Example #3.  If these are not adequate, 
		then move to the ADVANCED class routines demonstrated in Examples #1 and #2. 
		In other words, ignor the ADVANCED routines until you are sure you need them.

	This file contains C code examples that do I/O with the instruNet hardware.

	This C code is platform, Operating System, and processor independent. This code
	should run on Macintosh, and Windows 95/NT C and C++ compilers without
	any problems. 
	
	This file expects tabs every 4 characters.
	
	The the comments in this file will help you understand how to program instruNet.
	
	The instruNet disk includes the following examples:

		INET_EX1.C	- contains example C code that uses the instruNet hardware

	These examples are typically used with the following files:
		
		INET_INT.C  - contains low level source code that interfaces to instruNet driver
		INET_INT.H  - contains many low level enums and #defines that support instruNet
		INET_MCS.H  - contains macros that help set/read instruNet fields
	
	The instruNet disk includes two example programs:

	The instruNet disk includes three example programs:

		EXAMPLE #1		Shows how to use almost every instruNet macro and subroutine
						in a text window-based environment.  This example is quite lengthy,
						uses many ADVANCED functions, and is based on file INET_EX1.C.

		EXAMPLE #2		Shows an instruNet application that reads and writes
						several instruNet fields, and then digites.  This example is
						based on file INET_EX2.C, and uses many ADVANCED functions.

		EXAMPLE #3		Example #3  reads and writes fields in a simple way,
						and is the easiest of all the example programs. This example
						uses only the BASIC class functions (not ADVANCED), and is
						based on file INET_EX3.c
	
	-------------------------------- H I S T O R Y -----------------------------------
	HISTORY		Copyright c 1995-96 GWI.  All rights reserved.
	5/18/94		created - gsw
	11/22/95	added code for Microsoft Windows - kag

************************************************************************************ */

/* ================================== #includes ==================================== */

//	instruNet includes

	#include "INET_INT.H"				//	instruNet C Interface (for End User)
	#include "INET_MCS.H"				//	instruNet Macros & Functions (for End User)

//	ANSI includes
 
#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	=========== Macintosh ========== */

	#include <stdio.h>					//	for scanf(), printf()	
	#include <stdlib.h>					//	for malloc()	
	#include <console.h>				//	for C_CBREAK


#elif (MACHINE_TYPE_ == WINTEL_iNet) 	/*	========= Windows/Intel ======== */
									
	#include <stdio.h>					//	for scanf(), printf()	
	#include <stdlib.h>					//	for malloc()	
	#include <windows.h>				//  This is included in INOC_INT.H but is 
										//  is needed for inoc_ex1.h
#endif
								
/* ================================== externs ====================================== */

/* ================================== globals ====================================== */

char gAnswer_cStr[300]; 
										
#if (MACHINE_TYPE_ == WINTEL_iNet) 		/*	============  Windows/Intel ============ */

	HWND ghwndMain;
	HINSTANCE ghInstance, ghPrevInstance;
	HWND ghwndEditWindow = 0;					
	WNDPROC gDefaultEditProcedure = 0;
	LPSTR gAppTitle = "instruNet Sample Application";
	BOOL gIsCustomEditReturnHit;
	LPSTR gUserInput_cStrPtr = 0;

#endif

/* ================================= #defines ====================================== */

#define MAIN_WND_CLASS_NAME "IONC_EX1"

										/*	max number of channels to digitize */
#define MAX_NUM_DIGITIZE_CHANNELS  (MAX_NUM_DIGI_CH	/* 512 */)
 
/* ============================= static prototypes ================================= */

void AskQuestion( char * question_cStrPtr );

iNetError SetSettingsWithMacros(void);
iNetError Print_iNet_Fields(iNetINT32 maxNumberOffFieldsToPrint);
iNetError PrintOneField_iNet(iNetUINT8 netNum, iNetUINT8 devNum,
	iNetUINT8 modNum, iNetUINT8 chNum, iNetINT16 settingGroupNum,
	iNetINT16 fieldNum, char *message);
iNetError ReadWriteFields_iNet(void);
iNetError SimultaneouslyDigitizeAndAnalyze(iNetINT16 allowMouseDownToStopDigitize,
	iNetINT16 processOneScanAtATime, iNetINT32 numPointsPerScan,
	iNetFLT32 pointsPerSecondPerChannel, iNetINT16 *numChannelsDigitizedP,
	iNetChannelAddr *digitizeChAddr, iNetFLT32 **digitizeDataPP);
iNetError Print_iNet_Channels(iNetINT32 maxNumberOffChannelsToPrint);
iNetError ExerciseFileSaveLoadCode(iNetINT16 numChannelsDigitized, iNetChannelAddr *digitizeChAddr);
void Exercise_The_iNet_Macros(void);
void Setup_Window_For_Printing(void);
void Print_Line(char *cStr_lineToPrint);
void Process_OS_Events(void);
void Process_OS_Events_Until_Close(void);
void Convert_CrLf_To_Lf(char *in_, char *out_);
iNetError HighSpeedSimultaneousInputAndOutput(Boolean allowMouseDownToStopDigitize);
	
#if (MACHINE_TYPE_ == WINTEL_iNet) 	/*	============  Windows/Intel ============ */

	LRESULT CALLBACK MsWin_DemoWndProc  (HWND, WORD, WPARAM, LPARAM);
	LRESULT CALLBACK MsWin_CustomEditProcdure(HWND hWnd,WORD message,WPARAM wParam,LPARAM lParam );  // kag 1/22/96 
	static void MsWin_OnDestroy(HWND hWnd);
	static void MsWin_OnCommand(HWND hWnd, int id, HWND hwndCtl, UINT codeNotify);
	static void MsWin_OnSize(HWND hWnd, UINT state, int cx, int cy);
	static LPSTR MsWin_GetUserInputFromLastLineOfEditControl();	
	
#endif

/* =================================== C Code  ===================================== */
	
/* -------------------------------------------------------------------------------------
 *	FUNCTION  	main()
 *
 *	PURPOSE 	This is the main entry to an application program that shows an example
 *				of how to monitor and control the instruNet network.
 *				
 *	HISTORY		2/4/96		created - gsw
 * ----------------------------------------------------------------------------------- */

#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	============= Macintosh ============= */

int main()								/*	Main entry for a Macintosh application */
{

#elif (MACHINE_TYPE_ == WINTEL_iNet)	/*	=========== Windows/Intel =========== */

int PASCAL WinMain (					/*	Main entry for a windows application */
	HINSTANCE hInstance,
	HINSTANCE hPrevInstance,
    LPSTR lpCmdLine,
	int nCmdShow)
{
#endif
	iNetError e = iNetErr_None;
	iNetINT16 driverIsInstalledOK, numNetworks, numChannelsDigitized;
	iNetINT32 numPointsPerScan;
	iNetFLT32 pointsPerSecondPerChannel;

	iNetFLT32 *digitizeDataP[ MAX_NUM_DIGITIZE_CHANNELS ];
	iNetChannelAddr digitizeChAddr[ MAX_NUM_DIGITIZE_CHANNELS ];

	iNetChannelAddr vout, vin;
	Boolean chExists;
	iNetFLT32 vout_voltage, vin_value, vin_valueB, new_vout_voltage;

#if (MACHINE_TYPE_ == WINTEL_iNet)		/*	=========== Windows/Intel =========== */
	ghInstance = hInstance;
	ghPrevInstance = hPrevInstance;
	STOP_COMPILER_WARNING(nCmdShow);
	STOP_COMPILER_WARNING(lpCmdLine);
#endif

  	//	-----------------------------------------------------------------------
	//	Show instruNet World window and exit program when user closes
	//	the  instruNet World window.

	#ifdef DRIVER_IS_COMPILED_INTO_AN_APP	//	if debugging at the factory...

											//	load the instruNet driver
		Load_instruNet_Driver(FALSE /* forceInstruNetReset */, &driverIsInstalledOK, &numNetworks);

		#if 0								//	do some testing
			Setup_Window_For_Printing();
			Process_OS_Events();
			HighSpeedSimultaneousInputAndOutput( TRUE /* allowMouseDownToStopDigitize */);
		#endif

		#if 0								//	do some testing
			{	iNetINT16 testing = Get_iWp_LicenseState();
				Show_Simple_Alert(testing ? "1" : "0"); }
		#endif

		#if 0								//	more testing
			Set_iNet_Demo_Mode(1);
			e = Set_iNet_TIMING_Parameters(1000. /* ptsPerSecPerChan */, 1000 /* ptsPerScan */, 10 /* numScans */, 1, 1, 1, 1);
			numPointsPerScan = GET_POINTS_PER_SCAN(1);
		#endif
											//	Open the NETWORK window
		OPEN_iNet_WINDOW(iNetCM_OpenWindow_Network);

		CloseDriverAndReleaseDriverRam();	//	Closes the instruNet driver
		return e;

	#endif
	
  	//	-----------------------------------------------------------------------
	//	Initialize the printf() console text window.

	Setup_Window_For_Printing();
	Process_OS_Events();

	//	-----------------------------------------------------------------------
	//	Load instruNet driver and initialize instruNet network.
		
	e = Load_instruNet_Driver(FALSE /* forceInstruNetReset */, &driverIsInstalledOK, &numNetworks);							
	if (e != iNetErr_None)	{ Show_ALERT_if_hit_iNet_Error(e); return 0; }

	//	-----------------------------------------------------------------------
	//	Open the instruNet World View Window.
	//
	//	Since this is the 1st iNet call, it will initializes the iNet
	//	hardware and shows an alert if there is trouble.	
	
										//	open the instruNet World window, Network Page
	e = OPEN_iNet_WINDOW(iNetCM_OpenWindow_Network);
	if (e) { goto Exit; }
	Process_OS_Events();				//	tell the OS to do it's thing

	//	-----------------------------------------------------------------------
	//	Load address of 'Ch1 Vin+' and 'Ch3 Vout' into 'vin' and 'vout' structs
	//	(this assumes a instruNet 100 is the 1st device on the net).

										//	set Ch1 Vin+ address
	chExists = (Boolean) LoadChannelAddress( &vin, 1 /* net */, 1 /* device */, 1 /* module */, 1 /* chan */);
	if (!chExists)	{ goto Exit; }
										//	set Ch3 Vout address
	chExists = (Boolean) LoadChannelAddress( &vout, 1 /* net */, 1 /* device */, 1 /* module */, 3 /* chan */);
	if (!chExists)	{ goto Exit; }

	//	-----------------------------------------------------------------------
	//	Read 'Ch1 Vin+' value.
										//	read Ch1 Vin+ value
	vin_value = GetField_flt32( &vin, sgt_General, fldNum_General_valueEu, &e);
	if (e) { goto Exit; }

	//	-----------------------------------------------------------------------
	//	Set 'Ch3 Vout' voltage to 2.0Volts.

	vout_voltage = (iNetFLT32) 2.0;		//	set Ch3 Vout to 2.0Volts.
	e = SetField_flt32( &vout, sgt_General, fldNum_General_valueEu, vout_voltage);
	if (e) { goto Exit; }
		
	//	-----------------------------------------------------------------------
	//	Read 'Ch1 Vin+' value, by the EASIEST, LAZIEST, SIMPLEST way possible.

	vin_valueB = GetChannelValue_iNet(1 /* net */, 1 /* device */, 1 /* module */, 1 /* chan */, &e);
	if (e) { goto Exit; }

	//	-----------------------------------------------------------------------
	//	Set 'Ch3 Vout' voltage to 3.0Volts, by the EASIEST, LAZIEST, SIMPLEST way possible.

	new_vout_voltage = (iNetFLT32) 3.0;
	e = SetChannelValue_iNet(1 /* net */, 1 /* device */, 1 /* module */, 3 /* chan */, new_vout_voltage);

	//	-----------------------------------------------------------------------
	//	Print the values of the first 5 channels in your network.

	e = Print_iNet_Channels(5);			//	print value of 5 channels
	if (e) { goto Exit; }
	Process_OS_Events();				//	tell the OS to do it's thing
		
	//	-----------------------------------------------------------------------
	//	Print the 1st 3 fields in the iNet network
	
	e = Print_iNet_Fields(3);			//	Print the first 3 fields
	if (e) { goto Exit; }
	Process_OS_Events();				//	tell the OS to do it's thing
		
	//	-----------------------------------------------------------------------
	//	R/W specific fields.

	e = ReadWriteFields_iNet();			//	r/w specific fields
	if (e) { goto Exit; }
	Print_Line ((char *) "\r\n");		//	advance 1 line
	Process_OS_Events();				//	tell the OS to do it's thing

	//	-----------------------------------------------------------------------
	//	Digitize.
										//	tell the user what we are going to do, gsw 2/12/96
	Print_Line ((char *) "We will now digitize for 3secs (or until you press the mouse button).\r\n");		
		
	numPointsPerScan = 1000L;			//	Digitize 1000 point scans at a  
										//	1000 pts/sec/channel sample rate.
	pointsPerSecondPerChannel = (iNetFLT32) 1000.0;

	e = SimultaneouslyDigitizeAndAnalyze(TRUE /* allowMouseDownToStopDigitize */,
										 TRUE /* processOneScanAtATime */,
										 numPointsPerScan,
										 pointsPerSecondPerChannel /* sampleRate */,
										 &numChannelsDigitized /* loaded with # of digitized channels */,
										 &digitizeChAddr[0] /* loaded with address of digitized channels */,
										 &digitizeDataP[0] /* loaded with pointers to digitized data */);
	if (e) { goto Exit; }
	Process_OS_Events();				//	tell the OS to do it's thing

	//	-----------------------------------------------------------------------
	//	Demonstrate a high speed input and output (i.e. Vin & Vout). gsw 9/26/98

	#define RUN_HIGHSPEED_OUTPUT_AND_INPUT 0

	#if RUN_HIGHSPEED_OUTPUT_AND_INPUT
		HighSpeedSimultaneousInputAndOutput( TRUE /* allowMouseDownToStopDigitize */);			
	#endif

	//	-----------------------------------------------------------------------
	//	Store 1 scan of data (i.e. 1000pts) for each digitized channel in 
	//	a new directory (to illustrate some file i/o commands). gsw 1/28/97

	Print_Line ((char *) "We will now save to disk 1 scan of data from each digitized channel.\r\n");		

#if 0
	e = ExerciseFileSaveLoadCode(numChannelsDigitized, &digitizeChAddr[0]);
	if (e) { goto Exit; }
#endif

	//	-----------------------------------------------------------------------
	//	Exercise the macros.  Please see the Macro file (i.e. "INET_MCS.H")
	//	for documentation on each marcro. 

	#if 0 								/*	Set to 1 to exercise all iNet macros. */

		Print_Line ((char *) "We will now exercise the Macros.\r\n\r\n");

		Exercise_The_iNet_Macros();		//	exercise the macros
	
	#endif
	
	//	-----------------------------------------------------------------------
	//	Shows an example of setting up channels with some handy macros.
	
	#if 0 								/*	Set to 1 to run this code */
		SetSettingsWithMacros();
	#endif

	//	-----------------------------------------------------------------------
	//	Exit.

Exit:
										//	tell the user we are done
	Print_Line ((char *) "We just finished running Example#1.  For a simpler example, see Example #3.\r\n\r\n");

	if (e)		{						//	if we got an error, show an alert,
		Show_ALERT_if_hit_iNet_Error(e);
	}
	Process_OS_Events_Until_Close();	//	All done. Let the user manipulate
										//	the console window and close.

	CloseDriverAndReleaseDriverRam();	//	Closes the instruNet driver and 
										//	releases the memory used to hold it
	return e;
}


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	Print_iNet_Fields()	
 *
 *	PURPOSE 	This routine will print (to the screen) the value of the first
 *				'maxNumberOffFieldsToPrint' number of fields in your instruNet network.
 *				
 *	HISTORY		1/1/96		created - gsw
 *				11/30/95	hit hard - gsw
 * ----------------------------------------------------------------------------------- */
iNetError Print_iNet_Fields(
										//	this is the maximum # of fields to print
	iNetINT32 maxNumberOffFieldsToPrint)
{
	iNetUINT8 chNum, netNum, devNum, modNum;
	char message[300];					
	short numberOfDevices, numberOfChannels, numberOfFields, fieldNum, numberOfNetworks;
	short numberOfModules, numberOfSettings, settingNum;
	iNetError e = iNetErr_None;
										//	print title
	Print_Line ((char *) "\r\nFIELD VALUES\r\n");	

										//	get # of instruNet Networks in this Computer
	numberOfNetworks = (short) GET_NUMBER_OF_NETWORKS();	
	if ((e = Get_iNet_Error()) != iNetErr_None)	{ goto Exit; }
		
										//	scan through all Networks
	for (netNum = 1; netNum <= numberOfNetworks; netNum++)		{
										
										//	get # of Devices on this Network
		numberOfDevices = (short) GET_NUMBER_OF_DEVICES(netNum);	
		if ((e = Get_iNet_Error()) != iNetErr_None)	{ goto Exit; }

										//	scan through all Devices
		for (devNum = 1; devNum <= numberOfDevices; devNum++)		{
										
										//	get # of Modules in this Device
			numberOfModules = (short) GET_NUMBER_OF_MODULES(netNum, devNum);	
			if ((e = Get_iNet_Error()) != iNetErr_None)	{ goto Exit; }

										//	scan through all Modules
			for (modNum = 1; modNum <= numberOfModules; modNum++)		{
										
										//	get # of Channels in this Module
				numberOfChannels = (short) GET_NUMBER_OF_CHANNELS(netNum, modNum, devNum);	
				if ((e = Get_iNet_Error()) != iNetErr_None)	{ goto Exit; }

										//	scan through Channels
				for (chNum = 1; chNum <= numberOfChannels; chNum++)		{
										
										//	get # of Settings groups in this Channel				
					numberOfSettings = (short) GET_NUMBER_OF_SETTINGS(netNum, modNum, devNum, chNum);	
					if ((e = Get_iNet_Error()) != iNetErr_None)	{ goto Exit; }

										//	scan through Settings Groups
					for (settingNum = 1; settingNum <= numberOfSettings; settingNum++)		{
										
										//	get # of Fields in this setting			
						numberOfFields = (short) GET_NUMBER_OF_FIELDS(netNum, modNum, devNum, chNum, settingNum);	
						if ((e = Get_iNet_Error()) != iNetErr_None)	{ goto Exit; }

										//	scan through each Field
						for (fieldNum = 1; fieldNum <= numberOfFields; fieldNum++)		{
					
										//	print one Field in one line
							e = PrintOneField_iNet(netNum, devNum,
								modNum, chNum, settingNum,
								(iNetINT16) fieldNum, &message[0]);
								
										//	if we have printed enough fields...
							if (--maxNumberOffFieldsToPrint <= 0L)
								goto Exit;
						}
					}
				}
			}
		}
	}
	
Exit:

	if (e)	{							//	if we got an error, show an alert,
		Show_ALERT_if_hit_iNet_Error(e);
	}
	return e;
}


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	Print_iNet_Channels()	
 *
 *	PURPOSE 	This routine will print (to the screen) the value of 
 *				'maxNumberOffChannelsToPrint' number of channels in your iNet network.
 *				
 *	HISTORY		1/1/96		created - gsw
 *				11/30/95	hit hard - gsw
 * ----------------------------------------------------------------------------------- */
iNetError Print_iNet_Channels(
										//	this is the maximum # of channels to print
	iNetINT32 maxNumberOffChannelsToPrint)
{
	char message[300];					
	short numberOfDevices, devNum, numberOfChannels, chNum, numberOfNetworks, netNum;
	short modNum, numberOfModules;
	iNetError e = iNetErr_None;
										//	print title
	Print_Line ((char *) "\r\nCHANNEL VALUES\r\n");	

										//	get # of iNet Networks in this Computer
	numberOfNetworks = (short) GET_NUMBER_OF_NETWORKS();	
	if ((e = Get_iNet_Error()) != iNetErr_None)	{ goto Exit; }

										//	scan through all Networks
	for (netNum = 1; netNum <= numberOfNetworks; netNum++)		{
										
										//	get # of Devices on this Network
		numberOfDevices = (short) GET_NUMBER_OF_DEVICES(netNum);	
		if ((e = Get_iNet_Error()) != iNetErr_None)	{ goto Exit; }

										//	scan through all Devices
		for (devNum = 1; devNum <= numberOfDevices; devNum++)		{
										
										//	get # of Modules in this Device
			numberOfModules = (short) GET_NUMBER_OF_MODULES(netNum, devNum);	
			if ((e = Get_iNet_Error()) != iNetErr_None)	{ goto Exit; }

										//	scan through all Modules
			for (modNum = 1; modNum <= numberOfModules; modNum++)		{
										
										//	get # of Channels in this Module
				numberOfChannels = (short) GET_NUMBER_OF_CHANNELS(netNum, modNum, devNum);	
				if ((e = Get_iNet_Error()) != iNetErr_None)	{ goto Exit; }

										//	scan through Channels
				for (chNum = 1; chNum <= numberOfChannels; chNum++)		{
										
										//	print the value field in the
										//	GENERAL Settings area, which is
										//	the value of the channel.
					e = PrintOneField_iNet((iNetUINT8) netNum, (iNetUINT8) devNum,
							(iNetUINT8) modNum, (iNetUINT8) chNum, sgt_General,
							fldNum_General_valueEu, &message[0]);

										//	if we have printed enough channels...
					if (--maxNumberOffChannelsToPrint <= 0L)	{
						goto Exit;	
					}
				}
			}
		}
	}
	
Exit:

	if (e)	{							//	if we got an error, show an alert,
		Show_ALERT_if_hit_iNet_Error(e);
	}
	return e;
}


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	PrintOneField_iNet()	
 *
 *	PURPOSE 	This routine prints out 1 line of information on the specified field
 *				in the following format:
 *
 *				netNum, devNum, modNum, chNum, settingGroupNum, fieldNum,
 *				fieldName, FieldValue
 *				
 *	HISTORY		1/1/96		created - gsw
 *				11/30/95	hit hard - gsw
 * ----------------------------------------------------------------------------------- */
iNetError PrintOneField_iNet(
	iNetUINT8 netNum,					//	NETWORK number = {0...numNetworks}, 0 is Driver, 1 is 1st Controller
	iNetUINT8 devNum,				//	DEVICE number {0...numDevices}, 0 = "Controller", 1 is 1st device 
	iNetUINT8 modNum, 				//	MODULE number within a hardware DEVICE {1...32}. 
	iNetUINT8 chNum, 					//	CHANNEL number {1...32}.
	iNetINT16 settingGroupNumOrType,	//	SETTING Group number {1... numSettingsGroupsInChannel}
	iNetINT16 fieldNum, 				//	FIELD number {1...32}.
	char *message)						//	we fill this C string with info
										//	about the field
{	
	iNetError e;						
	char fieldNameStr[100], fieldValueStr[100];		
		
										//	get the name of this field,
	e = iNet_cString(netNum, devNum, modNum, chNum,
			settingGroupNumOrType, fieldNum, intention_getNameStr, &fieldNameStr[0]);
	if (e)	{ goto Exit; }
										//	get the value of this field,
	e = iNet_cString(netNum, devNum, modNum, chNum, 
			settingGroupNumOrType, fieldNum, intention_getValue, &fieldValueStr[0]);
	if (e)	{ goto Exit; }
										//	load C String message[] with info about field
	sprintf(&message[0],
		"%2d %2d %2d %2d %2d %2d %s: %s\r\n\0",
		(int) netNum, (int) devNum, (int) modNum, (int) chNum, (int) settingGroupNumOrType, (int) fieldNum,
		&fieldNameStr[0], &fieldValueStr[0]);

	Print_Line((char *) &message[0]);	//	print the message

Exit:

	if (e)	{							//	if we got an error, show an alert,
		Show_ALERT_if_hit_iNet_Error(e);
	}
	return e;
}


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	ReadWriteFields_iNet()
 *
 *	PURPOSE 	This routine reads and writes iNet hardware fields.
 *				
 *	HISTORY		1/1/96	created - gsw
 * ----------------------------------------------------------------------------------- */
iNetError ReadWriteFields_iNet(void)
{
	char message[300], s[300], valueStr[300];
	int /* %d wants int */ devNum, chNum, fieldNum, netNum, modNum, settingNum;
	iNetError e;

	Print_Line ((char *) "\r\nREAD AND WRITE FIELDS\r\n\r\n");

	//	-----------------------------------------------------------------------
	//	Get device #, channel #, and field # from user.

LookAtAField:
										//	prompt user
	AskQuestion( "Input netNum,devNum,modNum,chNum,setNum,fieldNum (e.g. \"1,1,1,1,1,1\") ? ");
	sscanf (gAnswer_cStr, "%d,%d,%d,%d,%d,%d", &netNum, &devNum, &modNum, &chNum, &settingNum, &fieldNum);

	Print_Line ((char *) "\r\n");		//	go to new line

	//	-----------------------------------------------------------------------
	//	Print one field on one line.

PrintTheField:
										//	print the value of this field
	e = PrintOneField_iNet((iNetUINT8) netNum, (iNetUINT8) devNum,
			(iNetUINT8) modNum, (iNetUINT8) chNum,
			(iNetINT16) settingNum, (iNetINT16) fieldNum, (char *) &message[0]);

	Print_Line ((char *) "\r\n");		//	go to new line

	//	-----------------------------------------------------------------------
	//	Ask question
										//  Introduced AskQuestion
	AskQuestion("Do you want to Modify this field's value, Look at another field, or Exit (M, L, E)?  ");
	sscanf (gAnswer_cStr, "%s", s);
	Print_Line ((char *) "\r\n");		//	go to new line

	switch (*s)	{

		case 'M':
		case 'm':
			AskQuestion( "What is the new value?  ");
			sscanf (gAnswer_cStr, "%f", &valueStr[0]);

										//	set this field to the new value, which
										//	is in the form of a C string
			e = iNet_cString((iNetUINT8) netNum, (iNetUINT8) devNum, (iNetUINT8) modNum,
				(iNetUINT8) chNum, (iNetINT16) settingNum, (iNetUINT8) fieldNum,
				intention_setValue, &valueStr[0]);
										
										//	if we got an error, bail out
			Show_ALERT_if_hit_iNet_Error(e);

			goto PrintTheField;			//	go print out the value
			break;

		case 'L':
		case 'l':
			goto LookAtAField;
			break;

		case 'E':
		case 'e':
		default:
			return iNetErr_None;
			break;
	}
}


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	SimultaneouslyDigitizeAndAnalyze()
 *
 *	PURPOSE 	This routine simultaneously digitizes and analyzes signals from an
 *				instruNet network.
 *				
 *	HISTORY		2/2/96		created - gsw
 * ----------------------------------------------------------------------------------- */

iNetError SimultaneouslyDigitizeAndAnalyze(
	iNetINT16 allowMouseDownToStopDigitize,
										//	if TRUE (1), we allow a mouse down to stop the
										//	digitize process; otherwise, if FALSE (0), will
										//	will run till finished.
	iNetINT16 processOneScanAtATime,	//	if TRUE (1), we access digitized data 1 complete
										//	Scan at a time; otherwise if FALSE (0), we process
										//	little segments of data, as they become available.
	iNetINT32 numPointsPerScan,			//	points per scan
	iNetFLT32 pointsPerSecondPerChannel,//	sample rate for each channel
	iNetINT16 *numChannelsDigitizedP,	//	filled with # of channels that are digitized
	iNetChannelAddr *digitizeChAddr,	//	we load this with addresses of digitized channels
	iNetFLT32 **digitizeDataPP)			//	we load this array with ptrs to the scan buffers
										//	used for each channel
{
	iNetError e;
	iNetINT16 numChannelsToDigitize;	//	# of channels to digitize
	iNetINT16 theNextDataAccessIsTheLast_ofEntireDigitization;
	iNetFLT32 *bufferStartAddrP;
	iNetINT16 weAreDoingOurFirstAccessOfData;
	iNetUINT32 pointToPullindex_base1, numPointsToPull, scanNumberToPull_base1;
	iNetFLT32 *firstPoint, *lastPoint;
	iNetINT16 theNextDataAccessWillPullToTheEndOfAScan, showAlertWas_ON_BeforeThisCall;
	iNetUINT32 last_scanNum_base1_pulled_inFull;
	iNetUINT32 last_scanNum_base1_pushed_inFull;
	iNetINT16 controllerIsFinishedWithDigitization, digChNum;
	iNetFLT32 differenceBetweenAdjacentPoints, thisPoint, previousPoint;
	
	iNetUINT32 totalPtsPulledSoFar[ MAX_NUM_DIGITIZE_CHANNELS ];
	double sumOfPoints[ MAX_NUM_DIGITIZE_CHANNELS ];
	
	iNetFLT32 dataValue_MIN[ MAX_NUM_DIGITIZE_CHANNELS ];
	iNetFLT32 dataValue_MAX[ MAX_NUM_DIGITIZE_CHANNELS ];
	iNetFLT32 difference_MIN[ MAX_NUM_DIGITIZE_CHANNELS ];
	iNetFLT32 difference_MAX[ MAX_NUM_DIGITIZE_CHANNELS ];
	
	iNetChannelAddr triggerChannel;
	Boolean chExists;

	//	-----------------------------------------------------------------------
	//	Clear all instruNet FIELD values to their default states (if you want).
		
	#if 0								//	set to 1 to press 'Clear' button in NETWORK page
		e = PRESS_iNet_BUTTON(iNet_ButtonID_Network_Clear);
		if (e)	{ goto Exit; }
	#endif
											
	//	-----------------------------------------------------------------------
	//	Specify 3 channels to digitize from.
	
	numChannelsToDigitize = 0;			//	count # of channels we are digitizing

										//	Ch1 Vin+ = 0th channel to digitize
	chExists = (Boolean) LoadChannelAddress( &digitizeChAddr [ 0 ],
		1 /* net */, 1 /* device */, 1 /* module */, 1 /* chan */);
	if (!chExists)	{ goto Exit; }
	numChannelsToDigitize++;			//	count # of channels we are digitizing

										//	Ch2 Vin- = 1st channel to digitize
	chExists = (Boolean) LoadChannelAddress( &digitizeChAddr [ 1 ],
		1 /* net */, 1 /* device */, 1 /* module */, 2 /* chan */);
	if (!chExists)	{ goto Exit; }
	numChannelsToDigitize++;			//	count # of channels we are digitizing

										//	Ch4 Vin+ = 2nd channel to digitize
	chExists = (Boolean) LoadChannelAddress( &digitizeChAddr [ 2 ],
		1 /* net */, 1 /* device */, 1 /* module */, 4 /* chan */);
	if (!chExists)	{ goto Exit; }
	numChannelsToDigitize++;			//	count # of channels we are digitizing
		
										//	make sure we are not digitizing too many channels
	if (numChannelsToDigitize > MAX_NUM_DIGITIZE_CHANNELS)	{
		goto Exit;
	}
		
	if (numChannelsDigitizedP != 0L) {	//	tell caller how many channels we are digitizing
		*numChannelsDigitizedP = numChannelsToDigitize;	
	}

	//	-----------------------------------------------------------------------
	//	Enabled digitizing for these channels.

										//	disable all channels for digitizing (initially)
	e = DisableAllChannelsForDigitizing();	
	if (e)	{ goto Exit; }
										//	scan thru all digitize channels
	for (digChNum = 0; digChNum < numChannelsToDigitize; digChNum++)	{

										//	enabled digitizing for this channel
		e = EnableChannelForDigitizing(&digitizeChAddr [ digChNum ]);
		if (e)	{ goto Exit; };
	}

	//	-----------------------------------------------------------------------
	//	Set TIMING parameters.  Tell it to digitize 1000 points per Scan at 1000
	//	points per second per channel (i.e. 1sec Scans) and do this
	//	3 times (i.e. 3 Scans).  We also tell iNet to put the digitized data
	//	into the DRIVER ram buffer (we will get access to this buffer as the
	//	data is digitized).  And we tell iNet to show an alert if we
	//	overflow a data buffer.  StripChartRecorder mode is turned ON, which
	//	means that the Scans will be continuous (last point of one Scan is directly
	//	followed by first point of next Scan).
	
	e = Set_iNet_TIMING_Parameters(
			pointsPerSecondPerChannel, 	//	samplesPerSecond
			numPointsPerScan,			//	pointsPerScan per channel
			3,							//	numberOfScans
			TRUE,						//	stripChartRecorder
			ion_gSaveDataPopup_ToRamBuffer,	//	saveDataOption
			TRUE, 						//	showAlertOnOverflow
			TRUE);						//	switch channels accurately
	if (e)	{ goto Exit;}				//	if we got an error, bail out

	//	-----------------------------------------------------------------------
	//	Set TRIGGER parameters.  We specify the trigger source to be the channel
	//	specified in the 'triggerChannel' channel address.  We also say that the
	//	channel is set to trigger on the rising edge of the source when it
	//	crosses 0.0.  At the moment, the trigger function is deactivated
	//	(i.e. ion_TriggerMode_Off).

										//	Ch1 Vin+ = trigger channel
	chExists = (Boolean) LoadChannelAddress( &triggerChannel, 1 /* net */, 1 /* device */, 1 /* module */, 1 /* chan */);
	if (!chExists)	{ goto Exit; }

	e = Set_iNet_TRIGGER_Parameters(
			ion_TriggerMode_Off,		//	triggerMode
			(float) 0.0,				//	triggerThreshold	
			ion_Rising,					//	triggerSlope
			(float) 0.0,				//	hysteresis
			&triggerChannel);			//	trigger source channel
	if (e)	{ goto Exit;}				//	if we got an error, bail out
	
	//	-----------------------------------------------------------------------
	//	Tell instruNet driver to show an alert if there is an error (in addition
	//	to returning an error code).
										//	Tell instruNet driver to show an alert
										//	(in addition to returning an error code)
										//	if it gets into trouble
	showAlertWas_ON_BeforeThisCall = (short) SHOW_MESSAGE_DIALOG_UPON_ERROR();

	//	-----------------------------------------------------------------------
	//	Initialize variables for mathematical processing of the digitized 
	//	waveforms.  We do this to show how you can analyze data WHILE it
	//	is being recorded, in real-time.  In this example, we calculate
	//	the average, minimum, maximum, minimum difference (i.e. minimum of
	//	the derivative), and maximum difference from each digitized channel.

	for (digChNum = 0; digChNum < numChannelsToDigitize; digChNum++)	{
	
		totalPtsPulledSoFar[digChNum] = 0L;			//	clear accumulator that keeps track 
													//	of the total # of point digitized
													//	in this routine.
		sumOfPoints[digChNum]    = 0.0;				//	clear the sum of points for each channel
		dataValue_MIN[digChNum]  = (float) +1.e30;	//	preset min data value
		dataValue_MAX[digChNum]  = (float) -1.e30;	//	preset max data value
		difference_MIN[digChNum] = (float) +1.e30;	//	preset min difference value
		difference_MAX[digChNum] = (float) -1.e30;	//	preset max difference value
	}
	
	//	-----------------------------------------------------------------------
	//	Tell instruNet to digitize, and process the segments (or Scans) of data
	//	as they become available.

	//	The data is stored in buffers, one Scan large, in memory that is
	//	managed by the instruNet driver.  The Access_Digitized_Data_In_Ram_Buffer()
	//	routine provides you with the address of this memory buffer, so that
	//	you can access the data.  In fact, if 'processOneScanAtATime' is set to
	//	FALSE (0), it will give you the address within the buffer of little chunks
	//	of data as they become available, so that you can little pieces as they
	//	are acquired.

	//	If the "Save Data" field (i.e. 'saveDataOption') in the "Record Options"
	//	settings group is set to "To Ram Buffer", as is done in this example, then
	//	the digitized data is automatically sent to ram buffers maintained by the
	//	instruNet driver; otherwise, if "Save Data" is set "To File", then the data 
	//	is spooled to disk.
	
	//	If you wanted to digitize 1 scan worth of data, and then save it to
	//	disk, set the "numberOfScans" paramater to 1, tell it to begin
	//	recording, and then when it is done recording, press the SAVE button
	//	in the record page via "PRESS_iNet_BUTTON(iNet_ButtonID_Record_Save)"
	//	to save the 1 scan to disk.
	
	//	If you wanted to record forever yet don't have an infinite amount of RAM,
	//	set the "numberOfScans" paramater to a really huge number, and then process
	//	the data, 1 scan at a time, in the RAM buffer maintained by the driver (which
	//	is continually overwritten).
	
	//	If you wanted to record for a long time and spool to disk while recording,
	//	set the "Save Data" field to "To File" and tell it to start digitizing.
	//	You could then scroll through the spooled data, post acquisition,
	//	in the instruNet World window.
	
	//	If you wanted the data to be sent to your own ram buffer (i.e. you create
	//	the buffer and maintain it) then:
	//
	//	a)	Set the "Save Data" field to "User Control".
	//	b)	Set the USER BUFFER Setting Groups fields accordingly:
	//		- Set the "Digitize" field to "On" to enable the channel for digitizing.
	//	    - Set the "User Addr" field to the address of your RAM buffer, which must
	//		  be able to hold 1 Scan's worth of FLT32 data (i.e. numberOfScans * 4 bytes).
	//		- Set the "Buff Byte" field to the size of your RAM buffer (which must be
	//		  at least (numberOfScans * 4) bytes long.
	//	c)	Start recording, and instead of being handed an address in the instruNet
	//		Driver's ram buffer by Access_Digitized_Data_In_Ram_Buffer( 'driverRamBuffer'
	//		set FALSE), you will be handed an address in your own buffer (so that you
	//		can monitor the data as it is being digitized).
	
	//	Keep in mind that data is overwritten in a RAM buffer if you digitize
	//	multiple scans.  For example, if 'numberOfScans' is set to 2, then the
	//	RAM buffer will be written to twice, and your 1st scan if data will be overwritten.
	//	If you can process the data before it is overwritten (e.g. calculated averages and
	//	spike positions, etc) that is very cool; otherwise, you might need to save it in
	//	some way.  For example, you could save it to disk, or you could save it to
	//	another RAM location.  If this is difficult, then please consider increasing
	//	the 'pointsPerScan' and setting the 'numberOfScans' parameter to 1, so that the 
	//	scan buffer is not overwritten.
	
	//	DATA ACQUISITION IS MUCH EASIER WHEN 'numberOfScans' IS SET TO 1.
	
										//	-----------------------------------
										//	START DIGITIZING
										//
	Stop_iNet_Digitizing();				//	make sure we are not digitizing and are
										//	ready to start digitizing, gsw 10/20/98

										//	Press the start button to tell instruNet
										//	to begin digitizing.
	e = PRESS_iNet_BUTTON( iNet_ButtonID_Record_Start );
	if (e)	{ goto Exit;}				//	if we got an error, bail out

										//	say that we are starting out
	weAreDoingOurFirstAccessOfData = TRUE;
	
	do 	{								//	loop until the digitization is finished..
	
										//	-----------------------------------
										//	SERVICE instruNet's DIGITIZE BUFFERS
										//
										//	It is ABSOLUTELY NECESSARY to let your
										//	computer's processor service instruNet digitizing
										//	buffers periodicaly (10 to .3 times per second)
										//	while digitizing.  This is done by continuously
										//	calling Service_All_iNet_Digitize_Buffers() in a loop.
		e = Service_All_iNet_Digitize_Buffers(
				digitizeChAddr[0].netNum,
				&theNextDataAccessIsTheLast_ofEntireDigitization,
				&theNextDataAccessWillPullToTheEndOfAScan,
				&controllerIsFinishedWithDigitization,
				&last_scanNum_base1_pulled_inFull,
				&last_scanNum_base1_pushed_inFull);
		if (e)	{ goto Exit;}			//	if we got an error, bail out
		
		numPointsToPull = 0L;			//	Initialize the number of points to
										//	pull our of the buffer.
										
										//	If we are to process the data little
										//	segments at a time, or we are processing
										//	1 scan at at time and we have that 1 
										//	scan available in the buffers, then
										//	go process the data.
		if (!processOneScanAtATime || theNextDataAccessWillPullToTheEndOfAScan)	{
			
										//	service the channels that have been
										//	turned on for digitizing
			for (digChNum = 0; digChNum < numChannelsToDigitize; digChNum++)	{
							
										//	-----------------------------------
										//	READ DIGITIZED DATA
										//
										//	Here we get an address in memory where
										//	the most recent digitized data resides.
										//
										//	'bufferStartAddrP[]'
										//	
										//		iNetFLT32 data buffer that is
										//		1 Scan long
										//	
										//	'pointToPullindex_base1'
										//
										//		new data is available starting at
										//		bufferStartAddrP[pointToPullindex_base1 - 1]
										//
										//	'numPointsToPull'
										//
										//		number of new data points that are
										//		available
										//
										//	'scanNumberToPull_base1'
										//
										//		scan number {1...numScans} of the
										//		scan that we are currently accessing.
				e = Access_Digitized_Data_In_Ram_Buffer(
						&digitizeChAddr[digChNum], TRUE /* driver ram buffer */,
						&bufferStartAddrP, &pointToPullindex_base1,
						&numPointsToPull, &scanNumberToPull_base1);
		
				if (e)	{ goto Exit;}	//	if we got an error, bail out

										//	remember the start address for each
										//	channel's scan buffer
				digitizeDataPP[ digChNum ] = bufferStartAddrP;

										//	-----------------------------------
										//	PROCESS DIGITIZED DATA
										//
										//	Now that we have some digitized data,
										//	we can process it.
										//
										//	This show examples of processing
										//	digitized data, immediately after it has
										//	been recorded.
										
										//	-----------
										//	Calculate the total # of pts processed to data
										
				totalPtsPulledSoFar[ digChNum ] = totalPtsPulledSoFar[digChNum] + numPointsToPull;
								
										//	-----------
										//	Calculate the sum of digitized points
										//	(this is an example of doing analysis
										//	while digitizing)
									
										//	calc pointer to 1st data point to process
				firstPoint = bufferStartAddrP + pointToPullindex_base1 - 1L;

										//	calc pointer to last data point to process
				lastPoint = firstPoint + numPointsToPull - 1L;
					
										//	scan through the digitized data stored in the
										//	RAM buffer
				while (firstPoint <= lastPoint)	{
										//	accumulate the sum of all points
					sumOfPoints[digChNum] += *firstPoint++;
				}
										//	-----------
										//	Calculate the maximum difference between
										//	adjacent points (i.e. the max and min of
										//	the derivative), the min data value,
										//	and the maximum data value.
										
										//	calc pointer to 1st data point to process
				firstPoint = bufferStartAddrP + pointToPullindex_base1 - 1L;
										//	if we are on our first access, start
										//	with the 2nd point -- otherwise the derivative
										//	gets messed up
				if (weAreDoingOurFirstAccessOfData)
					firstPoint++;
										//	scan through the digitized data stored in the
										//	RAM buffer
				while (firstPoint <= lastPoint)	{
									
										//	get value of this point and previous point
					thisPoint = *firstPoint;
					previousPoint = *(firstPoint-1);
					firstPoint++;		//	go to the next point
					
										//	calc minimum data value
					if (thisPoint < dataValue_MIN[digChNum])
						dataValue_MIN[digChNum] = thisPoint;

										//	calc maximum data value
					if (thisPoint > dataValue_MAX[digChNum])
						dataValue_MAX[digChNum] = thisPoint;

										//	calc the difference between 2 adjacent points
					differenceBetweenAdjacentPoints = thisPoint - previousPoint;
					
										//	calc minimum difference
					if (differenceBetweenAdjacentPoints < difference_MIN[digChNum])
						difference_MIN[digChNum] = differenceBetweenAdjacentPoints;
						
										//	calc maximum difference
					if (differenceBetweenAdjacentPoints > difference_MAX[digChNum])
						difference_MAX[digChNum] = differenceBetweenAdjacentPoints;
				}
			}
		}
										
		if (numPointsToPull > 0) {		//	say that we are no longer starting out
			weAreDoingOurFirstAccessOfData = FALSE;
		}
										//	-----------------------------------
										//	ABORT ON MOUSE DOWN
										//
										//	if we are to allow a mouse
										//	down to stop the digitizing...
		if (allowMouseDownToStopDigitize)	{
				
			if (MOUSE_BUTTON_IS_DOWN()){//	if the mouse button is down...
		
				Stop_iNet_Digitizing();	//	press the STOP button to stop digitizing
										
										//	note that we have stopped
				theNextDataAccessIsTheLast_ofEntireDigitization = TRUE;
			}
		}
										//	sit in loop while there is data to
										//	be pulled out of the buffers
	} while (!theNextDataAccessIsTheLast_ofEntireDigitization  iNetAnd  !controllerIsFinishedWithDigitization);

Exit:

	if (e == iNetErr_None)	{			//	if we had no errors...
		
		iNetStr255 message;

		sprintf((char *) &message[0],	//	print the message
			"We successfully digitized %ld points for each of %ld channels.\r\n\0",
			totalPtsPulledSoFar[0], numChannelsToDigitize);
	
		Print_Line((char *) &message[0]);	
	}

	if (e)		{						//	if we got an error, show an alert
		Show_ALERT_if_hit_iNet_Error(e);
	}

	Stop_iNet_Digitizing();				//	make sure we are not digitizing

	ResetChannelList();					//	clear the list of channels to digitize
										//	(to disconnect from the local channel buffers)

	return e;
}
				

/* -------------------------------------------------------------------------------------
 *	FUNCTION	HighSpeedSimultaneousInputAndOutput()
 *
 *	PURPOSE 	This demonstrates simultaneously recording from one channel and
 *				outputting to another channel, at a high speed, with Ch1Vin+ and Ch3Vout
 *				both in the DigitizeList.
 *
 *	HISTORY		10/20/98		created - gsw
 * ----------------------------------------------------------------------------------- */
iNetError HighSpeedSimultaneousInputAndOutput(
	Boolean allowMouseDownToStopDigitize)//	if true, a mouse down will stop the process
{
	//	-----------------------------------------------------------------------
	//	Local Variables.

	#define BUFF_PTS  		100			/*  pts digitized from each channel */

	#define NUM_DIGI_CH 	(2)			/* 	num of channels to digitize */
										/*  (careful to not blow stack with a number that is too large) */
	iNetError e;
	iNetINT16 digChNum, numCh;
	iNetUINT32 numScansToDigitize, last_scanNum_base1_pulled_inFull_oneCtlr;
	iNetFLT32 *pointP, *firstPointOfSegment, *lastPointOfSegment;
	iNetFLT32 sampleRate;
	iNetUINT32 pointToPullindex_base1, numPointsToPull, scanNumberToPull_base1;

	#define SAFETY 2L

	iNetFLT32	digiBuffer [ NUM_DIGI_CH + SAFETY ] [ BUFF_PTS + SAFETY ];

	iNetUINT32	totalPtsPulled	[ NUM_DIGI_CH + SAFETY ];
	double		accum			[ NUM_DIGI_CH + SAFETY ];
	iNetFLT32   value			[ NUM_DIGI_CH + SAFETY ];
	iNetFLT32   delta 			[ NUM_DIGI_CH + SAFETY ];
	iNetFLT32  *bufferStartAddrP[ NUM_DIGI_CH + SAFETY ];

	//	-----------------------------------------------------------------------
	//	Initialize.

	numScansToDigitize = 2000000000L;	//	# of scans to digitize
	sampleRate = (iNetFLT32) 1000.0;	//	sample rate
	numCh = 0;							//	start out with no channels in the list

	if (allowMouseDownToStopDigitize)	{
		Show_Simple_Alert("We will run a high speed feedback loop until you press the mouse button.");
	}

	e = ResetChannelList();				//	clear the list of channels to digitize
	if (e) { goto Exit; }
									
										//	Add 'Ch1 Vin' to list
	e = AddToChannelList(1,1,1,1 /* chNum */, &digiBuffer[ numCh++ ][ 0 ], BUFF_PTS * sizeof(iNetFLT32));
	if (e) { goto Exit; }
										//	Add 'Ch3 Vout' to list
	e = AddToChannelList(1,1,1,3 /* chNum */, &digiBuffer[ numCh++ ][ 0 ], BUFF_PTS * sizeof(iNetFLT32));
	if (e) { goto Exit; }
										//	make sure we don't overflow our array
	if (numCh > NUM_DIGI_CH) {	goto Exit; }		
		
	e = Set_iNet_TIMING_Parameters(		//	set the timing options
			(iNetFLT32) sampleRate, 	//	sample rate (points per second per channel)
			BUFF_PTS,					//	number of points in 1 scan, for 1 channel
			numScansToDigitize,			//	number of scans that are digitized when the
										//	start button is pressed
			TRUE,						//	if TRUE (1) we do Strip Chart Recorder work with
										//	Continuous Scans, otherwise if FALSE (0) we do
										//	Oscilloscope mode (non-Coninuous Scans)
			ion_gSaveDataPopup_ToRamBuffer /* 2 */,			
										//	Save Data option = { ion_gSaveDataPopup_Off,
										//	ion_gSaveDataPopup_ToRamBuffer, ion_gSaveDataPopup_ToDisk,
										//	ion_gSaveDataPopup_UserControl}
			TRUE,						//	if TRUE (1), the driver will show an alert if
										//	a data buffer overflows; otherwise if FALSE (0) 
										//	it just keeps going if there is an overflow.
			TRUE);						//	if TRUE (1), channel switching should be accurate;
										//	otherwise, it is less accurate, yet faster.
	if (e) { goto Exit; }
										//	start triangle wave output at 0.0 Volts
	for (digChNum = 0; digChNum < numCh; digChNum++)	{
		value[ digChNum ] = (iNetFLT32) 0.0;	
		delta[ digChNum ] = (iNetFLT32) (4.0 / 27.0);
	}

	Stop_iNet_Digitizing();				//	make sure we are not digitizing and are
										//	ready to start digitizing, gsw 10/2/97
						
										//	set up internal buffers before digitizing
										//	and set establish the actual sample rate
	e = (iNetError) SET_UP_DIGITIZATION();		
	if (e) { goto Exit; }														
		
	do	{								//	start digitizing and process little segments
										//	as they are acquired

										//	-----------------------------------
										//	GENERATE SOME OUTPUT DATA

										//	scan thru the list of digitized channels
		for (digChNum = 0; digChNum < numCh; digChNum++)	{
	
										//	if this is an output channel tied to a
										//	circular buffer (e.g. d/a)...
			if (g_Channel_dType[ digChNum ] == dir_hdwr_output_circBuf /* 2 */)	{

				iNetUINT32 pointToPushindex_base1, maxNumPointsToPush = 0L, scanNumberToPush_base1;

										//	-----------------
										//	Get location of place to put output data
				e = GetOutputSegmentBuffer(digChNum, &bufferStartAddrP[ digChNum ],
						&pointToPushindex_base1, &maxNumPointsToPush, &scanNumberToPush_base1);
				if (e) { goto Exit; }

										//	playing
				// if (maxNumPointsToPush > 33) { maxNumPointsToPush = 33; }

										//	-----------------
										//	if there is space in the buffer to place new data...
				if (maxNumPointsToPush > 0L) {

										//	define the 1st and last point of output segment
					firstPointOfSegment = bufferStartAddrP[ digChNum ] + pointToPushindex_base1 - 1L;
					lastPointOfSegment  = firstPointOfSegment + maxNumPointsToPush - 1L;
				
					{					//	synthesize a triangle wave
						register iNetFLT32 v = value[ digChNum ];
						register iNetFLT32 d = delta[ digChNum ];

										//	scan thru the little output SEGMENT
						for (pointP = firstPointOfSegment; pointP <= lastPointOfSegment; pointP++)	{
														
							v += d;		//	inc or dec the value
								
										//	switch directions if we hit a bound
							if (((iNetFLT32) v > 4.0) || ((iNetFLT32) v < -4.0))	{
								d *= -1.0; 
							}

							*pointP =v;	//	output the triangle wave
						}
						value[ digChNum ] = v;
						delta[ digChNum ] = d;
					}
				}							
										//	-----------------
										//	SUBMIT A SEGMENT OF OUTPUT DATA
										//
										//	(after we had loaded it into the buffer
										//	specified by GetOutputSegmentBuffer()).
										//
										//	Note: When GetOutputSegmentBuffer() gives you
										//	a buffer, it is not necessary to load the entire
										//	buffer -- only what you want.
				e = SubmitOutputSegment(digChNum, maxNumPointsToPush);
				if (e) { goto Exit; }
			}
		}
										//	---------------------
										//	SERVICE THE DIGITIZE BUFFERS

										//	digitize the channels in the channel list
										//	and service the digitize buffers
		e = DigitizeListOfChannels(FALSE /* allowMouseDownToStopDigitize */,
								   FALSE /* processOneScanAtATime */,
		 						  &last_scanNum_base1_pulled_inFull_oneCtlr);
		if (e) { goto Exit; }

										//	---------------------
										//	PROCESS THE INPUT DATA

										//	scan thru the list of digitized channels
		for (digChNum = 0; digChNum < numCh; digChNum++)	{
	
										//	if this is an input channel...
			if (g_Channel_dType[ digChNum ] != dir_hdwr_output_circBuf /* 2 */)	{

										//	GET A LITTLE SEGMENT OF DIGITIZED DATA
				e = GetDigitizedSegment(digChNum, &bufferStartAddrP[ digChNum ],
						&pointToPullindex_base1, &numPointsToPull, &scanNumberToPull_base1);
				if (e) { goto Exit; }

				if (numPointsToPull > 0L) {	//	PROCESS LITTLE SEGMENTS OF DATA

										//	define the 1st and last point of segment
					firstPointOfSegment = bufferStartAddrP[ digChNum ] + pointToPullindex_base1 - 1L;
					lastPointOfSegment  = firstPointOfSegment + numPointsToPull - 1L;
				
										//	scan thru the little SEGMENT (to calc the avg value)
					for (pointP = firstPointOfSegment; pointP <= lastPointOfSegment; pointP++)	{
										//	add this point to the accumulator
						accum[ digChNum ] += (double) *pointP;
					}	
										//	remember the total # of pts read so far for this channel
					totalPtsPulled[ digChNum ] += numPointsToPull;	
				}							
			}
		}
										//	-----------------------------------
										//	ABORT ON MOUSE DOWN
										//
										//	if we are to allow a mouse down to stop us...
		if (allowMouseDownToStopDigitize)	{
				
			if (MOUSE_BUTTON_IS_DOWN()){//	if the mouse button is down...
		
				Stop_iNet_Digitizing();	//	press the STOP button to stop digitizing
				break;					//	break out of while() loop
			}
		}

	} while ((iNetUINT32) last_scanNum_base1_pulled_inFull_oneCtlr < numScansToDigitize);

Exit:

	Stop_iNet_Digitizing();				//	make sure we are not digitizing

	ResetChannelList();					//	clear the list of channels to digitize
										//	(to disconnect from the local channel buffers)

	Show_Simple_Alert("We completed high speed input and output.");

	return e;
}


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	ExerciseFileSaveLoadCode()
 *
 *	PURPOSE 	This routine saves the digitize data to disk, and exercises some
 *				of the file i/o commands.  More specifically, it:
 *
 *					+	tells iNet driver to create a new directory for the data
 *					+	saves each channel to disk (i.e. data in DRIVER RAM BUFFER to file on disk)
 *					+   loads in information about each file
 *					+   loads the data on disk back into the DRIVER RAM BUFFER
 *				
 *	HISTORY		1/27/9		created - gsw
 * ----------------------------------------------------------------------------------- */
iNetError ExerciseFileSaveLoadCode(
	iNetINT16 numChannelsDigitized,		//	number of digitized channels
	iNetChannelAddr *digitizeChAddr)	// 	addresses of digitized channels
{
	iNetChannelAddr directory;
	iNetINT16 digChNum;
	iNetINT32 pointNum, scanNum, numPts;
	iNetError e = iNetErr_None;
										//	Specify Driver Channel = { Record Options, Directory}
	LoadChannelAddress( &directory, netNum_DRIVER, deviceNum_DRIVER, moduleNum_1stModule, driver_ChanNum_RecordOptions);

										//	----------------------------------
										//	create a new directory
	e = SetField_int32(	&directory, sgt_MasterDirectory, fldNum_Directory_dirCmd, dirCmd_CreateNewDirectory);
	if (e) { goto Exit; }

										//	loop thru each digitized channel
	for (digChNum = 0; digChNum < numChannelsDigitized; digChNum++) 	{

										//	----------------------------------
										//	Send command to save the data to disk in the
										//	new directory.  This save's entire scan to disk
										//	for 1 channel
		e = SetField_int32(	&digitizeChAddr[digChNum], sgt_File, fldNum_File_fileCmd, fileCmd_RamBufferToFile);
		if (e) { goto Exit; }

										//	----------------------------------
										//	Send command to load info about the file into
										//	the following fields (for diagnostic purposes):
										//	
										//		ScanNum = scan # of last point pushed into buffer = {1...numScans}
										//		PointNum = point # of last point pushed into buffer = {1...PtsPerScan}
										//		NumPts = pts per scan stored in the file for this channel

										//	read info about file on disk
		e = SetField_int32(	&digitizeChAddr[digChNum], sgt_File, fldNum_File_fileCmd, fileCmd_Get_File_Info);
		if (e) { goto Exit; }
										//	ScanNum = scan # of last point pushed into buffer = {1...numScans}
		scanNum = GetField_int32(&digitizeChAddr[digChNum], sgt_File, fldNum_File_ScanNum, &e);
		if (e) { goto Exit; }
										//	PointNum = point # of last point pushed into buffer = {1...PtsPerScan}
		pointNum = GetField_int32(&digitizeChAddr[digChNum], sgt_File, fldNum_File_PointNum, &e);
		if (e) { goto Exit; }
										//	NumPts = pts per scan stored in the file for this channel
		numPts = GetField_int32(&digitizeChAddr[digChNum], sgt_File, fldNum_File_NumPts, &e);
		if (e) { goto Exit; }

										//	----------------------------------
										//	Send command to load from disk the saved data.
										//	We must first specify where to load the data:
										//	
		scanNum = 1 /* only 1 scan in file */;	
										//		ScanNum = Scan # of 1st point pulled out of buffer = {1...numScans}
		pointNum = 1;					//		PointNum = Point # of 1st point pulled out of buffer = {1...PtsPerScan}
		/* numPt is already set to 1 scan size */
										//		NumPts = # of points to load in

										//	ScanNum = Scan # of 1st point pulled out of buffer = {1...numScans}
		e = SetField_int32(&digitizeChAddr[digChNum], sgt_File, fldNum_File_ScanNum, scanNum);
		if (e) { goto Exit; }
										//	PointNum = Point # of 1st point pulled out of buffer = {1...PtsPerScan}
		e = SetField_int32(&digitizeChAddr[digChNum], sgt_File, fldNum_File_PointNum, pointNum);
		if (e) { goto Exit; }
										//	NumPts = # of points to load in
		e = SetField_int32(&digitizeChAddr[digChNum], sgt_File, fldNum_File_NumPts, numPts);
		if (e) { goto Exit; }
										//	send command to load in the data from disk into the 
										//	driver's ram buffer
		e = SetField_int32(	&digitizeChAddr[digChNum], sgt_File, fldNum_File_fileCmd, fileCmd_FileToRamBuffer);
		if (e) { goto Exit; }
	}

Exit:
	return e;
}



/* -------------------------------------------------------------------------------------
 *	FUNCTION  	SetSettingsWithMacros()
 *
 *	PURPOSE 	This shows an example of setting up some channels using macros.
 *				
 *	HISTORY		11/18/97 created - gsw
 * ----------------------------------------------------------------------------------- */
iNetError SetSettingsWithMacros()
{
	iNetError e = iNetErr_None;
	
	//	----------------------------------------------------------------------
	//	Common functions.

	#define 	SetField		SetFieldValue_iNet
							
	//	----------------------------------------------------------------------
	//	Macros to set HARDWARE fields within channels.

	#define  Set_SensorType(addr_, sensorType_)		\
		SetField( addr_, sgt_VinHardware, fldNum_Vin_sensorType, sensorType_); if (e) { goto Exit; }

	#define  Set_WiringType(addr_, wiring_)		\
		SetField( addr_, sgt_VinHardware, fldNum_Vin_wiring, wiring_); if (e) { goto Exit; }

	#define  Set_Integration(addr_, seconds_)		\
		SetField( addr_, sgt_VinHardware, fldNum_Vin_ad_IntegrateSecs, (iNetFLT32) seconds_); if (e) { goto Exit; }
		
	#define  Set_AnaLpFltr(addr_, FreqCutoffHz_)		\
		SetField( addr_, sgt_VinHardware, fldNum_Vin_AnaLpFltr_Hz, (iNetFLT32) FreqCutoffHz_); if (e) { goto Exit; }

	#define  Set_VoltageRange(addr_, maxRangeInVolts_)		\
		SetField( addr_, sgt_VinHardware, fldNum_Vin_ad_VmaxRange, (iNetFLT32) maxRangeInVolts_); if (e) { goto Exit; }
	
	//	----------------------------------------------------------------------
	//	Macros to set CONSTANTS fields within channels.

	#define  Set_Constants_Ro(addr_, Ro_)		\
		SetField( addr_, sgt_VinConstants, fldNum_VinCon_Ro, (iNetFLT32) Ro_); if (e) { goto Exit; }

	#define  Set_Constants_Rshunt(addr_, Rshunt_)		\
		SetField( addr_, sgt_VinConstants, fldNum_VinCon_Rshunt, (iNetFLT32) Rshunt_); if (e) { goto Exit; }

	#define  Set_Constants_Vexc(addr_, Vexcitation_)		\
		SetField( addr_, sgt_VinConstants, fldNum_VinCon_Vexcitation, (iNetFLT32) Vexcitation_); if (e) { goto Exit; }

	#define  Set_Constants_Vinit(addr_, Vinit_)		\
		SetField( addr_, sgt_VinConstants, fldNum_VinCon_Vinit, (iNetFLT32) Vinit_); if (e) { goto Exit; }

	#define  Set_Constants_alpha(addr_, alpha_)		\
		SetField( addr_, sgt_VinConstants, fldNum_VinCon_alpha, (iNetFLT32) alpha_); if (e) { goto Exit; }

	#define  Set_Constants_delta(addr_, delta_)		\
		SetField( addr_, sgt_VinConstants, fldNum_VinCon_delta, (iNetFLT32) delta_); if (e) { goto Exit; }

	#define  Set_Constants_GF(addr_, GF_)		\
		SetField( addr_, sgt_VinConstants, fldNum_VinCon_GF, (iNetFLT32) GF_); if (e) { goto Exit; }

	#define  Set_Constants_v_Poisson(addr_, v_Poisson_)		\
		SetField( addr_, sgt_VinConstants, fldNum_VinCon_v_Poisson, (iNetFLT32) v_Poisson_); if (e) { goto Exit; }
		
	//	----------------------------------------------------------------------
	//	Macros to set MAPPING fields within channels.

	#define  Set_Mapping_internal1(addr_, internal1_)		\
		SetField( addr_, sgt_Mapping, fldNum_Mapping_internal1, (iNetFLT32) internal1_); if (e) { goto Exit; }

	#define  Set_Mapping_internal2(addr_, internal2_)		\
		SetField( addr_, sgt_Mapping, fldNum_Mapping_internal2, (iNetFLT32) internal2_); if (e) { goto Exit; }

	#define  Set_Mapping_external1(addr_, external1_)		\
		SetField( addr_, sgt_Mapping, fldNum_Mapping_external1, (iNetFLT32) external1_); if (e) { goto Exit; }

	#define  Set_Mapping_external2(addr_, external2_)		\
		SetField( addr_, sgt_Mapping, fldNum_Mapping_external2, (iNetFLT32) external2_); if (e) { goto Exit; }

	#define  Set_Mapping_scale(addr_, scale_)		\
		SetField( addr_, sgt_Mapping, fldNum_Mapping_scale, (iNetFLT32) scale_); if (e) { goto Exit; }

	#define  Set_Mapping_offset(addr_, offset_)		\
		SetField( addr_, sgt_Mapping, fldNum_Mapping_offset, (iNetFLT32) offset_); if (e) { goto Exit; }

	//	----------------------------------------------------------------------
	//	Define the addresses of the channels.
	
	#define 	GAGE1A		1,1,1,1		/*	channel address */
	#define 	GAGE2B		1,1,1,4		/*	channel address */

	//	----------------------------------------------------------------------
	//	Set up the channels (it is recommended that one set these fields in
	//	this order).

										//	set up GAGE1A channel
	Set_SensorType(  	GAGE1A, 	ion_SensorType_StrainGauge)
	Set_WiringType(		GAGE1A, 	ion_Wiring_Q_Bridge_Bend)
	Set_Mapping_scale(	GAGE1A, 	1e6 /* show uStrain instead of Strain */)
	Set_Integration( 	GAGE1A, 	0.0166)
	Set_Constants_Ro(	GAGE1A, 	350.0 /* ohms */)
	Set_Constants_GF(	GAGE1A, 	2.155)		\
	Set_Constants_Vexc(	GAGE1A, 	5.0 /* Volts */)
	Set_Constants_Vinit(GAGE1A, 	0.0 /* Volts */)
	Set_VoltageRange(	GAGE1A, 	0.010 /* Volts */)

										//	set up GAGE1B channel
	Set_SensorType(  	GAGE2B, 	ion_SensorType_StrainGauge)
	Set_WiringType(		GAGE2B, 	ion_Wiring_Q_Bridge_Bend)
	Set_Mapping_scale(	GAGE2B, 	1e6 /* show uStrain instead of Strain */)
	Set_Integration( 	GAGE2B, 	0.0166)
	Set_Constants_Ro(	GAGE2B, 	350.0 /* ohms */)
	Set_Constants_GF(	GAGE2B, 	2.155)		\
	Set_Constants_Vexc(	GAGE2B, 	-5.0 /* Volts */)
	Set_Constants_Vinit(GAGE2B, 	0.0 /* Volts */)
	Set_VoltageRange(	GAGE2B, 	0.010 /* Volts */)

	//	----------------------------------------------------------------------
	//	To set up Thermocouples, Thermistors, and Potentiometers, please
	//	see instruNet Script file "TakeData.iBs".

		//	Thermocouples
		//	Thermistors
		//	Potentiometers

	//	----------------------------------------------------------------------
	//	Read the channels.

Exit:
	if (e)		{						//	if we got an error, show an alert
		Show_ALERT_if_hit_iNet_Error(e);
	}
	
	return e;
}


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	Exercise_The_iNet_Macros()
 *
 *	PURPOSE 	Exercises the macros defined in the adjacent instruNet macro file.  Please
 *				see the macro file for documentation on these macros.
 *				
 *	HISTORY		2/4/96		created - gsw
 * ----------------------------------------------------------------------------------- */
void Exercise_The_iNet_Macros()
{
										//	Careful, if network address {1,1,1,1,1,1}
										//	does not exists (e.g. Model 100 attached
										//	to 1st Network Controller board), some of these
										//	macros will give errors. 
 	iNetUINT8 netNum = 1, devNum = 1, modNum = 1, chNum = 1, i;
 	iNetINT16 settingGroupNum = 1, fieldNum = 1;
 	iNetFLT32 flt32Value;
 	iNetINT32 int32Value;
 	iNetUINT32 networkState_Handle;
 	char cString[300];					
 	iNetStr15 *ptrToStringArray;
 	Boolean thisExists, showAlertStatus, showAlertWas_ON_BeforeThisCall;
	iNetError e;

//	===================== Open instruNet World Window (MACROS) ======================

	e = OPEN_iNet_WINDOW(iNetCM_OpenWindow_Network);

	e = OPEN_iNet_WINDOW(iNetCM_OpenWindow_Record);

	e = OPEN_iNet_WINDOW(iNetCM_OpenWindow_Test);

	e = OPEN_iNet_WINDOW(iNetCM_OpenWindow_SimpleTiming);
	
	
//	============================ Press instruNet Button (MACROS) =========================

	e = PRESS_iNet_BUTTON(iNet_ButtonID_Record_Start);
 
	Stop_iNet_Digitizing();	//	press the STOP button to stop digitizing
 
	e = PRESS_iNet_BUTTON(iNet_ButtonID_Record_Save);

 	e = PRESS_iNet_BUTTON(iNet_ButtonID_Record_Open);
 
	e = PRESS_iNet_BUTTON(iNet_ButtonID_Record_Options);
	
	e = PRESS_iNet_BUTTON(iNet_ButtonID_Record_Setup);

	// e = PRESS_iNet_BUTTON(iNet_ButtonID_Record_Print);

	e = PRESS_iNet_BUTTON(iNet_ButtonID_Record_Timing);
	
	e = PRESS_iNet_BUTTON(iNet_ButtonID_Record_Trigger);
 
 	e = PRESS_iNet_BUTTON(iNet_ButtonID_Record_Probe);
 
 	e = PRESS_iNet_BUTTON(iNet_ButtonID_Record_Directory);

 	e = PRESS_iNet_BUTTON(iNet_ButtonID_Record_CalOptions);

	e = PRESS_iNet_BUTTON(iNet_ButtonID_Network_Store);

	e = PRESS_iNet_BUTTON(iNet_ButtonID_Network_Restore);
 
	e = PRESS_iNet_BUTTON(iNet_ButtonID_Network_Save);

	e = PRESS_iNet_BUTTON(iNet_ButtonID_Network_Open);
 
	e = PRESS_iNet_BUTTON(iNet_ButtonID_Network_Clear);
 
	e = PRESS_iNet_BUTTON(iNet_ButtonID_Network_Reset);

	e = PRESS_iNet_BUTTON(iNet_ButtonID_Network_Recal);
 
	e = PRESS_iNet_BUTTON(iNet_ButtonID_Script_Save);

	e = PRESS_iNet_BUTTON(iNet_ButtonID_Script_SaveAs);

	e = PRESS_iNet_BUTTON(iNet_ButtonID_Script_Open);

	e = PRESS_iNet_BUTTON(iNet_ButtonID_Script_Clear);

	// e = PRESS_iNet_BUTTON(iNet_ButtonID_Script_Print);

	e = PRESS_iNet_BUTTON(iNet_ButtonID_Test_Search);

	e = PRESS_iNet_BUTTON(iNet_ButtonID_Test_Test);

	e = PRESS_iNet_BUTTON(iNet_ButtonID_Test_BigTest);
 
	e = PRESS_iNet_BUTTON(iNet_ButtonID_Test_SaveAs);

	e = PRESS_iNet_BUTTON(iNet_ButtonID_Test_Open);

	e = PRESS_iNet_BUTTON(iNet_ButtonID_Test_Clear);

	e = PRESS_iNet_BUTTON(iNet_ButtonID_Test_Report);

	// e = PRESS_iNet_BUTTON(iNet_ButtonID_Test_Print);

//	============================= ERROR CONTROL (MACROS) =============================

	showAlertWas_ON_BeforeThisCall = (Boolean) DONT_SHOW_MESSAGE_DIALOG_UPON_ERROR();
	
	showAlertWas_ON_BeforeThisCall = (Boolean) SHOW_MESSAGE_DIALOG_UPON_ERROR();

	showAlertStatus = (Boolean) GET_SHOW_MESSAGE_UPON_ERROR_STATUS();
	
	SET_ALL_ALERTS_ON_OFF(0);			//	turn off all alerts
	SET_ALL_ALERTS_ON_OFF(1);			//	turn on all alerts
	

//	========================= INTERROGATE NETWORK MACROS ===============================

	int32Value = GET_NUMBER_OF_NETWORKS();

	int32Value = GET_NUMBER_OF_DEVICES(netNum);

	int32Value = GET_NUMBER_OF_MODULES(netNum,devNum);
										
	int32Value = GET_NUMBER_OF_CHANNELS(netNum,devNum,modNum);
	
	int32Value = GET_NUMBER_OF_SETTINGS(netNum,devNum,modNum,chNum);

	int32Value = GET_NUMBER_OF_FIELDS(netNum,devNum,modNum,chNum,settingGroupNum);


//	=================== DETERMINE IF NETWORK ELEMENT EXISTS (MACROS) =====================

	thisExists = (Boolean) THIS_NETWORK_EXISTS(netNum);

	thisExists = (Boolean) THIS_DEVICE_EXISTS(netNum,devNum);

	thisExists = (Boolean) THIS_MODULE_EXISTS(netNum,devNum,modNum);
	
	thisExists = (Boolean) THIS_CHANNEL_EXISTS(netNum,devNum,modNum,chNum);

	thisExists = (Boolean) THIS_SETTING_EXISTS(netNum,devNum,modNum,chNum,settingGroupNum);

	thisExists = (Boolean) THIS_FIELD_EXISTS(netNum,devNum,modNum,chNum,settingGroupNum,fieldNum);


//	======================== Internal Memory Control (MACROS) ======================

	LOCK_INTERNAL_INET_MEMORY();

	UNLOCK_INTERNAL_INET_MEMORY();


//	============== Turn Channels on and off for digitizing (MACROS) ============

	e = ENABLE_CHANNEL_DIGITIZATION(netNum, devNum, modNum, chNum);
	
	e = DISABLE_CHANNEL_DIGITIZATION(netNum, devNum, modNum, chNum);
	
	DISABLE_ALL_CHANNEL_DIGITIZING(netNum);

	
//	========================== TIMING (MACROS) =================================

	flt32Value = (float) GET_SAMPLE_RATE_IN_HZ(netNum);
	
	flt32Value *= (float) 0.5;
	
	e = SET_SAMPLE_RATE_IN_HZ(netNum, flt32Value);

	int32Value = GET_POINTS_PER_SCAN(netNum);
	
	int32Value = int32Value / 2;

	e = SET_POINTS_PER_SCAN(netNum, int32Value);
	
	int32Value = GET_NUMBER_OF_SCANS(netNum);
	
	int32Value = int32Value / 2;

	e = SET_NUMBER_OF_SCANS(netNum, int32Value);

	int32Value = GET_SCAN_MODE(netNum);
	
	e = SET_SCAN_MODE(netNum, int32Value);
		
	int32Value = GET_NETWORK_BPS(netNum);
	
	e = SET_NETWORK_BPS(netNum, int32Value);
	
	int32Value = GET_SWITCHING(netNum);
	
	e = SET_SWITCHING(netNum, int32Value);

	int32Value = CONTROLLER_IS_DIGITIZING(netNum);
	
	int32Value = GET_DIGITIZE_STATUS(netNum);
	
	e = SET_DIGITIZE_INTO(ion_gSaveDataPopup_ToRamBuffer);
	
	e = SET_OVERFLOW_ALERT(ion_gOnOffPopup_Off);
	
	int32Value = GET_DEVICE_SIMULATION_STATE( netNum, devNum );

//	===================================== TRIGGER (MACROS) ==================================

	e = SET_TRIGGER_MODE(netNum, ion_TriggerMode_Off);
 
 	e = SET_TRIGGER_THRESHOLD(netNum, (float) 1.0 /* threshold EU */);

	e = SET_TRIGGER_SLOPE(netNum, ion_Rising);
	
	e = SET_TRIGGER_HYSTERESIS(netNum, 0.0);

	e = SET_TRIGGER_SOURCE(netNum, fldNum_Trigger_srcNet, netNum);
	e = SET_TRIGGER_SOURCE(netNum, fldNum_Trigger_srcModule, modNum);
	e = SET_TRIGGER_SOURCE(netNum, fldNum_Trigger_srcDevice, devNum);
	e = SET_TRIGGER_SOURCE(netNum, fldNum_Trigger_srcChannel, chNum);
 
 		
//	========================== USER INTERFACE (MACROS) ============================

	int32Value = GET_USER_INTERFACE_TYPE(netNum,devNum,modNum,chNum,settingGroupNum,fieldNum);
	
	int32Value = GET_USER_INTERFACE_CATAGORY(netNum,devNum,modNum,chNum,settingGroupNum,fieldNum);
	
	int32Value = GET_NATIVE_STORAGE_TYPE(netNum,devNum,modNum,chNum,settingGroupNum,fieldNum);
	
	int32Value = GET_CHANNEL_DIRECTION(netNum,devNum,modNum,chNum);

	int32Value = GET_CHANNEL_TYPE(netNum,devNum,modNum,chNum);

	int32Value = GET_CHANNEL_CATEGORY(netNum,devNum,modNum,chNum);

	int32Value = GET_CHANNEL_DIGITIZEABLE(netNum,devNum,modNum,chNum);

	int32Value = GET_NUMBER_OF_POPUP_ITEMS(netNum, deviceNum_CONTROLLER /* 0 */, moduleNum_1stModule /* 1 */,
		controller_ChanNum_C12_Digitizer /* 12 */, sgt_Timing /* -13 */, fldNum_Timing_scanMode /* 4 */);

	* (iNetUINT32 *) &ptrToStringArray = GET_PTR_TO_POPUP_ITEMS( netNum, deviceNum_CONTROLLER /* 0 */,
		moduleNum_1stModule /* 1 */, controller_ChanNum_C12_Digitizer /* 12 */, sgt_Timing /* -13 */,
		fldNum_Timing_scanMode /* 4 */);

	//	view Str15 C string array ptrToStringArray[0],  ptrToStringArray[1], etc in debugger
	
	* (iNetUINT32 *) &ptrToStringArray = GET_PTR_TO_ADDRESS_NAMES( netNum, deviceNum_CONTROLLER /* 0 */,
		moduleNum_1stModule /* 1 */, controller_ChanNum_C12_Digitizer /* 12 */, sgt_Timing /* -13 */,
		fldNum_Timing_scanMode /* 4 */);
		
	//	view Str15 C string array ptrToStringArray[0],  ptrToStringArray[1], etc in debugger


//	====================== GET and SET CHANNEL VALUES (MACROS) ======================
	
	flt32Value = (float) GET_CHANNEL_VALUE_FLT32(netNum,devNum,modNum,chNum);

	int32Value = GET_CHANNEL_VALUE_INT32(netNum,devNum,modNum,chNum);

	e = GET_CHANNEL_VALUE_C_STR255(netNum,devNum,modNum,chNum,&cString[0]);

	e = (iNetError) SET_CHANNEL_VALUE_FLT32(netNum_DRIVER, deviceNum_CONTROLLER, moduleNum_1stModule, driver_ChanNum_RecordOptions,flt32Value);

	e = SET_CHANNEL_VALUE_INT32(netNum_DRIVER, deviceNum_CONTROLLER, moduleNum_1stModule, driver_ChanNum_RecordOptions,int32Value);

	e = SET_CHANNEL_VALUE_C_STR255(netNum_DRIVER, deviceNum_CONTROLLER, moduleNum_1stModule, driver_ChanNum_RecordOptions,&cString[0]);


//	======================== GET and SET NETWORK STATE (MACROS) =====================

	networkState_Handle = GET_HANDLE_TO_NETWORK_STATE();
							
	SET_NETWORK_STATE_WITH_HANDLE(networkState_Handle);
		
	iNet_Dispose_Handle((char *) networkState_Handle);


//	======================= GET and SET NETWORK STATE (FUNCTIONS) ===================
{
	#define ARRAY_BYTES 8192

	iNetINT16 dataArray[ (ARRAY_BYTES >> 1) ];
	iNetUINT32 numBytesOfData;

	e = Get_instruNet_Network_State(&dataArray[0], ARRAY_BYTES, &numBytesOfData);

	e = Set_instruNet_Network_State(&dataArray[0], numBytesOfData);
}

//	========================== DRIVER MANAGEMENT (MACROS) ===========================

	int32Value = DRIVER_IS_INSTALLED();

	flt32Value = SECONDS_SINCE_RESET(netNum);

//	========================== SHOW PROBE DIALOG (MACROS) ===========================

	e = SHOW_PROBE_DIALOG(netNum,devNum,modNum,chNum,settingGroupNum);
		int32Value = netNum_PARAM(gRequestP);
		int32Value = deviceNum_PARAM(gRequestP);
	
	e = SHOW_CHANNEL_ADDRESS_DIALOG(netNum,devNum,modNum,chNum);
		int32Value = netNum_PARAM(gRequestP);
		int32Value = deviceNum_PARAM(gRequestP);
		int32Value = moduleNum_PARAM(gRequestP);
		int32Value = chanNum_PARAM(gRequestP);
	
	e = SHOW_FIELD_ADDRESS_DIALOG(netNum,devNum,modNum,chNum,settingGroupNum,fieldNum);
		int32Value = netNum_PARAM(gRequestP);
		int32Value = deviceNum_PARAM(gRequestP);
		int32Value = moduleNum_PARAM(gRequestP);
		int32Value = chanNum_PARAM(gRequestP);
		int32Value = settingGroupNum_PARAM(gRequestP);
		int32Value = settingGroupType_PARAM(gRequestP);
		int32Value = fieldNum_PARAM(gRequestP);
											//	2/18/97
	e = SHOW_PROBE_DIALOG_WITH_FIXED_CHANNEL(netNum,devNum,modNum,chNum,settingGroupNum,fieldNum);
		int32Value = settingGroupNum_PARAM(gRequestP);
		int32Value = settingGroupType_PARAM(gRequestP);	
											//	2/18/97
	e = SHOW_PROBE_DIALOG_WITH_FIXED_CHANNEL_AND_GROUP(netNum,devNum,modNum,chNum,settingGroupNum,fieldNum);

//	========================== PRINT 10 CONSECUTIVE CHANNELS ===========================

	netNum=1; devNum=1; modNum=1; chNum=1;
	for (i = 0; i < 10; i++)	{			//	print 10 channels
	
											//	select next channel address
		e = SelectNextChannelAddress(&netNum, &devNum, &modNum, &chNum, dir_hdwr_input_circBuf);
											//	print this channel
		e = PrintOneField_iNet((iNetUINT8) netNum, (iNetUINT8) devNum,
				(iNetUINT8) modNum, (iNetUINT8) chNum, sgt_General,
				fldNum_General_valueEu, &cString[0]);
	}
}


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	AskQuestion()
 *
 *	PURPOSE 	This is a wrapper to promt the user for input.  OS dependencies
 *				for this activity are buried here.  The response is captured in 
 *				the global gAnswer_cStr.  Note: There is NO check to see
 *				if we overrun the buffer in this function.
 *				
 *	HISTORY		1/22/96	created - kag
 * ----------------------------------------------------------------------------------- */
void AskQuestion( char * question_cStrPtr )
{
#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	============= Macintosh ============= */

	Print_Line ( question_cStrPtr );	//	prompt user
										
	scanf( "%s", gAnswer_cStr );		//  read input

#elif (MACHINE_TYPE_ == WINTEL_iNet)	/*	=========== Windows/Intel =========== */
										
	Print_Line ( question_cStrPtr );	//	prompt user

	Print_Line ((char *) "\r\n");		//	go to new line so I know where to find the user's input
	
	gIsCustomEditReturnHit = FALSE;		//  wait until the user hits the RETURN key
	while( !gIsCustomEditReturnHit ) {
		Process_OS_Events();
	}
										// get and return the response
	gUserInput_cStrPtr = MsWin_GetUserInputFromLastLineOfEditControl();
	strcpy( gAnswer_cStr, gUserInput_cStrPtr );

										// clean up
	GlobalFreePtr( gUserInput_cStrPtr );
	gUserInput_cStrPtr = 0;

#endif
}


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	Setup_Window_For_Printing()
 *
 *	PURPOSE 	This will open a window for printing.
 *				
 *	HISTORY		1/1/96	created - gsw
 *				11/95	Added Windows code - kag
 *              1/22/96 kag -  Hit this fucntion body hard
 * ----------------------------------------------------------------------------------- */
void Setup_Window_For_Printing()
{
#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	============= Macintosh ============= */

										
	#if THINKC_68K_MAC_COMPILER			/*	if we are running with 68K Think C... */

   	 	csetmode (C_CBREAK, stdin);		/*	tell it to open the console C text window. */
										/*	TRUE if the ansi console window has been opened */
		Print_Line ((char *) "\r\n");	//	advance 1 line (write to console window)
		gConsoleWindowHasBeenOpened = TRUE;

	#elif SYMANTEC_PPC_MAC_COMPILER     /*	else if we are running with ppc Think C... */
	
		cshow(stdout);					/*	show the console window */
  	 	csetmode (C_CBREAK, stdin);		/*	tell it to open the console C text window. */
		Print_Line ((char *) "\r\n");	//	advance 1 line (write to console window)
										/*	I once need to install sys 7.5.3 Upd 2 to get this
										 *	to run on a ppc 7100, gsw 8/96. */
		gConsoleWindowHasBeenOpened = TRUE;

   	#elif defined(__MWERKS__)			/*	else if we are running with Metrowerks CodeWarrior C... */

		short e;
		e = InstallConsole( 0 );		/*	open console window */
		Print_Line ((char *) "\r\n");	//	advance 1 line (write to console window)
										/*	I once need to install sys 7.5.3 Upd 2 to get this
										 *	to run on a ppc 7100, gsw 8/96. */
		gConsoleWindowHasBeenOpened = TRUE;

	#endif
	
#elif (MACHINE_TYPE_ == WINTEL_iNet)	/*	=========== Windows/Intel =========== */

	RECT clientRect;

	if (!ghPrevInstance)	{
   
	    WNDCLASS wc;
		memset( &wc, 0, sizeof( WNDCLASS ) );
	    
	    wc.style         = 0;
	    wc.lpfnWndProc   = (WNDPROC)MsWin_DemoWndProc;
	    wc.cbClsExtra    = 0;
	    wc.cbWndExtra    = 0;
	    wc.hInstance     = ghInstance;
	    wc.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
	    wc.hCursor       = LoadCursor (NULL, IDC_ARROW);
	    wc.hbrBackground = GetStockObject (WHITE_BRUSH);
	    wc.lpszMenuName  = (LPCTSTR) NULL;
	    wc.lpszClassName = (LPCTSTR) MAIN_WND_CLASS_NAME;

	    if (!RegisterClass (&wc)) {
	      
	    	MessageBox (NULL, (LPCTSTR) "WinMain(): RegisterClass() failed",
	                  (LPCTSTR) "Err! - IONC_EX1", MB_OK | MB_ICONEXCLAMATION);	
		}
	}

	ghwndMain = CreateWindow (MAIN_WND_CLASS_NAME, gAppTitle,
          WS_OVERLAPPEDWINDOW,
          CW_USEDEFAULT, CW_USEDEFAULT,
          CW_USEDEFAULT, CW_USEDEFAULT,
          NULL, NULL, ghInstance, NULL);

	if (ghwndMain != (void *) 0L)	
	{
			GetClientRect( ghwndMain, &clientRect /* 0,0 is main window upper left corner */);

			ghwndEditWindow = CreateWindow(
				"EDIT",
				NULL,
				WS_CHILD | WS_VISIBLE | ES_LEFT | ES_MULTILINE | WS_VSCROLL | WS_HSCROLL , 
				clientRect.left, 
				clientRect.top,
				clientRect.right - clientRect.left, 
				clientRect.bottom - clientRect.top,
				ghwndMain,
				(HMENU)0,  // child window id (don't let HMENU fool you)
				ghInstance,
				NULL );
			
			gDefaultEditProcedure = SubclassWindow(ghwndEditWindow, MsWin_CustomEditProcdure); 
	
			ShowWindow (ghwndMain, SW_SHOW);
			Process_OS_Events();
	}

#endif
}


#if (MACHINE_TYPE_ == WINTEL_iNet)	/*	=========== Windows/Intel =========== */

/* -------------------------------------------------------------------------------------
 *	FUNCTION  	MsWin_GetUserInputFromLastLineOfEditControl
 *
 *	PURPOSE 	Return a pointer to the last line of text in the edit control.  Presumably
 *				this is the line the user entered in response to a prompt.
 *				
 *				Return:  This returns a pointer to memory allocated here.  The caller must
 *				free it.
 *
 *	HISTORY		1/22/96 created - kag
 * ----------------------------------------------------------------------------------- */
static LPSTR MsWin_GetUserInputFromLastLineOfEditControl()
{
	int totalNumCharacters = Edit_GetTextLength(ghwndEditWindow);
	int responseLineNumber = Edit_LineFromChar( ghwndEditWindow, totalNumCharacters) - 1;
	int lineLength = Edit_LineLength(ghwndEditWindow, responseLineNumber);
	LPSTR pStr = (LPSTR) GlobalAllocPtr(GHND, lineLength + 1);

	if( !pStr ) {
		MessageBox(ghwndMain, "Out of memory while trying to process your response.", gAppTitle, MB_ICONSTOP );
	} else {

		Edit_GetLine(ghwndEditWindow, responseLineNumber, pStr, lineLength);

	}
	return pStr;
}

/* -------------------------------------------------------------------------------------
 *	FUNCTION  	MsWin_CustomEditProcdure
 *
 *	PURPOSE 	Used to change the behavior of the edit control used to simulate a MAC
 *				console window in the main window.
 *
 *	HISTORY		1/22/96 created - kag
 * ----------------------------------------------------------------------------------- */
LRESULT CALLBACK MsWin_CustomEditProcdure( HWND hWnd, WORD message, WPARAM wParam, LPARAM lParam )
{
	switch (message)
	{
		case WM_CHAR:
				// To come as close as possible to using scanf the way it is in the MAC
				// code, let's detect when the user hits the return key (presumably after
				// entering data in response to a prompt
			if( (TCHAR)wParam == VK_RETURN ) {
				gIsCustomEditReturnHit = TRUE;
			}
	}
	
	return (CallWindowProc(gDefaultEditProcedure, hWnd, message, wParam, lParam));
}

/* -------------------------------------------------------------------------------------
 *	FUNCTION  	MsWin_DemoWndProc
 *
 *	PURPOSE 	Main window message handler (Microsoft Windows).
 *				
 *	HISTORY		11/30/95 created - kag
 * ----------------------------------------------------------------------------------- */
LRESULT CALLBACK MsWin_DemoWndProc(HWND hWnd, WORD message, WPARAM wParam, LPARAM lParam )
{
	switch (message)	{
	
		HANDLE_MSG(hWnd, WM_COMMAND, MsWin_OnCommand);
		HANDLE_MSG(hWnd, WM_DESTROY, MsWin_OnDestroy);
		HANDLE_MSG(hWnd, WM_SIZE,    MsWin_OnSize);
			
		default:
			return (DefWindowProc(hWnd, message, wParam, lParam));
	}
	return (0);
}


/* ----------------------------------------------------------------------------
 *  FUNCTION    MsWin_OnDestroy
 *
 *  PURPOSE     Processes the WM_DESTROY message (Microsoft Windows).
 *				Since we are unconventionally simulating a MAC console
 *				window which may be waiting for input, we will force an
 *				exit of the application here.
 *				
 *	HISTORY		11/30/95 created - kag
 * -------------------------------------------------------------------------- */
static void MsWin_OnDestroy(HWND hWnd)
{
	STOP_COMPILER_WARNING( hWnd );
	
	exit (EXIT_SUCCESS);
}


/* ----------------------------------------------------------------------------
 *  FUNCTION    MsWin_OnCommand
 * 
 *  PURPOSE     Processes WM_COMMAND message (Microsoft Windows).
 *				
 *	HISTORY		11/30/95 created - kag
 * -------------------------------------------------------------------------- */
static void MsWin_OnCommand(HWND hWnd, int id, HWND hwndCtl, UINT codeNotify)
{
	STOP_COMPILER_WARNING(codeNotify);
	STOP_COMPILER_WARNING(hwndCtl);
	STOP_COMPILER_WARNING(hWnd);
	
	switch (id) {
	}
}

/* ----------------------------------------------------------------------------
 *  FUNCTION    MsWin_OnSize
 * 
 *  PURPOSE     Processes WM_SIZE message
 *				
 *	HISTORY		1/22/96 kag - created
 * -------------------------------------------------------------------------- */
static void MsWin_OnSize(HWND hWnd, UINT state, int cx, int cy)
{
	STOP_COMPILER_WARNING(state);
	STOP_COMPILER_WARNING(hWnd);

	//	MoveWindow() causes a WM_PAINT msg to be posted to the
	//	MAIN window to take care of repainting moved window.
	//	(i.e. a PAINT msg follows and ONSIZE msg)

	MoveWindow( ghwndEditWindow, 0, 0, cx, cy, TRUE );
}

#endif	/* if (MACHINE_TYPE_ == WINTEL_iNet) */


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	Print_Line()
 *
 *	PURPOSE 	This is passed a C string ('\0' termination) and prints it to the
 *				console window.
 *				
 *	HISTORY		1/1/96	created - gsw
 *				1/22/96 kag - Added Windows code
 * ----------------------------------------------------------------------------------- */
void Print_Line( char *cStr_lineToPrint)
{										
#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	============= Macintosh ============= */

	#if defined(__MWERKS__)				/*	if on the metrowerks compiler... */	
										/*	(metrowerks doesn't like \r\n) */
		char out_cStr[300];
										//	convert "\r\n" to "\n" 
		Convert_CrLf_To_Lf(cStr_lineToPrint, out_cStr);
										
		printf(out_cStr);				//	print this line
	
	#else
										
		printf(cStr_lineToPrint);		//	print this line

	#endif

#elif (MACHINE_TYPE_ == WINTEL_iNet)	/*	=========== Windows/Intel =========== */

	//	We are appending a line to the edit control.									

	int editTextLength = Edit_GetTextLength(ghwndEditWindow);

	if ( (editTextLength + strlen(cStr_lineToPrint)) > MAX_TE_LENGTH /* 32K on Mac, 30K Win32 */ )	{
		MessageBox(ghwndMain, "You have reached the 30K character limit for this text file.", gAppTitle, MB_ICONSTOP );
		return;
	}

	Edit_SetSel( ghwndEditWindow, editTextLength, editTextLength );

	Edit_ReplaceSel( ghwndEditWindow, cStr_lineToPrint );
	
	//	send message and return after the message has been fielded
	SendMessage(ghwndEditWindow, EM_SCROLLCARET, (WPARAM) 0, (LPARAM) 0);
	
	SetFocus( ghwndEditWindow );									

	Process_OS_Events();
	
#endif
}


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	Convert_CrLf_To_Lf()		
 *
 *	PURPOSE 	Converts "\r\n" to "\n" in C string form.
 *
 *	HISTORY		5/3/94	created - gsw
 * ----------------------------------------------------------------------------------- */
void Convert_CrLf_To_Lf(
	char *in_,							//	input C string 
	char *out_)							//	output C string
{
	register char *in = in_;
	register char *out = out_;
	register char cr = ASCII_CR;
	register char lf = ASCII_LF;
	while (*in != 0x00)	{				//	scan through the input string
										//	if we hit a \r\n...	
		if (*in == cr iNetAnd  *(in+1) == lf)	{
			in++;						//	skip the \r
		}
		*out++ = *in++;					//	otherwise, just copy the string
	}
	*out = 0x00;						//	null terminate
}


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	Process_OS_Events()
 *
 *	PURPOSE 	Process what we have now and move on.
 *				
 *	HISTORY		11/30/95	created - kag
 * ----------------------------------------------------------------------------------- */
void Process_OS_Events()
{										
#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	============= Macintosh ============= */

	/*	not necessary to do anything here */
	
#elif (MACHINE_TYPE_ == WINTEL_iNet)	/*	=========== Windows/Intel =========== */
	
	MSG msg;

	while (PeekMessage(&msg, 0,0,0, PM_NOREMOVE) )
	{
		if( GetMessage(&msg, 0, 0, 0) ) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

#endif
}

/* -------------------------------------------------------------------------------------
 *	FUNCTION  	Process_OS_Events_Until_Close
 *
 *	PURPOSE 	This processes all of the main windows messages until the user
 *				closes the window.
 *				
 *	HISTORY		11/30/95	created - kag
 * ----------------------------------------------------------------------------------- */
void Process_OS_Events_Until_Close()
{										
#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	============= Macintosh ============= */

	/*	not necessary to do anything here */

#elif (MACHINE_TYPE_ == WINTEL_iNet)	/*	=========== Windows/Intel =========== */
	
	MSG msg;

	while (GetMessage(&msg, 0, 0, 0)) {

		TranslateMessage(&msg);

		DispatchMessage(&msg);
	}

#endif
}


//	-------------------------------------------------------
//	Test snippets of code.

#if 0
	e = Load_instruNet_Driver(FALSE /* forceInstruNetReset */, &driverIsInstalledOK, &numNetworks);							
	e = OPEN_iNet_WINDOW(iNetCM_OpenWindow_Network);
	return 1;
#endif

#if 0
	Run_Digitize();
	return 1;
#endif

#if 0
	Open_instruNet_Window_Easy ( iNetCM_OpenWindow_Record );
	Open_instruNet_Window_Easy ( iNetCM_OpenWindow_Network );
	Open_instruNet_Window_Easy ( iNetCM_OpenWindow_Test );
	
	Open_instruNet_Window_Easy ( iNetCM_OpenWindow_FullProbe );
	Open_instruNet_Window_Easy ( iNetCM_OpenWindow_ChanSelection );
	Open_instruNet_Window_Easy ( iNetCM_OpenWindow_FieldSelection );
	Open_instruNet_Window_Easy ( iNetCM_OpenWindow_OneFieldSet );
	Open_instruNet_Window_Easy ( iNetCM_OpenWindow_AllFieldSets );
	
	Open_instruNet_Window_Easy ( iNetCM_OpenWindow_RecTiming );
	Open_instruNet_Window_Easy ( iNetCM_OpenWindow_RecTrigger );
	Open_instruNet_Window_Easy ( iNetCM_OpenWindow_RecOptions );		
	Open_instruNet_Window_Easy ( iNetCM_OpenWindow_CalOptions );		
	Open_instruNet_Window_Easy ( iNetCM_OpenWindow_RecFile );

	Open_instruNet_Window_Easy ( iNetCM_OpenWindow_SimpleTiming );
#endif

#if 0								//	show the probe dialog
	PRESS_iNet_BUTTON(iNet_ButtonID_Record_Probe);
#endif
