/* ************************************************************************************
INET_EX2.C							Companions: "INET_INT.C", "INET_INT.H", "INET_MCS.H"

	------------------------------- A B S T R A C T ----------------------------------
	
	RECOMMENDATION
	
		Try to do your applications first with the BASIC class routines, documented
		in INET_INT.C, and demonstrated in Example #3.  If these are not adequate, 
		then move to the ADVANCED class routines demonstrated in Examples #1 and #2. 
		In other words, ignor the ADVANCED routines until you are sure you need them.

	This file contains C code examples that do I/O with the instruNet hardware.

	This C code is platform, Operating System, and processor independent. This code
	should run on Macintosh, Windows 3.1, and Windows 95 C and C++ compilers without
	any problems. 
	
	This file expects tabs every 4 characters.
	
	The the comments in this file will help you understand how to program instruNet.
	
	This file is typically used with the following files:
		
		INET_EX2.C	- contains example C code that uses the instruNet hardware
		INET_INT.C  - contains low level source code that interfaces to instruNet driver
		INET_INT.H  - contains many low level enums and #defines that support instruNet
		INET_MCS.H  - contains macros that help set/read instruNet fields
	
	The instruNet disk includes three example programs:

		EXAMPLE #1		Shows how to use almost every instruNet macro and subroutine
						in a text window-based environment.  This example is quite lengthy,
						uses many ADVANCED functions, and is based on file INET_EX1.C.

		EXAMPLE #2		Shows an instruNet application that reads and writes
						several instruNet fields, and then digites.  This example is
						based on file INET_EX2.C, and uses many ADVANCED functions.

		EXAMPLE #3		Example #3  reads and writes fields in a simple way,
						and is the easiest of all the example programs. This example
						uses only the BASIC class functions (not ADVANCED), and is
						based on file INET_EX3.c

	-------------------------------- H I S T O R Y -----------------------------------
	HISTORY		Copyright c 1995-96 GWI.  All rights reserved.
	1/27/97		created - gsw

************************************************************************************ */

/* ================================== #includes ==================================== */

//	instruNet includes

	#include "INET_INT.H"				//	instruNet C Interface (for End User)
	#include "INET_MCS.H"				//	instruNet Macros & Functions (for End User)

//	ANSI includes
 
#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	=========== Macintosh ========== */

	#include <stdio.h>					//	for scanf(), printf()	
	#include <stdlib.h>					//	for malloc()	
	#include <console.h>				//	for C_CBREAK


#elif (MACHINE_TYPE_ == WINTEL_iNet) 	/*	========= Windows/Intel ======== */
									
	#include <stdio.h>					//	for scanf(), printf()	
	#include <stdlib.h>					//	for malloc()	
	#include <windows.h>				//  This is included in INOC_INT.H but is 
										//  is needed for inoc_ex1.h
#endif
								
/* ================================== externs ====================================== */

/* ================================== globals ====================================== */

#define  MAX_NUM_DIGITIZE_CHANNELS  64	/*	max number of channels to digitize */

#define WORK_WITH_FEW_PTS 	0			//	make TRUE (1) to work with fewer pts
#define LOOK_FOR_ZERO 		0			//	make TRUE (1) to look for zero's in your data
#define USE_USER_BUFFER 	0			//	make TRUe (1) to use UserRamBuffer (i.e. you
										//	C program's memory) instead of driverRamBuffer
										//	(i.e. memory allocated by instruNet Driver).

/* ================================= #defines ====================================== */

/* ============================= static prototypes ================================= */

iNetError DigitizeFromSelectedChannels(iNetINT16 allowMouseDownToStopDigitize,
	iNetINT16 processOneScanAtATime, iNetINT32 numPointsPerScan,
	iNetFLT32 pointsPerSecondPerChannel, iNetINT16 *numChannelsToDigitize,
	iNetChannelAddr *digitizeChAddr, iNetFLT32 **digitizeDataPP);

/* =================================== C Code  ===================================== */
	
/* -------------------------------------------------------------------------------------
 *	FUNCTION  	main()
 *
 *	PURPOSE 	This is the main entry to an application program that shows an example
 *				of how to monitor and control the instruNet network.
 *				
 *	HISTORY		2/4/96		created - gsw
 * ----------------------------------------------------------------------------------- */
 
#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	============= Macintosh ============= */

int main()								/*	Main entry for a Macintosh application */
{
#elif (MACHINE_TYPE_ == WINTEL_iNet)	/*	=========== Windows/Intel =========== */
										
int PASCAL WinMain (					/*	Main entry for a windows application */
	HINSTANCE hInstance, HINSTANCE hPrevInstance,  LPSTR lpCmdLine, int nCmdShow)
{
#endif
	iNetError e = iNetErr_None;
	iNetINT16 driverIsInstalledOK, numNetworks, numChannelsDigitized;
	Boolean chExists;
	iNetChannelAddr vout, vin;
	iNetFLT32 vin_value, vout_voltage;
	iNetFLT32 *digitizeDataP[ MAX_NUM_DIGITIZE_CHANNELS ];
	iNetINT32 numPointsPerScan;
	iNetFLT32 pointsPerSecondPerChannel;
	iNetINT32 digChNum, pointNum;
	FILE *file;
	char msg[300];

	iNetFLT32 avgValue[ MAX_NUM_DIGITIZE_CHANNELS ];
	iNetFLT32 minValue[ MAX_NUM_DIGITIZE_CHANNELS ];
	iNetFLT32 maxValue[ MAX_NUM_DIGITIZE_CHANNELS ];
	iNetChannelAddr digitizeChAddr[ MAX_NUM_DIGITIZE_CHANNELS ];

	//	-----------------------------------------------------------------------
	//	Load instruNet driver and initialize instruNet network.
		
	e = Load_instruNet_Driver(FALSE /* reset_instruNet */, &driverIsInstalledOK, &numNetworks);							
	if (e != iNetErr_None) { Show_ALERT_if_hit_iNet_Error(e); return 0; }
	
	Show_Simple_Alert("We just loaded the instruNet driver successfully via Load_instruNet_Driver().");

	//	-----------------------------------------------------------------------
	//	Store/Restore channel settings.
	
	//	store channel settings disk
	e = PRESS_iNet_BUTTON( iNetCM_BtnPress_Network_Store );
	if (e) { goto Exit; }

	//	Restore the instruNet settings to where they were when the user	
	//	last pressed the STORE button in the NETWORK page of instruNet World.
	//	This recalls ALL settings in the NETWORK page (e.g. integration time,
	//	sensor type, voltage range, etc).

	e = PRESS_iNet_BUTTON( iNetCM_BtnPress_Network_Restore );
	if (e) { goto Exit; }
		
	Show_Simple_Alert("We just restored all the channel settings that were previously stored via the Store button in the Network window.");

	//	-----------------------------------------------------------------------
	//	Load address of 'Ch1 Vin+' and 'Ch3 Vout' into 'vin' and 'vout' structs
	//	(this assumes a instruNet 100 is the 1st device on the net).

										//	set Ch1 Vin+ address
	chExists = (Boolean) LoadChannelAddress( &vin, 1 /* net */, 1 /* device */, 1 /* module */, 1 /* chan */);
	if (!chExists)	{ goto Exit; }
	
										//	set Ch3 Vout address
	chExists = (Boolean) LoadChannelAddress( &vout, 1 /* net */, 1 /* device */, 1 /* module */, 3 /* chan */);
	if (!chExists)	{ goto Exit; }
	
	//	-----------------------------------------------------------------------
	//	Read 'Ch1 Vin+' value.
										//	read Ch1 Vin+ value
	vin_value = GetField_flt32( &vin, sgt_General, fldNum_General_valueEu, &e);
	if (e) { goto Exit; }

	Show_Simple_Alert("We just read Channel Ch1 Vin's value.");

	//	-----------------------------------------------------------------------
	//	Set 'Ch3 Vout' voltage.

	vout_voltage = (iNetFLT32) 2.0;		//	set Ch3 Vout to 2.0Volts.
	e = SetField_flt32( &vout, sgt_General, fldNum_General_valueEu, vout_voltage);
	if (e) { goto Exit; }
		
	Show_Simple_Alert("We just set Channel Ch3 Vout to 2Volts.");

	//	-----------------------------------------------------------------------
	//	Digitize from several channels (we digitize 100 pts at 100pts/sec/channel).
	
	Show_Simple_Alert("We will now digitize 100pts at 100pts/sec/ch from several channels.");

	numPointsPerScan = 100L;			//	Digitize 100 point scans at a 
										//	100 pts/sec/channel sample rate.
	pointsPerSecondPerChannel = (iNetFLT32) 100.0;	

									#if WORK_WITH_FEW_PTS
									numPointsPerScan = 10L;
									pointsPerSecondPerChannel = (iNetFLT32) 5.0;	
									#endif

	e = DigitizeFromSelectedChannels(TRUE /* allowMouseDownToStopDigitize */,
							 	     TRUE /* processOneScanAtATime */,
									 numPointsPerScan,
									 pointsPerSecondPerChannel /* sampleRate */,
									 &numChannelsDigitized /* loaded with # of digitized channels */,
									 &digitizeChAddr[0] /* loaded with address of digitized channels */,
									 &digitizeDataP[0] /* loaded with pointers to digitized data */);
	if (e) { goto Exit; }
									#if LOOK_FOR_ZERO && !USE_USER_BUFFER
									//	look for a zero value in the data
									for (digChNum = 0; digChNum < numChannelsDigitized; digChNum++)	{
										iNetFLT32 *bufferP = digitizeDataP[ digChNum ];
										for (pointNum = 0; pointNum < numPointsPerScan; pointNum++)	{
											if (bufferP[pointNum] == 0.0) { Show_Simple_Alert((char *)"hey"); }
										}
									}
									#endif

	//	-----------------------------------------------------------------------
	//	Calculate average/min/max values digitized from each channel.

										//	calc sum of points for each digitized channel
	for (digChNum = 0; digChNum < numChannelsDigitized; digChNum++)	{

		register iNetFLT32 accum = (iNetFLT32) 0.0;
		register iNetFLT32 data;
		register iNetFLT32 min = (iNetFLT32) 1.0e30;
		register iNetFLT32 max = (iNetFLT32) -1.0e30;
		register iNetFLT32 *bufferP = digitizeDataP[ digChNum ];

										//	scan through every point and sum them
		for (pointNum = 0; pointNum < numPointsPerScan; pointNum++)	{

			data = *bufferP;			//	get data value from buffer
			bufferP++;					//	go to next point in buffer
			accum += data;				//	sum points
			if (data < min) { min = data; }	//	calculate minimum
			if (data > max) { max = data; }	//	calculate maximum
		}
										//	calc average
		avgValue[ digChNum ] = (iNetFLT32) ((double) accum / (double) numPointsPerScan);
		minValue[ digChNum ] = min;
		maxValue[ digChNum ] = max;
	}
					
	//	-----------------------------------------------------------------------
	//	Save data to disk in a text file.
										
	file = fopen( "DataFile", "w");		//	create new text file with name "DataFile"
										//	put header into text file
	fprintf(file, "Data stored in text file 'DataFile'%s", gCr);
										//	put vin and vout values into text file
	fprintf(file, "Vin = %f, Vout = %f%s", vin_value, vout_voltage, gCr);
										//	print table header
	fprintf(file, "DigCh\tAvg\tMin\tMax%s", gCr);
										//	scan through each digitized channel
	for (digChNum = 0; digChNum < numChannelsDigitized; digChNum++)	{
										//	print avg/min/max value for each channel
		sprintf(msg, "%d\t%f\t%f\t%f%s", ((int) /* %d wants int */ digChNum),
				((float) avgValue[ digChNum ]), ((float) minValue[ digChNum ]),
				((float) maxValue[ digChNum ]), gCr);
		fprintf(file, msg);
	}
	fclose( file );						//	close file

										//	show an alert
	Show_Simple_Alert("We just saved data in file 'DataFile' and are done.");

	//	-----------------------------------------------------------------------
	//	Exit.

Exit:

	Show_Simple_Alert("We are done with Example #2.  For a simpler example, please see Example#3.");

	if (e)		{						//	if we got an error, show an alert,
		Show_ALERT_if_hit_iNet_Error(e);
	}

	CloseDriverAndReleaseDriverRam();	//	Closes the instruNet driver and 
										//	releases the memory used to hold it

#if (MACHINE_TYPE_ == WINTEL_iNet)		/*	=========== Windows/Intel =========== */
	STOP_COMPILER_WARNING(hInstance);
	STOP_COMPILER_WARNING(hPrevInstance);
	STOP_COMPILER_WARNING(lpCmdLine);
	STOP_COMPILER_WARNING(nCmdShow);
#endif

	return e;
}



/* -------------------------------------------------------------------------------------
 *	FUNCTION  	DigitizeFromSelectedChannels()
 *
 *	PURPOSE 	This routine simultaneously digitizes several channels from an
 *				instruNet network.
 *				
 *	HISTORY		2/2/96		created - gsw
 * ----------------------------------------------------------------------------------- */

iNetError DigitizeFromSelectedChannels(
	iNetINT16 allowMouseDownToStopDigitize,
										//	if TRUE (1), we allow a mouse down to stop the
										//	digitize process; otherwise, if FALSE (0), will
										//	will run till finished.
	iNetINT16 processOneScanAtATime,	//	if TRUE (1), we access digitized data 1 complete
										//	Scan at a time; otherwise if FALSE (0), we process
										//	little segments of data, as they become available.
	iNetINT32 numPointsPerScan,			//	points per scan
	iNetFLT32 pointsPerSecondPerChannel,//	sample rate for each channel
	iNetINT16 *numChannelsDigitizedP,	//	filled with # of channels that are digitized
	iNetChannelAddr *digitizeChAddr,	//	we load this with addresses of digitized channels
	iNetFLT32 **digitizeDataPP)			//	we load this array with ptrs to the scan buffers
										//	used for each channel
{
	iNetError e;
	iNetINT16 numChannelsToDigitize;	//	# of channels to digitize
	iNetINT16 theNextDataAccessIsTheLast_ofEntireDigitization;
	iNetFLT32 *bufferStartAddrP;
	iNetUINT32 pointToPullindex_base1, numPointsToPull, scanNumberToPull_base1;
	iNetFLT32 *firstPoint, *lastPoint;
	iNetINT16 theNextDataAccessWillPullToTheEndOfAScan, showAlertWas_ON_BeforeThisCall;
	iNetUINT32 last_scanNum_base1_pulled_inFull, last_scanNum_base1_pushed_inFull;
	iNetINT16 controllerIsFinishedWithDigitization, digChNum, weAreDoingOurFirstAccessOfData;
	Boolean channelExists;
	Boolean useDriverRamBuffer = TRUE;
	iNetChannelAddr triggerChannel;
										//	used to keep track of digitizing
	iNetUINT32 totalPtsPulledSoFar[ MAX_NUM_DIGITIZE_CHANNELS ];
	iNetFLT32 sumOfPoints[ MAX_NUM_DIGITIZE_CHANNELS ];
	
	Boolean calcSumOfPoints;

	//	-----------------------------------------------------------------------
	//	Specify 3 channels to digitize from.
	
	numChannelsToDigitize = 0;			//	count # of channels we are digitizing

										//	Ch1 Vin+ = 0th channel to digitize
	channelExists = (Boolean) LoadChannelAddress( &digitizeChAddr [ 0 ],
		1 /* net */, 1 /* device */, 1 /* module */, 1 /* chan */);
	if (!channelExists)	{ goto Exit; }
	numChannelsToDigitize++;			//	count # of channels we are digitizing

										//	Ch2 Vin- = 1st channel to digitize
	channelExists = (Boolean) LoadChannelAddress( &digitizeChAddr [ 1 ],
		1 /* net */, 1 /* device */, 1 /* module */, 2 /* chan */);
	if (!channelExists)	{ goto Exit; }
	numChannelsToDigitize++;			//	count # of channels we are digitizing

										//	Ch4 Vin+ = 2nd channel to digitize
	channelExists = (Boolean) LoadChannelAddress( &digitizeChAddr [ 2 ],
		1 /* net */, 1 /* device */, 1 /* module */, 4 /* chan */);
	if (!channelExists)	{ goto Exit; }
	numChannelsToDigitize++;			//	count # of channels we are digitizing
		
										//	make sure we are not digitizing too many channels
	if (numChannelsToDigitize > MAX_NUM_DIGITIZE_CHANNELS)	{
		goto Exit;
	}
										#if WORK_WITH_FEW_PTS
											//	force it to digitize only from 1 channel
											numChannelsToDigitize = 1;
										#endif

	if (numChannelsDigitizedP != 0L) {	//	tell caller how many channels we are digitizing
		*numChannelsDigitizedP = numChannelsToDigitize;	
	}


	//	-----------------------------------------------------------------------
	//	Enabled digitizing for these channels.

										//	disable all channels for digitizing (initially)
	e = DisableAllChannelsForDigitizing();	
	if (e)	{ goto Exit; }
										//	scan thru all digitize channels
	for (digChNum = 0; digChNum < numChannelsToDigitize; digChNum++)	{

										#if USE_USER_BUFFER
										//	use UserRamBuffer instead of DriverRamBuffer
										float *p = (float *) malloc(numPointsPerScan * 4);
										e = SetField_int32(&digitizeChAddr [ digChNum ], sgt_UserRamBuffer, fldNum_UserBuffer_UserBufferAddr, (iNetINT32) p);
										e = SetField_int32(&digitizeChAddr [ digChNum ], sgt_UserRamBuffer, fldNum_UserBuffer_UserPtrSizeInBytes, numPointsPerScan * 4);
										e = SetField_int32(&digitizeChAddr [ digChNum ], sgt_UserRamBuffer, fldNum_UserBuffer_DigitizeOnOff, ion_gOnOffPopup_On);
										useDriverRamBuffer = FALSE;
										#else
										//	enabled digitizing for this channel
		e = EnableChannelForDigitizing(&digitizeChAddr [ digChNum ]);
		if (e)	{ goto Exit; };
										#endif
	}

	//	-----------------------------------------------------------------------
	//	Set TIMING parameters.  Tell it to digitize 'numPointsPerScan' points
	//	per Scan at 'pointsPerSecondPerChannel' sample rate and do this
	//	1 time (i.e. 1 Scans).  We also tell iNet to put the digitized data
	//	into the DRIVER ram buffer (we will get access to this buffer as the
	//	data is digitized).  And we tell iNet to show an alert if we
	//	overflow a data buffer.  StripChartRecorder mode is turned OFF, which
	//	means that the Scans will be separate (i.e. Oscillscope mode).
	
	e = Set_iNet_TIMING_Parameters(
			pointsPerSecondPerChannel, 	//	samplesPerSecond
			numPointsPerScan,			//	pointsPerScan per channel
			1,							//	numberOfScans
			FALSE,						//	false = oscilloscope mode
			ion_gSaveDataPopup_ToRamBuffer,	//	saveDataOption (save to ram buffer)
			TRUE, 						//	showAlertOnOverflow
			TRUE);						//	switch channels accurately
	if (e)	{ goto Exit;}				//	if we got an error, bail out

	//	-----------------------------------------------------------------------
	//	Set TRIGGER parameters.  We specify the trigger source to be the channel
	//	specified in the 'triggerChannel' channel address.  We also say that the
	//	channel is set to trigger on the rising edge of the source when it
	//	crosses 0.0.  At the moment, the trigger function is deactivated
	//	(i.e. ion_TriggerMode_Off).

										//	Ch1 Vin+ = trigger channel
	channelExists = (Boolean) LoadChannelAddress( &triggerChannel,
		1 /* net */, 1 /* device */, 1 /* module */, 1 /* chan */);
	if (!channelExists)	{ goto Exit; }

	e = Set_iNet_TRIGGER_Parameters(
			ion_TriggerMode_Off,		//	triggerMode
			(float) 0.0,				//	triggerThreshold	
			ion_Rising,					//	triggerSlope
			(float) 0.0,				//	preTriggerSeconds
			&triggerChannel);			//	trigger source channel
	if (e)	{ goto Exit;}				//	if we got an error, bail out
	
	//	-----------------------------------------------------------------------
	//	Tell instruNet driver to show an alert if there is an error (in addition
	//	to returning an error code).
										//	Tell instruNet driver to show an alert
										//	(in addition to returning an error code)
										//	if it gets into trouble
	showAlertWas_ON_BeforeThisCall = (short) SHOW_MESSAGE_DIALOG_UPON_ERROR();

	//	-----------------------------------------------------------------------
	//	Initialize variables for mathematical processing of the digitized 
	//	waveforms.

										//	initialize parameters
	for (digChNum = 0; digChNum < numChannelsToDigitize; digChNum++)	{
		totalPtsPulledSoFar[digChNum] = 0L;
		sumOfPoints[digChNum] = (iNetFLT32) 0.0;
	}

	//	-----------------------------------------------------------------------
	//	Tell instruNet to digitize, and process the segments (or Scans) of data
	//	as they become available.

	//	The data is stored in buffers, one Scan large, in memory that is
	//	managed by the instruNet driver.  The Access_Digitized_Data_In_Ram_Buffer()
	//	routine provides you with the address of this memory buffer, so that
	//	you can access the data.  In fact, if 'processOneScanAtATime' is set to
	//	FALSE (0), it will give you the address within the buffer of little chunks
	//	of data as they become available, so that you can little pieces as they
	//	are acquired.

	//	If the "Save Data" field (i.e. 'saveDataOption') in the "Record Options"
	//	settings group is set to "To Ram Buffer", as is done in this example, then
	//	the digitized data is automatically sent to ram buffers maintained by the
	//	instruNet driver; otherwise, if "Save Data" is set "To File", then the data 
	//	is spooled to disk.
	
	//	If you wanted to digitize 1 scan worth of data, and then save it to
	//	disk, set the "numberOfScans" paramater to 1, tell it to begin
	//	recording, and then when it is done recording, press the SAVE button
	//	in the record page via "PRESS_iNet_BUTTON(iNetCM_BtnPress_Record_Save)"
	//	to save the 1 scan to disk.
	
	//	If you wanted to record forever yet don't have an infinite amount of RAM,
	//	set the "numberOfScans" paramater to a really huge number, and then process
	//	the data, 1 scan at a time, in the RAM buffer maintained by the driver (which
	//	is continually overwritten).
	
	//	If you wanted to record for a long time and spool to disk while recording,
	//	set the "Save Data" field to "To File" and tell it to start digitizing.
	//	You could then scroll through the spooled data, post acquisition,
	//	in the instruNet World window.
	
	//	If you wanted the data to be sent to your own ram buffer (i.e. you create
	//	the buffer and maintain it) then:
	//
	//	a)	Set the "Save Data" field to "User Control".
	//	b)	Set the USER BUFFER Setting Groups fields accordingly:
	//		- Set the "Digitize" field to "On" to enable the channel for digitizing.
	//	    - Set the "User Addr" field to the address of your RAM buffer, which must
	//		  be able to hold 1 Scan's worth of FLT32 data (i.e. numberOfScans * 4 bytes).
	//		- Set the "Buff Byte" field to the size of your RAM buffer (which must be
	//		  at least (numberOfScans * 4) bytes long.
	//	c)	Start recording, and instead of being handed an address in the instruNet
	//		Driver's ram buffer by Access_Digitized_Data_In_Ram_Buffer( 'driverRamBuffer'
	//		set FALSE), you will be handed an address in your own buffer (so that you
	//		can monitor the data as it is being digitized).
	
	//	Keep in mind that data is overwritten in a RAM buffer if you digitize
	//	multiple scans.  For example, if 'numberOfScans' is set to 2, then the
	//	RAM buffer will be written to twice, and your 1st scan if data will be overwritten.
	//	If you can process the data before it is overwritten (e.g. calculated averages and
	//	spike positions, etc) that is very cool; otherwise, you might need to save it in
	//	some way.  For example, you could save it to disk, or you could save it to
	//	another RAM location.  If this is difficult, then please consider increasing
	//	the 'pointsPerScan' and setting the 'numberOfScans' parameter to 1, so that the 
	//	scan buffer is not overwritten.
	
	//	DATA ACQUISITION IS MUCH EASIER WHEN 'numberOfScans' IS SET TO 1.
	
										//	-----------------------------------
										//	START DIGITIZING
										//
	Stop_iNet_Digitizing();				//	make sure we are not digitizing and are
										//	ready to start digitizing, gsw 10/2/97
										
										//	Press the start button to tell instruNet
										//	to begin digitizing.
	e = PRESS_iNet_BUTTON(iNetCM_BtnPress_Record_Start);
	if (e)	{ goto Exit;}				//	if we got an error, bail out

										//	say that we are starting out
	weAreDoingOurFirstAccessOfData = TRUE;
	
	do 	{								//	loop until the digitization is finished..
	
										//	-----------------------------------
										//	SERVICE instruNet's DIGITIZE BUFFERS
										//
										//	It is ABSOLUTELY NECESSARY to let your
										//	computer's processor service 
										//	instruNet digitizing buffers periodicaly
										//	(10 to .3 times per second)
										//	while digitizing.  This is done
										//	by continuously calling 
										//	Service_All_iNet_Digitize_Buffers() in a loop.
		e = Service_All_iNet_Digitize_Buffers(
				digitizeChAddr[0].netNum,
				&theNextDataAccessIsTheLast_ofEntireDigitization,
				&theNextDataAccessWillPullToTheEndOfAScan,
				&controllerIsFinishedWithDigitization,
				&last_scanNum_base1_pulled_inFull,
				&last_scanNum_base1_pushed_inFull);
		if (e)	{ goto Exit;}			//	if we got an error, bail out
		
		numPointsToPull = 0L;			//	Initialize the number of points to
										//	pull our of the buffer.
										
										//	If we are to process the data little
										//	segments at a time, or we are processing
										//	1 scan at at time and we have that 1 
										//	scan available in the buffers, then
										//	go process the data.
		if (!processOneScanAtATime || theNextDataAccessWillPullToTheEndOfAScan)	{
			
										//	service the channels that have been
										//	turned on for digitizing
			for (digChNum = 0; digChNum < numChannelsToDigitize; digChNum++)	{
							
										//	-----------------------------------
										//	READ DIGITIZED DATA
										//
										//	Here we get an address in memory where
										//	the most recent digitized data resides.
										//
										//	'bufferStartAddrP[]'
										//	
										//		iNetFLT32 data buffer that is
										//		1 Scan long
										//	
										//	'pointToPullindex_base1'
										//
										//		new data is available starting at
										//		bufferStartAddrP[pointToPullindex_base1 - 1]
										//
										//	'numPointsToPull'
										//
										//		number of new data points that are
										//		available
										//
										//	'scanNumberToPull_base1'
										//
										//		scan number {1...numScans} of the
										//		scan that we are currently accessing.
				e = Access_Digitized_Data_In_Ram_Buffer(
						&digitizeChAddr[digChNum], useDriverRamBuffer,
						&bufferStartAddrP, &pointToPullindex_base1,
						&numPointsToPull, &scanNumberToPull_base1);
				if (e)	{ goto Exit;}	//	if we got an error, bail out

										//	remember the start address for each
										//	channel's scan buffer
				digitizeDataPP[ digChNum ] = bufferStartAddrP;

										//	-----------------------------------
										//	PROCESS DIGITIZED DATA
										//
										//	Now that we have some digitized data,
										//	we can process it.
										//
										//	This show examples of processing
										//	digitized data, immediately after it has
										//	been recorded.
										
										//	-----------
										//	Calculate the total # of pts processed to data
				totalPtsPulledSoFar[ digChNum ] = totalPtsPulledSoFar[ digChNum ] + numPointsToPull;
								
										//	-----------
										//	Calculate the sum of digitized points
										//	(this is an example of doing analysis
										//	while digitizing)
				
				calcSumOfPoints = TRUE;	//	true we if we are to calc the sum
										//	of the data points.

				if (calcSumOfPoints)  {	//	if we are to calc the sum of the points...

										//	calc pointer to 1st data point to process
					firstPoint = bufferStartAddrP + pointToPullindex_base1 - 1L;

										//	calc pointer to last data point to process
					lastPoint = firstPoint + numPointsToPull - 1L;
				
		
										//	scan through the digitized data stored in the
										//	RAM buffer
					while (firstPoint <= lastPoint)	{

										#if LOOK_FOR_ZERO
											//	look for a zero value
											if (*firstPoint == 0.0) { Show_Simple_Alert((char *)"hey") }
										#endif
										//	accumulate the sum of all points
						sumOfPoints[digChNum] += *firstPoint++;
					}
				}
			}
		}
										
		if (numPointsToPull > 0) {		//	say that we are no longer starting out
			weAreDoingOurFirstAccessOfData = FALSE;
		}
										//	-----------------------------------
										//	ABORT ON MOUSE DOWN
										//
										//	if we are to allow a mouse
										//	down to stop the digitizing...
		if (allowMouseDownToStopDigitize)	{
				
			if (MOUSE_BUTTON_IS_DOWN()){//	if the mouse button is down...
		
										//	press the STOP button to stop digitizing
				PRESS_iNet_BUTTON(iNetCM_BtnPress_Record_Stop);
										
										//	note that we have stopped
				theNextDataAccessIsTheLast_ofEntireDigitization = TRUE;
			}
		}
										//	sit in loop while there is data to
										//	be pulled out of the buffers
	} while (!theNextDataAccessIsTheLast_ofEntireDigitization  iNetAnd  !controllerIsFinishedWithDigitization);

Exit:

	if (e)		{						//	if we got an error, show an alert
		Show_ALERT_if_hit_iNet_Error(e);
	}

	return e;
}
	