/* ************************************************************************************
INET_EX3.C							Companions: "INET_INT.C", "INET_INT.H", "INET_MCS.H"

	------------------------------- A B S T R A C T ----------------------------------
	
	RECOMMENDATION
	
		Try to do your applications first with the BASIC class routines, documented
		in INET_INT.C, and demonstrated in Example #3.  If these are not adequate, 
		then move to the ADVANCED class routines demonstrated in Examples #1 and #2. 
		In other words, ignor the ADVANCED routines until you are sure you need them.

	This file contains C code examples that do I/O with the instruNet hardware.

	This C code is platform, Operating System, and processor independent. This code
	should run on Macintosh and Windows 95/98/Me/Nt/2k/Xp C and C++ compilers without
	any problems. 
	
	This file expects tabs every 4 characters.
	
	This file is typically used with the following files:
		
		INET_EX3.C	- contains example C code that uses the instruNet hardware
		INET_INT.C  - contains low level source code that interfaces to instruNet driver
		INET_INT.H  - contains many low level enums and #defines that support instruNet
		INET_MCS.H  - contains macros that help set/read instruNet fields
	
	The instruNet disk includes three example programs:

		EXAMPLE #1		Shows how to use almost every instruNet macro and subroutine
						in a text window-based environment.  This example is quite lengthy,
						uses many ADVANCED functions, and is based on file INET_EX1.C.

		EXAMPLE #2		Shows an instruNet application that reads and writes
						several instruNet fields, and then digites.  This example is
						based on file INET_EX2.C, and uses many ADVANCED functions.

		EXAMPLE #3		Example #3  reads and writes fields in a simple way,
						and is the easiest of all the example programs. This example
						uses only the BASIC class functions (not ADVANCED), and is
						based on file INET_EX3.c

	-------------------------------- H I S T O R Y -----------------------------------
	HISTORY		Copyright c 1995-96 GWI.  All rights reserved.
	4/21/97		created - gsw
	7/7/97		updated - gsw

************************************************************************************ */

/* ================================== #includes ==================================== */

//	instruNet includes

	#include "INET_INT.H"				//	instruNet C Interface (for End User)
	#include "INET_MCS.H"				//	instruNet Macros & Functions (for End User)

//	ANSI includes

	#include <stdio.h>					//	for scanf(), printf()	
	#include <stdlib.h>					//	for malloc()	
 
#if (MACHINE_TYPE_ == WINTEL_iNet) 		/*	========= Windows/Intel ======== */
	#include <windows.h>
#endif
								
/* ================================== externs ====================================== */

/* ================================== globals ====================================== */

/* ================================= #defines ====================================== */

/* ============================= static prototypes ================================= */

/* =================================== C Code  ===================================== */
	
/* -------------------------------------------------------------------------------------
 *	FUNCTION  	main()
 *
 *	PURPOSE 	This is the main entry to an application program that shows an example
 *				of how to monitor and control the instruNet network.
 *				
 *	HISTORY		2/4/96		created - gsw
 * ----------------------------------------------------------------------------------- */
 
#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	============= Macintosh ============= */
	int main()	{						
#elif (MACHINE_TYPE_ == WINTEL_iNet)	/*	=========== Windows/Intel =========== */
	int PASCAL WinMain (HINSTANCE hInst, HINSTANCE hPrevInst,  LPSTR lpCmdLine, int nCmdShow) {
#endif
	//	-----------------------------------------------------------------------
	//	Digitize parameters.

	#define BUFF_PTS  		100L		/*  pts digitized from each channel */

	#define NUM_DIGI_CH 	3L			/* 	num of channels to digitize */
										/*  (careful to not blow stack with a number that is too large) */

	//	-----------------------------------------------------------------------
	//	Local Variables

	iNetError e;
	iNetINT16 driverIsInstalledOK, numNetworks, digChNum, numCh, numCh_VoutChannel;
	iNetFLT32 ch1Value;
	iNetUINT32 numBytesOfData, numBytesRead, last_scanNum_base1_pulled_inFull_oneCtlr, ptNum;
	iNetFLT32 *bufferStartAddrP[ NUM_DIGI_CH  + 4], *pointP, *firstPointOfSegment, *lastPointOfSegment, sampleRate;
	iNetFLT32 *firstPointOf_Scan, *lastPointOf_Scan;
	iNetUINT32 pointToPullindex_base1, numPointsToPull, scanNumberToPull_base1;
	iNetFLT32 digiBuffer[ NUM_DIGI_CH ][ BUFF_PTS + 4 ];
	double accum[ NUM_DIGI_CH + 4 ], avg[ NUM_DIGI_CH + 4 ], accum_scan[ NUM_DIGI_CH + 4 ];
	long totalPtsPulled[ NUM_DIGI_CH + 4 ], numSegmentsRead[ NUM_DIGI_CH + 4], numScansToDigitize;
	long totalPtsPulled_scan[ NUM_DIGI_CH + 4 ], numScansRead[ NUM_DIGI_CH + 4 ];
	char msg[256];
	FILE *file;
	Boolean narrate = TRUE, doStripChartMode;

	#define SETTINGS_BYTES 10000L		/*	10KB should be enough for most network states */
										/*  (careful to not blow stack with a number that is too large) */

	iNetINT16 settings[ (SETTINGS_BYTES >> 1) ];

	//	-----------------------------------------------------------------------
	//	Load instruNet driver and initialize instruNet network.
		
	e = Load_instruNet_Driver(FALSE /* reset_instruNet */, &driverIsInstalledOK, &numNetworks);
	if (e) { return 0; }
	if (!driverIsInstalledOK) { return 0; }					
	
	if (narrate) { Show_Simple_Alert("We just loaded the instruNet driver successfully with Load_instruNet_Driver()."); }

	//	-----------------------------------------------------------------------
	//	Read 'Ch1 Vin+' value (i.e. Volts).

										//	read Ch1 Vin+ value
	ch1Value = GetChannelValue_iNet(1, 1 /* deviceNum */, 1, 1 /* chanNum */, &e);
	if (e) { goto Exit; }

	if (narrate) { Show_Simple_Alert("We just read channel Ch1's value with GetChannelValue_iNet()."); }

	//	-----------------------------------------------------------------------
	//	Set 'Ch3 Vout' voltage to 2.0.

	e = SetChannelValue_iNet(1, 1 /* deviceNum */, 1, 3 /* chanNum */, (iNetFLT32) 2.0 /* volts */);
	if (e) { goto Exit; }

	if (narrate) { Show_Simple_Alert("We just wrote channel Ch3's value with SetChannelValue_iNet()."); }

	//	-----------------------------------------------------------------------
	//	Open the instruNet Network Window, and get the state of all the
	//	instruNet channel settings.
	
	if (narrate) { Show_Simple_Alert("We will now open the instruNet Network window, where one can view and adjust channel settings."); }

	e = Open_instruNet_Window(iNetCM_OpenWindow_Network /* 2 */,0L,0L,0L,0L,0L,0L,0L);
	if (e) { goto Exit; }

	//	-----------------------------------------------------------------------
	//	We will now ask the instruNet driver to place all the channel
	//	settings into the settings[] array.

	if (narrate) { Show_Simple_Alert("We will ask the driver to place all the channel settings into the settings[] array."); }
										//	get the instruNet settings
	e = Get_instruNet_Network_State(&settings[0], SETTINGS_BYTES, &numBytesOfData );
	if (e) { goto Exit; }

	//	-----------------------------------------------------------------------
	//	Store all the channel settings to disk (after the user set them as desired
	//	in the NETWORK window).
										
	file = fopen( "iNetSettings", "w");	//	create new text file with name "iNetSettings"
										//	save the settings data to disk
	fwrite(&settings[0], sizeof(char), numBytesOfData, file);
	fclose( file );						//	close file

	if (narrate) { Show_Simple_Alert("We just saved all the instruNet channel settings in file 'iNetSettings'."); }

	//	-----------------------------------------------------------------------
	//	Read the settings file in from disk and then restore all the channel
	//	settings (to show that we know how to do it)
											
	file = fopen( "iNetSettings", "r");	//	create new text file with name "iNetSettings"
										//	save the settings data to disk
	numBytesRead = fread(&settings[0], sizeof(char), SETTINGS_BYTES, file);
	fclose( file );						//	close file

	if (narrate) { Show_Simple_Alert("We just loaded back from disk all the instruNet channel settings."); }

	//	-----------------------------------------------------------------------
	//	Restore the instruNet channel settings to that which was saved on disk.

	e = Set_instruNet_Network_State(&settings[0], numBytesRead);
	if (e) { goto Exit; }

	//	-----------------------------------------------------------------------
	//	Digitize one 100point scan of 3 channels at 100samples/second/channel in oscilloscope mode.

	numScansToDigitize = 1;				//	# of scans to digitize
	sampleRate = (iNetFLT32) 100.0;		//	sample rate
	doStripChartMode = FALSE;			//	do strip chart mode

	if (narrate) { Show_Simple_Alert("We will now reset the list of channels to digitize (i.e. set it's length to 0)."); }

	e = ResetChannelList();				//	clear the list of channels to digitize
	if (e) { goto Exit; }

	if (narrate) { Show_Simple_Alert("We will now add 3 channels (i.e. 'Ch1 Vin', 'Ch4 Vin' & 'Ch3 Vout') to the list of channels to digitize."); }

	numCh = 0;							//	start out with no channels in the list

										//	Add 'Ch1 Vin' to list
	e = AddToChannelList(1,1,1,1 /* chanNum */, &digiBuffer[ numCh++ ][ 0 ], BUFF_PTS * sizeof(iNetFLT32));
	if (e) { goto Exit; }

										//	Add 'Ch4 Vin' to list
	e = AddToChannelList(1,1,1,4 /* chanNum */, &digiBuffer[ numCh++ ][ 0 ], BUFF_PTS * sizeof(iNetFLT32));
	if (e) { goto Exit; }
										
	numCh_VoutChannel = numCh;			//	Add 'Ch3 Vout' to list (i.e. a d/a output channel).
	e = AddToChannelList(1,1,1,3 /* chanNum */, &digiBuffer[ numCh++ ][ 0 ], BUFF_PTS * sizeof(iNetFLT32));
	if (e) { goto Exit; }
										//	load Vout 3 channel buffer with a fast square wave
	for (ptNum = 0; ptNum < (BUFF_PTS - 1L); ptNum += 2)	{
		digiBuffer[ numCh_VoutChannel ][ ptNum   ] = +4.0 /* +4 Volts */;
		digiBuffer[ numCh_VoutChannel ][ ptNum+1 ] = -4.0 /* -4 Volts */;
	}

	if (numCh > NUM_DIGI_CH) {			//	make sure we don't overflow our array
		Show_Simple_Alert("We need to increment 'NUM_DIGI_CH' before adding more channels.");
		goto Exit;
	}

	if (narrate) { Show_Simple_Alert(
"We will now set the timing parameters to digitize one 100point scan at 100pts/sec/ch.  This will crash unless you have a real i100xx attached to a real i2x0 (i.e. Demo mode does not work)."); }

	e = Set_iNet_TIMING_Parameters(		//	set the timing options
			(iNetFLT32) sampleRate, 	//	sample rate (points per second per channel)
			BUFF_PTS,					//	number of points in 1 scan, for 1 channel
			numScansToDigitize,			//	number of scans that are digitized when the
										//	start button is pressed
			doStripChartMode,			//	if TRUE (1) we do Strip Chart Recorder work with
										//	Continuous Scans, otherwise if FALSE (0) we do
										//	Oscilloscope mode (non-Coninuous Scans)
			ion_gSaveDataPopup_ToRamBuffer /* 2 */,			
										//	Save Data option = { ion_gSaveDataPopup_Off,
										//	ion_gSaveDataPopup_ToRamBuffer, ion_gSaveDataPopup_ToDisk,
										//	ion_gSaveDataPopup_UserControl}
			TRUE,						//	if TRUE (1), the driver will show an alert if
										//	a data buffer overflows; otherwise if FALSE (0) 
										//	it just keeps going if there is an overflow.
			TRUE);						//	if TRUE (1), channel switching should be accurate;
										//	otherwise, it is less accurate, yet faster.
	if (e) { goto Exit; }

										//	Note: To specify a trigger, call
										//	Set_iNet_TRIGGER_Parameters().
	
	Stop_iNet_Digitizing();				//	make sure we are not digitizing and are
										//	ready to start digitizing, gsw 10/2/97

										//	digitize the channels in the channel list
										//	and return when 1scan has been digitized.
	e = DigitizeListOfChannels(TRUE	/* allowMouseDownToStopDigitize */,
							   TRUE /* processOneScanAtATime */,
		 					  &last_scanNum_base1_pulled_inFull_oneCtlr);
	if (e) { goto Exit; }

	if (narrate) { Show_Simple_Alert("We just finished digitizing one 1 second scan from 3 channels."); }

										//	calc the average value in each channel
	for (digChNum = 0; digChNum < numCh; digChNum++)	{
		accum[ digChNum ] = 0.0;		//	scan through each point
		for (ptNum = 0; ptNum < BUFF_PTS; ptNum++)	{
										//	add this point to the accumulator
			accum[ digChNum ] +=  digiBuffer[ digChNum ][ ptNum ];
		}								//	calc the average
		avg[ digChNum ] = accum[ digChNum ] / (double) BUFF_PTS;
	}

	if (narrate)	{
		sprintf(msg, "The average value digitized from each channel is: %f, %f and %f.",
			((float) avg[ 0 ]), ((float) avg[ 1 ]), ((float) avg[ 2 ]));
		Show_Simple_Alert(msg);
	}

	//	-----------------------------------------------------------------------
	//	Now demonstrate continuous recording via strip chart mode.  We will digitize
	//	10 scans and process the data while it is being digitized.  We digitize
	//	100pts/scan at 100pts/second/channel.

	if (narrate) { Show_Simple_Alert("We will now set the timing parameters to digitize ten 1 second scans at 100pts/sec, 100pts/scan, over a 10sec period."); }

	numScansToDigitize = 10;			//	# of scans to digitize
	sampleRate = (iNetFLT32) 100.0;		//	sample rate
	doStripChartMode = TRUE;			//	do strip chart mode

	e = Set_iNet_TIMING_Parameters(		//	set the timing options
			sampleRate, 				//	sample rate (points per second per channel)
			BUFF_PTS,					//	number of points in 1 scan, for 1 channel
			numScansToDigitize,			//	number of scans that are digitized when the
										//	start button is pressed
			doStripChartMode,			//	if TRUE (1) we do Strip Chart Recorder work with
										//	Continuous Scans, otherwise if FALSE (0) we do
										//	Oscilloscope mode (non-Coninuous Scans)
			ion_gSaveDataPopup_ToRamBuffer /* 2 */,			
										//	Save Data option = { ion_gSaveDataPopup_Off,
										//	ion_gSaveDataPopup_ToRamBuffer, ion_gSaveDataPopup_ToDisk,
										//	ion_gSaveDataPopup_UserControl}
			TRUE,						//	if TRUE (1), the driver will show an alert if
										//	a data buffer overflows; otherwise if FALSE (0) 
										//	it just keeps going if there is an overflow.
			TRUE);						//	if TRUE (1), channel switching should be accurate;
										//	otherwise, it is less accurate, yet faster.
	if (e) { goto Exit; }

										//	Note: To specify a trigger, call
										//	Set_iNet_TRIGGER_Parameters().

	if (narrate) { Show_Simple_Alert("We will digitize and analyze little segments of data as they are acquired over the 10sec period."); }

	for (digChNum = 0; digChNum < numCh; digChNum++)	{
		accum[ digChNum ] = 0.0;		//	zero the accumulator
		totalPtsPulled[ digChNum ] = 0L;
		numSegmentsRead[ digChNum ] = 0L;
	}

	Stop_iNet_Digitizing();				//	make sure we are not digitizing and are
										//	ready to start digitizing, gsw 10/2/97

	do	{								//	start digitizing and process little segments
										//	as they are acquired

										//	digitize the channels in the channel list
		e = DigitizeListOfChannels(FALSE /* allowMouseDownToStopDigitize */,
								   FALSE /* processOneScanAtATime */,
		 						  &last_scanNum_base1_pulled_inFull_oneCtlr);
		if (e) { goto Exit; }

										//	scan thru the list of digitized channels
		for (digChNum = 0; digChNum < numCh; digChNum++)	{
	
										//	---------------------
										//	GET A LITTLE SEGMENT OF DIGITIZED DATA

			e = GetDigitizedSegment(digChNum, &bufferStartAddrP[ digChNum ],
					&pointToPullindex_base1, &numPointsToPull, &scanNumberToPull_base1);
			if (e) { goto Exit; }

			if (numPointsToPull > 0L) {	//	if we have some new data...

										//	---------------------
										//	PROCESS LITTLE SEGMENTS OF DATA

										//	define the 1st and last point of segment
				firstPointOfSegment = bufferStartAddrP[ digChNum ] + pointToPullindex_base1 - 1L;
				lastPointOfSegment = firstPointOfSegment + numPointsToPull - 1L;
			
										//	scan thru the little SEGMENT (to calc the avg value)
				for (pointP = firstPointOfSegment; pointP <= lastPointOfSegment; pointP++)	{
										//	add this point to the accumulator
					accum[ digChNum ] += (double) *pointP;
				}	
										//	remember the total # of pts read so far for this channel
				totalPtsPulled[ digChNum ] += numPointsToPull;
				numSegmentsRead[ digChNum ] += 1L;
	
										//	---------------------
										//	PROCESS 1 WHOLE SCAN

										//	if this segment includes the last point in the 
										//	scan buffer AND WE ARE ON THE LAST DIGITIZE CHANNEL...
				if ( (lastPointOfSegment == (bufferStartAddrP[ digChNum ] + BUFF_PTS - 1L) )  iNetAnd	
				     (digChNum == (numCh-1)) )	{

										//	scan thru the list of digitized channels
					for (digChNum = 0; digChNum < numCh; digChNum++)	{

						firstPointOf_Scan = bufferStartAddrP[ digChNum ];
						lastPointOf_Scan  = firstPointOf_Scan + BUFF_PTS - 1;

										//	scan thru the SCAN buffer
						for (pointP = firstPointOf_Scan; pointP <= lastPointOf_Scan; pointP++)	{
										//	add this point to the accumulator
							accum_scan[ digChNum ] += (double) *pointP;
						}	
										//	remember the total # of pts read so far for this channel
						totalPtsPulled_scan[ digChNum ] += (1L + (lastPointOf_Scan - firstPointOf_Scan));
						numScansRead[ digChNum ] += 1L;	
					}
				}
			}							
		}

	} while ((iNetINT32) last_scanNum_base1_pulled_inFull_oneCtlr < numScansToDigitize);

	if (narrate) { Show_Simple_Alert("We just finished digitizing ten 1second scans from 3 channels."); }

										//	calc the average value in each channel
	for (digChNum = 0; digChNum < numCh; digChNum++)	{
		avg[ digChNum ] = accum[ digChNum ] / (double) (BUFF_PTS * numScansToDigitize);
		if ((accum[ digChNum ] - accum_scan[ digChNum ]) > 1.0) { Show_Simple_Alert("We hit an inconsistency."); }
	}

	if (narrate)	{
		sprintf(msg, "The average value digitized from each channel is: %f, %f and %f. We read %ld, %ld, %ld pts per channel from %ld, %ld, %ld segments.",
			((float) avg[ 0 ]), ((float) avg[ 1 ]), ((float) avg[ 2 ]),
			((long) totalPtsPulled[ 0 ]), ((long) totalPtsPulled[ 1 ]), ((long) totalPtsPulled[ 2 ]),
			((long) numSegmentsRead[ 0 ]), ((long) numSegmentsRead[ 1 ]), ((long) numSegmentsRead[ 2 ]));
		Show_Simple_Alert(msg);
	}

	//	-----------------------------------------------------------------------
	//	Exit.

Exit:

	Stop_iNet_Digitizing();				//	make sure we are not digitizing

	ResetChannelList();					//	clear the list of channels to digitize
										//	(to disconnect from the local channel buffers)

	if (e)	{
		Show_ALERT_if_hit_iNet_Error(e);//	show an alert if there was an error
	}
										
	if (narrate)	{					//	tell the user we are done
		Show_Simple_Alert("We will now close the instruNet driver with CloseDriverAndReleaseDriverRam().");
		Show_Simple_Alert("We are done with Example#3, the most simple example.");
	}

	CloseDriverAndReleaseDriverRam();	//	Closes the driver and releases it's memory
 
#if (MACHINE_TYPE_ == WINTEL_iNet)		/*	=========== Windows/Intel =========== */
	STOP_COMPILER_WARNING(hInst);
	STOP_COMPILER_WARNING(hPrevInst);
	STOP_COMPILER_WARNING(lpCmdLine);
	STOP_COMPILER_WARNING(nCmdShow);
#endif

	return e;
}

