/* ************************************************************************************
IONC_INT.C			 				Companions: "INET_INT.H", "INET_MCS.H"
   
	------------------------------- A B S T R A C T ----------------------------------
	  
	This file contains interface code for the C application that wants to communicate 
	with the instruNet DRIVER.   

	This C code is platform, Operating System, and processor independent. This code 
	should run on Macintosh and Windows 95 C and C++ compilers without any problems. 

	This file expects tabs every 4 characters. 
  
	---------------------------------- N O T E S -------------------------------------	
	
	RECOMMENDATION
	
		Try to do your applications first with the BASIC class routines, listed
		below, and demonstrated in Example #3.  If these are not adequate, then
		move to the ADVANCED class routines, listed below, and demonstrated
		in Examples #1 and #2.  In other words, ignor the ADVANCED routines
		until you are sure you need them.
		
	OVERVIEW 
	  
		The iNet() routine calls the instruNet driver which resides in a DLL, code
		resource, or shared library (depending on your operating system).  This is
		call is extremely important, since it is the method through which a 
		programmer controls and works with instruNet.  This file contains several
		wrapper functions that are very similar to iNet(), yet work with specific
		variable types.  For example, iNet_int32() is used to read or write any
		field in the instruNet field heirarchy, with a signed 32bit integer.
		iNet_flt32() does the same, yet with a 32bit floating point variable.
		The source code in this file can be thought of a the glue between the C
		programmer, and the instruNet driver.  The caller of these routines does
		not need to be familiar with the inner workings of this source code, yet
		only the functionality of the routines themselves.
		
	--------------------------- B A S I C   R O U T I N E S --------------------------	
		 
	BASIC READ/WRITE CHANNEL VALUE

		GetChannelValue_iNet()			Read value of instruNet channel.
		SetChannelValue_iNet()			Set value of instruNet channel.
		GetFieldValue_iNet()			Read value of instruNet field.
		SetFieldValue_iNet()			Set value of instruNet field.
	
	BASIC DIGITIZE
	
		ResetChannelList()				Reset the list of channels to digitize.
		AddToChannelList()				Add a channel to the list of channels to digitize.
		Set_iNet_TIMING_Parameters()	Set digitize timing parameters (e.g. sampleRate, ptsPerScan, etc)
		Set_iNet_TRIGGER_Parameters()	Set digitize trigger parameters (e.g. trigger on/off, trigger source)
		DigitizeListOfChannels()		Tell instruNet to begin digitizing
		GetDigitizedSegment()			Get a little segment of digitized data.
		PutEnabledChInDigitizeList()	Place enabled channels in the digitize list.
		Stop_iNet_Digitizing()			Tells instruNet to stop digitizing
		ThisIsAlreadyInChannelList()	Returns true if this channel is already in digitize list.
		CalcDigiIntegrationTimes()		Calc the total integration time for one network.
		GetDigitizeStatus()				Returns status of instruNet DSP card.
		GetOutputSegmentBuffer()		Get location of output segment buffer
		SubmitOutputSegment()			Tell driver that an output segment has been submitted

	BASIC UTILITY

		Load_instruNet_Driver()			Loads instruNet driver in to memory & resets network		
		CloseDriverAndReleaseDriverRam()Closes instruNet driver and releases memory
		Show_ALERT_if_hit_iNet_Error()	Shows an alert if instruNet hit an error
		Show_Simple_Alert()				Displays a message in an alert dialog box.
		Open_instruNet_Window()			Opens an instruNet World window.
		Press_instruNet_Button()		Press an instruNet World button.
		Get_instruNet_Network_State()	Load all channel settings into an array.
		Set_instruNet_Network_State()	Restore all channel settings from data in array.
		SelectNextChannelAddress()		Select the next channel address in the network.
		Get_iWp_LicenseState()			Returns 1 if licensed for iW+, 0 otherwise.

	------------------------- A D V A N C E D   R O U T I N E S ----------------------	
	
	ADVANCED READ/WRITE FIELD

		SetField_int32() 				Set instruNet field with 32bit signed integer number
		SetField_uint32() 				Set instruNet field with 32bit unsigned integer number
		SetField_flt32() 				Set instruNet field with 32bit floating point number
		SetField_cStr() 				Set instruNet field with C string (terminated with 0x00)

		GetField_int32() 				Read instruNet field into a 32bit signed integer variable
		GetField_uint32() 				Read instruNet field into a 32bit unsigned integer variable
		GetField_flt32() 				Read instruNet field into a 32bit floating point variable
		GetField_cStr() 				Read instruNet field into a C string variable (terminated with 0x00) 

		iNet_int32()					Reads or writes to a field via a 32bit signed integer variable
		iNet_uint32()					Reads or writes to a field via a 32bit unsigned integer variable
		iNet_flt32()					Reads or writes to a field via a 32bit floating point variable
		iNet_cString()					Reads or writes to a field via a C string variable
		iNet_pString()					Reads or writes to a field via a Pascal string variable
		iNet_DLL()						Reads or writes to a field via any type of variable

	ADVANCED DIGITIZE

		LoadChannelAddress()			Specify an instruNet {netNum, devNum, modNum, chNum} address
		GetChannelAddress()				Reads an instruNet address {netNum, devNum, modNum, chNum} address
		EnableChannelForDigitizing()	Mark this channel for digitizing (i.e. when digitizing is started)
		DisableAllChannelsForDigitizing() 		Disable all channels for digitizing
		Service_All_iNet_Digitize_Buffers()	 	Services internal buffers while digitizing
		Access_Digitized_Data_In_Ram_Buffer()	Provides access to digitized data (even during acquisition)
		Get_Ctlr_Ram_Buff_Info()		Get info about controller ram buffer

	ADVANCED UTILITY

		Flt32ToDouble_iNet()			Convert an array of flt32 to array of doubles
		DoubleToFlt32_iNet()			Convert an array of doubles to an array of flt32
		iNet_Peek_int16()				Reads 16bit integer number from a specific logical address
		iNet_Peek_int32()				Reads 32bit integer number from a specific logical address
		iNet_Peek_flt32()				Reads 32bit floating point number from a specific logical address
		
		iNet_Poke_int16()				Writes 16bit integer number to a specific logical address
		iNet_Poke_int32()				Writes 32bit integer number to a specific logical address
		iNet_Poke_flt32()				Writes 32bit floating point number to a specific logical address

		iNet_GetStaticVar()				Read internal static variables initiailized to 0.
		iNet_SetStaticVar()				Write to internal static variable.
		
		iNet_Get_VarPtr()				Returns a pointer to the passed variable

		iNet_memcopy()					Copies a memory block from one place to another

		Get_iNet_Error()				Gets error code to last instruNet function call
		Get_Last_iNet_Call()			Returns pointer to struct with params to last iNet call
		GetiNetTime_Double()			Returns time since reset, in a double variable
		Set_iNet_Demo_Mode()			Sets the instruNet demonstration mode.
		Get_Win32_DLL_VersionNum()		Returns Windows iNet32.dll version #.

	ANALYSIS

		fft_DLL()						Calculate an fft.
		MultiplyArrayByHammWindow_DLL() Multiple flt32 Array by Hamming window.
		CalculateWindow_DLL()			Calculate a hamming window.
		MultiplyTwoFlt32Arrays_DLL()	Multiply 2 flt32 arrays.
		ComplexToMagnitudes_DLL()		Convert complex pairs in an array to magnitudes.
		MultConstantByArray_DLL()		Multiply a constant by a flt32 array.

	-------------------------------- H I S T O R Y -----------------------------------
	HISTORY		Copyright c 1994-98 GWI.  All rights reserved. 
	5/18/94		created - gsw 
 
************************************************************************************ */

/* ================================== #includes ==================================== */
  
//	ANSI includes    

	#include <stdio.h>					//	for sprintf()
	#include <string.h>					//	for memcpy()

//	instruNet includes

	#include "INET_INT.H"				//	instruNet C Interface (for End User)
	#include "INET_MCS.H"				//	instruNet Macros & Functions (for End User)
	
#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	============== Macintosh =============== */

	#include <MixedMode.h>
	#include <Files.h>
	#include <Folders.h>				/*	for FindFolder() */ 
	#include <Resources.h>				/*	for OpenRFPerm() */
	#include <QuickDraw.h>				/*	For qd globals */
	#include <Fonts.h>					/*	For InitFonts() */
	#include <Events.h>					/*	For FlushEvents() */
	#include <Windows.h>				/*	For InitWindows() */
	#include <Menus.h>					/*	For InitMenus() */
	#include <TextEdit.h>				/*	For TEInit()    */
	#include <Dialogs.h>				/*	For InitDialogs() */
	#include <Memory.h>					/*	For MaxApplZone() */
	#include <Resources.h>				/*	For OpenResFile() */ 
	
	#include <TextUtils.h>				/*	NumToString() */
	#include <LowMem.h>					/*	LMSetCurDirStore() */
	
	#if OLDROUTINENAMES					/*	codewarrior 1.6 */
	   #include <GestaltEqu.h>			/*	For Gestalt() */
	#else								/*	codewarrior >= 3 */
	   #include <Gestalt.h>				/*	For Gestalt() */
	   #include <sound.h>				/*	for Sysbeep() */
	#endif
	
	#define StripAddress_interface(p)	StripAddress(p)
	
#elif (MACHINE_TYPE_ == WINTEL_iNet) 	/*	============  Windows/Intel ============ */

	#define StripAddress_interface(p)	(p)
		
#else
 
	... it is a compiler error if we get here

#endif

/* ================================== externs ====================================== */

/* ================================== globals ====================================== */

										//	TRUE if the ansi console window has been opened
Boolean gConsoleWindowHasBeenOpened = FALSE;

iNetUINT8 *gRequestP = 0L;				//	points to the request struct passed to iNet()

										//	this is used to pass parameters back and
										//	forth between the driver
iNetUINT8	gRequest_struct[SIZE_OF_iNet_Request + 8 /* safety */];

iNetINT16	gDriverIsInstalledOk = FALSE;
iNetINT16	gNumberOfNetworks = 0;
Boolean		gWeTriedToLoadDriver = FALSE;
										//	temp storage
iNetINT32	gTempINT32iNet, gTempCompiler, gTemp32A, gTemp32B, gTemp32C, gTemp32_seti, gTemp32_setd, gMaxNumChars;
iNetINT16	gTempINT16iNet;	
iNetUINT16	gTempUINT16iNet;
iNetFLT32	gTempFLT32iNet, gTempFLT32iNetA, gTempFLT32iNetB, gTempFLT32iNetC, gTempFLT32_conv, gTempFLT32_setf;	
double		gTempDoubleiNet, gTempDoubleiNetA, gTempDoubleiNetB, gTempDoubleiNetC, gTempDouble_setPos, gTempD_conv;

static short gFileRefNum = -1;			//	gsw 8/25/98

#define VERS_LEN 24						/*	"iNet95.dll" Win32 DLL Version # */
char gDLL_ver_cStr[ VERS_LEN + 4 ] = "?";			
iNetUINT32 gDLL_ver_majorVer = 0L;		
iNetUINT32 gDLL_ver_minorVer = 0L;
iNetUINT32 gDLL_ver_subminorVer = 0L;
iNetUINT32 gDLL_ver_subSubVer = 0L;

void *giNet_Main_EntryPtr_= (void *)0L;	//	Ptr to iNet Driver code in RAM

#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	============= Macintosh ============= */
	Handle gRsrcH = 0L;					//	handle to instruNet code rsrc
	#if defined(__MWERKS__)				/*	if on the metrowerks compiler... */	
		char *gCr = "\n";				//	Define carriage return chacter (depends
	#else								//	on C compiler).
		char *gCr = "\r\n";
	#endif
#elif (MACHINE_TYPE_ == WINTEL_iNet)	/*	=========== Windows/Intel =========== */
	HANDLE gLib = (HANDLE) NULL;		//	handle to iNet32.dll
	char *gCr = "\r\n";
#endif

/* ================================= #defines ====================================== */

/* ============================= static prototypes ================================= */

static unsigned char *CtoP_Strcpy_local(unsigned char *dest_P, char *src_C);
static void iNet_memcopy_local(void *srcAddr, void *destAddr,
	iNetUINT32 numBytes);
static void iNet_Dispose_Handle_local(char * handleInRam);
static Boolean iNet_Lock_Handle_local(char *handleInRam, iNetUINT32 *hSizeBytesP,
	iNetINT16 **hAddressPP);
static void iNet_Unlock_Handle_local(char *handleInRam);
static iNetError Get_iNet_Error_local(void);
static iNetError /* iNetINT16 */ DisableAllChannelsForDigitizing_local(void);		
static iNetError Show_ALERT_if_hit_iNet_Error_local(iNetError e);
static iNetError ProcessBufferData(iNetChannelAddr *channelAddr,
	Boolean driverRamBuffer, iNetFLT32 **bufferStartAddrP,
	iNetUINT32 *pointToPullindex_base1, iNetUINT32 *numPointsToPull,
	iNetUINT32 *scanNumberToPull_base1, iNetINT16 intention, iNetUINT32 submitValue);
	
#if (MACHINE_TYPE_ == WINTEL_iNet)		/*	=========== Windows/Intel =========== */
	static void Get_Win32_DLL_VersionNum(char *dllName_cStr, char *version_cStr, 
		iNetUINT32 *majorVerP, iNetUINT32 *minorVerP, iNetUINT32 *subminorVerP, iNetUINT32 *subsubVerP);
#endif

/* =================================== C Code  ===================================== */

//	===================== CLASS: BASIC READ/WRITE CHANNEL ========================
//	===================== CLASS: BASIC READ/WRITE CHANNEL ========================
//	===================== CLASS: BASIC READ/WRITE CHANNEL ========================

/* -----------------------------------------------------------------------------------
 *	FUNCTION  	GetChannelValue_iNet()			CLASS: BASIC READ/WRITE CHANNEL
 *
 *				Returns the value of the specified channel.
 *
 *	HISTORY		1/27/97	created - gsw
 * --------------------------------------------------------------------------------- */
iNetFLT32 EXT_FUNC_TYPE GetChannelValue_iNet(	
	iNetUINT8 netNum,					//	NETWORK number, 0 is Driver, 1 is 1st card
	iNetUINT8 devNum,					//	DEVICE number, 0 = "Controller", 1 = 1st device on network
	iNetUINT8 modNum, 					//	MODULE number within a hardware DEVICE {1...32}.
	iNetUINT8 chNum, 					//	Hardware CHANNEL number {1...32}.
	iNetError *errP)					//	if this != 0L, we load it with error value
{
	iNetFLT32 flt32Value;				//	read the value of the channel
	iNetError e = (iNetError) iNet(netNum, devNum, modNum, chNum,
						(iNetINT16) sgt_General, (iNetINT16) fldNum_General_valueEu,
			  			(iNetINT16) intention_getValue, (iNetUINT8) iNetDT_FLT32,
			 			(void *) &flt32Value);			 			 
	if (errP != 0L) { *errP = e; }		//	get the error code	
	return flt32Value;					//	return the value
}
										//	generic only uses {short, long, double}
		double EXT_FUNC_TYPE GetChannelValue_iNet_Generic(short netNum, short devNum,
			short modNum, short chNum, short *errP)				
		{
			double doubleValue;			//	read the value of the channel
			iNetError e = (iNetError) iNet((iNetUINT8) netNum, (iNetUINT8) devNum, (iNetUINT8) modNum, (iNetUINT8) chNum,
				(iNetINT16) sgt_General, (iNetINT16) fldNum_General_valueEu, (iNetINT16) intention_getValue, (iNetUINT8) iNetDT_Double,
				(void *) &doubleValue);			 			 
			if (errP != 0L) { *errP = e; }
			return doubleValue;			//	return the value
		}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	SetChannelValue_iNet()			CLASS: BASIC READ/WRITE CHANNEL
 *
 *				Returns the value of the specified channel.
 *
 *	HISTORY		1/27/97	created - gsw
 * --------------------------------------------------------------------------------- */
iNetError /* iNetINT16 */ EXT_FUNC_TYPE SetChannelValue_iNet(	
	iNetUINT8 netNum,					//	NETWORK number, 0 is Driver, 1 is 1st card
	iNetUINT8 devNum,					//	DEVICE number, 0 = "Controller", 1 = 1st device on network
	iNetUINT8 modNum, 					//	MODULE number within a hardware DEVICE {1...32}.
	iNetUINT8 chNum, 					//	Hardware CHANNEL number {1...32}.
	iNetFLT32 flt32_newValue)			//	set the channel with the new value
{
	iNetError e = (iNetError) iNet(netNum,  devNum, modNum, chNum,
						(iNetINT16) sgt_General, (iNetINT16) fldNum_General_valueEu,
			  			(iNetINT16) intention_setValue, (iNetUINT8) iNetDT_FLT32,
			 			(void *) &flt32_newValue);
	return e;							//	return the error code
}

										//	generic only uses {short, long, double}
		short /* iNetINT16 error */ EXT_FUNC_TYPE SetChannelValue_iNet_Generic(	
			short netNum, short devNum, short modNum, short chNum, double newValue)
		{								//	set the channel with the new value
			iNetError e = (iNetError) iNet((iNetUINT8) netNum,  (iNetUINT8) devNum, (iNetUINT8) modNum,
				(iNetUINT8) chNum, (iNetINT16) sgt_General, (iNetINT16) fldNum_General_valueEu,
				(iNetINT16) intention_setValue, (iNetUINT8) iNetDT_Double, (void *) &newValue);
			return e;
		}

/* -----------------------------------------------------------------------------------
 *	FUNCTION  	GetFieldValue_iNet()			CLASS: BASIC READ/WRITE CHANNEL
 *
 *				Returns the value of the specified field.
 *
 *	HISTORY		10/10/97	created - gsw
 * --------------------------------------------------------------------------------- */
iNetFLT32 EXT_FUNC_TYPE GetFieldValue_iNet(	
	iNetUINT8 netNum,					//	NETWORK number, 0 is Driver, 1 is 1st card
	iNetUINT8 devNum,					//	DEVICE number, 0 = "Controller", 1 = 1st device on network
	iNetUINT8 modNum, 					//	MODULE number within a hardware DEVICE {1...32}.
	iNetUINT8 chNum, 					//	Hardware CHANNEL number {1...32}.
	iNetINT16 settingGroupNumOrType,	//	if < 0, settings Type = { sgt_VinHardware (-3), sgt_Vout (-4), etc. }
										//	if > 0, settings Number = Setting popup menu choice.
	iNetINT16 fieldNum,					//	Field number within the settingGroup = { 1...8}
										//	See fgNums_... enum's in iNet_INET.H file.
	iNetError *errP)					//	if this != 0L, we load it with error value
{
	iNetFLT32 flt32Value;				//	read the field within the channel
	iNetError e = (iNetError) iNet(netNum, devNum, modNum, chNum, settingGroupNumOrType, fieldNum,
			  			(iNetINT16) intention_getValue, (iNetUINT8) iNetDT_FLT32, (void *) &flt32Value);			 			 
	if (errP != 0L) { *errP = e; }		//	get the error code	
	return flt32Value;					//	return the value
}

/* -----------------------------------------------------------------------------------
 *	FUNCTION  	SetFieldValue_iNet()			CLASS: BASIC READ/WRITE CHANNEL
 *
 *				Sets the value of the specified field.
 *
 *	HISTORY		10/10/97	created - gsw
 * --------------------------------------------------------------------------------- */
iNetError /* iNetINT16 */ EXT_FUNC_TYPE SetFieldValue_iNet(	
	iNetUINT8 netNum,					//	NETWORK number, 0 is Driver, 1 is 1st card
	iNetUINT8 devNum,					//	DEVICE number, 0 = "Controller", 1 = 1st device on network
	iNetUINT8 modNum, 					//	MODULE number within a hardware DEVICE {1...32}.
	iNetUINT8 chNum, 					//	Hardware CHANNEL number {1...32}.
	iNetINT16 settingGroupNumOrType,	//	if < 0, settings Type = { sgt_VinHardware (-3), sgt_Vout (-4), etc. }
										//	if > 0, settings Number = Setting popup menu choice.
	iNetINT16 fieldNum,					//	Field number within the settingGroup = { 1...8}
										//	See fgNums_... enum's in iNet_INET.H file.
	iNetFLT32 flt32_newValue)			//	set the field with the new value
{
										//	write channel field
	iNetError e = (iNetError) iNet(netNum, devNum, modNum, chNum, settingGroupNumOrType, fieldNum,
			  			(iNetINT16) intention_setValue, (iNetUINT8) iNetDT_FLT32, (void *) &flt32_newValue);			 			 
	return e;							//	return the error value
}

//	=========================== CLASS: BASIC DIGITIZE ============================
//	=========================== CLASS: BASIC DIGITIZE ============================
//	=========================== CLASS: BASIC DIGITIZE ============================

										/*	addresses of digitize channels */
iNetChannelAddr gChAddr[ MAX_NUM_DIGI_CH + 2L ];

										/*	false if using the USER's buffer; true
										 *	if using the DRIVER's ram buffer for
										 *	digitized data. */
Boolean gUseDriverRamBuffer[ MAX_NUM_DIGI_CH + 2L ];

										/*	(dir_hdwr_input_circBuf = 1) for input channels (e.g. a/d),
										 *	(dir_hdwr_output_circBuf = 2) for output channels
										 *	(e.g. d/a), and (dir_hdwr_inAndOut_circBuf = 4) for both input
										 *	and output channel, yet digitized as in input (e.g.
										 *	iNet 100 digital i/o, controller digital/timer i/o).
										 *	gsw 10/27/97. */
char g_Channel_dType[ MAX_NUM_DIGI_CH + 2L ];

										/*	contains the 1st address of the digitize
										 *	scan buffers, 1 for each channel digitized */
iNetFLT32 *gBufferStartAddrP[ MAX_NUM_DIGI_CH + 2L ];

										/*	index of the first point of the segment that we need
										 *	to process (base 1 index) = {1... numPtsPerScan} */
iNetUINT32 gPointToPullindex_base1[ MAX_NUM_DIGI_CH + 2L ];

										/*	contains the scan number {1...numScans}
										 *	of the scan that we are currently pulling (base 1). */
iNetUINT32 gScanNumberToPull_base1[ MAX_NUM_DIGI_CH + 2L ];

										/*	contains the # of new pts to pull out of the buffer */
iNetUINT32 gNumPointsToPull[ MAX_NUM_DIGI_CH + 2L ];

iNetINT16 gNumDigiChans = 0;			/* 	# of channels in gChAddr digitize channel list */

Boolean gWeAreDigitizing = FALSE;		/*	true if we are digitizing */

/*	These macros are used to set fields in the Timing or Trigger areas.
 *
 *		SET_FLT32_DIGI()				Set Timing/Trigger field with flt32 value.
 *		SET_INT32_DIGI()				Set Timing/Trigger field with int32 value.
 *		SET_INT32_DRVR()				Set Driver field with int32 value.
 */
										//	Set Timing/Trigger field with flt32 value.
#define SET_FLT32_DIGI(netNum_, settingGp_, fieldNum_, flt32Value_)	\
	(iNetError) iNet((iNetUINT8) netNum_, (iNetUINT8) deviceNum_CONTROLLER /* 0 */,  \
		(iNetUINT8) moduleNum_1stModule /* 1 */, (iNetUINT8) controller_ChanNum_C12_Digitizer /* 12 */, \
		(iNetINT16) settingGp_, (iNetINT16) fieldNum_, \
		(iNetINT16) (((gTempFLT32_setf = (flt32Value_)) != 2491251245.23412e-312) ? intention_setValue /* 2 */ : intention_getValue), \
		(iNetUINT8) iNetDT_FLT32, &gTempFLT32_setf)

										//	Set Timing/Trigger field with int32 value.
#define SET_INT32_DIGI(netNum_, settingGp_, fieldNum_, int32Value_)	\
	(iNetError) iNet((iNetUINT8) netNum_, (iNetUINT8) deviceNum_CONTROLLER /* 0 */,  \
		(iNetUINT8) moduleNum_1stModule /* 1 */, (iNetUINT8) controller_ChanNum_C12_Digitizer /* 12 */, \
		(iNetINT16) settingGp_, (iNetINT16) fieldNum_, \
		(iNetINT16) (((gTemp32_seti = (int32Value_)) != 2191251245L) ? intention_setValue /* 2 */ : intention_setValue), \
		(iNetUINT8) iNetDT_INT32, &gTemp32_seti)

										//	Set Driver field with int32 value.
#define SET_INT32_DRVR(chanNum_, settingGp_, fieldNum_, int32Value_)	\
	(iNetError) iNet((iNetUINT8) netNum_DRIVER, (iNetUINT8) deviceNum_DRIVER /* 0 */,  \
		(iNetUINT8) moduleNum_1stModule /* 1 */, (iNetUINT8)  chanNum_, \
		(iNetINT16) settingGp_, (iNetINT16) fieldNum_, \
		(iNetINT16) (((gTemp32_setd = (int32Value_)) != 2191251245L) ? intention_setValue /* 2 */ : intention_setValue), \
		(iNetUINT8) iNetDT_INT32, &gTemp32_setd)

/* -------------------------------------------------------------------------------------
 *	FUNCTION  	PutEnabledChInDigitizeList()		CLASS: BASIC DIGITIZE
 *
 *	PURPOSE 	Place enabled channels in the digitize list.
 *				
 *	HISTORY		4/21/97  created - gsw
 * ----------------------------------------------------------------------------------- */
iNetError /* iNetINT16 */ EXT_FUNC_TYPE PutEnabledChInDigitizeList(
	iNetFLT32 *sampleRateP,				//	we load with the sample-rate of the 1st controller
	iNetINT32 *pointsPerScanP,			//	we load with the pts-per-scan of the 1st controller
	iNetINT32 *numChToDigitizeP)
{
	iNetUINT8 chNum, netNum, devNum, modNum;
	iNetINT16 numberOfDevices, numberOfChannels, numberOfNetworks;
	iNetINT16 numberOfModules, turnOn;
	iNetError e = iNetErr_None;
	Boolean isEnabledForDigitizing;
	
	//	--------------------------------------------------------------
	//	Tell it to stop digitizing (if it was digitizing).
	
	e = Stop_iNet_Digitizing();			//	make sure we are not digitizing
	if (e)	{ goto Exit; }				//	if we got an error, bail out
	
	gWeAreDigitizing = FALSE;			//	specify that we are not digitizing

	gNumDigiChans = 0;					//	set the # of channels in the digitize
										//	channel list to 0.

										//	return the sample rate.
	if (sampleRateP != 0L)  {   *sampleRateP    = GET_SAMPLE_RATE_IN_HZ( netNum_1stController ); }

										//	return the pts-per-scan
	if (pointsPerScanP != 0L) { *pointsPerScanP = GET_POINTS_PER_SCAN( netNum_1stController ); }
	
	//	--------------------------------------------------------------
	//	Scan through all channels and with a channel that had been enabled
	//	for digitizing, add that the to the DigitizeChannelList.
	
										//	get # of instruNet Networks in this Computer
	numberOfNetworks = (short) GET_NUMBER_OF_NETWORKS();	
	if ((e = Get_iNet_Error()) != iNetErr_None)	{ goto Exit; }
		
	if (numberOfNetworks > 0)	{
										//	scan through all Networks
		for (netNum = 1; netNum <= numberOfNetworks; netNum++)		{
										
										//	get # of Devices on this Network
			numberOfDevices = (short) GET_NUMBER_OF_DEVICES(netNum);	
			if ((e = Get_iNet_Error()) != iNetErr_None)	{ goto Exit; }

										//	scan through all Devices
			for (devNum = 1; devNum <= numberOfDevices; devNum++)		{
										
										//	get # of Modules in this Device
				numberOfModules = (short) GET_NUMBER_OF_MODULES(netNum, devNum);	
				if ((e = Get_iNet_Error()) != iNetErr_None)	{ goto Exit; }

										//	scan through all Modules
				for (modNum = 1; modNum <= numberOfModules; modNum++)		{
										
										//	get # of Channels in this Module
					numberOfChannels = (short) GET_NUMBER_OF_CHANNELS(netNum, modNum, devNum);	
					if ((e = Get_iNet_Error()) != iNetErr_None)	{ goto Exit; }

										//	scan through Channels
					for (chNum = 1; chNum <= numberOfChannels; chNum++)		{

										//	if ch is enabled for digitizing
						e = iNet(netNum, devNum, modNum, chNum, sgt_Display /* -7 */,
							fldNum_Display_dispOnOff /* 1 */, intention_getValue /* 8006 */,
							iNetDT_INT16, (void *) &turnOn);
						if (e) { return e; }
					
						isEnabledForDigitizing = (Boolean) (turnOn == ion_gOnOffPopup_On /* 1 */);

										//	if ch is enabled for digitizing
						if (isEnabledForDigitizing)	{

										//	add this channel to the list of
										//	digitize channels, and tell it to
										//	us the user buffer.
							e = AddToChannelList(netNum, devNum, modNum, chNum,
										0L /* bufferP */, 0L /* bufferSizeBytes */);
										
							if (e != iNetErr_None)	{ goto Exit; }
						}
					}
				}
			}
		}
	}
	
Exit:	
										//	get the # of pts enable for digitizing
	if (numChToDigitizeP != 0L) { *numChToDigitizeP = gNumDigiChans; }
	
	return e;
}

		short /* iNetError */ EXT_FUNC_TYPE PutEnabledChInDigitizeList_generic(
			double *sampleRate_doubleP, long *pointsPerScanP, long *numChToDigitizeP)
		{
			iNetFLT32 sampleRate_flt32;
			
			iNetError e = PutEnabledChInDigitizeList(&sampleRate_flt32, pointsPerScanP, numChToDigitizeP);
				
			if (sampleRate_doubleP != 0L)  { *sampleRate_doubleP  = sampleRate_flt32; }
			
			return (short) e;
		}


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	ResetChannelList()		CLASS: BASIC DIGITIZE
 *
 *	PURPOSE 	Resets the list of digitize channels (i.e. sets it's length to 0).
 *				
 *	HISTORY		4/21/97  created - gsw
 * ----------------------------------------------------------------------------------- */
iNetError /* iNetINT16 */ EXT_FUNC_TYPE ResetChannelList()
{					
	gNumDigiChans = 0;					//	set the # of channels in the digitize
										//	channel list to 0.

	Stop_iNet_Digitizing();				//	make sure we are NOT digitizing
							
	gWeAreDigitizing = FALSE;			//	specify that we are not digitizing

										//	disable all digitize channels
										//	(kills their buffers, and disables
										//	digitizing for all channels).
	return DisableAllChannelsForDigitizing_local();
}
										//	generic only uses {short, long, double}
		short /* iNetError */ EXT_FUNC_TYPE ResetChannelList_Generic() { return ResetChannelList(); }


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	AddToChannelList()		CLASS: BASIC DIGITIZE
 *
 *	PURPOSE 	Appends the channel to the list of digitize channels, and
 *				also specifies where it's data is to be stored.
 *				
 *	HISTORY		4/21/97  created - gsw
 * ----------------------------------------------------------------------------------- */
iNetError /* iNetINT16 */ EXT_FUNC_TYPE AddToChannelList(
	iNetUINT8 netNum,					//	NETWORK number = {0...numNetworks}, 0 is Driver,
	iNetUINT8 devNum,					//	DEVICE number {0...numDevices}, 0 = "Controller", 1 = 1st device on network
	iNetUINT8 modNum, 					//	MODULE number within a hardware DEVICE {1...32}.
										//	Many devices have only 1 module.
	iNetUINT8 chNum, 					//	Hardware CHANNEL number {1...32}. Each device contains
										//	a number of channels, each of which has it's own
										//	channel number.
	iNetFLT32 *bufferP,					//	specify the buffer address where digitize 
										//	data is stored; or pass 0L to if you
										//	want instruNet to maintain a ram buffer [and then
										//	you can call GetChannelBuffer() to find out where
										//	it is].
	iNetUINT32 bufferSizeBytes)			//	Specify the buffer size in bytes.  instruNet stores
										//	data as 4byte floats, therefore make sure you take the
										//	number of points per scan and multiply by
										//	4 to get the number of bytes needed for the scan buffer.
{
	iNetINT16 thisChExists, turnOn;
	iNetError e;
										//	make sure they don't give us a buffer that is empty
	if (bufferP != 0L  iNetAnd  bufferSizeBytes == 0L) { return iNetErr_General; }
										
										//	see if channel exists
	e = iNet(netNum, devNum, modNum, chNum,1,1,intention_ThisExists_Channel /* 8006 */, iNetDT_INT16, &thisChExists);
	if (e) { return e; }
										//	if channel doesn't exist, bail out
	if (!thisChExists) { return iNetErr_ChannelNumOutOfRange;}

										//	make sure we don't overflow our list of channels
	if (gNumDigiChans > MAX_NUM_DIGI_CH) { return iNetErr_General; }
	
										//	remember this channel address
	gChAddr[ gNumDigiChans ].netNum 	= netNum;
	gChAddr[ gNumDigiChans ].devNum	= devNum;
	gChAddr[ gNumDigiChans ].modNum	= modNum;
	gChAddr[ gNumDigiChans ].chNum	= chNum;

	gBufferStartAddrP[ gNumDigiChans ]	= 0L;
	gNumPointsToPull[  gNumDigiChans ]	= 0L;
	gScanNumberToPull_base1[ gNumDigiChans ] = 0L;
	gPointToPullindex_base1[ gNumDigiChans ] = 0L;

										//	remember if we are using the user's buffer
										//	or the driver's buffer
	gUseDriverRamBuffer[ gNumDigiChans ] = (Boolean) (bufferP == 0L);		

										//	get channel direction (a/d, d/a, both..)	gsw 10/29/97
										//	{dir_hdwr_input_circBuf, dir_hdwr_output_circBuf, dir_hdwr_inAndOut_circBuf}
	g_Channel_dType [ gNumDigiChans ] = (char) GET_CHANNEL_CATEGORY(netNum, devNum, modNum, chNum);
	e = Get_iNet_Error_local();
	if (e) { return e; }
										
										//	if using the USER's ram buffer for digitized data...
	if ( ! gUseDriverRamBuffer[ gNumDigiChans ] )	{			

										//	set the user buffer address
		e = iNet(netNum, devNum, modNum, chNum, sgt_UserRamBuffer /* -16 */,
			fldNum_UserBuffer_UserBufferAddr /* 2 */, intention_setValue /* 8006 */,
			iNetDT_UINT32, (void *) &bufferP);
		if (e) { return e; }
										//	remember the buffer location
		gBufferStartAddrP[ gNumDigiChans ] = bufferP;
										//	set the user buffer size
		e = iNet(netNum, devNum, modNum, chNum, sgt_UserRamBuffer /* -16 */,
			fldNum_UserBuffer_UserPtrSizeInBytes /* 3 */, intention_setValue /* 8006 */,
			iNetDT_UINT32, (void *) &bufferSizeBytes);
		if (e) { return e; }
										//	enable digitizing into this buffer
		turnOn = (iNetINT32) ion_gOnOffPopup_On /* 1 */;
		e = iNet(netNum, devNum, modNum, chNum, sgt_UserRamBuffer /* -16 */,
			fldNum_UserBuffer_DigitizeOnOff /* 1 */, intention_setValue /* 8006 */,
			iNetDT_INT16, (void *) &turnOn);
		if (e) { return e; }
	}
	else	{							//	use work with DRIVER's ram buffer

										//	mark this Channel for digitizing
										//	(the driver will automatically
										//	allocate memory for this channel
										//	when we go to digitize).
		e = ENABLE_CHANNEL_DIGITIZATION(netNum, devNum, modNum, chNum);
		if (e) { return e; }
	}

	gNumDigiChans++;					//	log that we added this to the list 
			  
	return e;
}
										//	generic only uses {short, long, double}
		short /* iNetError */ EXT_FUNC_TYPE AddToChannelList_Generic(
			short netNum, short devNum, short modNum, short chNum,
			long * /* (iNetFLT32 *) */ bufferP, long bufferSizeBytes)
		{
			return AddToChannelList((iNetUINT8) netNum,  (iNetUINT8) devNum, (iNetUINT8) modNum,
				(iNetUINT8) chNum, (iNetFLT32 *) bufferP, (iNetUINT32) bufferSizeBytes);
		}
										//	generic only uses {short, long, double}
										//	specifies a DRIVER RAM BUFFER, gsw 8/20/97
		short /* iNetError */ EXT_FUNC_TYPE AddToChannelList_DB_Generic(
			short netNum, short devNum, short modNum, short chNum, long bufferSizeBytes)
		{
			return AddToChannelList((iNetUINT8) netNum,  (iNetUINT8) devNum, (iNetUINT8) modNum,
				(iNetUINT8) chNum, (iNetFLT32 *) 0L /* driver buffer */, (iNetUINT32) bufferSizeBytes);
		}


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	ThisIsAlreadyInChannelList()		CLASS: BASIC DIGITIZE
 *
 *	PURPOSE 	Returns TRUE if the {netNum, devNum, modNum, devNum} is 
 *				already in the list of channels to be digitized; false otherwise.
 *				
 *	HISTORY		9/31/97  created - gsw
 * ----------------------------------------------------------------------------------- */
iNetINT16 EXT_FUNC_TYPE ThisIsAlreadyInChannelList(
	iNetUINT8 netNum,					//	NETWORK number = {0...numNetworks}, 0 is Driver,
	iNetUINT8 devNum,					//	DEVICE number {0...numDevices}, 0 = "Controller", 1 = 1st device on network
	iNetUINT8 modNum, 					//	MODULE number within a hardware DEVICE {1...32}.
	iNetUINT8 chNum) 					//	Hardware CHANNEL number {1...32}. Each device contains
{
	register int digChNum;
										//	make sure we don't overflow our list of channels
	if ((gNumDigiChans > 0L) iNetAnd (gNumDigiChans <= MAX_NUM_DIGI_CH)) {
		
										//	remember this channel address
		for (digChNum = 0; digChNum < gNumDigiChans; digChNum++)	{

										//	if this channel is in the digitize list...
			if (	(gChAddr[ digChNum ].chNum	== chNum)		iNetAnd
					(gChAddr[ digChNum ].devNum	== devNum)	iNetAnd
					(gChAddr[ digChNum ].modNum	== modNum)	iNetAnd
					(gChAddr[ digChNum ].netNum 	== netNum)	)	{
					
				return (iNetINT16) TRUE;
			}
		}
	}
	
	return (iNetINT16) FALSE;
}


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	Set_iNet_TIMING_Parameters()	CLASS: BASIC DIGITIZE
 *
 *	PURPOSE 	Sets the instruNet TIMING Parameters.  All networks are set the
 *				same.
 *				
 *	HISTORY		11/30/95  created - gsw
 * ----------------------------------------------------------------------------------- */

iNetError /* iNetINT16 */ EXT_FUNC_TYPE Set_iNet_TIMING_Parameters(
	iNetFLT32 samplesPerSecond,			//	sample rate (points per second per channel)
	iNetINT32 pointsPerScan,			//	number of points in 1 scan, for 1 channel
	iNetINT32 numberOfScans,			//	number of scans that are digitized when the
										//	start button is pressed
	iNetINT16 stripChartRecorder,		//	if TRUE (1) we do Strip Chart Recorder work with
										//	Continuous Scans, otherwise if FALSE (0) we do
										//	Oscilloscope mode (non-Coninuous Scans)
	iNetINT16 saveDataOption,			//	Save Data option = { ion_gSaveDataPopup_Off,
										//	ion_gSaveDataPopup_ToRamBuffer, ion_gSaveDataPopup_ToDisk,
										//	ion_gSaveDataPopup_UserControl}
	iNetINT16 showAlertOnOverflow,		//	if TRUE (1), the driver will show an alert if
										//	a data buffer overflows; otherwise if FALSE (0) 
										//	it just keeps going if there is an overflow.
	iNetINT16 switchAccurately)			//	if TRUE (1), channel switching should be accurate;
										//	otherwise, it is less accurate, yet faster.
{
	iNetError e = iNetErr_None;
	iNetUINT8 netNum;
	iNetINT16 showAlert;
										//	update # of iNet Networks on this Computer
										//	(this could have changed if we just RESET
										//	the network), gsw 7/1/02
	gNumberOfNetworks = (iNetINT16) GET_NUMBER_OF_NETWORKS();

										//	set the Save Data option
	SET_INT32_DRVR(driver_ChanNum_RecordOptions, sgt_RecordOptions, fldNum_RecordOptions_digitizeInto, saveDataOption);
	if (e)	{ goto Exit; }

										//	set showAlert on overflow policy
	showAlert = (iNetINT16) ((showAlertOnOverflow != 0) ? ion_gOnOffPopup_On : ion_gOnOffPopup_Off);
	SET_INT32_DRVR(driver_ChanNum_RecordOptions, sgt_RecordOptions, fldNum_RecordOptions_overflowAlert, showAlert);
	if (e)	{ goto Exit; }

	if (gNumberOfNetworks > 0)	{
										//	set timing params for each network	
		for (netNum = 1; netNum <= gNumberOfNetworks; netNum++)		{

										//	set sample rate
			e = SET_FLT32_DIGI(netNum, sgt_Timing, fldNum_Timing_sampleRate, samplesPerSecond);
			if (e)	{ goto Exit; }
										//	set the points per scan
			e = SET_INT32_DIGI(netNum, sgt_Timing, fldNum_Timing_ptsPerScan, pointsPerScan);
			if (e)	{ goto Exit; }
										//	set the number of scans
			e = SET_INT32_DIGI(netNum, sgt_Timing, fldNum_Timing_noOfScans, numberOfScans);
			if (e)	{ goto Exit; }
										//	set Strip Chart Recorder or Oscilloscope mode
			e = SET_INT32_DIGI(netNum, sgt_Timing, fldNum_Timing_scanMode, 
				((stripChartRecorder != 0) ? ion_gScanModePopup_StripChart : ion_gScanModePopup_Oscilloscope));
			if (e)	{ goto Exit; }
										//	set the channel switching mode
			e = SET_INT32_DIGI(netNum, sgt_Timing, fldNum_Timing_switching, 
				(switchAccurately != 0) ? ion_gSwitchingPopup_Accurate : ion_gSwitchingPopup_Fast);
			if (e)	{ goto Exit; }
		}
	}

Exit:

	if (e)	{							//	if we got an error, show an alert
		Show_ALERT_if_hit_iNet_Error_local(e);
	}
	return e;
}
										//	generic only uses {short, long, double}
		short /* iNetError */ EXT_FUNC_TYPE Set_iNet_TIMING_Parameters_Generic(double samplesPerSecond, 
			long pointsPerScan, long numberOfScans, short stripChartRecorder, short saveDataOption,
			short showAlertOnOverflow, short switchAccurately)
		{
			return Set_iNet_TIMING_Parameters((iNetFLT32) samplesPerSecond, pointsPerScan,
				numberOfScans, stripChartRecorder, saveDataOption, showAlertOnOverflow, 
				switchAccurately );
		}

/* -------------------------------------------------------------------------------------
 *	FUNCTION  	Set_iNet_TRIGGER_Parameters()	CLASS: BASIC DIGITIZE
 *
 *	PURPOSE 	Sets the instruNet TRIGGER Parameters.  This function sets the trigger
 *				for all networks to the same channel in each network.
 *				
 *	HISTORY		11/30/95  created - gsw
 * ----------------------------------------------------------------------------------- */
iNetError /* iNetINT16 */ EXT_FUNC_TYPE Set_iNet_TRIGGER_Parameters(
	iNetINT16 triggerMode,				//	trigger mode = {ion_TriggerMode_Off,
										//	ion_TriggerMode_Auto, ion_TriggerMode_Norm}
	iNetFLT32 triggerThreshold,			//	trigger threshold in engineering units
	iNetINT16 triggerSlope,				//	trigger slope = {ion_Rising, ion_Falling}
	iNetFLT32 hysteresisEu,				//	hysteresis, in eu
	iNetChannelAddr *triggerSourceP)	//	trig source Channel Address = {netNum, devNum, modNum, chNum}
										//	NETWORK number = {0...numNetworks}, 0 is Driver,
{
	iNetError e = iNetErr_None;
	iNetUINT8 netNum, netNum_trig = 1, deviceNum_trig = 1, moduleNum_trig = 1, chanNum_trig = 1;
	
	if (triggerSourceP != 0L)	{
		netNum_trig 	= triggerSourceP->netNum;
		deviceNum_trig 	= triggerSourceP->devNum;
		moduleNum_trig 	= triggerSourceP->modNum;
		chanNum_trig 	= triggerSourceP->chNum;
	}
	if (gNumberOfNetworks > 0)	{
										//	set trigger params for each network	
		for (netNum = 1; netNum <= gNumberOfNetworks; netNum++)		{

										//	set trigger mode
			e = SET_INT32_DIGI(netNum, sgt_Trigger, fldNum_Trigger_triggerModePop, triggerMode);
			if (e)	{ goto Exit; }
										//	set trigger threshold engineering units
			e = SET_FLT32_DIGI(netNum, sgt_Trigger, fldNum_Trigger_thresholdEu, triggerThreshold);
			if (e)	{ goto Exit; }
										//	set the trigger slope
			e = SET_INT32_DIGI(netNum, sgt_Trigger, fldNum_Trigger_slopeRisFalPop, triggerSlope);
			if (e)	{ goto Exit; }
										//	set pre trigger seconds
			e = SET_FLT32_DIGI(netNum, sgt_Trigger, fldNum_Trigger_hysteresis, hysteresisEu);
			if (e)	{ goto Exit; }
										//	set trigger source netNum
			e = SET_INT32_DIGI(netNum, sgt_Trigger, fldNum_Trigger_srcNet, 		netNum_trig);
			if (e)	{ goto Exit; }
										//	set trigger source devNum
			e = SET_INT32_DIGI(netNum, sgt_Trigger, fldNum_Trigger_srcDevice, 	deviceNum_trig);
			if (e)	{ goto Exit; }
										//	set trigger source modNum
			e = SET_INT32_DIGI(netNum, sgt_Trigger, fldNum_Trigger_srcModule, 	moduleNum_trig);
			if (e)	{ goto Exit; }
										//	set trigger source chNum
			e = SET_INT32_DIGI(netNum, sgt_Trigger, fldNum_Trigger_srcChannel, 	chanNum_trig);
			if (e)	{ goto Exit; }
		}
	}

Exit:

	if (e)		{						//	if we got an error, show an alert
		Show_ALERT_if_hit_iNet_Error_local(e);
	}

	return e;
}
										//	generic only uses {short, long, double}
		short /* iNetError */ EXT_FUNC_TYPE Set_iNet_TRIGGER_Parameters_Generic( short triggerMode,
			double triggerThreshold, short triggerSlope, double hysteresisEu,
			short netNum_trig, short deviceNum_trig, short moduleNum_trig, short chanNum_trig)
		{
			iNetChannelAddr triggerSource;
			
			triggerSource.netNum 	= (iNetUINT8) netNum_trig;
			triggerSource.devNum = (iNetUINT8) deviceNum_trig;
			triggerSource.modNum = (iNetUINT8) moduleNum_trig;
			triggerSource.chNum 	= (iNetUINT8) chanNum_trig;
			
			return Set_iNet_TRIGGER_Parameters(triggerMode, (iNetFLT32) triggerThreshold,
				triggerSlope, (iNetFLT32) hysteresisEu, &triggerSource);
		}

/* -------------------------------------------------------------------------------------
 *	FUNCTION  	Stop_iNet_Digitizing()		CLASS: BASIC DIGITIZE
 *
 *	PURPOSE 	Tells instruNet to stop digitizing.
 *				
 *	HISTORY		9/25/97  created - gsw
 * ----------------------------------------------------------------------------------- */
short /* iNetError */ EXT_FUNC_TYPE Stop_iNet_Digitizing()
{
										//	press the STOP button to stop digitizing
	iNetError e = SET_INT32_DRVR(driver_ChanNum_PushButton, sgt_General,
		fldNum_General_valueEu, iNet_ButtonID_Record_Stop);

	gWeAreDigitizing = FALSE;			//	specify that we are not digitizing
	
	return e;
}

/* -------------------------------------------------------------------------------------
 *	FUNCTION  	DigitizeChannelList()						CLASS: BASIC DIGITIZE
 *
 *	PURPOSE 	Digitizes the channels in the channel list [i.e. that 
 *				were added to the list by AddToChannelList()].
 *				
 *	HISTORY		4/20/97  created - gsw
 * ----------------------------------------------------------------------------------- */
iNetError /* iNetINT16 */ EXT_FUNC_TYPE DigitizeListOfChannels(
	iNetINT16 allowMouseDownToStopDigitize,
										//	if TRUE (1), we allow a mouse down to stop the
										//	digitize process; otherwise, if FALSE (0), will
										//	will run till finished.
	iNetINT16 processOneScanAtATime,	//	if TRUE (1), we access digitized data 1 complete
										//	Scan at a time; otherwise if FALSE (0), we process
										//	little segments of data, as they become available.
	iNetUINT32 *last_scanNum_base1_pulled_inFull_oneCtlr)
										//	last scan # (base 1) that we pulled (e.g. a/d) in 'netNum_forScanInfo'
										//	full from controller ram buffer TO user or
										//	driver ram buffer when Service_All_iNet_Digitize_Buffers()
										//	was called.
{
	iNetError e;
	iNetINT16 theNextDataAccessIsTheLast_ofEntireDigitization, digChNum;
	iNetINT16 theNextDataAccessWillPullToTheEndOfAScan, controllerIsFinishedWithDigitization;
	iNetUINT32 last_scanNum_base1_pulled_inFull, last_scanNum_base1_pushed_inFull;
	Boolean waitingForEndOfScan = (Boolean) processOneScanAtATime;
	iNetINT16 weAreStuckWaitingForATurnedOffController = 0;
	
	if (last_scanNum_base1_pulled_inFull_oneCtlr != 0L) { *last_scanNum_base1_pulled_inFull_oneCtlr = 0L; }
	
										//	if there are no channels in the digitize
										//	channel list, then bail out
	if (gNumDigiChans < 1) { return iNetErr_General; }
	
	//	-----------------------------------
	//	TELL IT TO START DIGITIZING.

	if ( ! gWeAreDigitizing )	{		//	if we are not digitizing yet...
	
										//	Press the START button to tell 
										//	instruNet to begin digitizing.
		e = SET_INT32_DRVR(driver_ChanNum_PushButton, sgt_General, fldNum_General_valueEu, iNet_ButtonID_Record_Start);
		if (e)	{ goto Exit; }			//	if we got an error, bail out
	
										//	zero out our segment description variables
		for (digChNum = 0; digChNum < gNumDigiChans; digChNum++)	{
			gPointToPullindex_base1[ digChNum ] = 0L;
			gNumPointsToPull[ digChNum ] = 0L;
			gScanNumberToPull_base1[ digChNum ] = 0L;
		}
		
		gWeAreDigitizing = TRUE;
	}

	do 	{								//	loop until the digitization is finished..
	
										//	-----------------------------------
										//	SERVICE instruNet's DIGITIZE BUFFERS
										//
										//	It is ABSOLUTELY NECESSARY to let your
										//	computer's processor service 
										//	instruNet digitizing buffers periodicaly
										//	(10 to .3 times per second)
										//	while digitizing.  This is done
										//	by continuously calling 
										//	Service_All_iNet_Digitize_Buffers() in a loop.
		e = Service_All_iNet_Digitize_Buffers(
				gChAddr[0].netNum,
				&theNextDataAccessIsTheLast_ofEntireDigitization,
				&theNextDataAccessWillPullToTheEndOfAScan,
				&controllerIsFinishedWithDigitization,
				&last_scanNum_base1_pulled_inFull,
				&last_scanNum_base1_pushed_inFull);
	
		if (e)	{ goto Exit;}			//	if we got an error, bail out
		
		if (last_scanNum_base1_pulled_inFull_oneCtlr != 0L) {
			*last_scanNum_base1_pulled_inFull_oneCtlr = last_scanNum_base1_pulled_inFull;
		}
										//	If we are to process the data little
										//	segments at a time, or we are processing
										//	1 scan at at time and we have that 1 
										//	scan available in the buffers, then
										//	go process the data.
		if (!processOneScanAtATime || theNextDataAccessWillPullToTheEndOfAScan)	{
			
			waitingForEndOfScan = FALSE;
			
										//	service the channels that have been
										//	turned on for digitizing
			for (digChNum = 0; digChNum < gNumDigiChans; digChNum++)	{

										//	-----------------------------------
										//	READ DIGITIZED DATA
										//
										//	Here we get an address in memory where
										//	the most recent digitized data resides.
										//
										//	'gBufferStartAddrP[]'
										//	
										//		iNetFLT32 data buffer that is
										//		1 Scan long
										//	
										//	'gPointToPullindex_base1'
										//
										//		new data is available starting at
										//		gBufferStartAddrP[gPointToPullindex_base1 - 1]
										//
										//	'gNumPointsToPull'
										//
										//		number of new data points that are
										//		available
										//
										//	'gScanNumberToPull_base1'
										//
										//		scan number {1...numScans} of the
										//		scan that we are currently accessing.
				e = Access_Digitized_Data_In_Ram_Buffer(
										&gChAddr[ digChNum ],
										gUseDriverRamBuffer[ digChNum ],
										&gBufferStartAddrP[ digChNum ],
										&gPointToPullindex_base1[ digChNum ],
										&gNumPointsToPull[ digChNum ],
										&gScanNumberToPull_base1[ digChNum ]);

				if (e)	{ goto Exit;}	//	if we got an error, bail out
			}
		}
										//	if we are to allow a mouse
										//	down to stop the digitizing...
		if (allowMouseDownToStopDigitize)	{
				
			if (MOUSE_BUTTON_IS_DOWN()){//	if the mouse button is down...
		
				e = Stop_iNet_Digitizing();	//	press the STOP button to stop digitizing
				if (e)	{ goto Exit; }	//	if we got an error, bail out

				gWeAreDigitizing =FALSE;//	specify that we are no longer digitizing			
				
										//	note that we have stopped
				theNextDataAccessIsTheLast_ofEntireDigitization = TRUE;
			}
		}
		
		if (waitingForEndOfScan)	{	//	if we are to wait till the end of the scan...
										//	bug fix to avoid infinite loop, gsw 9/25/97
		
										//	if the controller is not digitizing,	gsw 9/25/97
										//	then we might get into an infinite loop
			if (! CONTROLLER_IS_DIGITIZING( netNum_1stController ))	{
				weAreStuckWaitingForATurnedOffController++;
			}
										//	if we are stuck waiting for data from an
										//	off controller...			gsw 9/25/97
			if (weAreStuckWaitingForATurnedOffController > 5)	{
				gWeAreDigitizing =FALSE;//	note that we are no longer digitizing
										//	note that we have stopped
				theNextDataAccessIsTheLast_ofEntireDigitization = TRUE;
				waitingForEndOfScan = TRUE;
			}
		}
										//	if we are to return one full scan
										//	then sit here till we have it
	} while (waitingForEndOfScan);		//	!theNextDataAccessIsTheLast_ofEntireDigitization  iNetAnd  !controllerIsFinishedWithDigitization);
										
Exit:
										//	if we are finished with the digitizing session...
	if (theNextDataAccessIsTheLast_ofEntireDigitization)	{
		gWeAreDigitizing = FALSE;		//	note that we are no longer digitizing
	}

	if (e)		{						//	if we got an error, show an alert
		Show_ALERT_if_hit_iNet_Error_local(e);
	}

	return e;
}
										//	generic only uses {short, long, double}
		short /* iNetError */ EXT_FUNC_TYPE DigitizeListOfChannels_Generic( short allowMouseDownToStopDigitize,
			short processOneScanAtATime, long *last_scanNum_base1_pulled_inFull_oneCtlr)
		{
			return DigitizeListOfChannels( allowMouseDownToStopDigitize, processOneScanAtATime,
				(iNetUINT32 *) last_scanNum_base1_pulled_inFull_oneCtlr );
		}


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	GetDigitizedSegment()	CLASS: BASIC DIGITIZE
 *
 *	PURPOSE 	While digitizing, this will give you a segment of digitized data.
 *				It will pass you an address to the first FLT32 point in the
 *				buffer (i.e. beginning of the entire buffer), the index into the
 *				buffer to begin reading data, and the # of points to pull.
 *
 *				1st point to read
 *					= (*bufferStartAddrP) + *pointToPullindex_base1 - 1
 *
 *				last point to read
 *					= (*bufferStartAddrP) + *pointToPullindex_base1 + *numPointsToPull - 2
 *				
 *				The buffer is 1 scan long, and the scan number {1...numScans}
 *				that we are reading is returned in '*scanNumberToPull_base1'.
 *
 *				The # of times the iNet circular buffers overflowed is returned
 *				in '*totalNumOfTimesWeOverflowed'.
 *				
 *	HISTORY		4/20/97  created - gsw
 * ----------------------------------------------------------------------------------- */
iNetError /* iNetINT16 */ EXT_FUNC_TYPE GetDigitizedSegment(
	iNetINT16 digChNum,					//	index into the digitize channel list
										//	= {0.... numDigitizeChannels-1}
	iNetFLT32 **bufferStartAddrP,		//	we will load this with the address to the 
										//	first FLT32 point in the buffer
	iNetUINT32 *pointToPullindex_base1,	//	we will load this with the {1... numPtsPerScan)
										//	index of the first point of the segment that we need
										//	to process (base 1 index).
	iNetUINT32 *numPointsToPull,		//	we will load this with the # of points to to pull
										//	out of the buffer
	iNetUINT32 *scanNumberToPull_base1)	//	we load this with the scan number {1...numScans}
										//	of the scan that we are currently pulling (base 1).
{										//	make sure we don't overflow our list of channels
	if ((digChNum > gNumDigiChans)  		iNetOr
		(gNumDigiChans > MAX_NUM_DIGI_CH)	iNetOr
		(digChNum > MAX_NUM_DIGI_CH)) { return iNetErr_General; }

										//	return the last digitized segment
	if (bufferStartAddrP != 0L) 	 { *bufferStartAddrP 	   = gBufferStartAddrP[ digChNum ]; }
	if (pointToPullindex_base1 !=0L) { *pointToPullindex_base1 = gPointToPullindex_base1[ digChNum ]; }
	if (numPointsToPull != 0L)		 { *numPointsToPull 	   = gNumPointsToPull[ digChNum ]; }
	if (scanNumberToPull_base1 !=0L) { *scanNumberToPull_base1 = gScanNumberToPull_base1[ digChNum ]; }
	
	return iNetErr_None;
}

		short /* iNetError */ EXT_FUNC_TYPE GetDigitizedSegment_Generic(short digChNum,
			/* (iNetFLT32 **) */ void **bufferStartAddrP, long *pointToPullindex_base1,
			long *numPointsToPull, long *scanNumberToPull_base1)
		{
			return GetDigitizedSegment(digChNum, (iNetFLT32 **) bufferStartAddrP,
				(iNetUINT32 *) pointToPullindex_base1, (iNetUINT32 *) numPointsToPull,
				(iNetUINT32 *) scanNumberToPull_base1);
		}

		short /* iNetError */ EXT_FUNC_TYPE GetDigitizedSegment_Vee(short digChNum,
			/* (iNetFLT32 **) */ long *bufferStartAddrP, long *pointToPullindex_base1,
			long *numPointsToPull, long *scanNumberToPull_base1)
		{
			return GetDigitizedSegment(digChNum, (iNetFLT32 **) bufferStartAddrP,
				(iNetUINT32 *) pointToPullindex_base1, (iNetUINT32 *) numPointsToPull,
				(iNetUINT32 *) scanNumberToPull_base1);
		}

		
/* -------------------------------------------------------------------------------------
 *	FUNCTION  	GetOutputSegmentBuffer()	CLASS: BASIC DIGITIZE
 *
 *	PURPOSE 	While digitizing, this will give you a buffer to place
 *				digitize data, for output channels.
 *
 *				It will pass you an address to the first FLT32 point in the
 *				buffer (i.e. beginning of the entire buffer), the index into the
 *				buffer to begin writing data, and the # of points to push into
 *				the buffer.  For details of how this works, please see HighSpeedOutput()
 *				in file INET_EX1.c.
 *
 *				1st point to written
 *					= (*bufferStartAddrP) + *pointToPushindex_base1 - 1
 *
 *				last point to write
 *					= (*bufferStartAddrP) + *pointToPushindex_base1 + *numPointsToPush - 2
 *				
 *				The buffer is 1 scan long, and the scan number {1...numScans}
 *				that we are writing is returned in '*scanNumberToPush_base1'.
 *				
 *				IF THIS ROUTINE IS NOT CALLED, THEN INSTRUNET WILL TAKE WHAT IS ALREADY
 *				IN THE OUTPUT BUFFER, AND REPEATEDLY OUTPUT IT.  THIS ROUTINE IS 
 *				DESIGNED TO OUTPUT DATA THAT IS GENERATED AS THE DIGITIZING OCCURS,
 *				WHERE INSTRUNET IS FED LITTLE SEGMENTS AT A TIME.  IN THIS CASE,
 *				IF YOU DON'T KEEP IT WILL GIVE YOU A BUFFER UNDERFLOW ERROR.
 *
 *	HISTORY		10/20/98  created - gsw	sssssssss
 * ----------------------------------------------------------------------------------- */

iNetError /* iNetINT16 */ EXT_FUNC_TYPE GetOutputSegmentBuffer(
	iNetINT16 digChNum,					//	index into the digitize channel list
										//	= {0.... numDigitizeChannels-1}
	iNetFLT32 **bufferStartAddrP,		//	we will load this with the address to the 
										//	first FLT32 point in the buffer
	iNetUINT32 *pointToPushindex_base1,	//	we will load this with the {1... numPtsPerScan)
										//	index of the first point of the segment that we need
										//	to process (base 1 index).
	iNetUINT32 *maxNumPointsToPush,		//	we will load this with the max # of points
										//	one could push into the buffer
	iNetUINT32 *scanNumberToPush_base1)	//	we load this with the scan number {1...numScans}
										//	of the scan that we are currently pushing (base 1).
{										//	make sure we don't overflow our list of channels
	
	//	--------------------------------------------------------------------------------
	//	ERROR CHECKING

	if ((digChNum > gNumDigiChans)  		iNetOr
		(gNumDigiChans > MAX_NUM_DIGI_CH)	iNetOr
		(digChNum > MAX_NUM_DIGI_CH)) { return iNetErr_General; }

										//	if this is NOT an output channel tied to a
										//	circular buffer (e.g. d/a)...
	if (g_Channel_dType[ digChNum ] != dir_hdwr_output_circBuf /* 2 */)	{
		return iNetErr_ReadOnlyField;	//	cannot write to this channel
	}

	//	--------------------------------------------------------------------------------
	//	MAKE SURE WE ARE WORKING WITH A NEW ENOUGH iNet32.dll.

	#if (MACHINE_TYPE_ == WINTEL_iNet)	/*	=========== Windows/Intel =========== */	
		
										//	Make sure you are working with iNet32.dll
										//	version >= 1.30.3

		if ((gDLL_ver_majorVer > 1)	||
			(gDLL_ver_majorVer == 1 && gDLL_ver_minorVer > 30)	||
			(gDLL_ver_majorVer == 1 && gDLL_ver_minorVer == 30 && gDLL_ver_subminorVer >= 3))	{

			;							//	high speed output is ok w/ >= 1.30.3
		}
		else	{						//	else, you need a newer dll to do high speed output
			static Boolean alertedLimitation = FALSE;
			if (! alertedLimitation)	{
				alertedLimitation = TRUE;
				Show_Simple_Alert((char *) "High speed output requires instruNet software version >= 1.30.3.");
			}
			return iNetErr_General; 
		}

	#endif

	//	--------------------------------------------------------------------------------
	//	GET OUTPUT SEGMENT BUFFER

	return ProcessBufferData(&gChAddr[ digChNum ], gUseDriverRamBuffer[ digChNum ],
				bufferStartAddrP, pointToPushindex_base1, maxNumPointsToPush,
				scanNumberToPush_base1, intention_getOutSegmentBuffer, 0L);
}	


		
/* -------------------------------------------------------------------------------------
 *	FUNCTION  	SubmitOutputSegment()	CLASS: BASIC DIGITIZE
 *
 *	PURPOSE 	To output generated data while digitizing, one must:
 *
 *				#1)	call Stop_iNet_Digitizing() to stop any previous digitizing.
 *				#2)	call Set_iNet_TIMING_Parameters() to set the timing parameters.
 *				#3) call SET_UP_DIGITIZATION() to set up internal buffers.
 *				#4)	call GetOutputSegmentBuffer() to get the location of the place
 *					to put the output data.
 *				#5) move some data into this buffer (fill as much of it as you want).
 *				#6) call SubmitOutputSegment() to tell it how much you filled.
 *				#7)	tell it to start digitizing.
 *
 *				Please see GetOutputSegmentBuffer() for more details.
 *
 *	HISTORY		10/20/98  created - gsw
 * ----------------------------------------------------------------------------------- */

iNetError /* iNetINT16 */ EXT_FUNC_TYPE SubmitOutputSegment(
	iNetINT16 digChNum,					//	index into the digitize channel list
										//	= {0.... numDigitizeChannels-1}
	iNetUINT32 numPointsToPush)			//	this is the # of pts loaded into the output buffer
{
	return ProcessBufferData(&gChAddr[ digChNum ], gUseDriverRamBuffer[ digChNum ],
				0L, 0L, 0L, 0L, intention_submitOutSegment, numPointsToPush);
}


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	CalcDigiIntegrationTimes()				CLASS: BASIC DIGITIZE
 *
 *	PURPOSE 	Returns the number of channels being integrated in the 'network', and
 *				their total integration time.  This effects the 'network's DSP card's 
 *				max sample rate.
 *
 *				For example, if 3 our of 8 channels in network #2 each integrate for 1ms
 *				(the other 5 do not integrate), then the max sample rate for that DSP card
 *				would be approximately 1 / 3ms = 333s/sec/channel.  In this example,
 *				CalcDigiIntegrationTimes() would return 3 integration channel and 0.003 total
 *				integration time.
 *				
 *	HISTORY		10/23/97	created - gsw
 *				10/27/97	updated - gsw
 * ----------------------------------------------------------------------------------- */
iNetError /* iNetINT16 */ EXT_FUNC_TYPE CalcDigiIntegrationTimes(
	iNetUINT8 netNum,					//	NETWORK number = {0...numNetworks} to analyze
	iNetINT16 *numIntegrationChannels,	//	loaded with the # of channels in digitize list,
										//	that integrate.
	iNetFLT32 *totalIntegrationTimeSecs)//	loaded with the total integration time for this
										//	DSP card.
{
	int digChNum;
	iNetFLT32 integrationTimeSecs;
	iNetError e = iNetErr_None;
	
	*numIntegrationChannels = 0;
	*totalIntegrationTimeSecs = (iNetFLT32) 0.0;

										//	make sure we don't overflow our list of channels
	if ((gNumDigiChans > 0L) iNetAnd (gNumDigiChans <= MAX_NUM_DIGI_CH)) {
		
										//	remember this channel address
		for (digChNum = 0; digChNum < gNumDigiChans; digChNum++)	{

										//	if this channel is in the specified "network"...
			if (gChAddr[ digChNum ].netNum 	== netNum)	{
					
										//	get integration amount for this channel
				e = (iNetError) iNet( gChAddr[ digChNum ].netNum, gChAddr[ digChNum ].devNum,
					gChAddr[ digChNum ].modNum, gChAddr[ digChNum ].chNum,
					sgt_VinHardware, fldNum_Vin_ad_IntegrateSecs, (iNetINT16) intention_getValue,
					(iNetUINT8) iNetDT_FLT32, (void *) &integrationTimeSecs);			 			 
	
										//	if this channel is integrated...
				if (integrationTimeSecs > 0.0)	{
				
					(*numIntegrationChannels) += 1L;
				
					(*totalIntegrationTimeSecs) += (iNetFLT32) integrationTimeSecs;
				}
			}
		}
	}
	
	return (iNetINT16) e;
}


//	=========================== CLASS: BASIC UTILITY ===========================
//	=========================== CLASS: BASIC UTILITY ===========================
//	=========================== CLASS: BASIC UTILITY ===========================

/* -----------------------------------------------------------------------------------
 *	FUNCTION  	Load_instruNet_Driver()		CLASS: BASIC UTILITY
 *
 *				Loads the instruNet Driver file; and if 'reset_instruNet' is
 *				TRUE (1), we reset the instruNet network.
 *
 *	HISTORY		1/27/97	created - gsw
 * --------------------------------------------------------------------------------- */
iNetError /* iNetINT16 */ EXT_FUNC_TYPE Load_instruNet_Driver(
	iNetINT16 forceInstruNetReset,		//	if true (1), we force the resetting of the 
										//	instruNet network (this is not always necessary,
										//	since we automatically reset when the instruNet
										//	driver is opened). gsw 2/18/97
	iNetINT16 *driverIsInstalledOK,		//	loaded with TRUE (1) if driver is installed ok;
										//	[e.g. Win95 'iNet32.DLL' or Mac 'instruNet Driver (ppc)'] 
										//	FALSE (0) otherwise.
	iNetINT16 *numNetworks)				//	loaded with the number of networks (i.e. controllers
										//	found)
{
	iNetError e;
	Boolean weAreLoadingDriverAndWillNotNeedToForceReset;
	
	gWeTriedToLoadDriver = TRUE;		//	remember that we attempted to load the iNet driver

										//	if we are here for the 1st time...
	weAreLoadingDriverAndWillNotNeedToForceReset = (giNet_Main_EntryPtr_ == (void *) 0L);

										//	*** This will load the driver since it is
										//	*** the 1st call to the iNet() subroutine.
	gDriverIsInstalledOk = (iNetINT16) DRIVER_IS_INSTALLED();
	
	e = Get_iNet_Error_local();			//	get error code
									
										//	if we failed to load, then exit
	if (e != iNetErr_None) { goto Exit; }
		
										//	else driver load had a problem, then exit 
	if ( !gDriverIsInstalledOk ) { e = iNetErr_CouldNotLocateDriverFile; goto Exit; }

										//	get # of iNet Networks in this Computer
	gNumberOfNetworks = (iNetINT16) GET_NUMBER_OF_NETWORKS();
	
	if (forceInstruNetReset != 0)	{	//	if we need to force the reset...
					
										//	if we did not just load the driver, then
										//	we will need to force the reset (otherwise,
										//	loading the driver will force the reset). gsw 8/1/98
		if ( ! weAreLoadingDriverAndWillNotNeedToForceReset )	{
						
										//	press 'Reset' button in NETWORK page	
			e = PRESS_iNet_BUTTON(iNet_ButtonID_Network_Reset);
			if (e != iNetErr_None) { goto Exit; }
		}
	}
	
Exit:

	if (driverIsInstalledOK != 0L)	{ *driverIsInstalledOK = gDriverIsInstalledOk; }

	if (numNetworks != 0L)	{ *numNetworks = gNumberOfNetworks; }

	return e;
}
										//	generic only uses {short, long, double}
		short /* iNetError */ EXT_FUNC_TYPE Load_instruNet_Driver_Generic(
			short forceInstruNetReset, short *driverIsInstalledOK, short *numNetworks)		
		{
			return Load_instruNet_Driver(forceInstruNetReset, driverIsInstalledOK, numNetworks);
		}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	CloseDriverAndReleaseDriverRam()		CLASS: BASIC UTILITY			
 *
 *	PURPOSE 	This routine closes the instruNet driver and releases the
 *				memory used to hold it. This will delete ram use to hold 
 *				iNet95.vxd, iNetNT.dvr, internal buffers, etc.
 *
 *				ONLY CALL THIS ONCE, AND DON'T CALL iNet() AFTER CALLING
 *				THIS ROUTINE (e.g. CALL THIS WHEN YOUR APPLICATION QUITS).
 *
 *				Exiting an application usually releases the ram used to hold
 *				the driver as well; therefore, this is sometimes unnecessary
 *				when someone is exiting the calling application.
 *
 *				With the Mac Pcmcia iNet-230, and in some other cases,
 *				one MUST call this routine, else the computer will crash
 *				since this driver installs a callback that needs to
 *				be released before exiting. 
 *				
 *	HISTORY		9/8/96	created - gsw
 * --------------------------------------------------------------------------------- */
void EXT_FUNC_TYPE CloseDriverAndReleaseDriverRam()	
{	
	static Boolean weAlreadyClosedTheDriver = FALSE;
	
	//	---------------------------------------------------------
	//	If instruNet driver is already closed, bail out of here.

	if (giNet_Main_EntryPtr_ == (void *) 0L)	{	
		return;							//	bail out
	}
	
	if (weAlreadyClosedTheDriver)	{	//	if we already have closed, get out of here
		return;
	}
	
	//	---------------------------------------------------------
	//	Dispose of all iNet state (e.g. iNet95.vxd, iNetNT.dvr, internal 
	//	buffers, etc). You should only call this ONCE per application
	//	launch (e.g. when your application Quits).

	DISPOSE_OF_ALL_NETWORK_STATE();		//	DON'T CALL iNet() AFTER THIS (since
										//	this shuts down drivers).

	//	---------------------------------------------------------
	//	Free up ram used to hold the instruNet driver itself.
	
#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	============= Macintosh =========== */
	
	if (gRsrcH != 0L)	{				//	if iNet driver code rsrc has been loaded...

		OSErr e;
		DetachResource( gRsrcH );		//	detach rsrc from file, make it independent
		e = ResError();
		
		// gsw 9/5/98: RemoveResource( gRsrcH ); same as RmveResource(), remove file from rsrc map
		// gsw 9/5/98: e = ResError();

		HUnlock ( gRsrcH );				//	unlock the rsrc, gsw 9/5/98 
		e = MemError();
		DisposeHandle( gRsrcH );		//	delete the memory, gsw 9/5/98						
		e = MemError();

		if (gFileRefNum > 0)	{		
			e = FSClose( gFileRefNum );	//	closes the open file, gsw 9/5/98
			gFileRefNum = -1;			//	file is no longer
		}
		
		gRsrcH = 0L;					//	rsrc is no longer
	}

#elif (MACHINE_TYPE_ == WINTEL_iNet) 	/*	============  Windows/Intel =========== */

										/*	true if we are on Windows Nt/2k/Xp/>, gsw 9/1/97 */
	#define IS_WINDOWS_Nt_2k_Xp_local     ((BOOL) (GetVersion() < 0x80000000))

	if (! IS_WINDOWS_Nt_2k_Xp_local){	//	if not on Win Nt/2k/Xp/>, then close the library
										//	(otherwise, we get an exception error
										//	if we try to close the library with
										//	Win Nt/2k/Xp/>). Not sure why. gsw 9/1/97
										
										//	Should we call FreeLib() under WinNT ??
		if (gLib != 0L)	{
			CloseHandle( gLib );		//	close iNet32.dll driver file, gsw 12/1/96
		}
	}
										/* get Stefan's input here
										else	{							//	else we are on WinNT...
											if (gLib != 0L)	{
												FreeLibrary( gLib );		//	close iNet32.dll driver file, gsw 10/15/97
											}
										}
										*/
	
	gLib = (HANDLE) NULL;
	
#endif

	giNet_Main_EntryPtr_ = 0L;			//	show that we have closed the driver, gsw 8/1/98	

	weAlreadyClosedTheDriver = TRUE;	//	remember that we already closed
	
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	libEntry()										CLASS: SUPPORT
 *
 *				This is called when the DLL is loaded. The HP Vee interface needs this.
 *
 *	HISTORY		1/27/97	created - gsw
 * --------------------------------------------------------------------------------- */
 
Boolean gDLLisLoaded = FALSE;

void EXT_FUNC_TYPE libEntry()
{
	if ( !gDLLisLoaded )	{
		Load_instruNet_Driver( FALSE /* forceInstruNetReset */, 0L, 0L);
		gDLLisLoaded = TRUE;
	}
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	libExit()										CLASS: SUPPORT
 *
 *				This is called when the DLL is unloaded. The HP Vee interface needs this.
 *
 *	HISTORY		1/27/97	created - gsw
 * --------------------------------------------------------------------------------- */
void EXT_FUNC_TYPE libExit()
{
	if ( gDLLisLoaded )	{
		CloseDriverAndReleaseDriverRam();
		gDLLisLoaded = FALSE;
	}
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	Show_ALERT_if_hit_iNet_Error()		CLASS: BASIC UTILITY		
 *
 *	PURPOSE 	This communicates an error value to the user, the easiest simplest
 *				way possible.
 *				
 *	HISTORY		11/30/95	created - gsw
 *				1/22/96		kag - Added Windows code
 * --------------------------------------------------------------------------------- */
										//	local version
static iNetError Show_ALERT_if_hit_iNet_Error_local(iNetError e)
{
	if (e != iNetErr_None)	{			//	if we got an error...
	
		unsigned char msg[200], numStr[20];
		register unsigned char *p, *msgP;

										/*	============= Macintosh =========== */
		#if (MACHINE_TYPE_ == MACINTOSH_iNet)

										//	make sure the Mac OS has been init'ed
			Initialize_OperatingSystem_If_Necessary_main();
			
			NumToString(e, numStr);		//	convert error # to pascal string
			
										/*	============  Windows/Intel ============ */
		#elif (MACHINE_TYPE_ == WINTEL_iNet)

			ltoa(((long) e), ((char *) &numStr[1]), 10 /* base */);
			numStr[0] = 0;
			p = &numStr[1];
			while (*p++)	{			//	make pStr
				numStr[0]++;
			}

		#endif

		//	We don't use scanf since I did not want to include the ansi
		//	library on the short calling stub for instruNet World.
																					
										//	if we could not locate the driver file...
		if (e == iNetErr_CouldNotLocateDriverFile)	
			p = (unsigned char *) "instruNet Driver could not be found, instruNet error #";
		else if (e == iNetErr_RanOutOfMemory)
			p = (unsigned char *) "Ran out of memory loading instruNet Driver, instruNet error #";
		else
			p = (unsigned char *) "Hit instruNet Error #";
			
		msgP = &msg[1];
		while (*p)	{					//	copy message into msg[]
			*msgP++ = *p++;
		}
		p = &numStr[1];
		while (numStr[0]--)	{			//	append error # to msg[]
			*msgP++ = *p++;
		}
		*msgP++ = '.';					//	append "." to msg[]

		*msgP++ = 0x00;					//	C string null char
		
		msg[0] = msgP - &msg[0] - 1;	//	set pascal string length
		
		Show_Simple_Alert((char *) &msg[1]);
	}
	return e;
}
										//	external version
		iNetError EXT_FUNC_TYPE Show_ALERT_if_hit_iNet_Error(iNetError e)
		{	return Show_ALERT_if_hit_iNet_Error_local(e); }

										//	generic only uses {short, long, double}
		short /* iNetError */ EXT_FUNC_TYPE Show_ALERT_if_hit_iNet_Error_Generic(short e)
		{	return Show_ALERT_if_hit_iNet_Error_local(e); }


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	Show_Simple_Alert()				CLASS: BASIC UTILITY		
 *
 *	PURPOSE 	Shows a simple alert message.
 *				
 *	HISTORY		12/20/96	created - gsw
 * --------------------------------------------------------------------------------- */
 
void EXT_FUNC_TYPE Show_Simple_Alert(
	char *cString)						//	message to display, cString
{
#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	============= Macintosh =========== */

	iNetStr255 pString;
										//	make sure the Mac OS has been init'ed, gsw 1/27/97
	Initialize_OperatingSystem_If_Necessary_main();

	CtoP_Strcpy_local(pString, cString);//	convert C string to Pascal string

	ParamText(pString, 0L, 0L, 0L);		//	tell alert to displays this msg
			
	#define SIMPLE_SYSTEM_ALERT_RSRC_ID -16411
												
										//	show the alert
	Alert(SIMPLE_SYSTEM_ALERT_RSRC_ID, 0L);

#elif (MACHINE_TYPE_ == WINTEL_iNet)	/*	============  Windows/Intel ============ */

	iNetStr255 dummyA, dummyB;
	dummyA[0] = dummyB[0] = 0x00;		//	tell compiler to use this to avoid warning
	CtoP_Strcpy_local(dummyB, (char *) dummyA);

										//	show the alert
	MessageBox((HANDLE) NULL, cString, "instruNet Alert", MB_ICONEXCLAMATION );
	
#endif
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	Press_instruNet_Button()		CLASS: BASIC UTILITY
 *
 *				Presses a button in instruNet World
 *
 *	HISTORY		9/5/97	created - gsw
 * --------------------------------------------------------------------------------- */
iNetError /* iNetINT16 */ EXT_FUNC_TYPE Press_instruNet_Button(
	iNetINT16 buttonToPress)			//	button to press.  See iNetCM_BtnPress...
										//	in file INET_INT.H.  For example, 
										//	iNet_ButtonID_Record_Save is 5.
{
	iNetError e = PRESS_iNet_BUTTON( buttonToPress );

										//	update # of iNet Networks on this Computer
										//	(this could have changed if we just RESET
										//	the network), gsw 7/1/02
	gNumberOfNetworks = (iNetINT16) GET_NUMBER_OF_NETWORKS();

	return e;
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	Open_instruNet_Window()		CLASS: BASIC UTILITY
 *
 *				Opens an instruNet window.
 *
 *	HISTORY		3/6/97	created - gsw
 *				9/5/97	added 9..12 - gsw
 * --------------------------------------------------------------------------------- */
iNetError /* iNetINT16 */ EXT_FUNC_TYPE Open_instruNet_Window(		
	iNetINT16 iNetWindowType,			//	select window type:
										//	  1  = iNetCM_OpenWindow_Record
										//	  2  = iNetCM_OpenWindow_Network
										//	  3  = iNetCM_OpenWindow_Test
										//	  4  = iNetCM_OpenWindow_FullProbe
										//	  5  = iNetCM_OpenWindow_ChanSelection
										//	  6  = iNetCM_OpenWindow_FieldSelection
										//	  7  = iNetCM_OpenWindow_OneFieldSet
										//	  8  = iNetCM_OpenWindow_AllFieldSets										
										//	  9  = iNetCM_OpenWindow_RecTiming
										//	  10 = iNetCM_OpenWindow_RecTrigger
										//	  11 = iNetCM_OpenWindow_RecOptions
										//	  12 = iNetCM_OpenWindow_RecFile										
										//	  13 = iNetCM_OpenWindow_SimpleTiming
										//		   (pts-per-scan and sample-rate)
										//    14 = iNetCM_OpenWindow_CalOptions

	iNetINT16 *netNumP,					//	if != 0L, this is the network #
	iNetINT16 *deviceNumP,				//	if != 0L, this is the device #
	iNetINT16 *moduleNumP,				//	if != 0L, this is the module #
	iNetINT16 *chanNumP,				//	if != 0L, this is the channel #
	iNetINT16 *settingGpP,				//	if != 0L, this is the settingGroupNumOrType
	iNetINT16 *fieldNumP,				//	if != 0L, this is the field #
	iNetINT16 *userPressedOkP)			//	if != 0L, we load with True if user pressed OK
{
	iNetError e;
	iNetUINT8 netNum, devNum, modNum, chNum;
	iNetINT16 settingGp, fieldNum, intention, data = 0;
	
	if (userPressedOkP != 0L) { *userPressedOkP = TRUE; }

										//	make sure the value is in range
	if (iNetWindowType > iNet_OpenWindow_Command_FieldValues_MAX  iNetOr
	    iNetWindowType < iNet_OpenWindow_Command_FieldValues_MIN) {
		iNetWindowType = iNetCM_OpenWindow_Network;
	}

										//	if this is an instuNet main window...
	if (iNetWindowType == iNetCM_OpenWindow_Record	iNetOr
		iNetWindowType == iNetCM_OpenWindow_Network iNetOr
		iNetWindowType == iNetCM_OpenWindow_Test	iNetOr
		iNetWindowType == iNetCM_OpenWindow_SimpleTiming)	{
		
										//	open the window
		e = (iNetError) iNet( (iNetUINT8) netNum_DRIVER /* 0 */, (iNetUINT8) deviceNum_DRIVER /* 0 */, 
					(iNetUINT8) moduleNum_1stModule /* 1 */, (iNetUINT8) driver_ChanNum_OpenWindow /* 1 */,
					(iNetINT16) sgt_General /* -6 */, (iNetINT16) fldNum_General_valueEu /* 1 */,
		  			(iNetINT16) intention_setValue /* 2 */, (iNetUINT8) iNetDT_INT16, (void *) &iNetWindowType);
	}
	else if (iNetWindowType == iNetCM_OpenWindow_RecTiming)	{
										//	open Timing dialog, gsw 9/5/97
		e = PRESS_iNet_BUTTON( iNet_ButtonID_Record_Timing );
	}
	else if (iNetWindowType == iNetCM_OpenWindow_RecTrigger)	{
										//	open Trigger dialog, gsw 9/5/97
		e = PRESS_iNet_BUTTON( iNet_ButtonID_Record_Trigger );
	}
	else if (iNetWindowType == iNetCM_OpenWindow_RecOptions)	{
										//	open Record Options dialog, gsw 9/5/97
		e = PRESS_iNet_BUTTON( iNet_ButtonID_Record_Options );
	}
	else if (iNetWindowType == iNetCM_OpenWindow_RecOptions)	{
										//	open Record File dialog, gsw 9/5/97
		e = PRESS_iNet_BUTTON( iNet_ButtonID_Record_Directory );
	}
	else if (iNetWindowType == iNetCM_OpenWindow_CalOptions)	{
										//	open Record File dialog, gsw 2/15/98
		e = PRESS_iNet_BUTTON( iNet_ButtonID_Record_CalOptions );
	}
	else	{							//	else, open the probe dialog
		
										//	specify initial address
		netNum 		= (iNetUINT8) ((netNumP != 0L) 	  ? *netNumP 	: netNum_1stController);
		devNum 	= (iNetUINT8) ((deviceNumP != 0L) ? *deviceNumP : deviceNum_1stDevice);
		modNum 	= (iNetUINT8) ((moduleNumP != 0L) ? *moduleNumP : moduleNum_1stModule);
		chNum 	= (iNetUINT8) ((chanNumP != 0L)   ? *chanNumP 	: chanNum_1stChannel);
		settingGp 	= (iNetINT16) ((settingGpP != 0L) ? *settingGpP : sgt_General);
		fieldNum 	= (iNetINT16) ((fieldNumP != 0L)  ? *fieldNumP  : fldNum_General_valueEu);
	
		switch (iNetWindowType)	{		//	select type of window
			default:
			case iNetCM_OpenWindow_FullProbe:
				intention = intention_showProbeDialog_Full;
				break;
			case iNetCM_OpenWindow_ChanSelection:
				intention = intention_showProbeDialog_ChannelAddrOnly;
				break;
			case iNetCM_OpenWindow_FieldSelection:
				intention = intention_showProbeDialog_FieldAddrOnly;
				break;
			case iNetCM_OpenWindow_OneFieldSet:
				intention = intention_showProbeDialog_fixedChanAddrAndSettingGp;
				break;
			case iNetCM_OpenWindow_AllFieldSets:
				intention = intention_showProbeDialog_fixedChanAddr;
				break;
		}
										//	open the window
		e = (iNetError) iNet(netNum, devNum, modNum, chNum, settingGp, fieldNum,
						 intention, iNetDT_INT16, (void *) &data);
		
										//	tell caller what user selected
		if (netNumP != 0L)		{ *netNumP 		= netNum_PARAM(gRequestP);		}
		if (deviceNumP != 0L)	{ *deviceNumP 	= deviceNum_PARAM(gRequestP);	}
		if (moduleNumP != 0L)	{ *moduleNumP 	= moduleNum_PARAM(gRequestP);	}
		if (chanNumP != 0L) 	{ *chanNumP 	= chanNum_PARAM(gRequestP);		}
		if (settingGpP != 0L)	{ *settingGpP 	= settingGroupNum_PARAM(gRequestP);	}
		if (fieldNumP != 0L) 	{ *fieldNumP  	= fieldNum_PARAM(gRequestP);	}
		if (userPressedOkP!=0L) { *userPressedOkP = (iNetINT16) A_PARAM(gRequestP); }
	} 

										//	update # of iNet Networks in this Computer
										//	(this could have changed if we just RESET
										//	the network), gsw 7/1/02
	gNumberOfNetworks = (iNetINT16) GET_NUMBER_OF_NETWORKS();

	return e;
}
										//	generic only uses {short, long, double}
		short /* iNetError */ EXT_FUNC_TYPE Open_instruNet_Window_Generic(		
			short iNetWindowType, short *netNumP, short *deviceNumP, short *moduleNumP,
			short *chanNumP, short *settingGpP, short *fieldNumP, short *userPressedOkP)
		{ 
			return Open_instruNet_Window(iNetWindowType, netNumP, deviceNumP, moduleNumP,
				chanNumP, settingGpP, fieldNumP, userPressedOkP); 
		}

		short /* iNetError */ EXT_FUNC_TYPE Open_instruNet_Window_Easy( short iNetWindowType)
		{ return Open_instruNet_Window(iNetWindowType, 0L, 0L, 0L, 0L, 0L, 0L, 0L); }


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	Get_instruNet_Network_State()		CLASS: BASIC UTILITY
 *
 *	PURPOSE		This fills a 10KB array with the state of the instruNet network.
 *				We return an iNetError error code.
 * 
 *	HISTORY		3/15/97	created - gsw
 * --------------------------------------------------------------------------------- */

iNetError /* iNetINT16 */ EXT_FUNC_TYPE Get_instruNet_Network_State(
	iNetINT16 *arrayStart,				//	start position of 10KB array in which we
										//	place data that discribes all the instruNet
										//	field settings.
	iNetUINT32 arraySizeBytes,			//	size of passed array (i.e. arrayStart[]) in bytes
	iNetUINT32 *numBytesOfDataP)		//	filled with # of bytes that we loaded into
										//	the passed array
{
	iNetUINT32 hSize;
	char *hAddress, *networkState_Handle;
	iNetError e = iNetErr_None;
	
	if (numBytesOfDataP != 0L) { *numBytesOfDataP = 0L; }
	if (arrayStart == (iNetINT16 *) 0L) { return iNetErr_General; }
	
										//	get a handle with network state data
	networkState_Handle = (char *) GET_HANDLE_TO_NETWORK_STATE();

										//	check if things are ok
	e = (iNetError) Get_iNet_Error_local();
	if (e != iNetErr_None)	{ return e; }
	if (networkState_Handle == (char *) 0L) { return iNetErr_General; }
	
										//	lock handle and get address
	if (!iNet_Lock_Handle_local(networkState_Handle, &hSize, (iNetINT16 **) &hAddress)) { return iNetErr_General; }
	
	if (hSize > arraySizeBytes) { 		//	make sure we don't overstep callers array
		hSize = arraySizeBytes; e = iNetErr_General;
	}

	if (numBytesOfDataP != 0L) { *numBytesOfDataP = hSize; }
	
										//	copy data into caller's array
	iNet_memcopy_local((void *) hAddress, (void *) arrayStart, hSize);
		
										//	null terminate (safety measure)
	*(((iNetUINT8 *) arrayStart) + hSize) = 0x00;
	
										//	dispose of handle that contains state
	iNet_Dispose_Handle_local(networkState_Handle);

	return e;
}
										//	generic only uses {short, long, double}
short /* iNetError */ EXT_FUNC_TYPE Get_instruNet_Network_State_Generic(
	short *arrayStart, long arraySizeBytes, long *numBytesOfDataP)
{
	return Get_instruNet_Network_State(arrayStart, (iNetUINT32) arraySizeBytes, (iNetUINT32 *) numBytesOfDataP);
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	Set_instruNet_Network_State()			CLASS: BASIC UTILITY
 *
 *	PURPOSE		This takes data from a 10KB array with the state of the instruNet,
 *				and loads it into the instruNet driver.
 * 
 *	HISTORY		3/15/97	created - gsw
 * --------------------------------------------------------------------------------- */

iNetError /* iNetINT16 */ EXT_FUNC_TYPE Set_instruNet_Network_State(
	iNetINT16  *arrayStart,				//	start position of source array with data
	iNetUINT32 dataSize)				//	# of bytes of valid data in the array
{
	iNetUINT32 hSize;
	char *hAddress, *networkState_Handle;
	iNetError e = iNetErr_None;
	
	if (dataSize == 0L) 	{ return e; }
	
	if (arrayStart == (iNetINT16  *) 0L) 	{ return e; }
	
#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	=============== Macintosh ============== */
										//	create a new handle
	networkState_Handle = (char *) NewHandle(dataSize);
										//	lock the handle
#elif (MACHINE_TYPE_ == WINTEL_iNet) 	/*	=============  Windows/Intel =========== */
										//	create a new handle
	networkState_Handle = (char *) /* HGLOBAL */ GlobalAlloc(GHND, dataSize);

#endif
	if (networkState_Handle == (char *) 0L) { return iNetErr_General; }

										//	lock handle and get address
	if (!iNet_Lock_Handle_local(networkState_Handle, &hSize, (iNetINT16 **) &hAddress)) { return iNetErr_General; }

	if (hAddress == (char *) 0L  iNetOr  hSize != dataSize) { return iNetErr_General; }

										//	copy data into caller's array
	iNet_memcopy_local((void *) arrayStart, (void *) hAddress, dataSize);

										//	unlock handle
	iNet_Unlock_Handle_local(networkState_Handle);
	hAddress = 0L;
										//	set the state of the instruNet network 
	SET_NETWORK_STATE_WITH_HANDLE(networkState_Handle);

	e = (iNetError) Get_iNet_Error_local();
	
										//	dispose of handle that contains state
	iNet_Dispose_Handle_local(networkState_Handle);

	return e;
}
										//	generic only uses {short, long, double}
short /* iNetError */ EXT_FUNC_TYPE Set_instruNet_Network_State_Generic(
	short  *arrayStart,	long dataSize)
{
	return Set_instruNet_Network_State(arrayStart, (iNetUINT32) dataSize);
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	SelectNextChannelAddress()			CLASS: BASIC UTILITY
 *
 *	PURPOSE		Given a channel address, this selects the next address in the
 *				network of type 'channelType_toLookFor'.  For example, if you
 *				pass in {1,1,1,1, dir_hdwr_input_circBuf} it will return {1,1,1,2}.
 * 
 *	HISTORY		1/24/98	created - gsw
 * --------------------------------------------------------------------------------- */
iNetError /* iNetINT16 */ EXT_FUNC_TYPE SelectNextChannelAddress(
	iNetUINT8 *netNum,					//	INPUT: current channel address
	iNetUINT8 *devNum,					//	OUTPUT: next channel address
	iNetUINT8 *modNum,
	iNetUINT8 *chNum,
	iNetINT16 channelType_toLookFor)	/*	Channel Type to look for.
										 *	(dir_hdwr_input_circBuf = 1) for input channels (e.g. a/d),
										 *	(dir_hdwr_output_circBuf = 2) for output channels
										 *	(e.g. d/a), and (dir_hdwr_inAndOut_circBuf = 4) for both input
										 *	and output channel, yet digitized as in input (e.g.
										 *	iNet 100 digital i/o, controller digital/timer i/o).  */
{
	iNetINT16 numberOfNetworks, numberOfChannels, chanType, numberOfDevices, numberOfModules;
	Boolean onSeedCh = TRUE;
	iNetError e = iNetErr_None;
	
	//	----------------------------------------------------------------
	//	If our seed channel doesn't exists, return the default channel {1,1,1,1}.

										//	if our seed channel doesn't exists,
										//	return the default channel {1,1,1,1}	
	if ( ! THIS_CHANNEL_EXISTS(*netNum, *devNum, *modNum, *chNum)) {	
				
ReturnDefaultBaseAddress:

		*netNum = *devNum = *modNum = *chNum = 1;
		return e;
	}

	//	----------------------------------------------------------------
	//	Search for the next channel of type 'channelType_toLookFor'.

										//	--------------------------------
										//	Scan through the networks

	numberOfNetworks = (short) GET_NUMBER_OF_NETWORKS();	
	if ((e = Get_iNet_Error()) != iNetErr_None)	{ goto ReturnDefaultBaseAddress; }
	while (*netNum <= numberOfNetworks)	{

										//	--------------------------------
										//	Scan through the devices

		numberOfDevices = (short) GET_NUMBER_OF_DEVICES(*netNum);	
		if ((e = Get_iNet_Error()) != iNetErr_None)	{ goto ReturnDefaultBaseAddress; }
		while (*devNum <= numberOfDevices)	{

										//	--------------------------------
										//	Scan through the modules

			numberOfModules = (short) GET_NUMBER_OF_MODULES(*netNum, *devNum);	
			if ((e = Get_iNet_Error()) != iNetErr_None)	{ goto ReturnDefaultBaseAddress; }
			while (*modNum <= numberOfModules)	{
			
										//	--------------------------------
										//	Scan through the channels
	
				numberOfChannels = (short) GET_NUMBER_OF_CHANNELS(*netNum, *modNum, *devNum);	
				if ((e = Get_iNet_Error()) != iNetErr_None)	{ goto ReturnDefaultBaseAddress; }
				while (*chNum <= numberOfChannels)	{
						
										//	get channel direction (a/d, d/a, both..)	gsw 10/29/97
										//	{dir_hdwr_input_circBuf, dir_hdwr_output_circBuf, dir_hdwr_inAndOut_circBuf}
					chanType = (short) GET_CHANNEL_CATEGORY(*netNum, *devNum, *modNum, *chNum);
				
										//	if this channel is good, then we're done
					if (chanType == channelType_toLookFor iNetAnd !onSeedCh) { return e; }

					onSeedCh = FALSE;	//	we are no longer on the seed channel
					
					*chNum += 1;		//	try next channel
				}

				*chNum = 1;			//	start w/ ch1 in next module
			
				*modNum += 1;		//	try next module
			}

			*modNum = 1;				//	start w/ module 1 in next device
		
			*devNum += 1;			//	try next device
		}

		*devNum = 1;					//	start w/ device 1 in next network
	
		*netNum += 1;					//	try next network
	}
	
	goto ReturnDefaultBaseAddress;
}


//	======================= CLASS: ADVANCED READ/WRITE FIELD =====================
//	======================= CLASS: ADVANCED READ/WRITE FIELD =====================
//	======================= CLASS: ADVANCED READ/WRITE FIELD =====================

/* -----------------------------------------------------------------------------------
 *										CLASS: ADVANCED READ/WRITE FIELD
 *
 *	FUNCTION  	SetField_int32()		Writes a field in the instruNet network with
 *										a signed 32bit integer value (+/- 2billion).
 *
 *				SetField_uint32()		Writes a field in the instruNet network with
 *										an unsigned 32bit integer value (0-4billion).
 *
 *				SetField_flt32()		Writes a field in the instruNet network with
 *										a 32bit floating point number.
 *
 *				SetField_cStr()			Writes a field in the instruNet network with
 *										a C string (terminated with a 0x00 character)
 *
 *	HISTORY		1/27/97	created - gsw
 * --------------------------------------------------------------------------------- */

										//	Writes a field in the instruNet network with
										//	a signed 32bit integer value (+/- 2billion).
iNetError /* iNetINT16 */ EXT_FUNC_TYPE SetField_int32(
										
	iNetChannelAddr *chAddrP,			//	Channel Address = {netNum, devNum, modNum, chNum}/	NETWORK number = {0...numNetworks}, 0 is Driver,
	iNetINT16 settingGroupNumOrType,	//	if < 0, settings Type = { sgt_VinHardware (-3), sgt_Vout (-4), etc. }
										//	if > 0, settings Number = Setting popup menu choice.
	iNetINT16 fieldNum,					//	Field number within the settingGroup = { 1...8}
										//	See fgNums_... enum's in iNet_INET.H file.
	iNetINT32 int32Value)				//	we set field with this value
{
	return (iNetError) iNet(chAddrP->netNum, chAddrP->devNum, chAddrP->modNum, 
			chAddrP->chNum, settingGroupNumOrType, fieldNum,
			  (iNetINT16) intention_setValue, (iNetUINT8) iNetDT_INT32,
			  (void *) &int32Value);
}
										//	------------------------------------------
										//	Writes a field in the instruNet network with
										//	an unsigned 32bit integer value (0-4billion).
iNetError /* iNetINT16 */ EXT_FUNC_TYPE SetField_uint32(

	iNetChannelAddr *chAddrP,			//	Channel Address = {netNum, devNum, modNum, chNum}/	NETWORK number = {0...numNetworks}, 0 is Driver,
	iNetINT16 settingGroupNumOrType,	//	if < 0, settings Type = { sgt_VinHardware (-3), sgt_Vout (-4), etc. }
										//	if > 0, settings Number = Setting popup menu choice.
	iNetINT16 fieldNum,					//	Field number within the settingGroup = { 1...8}
										//	See fgNums_... enum's in iNet_INET.H file.
	iNetUINT32 uint32Value)				//	we set field with this value
{
	return (iNetError) iNet(chAddrP->netNum, chAddrP->devNum, chAddrP->modNum, 
			chAddrP->chNum, settingGroupNumOrType, fieldNum,
			  (iNetINT16) intention_setValue, (iNetUINT8) iNetDT_UINT32,
			  (void *) &uint32Value);
}
										//	------------------------------------------
										//	Writes a field in the instruNet network with
										//	a 32bit floating point number.
iNetError /* iNetINT16 */ EXT_FUNC_TYPE SetField_flt32(
										
	iNetChannelAddr *chAddrP,			//	Channel Address = {netNum, devNum, modNum, chNum}/	NETWORK number = {0...numNetworks}, 0 is Driver,
	iNetINT16 settingGroupNumOrType,	//	if < 0, settings Type = { sgt_VinHardware (-3), sgt_Vout (-4), etc. }
										//	if > 0, settings Number = Setting popup menu choice.
	iNetINT16 fieldNum,					//	Field number within the settingGroup = { 1...8}
										//	See fgNums_... enum's in iNet_INET.H file.
	iNetFLT32 flt32Value)				//	we set field with this value
{
	return (iNetError) iNet(chAddrP->netNum, chAddrP->devNum, chAddrP->modNum, 
			chAddrP->chNum, settingGroupNumOrType, fieldNum,
			  intention_setValue, iNetDT_FLT32, &flt32Value);
}
										//	------------------------------------------
										//	Writes a field in the instruNet network with
										//	a C string (terminated with a 0x00 character)
iNetError /* iNetINT16 */ EXT_FUNC_TYPE SetField_cStr(
										
	iNetChannelAddr *chAddrP,			//	Channel Address = {netNum, devNum, modNum, chNum}/	NETWORK number = {0...numNetworks}, 0 is Driver,
	iNetINT16 settingGroupNumOrType,	//	if < 0, settings Type = { sgt_VinHardware (-3), sgt_Vout (-4), etc. }
										//	if > 0, settings Number = Setting popup menu choice.
	iNetINT16 fieldNum,					//	Field number within the settingGroup = { 1...8}
										//	See fgNums_... enum's in iNet_INET.H file.
	char *cString)						//	we set field with this value
{
	return (iNetError) iNet(chAddrP->netNum, chAddrP->devNum, chAddrP->modNum, 
			chAddrP->chNum, settingGroupNumOrType, fieldNum,
			  intention_setValue, iNetDT_C_Str255, cString);
}

/* -----------------------------------------------------------------------------------
 *										CLASS: ADVANCED READ/WRITE FIELD
 *
 *	FUNCTION  	GetField_int32()		Reads a field in the instruNet network & returns
 *										it's value in a signed 32bit integer (+/- 2billion).
 *
 *				GetField_uint32()		Reads a field in the instruNet network & returns
										it's value in a unsigned 32bit integer (0-4billion).
 *
 *				GetField_flt32()		Reads a field in the instruNet network & returns
 *										it's value in a 32bit floating point variable.
 *
 *				GetField_cStr()			Reads a field in the instruNet network & returns
 *										it's value in a C string (0x00 terminated).
 *
 *	HISTORY		1/27/97	created - gsw
 * --------------------------------------------------------------------------------- */

iNetINT32 EXT_FUNC_TYPE GetField_int32(	//	Reads a field in the instruNet network & returns
										//	it's value in a signed 32bit integer (+/- 2billion).
										
	iNetChannelAddr *chAddrP,			//	Channel Address = {netNum, devNum, modNum, chNum}/	NETWORK number = {0...numNetworks}, 0 is Driver,
	iNetINT16 settingGroupNumOrType,	//	if < 0, settings Type = { sgt_VinHardware (-3), sgt_Vout (-4), etc. }
										//	if > 0, settings Number = Setting popup menu choice.
	iNetINT16 fieldNum,					//	Field number within the settingGroup = { 1...8}
										//	See fgNums_... enum's in iNet_INET.H file.
	iNetError *errP)					//	if this != 0L, we load it with error value
{
	iNetINT32 int32Value = 0L;
	iNetError e;
	
	e = (iNetError) iNet(chAddrP->netNum, chAddrP->devNum, chAddrP->modNum, 
			chAddrP->chNum, settingGroupNumOrType, fieldNum,
			  (iNetINT16) intention_getValue, (iNetUINT8) iNetDT_INT32, 
			  (void *) &int32Value);

	if (errP != 0L)	{ *errP = e; }
			  
	return int32Value;
}
										//	------------------------------------------
iNetUINT32 EXT_FUNC_TYPE GetField_uint32(//	Reads a field in the instruNet network & returns
										//	it's value in a unsigned 32bit integer (0-4billion).
										
	iNetChannelAddr *chAddrP,			//	Channel Address = {netNum, devNum, modNum, chNum}/	NETWORK number = {0...numNetworks}, 0 is Driver,
	iNetINT16 settingGroupNumOrType,	//	if < 0, settings Type = { sgt_VinHardware (-3), sgt_Vout (-4), etc. }
										//	if > 0, settings Number = Setting popup menu choice.
	iNetINT16 fieldNum,					//	Field number within the settingGroup = { 1...8}
										//	See fgNums_... enum's in iNet_INET.H file.
	iNetError *errP)					//	if this != 0L, we load it with error value
{
	iNetUINT32 uint32Value = 0L;
	iNetError e;
	
	e = (iNetError) iNet(chAddrP->netNum, chAddrP->devNum, chAddrP->modNum, 
			chAddrP->chNum, settingGroupNumOrType, fieldNum,
			   (iNetINT16) intention_getValue, (iNetUINT8) iNetDT_UINT32, (void *) &uint32Value);

	if (errP != 0L)	{ *errP = e; }
			  
	return fieldNum;
}
										//	------------------------------------------
iNetFLT32 EXT_FUNC_TYPE GetField_flt32(	//	Reads a field in the instruNet network & returns
										//	it's value in a 32bit floating point variable.
										
	iNetChannelAddr *chAddrP,			//	Channel Address = {netNum, devNum, modNum, chNum}/	NETWORK number = {0...numNetworks}, 0 is Driver,
	iNetINT16 settingGroupNumOrType,	//	if < 0, settings Type = { sgt_VinHardware (-3), sgt_Vout (-4), etc. }
										//	if > 0, settings Number = Setting popup menu choice.
	iNetINT16 fieldNum,					//	Field number within the settingGroup = { 1...8}
										//	See fgNums_... enum's in iNet_INET.H file.
	iNetError *errP)					//	if this != 0L, we load it with error value
{
	iNetFLT32 flt32Value = (iNetFLT32) 0.0;
	iNetError e;
	
	e = (iNetError) iNet(chAddrP->netNum, chAddrP->devNum, chAddrP->modNum, 
			chAddrP->chNum, settingGroupNumOrType, fieldNum,
			  (iNetINT16)  intention_getValue, (iNetUINT8) iNetDT_FLT32, (void *) &flt32Value);

	if (errP != 0L)	{ *errP = e; }
			  
	return flt32Value;
}

										//	------------------------------------------
iNetINT16 EXT_FUNC_TYPE GetField_cStr(	//	Reads a field in the instruNet network & returns
										//	it's value in a C string (0x00 terminated).
										
	iNetChannelAddr *chAddrP,			//	Channel Address = {netNum, devNum, modNum, chNum}/	NETWORK number = {0...numNetworks}, 0 is Driver,
	iNetINT16 settingGroupNumOrType,	//	if < 0, settings Type = { sgt_VinHardware (-3), sgt_Vout (-4), etc. }
										//	if > 0, settings Number = Setting popup menu choice.
	iNetINT16 fieldNum,					//	Field number within the settingGroup = { 1...8}
										//	See fgNums_... enum's in iNet_INET.H file.
	char *cString)						//	we set field with this value
{
	*cString = 0x00;
	
	return iNet(chAddrP->netNum, chAddrP->devNum, chAddrP->modNum, 
			chAddrP->chNum, settingGroupNumOrType, fieldNum,
			  intention_getValue, iNetDT_C_Str255, cString);
}

/* -----------------------------------------------------------------------------------
 *	FUNCTION  	iNet_int32()			CLASS: ADVANCED READ/WRITE FIELD			
 *
 *	PURPOSE 	Calls iNet() to read or write a field in the instruNet network with a
 *				signed 32bit integer value (+/- 2billion).
 *
 *				If we are setting a field value (i.e. intention == intention_setValue)
 *				then we return an iNetError error code. If intention is not
 *				intention_setValue and we get an error condition, then we return
 *				0; otherwise we return the resulting field value.
 *
 *				If you want to write to the field, set the 'intention' field to
 *				'intention_setValue' and pass the new value in 'int32Value'.
 *
 *				If you want to read the value of a field, set the 'intention' field 
 *				to 'intention_getValue' and then refer to the returned value.
 *
 *	HISTORY		2/4/96	created - gsw
 * --------------------------------------------------------------------------------- */
iNetINT32 EXT_FUNC_TYPE iNet_int32(
	iNetUINT8 netNum,					//	NETWORK number = {0...numNetworks}, 0 is Driver,
	iNetUINT8 devNum,				//	DEVICE number {0...numDevices}, 0 = "Controller", 1 = 1st device on network
	iNetUINT8 modNum, 				//	MODULE number within a hardware DEVICE {1...32}.
										//	Many devices have only 1 module.
	iNetUINT8 chNum, 					//	Hardware CHANNEL number {1...32}. Each device contains
										//	a number of channels, each of which has it's own
										//	channel number.
	iNetINT16 settingGroupNumOrType,	//	if < 0, settings Type = { sgt_VinHardware (-3), sgt_Vout (-4), etc. }
										//	if > 0, settings Number = Setting popup menu choice.
	iNetINT16 fieldNum,					//	Field number within the settingGroup = { 1...8}
										//	See fgNums_... enum's in iNet_INET.H file.
	iNetINT16 intention,				//	Tells iNet() what to do with the field.  Please
										//	ion_intention enum for details. ion_intention =
										//	{intention_getValue, intention_setValue,
										//	intention_getNameStr, intention_getMaxValue, etc}
	iNetINT32 int32Value)				//	we set field with this value if that is
										//	the specified intention
{
	iNetError e;
										//	ask the driver to perform the "intention"
	e = (iNetError) iNet(netNum, devNum, modNum, chNum, settingGroupNumOrType, fieldNum,
			  intention, iNetDT_INT32, &int32Value);

	if (intention==intention_setValue){	//	if we set a field value...
	
		return (iNetINT32) e;			//	return the iNet error code
	}
	else	{
		if (e == iNetErr_None)			//	if there was no error
			return int32Value;			//	return the resulting value
		else
			return (iNetINT32) 0L;		//	otherwise, return 0 (we got an error)
	}
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	iNet_uint32()			CLASS: ADVANCED READ/WRITE FIELD				
 *
 *	PURPOSE 	Calls iNet() to read or write a field in the iNet network with an
 *				unsigned 32bit integer value (0 to 4billion). 
 *
 *				If we are setting a field value (i.e. intention == intention_setValue)
 *				then we return an iNetError error code. If intention is not
 *				intention_setValue and we get an error condition, then we return
 *				0; otherwise we return the resulting field value.
 *
 *				If you want to write to the field, set the 'intention' field to
 *				'intention_setValue' and pass the new value in 'uint32Value'.
 *
 *				If you want to read the value of a field, set the 'intention' field 
 *				to 'intention_getValue' and then refer to the returned value.
 *
 *	HISTORY		2/4/96	created - gsw
 * --------------------------------------------------------------------------------- */
iNetUINT32 EXT_FUNC_TYPE iNet_uint32(
	iNetUINT8 netNum,					//	NETWORK number, 0 is Driver, 1 is 1st card
	iNetUINT8 devNum,				//	DEVICE number, 0 = "Controller", 1 = 1st device on network
	iNetUINT8 modNum, 				//	MODULE number within a hardware DEVICE {1...32}.
	iNetUINT8 chNum, 					//	Hardware CHANNEL number {1...32}.
	iNetINT16 settingGroupNumOrType,	//	if < 0, settings Type = { sgt_VinHardware (-3), sgt_Vout (-4), etc. }
										//	if > 0, settings Number = Setting popup menu choice.
	iNetINT16 fieldNum,					//	Field number within the settingGroup = { 1...8}
										//	See fgNums_... enum's in iNet_INET.H file.
	iNetINT16 intention,				//	Tells iNet() what to do with the field.  Please
										//	ion_intention enum for details. ion_intention =
										//	{intention_getValue, intention_setValue,
										//	intention_getNameStr, intention_getMaxValue, etc}
	iNetUINT32 uint32Value)				//	we set field with this value if that is
										//	the specified intention
{
	iNetError e;
										//	ask the driver to perform the "intention"
	e = (iNetError) iNet(netNum, devNum, modNum, chNum, settingGroupNumOrType,
			fieldNum, intention, iNetDT_UINT32, &uint32Value);
			  
										//	if we set a field value...
	if (intention == intention_setValue)	{
		return (iNetUINT32) e;			//	return the iNet error code
	}
	else	{
		if (e == iNetErr_None)			//	if there was no error
			return uint32Value;			//	return the resulting value
		else
			return (iNetUINT32) 0L;		//	otherwise, return 0 (we got an error)
	}
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	iNet_flt32()			CLASS: ADVANCED READ/WRITE FIELD				
 *
 *	PURPOSE 	Calls iNet() to read or write a field in the iNet network with a
 *				32bit floating point number.
 *
 *				If we are setting a field value (i.e. intention == intention_setValue)
 *				then we return an iNetError error code. If intention is not
 *				intention_setValue and we get an error condition, then we return
 *				0; otherwise we return the resulting field value.
 *
 *				If you want to write to the field, set the 'intention' field to
 *				'intention_setValue' and pass the new value in 'flt32Value'.
 *
 *				If you want to read the value of a field, set the 'intention' field 
 *				to 'intention_getValue' and then refer to the returned value.
 *
 *	HISTORY		2/4/96	created - gsw
 * --------------------------------------------------------------------------------- */
iNetFLT32 EXT_FUNC_TYPE iNet_flt32(
	iNetUINT8 netNum,					//	NETWORK number, 0 is Driver, 1 is 1st card
	iNetUINT8 devNum,				//	DEVICE number, 0 = "Controller", 1 = 1st device on network
	iNetUINT8 modNum, 				//	MODULE number within a hardware DEVICE {1...32}.
	iNetUINT8 chNum, 					//	Hardware CHANNEL number {1...32}.
	iNetINT16 settingGroupNumOrType,	//	if < 0, settings Type = { sgt_VinHardware (-3), sgt_Vout (-4), etc. }
										//	if > 0, settings Number = Setting popup menu choice.
	iNetINT16 fieldNum,					//	Field number within the settingGroup = { 1...8}
										//	See fgNums_... enum's in iNet_INET.H file.
	iNetINT16 intention,				//	Tells iNet() what to do with the field.  Please
										//	ion_intention enum for details. ion_intention =
										//	{intention_getValue, intention_setValue,
										//	intention_getNameStr, intention_getMaxValue, etc}
	iNetFLT32 flt32Value)				//	we set field with this value if that is
										//	the specified intention
{
	iNetError e;
										//	ask the driver to perform the "intention"
	e = (iNetError) iNet(netNum, devNum, modNum, chNum, settingGroupNumOrType, fieldNum,
			  intention, iNetDT_FLT32, &flt32Value);
			  
										//	if we set a field value...
	if (intention == intention_setValue)	{
		return (iNetFLT32) e;			//	return the iNet error code
	}
	else	{
		if (e == iNetErr_None)			//	if there was no error
			return flt32Value;			//	return the resulting value
		else
			return (iNetFLT32) 0L;		//	otherwise, return 0 (we got an error)
	}
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	iNet_cString()			CLASS: ADVANCED READ/WRITE FIELD		
 *
 *	PURPOSE 	Calls iNet() to read or write a field in the iNet network with a
 *				C string variable (0-255 chars, 0 terminated).
 *
 *				Return an iNetError error code.
 *
 *				If you want to write to the field, set the 'intention' field to
 *				'intention_setValue' and pass the new value in 'cString'.
 *
 *				If you want to read the value of a field, set the 'intention' field 
 *				to 'intention_getValue' and then refer to the returned value.
 *
 *	HISTORY		2/4/96	created - gsw
 * --------------------------------------------------------------------------------- */
iNetError EXT_FUNC_TYPE iNet_cString(
	iNetUINT8 netNum,					//	NETWORK number, 0 is Driver, 1 is 1st card
	iNetUINT8 devNum,				//	DEVICE number, 0 = "Controller", 1 = 1st device on network
	iNetUINT8 modNum, 				//	MODULE number within a hardware DEVICE {1...32}.
	iNetUINT8 chNum, 					//	Hardware CHANNEL number {1...32}.
	iNetINT16 settingGroupNumOrType,	//	if < 0, settings Type = { sgt_VinHardware (-3), sgt_Vout (-4), etc. }
										//	if > 0, settings Number = Setting popup menu choice.
	iNetINT16 fieldNum,					//	Field number within the settingGroup = { 1...8}
										//	See fgNums_... enum's in iNet_INET.H file.
	iNetINT16 intention,				//	Tells iNet() what to do with the field.  Please
										//	ion_intention enum for details. ion_intention =
										//	{intention_getValue, intention_setValue,
										//	intention_getNameStr, intention_getMaxValue, etc}
	char *cString)						//	0-255 chars, 0 terminated, C string
{
	iNetError e;
										//	ask the driver to perform the "intention"
	e = (iNetError) iNet(netNum, devNum, modNum, chNum, settingGroupNumOrType, fieldNum,
			  intention, iNetDT_C_Str255, cString);
			  
	return e;							//	returns iNetError error code
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	iNet_pString()			CLASS: ADVANCED READ/WRITE FIELD				
 *
 *	PURPOSE 	Calls iNet() to read or write a field in the iNet network with a
 *				pascal string variable (0-255 chars, 0th char is the length).
 *
 *				Return an iNetError error code.
 *
 *				If you want to write to the field, set the 'intention' field to
 *				'intention_setValue' and pass the new value in 'pString'.
 *
 *				If you want to read the value of a field, set the 'intention' field 
 *				to 'intention_getValue' and then refer to the returned value.
 *
 *	HISTORY		2/4/96	created - gsw
 * --------------------------------------------------------------------------------- */
iNetError EXT_FUNC_TYPE iNet_pString(
	iNetUINT8 netNum,					//	NETWORK number, 0 is Driver, 1 is 1st card
	iNetUINT8 devNum,				//	DEVICE number, 0 = "Controller", 1 = 1st device on network
	iNetUINT8 modNum, 				//	MODULE number within a hardware DEVICE {1...32}.
	iNetUINT8 chNum, 					//	Hardware CHANNEL number {1...32}.
	iNetINT16 settingGroupNumOrType,	//	if < 0, settings Type = { sgt_VinHardware (-3), sgt_Vout (-4), etc. }
										//	if > 0, settings Number = Setting popup menu choice.
	iNetINT16 fieldNum,					//	Field number within the settingGroup = { 1...8}
										//	See fgNums_... enum's in iNet_INET.H file.
	iNetINT16 intention,				//	Tells iNet() what to do with the field.  Please
										//	ion_intention enum for details. ion_intention =
										//	{intention_getValue, intention_setValue,
										//	intention_getNameStr, intention_getMaxValue, etc}
	unsigned char *pString)				//	0-255 chars, 0th char is the length, Pascal string
{
	iNetError e;
										//	ask the driver to perform the "intention"
	e = (iNetError) iNet(netNum, devNum, modNum, chNum, settingGroupNumOrType, fieldNum,
			  intention, iNetDT_P_Str255, pString);
			  
	return e;							//	returns iNetError error code
}

										//	generic only uses {short, long, double}
		short /* iNetError */ EXT_FUNC_TYPE iNet_Generic( short netNum, short devNum, short modNum,
			short chNum, short settingGroupNumOrType, short fieldNum, short intention,
			short argType, short *ptrToArg)
		{
			return (short) iNet((iNetUINT8) netNum, (iNetUINT8) devNum, (iNetUINT8) modNum,
				(iNetUINT8) chNum, settingGroupNumOrType, fieldNum, intention,
				(iNetUINT8) argType, (void *) ptrToArg);
		}

		short /* iNetError */ EXT_FUNC_TYPE iNet_rw_int16( short netNum, short devNum, short modNum,
			short chNum, short settingGroupNumOrType, short fieldNum, short intention,
			short *ptrToArg)
		{
			return (short) iNet((iNetUINT8) netNum, (iNetUINT8) devNum, (iNetUINT8) modNum,
				(iNetUINT8) chNum, settingGroupNumOrType, fieldNum, intention,
				(iNetUINT8) iNetDT_INT16, (void *) ptrToArg);
		}

		short /* iNetError */ EXT_FUNC_TYPE iNet_rw_int32( short netNum, short devNum, short modNum,
			short chNum, short settingGroupNumOrType, short fieldNum, short intention,
			long *ptrToArg)
		{
			return (short) iNet((iNetUINT8) netNum, (iNetUINT8) devNum, (iNetUINT8) modNum,
				(iNetUINT8) chNum, settingGroupNumOrType, fieldNum, intention,
				(iNetUINT8) iNetDT_INT32, (void *) ptrToArg);
		}

		short /* iNetError */ EXT_FUNC_TYPE iNet_rw_Double( short netNum, short devNum, short modNum,
			short chNum, short settingGroupNumOrType, short fieldNum, short intention,
			double *ptrToArg)
		{
			return (short) iNet((iNetUINT8) netNum, (iNetUINT8) devNum, (iNetUINT8) modNum,
				(iNetUINT8) chNum, settingGroupNumOrType, fieldNum, intention,
				(iNetUINT8) iNetDT_Double, (void *) ptrToArg);
		}

		short /* iNetError */ EXT_FUNC_TYPE iNet_rw_cStr( short netNum, short devNum, short modNum,
			short chNum, short settingGroupNumOrType, short fieldNum, short intention,
			char *ptrToArg)
		{
			return (short) iNet((iNetUINT8) netNum, (iNetUINT8) devNum, (iNetUINT8) modNum,
				(iNetUINT8) chNum, settingGroupNumOrType, fieldNum, intention,
				(iNetUINT8) iNetDT_C_Str255, (void *) ptrToArg);
		}

/* -----------------------------------------------------------------------------------
 *	FUNCTION  	iNet_DLL()				CLASS: ADVANCED READ/WRITE FIELD
 *
 *	PURPOSE 	This is the main entry point fo the instruNet DRIVER, that
 *				is exported out to a Win95 DLL.
 *				
 *	ON RETURN	Returns an iNet Error code.
 *				
 *	HISTORY		5/3/95	created - gsw
 * --------------------------------------------------------------------------------- */
iNetError EXT_FUNC_TYPE iNet_DLL(		/*	Exported out to Win95 DLL */

	iNetUINT8 netNum,					//	NETWORK number, 0 is Driver, 1 is 1st card
	iNetUINT8 devNum,				//	DEVICE number, 0 = "Controller", 1 = 1st device on network
	iNetUINT8 modNum, 				//	MODULE number within a hardware DEVICE {1...32}.
	iNetUINT8 chNum, 					//	Hardware CHANNEL number {1...32}.
	iNetINT16 settingGroupNumOrType,	//	if < 0, settings Type = { sgt_VinHardware (-3), sgt_Vout (-4), etc. }
										//	if > 0, settings Number = Setting popup menu choice.
	iNetINT16 fieldNum,					//	Field number within the settingGroup = { 1...8}
										//	See fgNums_... enum's in iNet_INET.H file.
	iNetINT16 intention,				//	Tells iNet() what to do with the field.  Please
										//	ion_intention enum for details. ion_intention =
										//	{intention_getValue, intention_setValue,
										//	intention_getNameStr, intention_getMaxValue, etc}
	iNetUINT8 argType,					//	Argument type. iNetDataType =
										//	{iNetDT_INT16 (0), iNetDT_UINT16 (2),
										//	iNetDT_INT32 (3), iNetDT_UINT32 (4), 
										//	iNetDT_FLT32 (5), iNetDT_Double (6),
										//	iNetDT_P_Str15 (8), iNetDT_P_Str255 (9)
										//	iNetDT_C_Str15 (10), iNetDT_C_Str255 (11)
	void *ptrToArg)						//	pointer to argument
{
	return (iNetError) iNet(netNum, devNum,  modNum, chNum, 
		settingGroupNumOrType, fieldNum, intention, argType, ptrToArg);
}

//	=============================== CLASS: ADVANCED DIGITIZE ===============================
//	=============================== CLASS: ADVANCED DIGITIZE ===============================
//	=============================== CLASS: ADVANCED DIGITIZE ===============================

/* -----------------------------------------------------------------------------------
 *	FUNCTION  	LoadChannelAddress()			CLASS: ADVANCED DIGITIZE
 *
 *				Defines a channel's address.  Returns TRUE (1) if this channel exists;
 *				FALSE (0) otherwise.
 *
 *	HISTORY		1/27/97	created - gsw
 * --------------------------------------------------------------------------------- */
iNetINT16 /* Boolean */ EXT_FUNC_TYPE LoadChannelAddress(		
	iNetChannelAddr *chAddrP,			//	Channel Address = {netNum, devNum, modNum, chNum}/	NETWORK number = {0...numNetworks}, 0 is Driver,
	iNetUINT8 netNum,					//	NETWORK number, 0 is Driver, 1 is 1st card
	iNetUINT8 devNum,					//	DEVICE number, 0 = "Controller", 1 = 1st device on network
	iNetUINT8 modNum, 					//	MODULE number within a hardware DEVICE {1...32}.
	iNetUINT8 chNum) 					//	Hardware CHANNEL number {1...32}.
{
	chAddrP->netNum = netNum; 
	chAddrP->devNum = devNum; 
	chAddrP->modNum = modNum; 
	chAddrP->chNum = chNum;
										//	determine if this channel exists...
	return (iNetINT16) (((Boolean) THIS_CHANNEL_EXISTS(netNum, devNum, modNum, chNum)) ? 1 : 0);
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	GetChannelAddress()		 		CLASS: ADVANCED DIGITIZE
 *
 *				Returns a channel's address, which is stored in a passed
 *				iNetChannelAddr struct.  If you pass a 0L instead of the address
 *				of an iNetINT16, the address is not loaded.
 *
 *	HISTORY		3/6/97	created - gsw
 * --------------------------------------------------------------------------------- */
void EXT_FUNC_TYPE GetChannelAddress(		
	iNetChannelAddr *chAddrP,			//	Channel Address = {netNum, devNum, modNum, chNum}/	NETWORK number = {0...numNetworks}, 0 is Driver,
	iNetINT16 *netNumP,					//	this function loads this with network #								
	iNetINT16 *deviceNumP,				//	this function loads this with device #
	iNetINT16 *moduleNumP, 				//	this function loads this with module #
	iNetINT16 *chanNumP) 				//	this function loads this with channel #
{
	if (netNumP != (iNetINT16 *) 0L) 	{ *netNumP    = (iNetINT16) chAddrP->netNum; }
	if (deviceNumP != (iNetINT16 *) 0L) { *deviceNumP = (iNetINT16) chAddrP->devNum; } 
	if (moduleNumP != (iNetINT16 *) 0L) { *moduleNumP = (iNetINT16) chAddrP->modNum; }
	if (chanNumP != (iNetINT16 *) 0L) 	{ *chanNumP   = (iNetINT16) chAddrP->chNum; }
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	EnableChannelForDigitizing()	CLASS: ADVANCED DIGITIZE
 *
 *				Specifies that this channel is enabled for digitizing.
 *
 *	HISTORY		1/27/97	created - gsw
 * --------------------------------------------------------------------------------- */
iNetError /* iNetINT16 */ EXT_FUNC_TYPE EnableChannelForDigitizing(		
	iNetChannelAddr *chAddrP)			//	Channel Address = {netNum, devNum, modNum, chNum}/	NETWORK number = {0...numNetworks}, 0 is Driver,
{
										//	mark this Channel for digitizing
	return ENABLE_CHANNEL_DIGITIZATION(chAddrP->netNum, chAddrP->devNum, chAddrP->modNum, chAddrP->chNum);
}

/* -----------------------------------------------------------------------------------
 *	FUNCTION  	DisableAllChannelsForDigitizing()	CLASS: ADVANCED DIGITIZE
 *
 *				Disables ALL channels for digitizing.
 *
 *	HISTORY		1/27/97	created - gsw
 * --------------------------------------------------------------------------------- */
										//	external version
iNetError /* iNetINT16 */ EXT_FUNC_TYPE DisableAllChannelsForDigitizing(void)
{
	return DisableAllChannelsForDigitizing_local();
}
										//	local version
static iNetError /* iNetINT16 */ DisableAllChannelsForDigitizing_local(void)		
{
	iNetUINT8 netNum;
	iNetError e = iNetErr_None;

	if (gNumberOfNetworks > 0)	{
										//	set timing params for each network	
		for (netNum = 1; netNum <= gNumberOfNetworks; netNum++)		{
										//	disables all channels in all networks for digitizing
			DISABLE_ALL_CHANNEL_DIGITIZING( netNum );
		}
	}
	
	return e;
}

/* -----------------------------------------------------------------------------------
 *	FUNCTION  	Service_All_iNet_Digitize_Buffers()		CLASS: ADVANCED DIGITIZE		
 *
 *	PURPOSE 	This routine must be called continuously while digitizing to allow
 *				the processor to service all digitize buffers.
 *
 *	HISTORY		11/30/95	created - gsw 
 *				5/12/96		updated - gsw
 * --------------------------------------------------------------------------------- */
iNetError EXT_FUNC_TYPE Service_All_iNet_Digitize_Buffers(
	iNetUINT8 netNum_forScanInfo,		//	we get scan # info from this controller,
										//	even though we service ALL controllers
	iNetINT16 *theNextDataAccessIsTheLast_ofEntireDigitization_allCtlr,
										//	Returns TRUE (1) if the next data pulled out
										//	out of the buffers via Access_Digitized_Data_In_Ram_Buffer()
										//	is that last pull for all networks; FALSE (0) otherwise
										//	(i.e. at least 1 network still has data in it)
	iNetINT16 *theNextDataAccessWillPullToTheEndOfAScan_oneCtlr,
										//	Returns TRUE (1) if the next data access via
										//	Access_Digitized_Data_In_Ram_Buffer() accesses data
										//	to the end of 1 Scan in network 'netNum_forScanInfo' (i.e. 1 
										//	Scan's worth of data); FALSE (0) otherwise.  If you do not
										//	access 1 scan at a time, you can access little
										//	segments at a time, as they become available.
	iNetINT16 *controllerIsFinishedWithDigitization_allCtlr,
										//	set to TRUE if controller is finished with digitization
										//	(i.e. the last piece of data has been moved from controller
										//	ram buffer to host memory ram buffer); FALSE otherwise.
	iNetUINT32 *last_scanNum_base1_pulled_inFull_oneCtlr,
										//	last scan # (base 1) that we pulled (e.g. a/d) in 'netNum_forScanInfo'
										//	full from controller ram buffer TO user or
										//	driver ram buffer when Service_All_iNet_Digitize_Buffers()
										//	was called.
	iNetUINT32 *last_scanNum_base1_pushed_inFull_oneCtlr)
										//	last scan # (base 1) that we pushed (e.g. d/a) in 'netNum_forScanInfo'
										//	full from user or driver ram buffer TO controller
										//	ram buffer when Service_All_iNet_Digitize_Buffers()
										//	was called.
{
	iNetINT32 theNextDataAccessIsTheLast_ofEntireDigitization_int32;
	iNetError e; 
										//	service the digitize buffers
	theNextDataAccessIsTheLast_ofEntireDigitization_int32
		= (iNetINT32) SERVICE_ALL_DIGITIZE_BUFFERS(netNum_forScanInfo);
		
										//	ready the response
	e = (iNetError) Get_iNet_Error_local();
	if (theNextDataAccessIsTheLast_ofEntireDigitization_allCtlr != 0L)	{
		*theNextDataAccessIsTheLast_ofEntireDigitization_allCtlr
			= (iNetINT16) ((theNextDataAccessIsTheLast_ofEntireDigitization_int32 == 0L) ? 0 : 1);
	}

	if (theNextDataAccessWillPullToTheEndOfAScan_oneCtlr != 0L)	{	
		*theNextDataAccessWillPullToTheEndOfAScan_oneCtlr = (iNetINT16) ((A_PARAM(gRequestP) == 0L) ? 0 : 1); }
	if (last_scanNum_base1_pulled_inFull_oneCtlr != 0L)	{
		*last_scanNum_base1_pulled_inFull_oneCtlr = B_PARAM(gRequestP); }
	if (last_scanNum_base1_pushed_inFull_oneCtlr != 0L)	{
		*last_scanNum_base1_pushed_inFull_oneCtlr = C_PARAM(gRequestP); }
	if (controllerIsFinishedWithDigitization_allCtlr != 0L)	{
		*controllerIsFinishedWithDigitization_allCtlr =(iNetINT16) ((D_PARAM(gRequestP) == 0L) ? 0 : 1); }
	
	return e;
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	Access_Digitized_Data_In_Ram_Buffer()		CLASS: ADVANCED DIGITIZE				
 *
 *	PURPOSE 	While digitizing, this will pull a segment out of the iNet 
 *				DRIVER RAM BUFFER or USER RAM BUFFER and pass you an address
 *				to the first FLT32 point in the buffer (i.e. beginning of the
 *				entire buffer), the index into the buffer to begin reading
 *				data, and the # of points to pull.
 *
 *				1st point to read
 *					= (*bufferStartAddrP) + *pointToPullindex_base1 - 1
 *
 *				last point to read
 *					= (*bufferStartAddrP) + *pointToPullindex_base1 + *numPointsToPull - 2
 *				
 *				The buffer is 1 scan long, and the scan number {1...numScans}
 *				that we are reading is returned in '*scanNumberToPull_base1'.
 *
 *				The # of times the iNet circular buffers overflowed is returned
 *				in '*totalNumOfTimesWeOverflowed'.
 *
 *	HISTORY		11/25/95	created - gsw
 *				11/30/95	modified - gsw
 * --------------------------------------------------------------------------------- */
iNetError EXT_FUNC_TYPE Access_Digitized_Data_In_Ram_Buffer(
	iNetChannelAddr *channelAddr,		//	Channel Address = {netNum, devNum, modNum, chNum}/	NETWORK number = {0...numNetworks}, 0 is Driver,
	Boolean driverRamBuffer,			//	if TRUE, we go to DRIVER RAM BUFFER, otherwise,
										//	we go to USER RAM BUFFER.
	iNetFLT32 **bufferStartAddrP,		//	we will load this with the address to the 
										//	first FLT32 point in the buffer
	iNetUINT32 *pointToPullindex_base1,	//	we will load this with the {1... numPtsPerScan)
										//	index of the first point of the segment that we need
										//	to process (base 1 index).
	iNetUINT32 *numPointsToPull,		//	we will load this with the # of points to to pull
										//	out of the buffer
	iNetUINT32 *scanNumberToPull_base1)	//	we load this with the scan number {1...numScans}
										//	of the scan that we are currently pulling (base 1).
{
										//	pull digitize data out of buffer
	return ProcessBufferData(channelAddr, driverRamBuffer, bufferStartAddrP,
			pointToPullindex_base1, numPointsToPull, scanNumberToPull_base1,
			intention_getValueAndPullData, 0L);
}

/* -----------------------------------------------------------------------------------
 *	FUNCTION  	ProcessBufferData()				
 *
 *	PURPOSE 	Processes buffer data in some way.
 *
 *	HISTORY		10/28/98	created - gsw
 * --------------------------------------------------------------------------------- */
static iNetError ProcessBufferData(
	iNetChannelAddr *channelAddr,		//	Channel Address = {netNum, devNum, modNum, chNum}/	NETWORK number = {0...numNetworks}, 0 is Driver,
	Boolean driverRamBuffer,			//	if TRUE, we go to DRIVER RAM BUFFER, otherwise,
										//	we go to USER RAM BUFFER.
	iNetFLT32 **bufferStartAddrP,		//	we will load this with the address to the 
										//	first FLT32 point in the buffer
	iNetUINT32 *pointToPullindex_base1,	//	we will load this with the {1... numPtsPerScan)
										//	index of the first point of the segment that we need
										//	to process (base 1 index).
	iNetUINT32 *numPointsToPull,		//	we will load this with the # of points to to pull
										//	out of the buffer
	iNetUINT32 *scanNumberToPull_base1,	//	we load this with the scan number {1...numScans}
										//	of the scan that we are currently pulling (base 1).
	iNetINT16 intention,				//	type of processing
	iNetUINT32 submitValue)				//	data to pass to driver
{
	iNetError e;
	iNetINT16 sgt_SettingType, fieldNum;
	iNetUINT32 returnedValue;

	if (driverRamBuffer)	{			//	if we go to the DRIVER RAM BUFFER...
		sgt_SettingType = sgt_DriverRamBuffer;
		fieldNum = fldNum_DvrBuffer_BufferAddr;
	}
	else	{							//	else, we go to the USER RAM BUFFER
		sgt_SettingType = sgt_UserRamBuffer;
		fieldNum = fldNum_UserBuffer_UserBufferAddr;
	}

	D_PARAM(gRequestP) = submitValue;
										//	tell iNet driver to pull data out of buffers
	e = (iNetError) iNet(channelAddr->netNum, channelAddr->devNum, channelAddr->modNum,
			channelAddr->chNum, sgt_SettingType, fieldNum, intention,
			iNetDT_UINT32, &returnedValue);
	
										//	return info to the caller
	if (bufferStartAddrP != 0L) 	{ *bufferStartAddrP 	  = *(iNetFLT32 **) &returnedValue; }
	if (pointToPullindex_base1 !=0L){ *pointToPullindex_base1 = A_PARAM(gRequestP); }
	if (numPointsToPull != 0L)		{ *numPointsToPull 		  = B_PARAM(gRequestP); }
	if (scanNumberToPull_base1 !=0L){ *scanNumberToPull_base1 = C_PARAM(gRequestP); }

	return e;
}

/* -----------------------------------------------------------------------------------
 *	FUNCTION  	Get_Ctlr_Ram_Buff_Info()		CLASS: ADVANCED DIGITIZE	
 *
 *	PURPOSE 	Returns info about a controller ram buffer.
 *
 *	HISTORY		2/28/98	 created - gsw
 * --------------------------------------------------------------------------------- */
iNetError /* iNetINT16 */ EXT_FUNC_TYPE Get_Ctlr_Ram_Buff_Info(
	iNetUINT8 netNum_forBuffInfo,		//	we get info from this controller,
	iNetINT32 *ctlrRamBufferSizePts, 	//	filled with # of pts of space available in 
										//	controller ram buffer.
	iNetINT32 *numPtsNotYetPulled)		//	filled with # of pts in buffer that have yet
										//	to be pulled out.
{
	iNetError e;
										//	get info about controller ram buffer
	GET_CTLR_BUFFER_INFO(netNum_forBuffInfo);
	e = (iNetError) Get_iNet_Error_local();
		
										//	return info about ctlr ram buffer
	if (ctlrRamBufferSizePts != 0L)	{ *ctlrRamBufferSizePts = A_PARAM(gRequestP);}
	if (numPtsNotYetPulled != 0L)	{ *numPtsNotYetPulled   = B_PARAM(gRequestP); }
	
	return e;
}

/* -----------------------------------------------------------------------------------
 *	FUNCTION  	Set_iNet_Demo_Mode()		CLASS: ADVANCED UTILITY	
 *
 *	PURPOSE		Sets instruNet demo mode.
 *
 *	HISTORY		2/28/98	 created - gsw
 * --------------------------------------------------------------------------------- */
iNetError /* iNetINT16 */ EXT_FUNC_TYPE Set_iNet_Demo_Mode(
	iNetINT16 demoMode)					//	0:	iNet_NoDemo - real mode (no demo)
										//	1:	iNet_FullSimulation - we fully simulate iNet
										//		hardware w/ triangle waves feeding a fake
										//		iNet-100 & iNet-200.  This will reset instruNet.
										//	2:	iNet_CorruptData - we read real instruNet
										//		hardware, yet we alter the data randomly, to
										//		make it unuseable.
{
	iNetError e;
	SET_DEMO_MODE( demoMode );
	e = (iNetError) Get_iNet_Error_local();
										//	Update # of iNet Networks on this Computer.
										//	This could have changed after setting the 
										//	demo mode, gsw 7/1/02
	gNumberOfNetworks = (iNetINT16) GET_NUMBER_OF_NETWORKS();

	return e;
}
	
/* -------------------------------------------------------------------------------------
 *	FUNCTION  	Get_Win32_DLL_VersionNum()		
 *
 *	PURPOSE		Gets the version # of a windows DLL and
 *				puts it into 'version_cStr'.  This code came off the
 *				Microsoft CD "Code to Extract SDK DLL Version Information".
 *				
 *	HISTORY		10/20/98	added - gsw
 * ----------------------------------------------------------------------------------- */

#if (MACHINE_TYPE_ == WINTEL_iNet)	/*	=========== Windows/Intel =========== */

									/*	if we are NOT building tiny app to open window... */  
#if !defined(BUILD_SNUB_APL_TO_CALL_iNet_DRIVER)

typedef struct TRANSARRAY {
	WORD wLanguageID;
	WORD wCharacterSet;
} TRANSARRAY;

static void Get_Win32_DLL_VersionNum(
	char *dllName_cStr,			//	name of DLL (e.g. "iNet32.dll")
	char *version_cStr,			//	loaded w/ ver string  (e.g. "1,22,3,2")
	iNetUINT32 *majorVerP,		//	loaded w/ major ver # (e.g. 1)
	iNetUINT32 *minorVerP,		//	loaded w/ minor ver # (e.g. 22)
	iNetUINT32 *subminorVerP,	//	loaded w/ subminor ver # (e.g. 3)
	iNetUINT32 *subSubVerP)		//	loaded w/ subsub ver # (e.g. 2)
{							
	DWORD dwVerInfoSize;		// Size of version information block
	void *lpVersion = (void *) 0L;			
								// String pointer to 'version' text
	TRANSARRAY *lpTranslation = (TRANSARRAY *) 0L;	
								// String pointer to translation table
	DWORD dwVerHnd = 0;			// An 'ignored' parameter, always '0'
	UINT puVersionLen = 0;
	UINT puTranslationLen = 0;
	BOOL bRetCode;
	char WinDir[_MAX_PATH];
	
	strcpy(version_cStr, "?");
	*majorVerP = *minorVerP = *subminorVerP = *subSubVerP = 0L;

								/* Win95: "Win95\system\iNet32.dll" */
								/* WinNT: "WinNT\system32\iNet32.dll" */
	GetSystemDirectory(WinDir, _MAX_PATH);
	strcat(WinDir, "\\");
	strcat(WinDir, dllName_cStr /* e.g. "iNet32.dll" */);

	dwVerInfoSize = GetFileVersionInfoSize(WinDir, &dwVerHnd);
	if (dwVerInfoSize > 0) {

		LPSTR lpstrVffInfo;
		HANDLE  hMem = 0L;
		char query_cStr[128], separatorA[128], separatorB[128], separatorC[128];

		hMem = GlobalAlloc(GMEM_MOVEABLE, dwVerInfoSize);
		if (hMem)	{

			lpstrVffInfo  = (char *) GlobalLock(hMem);

			//	c compiler must include "Version.lib" for this
			GetFileVersionInfo((LPSTR) WinDir, dwVerHnd,
				dwVerInfoSize, (LPVOID) lpstrVffInfo);
			
			//	get ptr to translation table information
			//	c compiler must include "Version.lib" for this
			bRetCode = VerQueryValue((LPVOID) lpstrVffInfo, 
				"\\VarFileInfo\\Translation",
				(LPVOID *) &lpTranslation, &puTranslationLen);

			if (bRetCode iNetAnd puTranslationLen > 0 iNetAnd lpTranslation != 0L)	{
				
				sprintf(query_cStr, "\\StringFileInfo\\%04x%04x\\ProductVersion",
					lpTranslation[0].wLanguageID, lpTranslation[0].wCharacterSet);

				/* Product Version */
				bRetCode = VerQueryValue((LPVOID) lpstrVffInfo, query_cStr,
					(LPVOID *) &lpVersion, &puVersionLen);
			
				if(bRetCode iNetAnd puVersionLen < VERS_LEN iNetAnd puVersionLen > 1) {

					//	Remember the Product Version (e.g. 1,22,3,0)
					strcpy(version_cStr, lpVersion);

					//	Extract major/minor/subminor version values
					//	(e.g. 1, 22, 3)
					sscanf(version_cStr, "%ld%s%ld%s%lds%ld", majorVerP,
						separatorA, minorVerP, separatorB, subminorVerP,
						separatorC, subSubVerP);
				}
			}					
			GlobalFree( hMem );
		}
	}
}

#endif

#endif

//	=============================== CLASS: ADVANCED UTILITY ===============================
//	=============================== CLASS: ADVANCED UTILITY ===============================
//	=============================== CLASS: ADVANCED UTILITY ===============================

/* -----------------------------------------------------------------------------------
 *	FUNCTION  							CLASS: ADVANCED UTILITY
 *
 *				iNet_Peek_int16()		Returns the signed 16bit integer value at 'address'.
 *				iNet_Peek_int32()		Returns the signed 32bit integer value at 'address'.
 *				iNet_Peek_flt32()		Returns the signed 32bit floating point value at 'address'.
 * 
 *	HISTORY		2/6/96	created - gsw
 * --------------------------------------------------------------------------------- */

iNetINT16 EXT_FUNC_TYPE iNet_Peek_int16(
	iNetUINT32 address)					//	address to read
{
	return *((iNetINT16 *) address);
}

iNetINT32 EXT_FUNC_TYPE iNet_Peek_int32(
	iNetUINT32 address)					//	address to read
{
	return *((iNetINT32 *) address);
}

iNetFLT32 EXT_FUNC_TYPE iNet_Peek_flt32(
	iNetUINT32 address)					//	address to read
{
	return *((iNetFLT32 *) address);
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  							CLASS: ADVANCED UTILITY
 *
 *				iNet_Poke_int16()		Sets the 16bit integer value at 'address'.
 *				iNet_Poke_int32()		Sets the 32bit integer value at 'address'.
 *				iNet_Poke_flt32()		Sets the 32bit floating point value at 'address'.
 * 
 *	HISTORY		2/6/96	created - gsw
 * --------------------------------------------------------------------------------- */

void EXT_FUNC_TYPE iNet_Poke_int16(	
	iNetUINT32 address,					//	address to write to
	iNetINT16 int16value)				//	value to write
{
	*((iNetINT16 *) address) = int16value;
}

void EXT_FUNC_TYPE iNet_Poke_int32(
	iNetUINT32 address,					//	address to write to
	iNetINT32 int32value)				//	value to write
{
	*((iNetINT32 *) address) = int32value;
}

void EXT_FUNC_TYPE iNet_Poke_flt32(	
	iNetUINT32 address,					//	address to write to
	iNetFLT32 flt32value)				//	value to write
{
	*((iNetFLT32 *) address) = flt32value;
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  							CLASS: ADVANCED UTILITY
 *
 *				iNet_GetStaticVar()		Returns value of internal static variable.
 *				iNet_SetStaticVar()		Sets value of internal static variable
 * 
 *				These are used to store up to 10 static variables; which are 
 *				initialized to 0.
 *
 *				This is used by crazy HP Vee.
 *
 *	HISTORY		2/6/96	created - gsw
 * --------------------------------------------------------------------------------- */

#define NUM_S_VARS 10					/*	# of static vars */

iNetINT16 giNetStatusVar[ NUM_S_VARS ] = { 0,0,0,0,0,0,0,0,0,0 };

short EXT_FUNC_TYPE iNet_GetStaticVar(	
	short address)						//	0...9 static var address
{
	if (address < 0) { return 0; }		//	error check
	if (address >= NUM_S_VARS) { return 0; }
	
	return giNetStatusVar[ address ];
}

void EXT_FUNC_TYPE iNet_SetStaticVar(	
	short address,						//	0...9 static var address
	short value)						//	value to load into var
{
	if (address < 0) { return; }		//	error check
	if (address >= NUM_S_VARS) { return; }
	
	giNetStatusVar[ address ] = value;
	
	return;
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	iNet_Get_VarPtr()		CLASS: ADVANCED UTILITY
 *
 *	PURPOSE		Returns the address of where in memory the passed variable is stored.
 * 
 *	HISTORY		2/6/96	created - gsw
 * --------------------------------------------------------------------------------- */

iNetUINT32 EXT_FUNC_TYPE iNet_Get_VarPtr(
	void *Addr_Of_A_Variable)			//	variable
{
	return (iNetUINT32) Addr_Of_A_Variable;
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	Get_iNet_Error()		CLASS: ADVANCED UTILITY
 *
 *	PURPOSE 	Returns the most recent iNet Error code.
 *
 *	HISTORY		2/4/96	created - gsw
 * --------------------------------------------------------------------------------- */
iNetError EXT_FUNC_TYPE Get_iNet_Error()
{										//	This is the error value returned by the
										//	last call to iNet().	
	return (iNetError) ((gRequestP != 0L) ? error_PARAM(gRequestP) : iNetErr_ControllerNotInitialized);
}

static iNetError Get_iNet_Error_local()	//	local version (not DLL)
{										//	This is the error value returned by the
										//	last call to iNet().	
	return (iNetError) ((gRequestP != 0L) ? error_PARAM(gRequestP) : iNetErr_ControllerNotInitialized);
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	Get_Last_iNet_Call()			CLASS: ADVANCED UTILITY
 *
 *	PURPOSE 	Returns a pointer to a structure that contains the parameters
 *				passed to the last instruNet() call.
 *
 *	HISTORY		2/4/96	created - gsw
 * --------------------------------------------------------------------------------- */
void EXT_FUNC_TYPE Get_Last_iNet_Call(
	iNetUINT8 **gRequestPP)				//	filled with a pointer to the request struct,
										//	which contains the parameters to the last iNet() call.
{
	*gRequestPP = gRequestP;
}


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	GetiNetTime_Double()
 *
 *	PURPOSE 	Returns time since reset, in a double variable.
 *				
 *	HISTORY		4/4/97		created - gsw
 * ----------------------------------------------------------------------------------- */
double EXT_FUNC_TYPE GetiNetTime_Double( void )
{
	iNetError e;
	char cStr[256];
	double time = 0.0;
	
	e = iNet_cString((iNetUINT8) netNum_1stController /* 1 */, (iNetUINT8) deviceNum_CONTROLLER /* 0 */, \
		(iNetUINT8) moduleNum_1stModule /* 1 */, (iNetUINT8) controller_ChanNum_C11_Time /* 11 */, \
		(iNetINT16) sgt_General /* -6 */, (iNetINT16) fldNum_General_valueEu /* 1 */, \
		(iNetINT16) intention_getValue /* 1 */, cStr);
									
										//	if we got an error, assume 0. gsw 7/28/98
	if (e != iNetErr_None) { time = 0.0; }

	sscanf (cStr, "%lf", &time);
	
	return time;
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  							CLASS: ADVANCED UTILITY
 *
 *				iNet_memcopy()			DLL version
 *				iNet_memcopy_local()	local version		
 *
 *	PURPOSE 	Moves 'numBytes' of data starting at 'srcAddr' address to 'destAddr'
 *				address. 
 *
 *	HISTORY		2/6/96	created - gsw
 * --------------------------------------------------------------------------------- */
void EXT_FUNC_TYPE iNet_memcopy(
	void *srcAddr,						//	source address
	void *destAddr,						//	destination address
	iNetUINT32 numBytes)				//	number of bytes to move
{
#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	=============== Macintosh ============== */

	BlockMoveData((const void *) srcAddr, (void *) destAddr, (Size) numBytes);

#elif (MACHINE_TYPE_ == WINTEL_iNet) 	/*	=============  Windows/Intel =========== */

	memcpy((void *) (destAddr), (const void *) (srcAddr), (size_t) (numBytes));

#endif
}
										
static void iNet_memcopy_local(			//	local version of memcopy (not DLL version)
	void *srcAddr,						//	source address
	void *destAddr,						//	destination address
	iNetUINT32 numBytes)				//	number of bytes to move
{
#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	=============== Macintosh ============== */

	BlockMoveData((const void *) srcAddr, (void *) destAddr, (Size) numBytes);

#elif (MACHINE_TYPE_ == WINTEL_iNet) 	/*	=============  Windows/Intel =========== */

	memcpy((void *) (destAddr), (const void *) (srcAddr), (size_t) (numBytes));

#endif
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	Flt32ToDouble_iNet()						CLASS: ADVANCED UTILITY
 *				DoubleToFlt32_iNet()
 *				
 *				Convert array of flt32 to array of double; or Convert array of double
 *				to array of flt32. srcAddr and destAddr can be the same buffer (i.e.
 *				this will work with data in place).	
 *
 *	HISTORY		2/6/96	created - gsw
 * --------------------------------------------------------------------------------- */
 										//	Generic uses {short, long, double}
void EXT_FUNC_TYPE Flt32ToDouble_iNet_Generic(
	long * /* (iNetFLT32 *) */ srcAddr_flt32Array,			
										//	source address (array of flt32's)
	double *destAddr_doubleArray,		//	destination address (array of doubles's)
	long numPoints)						//	number of points to convert
{
	register iNetFLT32 *srcAddr = (iNetFLT32 *) srcAddr_flt32Array;
	register double *destAddr = destAddr_doubleArray;

	srcAddr += (numPoints - 1L);		//	start with last point in case the
	destAddr += (numPoints - 1L);		//	src and dest buffers are the same
	
	while (numPoints-- > 0L)	{
		*destAddr-- = (double) *srcAddr--;
	}
}
		void EXT_FUNC_TYPE Flt32ToDouble_iNet_Vee(long /* (iNetFLT32 *) */ srcAddr_flt32Array,
			double *destAddr_doubleArray, long numPoints)
		{
			Flt32ToDouble_iNet_Generic((long *) srcAddr_flt32Array, destAddr_doubleArray, numPoints);
		}
 										//	Generic uses {short, long, double}
void EXT_FUNC_TYPE DoubleToFlt32_iNet_Generic(
	double *srcAddr_doubleArray,		//	source address (array of doubles's)
	long * /* (iNetFLT32 *) */ destAddr_flt32Array,		
										//	destination address (array of flt32's)
	long numPoints)						//	number of points to convert
{
	register double *srcAddr = srcAddr_doubleArray;
	register iNetFLT32 *destAddr = (iNetFLT32 *) destAddr_flt32Array;
	while (numPoints-- > 0L)	{
		*destAddr++ = (iNetFLT32) *srcAddr++;
	}
}
		void EXT_FUNC_TYPE DoubleToFlt32_iNet_Vee( double *srcAddr_doubleArray,
			long /* (iNetFLT32 *) */ destAddr_flt32Array, long numPoints)	
		{
			DoubleToFlt32_iNet_Generic( srcAddr_doubleArray, (long *) destAddr_flt32Array, numPoints);
		}

//	=============================== CLASS: SUPPORT ===============================
//	=============================== CLASS: SUPPORT ===============================
//	=============================== CLASS: SUPPORT ===============================

/* -----------------------------------------------------------------------------------
 *	FUNCTION  									CLASS: SUPPORT
 *
 *				iNet_Dispose_Handle()			Dispose of handle, external version.
 *				iNet_Dispose_Handle_local()		Dispose of handle, local version.	
 *				iNet_Unlock_Handle_local()		Unlock handle, local version.
 *				iNet_Lock_Handle_local()		Lock handle, local version.
 * 
 *	HISTORY		3/15/97	created - gsw
 * --------------------------------------------------------------------------------- */

void EXT_FUNC_TYPE iNet_Dispose_Handle(
	char *handleInRam)					//	handle to dispose of 
{
	if (handleInRam == (iNetUINT32) 0L) { return; }

#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	=============== Macintosh ============== */

	#if OLDROUTINENAMES					/*	codewarrior 1.6 */
		DisposHandle((*(Handle *) &handleInRam));	
	#else								/*	codewarrior >= 3 */
		DisposeHandle((*(Handle *) &handleInRam));	
	#endif
	
#elif (MACHINE_TYPE_ == WINTEL_iNet) 	/*	=============  Windows/Intel =========== */

	GlobalFree((*(HGLOBAL *) &handleInRam));	

#endif
}

static void iNet_Dispose_Handle_local(	//	local version (not DLL version)
	char *handleInRam)					//	handle to dispose of 
{
	if (handleInRam == (iNetUINT32) 0L) { return; }

#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	=============== Macintosh ============== */

	#if OLDROUTINENAMES					/*	codewarrior 1.6 */
		DisposHandle((*(Handle *) &handleInRam));
	#else								/*	codewarrior >= 3 */
		DisposeHandle((*(Handle *) &handleInRam));	
	#endif
		
#elif (MACHINE_TYPE_ == WINTEL_iNet) 	/*	=============  Windows/Intel =========== */

	GlobalFree((*(HGLOBAL *) &handleInRam));	

#endif
}

static Boolean iNet_Lock_Handle_local(	//	local version (not DLL version)
	char *handleInRam,					//	handle to lock
	iNetUINT32 *hSizeBytesP, 			//	if != 0L, we load with handle size
	iNetINT16 **hAddressPP) 			//	if != 0L, we load with address to block
{
	Boolean weDidOk = TRUE;
	char *hAddress;
	
	if (handleInRam == (iNetUINT32) 0L) { return FALSE; }

#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	=============== Macintosh ============== */
	
	HLock( *(char***) &handleInRam );	//	lock the handle
	
										//	get the size of the data in the handle
	if (hSizeBytesP != 0L) { *hSizeBytesP = GetHandleSize( *(char***) &handleInRam ); }
	
	hAddress = *(*(char***) &handleInRam);

#elif (MACHINE_TYPE_ == WINTEL_iNet) 	/*	=============  Windows/Intel =========== */
	
	if (hSizeBytesP != 0L) { *hSizeBytesP = GlobalSize( *(HGLOBAL *) &handleInRam ); }
	
	hAddress = (char *) GlobalLock( *(HGLOBAL *) &handleInRam );
										//	not necessary: if( p ) { __msWin_IncrementLockCount( h ); }
#endif

	if (hAddressPP != 0L) { *hAddressPP = (iNetINT16 *) hAddress; }
	
	if (hAddress == 0L) { weDidOk = FALSE; }
	
	return weDidOk;
}


static void iNet_Unlock_Handle_local(	//	local version (not DLL version)
	char *handleInRam)					//	handle to unlock 
{
	if (handleInRam == (iNetUINT32) 0L) { return; }
										//	unlock the handle
#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	=============== Macintosh ============== */

	HUnlock( *(char***) &handleInRam );		

#elif (MACHINE_TYPE_ == WINTEL_iNet) 	/*	=============  Windows/Intel =========== */

	GlobalUnlock( *(HGLOBAL *) &handleInRam );

#endif
}

/* -----------------------------------------------------------------------------------
 *	FUNCTION  	Get_DLL_Globals()					CLASS: ADVANCED DIGITIZE				
 *
 *	PURPOSE 	Returns global variables within the DLL.
 *
 *	HISTORY		10/16/97	created - gsw
 * --------------------------------------------------------------------------------- */
iNetError EXT_FUNC_TYPE Get_DLL_Globals(
	iNetUINT32 /* HWND */ *ghwndPortWnd_,//	Window target for graphics calls
	iNetUINT32 /* HDC */ *ghdcPortDC_,   //	Device context used in graphics calls.  It is set in 
										//	SetPort_ so that the graphics routines don't need to fetch
										//	the device context for ghwndPortWnd individually.
	iNetUINT32 /* HCURSOR */ *ghMSCursor_,//	Used by SetCursor_iNet
										//	Pen globals used to track pen characteristics so that they
										//	can be changed a la the MAC.  e.g. Changing the foreground
										//	color on the MAC implicitly changes the color of the pen.
	iNetUINT32 /* HINSTANCE */ *ghInstance_)
{
	iNetUINT32 uint32LocalValue;
	iNetError e;
										//	tell iNet driver to pull data out of buffers
	e = (iNetError) iNet(0, 0, 0, 0, 0, 0, intention_getDLLglobals,  iNetDT_UINT32, &uint32LocalValue);
	
										//	return the info to the caller
	if (ghwndPortWnd_ != 0L)			{ *ghwndPortWnd_= A_PARAM(gRequestP); }
	if (ghdcPortDC_ != 0L)				{ *ghdcPortDC_ 	= B_PARAM(gRequestP); }
	if (ghMSCursor_ != 0L)				{ *ghMSCursor_ 	= C_PARAM(gRequestP); }
	if (ghInstance_ != 0L)				{ *ghInstance_ 	= D_PARAM(gRequestP); }

	return e;
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	iNet()				 	CLASS: SUPPORT
 *
 *	PURPOSE 	This is the main entry point fo the instruNet DRIVER.  This routine 
 *				reads and/or writes instruNet fields in the network hierarchy. 
 *				
 *	ON RETURN	Returns an iNet Error code.
 *				
 *	HISTORY		5/3/95	created - gsw
 * --------------------------------------------------------------------------------- */

#define  NUM_BEEPS_IF_NO_DRIVER_FILE  4

#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	=============== Macintosh ============== */
	
	//	------------------------------------------------------------------
	//	Main Entry Point Universal Procedure Pointer info.  These
	//	are used by the Macintosh to call an external sub-routine in
	//	a code resource, that might be compiled differently than
	//	the calling application.  Apple tells us to do this.
		
	typedef pascal void (*iNet_ProcPtr) (iNetUINT8 *);
	
	enum {
		upp_iNet_ProcInfo = kPascalStackBased
			 | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(iNetUINT8 *)))
	};
										// #if 1	//	9/24/98
										//	68K to 68K do not need UPP
										//	old ppc compiler to old ppc compiler do not need UPP
										//	new ppc compiler to old ppc compiler do not need UPP
	#if (USESROUTINEDESCRIPTORS || ((COMPILING_FOR_PPC_MAC && TARGET_OS_MAC) && TARGET_RT_MAC_CFM))
										// #else
										//  	if USESROUTINEDESCRIPTORS
										// #endif
		typedef UniversalProcPtr iNet_UPP;
		#define CALL_iNet_PROCEDURE(userRoutine, p)   \
			CallUniversalProc((UniversalProcPtr) userRoutine, upp_iNet_ProcInfo, p)
	#else

		typedef iNet_ProcPtr iNet_UPP;
		#define CALL_iNet_PROCEDURE(userRoutine, p)   (*userRoutine) (p)

	#endif
	
	typedef iNet_UPP iNet_Main_EntryPtr;

#elif (MACHINE_TYPE_ == WINTEL_iNet) 	/*	=============  Windows/Intel =========== */

										//	ms compiler 6 sometimes shows a warning
										//	w/ CALL_iNet_PROCEDURE() since it thinks it
										//	is defined twice, yet it is really defined
										//	once yet the MS compiler 6 thinks the Mac
										//	code is included when it is not.
	typedef void (pascal* iNet_ProcPtr) (iNetUINT8 *);
	typedef iNet_ProcPtr iNet_Main_EntryPtr;
	#define CALL_iNet_PROCEDURE(userRoutine, p)	(*userRoutine)(p)
	
#endif

static iNetError LoadDriverCodeIntoRam(iNet_Main_EntryPtr *iNet_Main_EntryPtrP,
	Boolean hollerIf_iNet_DriverFileIsAbsent, iNetINT32 *iNetFile_ResRefNum);
	
iNetINT16 iNet(							/*	This is not a DLL since other functions in this file
										 *	call it.  If you want a DLL version of iNet(), call
										 *	iNet_DLL(). */

	iNetUINT8 netNum,					//	NETWORK number = {0...numNetworks}, 0 is Driver,
	iNetUINT8 devNum,					//	DEVICE number {0...numDevices}, 0 = "Controller", 1 = 1st device on network
	iNetUINT8 modNum, 					//	MODULE number within a hardware DEVICE {1...32}.
										//	Many devices have only 1 module.
	iNetUINT8 chNum, 					//	Hardware CHANNEL number {1...32}. Each device contains
										//	a number of channels, each of which has it's own
										//	channel number.
	iNetINT16 settingGroupNumOrType,	//	if < 0, settings Type = { sgt_VinHardware (-3), sgt_Vout (-4), etc. }
										//	if > 0, settings Number = Setting popup menu choice.
	iNetINT16 fieldNum,					//	Field number within the settingGroup = { 1...8}
										//	See fgNums_... enum's in iNet_INET.H file.
	iNetINT16 intention,				//	Tells iNet() what to do with the field.  Please
										//	ion_intention enum for details. ion_intention =
										//	{intention_getValue, intention_setValue,
										//	intention_getNameStr, intention_getMaxValue, etc}
	iNetUINT8 argType,					//	Argument type. iNetDataType =
										//	{iNetDT_INT16 (0), iNetDT_UINT16 (2),
										//	iNetDT_INT32 (3), iNetDT_UINT32 (4), 
										//	iNetDT_FLT32 (5), iNetDT_Double (6),
										//	iNetDT_P_Str15 (8), iNetDT_P_Str255 (9)
										//	iNetDT_C_Str15 (10), iNetDT_C_Str255 (11)
	void *ptrToArg)						//	this is where data of type 'argType' is kept
										//	(we read or write to this location)
{
	iNetINT16 i;
	
	//	-----------------------------------------------------------------
	//	If we are here for the 1st time, initialize the operating system,
	//	if necessary, and prepare the request struct.

										//	if we are here for the 1st time...
	if (giNet_Main_EntryPtr_ == (void *) 0L)	{	
		
		gRequestP = &gRequest_struct[0];//	remember the address of the gRequest_struct that
										//	is used to communicate with the driver
		
										//	strip off unused bits,
		gRequestP = (iNetUINT8 *) StripAddress_interface((char *) gRequestP);
		
										//	Initialize the operating system, if it has not
										//	yet been initialized (segment loader needs
										//	initialization before we load segments, if
										//	on an old cpu, like a Mac II).  Also,
										//	UseCurRes() must point to the iNet Driver code 
										//	rsrc file when we 1st call iNet,
										//	else the SegmentLoaded has trouble.
		Initialize_OperatingSystem_If_Necessary_main();
																				
										//	clear the request struct
		for (i = 0; i < SIZE_OF_iNet_Request; i++)	{
			*(((unsigned char *) &gRequest_struct[0]) + i) = 0x00;
		}
										//	load the key field (the driver
										//	will look to see that this is
										//	correct).
		key_PARAM(&gRequest_struct[0]) = 0x55b4;

										//	if the variable sizes are not
										//	correct, this interface file will
										//	not work.
		if ((sizeof(iNetUINT8) != 1)	||	(sizeof(iNetINT8)  != 1)	||
			(sizeof(iNetUINT16) != 2)	||	(sizeof(iNetINT16) != 2)	||
			(sizeof(iNetUINT16) != 2)	||	(sizeof(iNetINT32) != 4)	||
			(sizeof(iNetUINT32) != 4)	||	(sizeof(iNetFLT32) != 4)	||
			(sizeof(iNetUINT8 *) != 4))	{
			
										//	Check your compiler options or
										//	the definitions of the above
										//	types, to make sure that
										//	these sizes are correct.
			return (iNetError) (error_PARAM(&gRequest_struct[0]) = iNetErr_InterfaceCompiledBadly);
		}

										/*	=========== Windows/Intel =========== */
		#if (MACHINE_TYPE_ == WINTEL_iNet)
		
										/*	if we are NOT building tiny app to open window... */  
			#if !defined(BUILD_SNUB_APL_TO_CALL_iNet_DRIVER)

										//	get the DLL version
				Get_Win32_DLL_VersionNum(iNetDRIVER_NAME /* e.g. "iNet32.dll" */,
					gDLL_ver_cStr 			/* e.g. "1.22.4" */,
					&gDLL_ver_majorVer		/* e.g. 1 */, 
					&gDLL_ver_minorVer		/* e.g. 22 */, 
					&gDLL_ver_subminorVer 	/* e.g. 4 */,
					&gDLL_ver_subSubVer 	/* e.g. 4 */);

			#endif

		#endif
	}

	//	-----------------------------------------------------------------
	//	Load parameters into the gRequest_struct, which is passed to
	//	the driver (i.e. a pointer to it).
	
	netNum_PARAM( &gRequest_struct )	= netNum;
	deviceNum_PARAM( &gRequest_struct )	= devNum;
	moduleNum_PARAM( &gRequest_struct )	= modNum;
	chanNum_PARAM( &gRequest_struct )	= chNum;

	settingGroupNum_PARAM(&gRequest_struct) = settingGroupNumOrType;
	settingGroupType_PARAM(&gRequest_struct) = 0;

	fieldNum_PARAM( &gRequest_struct )	= fieldNum;
	intention_PARAM( &gRequest_struct )	= intention;
	argType_PARAM( &gRequest_struct )	= argType;
	ptrToArg_PARAM( &gRequest_struct )	= ptrToArg;
	error_PARAM( &gRequest_struct )		= iNetErr_DriverDidNotSetErrCode;

										#if 0
										static iNet_Request request;
										request.netNum = netNum;
										request.devNum = devNum;
										request.modNum = modNum;
										request.chNum = chNum;
										
										request.settingGroupNum = settingGroupNumOrType;
										request.settingGroupType = 0;
										
										request.fieldNum = fieldNum;
										request.intention = intention;
										request.argType = argType;
										request.ptrToArg = ptrToArg;
										request.error = iNetErr_DriverDidNotSetErrCode;
										#endif
	
	//	-----------------------------------------------------------------
	//	Load the iNet Driver into RAM, if necessary.
								 		
										/* 	if DRIVER code is compiled into a CODE rsrc (or DLL)... */
#if !defined(DRIVER_IS_COMPILED_INTO_AN_APP)		

	if (giNet_Main_EntryPtr_ == (void *) 0L)	{
										//	if we have not yet loaded the driver CODE into RAM,
										//	(i.e. this is the 1st call to
										//	the DLL/codeRsrc).
										
										/*	=============== Macintosh ============== */
		#if (MACHINE_TYPE_ == MACINTOSH_iNet)	
			
			iNetINT16 currentRsrc_org = 0;
			iNetINT32 iNetFile_ResRefNum = 0L;
										//	Store the 1st resource file (i.e.  
										//	The rsrc file that is 1st in the list of 
										//	attached files) for the 1st call to
										//	iNet (since iNet will change it).
			currentRsrc_org = CurResFile();		
			
										//	On the 1st call to a Mac CodeRsrc, we
										//	tsfr a ptr to the QuickDraw globals
										//	in the F Param.
										//
										//	Careful, if this file is compiled in a
										//	a4 based 68K code rsrc, the reference to a5 based
										//	qd globals might not work.  You might need the
										//	trick described in IM Memory pg4-18. The
										//	"CurrentA5(), qdP = &dq" trick often results
										//	in LINKER bad resolved references, which 
										//	will screw you at run time.
			F_PARAM(&gRequest_struct[0]) = (iNetUINT32) &qd;

			if (&qd == 0L)	{			//	if the quickdraw globals ptr is bad...
				error_PARAM(&gRequest_struct) = iNetErr_CrtRect_Failed;
				goto Restore;			//	bail out with error	
			}
										//	if the quickdraw globals ptr is bad...
			if (qd.screenBits.bounds.left < -4000	||  qd.screenBits.bounds.left > +4000	||
				qd.screenBits.bounds.right < -4000	||  qd.screenBits.bounds.right > +4000	||
				qd.screenBits.bounds.top < -4000	||  qd.screenBits.bounds.top > +4000	||
				qd.screenBits.bounds.bottom < -4000 ||  qd.screenBits.bounds.bottom > +4000)	{

										//	bail out with error			
				error_PARAM(&gRequest_struct) = iNetErr_CrtRect_Failed;
				goto Restore;
			}
										//	Load Driver Code into RAM (Macintosh).
										//	We will holler (e.g. beep) if the 
										//	instruNet Driver file is absent only if the
										//	caller is not asking for the # of networks,
										//	or whether or not an instruNet Driver file exists).
			error_PARAM(&gRequest_struct) = LoadDriverCodeIntoRam((iNet_Main_EntryPtr*) &giNet_Main_EntryPtr_,
											(intention != intention_GetNumNetworks),
											&iNetFile_ResRefNum);

										/*	============  Windows/Intel =========== */
		#elif (MACHINE_TYPE_ == WINTEL_iNet) 	

										//	Load Driver Code into RAM (Windows).
										//	The last parameter is not used.
			error_PARAM(&gRequest_struct) = LoadDriverCodeIntoRam((iNet_Main_EntryPtr*) &giNet_Main_EntryPtr_,
											(Boolean) (intention != intention_GetNumNetworks),
											(iNetINT32 *) NULL);
		#endif

										/*	=============== Macintosh ============== */
		#if (MACHINE_TYPE_ == MACINTOSH_iNet)	

										//	On the 1st call to a Mac CodeRsrc, we
										//	tsfr the instruNet Driver file rsrcFileRefNum
										//	in the "D" parameter.
			D_PARAM(&gRequest_struct[0]) = (iNetUINT32) iNetFile_ResRefNum;

		#endif
										//	Strip of unused bits.
		giNet_Main_EntryPtr_ = (void *) StripAddress_interface((char *) giNet_Main_EntryPtr_);

										//	If we got an error, bail out.
		if (error_PARAM(&gRequest_struct) != iNetErr_None)	{	
			goto Restore;				
		}
										//	If we could not locate the driver, bail
										//	out with an error code that says this.
		if (giNet_Main_EntryPtr_ == (void *) 0L)	{
			error_PARAM(&gRequest_struct) = iNetErr_CouldNotLocateDriverFile;
			goto Restore;				
		}
										//	Call the main entry point of the driver
										//	(the driver code is typically in a Macintosh 
										//	code resource or Windows DLL) for the
										//	1st time.  If we do not call the 
										//	Driver for the first time with the
										//	Macintosh 1st Res File set to the 
										//	code rsrc file on a 68K Mac, it will crash.
		CALL_iNet_PROCEDURE(((iNet_Main_EntryPtr) giNet_Main_EntryPtr_), ((iNetUINT8 *) &gRequest_struct[0]));

	Restore:
										/*	=============== Macintosh ============== */
		#if (MACHINE_TYPE_ == MACINTOSH_iNet)	
			
										//	Restore the 1st resource file (i.e.  
										//	The rsrc file that is 1st in the list of 
										//	attached files).  The segment loaded
										//	for the Mac 68k code rsrc wants the
										//	1st file in the list to be the code
										//	rsrc, which is taken care of by
			UseResFile(currentRsrc_org);
	
		#endif

		return error_PARAM(&gRequest_struct);
	}
	else	{							//	else we are on our >= 2nd call to
										//	the CodeRsrc/DLL.
	
										//	Call the main entry point of the driver:
										//	
										//	Windows95:
										//
										//		The code we call is a 32bit DLL.
										//
										//	Macintosh:
										//
										//		If on a ppc motherboard:
										//
										//			The code we call is a ppc compiled
										//			code rsrc in file "instruNet Driver (ppc)".
										//			If this calling INT.c file is 68K compiled
										//			and we are running under emulation, the
										//			1st code rsrc line that is executed is a 
										//			mixed mode trap that switches into ppc native.
										//
										//		If on a 68K motherboard:
										//
										//			The code we call is a 68K compiled code rsrc 
										//			in file "instruNet Driver (68K&FPU)".
										//
		CALL_iNet_PROCEDURE(((iNet_Main_EntryPtr) giNet_Main_EntryPtr_), ((iNetUINT8 *) &gRequest_struct[0]));
	}	
										
#else									/* 	else DRIVER code is compiled into an application... */

										//	make dummy call to turn off compiler warnings
	LoadDriverCodeIntoRam((iNet_Main_EntryPtr*) 0L, 0L, 0L);

										//	if we are here for the 1st time...
	if (giNet_Main_EntryPtr_ == (void *) 0L){
										
										//	mark that we have already been here
		giNet_Main_EntryPtr_ = (void *) 2L;
	}
										//	call the main entry point of the driver directly
										//	(let the linker link us)
										//	(the driver code is typically in a code resource)
	iNet_Main_Entry(&gRequest_struct[0]);	
	
#endif
										//	return an iNetError code
	return error_PARAM(&gRequest_struct[0]);
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	LoadDriverCodeIntoRam()			CLASS: SUPPORT		
 *
 *	PURPOSE 	This routine will load the instruNet Driver code into RAM, lock it down
 *				and let it sit there forever.  This is down once, when iNet()
 *				is first called.
 *
 *				When we open Mac rsrc file, it becomes the 1st rsrc file.
 *				
 *	ON RETURN	Returns an iNet Error code.
 *				
 *	HISTORY		5/3/95	created - gsw
 *				1/22/96 kag - modified Windows code
 *				3/1/96	added 'hollerIf_iNet_DriverFileIsAbsent' - gsw
 * --------------------------------------------------------------------------------- */

static iNetError LoadDriverCodeIntoRam(
	iNet_Main_EntryPtr *iNet_Main_EntryPtrP,
	Boolean hollerIf_iNet_DriverFileIsAbsent,
										//	if true, we holler if the instruNet
										//	driver is absent (e.g. beep)
	iNetINT32 *iNetFile_ResRefNum)		//	we load this with the instruNet file
										//	rsrc File Ref Num, if on a Macintosh.
									
{
#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	============= Macintosh =========== */
	
	iNetError retErrCode;
	OSErr e;
	SysEnvRec dummyA, macEnv, dummyB;
	short vRefNum, WDrefNum = 0, ioWDVRefNum_org, orgResFile, currentVolume_org;
	unsigned char currentVolumeName_org[300], name[300];
	long ioWDDirID_org, dirID, result;
	Boolean weAreOnA_PPC_Motherboard, weFoundTheDriverFile;
 	OSErr gesaltErrorVal;
	FSSpec driverFileSpec;
	FCBPBRec pbRec;
	
	static weAlreadyBeepedDueToError = FALSE;

	dummyA.systemVersion = dummyB.systemVersion = 0L;
	
	//	--------------------------------------------------------------
	//	Driver File Names.
	//
	//	We have 2 version of the driver, ppc and 68K.  If on a ppc motherboard,
	//	we use the ppc version; otherwise, 68K.  If on a ppc motherboard
	//	and the application is running in 68K emulation mode, then it
	//	should switch into ppc native, and run the ppc driver.

	#define		iNetDRIVER_NAME_68KFPU_Mac		"\pinstruNet Driver (68K&FPU)"
	#define		iNetDRIVER_NAME_PPC_Mac			"\pinstruNet Driver (ppc)"

	#define		iNetDRIVER_FILE_CREATOR_Mac		'ioNe'
 	#define		iNetDRIVER_RSRC_TYPE_Mac		'ioNd'
 	#define		iNetDRIVER_RSRC_NUM_Mac			1

	//	--------------------------------------------------------------
	//	Initialize vars.

	gRsrcH = 0L;

	//	make sure we got a good place to store file
	if (iNet_Main_EntryPtrP == (void *) 0L)	{ return iNetErr_CouldNotLocateDriverFile; }

	*iNetFile_ResRefNum	 = 0L;		
	*iNet_Main_EntryPtrP = 0L;
	weFoundTheDriverFile = FALSE;
	retErrCode			 = iNetErr_CouldNotLocateDriverFile;

	//	--------------------------------------------------------------
	//	Initialize the Macintosh OS, if it has not yet been initialized.
	//
	//	This is a little tricky, since the caller might do this later on
	//	themselves, yet the iNet Code rsr will do this any way.  It
	//	is safer to do this before we do any Mac file or SysEnvirons()
	//	work.  Generally, the caller should init the Mac OS before
	//	calling iNet().  When that happens, we see that it is
	//	init'ed, and then do nothing in 
	//	Initialize_OperatingSystem_If_Necessary_main().

	Initialize_OperatingSystem_If_Necessary_main();
	
	//	--------------------------------------------------------------
	//	Get info about the Macintosh Operating System.  On some
	//	computers, you might need to Init the Mac OS before calling
	//	SysEnvirons().

	e = SysEnvirons(1, &macEnv);		//	get info on the environment

	result = CurResFile();				//	clear any error codes in globals, gsw 8/28/96
	UseResFile(result);					//	(we do this to fix a bug in the SoftFPU control panel).

	if (e != noErr)		{				//	if we got an error...
		return (iNetError) OSErr_TO_iNetError(e);
	}
	//	--------------------------------------------------------------
	//	Bail out if on Macintosh < System 7.0 (we are only compatible
	//	with System 7.0 and beyond).

	if (macEnv.systemVersion < 0x0700) { return iNetErr_NonCompatible_OS; }

	//	--------------------------------------------------------------
	//	Determine if our mother board has a PPC or 68K processor.

	weAreOnA_PPC_Motherboard = FALSE;
	if (macEnv.systemVersion >= 0x0710){//	if >= than system 7.1			
			
										//	ask about the motherboard processor
 		gesaltErrorVal = Gestalt('sysa', &result);
 			
    									//	with 'sysa' result is 1 if 68K, 2 if ppc
		if (gesaltErrorVal == noErr  iNetAnd  (result != 1))	{
  			weAreOnA_PPC_Motherboard = TRUE;
		}
	}

	//	--------------------------------------------------------------
	//	If we are on a 68K motherboard, make sure it has an fpu.

	if (!weAreOnA_PPC_Motherboard)	{	//	if on a 68K motherboard...
	
		if (!macEnv.hasFPU)	{			//	if this computer does NOT have a floating point
										//	processor, then bail (driver requires it).
										//	(ppc motherboard will report us as not having a
										//	an fpu since an fpu is 881 and the ppc
										//	does not support the 881 emulation).
		
			return iNetErr_DrvrNeedsFpu_Failed;
		}
	}

	//	--------------------------------------------------------------
	//	Store FILE state.
										//	store the current volume	
	GetVol(currentVolumeName_org, &currentVolume_org);	
	ioWDDirID_org = LMGetCurDirStore();	//	Working Directory "Directory ID #"
	ioWDVRefNum_org = LMGetSFSaveDisk();//	Working Directory "= negated Vol Ref Num"

	//	--------------------------------------------------------------
	//	We will now look for the driver in the System Extensions folder.

										//	Locate the Extensions folder, within the System folder
	e = FindFolder(kOnSystemDisk, kExtensionFolderType, kDontCreateFolder, &vRefNum, &dirID);	
 			
 	if (e == noErr iNetAnd dirID != 0L){//	if there was no error 
 				
										//	make FSSpec for driver in the System Extensions folder
		e = FSMakeFSSpec(vRefNum, dirID, weAreOnA_PPC_Motherboard ? iNetDRIVER_NAME_PPC_Mac : iNetDRIVER_NAME_68KFPU_Mac, &driverFileSpec);

		if (e == noErr)	{				//	if we found it ok...
		
										//	try to open the resource fork to the driver file
			gFileRefNum = FSpOpenResFile(&driverFileSpec, fsRdPerm);

	 									//	if we found it, go load it
			if ((e = ResError()) == noErr  &&  gFileRefNum != -1) 	{
																
										//	we found the driver file
				weFoundTheDriverFile = TRUE;
			}
		}
	}

	//	--------------------------------------------------------------
	//	We will now look for the driver in the same folder as the
	//	application program.

	if ( ! weFoundTheDriverFile)	{	//	if we have not yet found the driver file...

										//	get the volume ref num of the application file
		e = GetVRefNum(LMGetCurApRefNum(), &vRefNum);
	
		if (e == noErr)	{
							
			unsigned char *p = (unsigned char *) &pbRec;
			long size = sizeof( FCBPBRec );

			while (size--)	{			//	clear the struct
				*p++ = 0x00; 
			}			

			pbRec.ioVRefNum = vRefNum;
			pbRec.ioRefNum = LMGetCurApRefNum();
			pbRec.ioNamePtr = name;

										//	get the dirID of the parant folder to the application file
			e = PBGetFCBInfo (&pbRec, FALSE);
			
			if (e == noErr iNetAnd pbRec.ioFCBParID != 0)	{
			
										//	make FSSpec for driver in the same directory as
										//	as the application file
				e = FSMakeFSSpec(vRefNum, pbRec.ioFCBParID,
					(weAreOnA_PPC_Motherboard ? iNetDRIVER_NAME_PPC_Mac : iNetDRIVER_NAME_68KFPU_Mac),
					&driverFileSpec);

										//	if we found it ok...
				if (e == noErr iNetAnd driverFileSpec.parID != 0L)	{
						
										//	open the resource fork to the driver file
					gFileRefNum = FSpOpenResFile(&driverFileSpec, fsRdPerm);

		 								//	if we found it, go load it
					if ((e = ResError()) == noErr  &&  gFileRefNum != -1) 	{
											
										//	we found the driver file
						weFoundTheDriverFile = TRUE;
					}
				}
			}
		}
	}

	//	--------------------------------------------------------------
	//	Load the driver into ram.

	if (weFoundTheDriverFile)	{		//	if we found the driver file...

										//	load the code resource, and get a handle to it
										//
										//	We only load the 'ioNd' code segment, which
										//	is the 1st code rsrc segment (e.g. 68k).  The
										//	other code rsrc segments are in 
										//	rsrc 'CCOD' and get loaded with LoadSeg()
										//	when a function in there is accessed --
										//	we do this (i.e. load all segments, e.g. 68k)
										//	on the 1st call to iNet().  They are marked
										//	preLoad, protected, locked, therefore
										//	LoadSeg(68k) should not need to be called
										//	again after this.

										
		orgResFile = CurResFile();		//	store the current resource file
		UseResFile( gFileRefNum );		//	make this the current resource
		if ((e = ResError()) == noErr)	{

										//	get the driver code
			gRsrcH = Get1IndResource( iNetDRIVER_RSRC_TYPE_Mac, iNetDRIVER_RSRC_NUM_Mac );
			e = ResError();				//	get the error...

										//	if we loaded the rsrc ok
			if (gRsrcH  &&  (e == noErr))	{
			
				//	Don't DetachResource(gRsrcH) since it will no longer look like a 
				//	rsrc if we do this, and this is not necessary.  We want the
				//	'ioNd' loaded and locked down.

				MoveHHi ( gRsrcH );		//	put the driver code into high memory, and lock it down
				HLock ( gRsrcH) ;		//	put the driver code into high memory, and lock it down

										//	return the file rsrc ref num of the driver
				*iNetFile_ResRefNum = (iNetINT32) gFileRefNum;
								
										//	return ptr to the code. in the case
										//	of ppc code rsrc, the UPP is stored
										//	at the beginning of the code rsrc file,
										//	therefore, we do not need to create a UPP.
				*iNet_Main_EntryPtrP = (iNet_Main_EntryPtr) *gRsrcH;
				
				retErrCode = iNetErr_None;//	good
			}
			else	{					//	else we failed to load the rsrc

										//	(we'll assume it's due to not enought ram)
				retErrCode = iNetErr_RanOutOfMemory;
			}
		}
		UseResFile( orgResFile );		//	restore the current resource file
	}

	//	--------------------------------------------------------------
	//	Else, we could not locate driver file.

	else	{							//	else we could NOT find the driver file...

										//	if we are to holler if the iNet
										//	driver is not in place...
		if (hollerIf_iNet_DriverFileIsAbsent)	{
										
										//	if we already beeped 4x because we did 
			if (!weAlreadyBeepedDueToError)	{	
										//	not find the driver file...
				#if OLDROUTINENAMES		/*	codewarrior 1.6 */
					long finalTicks;
				#else					/*	codewarrior >= 3 */
					unsigned long finalTicks;
				#endif
				short i;
			
				Delay (60, &finalTicks);//	delay for 1sec to isolate audio signal, gsw 3/1/96
	
										//	beep 6 times to indicate that could
										//	not get driver file going
				for (i = 0; i < NUM_BEEPS_IF_NO_DRIVER_FILE; i++)	{
			
					SysBeep (12);		//	beep for 200ms
									
					Delay (30, &finalTicks);//	delay for 500ms
				}
	
				Delay (60, &finalTicks);//	delay for 1sec to isolate audio signal, gsw 3/1/96
		
				weAlreadyBeepedDueToError = TRUE;
			}
		}
	}

	//	-------------------------------
	//	Restore FILE state.
										//	restore the current volume
	SetVol(currentVolumeName_org, currentVolume_org);	
	LMSetCurDirStore(ioWDDirID_org);	//	Working Directory "Directory ID #"
	LMSetSFSaveDisk(ioWDVRefNum_org);	//	Working Directory "= negated Vol Ref Num"
	
	return retErrCode;					//	exit
	

#elif (MACHINE_TYPE_ == WINTEL_iNet) 	/*	============  Windows/Intel =========== */

										//	turns off compiler warnings
	STOP_COMPILER_WARNING( iNetFile_ResRefNum );
	STOP_COMPILER_WARNING( hollerIf_iNet_DriverFileIsAbsent );

										//	make sure we got a good place to store file
	if (iNet_Main_EntryPtrP == (void*) 0L)	{ return iNetErr_CouldNotLocateDriverFile; }

	//	--------------------------------------------------------------
	//	Locate the instruNet Driver DLL inside the
	//	system directory (Windows 95).
										
	#define  iNetENTRY_POINT_NAME  "iNet_Main_Entry"
	
										//	Load DLL library file.
										/*	Win95: "Win95\system\iNet32.dll" */
										/*	WinNT: "WinNT\system32\iNet32.dll" */
    if (!(gLib = LoadLibrary (iNetDRIVER_NAME /* "iNet32.dll" */)))	{
    
		return iNetErr_CouldNotLocateDriverFile;
	}
    else	{
    

		//	Needed to track the handle to the DLL
		//	It is defined in ionc_gui.c
		
		*iNet_Main_EntryPtrP = 
			(iNet_ProcPtr) GetProcAddress (gLib, iNetENTRY_POINT_NAME);
			
		return (iNetError)(((*iNet_Main_EntryPtrP != 0L) ? iNetErr_None : iNetErr_CouldNotLocateDriverFile));
    }
#endif
}

/* -------------------------------------------------------------------------------------
 *	FUNCTION  	Initialize_OperatingSystem_If_Necessary_main()		CLASS: SUPPORT
 *
 *	PURPOSE 	Initializes operating system utilities, if they are not already
 *				initialized.
 *				
 *	HISTORY		1/1/96	created - gsw
 *				1/1/96  kag - Windows code not needed
 * ----------------------------------------------------------------------------------- */
void Initialize_OperatingSystem_If_Necessary_main()
{
#if (MACHINE_TYPE_ == MACINTOSH_iNet)	/*	=============== Macintosh ============== */
										
	if (qd.thePort == 0L)	{			//	if the toolbox has yet to be initialized...

										//	Careful, if this file is compiled in a
										//	a4 based 68K code rsrc, the reference to a5 based
										//	qd globals might not work.  You might need the
										//	trick described in IM Memory pg4-18. The
										//	"CurrentA5(), qdP = &dq" trick often results
										//	in LINKER bad resolved references, which 
										//	will screw you at run time.
		#if defined(DRIVER_IS_COMPILED_INTO_AN_APP)
			GrafPtr savePort;
		#endif
										/* 	true if DRIVER code is compiled into an */
										/*	application (i.e. console window is showing) */		
		#if defined(DRIVER_IS_COMPILED_INTO_AN_APP)
										
										//	if the ansi console window is up...
			if (gConsoleWindowHasBeenOpened) {

				GetPort(&savePort);		//	remember the current port (e.g. development
			}							//	environment may have shown a window
										//	without initializing the toolbox ??).
										
										//	In a compiled application, you cannot
										//	call GetPort() before InitGraf(), only
										//	under the ThinkC debugger can you do that.
										
										//	If the console window is showing, then
										//	this GetPort call will do ok, otherwise, 
										//	it will crash since InitGraf() has
										//	not been called. 6/1/96
		#endif	
		
		InitGraf(&qd.thePort);
		InitFonts();
		FlushEvents(everyEvent, 0);
		InitWindows();
		InitMenus();
		TEInit();
		InitDialogs(0L);
		InitCursor();
		MaxApplZone();
		FlushEvents(0, everyEvent);
		
										/* 	true if DRIVER code is compiled into an application */
										/*	(i.e. console window is showing) */		
		#if defined(DRIVER_IS_COMPILED_INTO_AN_APP)

										//	if the ansi console window is up...
			if (gConsoleWindowHasBeenOpened) {
			
				SetPort(savePort);		//	Set the port back to where it was
			}

		#endif
	}

#elif (MACHINE_TYPE_ == WINTEL_iNet) 	/*	============ Windows/Intel ============ */


#endif
}


/* ----------------------------------------------------------------------------------
 *	FUNCTION  	CtoP_Strcpy_local()				CLASS: SUPPORT
 *
 *	PURPOSE  	Copies the entire C string 'src_C' into the PASCAL string pointed
 *				to by 'dest_P''.  THIS DOES NOT MOVE MEMORY.
 *
 *	HISTORY		11/29/95	created - gsw
 * ---------------------------------------------------------------------------------- */
static unsigned char *CtoP_Strcpy_local(
	unsigned char *dest_P,			//	destination PASCAL string
	char *src_C)					//	source C string
{
	register unsigned char *src = (unsigned char *) src_C;
	register unsigned char *dest = (unsigned char *) dest_P;
	register unsigned char numChars = 0;
	dest++;							//	go to the first char of the p string
	while (*src != 0)	{
		*dest++ = *src++;
		numChars++;					//	keep track of the # of chars
	}
	*dest_P = numChars;				//	record the # of chars
	return dest_P;					//	return ptr to pascal string
}


/* -----------------------------------------------------------------------------------
 *	FUNCTION  	Snub Application main()			CLASS: SUPPORT	
 *
 *	PURPOSE 	This is a tiny application main that simply opens the
 *				instruNet World window via the iNet() driver.
 *				
 *	HISTORY		11/30/95	created - gsw
 * --------------------------------------------------------------------------------- */

#if defined(BUILD_SNUB_APL_TO_CALL_iNet_DRIVER)
										/*	if we are to build a tiny application to 
										 *	open the iNet window via the driver...  
										 *	BUILD_SNUB_APL_TO_CALL_iNet_DRIVER is 
										 *	typically defined in the Project prefix. */

	#if (MACHINE_TYPE_==MACINTOSH_iNet)	/*	============= Macintosh ============= */

		int main()						/*	Main entry for a Macintosh application */
		{
	#elif (MACHINE_TYPE_ == WINTEL_iNet)/*	=========== Windows/Intel =========== */
											
		int PASCAL WinMain (			/*	Main entry for a windows application */
			HINSTANCE hInstance,
			HINSTANCE hPrevInstance,
			LPSTR lpCmdLine,
			int nCmdShow)
		{
	#endif
		iNetError e = iNetErr_None;
		iNetINT16 driverIsInstalledOK, numNetworks;
	
		#define DO_SOME_TESTING			 0
		#define OPEN_INET_WINDOW		 1
			
		//	------------------------------------------------------
		//	Load instruNet Driver, gsw 5/15/03

		e = Load_instruNet_Driver(TRUE /* forceInstruNetReset */, &driverIsInstalledOK, &numNetworks);
		if ((e != iNetErr_None) || (driverIsInstalledOK == FALSE)) { goto ExitWithError; }

		//	------------------------------------------------------
		//	Test Button Press Code

		#if DO_SOME_TESTING
		
			e = PRESS_iNet_BUTTON( iNet_ButtonID_Network_Clear );
			if (e)						//	bail if we hit an error
				goto Exit;
				
			e = PRESS_iNet_BUTTON( iNet_ButtonID_Network_Reset );
			if (e)						//	bail if we hit an error
				goto Exit;
				
			e = PRESS_iNet_BUTTON( iNet_ButtonID_Record_Open );
			if (e)						//	bail if we hit an error
				goto Exit;
				
			e = PRESS_iNet_BUTTON( iNet_ButtonID_Network_Store );
			if (e)						//	bail if we hit an error
				goto Exit;
Exit:		
		#endif		
			
		//	------------------------------------------------------
		//	Open instruNet World window.

		#if OPEN_INET_WINDOW		
										//	open the instruNet World window,
										//	Network Page
			e = OPEN_iNet_WINDOW( iNetCM_OpenWindow_Network );

			if (e) {					//	if we got an iNet() error...
ExitWithError:				
										//	show an alert to tell the user
										//	about the iNet error code.
				Show_ALERT_if_hit_iNet_Error_local( e );
												
										//	tell the network to fully reset
				PRESS_iNet_BUTTON( iNet_ButtonID_Network_Reset );
																				
										//	Closes the instruNet driver and
										//	releases the memory used to hold it.
				CloseDriverAndReleaseDriverRam();

				return 0;				//	the error might not happen again
			}

		#endif
	
		if (e)	{						//	if we got an error, show an alert
			Show_ALERT_if_hit_iNet_Error_local(e);
		}
										//	Closes the instruNet driver and
										//	releases the memory used to hold it.
		CloseDriverAndReleaseDriverRam();

		return 0;
	}
	
#endif	/* #if BUILD_SNUB_APL_TO_CALL_iNet_DRIVER */

//	---------------------------------------------------------------------------- 
//	Compile some internal routines, used by Win32 iNet32.DLL.

#if (MACHINE_TYPE_ == WINTEL_iNet)	/*	=========== Windows/Intel =========== */

									/* 	if building iNet32_Apl or iNet32.DLL... */
	#if defined(DRIVER_IS_COMPILED_INTO_AN_APP)	|| defined(DRIVER_IS_COMPILED_INTO_A_WINDOWS_DLL)

		void fftOnFloats (iNetFLT32 *x, iNetINT16 power);
		iNetINT16 CalculateWindow(iNetINT32 numPts, iNetINT16 type, iNetFLT32 *windowPf);
		iNetFLT32 *GiveMeAHammingWindow(iNetINT16 power);
		void MultiplyTwoFlt32Arrays(iNetFLT32 *sourceArray,
			 iNetFLT32 *destArray, iNetINT32 numPts);
		void ComplexToMagnitudes( iNetFLT32 *sourceArray, iNetINT32 numComplexPairs);
		void MultConstantByArray(iNetFLT32 *sourceArray, iNetFLT32 *destArray,
			iNetFLT32 constant, iNetINT32 numPts);
		void AddConstantToArray(iNetFLT32 *sourceArray, iNetFLT32 *destArray, 
			iNetFLT32 constant, iNetINT32 numPts);
		iNetINT16 MultiplyArrayByHammWindow( iNetFLT32 *data, iNetINT16 power);

	#endif

#endif

/* 	---------------------------------------------------------------------------- 
 *	FUNCTION  	Get_iWp_LicenseState()					 
 *
 *	PURPOSE 	Returns 1 (TRUE) if licensed for iW+, 0 otherwise. 
 *
 *	HISTORY		7/5/03	created, gsw
 * ----------------------------------------------------------------------------- */

iNetINT16 EXT_FUNC_TYPE Get_iWp_LicenseState( void )
{
									/*	============= Macintosh ============= */
#if (MACHINE_TYPE_ == MACINTOSH_iNet)	

	return 0 /* no license */;
									/*	=========== Windows/Intel =========== */
#elif (MACHINE_TYPE_ == WINTEL_iNet)	

	iNetINT32 showAlertIsEnabled = GET_SHOW_MESSAGE_UPON_ERROR_STATUS();
	iNetINT32 licenseIsInstalled;
									//	if show alert is turned on
	if (showAlertIsEnabled != 0)	{
		DONT_SHOW_MESSAGE_DIALOG_UPON_ERROR();
		licenseIsInstalled = GET_IWP_LICENSE_STATE();
		SHOW_MESSAGE_DIALOG_UPON_ERROR();
	}
	else	{						//	else the error alert will not show
		licenseIsInstalled = GET_IWP_LICENSE_STATE();
	}
									//	return 1 (TRUE) if licensed for iW+, 0 otherwise
	return (licenseIsInstalled == 0L) ? 0 : 1;

#endif
}


/* 	---------------------------------------------------------------------------- 
 *	FUNCTION  	fft_DLL()					 
 *
 *	PURPOSE 	This function computes the fft on an array of numbers.
 *				It is given 2^power real values and returns 2^(power-1)
 *				complex pairs (real, imaginary, real, imaginary).  Please
 *				give it an array that is 2 longer than 2^power -- the algorithm
 *				needs this.  Several notes:
 *
 *				*	This routine is given 2^power real values.
 *				*	The x[] array must contain 2 extra elements at 
 *					the end -- CAREFUL!!!  CAREFUL!!!
 *				*	We return 2^(power-1) complex pairs values.
 *				*	If you take a sine wave and multiply it by a hamming
 *					window, it is like convolving the frequency response
 *					of the hamming window with a delta function, and
 *					that is what you see at the fft output.  The
 *					frequency response of the hamming window is
 *					one main lobe with side lobes 42dB down.
 *
 *	HISTORY		8/10/98		tsfr'ed to iNet - gsw
 * ----------------------------------------------------------------------------- */
					
void EXT_FUNC_TYPE fft_DLL(			/*	Exported out to Win95 DLL */
	iNetFLT32 *data,				//	input array & ouput array (12 byte doubles)
	iNetINT16 power)				//	number of points = 2^power
{ 
									/*	============= Macintosh ============= */
#if (MACHINE_TYPE_ == MACINTOSH_iNet)									
	//	not supported
	STOP_COMPILER_WARNING ( data );
	STOP_COMPILER_WARNING ( power );
	return;
									/*	=========== Windows/Intel =========== */
#elif (MACHINE_TYPE_ == WINTEL_iNet)	

									/* 	if building iNet32_Apl or iNet32.DLL... */
	#if defined(DRIVER_IS_COMPILED_INTO_AN_APP)	|| defined(DRIVER_IS_COMPILED_INTO_A_WINDOWS_DLL)
		fftOnFloats (data, power);	//	use internal version

	#else							//	else, use version inside dll, called from apl

		typedef void (__stdcall *fft_DLL_proc) (iNetFLT32 *data, iNetINT16 power);

		static fft_DLL_proc fft_DLL_procP = 0L;

		if (gLib == 0) { return; }
									//	load a ptr to this procedure, within the dll
		if (fft_DLL_procP == (fft_DLL_proc) 0L) {
			fft_DLL_procP = (fft_DLL_proc) GetProcAddress (gLib, "fft_DLL");
			if (fft_DLL_procP == 0L) { return; }
		}
									
		(*fft_DLL_procP)(data, power);

	#endif
#endif
}


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	MultiplyArrayByHammWindow()
 *
 *	PURPOSE 	This multiplies the flt32 array by a Hamming window, 
 *				# of pts = 2 ^ power, power is {1...18}.  Returns TRUE if successful.
 *
 *	HISTORY		8/10/98 created - gsw
 * ----------------------------------------------------------------------------------- */

iNetINT16 EXT_FUNC_TYPE MultiplyArrayByHammWindow_DLL( /* Exported out to Win95 DLL */
	iNetFLT32 *data,
	iNetINT16 power)
{
									/*	============= Macintosh ============= */
#if (MACHINE_TYPE_ == MACINTOSH_iNet)									
	//	not supported
	STOP_COMPILER_WARNING ( data );
	STOP_COMPILER_WARNING ( power );
	return iNetErr_None;
									/*	=========== Windows/Intel =========== */
#elif (MACHINE_TYPE_ == WINTEL_iNet)	

									/* 	if building iNet32_Apl or iNet32.DLL... */
	#if defined(DRIVER_IS_COMPILED_INTO_AN_APP)	|| defined(DRIVER_IS_COMPILED_INTO_A_WINDOWS_DLL)
									//	use internal version
		MultiplyArrayByHammWindow (data, power);		
		return TRUE;
	#else							//	else, use version inside dll, called from apl
								
		typedef iNetINT16 (__stdcall *MbyH_DLL_proc) (iNetFLT32 *data, iNetINT16 power);

		static MbyH_DLL_proc MbyH_DLL_procP = 0L;

		if (gLib == 0L) { return FALSE; }
									//	load a ptr to this procedure, within the dll
		if (MbyH_DLL_procP == (MbyH_DLL_proc) 0L) {
			MbyH_DLL_procP = (MbyH_DLL_proc) GetProcAddress (gLib, "MultiplyArrayByHammWindow_DLL");
			if (MbyH_DLL_procP == 0L) { return FALSE; }
		}
									//	call the routine
		return ((*MbyH_DLL_procP) (data, power));

	#endif
#endif
}

/* -------------------------------------------------------------------------------------
 *	FUNCTION  	CalculateWindow_DLL()
 *
 *	PURPOSE 	This uses the cos() function to create a float (0.0 to 1.0 values)
 *				window (Hamm, Hann, Blackman). Returns TRUE if successful; FALSE otherwise.
 *
 *				Ham:		h(n) = .54 - .46*cos(2n/(N-1))	0 <= n <= N-1
 *				Hann:		h(n) = .50 - .50*cos(2n/(N-1))	0 <= n <= N-1
 *				Blackman	h(n) = .42 - .50*cos(2n/(N-1)) + .08*cos(4n/(N-1))	0 <= n <= N-1
 *
 *	HISTORY		8/10/98	created - gsw
 * ----------------------------------------------------------------------------------- */
iNetINT16 EXT_FUNC_TYPE CalculateWindow_DLL(	/*	Exported out to Win95 DLL */
	iNetINT32 numPts,				//	# of pts in the window
									//	this can be odd or even; ranges from 2 to 16384
	iNetINT16 type,					//	WindowType = { jHamm, jHann, jBlackman }
	iNetFLT32 *windowPf)			//	we fill this ptr w/ a window
{
									/*	============= Macintosh ============= */
#if (MACHINE_TYPE_ == MACINTOSH_iNet)									
	//	not supported
	STOP_COMPILER_WARNING ( numPts );
	STOP_COMPILER_WARNING ( type );
	STOP_COMPILER_WARNING ( windowPf );
	return TRUE;
									/*	=========== Windows/Intel =========== */
#elif (MACHINE_TYPE_ == WINTEL_iNet)	

									/* 	if building iNet32_Apl or iNet32.DLL... */
	#if defined(DRIVER_IS_COMPILED_INTO_AN_APP)	|| defined(DRIVER_IS_COMPILED_INTO_A_WINDOWS_DLL)
									//	use internal version
		CalculateWindow (numPts, type /* iNetINT16 */, windowPf);
		return TRUE;

	#else							//	else, use version inside dll, called from apl
								
		typedef iNetINT16 (__stdcall *CalcWind_DLL_proc) (iNetINT32 numPts, iNetINT16 type, iNetFLT32 *windowPf);

		static CalcWind_DLL_proc CalcWind_DLL_procP = 0L;

		if (gLib == 0L) { return FALSE; }
									//	load a ptr to this procedure, within the dll
		if (CalcWind_DLL_procP == (CalcWind_DLL_proc) 0L) {
			CalcWind_DLL_procP = (CalcWind_DLL_proc) GetProcAddress (gLib, "CalculateWindow_DLL");
			if (CalcWind_DLL_procP == 0L) { return FALSE; }
		}
									//	call the routine
		return ((*CalcWind_DLL_procP) (numPts, type, windowPf));

	#endif

#endif
}

/* -------------------------------------------------------------------------------------
 *	FUNCTION  	MultiplyTwoFlt32Arrays_DLL()
 *
 *	PURPOSE 	This multiplies two flt32 arrays.
 *
 *	HISTORY		8/10/98	created - gsw
 * ----------------------------------------------------------------------------------- */

void EXT_FUNC_TYPE MultiplyTwoFlt32Arrays_DLL(	/*	Exported out to Win95 DLL */
	iNetFLT32 *sourceArray,			//	srcArray
	iNetFLT32 *destArray,			//	destArray = srcArray * destArray
	iNetINT32 numPts)				//	# of pts to multiply
{
									/*	============= Macintosh ============= */
#if (MACHINE_TYPE_ == MACINTOSH_iNet)									
	//	not supported
	STOP_COMPILER_WARNING ( sourceArray );
	STOP_COMPILER_WARNING ( destArray );
	STOP_COMPILER_WARNING ( numPts );
									/*	=========== Windows/Intel =========== */
#elif (MACHINE_TYPE_ == WINTEL_iNet)	

									/* 	if building iNet32_Apl or iNet32.DLL... */
	#if defined(DRIVER_IS_COMPILED_INTO_AN_APP)	|| defined(DRIVER_IS_COMPILED_INTO_A_WINDOWS_DLL)
									//	use internal version
		MultiplyTwoFlt32Arrays (sourceArray, destArray, numPts);		

	#else							//	else, use version inside dll, called from outside dll
							
		typedef void (__stdcall *MultArrays_DLL_proc) (iNetFLT32 *sourceArray, iNetFLT32 *destArray, iNetINT32 numPts);

		static MultArrays_DLL_proc MultArrays_DLL_procP = 0L;

		if (gLib == 0L) { return; }
									//	load a ptr to this procedure, within the dll
		if (MultArrays_DLL_procP == (MultArrays_DLL_proc) 0L) {
			MultArrays_DLL_procP = (MultArrays_DLL_proc) GetProcAddress (gLib, "MultiplyTwoFlt32Arrays_DLL");
			if (MultArrays_DLL_procP == 0L) { return; }
		}
									//	call the routine
		(*MultArrays_DLL_procP) (sourceArray, destArray, numPts);

	#endif
#endif
	return;
}

/* -------------------------------------------------------------------------------------
 *	FUNCTION  	ComplexToMagnitudes_DLL()
 *
 *	PURPOSE 	This converts {real, imaginary} complex pairs in a flt32 array to
 *				magnitudes = sqrt(real^2 + imaginary^2).
 *
 *	HISTORY		8/10/98	created - gsw
 * ----------------------------------------------------------------------------------- */

void EXT_FUNC_TYPE ComplexToMagnitudes_DLL(	/*	Exported out to Win95 DLL */
	iNetFLT32 *sourceArray,			//	srcArray
	iNetINT32 numComplexPairs)		//	# of complex pairs to process
{
									/*	============= Macintosh ============= */
#if (MACHINE_TYPE_ == MACINTOSH_iNet)									
	//	not supported
	STOP_COMPILER_WARNING ( sourceArray );
	STOP_COMPILER_WARNING ( numComplexPairs );
									/*	=========== Windows/Intel =========== */
#elif (MACHINE_TYPE_ == WINTEL_iNet)	

									/* 	if building iNet32_Apl or iNet32.DLL... */
	#if defined(DRIVER_IS_COMPILED_INTO_AN_APP)	|| defined(DRIVER_IS_COMPILED_INTO_A_WINDOWS_DLL)
									//	use internal version
		ComplexToMagnitudes (sourceArray, numComplexPairs);		

	#else							//	else, use version inside dll, called from outside dll
							
		typedef void (__stdcall *CtoM_DLL_proc) (iNetFLT32 *sourceArray, iNetINT32 numComplexPairs);

		static CtoM_DLL_proc CtoM_DLL_procP = 0L;

		if (gLib == 0L) { return; }
									//	load a ptr to this procedure, within the dll
		if (CtoM_DLL_procP == (CtoM_DLL_proc) 0L) {
			CtoM_DLL_procP =  (CtoM_DLL_proc) GetProcAddress (gLib, "ComplexToMagnitudes_DLL");
			if (CtoM_DLL_procP == 0L) { return; }
		}
									//	call the routine
		(*CtoM_DLL_procP) (sourceArray, numComplexPairs);

	#endif
#endif
	return;
}


/* -------------------------------------------------------------------------------------
 *	FUNCTION  	MultConstantByArray_DLL()
 *
 *	PURPOSE 	Multiply a constant by a flt32 array.
 *
 *	HISTORY		8/10/98	created - gsw
 * ----------------------------------------------------------------------------------- */

void EXT_FUNC_TYPE MultConstantByArray_DLL(	/*	Exported out to Win95 DLL */
	iNetFLT32 *sourceArray,			//	source array
	iNetFLT32 *destArray,			//	destination array
	iNetFLT32 constant,				//	constant value
	iNetINT32 numPts)				//	# of points to process
{
									/*	============= Macintosh ============= */
#if (MACHINE_TYPE_ == MACINTOSH_iNet)									
	//	not supported
	STOP_COMPILER_WARNING ( sourceArray );
	STOP_COMPILER_WARNING ( destArray );
	STOP_COMPILER_WARNING ( constant );
	STOP_COMPILER_WARNING ( numPts );
									/*	=========== Windows/Intel =========== */
#elif (MACHINE_TYPE_ == WINTEL_iNet)	

									/* 	if building iNet32_Apl or iNet32.DLL... */
	#if defined(DRIVER_IS_COMPILED_INTO_AN_APP)	|| defined(DRIVER_IS_COMPILED_INTO_A_WINDOWS_DLL)
									//	use internal version
		if (numPts == 0L) { return; }
		MultConstantByArray (sourceArray, destArray, constant, numPts);		

	#else							//	else, use version inside dll, called from outside dll
							
		typedef void (__stdcall *MCA_DLL_proc) (iNetFLT32 *sourceArray,	
				iNetFLT32 *destArray, iNetFLT32 constant, iNetINT32 numPts);

		static MCA_DLL_proc MCA_DLL_procP = 0L;

		if (gLib == 0L) { return; }
									//	load a ptr to this procedure, within the dll
		if (MCA_DLL_procP == (MCA_DLL_proc) 0L) {
			MCA_DLL_procP =  (MCA_DLL_proc) GetProcAddress (gLib, "MultConstantByArray_DLL");
			if (MCA_DLL_procP == 0L) { return; }
		}
									//	call the routine
		(*MCA_DLL_procP) (sourceArray, destArray, constant, numPts);

	#endif
#endif
	return;
}




